#include "amodule.h"
#include "apluginarray.h"
#include "atrack.h"
#include "cache.h"
#include "edl.h"
#include "edlsession.h"
#include "file.h"
#include "mwindow.h"
#include "playbackengine.h"
#include "pluginbuffer.h"
#include "pluginserver.h"
#include "preferences.h"
#include "recordableatracks.h"
#include "mainsession.h"

APluginArray::APluginArray()
 : PluginArray()
{
}

APluginArray::~APluginArray()
{
	file->stop_audio_thread();
	for(int i = 0; i < total_tracks(); i++)
	{
		delete modules[i];
	}
	delete tracks;
}

void APluginArray::create_buffers()
{
	file->start_audio_thread(buffer_size, RING_BUFFERS);
	if(!plugin_server->realtime) realtime_buffers = file->get_audio_buffer();
}

void APluginArray::create_modules()
{
	modules = new Module*[total_tracks()];
	for(int i = 0; i < total_tracks(); i++)
	{
		modules[i] = new AModule(0, 0, tracks->values[i]);
		modules[i]->cache = cache;
		modules[i]->edl = edl;
		modules[i]->create_objects();
		modules[i]->render_init();
	}
}

void APluginArray::load_module(int module, long input_position, long len)
{
	realtime_buffers = file->get_audio_buffer();
	((AModule*)modules[module])->render(realtime_buffers[module], 
		len, 
		input_position,
		PLAY_FORWARD);
}

void APluginArray::process_realtime(int module, long input_position, long len)
{
	values[module]->process_realtime(&realtime_buffers[module], 
			&realtime_buffers[module],
			input_position, 
			len,
			0);
}

int APluginArray::process_loop(int module, long &write_length)
{
	int result = values[module]->process_loop(&realtime_buffers[module], write_length);
	return result;
}


int APluginArray::write_buffers(long len)
{
	int result = file->write_audio_buffer(len);
	if(!plugin_server->realtime && !done && !result) realtime_buffers = file->get_audio_buffer();
	return result;
}

long APluginArray::get_bufsize()
{
	return mwindow->edl->session->sample_rate;
}


void APluginArray::get_recordable_tracks()
{
	tracks = new RecordableATracks(mwindow->edl->tracks);
}

int APluginArray::total_tracks()
{
	return tracks->total;
}

Track* APluginArray::track_number(int number)
{
	return (Track*)tracks->values[number];
}
