#include "assets.h"
#include "awindowgui.inc"
#include "batch.h"
#include "cache.h"
#include "edl.h"
#include "file.h"
#include "filexml.h"
#include "filesystem.h"
#include "indexfile.h"
#include "quicktime.h"
#include "mainsession.h"
#include "threadindexer.h"
#include <string.h>

Assets::Assets(EDL *edl) : List<Asset>()
{
	this->edl = edl;
	thread_indexer_running = 0;
}

Assets::~Assets()
{
	delete_all();
}

int Assets::load(ArrayList<PluginServer*> *plugindb, 
	FileXML *file, 
	unsigned long load_flags)
{
	int result = 0;

	while(!result)
	{
		result = file->read_tag();
		if(!result)
		{
			if(file->tag.title_is("/ASSETS"))
			{
				result = 1;
			}
			else
			if(file->tag.title_is("ASSET"))
			{
				char *path = file->tag.get_property("SRC");
				Asset new_asset(path ? path : SILENCE);
				new_asset.read(plugindb, file);
				update(&new_asset);
			}
		}
	}
	return 0;
}

int Assets::save(ArrayList<PluginServer*> *plugindb, FileXML *file, char *path)
{
	file->tag.set_title("ASSETS");
	file->append_tag();
	file->append_newline();

	for(Asset* current = first; current; current = NEXT)
	{
		current->write(plugindb, 
			file, 
			0, 
			path);
	}

	file->tag.set_title("/ASSETS");
	file->append_tag();
	file->append_newline();	
	file->append_newline();	
	return 0;
}

Assets& Assets::operator=(Assets &assets)
{
//printf("Assets::operator= 1 %p\n", last);
	while(last) delete last;

//printf("Assets::operator= 2\n");
	for(Asset *current = assets.first; current; current = NEXT)
	{
//printf("Assets::operator= 3\n");
		Asset *new_asset;
//printf("Assets::operator= 4\n");
		append(new_asset = new Asset);
//printf("Assets::operator= 5\n");
		*new_asset = *current;
//printf("Assets::operator= 6\n");
	}
//printf("Assets::operator= 7\n");

	return *this;
}


void Assets::update_index(Asset *asset)
{
	for(Asset* current = first; current; current = NEXT)
	{
		if(current->test_path(asset->path))
		{
			current->update_index(asset);
		}
	}
}

Asset* Assets::update(Asset *asset)
{
	if(!asset) return 0;

	for(Asset* current = first; current; current = NEXT)
	{
// Asset already exists.
		if(current->test_path(asset->path)) 
		{
			return current;
		}
	}

// Asset doesn't exist.
	Asset *asset_copy = new Asset(*asset);
	append(asset_copy);
	return asset_copy;
}

int Assets::delete_all()
{
	while(last)
	{
		remove(last);
	}
	return 0;
}

Asset* Assets::update(const char *path)
{
	Asset* current = first;

	while(current)
	{
		if(current->test_path(path)) 
		{
			return current;
		}
		current = NEXT;
	}

	return append(new Asset(path));
}

Asset* Assets::get_asset(const char *filename)
{
	Asset* current = first;
	Asset* result = 0;

	while(current)
	{
//printf("Assets::get_asset %p %s\n", filename, filename);
		if(current->test_path(filename))
		{
			result = current;
			break;
		}
		current = current->next;
	}

	return result;	
}

int Assets::build_indexes()
{
	if(!thread_indexer_running)
	{
		thread_indexer_running = 1;
		thread_indexer->start_build();
	}
	return 0;
}

int Assets::interrupt_indexes()
{
	if(thread_indexer_running)
	{
		thread_indexer_running = 0;
		thread_indexer->interrupt_build();
	}
	return 0;
}

int Assets::number_of(Asset *asset)
{
	int i;
	Asset *current;

	for(i = 0, current = first; current && current != asset; i++, current = NEXT)
		;

	return i;
}

Asset* Assets::asset_number(int number)
{
	int i;
	Asset *current;

	for(i = 0, current = first; i < number && current; i++, current = NEXT)
		;
	
	return current;
}

int Assets::update_old_filename(char *old_filename, char *new_filename)
{
	for(Asset* current = first; current; current = NEXT)
	{
		if(!strcmp(current->path, old_filename))
		{
			current->update_path(new_filename);
		}
	}
	return 0;
}


int Assets::dump()
{
	for(Asset *current = first; current; current = NEXT)
	{
		current->dump();
	}
	return 0;
}


// ==========================================================

Asset::Asset() : ListItem<Asset>()
{
	init_values();
}

Asset::Asset(Asset &asset) : ListItem<Asset>()
{
	init_values();
	*this = asset;
}

Asset::Asset(const char *path) : ListItem<Asset>()
{
	init_values();
	strcpy(this->path, path);
}

Asset::Asset(const int plugin_type, const char *plugin_title) : ListItem<Asset>()
{
	init_values();
}

Asset::~Asset()
{
	if(index_offsets) delete [] index_offsets;
	if(index_buffer) delete index_buffer;
}


int Asset::init_values()
{
	path[0] = 0;
	strcpy(folder, MEDIA_FOLDER);
	format = 0;
	channels = 0;
	sample_rate = 0;
	bits = 0;
	byte_order = 0;
	signed_ = 0;
	header = 0;
	dither = 0;
	audio_data = 0;
	video_data = 0;

	layers = 0;
	frame_rate = 0;
	width = 0;
	height = 0;
	strcpy(compression, QUICKTIME_YUV2);
	quality = 100;

	audio_length = 0;
	video_length = 0;

	reset_index();
	id = EDL::next_id();
	return 0;
}

int Asset::reset_index()
{
	index_status = INDEX_NOTTESTED;
	index_start = old_index_end = index_end = 0;
	index_offsets = 0;
	index_zoom = 0;
	index_bytes = 0;
	index_buffer = 0;
	return 0;
}

void Asset::copy_format(Asset *asset)
{
	if(index_offsets) delete [] index_offsets;
	if(index_buffer) delete index_buffer;

	index_offsets = 0;
	index_buffer = 0;
	index_status = asset->index_status;
	index_start = asset->index_start;
	index_end = asset->index_end;
	index_zoom = asset->index_zoom;
	index_bytes = asset->index_bytes;
	old_index_end = asset->old_index_end;

	audio_data = asset->audio_data;
	format = asset->format;
	channels = asset->channels;
	sample_rate = asset->sample_rate;
	bits = asset->bits;
	byte_order = asset->byte_order;
	signed_ = asset->signed_;
	header = asset->header;
	dither = asset->dither;

	video_data = asset->video_data;
	layers = asset->layers;
	frame_rate = asset->frame_rate;
	width = asset->width;
	height = asset->height;
	quality = asset->quality;
	strcpy(compression, asset->compression);
	if(asset->index_offsets && audio_data)
	{
		index_offsets = new long[channels];
		for(int i = 0; i < channels; i++)
		{
			index_offsets[i] = asset->index_offsets[i];
		}
	}

	this->audio_length = asset->audio_length;
	this->video_length = asset->video_length;
}

Asset& Asset::operator=(Asset &asset)
{
	strcpy(this->path, asset.path);
	strcpy(this->folder, asset.folder);
	copy_format(&asset);
	return *this;
}

int Asset::operator==(Asset &asset)
{
	if((!strcmp(asset.path, path)) &&

		format == asset.format && 
		channels == asset.channels && 
		sample_rate == asset.sample_rate && 
		bits == asset.bits && 
		byte_order == asset.byte_order && 
		signed_ == asset.signed_ && 
		header == asset.header && 
		dither == asset.dither &&

		layers == asset.layers && 
		frame_rate == asset.frame_rate &&
		quality == asset.quality &&
		width == asset.width &&
		height == asset.height &&
		!strcmp(compression, asset.compression)
	) return 1;
	else
	return 0;
}

int Asset::operator!=(Asset &asset)
{
	return !(*this == asset);
}

int Asset::test_path(const char *path)
{
// 	char old_name[1024], new_name[1024];
// 	FileSystem fs;
// 
// 	fs.extract_name(old_name, this->path);
// 	fs.extract_name(new_name, path);
	if(!strcasecmp(this->path, path)) 
		return 1; 
	else 
		return 0;
}

int Asset::test_plugin_title(const char *path)
{
}

int Asset::read(ArrayList<PluginServer*> *plugindb, FileXML *file)
{
	int result = 0;

// Check for relative path.
	char new_path[1024];
	strcpy(new_path, path);
	char asset_directory[1024], input_directory[1024];
	FileSystem fs;
	fs.set_current_dir("");

	fs.extract_dir(asset_directory, path);

// No path in asset
	if(!asset_directory[0])
	{
		fs.extract_dir(input_directory, file->filename);

// Input file has a path
		if(input_directory[0])
		{
			sprintf(path, "%s/%s", input_directory, new_path);
		}
	}

	while(!result)
	{
		result = file->read_tag();
		if(!result)
		{
			if(file->tag.title_is("/ASSET"))
			{
				result = 1;
			}
			else
			if(file->tag.title_is("AUDIO"))
			{
				read_audio(file);
			}
			else
			if(file->tag.title_is("FORMAT"))
			{
				char *string = file->tag.get_property("TYPE");
				format = File::strtoformat(plugindb, string);
			}
			else
			if(file->tag.title_is("FOLDER"))
			{
				strcpy(folder, file->read_text());
			}
			else
			if(file->tag.title_is("VIDEO"))
			{
				read_video(file);
			}
			else
			if(file->tag.title_is("INDEX"))
			{
				read_index(file);
			}
		}
	}
	return 0;
}

int Asset::read_audio(FileXML *file)
{
	channels = file->tag.get_property("CHANNELS", 2);
	sample_rate = file->tag.get_property("RATE", 44100);
	bits = file->tag.get_property("BITS", 16);
	byte_order = file->tag.get_property("BYTE_ORDER", 1);
	signed_ = file->tag.get_property("SIGNED", 1);
	header = file->tag.get_property("HEADER", 0);
	dither = file->tag.get_property("DITHER", 0);

	audio_length = file->tag.get_property("AUDIO_LENGTH", 0);
	audio_data = 1;
	return 0;
}

int Asset::read_video(FileXML *file)
{
	height = file->tag.get_property("HEIGHT", height);
	width = file->tag.get_property("WIDTH", width);
	layers = file->tag.get_property("LAYERS", layers);
	frame_rate = file->tag.get_property("FRAMERATE", frame_rate);
	compression[0] = 0;
	file->tag.get_property("COMPRESSION", compression);
	quality = file->tag.get_property("QUALITY", quality);

	video_length = file->tag.get_property("VIDEO_LENGTH", 0);
	video_data = 1;
	return 0;
}

int Asset::read_index(FileXML *file)
{
	if(index_offsets) delete [] index_offsets;
	index_offsets = new long[channels];
	for(int i = 0; i < channels; i++) index_offsets[i] = 0;
	int current_offset = 0;
	int result = 0;

	index_zoom = file->tag.get_property("ZOOM", 1);
	index_bytes = file->tag.get_property("BYTES", (longest)0);

	while(!result)
	{
		result = file->read_tag();
		if(!result)
		{
			if(file->tag.title_is("/INDEX"))
			{
				result = 1;
			}
			else
			if(file->tag.title_is("OFFSET"))
			{
				if(current_offset < channels)
				{
					index_offsets[current_offset++] = file->tag.get_property("FLOAT", 0);
				}
			}
		}
	}
	return 0;
}

// Output path is the path of the output file if name truncation is desired.
// It is a "" if complete names should be used.

int Asset::write(ArrayList<PluginServer*> *plugindb, 
	FileXML *file, 
	int include_index, 
	char *output_path)
{
	char new_path[1024];
	char asset_directory[1024], output_directory[1024];
	FileSystem fs;

// Make path relative
	fs.extract_dir(asset_directory, path);
	if(output_path && output_path[0]) 
		fs.extract_dir(output_directory, output_path);
	else
		output_directory[0] = 0;
	if(!strcmp(asset_directory, output_directory))
	{
		fs.extract_name(new_path, path);
	}
	else
	{
		strcpy(new_path, path);
	}

	file->tag.set_title("ASSET");
	file->tag.set_property("SRC", new_path);
	file->append_tag();
	file->append_newline();

	file->tag.set_title("FOLDER");
	file->append_tag();
	file->append_text(folder);
	file->tag.set_title("/FOLDER");
	file->append_tag();
	file->append_newline();

// Write the format information
	file->tag.set_title("FORMAT");

	{
		file->tag.set_property("TYPE", 
			File::formattostr(plugindb, format));
	}

	file->append_tag();
	file->append_newline();

	if(audio_data) write_audio(file);
	if(video_data) write_video(file);
	if(index_status == 0 && include_index) write_index(file);  // index goes after source

	file->tag.set_title("/ASSET");
	file->append_tag();
	file->append_newline();
	return 0;
}

int Asset::write_audio(FileXML *file)
{
	file->tag.set_title("AUDIO");
	file->tag.set_property("CHANNELS", channels);
	file->tag.set_property("RATE", sample_rate);
	file->tag.set_property("BITS", bits);
	file->tag.set_property("BYTE_ORDER", byte_order);
	file->tag.set_property("SIGNED", signed_);
	file->tag.set_property("HEADER", header);
	file->tag.set_property("DITHER", dither);
	
	file->tag.set_property("AUDIO_LENGTH", audio_length);
	
	file->append_tag();
	file->append_newline();
	return 0;
}

int Asset::write_video(FileXML *file)
{
	file->tag.set_title("VIDEO");
	file->tag.set_property("HEIGHT", height);
	file->tag.set_property("WIDTH", width);
	file->tag.set_property("LAYERS", layers);
	file->tag.set_property("FRAMERATE", frame_rate);
	if(compression[0])
		file->tag.set_property("COMPRESSION", compression);
	file->tag.set_property("QUALITY", quality);

	file->tag.set_property("VIDEO_LENGTH", video_length);
	file->append_tag();
	file->append_newline();
	return 0;
}

int Asset::write_index(FileXML *file)
{
	file->tag.set_title("INDEX");
	file->tag.set_property("ZOOM", index_zoom);
	file->tag.set_property("BYTES", index_bytes);
	file->append_tag();
	file->append_newline();

	if(index_offsets)
	{
		for(int i = 0; i < channels; i++)
		{
			file->tag.set_title("OFFSET");
			file->tag.set_property("FLOAT", index_offsets[i]);
			file->append_tag();
		}
	}

	file->append_newline();
	file->tag.set_title("/INDEX");
	file->append_tag();
	file->append_newline();
	return 0;
}

int Asset::update_path(char *new_path)
{
	strcpy(path, new_path);
	return 0;
}

void Asset::update_index(Asset *asset)
{
	index_status = asset->index_status;     // 0 ready  1 not tested  2 being built  3 small source
	index_zoom = asset->index_zoom; 	 // zoom factor of index data
	index_start = asset->index_start;	 // byte start of index data in the index file
	index_bytes = asset->index_bytes;	 // Total bytes in source file for comparison before rebuilding the index
	index_end = asset->index_end;
	old_index_end = asset->old_index_end;	 // values for index build
	index_offsets = asset->index_offsets;  // offsets of channels in index file in floats
	index_buffer = asset->index_buffer;    // pointer
}


int Asset::dump()
{
	printf("  asset::dump\n");
	printf("   %p %s\n", this, path);
	printf("   index_status %d\n", index_status);
	printf("   format %d\n", format);
	printf("   audio_data %d channels %d samplerate %d bits %d byte_order %d signed %d header %d dither %d\n",
		audio_data, channels, sample_rate, bits, byte_order, signed_, header, dither);
	printf("   audio_length %ld\n", audio_length);
	printf("   video_data %d layers %d framerate %f width %d height %d compression %c%c%c%c\n",
		video_data, layers, frame_rate, width, height, compression[0], compression[1], compression[2], compression[3]);
	printf("   video_length %ld\n", video_length);
	return 0;
}
