#ifndef ASSETS_H
#define ASSETS_H

#include <stdio.h>

#include "arraylist.h"
#include "assets.inc"
#include "batch.inc"
#include "edl.inc"
#include "filexml.inc"
#include "guicast.h"
#include "linklist.h"
#include "pluginserver.inc"
#include "sharedlocation.h"
#include "threadindexer.inc"

class Assets : public List<Asset>
{
public:
	Assets(EDL *edl);
	virtual ~Assets();

	int load(ArrayList<PluginServer*> *plugindb, FileXML *xml, unsigned long load_flags);
	int save(ArrayList<PluginServer*> *plugindb, FileXML *xml, char *output_path);
	Assets& operator=(Assets &assets);

// Enter a new asset into the table.
// If the asset already exists return the asset which exists.
// If the asset doesn't exist, store a copy of the argument and return the copy.
	Asset* update(Asset *asset);
// Update the index information for assets with the same path
	void update_index(Asset *asset);


// Parent EDL
	EDL *edl;
	








	int delete_all();
	int dump();

// return the asset containing this path or create a new asset containing this path
	Asset* update(const char *path);

// Insert the asset into the list if it doesn't exist already but
// don't create a new asset.
	void update_ptr(Asset *asset);

// return the asset containing this path or 0 if not found
	Asset* get_asset(const char *filename);

// return number of the asset
	int number_of(Asset *asset);
	Asset* asset_number(int number);

	int update_old_filename(char *old_filename, char *new_filename);
	int build_indexes();
	int interrupt_indexes();

	ThreadIndexer *thread_indexer;
	int thread_indexer_running;
};



// Asset can be one of the following:
// 1) a pure media file
// 2) an EDL
// 3) a log
// The EDL can reference itself if it contains a media file
class Asset : public ListItem<Asset>
{
public:
	Asset();
	Asset(Asset &asset);
	Asset(const char *path);
	Asset(const int plugin_type, const char *plugin_path);
	~Asset();

	int init_values();
	int dump();

	void copy_format(Asset *asset);
// Executed during index building only
	void update_index(Asset *asset);
	Asset& operator=(Asset &asset);
	int operator==(Asset &asset);
	int operator!=(Asset &asset);
	int test_path(const char *path);
	int test_plugin_title(const char *path);
	int read(ArrayList<PluginServer*> *plugindb, FileXML *xml);
	int read_audio(FileXML *xml);
	int read_video(FileXML *xml);
	int read_index(FileXML *xml);
	int reset_index();  // When the index file is wrong, reset the asset values
	int write(ArrayList<PluginServer*> *plugindb, 
		FileXML *xml, 
		int include_index, 
		char *output_path);
	int write_audio(FileXML *xml);
	int write_video(FileXML *xml);
	int write_index(FileXML *xml);
	int update_path(char *new_path);

// Path to file
	char path[BCTEXTLEN];

// Folder in resource manager
	char folder[BCTEXTLEN];

// Determines the file engine to use
	int format;      // format of file

// contains audio data
	int audio_data;
	int channels;
	int sample_rate;
	int bits;
	int byte_order;
	int signed_;
	int header;
	int dither;
// for mpeg audio compression
	int audio_bitrate;
// 2 - 3
	int audio_derivative;
	long audio_length;

// contains video data
	int video_data;       
	int layers;
	double frame_rate;
	int width, height;
// String or FourCC describing compression
	char compression[BCTEXTLEN];
// for mpeg video compression
	int iframe_distance;
	int bframe_distance;
	int progressive;
	int denoise;
	int video_bitrate;
// 1 - 2
	int video_derivative;
	int quantization;
	
// for jpeg compression
	int quality;     
// Length in units of asset
	long video_length;

// Edits store data for the transition

// index info
	int index_status;     // 0 ready  1 not tested  2 being built  3 small source
	long index_zoom;      // zoom factor of index data
	long index_start;     // byte start of index data in the index file
	longest index_bytes;     // Total bytes in source file for comparison before rebuilding the index
	long index_end, old_index_end;    // values for index build
	long* index_offsets;  // offsets of channels in index file in floats
	float* index_buffer;  
	int id;
};




#endif
