#include "attachmentpoint.h"
#include "filexml.h"
#include "edl.h"
#include "edlsession.h"
#include "mwindow.h"
#include "plugin.h"
#include "pluginserver.h"
#include "renderengine.h"
#include "transportque.h"
#include "virtualnode.h"



AttachmentPoint::AttachmentPoint(RenderEngine *renderengine, Plugin *plugin)
{
	reset_parameters();
	this->plugin = plugin;
	this->renderengine = renderengine;
	plugin_server = renderengine->scan_plugindb(plugin->title);
//printf("AttachmentPoint::AttachmentPoint %s %p\n", plugin->title, plugin_server);
}

AttachmentPoint::~AttachmentPoint()
{
	delete_buffer_vectors();
}


int AttachmentPoint::reset_parameters()
{
	current_buffer = 0;
//	plugin_type = 0;
	plugin_server = 0;
//	plugin_type = 0;
	plugin_data[0] = 0;
	strcpy(plugin_title, default_title());
	new_total_input_buffers = 0;
	total_input_buffers = 0;
	return 0;
}

int AttachmentPoint::render_init()
{
//printf("AttachmentPoint::render_init 1\n");
	if(plugin_server && plugin->on)
	{
// Start new plugins or continue existing plugins
// Create new plugin servers
//printf("AttachmentPoint::render_init 2 %d %d\n", virtual_plugins.total, new_virtual_plugins.total);
		if(virtual_plugins.total != new_virtual_plugins.total)
		{
//printf("AttachmentPoint::render_init 3\n");
			plugin_servers.remove_all_objects();
//printf("AttachmentPoint::render_init 4\n");
			for(int i = 0; i < new_virtual_plugins.total; i++)
			{
//printf("AttachmentPoint::render_init 5\n");
				if(i == 0 || !plugin_server->multichannel)
				{
//printf("AttachmentPoint::render_init 5.1\n");
					plugin_servers.append(new PluginServer(*plugin_server));
//printf("AttachmentPoint::render_init 5.2 %p %p\n", plugin_servers.values[i], plugin);
					plugin_servers.values[i]->open_plugin(0, renderengine->edl, plugin);
//printf("AttachmentPoint::render_init 5.3\n");
					plugin_servers.values[i]->init_realtime(renderengine->edl->session->real_time_playback &&
						renderengine->command->realtime,
						new_total_input_buffers);
//printf("AttachmentPoint::render_init 5.4\n");
				}
//printf("AttachmentPoint::render_init 6\n");
			}
		}
	}

//printf("AttachmentPoint::render_init 7\n");
	virtual_plugins.remove_all();
	for(int i = 0; i < new_virtual_plugins.total; i++)
		virtual_plugins.append(new_virtual_plugins.values[i]);
	new_virtual_plugins.remove_all();
//printf("AttachmentPoint::render_init 8\n");

	total_input_buffers = new_total_input_buffers;
	new_total_input_buffers = 0;
//printf("AttachmentPoint::render_init 9\n");

	delete_buffer_vectors();
	new_buffer_vectors();
//printf("AttachmentPoint::render_init 10\n");
	return 0;
}

int AttachmentPoint::render_stop(int duplicate)
{
// stop plugins
// Can't use the on value here because it may have changed.
	if(plugin_server && virtual_plugins.total && !duplicate)
	{
// close the plugins if not shared
		for(int i = 0; i < virtual_plugins.total; i++)
		{
			if(i == 0 || !plugin_server->multichannel)
			{
				plugin_servers.values[i]->realtime_stop();
				plugin_servers.values[i]->close_plugin();
			}
		}
		plugin_servers.remove_all_objects();
	}
	return 0;
}

int AttachmentPoint::attach_virtual_plugin(VirtualNode *virtual_plugin)
{
	int in_buffer_number = 0;

// add virtual plugin to list of new virtual plugins
	new_virtual_plugins.append(virtual_plugin);

// Increment input buffer counter
	if(plugin_server && plugin->on)
	{
		in_buffer_number = new_total_input_buffers++;
	}
//printf("AttachmentPoint::attach_virtual_plugin %p %d %d %d\n", plugin_server, plugin->on, in_buffer_number, new_total_input_buffers);

	return in_buffer_number;
}

int AttachmentPoint::sort(VirtualNode *virtual_plugin)
{
	int result = 0;

	for(int i = 0; i < new_virtual_plugins.total && !result; i++)
	{
// if no virtual plugins attached to this are waiting return 1
		if(!new_virtual_plugins.values[i]->waiting_real_plugin) result = 1;
	}

	return result;
}

int AttachmentPoint::multichannel_shared(int search_new)
{
	if(search_new)
	{
		if(new_virtual_plugins.total && 
			plugin_server && 
			plugin_server->multichannel) return 1;
	}
	else
	{
		if(virtual_plugins.total && 
			plugin_server && 
			plugin_server->multichannel) return 1;
	}
	return 0;
}

int AttachmentPoint::singlechannel()
{
	if(plugin_server && !plugin_server->multichannel) return 1;
	return 0;
}

void AttachmentPoint::render(long current_position, long fragment_size)
{
// All buffers must be armed before proceeding
//printf("AttachmentPoint::render %p %d %d %d\n", plugin_server, plugin_server->multichannel, current_buffer, total_input_buffers - 1);
	if(plugin_server)
	{
		if(current_buffer == total_input_buffers - 1 ||
			!plugin_server->multichannel)
		{
				dispatch_plugin_server(current_buffer, 
					current_position, 
					fragment_size);
		}
	}
	current_buffer++;
	if(current_buffer >= total_input_buffers) current_buffer = 0;
}












AttachmentPoint::AttachmentPoint(AttachmentPoint *that)
{
	reset_parameters();
	copy_from(*that);
}

AttachmentPoint::AttachmentPoint(MWindow *mwindow)
{
	reset_parameters();
	this->mwindow = mwindow;
}

AttachmentPoint& AttachmentPoint::operator=(const AttachmentPoint &that)
{
	copy_from(that);
}

int AttachmentPoint::operator==(const AttachmentPoint &that)
{
	int result = 0;

	if(identical(that)) result = 1;

	return result;
}

int AttachmentPoint::identical(const AttachmentPoint &that)
{
	int result = 0;

	if(plugin->plugin_type == that.plugin->plugin_type)
	{
		switch(plugin->plugin_type)
		{
			case 0:
				result = 1;
				break;
			
			case 1:
				if(!strcmp(plugin_title, that.plugin_title) &&
					!strcmp(plugin_data, that.plugin_data)) result = 1;
				break;
			
			case 2:
				if(shared_plugin_location == that.shared_plugin_location) result = 1;
				break;

			case 3:
				if(shared_module_location == that.shared_module_location) result = 1;
				break;
		}
	}

	return result;
}

int AttachmentPoint::copy_from(const AttachmentPoint &that)
{
	detach();
	shared_plugin_location = that.shared_plugin_location;
	shared_module_location = that.shared_module_location;
//	in = that.in;
//	out = that.out;
//	show = that.show;
//	on = that.on;
//	plugin_type = that.plugin_type;
	mwindow = that.mwindow;
	strcpy(plugin_title, that.plugin_title);
	strcpy(plugin_data, that.plugin_data);

// Need to close plugin if it's open before replacing.
	if(that.plugin_server) plugin_server = new PluginServer(*(that.plugin_server));
	return 0;
}

int AttachmentPoint::dump()
{
	printf("    Attachmentpoint %x\n", this);
	if(plugin_server) plugin_server->dump();
}

// REMOVE
int AttachmentPoint::detach(int update_edits)
{
}

// REMOVE
int AttachmentPoint::attach(int is_loading)
{
}

int AttachmentPoint::update(int plugin_type, int in, int out, char* title, SharedPluginLocation *shared_plugin_location, SharedModuleLocation *shared_module_location)
{
//	this->in = in; this->out = out;
//	this->plugin_type = plugin_type;

	strcpy(this->plugin_title, title);

	if(shared_plugin_location && shared_module_location)
	{
		this->shared_plugin_location = *shared_plugin_location;
		this->shared_module_location = *shared_module_location;
	}
	update_derived();
}

// REMOVE
int AttachmentPoint::show_gui()
{
}

// REMOVE
int AttachmentPoint::hide_gui()
{
}

// REMOVE
int AttachmentPoint::set_show(int value)
{
	return 0;
}


int AttachmentPoint::save(FileXML *xml, char *block_title)
{
	char string[1024];
	xml->tag.set_title(block_title);
//	xml->tag.set_property("TYPE", (long)plugin_type);
	xml->append_tag();
	xml->append_newline();

// only save the title if there is a plugin
// 	if(in)
// 	{
// 		xml->tag.set_title("IN");
// 		xml->append_tag();
// 	}
// 
// 	if(out)
// 	{
// 		xml->tag.set_title("OUT");
// 		xml->append_tag();
// 	}
// 
// 	if(show)
// 	{
// 		xml->tag.set_title("SHOW");
// 		xml->append_tag();
// 	}
// 
// 	if(on)
// 	{
// 		xml->tag.set_title("ON");
// 		xml->append_tag();
// 	}
// 
// 
// // save extra information if it is a shared plugin
// 	switch(plugin_type)
// 	{
// 		case 2:
// 			shared_plugin_location.save(xml);
// 			break;
// 			
// 		case 3:
// 			shared_module_location.save(xml);
// 			break;
// 	}
// 
// 	if(plugin_type)
// 	{
// 		if(plugin_server)
// 		{
// 			xml->tag.set_title("DATA");
// 			xml->append_tag();
// 			xml->append_newline();
// 
// 			xml->append_text(plugin_data);
// 
// 			xml->tag.set_title("/DATA");
// 			xml->append_tag();
// 			xml->append_newline();
// 		}
// 
// 		xml->tag.set_title("TITLE");
// 		xml->append_tag();
// 		xml->append_text(plugin_title);
// 		xml->tag.set_title("/TITLE");
// 		xml->append_tag();
// 		xml->append_newline();
// 	}
// 
// 	xml->append_newline();
// 	sprintf(string, "/%s", block_title);
// 	xml->tag.set_title(string);
// 	xml->append_tag();
// 	xml->append_newline();
	return 0;
}

int AttachmentPoint::load(FileXML *xml, int track_offset, char *terminator)
{
	int result = 0;
// 	in = 0;
// 	out = 0;
// 	show = 0;
// 	on = 0;
// 
// 	plugin_type = xml->tag.get_property("TYPE", (long)0);
// 	
// 	do{
// 		result = xml->read_tag();
// 
// 		if(!result)
// 		{
// 			if(xml->tag.title_is(terminator))
// 			{
// 				result = 1;
// 			}
// 			else
// 			if(xml->tag.title_is("IN"))
// 			{
// 				in = 1;
// 			}
// 			else
// 			if(xml->tag.title_is("OUT"))
// 			{
// 				out = 1;
// 			}
// 			else
// 			if(xml->tag.title_is("SHOW"))
// 			{
// 				show = 1;
// 			}
// 			else
// 			if(xml->tag.title_is("ON"))
// 			{
// 				on = 1;
// 			}
// 			else
// 			if(xml->tag.title_is("SHARED"))
// 			{
// 				shared_plugin_location.load(xml, track_offset);
// 			}
// 			else
// 			if(xml->tag.title_is("MODULE"))
// 			{
// 				shared_module_location.load(xml, track_offset);
// 			}
// 			else
// 			if(xml->tag.title_is("DATA"))
// 			{
// 				xml->read_text_until("/DATA", plugin_data);
// 			}
// 			else
// 			if(xml->tag.title_is("TITLE"))
// 			{
// 				strcpy(plugin_title, xml->read_text());
// 			}
// 		}
// 	}while(!result);
// 
// 	if(plugin_type == 1)
// 	{
// // plugin must be an executable to start
// 		attach(1);
// 	}
// 
// 	update_display();
	return 0;
}

char* AttachmentPoint::get_module_title()
{
	return "AttachmentPoint::get_module_title";
}


int AttachmentPoint::get_configuration_change(char *data)
{
	strcpy(plugin_data, data);
	
	if(plugin_server->multichannel && virtual_plugins.total)
	{
//		virtual_plugins.values[0]->plugin_server->get_configuration_change(plugin_data);
	}
	else
	for(int i = 0; i < virtual_plugins.total; i++)
	{
//		virtual_plugins.values[i]->plugin_server->get_configuration_change(plugin_data);
	}
}

int AttachmentPoint::render(int double_buffer_in, int double_buffer_out, 
				long fragment_position_in, long fragment_position_out,
				long size, int node_number, 
				long source_position, long source_len, 
				FloatAutos *autos,
				FloatAuto **start_auto,
				FloatAuto **end_auto,
				int reverse)
{
// This routine is called only for multichannel plugins.  The virtual node calls 
// the plugin server directly for other plugins.
// 	int result = 0;
// 	plugin_servers.values[node_number]->arm_buffer(node_number,
// 							fragment_position_in,
// 							fragment_position_out,
// 							double_buffer_in,
// 							double_buffer_out);
// 
// 	for(int i = 0; i < virtual_plugins.total; i++)
// 	{
// // all virtual plugins must have render_count == 0 before rendering
// 		if(virtual_plugins.values[i]->render_count) result = 1;
// 	}
// 
// 	if(!result)
// 	{
// 		if(virtual_plugins.values[0]->plugin_server)
// 		{
// // Send automation to plugin
// 			virtual_plugins.values[0]->plugin_server->set_automation(autos, start_auto, end_auto, reverse);
// // Render plugin
// 			virtual_plugins.values[0]->plugin_server->process_realtime(source_len,
// 																		source_position,
// 																		size);
// 		}
// 
// // reset all virtual plugins
// 		for(int i = 0; i < virtual_plugins.total; i++)
// 		{
// 			virtual_plugins.values[i]->render_count = 1;
// 		}
// 	}
}
