#include "automation.h"
#include "autos.h"
#include "atrack.inc"
#include "bezierautos.h"
#include "colors.h"
#include "edl.h"
#include "edlsession.h"
#include "filexml.h"
#include "floatautos.h"
#include "intauto.h"
#include "intautos.h"
#include "panauto.h"
#include "panautos.h"
#include "intautos.h"
#include "track.h"
#include "transportque.inc"


Automation::Automation(EDL *edl, Track *track)
{
	this->edl = edl;
	this->track = track;
	camera_autos = 0;
	projector_autos = 0;
	pan_autos = 0;
	play_autos = 0;
	mute_autos = 0;
	fade_autos = 0;
	mode_autos = 0;
}

Automation::~Automation()
{
	if(play_autos) delete play_autos;
	if(mute_autos) delete mute_autos;

	if(mode_autos) delete mode_autos;
	if(fade_autos) delete fade_autos;
	if(pan_autos) delete pan_autos;
	if(camera_autos) delete camera_autos;
	if(projector_autos) delete projector_autos;
}

int Automation::create_objects()
{
	play_autos = new IntAutos(edl, track);
	play_autos->create_objects();
	((IntAuto*)play_autos->default_auto)->value = 1;
	mute_autos = new IntAutos(edl, track);
	mute_autos->create_objects();
	((IntAuto*)mute_autos->default_auto)->value = 0;
	return 0;
}

Automation& Automation::operator=(Automation& automation)
{
	copy_from(&automation);
	return *this;
}

void Automation::copy_from(Automation *automation)
{
	play_autos->copy_from(automation->play_autos);
	mute_autos->copy_from(automation->mute_autos);
	if(camera_autos) camera_autos->copy_from(automation->camera_autos);
	if(projector_autos) projector_autos->copy_from(automation->projector_autos);
	if(fade_autos) fade_autos->copy_from(automation->fade_autos);
	if(pan_autos) pan_autos->copy_from(automation->pan_autos);
	if(mode_autos) mode_autos->copy_from(automation->mode_autos);
}



int Automation::load(FileXML *file, int &current_plugin, int &current_channel)
{
	if(file->tag.title_is("PLAYAUTOS") && play_autos)
	{
		play_autos->load(file);
	}
	else
	if(file->tag.title_is("MUTEAUTOS") && mute_autos)
	{
		mute_autos->load(file);
	}
	else
	if(file->tag.title_is("FADEAUTOS") && fade_autos)
	{
		fade_autos->load(file);
	}
	else
	if(file->tag.title_is("PANAUTOS") && pan_autos)
	{
		pan_autos->load(file);
	}
	else
	if(file->tag.title_is("CAMERAAUTOS") && camera_autos)
	{
		camera_autos->load(file);
	}
	else
	if(file->tag.title_is("PROJECTORAUTOS") && projector_autos)
	{
		projector_autos->load(file);
	}
	else
	if(file->tag.title_is("MODEAUTOS") && mode_autos)
	{
		mode_autos->load(file);
	}
	return 0;
}

int Automation::copy(long start, 
	long end, 
	FileXML *file)
{
// Always save default of these
	if(play_autos /* && play_autos->total() */)
	{
		file->tag.set_title("PLAYAUTOS");
		file->append_tag();
		file->append_newline();
		play_autos->copy(start, 
						end, 
						file);
		file->append_newline();
		file->tag.set_title("/PLAYAUTOS");
		file->append_tag();
		file->append_newline();
	}

	if(mute_autos /* && mute_autos->total() */)
	{
		file->tag.set_title("MUTEAUTOS");
		file->append_tag();
		file->append_newline();
		mute_autos->copy(start, 
						end, 
						file);
		file->append_newline();
		file->tag.set_title("/MUTEAUTOS");
		file->append_tag();
		file->append_newline();
	}

	if(fade_autos /* && fade_autos->total() */)
	{
		file->tag.set_title("FADEAUTOS");
		file->append_tag();
		file->append_newline();
		fade_autos->copy(start, 
						end, 
						file);
		file->append_newline();
		file->tag.set_title("/FADEAUTOS");
		file->append_tag();
		file->append_newline();
	}

	if(camera_autos)
	{
		file->tag.set_title("CAMERAAUTOS");
		file->append_tag();
		file->append_newline();
		camera_autos->copy(start, 
						end, 
						file);
		file->tag.set_title("/CAMERAAUTOS");
		file->append_tag();
		file->append_newline();
	}

	if(projector_autos)
	{
		file->tag.set_title("PROJECTORAUTOS");
		file->append_tag();
		file->append_newline();
		projector_autos->copy(start, 
						end, 
						file);
		file->tag.set_title("/PROJECTORAUTOS");
		file->append_tag();
		file->append_newline();
	}

	if(pan_autos)
	{
		file->tag.set_title("PANAUTOS");
		file->append_tag();
		file->append_newline();
		pan_autos->copy(start, 
				end, 
				file);
		file->append_newline();
		file->tag.set_title("/PANAUTOS");
		file->append_tag();
		file->append_newline();
	}

	if(mode_autos)
	{
		file->tag.set_title("MODEAUTOS");
		file->append_tag();
		file->append_newline();
		mode_autos->copy(start, 
				end, 
				file);
		file->append_newline();
		file->tag.set_title("/MODEAUTOS");
		file->append_tag();
		file->append_newline();
	}

	return 0;
}


void Automation::clear(long start, long end)
{
	play_autos->clear(start, end, 1, 1);
	mute_autos->clear(start, end, 1, 1);
	fade_autos->clear(start, end, 1, 1);

	if(camera_autos)
		camera_autos->clear(start, end, 1, 1);
	if(projector_autos)
		projector_autos->clear(start, end, 1, 1);
	if(pan_autos)
		pan_autos->clear(start, end, 1, 1);
	if(mode_autos)
		mode_autos->clear(start, end, 1, 1);
}

void Automation::paste_silence(long start, long end)
{
// Unit conversion done in calling routine
	play_autos->paste_silence(start, end);
	mute_autos->paste_silence(start, end);
	fade_autos->paste_silence(start, end);

	if(camera_autos)
		camera_autos->paste_silence(start, end);
	if(projector_autos)
		projector_autos->paste_silence(start, end);
	if(pan_autos) 
		pan_autos->paste_silence(start, end);
	if(mode_autos) 
		mode_autos->paste_silence(start, end);
}

// We don't replace it in pasting but
// when inserting the first EDL of a load operation we need to replace
// the default keyframe.
void Automation::insert_track(Automation *automation, 
	long start_unit, 
	long length_units,
	int replace_default)
{
	play_autos->insert_track(automation->play_autos, 
		start_unit, 
		length_units, 
		replace_default);
	mute_autos->insert_track(automation->mute_autos, 
		start_unit, 
		length_units, 
		replace_default);
	if(camera_autos) camera_autos->insert_track(automation->camera_autos, 
		start_unit, 
		length_units, 
		replace_default);
	if(projector_autos) projector_autos->insert_track(automation->projector_autos, 
		start_unit, 
		length_units, 
		replace_default);
	if(fade_autos) fade_autos->insert_track(automation->fade_autos, 
		start_unit, 
		length_units, 
		replace_default);
	if(pan_autos) pan_autos->insert_track(automation->pan_autos, 
		start_unit, 
		length_units, 
		replace_default);
	if(mode_autos) mode_autos->insert_track(automation->mode_autos, 
		start_unit, 
		length_units, 
		replace_default);
}

void Automation::resample(double old_rate, double new_rate)
{
// Run resample for all the autos structures and all the keyframes
	play_autos->resample(old_rate, new_rate);
	mute_autos->resample(old_rate, new_rate);
	if(camera_autos) camera_autos->resample(old_rate, new_rate);
	if(projector_autos) projector_autos->resample(old_rate, new_rate);
	if(fade_autos) fade_autos->resample(old_rate, new_rate);
	if(pan_autos) pan_autos->resample(old_rate, new_rate);
	if(mode_autos) mode_autos->resample(old_rate, new_rate);
}




int Automation::direct_copy_possible(long start, int direction)
{
	return 1;
}

void Automation::dump()
{
	printf("   Automation %p\n", this);
	printf("    play_autos %p\n", play_autos);
	play_autos->dump();
	printf("    mute_autos %p\n", mute_autos);
	mute_autos->dump();
	if(fade_autos)
	{
		printf("    fade_autos %p\n", fade_autos);
		fade_autos->dump();
	}
	if(pan_autos)
	{
		printf("    pan_autos %p\n", pan_autos);
		pan_autos->dump();
	}
	if(camera_autos)
	{
		printf("    camera_autos %p\n", camera_autos);
		camera_autos->dump();
	}
	if(projector_autos)
	{
		printf("    projector_autos %p\n", projector_autos);
		projector_autos->dump();
	}
}
