#include "auto.h"
#include "cache.h"
#include "commonrender.h"
#include "edl.h"
#include "edlsession.h"
#include "virtualconsole.h"
#include "module.h"
#include "mwindow.h"
#include "patch.h"
#include "patchbay.h"
#include "playabletracks.h"
#include "preferences.h"
#include "renderengine.h"
#include "mainsession.h"
#include "intautos.h"
#include "track.h"
#include "tracks.h"
#include "transportque.h"


CommonRender::CommonRender(RenderEngine *renderengine)
 : Thread()
{
	this->renderengine = renderengine;
	reset_parameters();
}

CommonRender::~CommonRender()
{
	delete_vconsole();
	if(modules)
	{
		for(int i = 0; i < total_modules; i++)
			delete modules[i];
	 	delete [] modules;
	}
//printf("CommonRender::~CommonRender 1\n");
}

void CommonRender::reset_parameters()
{
	total_modules = 0;
	set_synchronous(1);
	modules = 0;
	vconsole = 0;
	done = 0;
	interrupt = 0;
	last_playback = 0;
	asynchronous = 0;
	restart_plugins = 0;
}

void CommonRender::arm_command()
{
//printf("CommonRender::arm_command 1 %d\n", renderengine->command->playbackstart);
	current_position = tounits(renderengine->command->playbackstart, 0);
//printf("CommonRender::arm_command 2 %d\n", current_position);
	create_modules();
//printf("CommonRender::arm_command 3\n");
	build_virtual_console();
//printf("CommonRender::arm_command 4\n");
	init_output_buffers();
	vconsole->start_playback();
	start_plugins();

	done = 0;
	interrupt = 0;
	last_playback = 0;
	restart_plugins = 0;
//printf("CommonRender::arm_command 5\n");
}



void CommonRender::create_modules()
{
//printf("VirtualConsole::create_modules 1\n");
// Create a module for every track, playable or not
	Track *current = renderengine->edl->tracks->first;
	int module = 0;

//printf("VirtualConsole::create_modules 1\n");
	if(!modules)
	{
		total_modules = get_total_tracks();
		modules = new Module*[total_modules];
//printf("VirtualConsole::create_modules 1 %d\n", total_modules);

		for(module = 0; module < total_modules && current; current = NEXT)
		{
//printf("VirtualConsole::create_modules 2 %d %p\n", total_modules, current);
			if(current->data_type == data_type)
			{
				modules[module] = new_module(current);
				modules[module]->create_objects();
				module++;
			}
		}
	}
//printf("VirtualConsole::create_modules 3\n");
}

void CommonRender::start_plugins()
{
//printf("CommonRender::start_plugins 1\n");
// Only start if virtual console was created
	if(restart_plugins)
	{
		for(int i = 0; i < total_modules; i++)
		{
			modules[i]->render_init();
		}
	}
//printf("CommonRender::start_plugins 2\n");
}

void CommonRender::build_virtual_console()
{
// Create new virtual console object
//printf("CommonRender::build_virtual_console 1 %p\n", vconsole);
	if(!vconsole)
	{
		vconsole = new_vconsole_object();
//printf("CommonRender::build_virtual_console 2\n");
	}

// Create nodes
	vconsole->create_objects();
//printf("CommonRender::build_virtual_console 3\n");
}

void CommonRender::start_command()
{
	if(renderengine->command->realtime)
	{
		start_lock.lock();
		Thread::start();
		start_lock.lock();
		start_lock.unlock();
	}
}

int CommonRender::restart_playback()
{
	delete_vconsole();
	vconsole = 0;
	create_modules();
	build_virtual_console();
	vconsole->start_playback();
	start_plugins();

	done = 0;
	interrupt = 0;
	last_playback = 0;
	restart_plugins = 0;
	return 0;
}

void CommonRender::delete_vconsole()
{
	if(vconsole)
		delete vconsole;
	vconsole = 0;
}

int CommonRender::get_boundaries(long &current_render_length)
{
	long loop_end = tounits(renderengine->edl->session->loop_end, 1);
	long loop_start = tounits(renderengine->edl->session->loop_start, 0);
	long start_position = tounits(renderengine->command->start_position, 0);
	long end_position = tounits(renderengine->command->end_position, 1);


//printf("CommonRender::get_boundaries 1 %d %d %d %f\n", current_render_length, current_position, end_position, renderengine->command->end_position);
// test absolute boundaries if no loop and not infinite
	if(renderengine->command->single_frame() || 
		(!renderengine->edl->session->loop_playback && 
		!renderengine->command->infinite))
	{
//printf("CommonRender::get_boundaries 1.1 %d\n", current_render_length);
		if(renderengine->command->get_direction() == PLAY_FORWARD)
		{
//printf("CommonRender::get_boundaries 1.2 %d\n", current_render_length);
			if(current_position + current_render_length >= end_position)
			{
				last_playback = 1;
				current_render_length = end_position - current_position;
//printf("CommonRender::get_boundaries 1.3 %d\n", current_render_length);
			}
		}
// reverse playback
		else               
		{
//printf("CommonRender::get_boundaries 1.4 %d\n", current_render_length);
			if(current_position - current_render_length <= start_position)
			{
				last_playback = 1;
				current_render_length = current_position - start_position;
//printf("CommonRender::get_boundaries 1.5 %d\n", current_render_length);
			}
		}
	}
//printf("CommonRender::get_boundaries 2 %d\n", current_render_length);

// test against loop boundaries
	if(!renderengine->command->single_frame() &&
		renderengine->edl->session->loop_playback && 
		!renderengine->command->infinite)
	{

		if(renderengine->command->get_direction() == PLAY_FORWARD)
		{
			long segment_end = current_position + current_render_length;
			if(segment_end > loop_end)
			{
				current_render_length = loop_end - current_position;
			}
		}
		else
		{
			long segment_end = current_position - current_render_length;
			if(segment_end < loop_start)
			{
				current_render_length = current_position - loop_start;
			}
		}
	}
	
//printf("CommonRender::get_boundaries 3 %d\n", current_render_length);
	if(renderengine->command->single_frame())
		current_render_length = 1;

//printf("CommonRender::get_boundaries 4 %d\n", current_render_length);
	return 0;
}

void CommonRender::run()
{
	start_lock.unlock();
}





















CommonRender::CommonRender(MWindow *mwindow, RenderEngine *renderengine)
 : Thread()
{
	this->mwindow = mwindow;
	this->renderengine = renderengine;
	current_position = 0;
	interrupt = 0;
	done = 0;
	last_playback = 0;
	vconsole = 0;
	asynchronous = 1;
}


int CommonRender::wait_for_completion()
{
	join();
}




// REMOVE
int CommonRender::test_virtualnodes(long current_position, 
	long &current_input_length, 
	int data_type)
{
	int result = 0;
	int direction = renderengine->command->get_direction();
	long nearest_auto, longest_duration;
	Track* current_track;
	Auto* current_auto;



//printf("ARender:: result %d length %ld\n", result, current_input_length);
	return result;
}


int CommonRender::advance_position(long current_render_length)
{
	long loop_end = tounits(renderengine->edl->session->loop_end, 1);
	long loop_start = tounits(renderengine->edl->session->loop_start, 0);

// advance the playback position
	if(renderengine->command->get_direction() == PLAY_REVERSE)
		current_position -= current_render_length;
	else
		current_position += current_render_length;

//printf("CommonRender::advance_position 1 %d\n", current_render_length);
// test loop again
	if(renderengine->edl->session->loop_playback && 
		!renderengine->command->infinite)
	{
		if(renderengine->command->get_direction() == PLAY_REVERSE)
		{
			if(current_position <= loop_start)
				current_position = loop_end;
		}
		else
		{
			if(current_position >= loop_end)
				current_position = loop_start + (current_position - loop_end);
		}
	}
	return 0;
}

long int CommonRender::tounits(double position, int round)
{
	return (long)position;
}

double CommonRender::fromunits(long position)
{
	return (double)position;
}
