#include "autos.h"
#include "cplayback.h"
#include "ctracking.h"
#include "cwindow.h"
#include "cwindowgui.h"
#include "defaults.h"
#include "edl.h"
#include "playbackengine.h"
#include "playtransport.h"
#include "preferences.h"
#include "track.h"
#include "tracks.h"
#include "transportque.h"
#include "mwindow.h"


CWindow::CWindow(MWindow *mwindow) : Thread()
{
	this->mwindow = mwindow;
}


CWindow::~CWindow()
{
	delete playback_engine;
	delete playback_cursor;
}

int CWindow::create_objects()
{
	destination = mwindow->defaults->get("CWINDOW_DESTINATION", 0);

	gui = new CWindowGUI(mwindow, this);
    gui->create_objects();

	playback_engine = new CPlayback(mwindow, this, gui->canvas);

//printf("CWindow::create_objects 1\n");
// Start command loop
	playback_engine->create_objects();
//printf("CWindow::create_objects 1\n");
	gui->transport->set_engine(playback_engine);
//printf("CWindow::create_objects 1\n");
	playback_cursor = new CTracking(mwindow, this);
//printf("CWindow::create_objects 1\n");
	playback_cursor->create_objects();
//printf("CWindow::create_objects 2\n");
    return 0;
}

Track* CWindow::calculate_affected_track()
{
	Track* affected_track = 0;
	for(Track *track = mwindow->edl->tracks->first;
		track;
		track = track->next)
	{
		if(track->data_type == TRACK_VIDEO &&
			track->record)
		{
			affected_track = track;
			break;
		}
	}
	return affected_track;
}

Auto* CWindow::calculate_affected_auto(Autos *autos)
{
	Auto* affected_auto = 0;
	affected_auto = autos->get_auto_for_editing();
	return affected_auto;
}

void CWindow::run()
{
	gui->run_window();
}

void CWindow::update_position()
{
//printf("CWindow::update_position 1\n");
	gui->lock_window();
	gui->slider->set_position();
	gui->unlock_window();
//printf("CWindow::update_position 1\n");
	playback_engine->que->send_command(CURRENT_FRAME, 
		CHANGE_NONE,
		mwindow->edl,
		1);
//printf("CWindow::update_position 2\n");
}


