#include "apanel.h"
#include "automation.h"
#include "autos.h"
#include "bezierauto.h"
#include "bezierautos.h"
#include "canvas.h"
#include "clip.h"
#include "cpanel.h"
#include "cplayback.h"
#include "cursors.h"
#include "cwindow.h"
#include "cwindowgui.h"
#include "cwindowtool.h"
#include "editpanel.h"
#include "edl.h"
#include "edlsession.h"
#include "localsession.h"
#include "mainclock.h"
#include "mainmenu.h"
#include "meterpanel.h"
#include "mwindow.h"
#include "mwindowgui.h"
#include "mwindow.h"
#include "playtransport.h"
#include "mainsession.h"
#include "theme.h"
#include "trackcanvas.h"
#include "tracks.h"
#include "transportque.h"

#define AUTO_ZOOM "Auto"

CWindowGUI::CWindowGUI(MWindow *mwindow, CWindow *cwindow)
 : BC_Window(PROGRAM_NAME ": Compositor",
 	mwindow->session->cwindow_x, 
    mwindow->session->cwindow_y, 
    mwindow->session->cwindow_w, 
    mwindow->session->cwindow_h,
    100,
    100,
    1,
    1,
    1)
{
	this->mwindow = mwindow;
    this->cwindow = cwindow;
	affected_track = 0;
	x_offset = 0;
	y_offset = 0;
	x_origin = 0;
	y_origin = 0;
	current_operation = CWINDOW_NONE;
	tool_panel = 0;
	translating_zoom = 0;
}

CWindowGUI::~CWindowGUI()
{
	if(tool_panel) delete tool_panel;
}

int CWindowGUI::create_objects()
{
//printf("CWindowGUI::create_objects 1\n");
	set_icon(mwindow->theme->cwindow_icon);

//printf("CWindowGUI::create_objects 1\n");
//	destinations.append(new BC_ListBoxItem("None"));

//printf("CWindowGUI::create_objects 1\n");
	mwindow->theme->get_cwindow_sizes(mwindow, this);
	mwindow->theme->draw_cwindow_bg(mwindow, this);
//printf("CWindowGUI::create_objects 1\n");

// 	automation_panel = new APanel(mwindow, 
// 		this, 
// 		mwindow->theme->cauto_x,
// 		mwindow->theme->cauto_y,
// 		mwindow->theme->cauto_w,
// 		mwindow->theme->cauto_h);
// 	automation_panel->create_objects();

//printf("CWindowGUI::create_objects 1\n");
// Meters required by composite panel
	meters = new CWindowMeters(mwindow, 
		this,
		mwindow->theme->cmeter_x,
		mwindow->theme->cmeter_y,
		mwindow->theme->cmeter_h);
	meters->create_objects();
//printf("CWindowGUI::create_objects 1\n");

//printf("CWindowGUI::create_objects 1\n");

	composite_panel = new CPanel(mwindow, 
		this, 
		mwindow->theme->ccomposite_x,
		mwindow->theme->ccomposite_y,
		mwindow->theme->ccomposite_w,
		mwindow->theme->ccomposite_h);
	composite_panel->create_objects();
//printf("CWindowGUI::create_objects 1\n");

	canvas = new CWindowCanvas(mwindow, this);
	canvas->create_objects(mwindow->edl);
//printf("CWindowGUI::create_objects 1\n");

	add_subwindow(slider = new CWindowSlider(mwindow, 
		cwindow, 
		mwindow->theme->cslider_x,
		mwindow->theme->cslider_y, 
		mwindow->theme->cslider_w));
//printf("CWindowGUI::create_objects 1\n");

	transport = new CWindowTransport(mwindow, 
		this, 
		mwindow->theme->ctransport_x, 
		mwindow->theme->ctransport_y);
	transport->create_objects();
	transport->set_slider(slider);

	edit_panel = new EditPanel(mwindow, 
		this, 
		mwindow->theme->cedit_x, 
		mwindow->theme->cedit_y,
		mwindow->edl->session->editing_mode, 
		0,
		1,
		0, 
		0,
		1,
		1,
		0,
		1,
		1,
		0,
		1,
		1,
		1);
	edit_panel->set_meters(meters);
	edit_panel->create_objects();
//printf("CWindowGUI::create_objects 1\n");

// 	add_subwindow(clock = new MainClock(mwindow, 
// 		mwindow->theme->ctime_x, 
// 		mwindow->theme->ctime_y));
// 
	zoom_panel = new CWindowZoom(mwindow, 
		this, 
		mwindow->theme->czoom_x, 
		mwindow->theme->czoom_y);
	zoom_panel->create_objects();
	zoom_panel->zoom_text->add_item(new BC_MenuItem(AUTO_ZOOM));
	if(!mwindow->edl->session->cwindow_scrollbars) zoom_panel->set_text(AUTO_ZOOM);
//printf("CWindowGUI::create_objects 1\n");

// 	destination = new CWindowDestination(mwindow, 
// 		this, 
// 		mwindow->theme->cdest_x,
// 		mwindow->theme->cdest_y);
// 	destination->create_objects();

// Must create after meter panel
	tool_panel = new CWindowTool(mwindow, this);
	tool_panel->Thread::start();
	
	set_operation(mwindow->edl->session->cwindow_operation);
	canvas->draw_refresh();
//printf("CWindowGUI::create_objects 2\n");
	return 0;
}

int CWindowGUI::translation_event()
{
	mwindow->session->cwindow_x = get_x();
	mwindow->session->cwindow_y = get_y();
	return 0;
}

int CWindowGUI::resize_event(int w, int h)
{
	mwindow->session->cwindow_x = get_x();
	mwindow->session->cwindow_y = get_y();
	mwindow->session->cwindow_w = w;
	mwindow->session->cwindow_h = h;

//printf("CWindowGUI::resize_event 1\n");
	mwindow->theme->get_cwindow_sizes(mwindow, this);
	mwindow->theme->draw_cwindow_bg(mwindow, this);

//printf("CWindowGUI::resize_event 1\n");
	composite_panel->reposition_buttons(mwindow->theme->ccomposite_x,
		mwindow->theme->ccomposite_y);
//printf("CWindowGUI::resize_event 1\n");

	canvas->reposition_window(mwindow->edl,
		mwindow->theme->ccanvas_x,
		mwindow->theme->ccanvas_y,
		mwindow->theme->ccanvas_w,
		mwindow->theme->ccanvas_h);

	slider->reposition_window(mwindow->theme->cslider_x,
		mwindow->theme->cslider_y, 
		mwindow->theme->cslider_w);
//printf("CWindowGUI::resize_event 1\n");

	transport->reposition_buttons(mwindow->theme->ctransport_x, 
		mwindow->theme->ctransport_y);
//printf("CWindowGUI::resize_event 1\n");

	edit_panel->reposition_buttons(mwindow->theme->cedit_x, 
		mwindow->theme->cedit_y);
//printf("CWindowGUI::resize_event 1\n");

//	clock->reposition_window(mwindow->theme->ctime_x, 
//		mwindow->theme->ctime_y);
//printf("CWindowGUI::resize_event 1\n");

	zoom_panel->reposition_window(mwindow->theme->czoom_x, 
		mwindow->theme->czoom_y);
//printf("CWindowGUI::resize_event 1\n");

// 	destination->reposition_window(mwindow->theme->cdest_x,
// 		mwindow->theme->cdest_y);
//printf("CWindowGUI::resize_event 1\n");

	meters->reposition_window(mwindow->theme->cmeter_x,
		mwindow->theme->cmeter_y,
		mwindow->theme->cmeter_h);
//printf("CWindowGUI::resize_event 2\n");

	BC_WindowBase::resize_event(w, h);
	return 1;
}

void CWindowGUI::set_operation(int value)
{
//printf("CWindowGUI::set_operation 1 %d\n", value);
	switch(value)
	{
		case CWINDOW_CROP:
			tool_panel->start_tool(new CWindowCropGUI(mwindow, tool_panel));
			canvas->draw_refresh();
			break;
		case CWINDOW_CAMERA:
			tool_panel->start_tool(new CWindowCameraGUI(mwindow, tool_panel));
			canvas->draw_refresh();
			break;
		case CWINDOW_PROJECTOR:
			tool_panel->start_tool(new CWindowProjectorGUI(mwindow, tool_panel));
			canvas->draw_refresh();
			break;
		default:
			tool_panel->stop_tool();
			canvas->draw_refresh();
			break;
	}
}

int CWindowGUI::close_event()
{
	hide_window();
	mwindow->session->show_cwindow = 0;
	mwindow->gui->lock_window();
	mwindow->gui->mainmenu->show_cwindow->set_checked(0);
	mwindow->gui->unlock_window();
	mwindow->save_defaults();
	return 1;
}


int CWindowGUI::keypress_event()
{
	int result = 0;

	switch(get_keypress())
	{
		case 'w':
		case 'W':
			close_event();
			result = 1;
			break;
	}
	
	if(!result) result = transport->keypress_event();
	
	return result;
}




CWindowMeters::CWindowMeters(MWindow *mwindow, CWindowGUI *gui, int x, int y, int h)
 : MeterPanel(mwindow, 
		gui,
		x,
		y,
		h,
		mwindow->edl->session->audio_channels,
		mwindow->edl->session->cwindow_meter)
{
	this->mwindow = mwindow;
	this->gui = gui;
}

CWindowMeters::~CWindowMeters()
{
}

int CWindowMeters::change_status_event()
{
//printf("CWindowMeters::change_status_event 1 %d\n", gui->meters->use_meters);
	mwindow->edl->session->cwindow_meter = gui->meters->use_meters;
	mwindow->theme->get_cwindow_sizes(mwindow, gui);
	gui->resize_event(gui->get_w(), gui->get_h());
	return 1;
}



#define MIN_ZOOM 0.25
#define MAX_ZOOM 4.0

CWindowZoom::CWindowZoom(MWindow *mwindow, CWindowGUI *gui, int x, int y)
 : ZoomPanel(mwindow, 
 	gui, 
	(long)mwindow->session->cwindow_zoom, 
	x, 
	y,
	100, 
	MIN_ZOOM, 
	MAX_ZOOM, 
	ZOOM_PERCENTAGE)
{
	this->mwindow = mwindow;
	this->gui = gui;
}

CWindowZoom::~CWindowZoom()
{
}

int CWindowZoom::handle_event()
{
	if(!strcasecmp(AUTO_ZOOM, get_text()))
	{
		mwindow->edl->session->cwindow_scrollbars = 0;
	}
	else
	{
		mwindow->edl->session->cwindow_scrollbars = 1;
	}

//printf("CWindowZoom::handle_event 1 %d %d\n", gui->canvas->get_xscroll(), gui->canvas->get_yscroll());
	gui->canvas->update_zoom(gui->canvas->get_xscroll(), 
		gui->canvas->get_yscroll(), 
		get_value());
//printf("CWindowZoom::handle_event 2 %d %d\n", gui->canvas->get_xscroll(), gui->canvas->get_yscroll());
	gui->canvas->reposition_window(mwindow->edl, 
		mwindow->theme->ccanvas_x,
		mwindow->theme->ccanvas_y,
		mwindow->theme->ccanvas_w,
		mwindow->theme->ccanvas_h);
//printf("CWindowZoom::handle_event 3 %d %d\n", gui->canvas->get_xscroll(), gui->canvas->get_yscroll());
	gui->canvas->draw_refresh();
//printf("CWindowZoom::handle_event 4 %d %d\n", gui->canvas->get_xscroll(), gui->canvas->get_yscroll());
	return 1;
}



CWindowSlider::CWindowSlider(MWindow *mwindow, CWindow *cwindow, int x, int y, int pixels)
 : BC_PercentageSlider(x, 
			y,
			0,
			pixels, 
			pixels, 
			0, 
			1, 
			0)
{
	this->mwindow = mwindow;
	this->cwindow = cwindow;
}

CWindowSlider::~CWindowSlider()
{
}

int CWindowSlider::handle_event()
{
	mwindow->gui->lock_window();
	mwindow->select_point((double)get_value());
	mwindow->gui->unlock_window();
	return 1;
}

void CWindowSlider::set_position()
{
	double new_length = mwindow->edl->tracks->total_playable_length();


	if(mwindow->edl->local_session->selectionstart >= new_length) 
		update(get_pointer_motion_range(), 
			new_length, 
			0, 
			new_length);
	else
		update(get_pointer_motion_range(), 
			mwindow->edl->local_session->selectionstart, 
			0, 
			new_length);
}





// CWindowDestination::CWindowDestination(MWindow *mwindow, CWindowGUI *cwindow, int x, int y)
//  : BC_PopupTextBox(cwindow, 
//  	&cwindow->destinations, 
// 	cwindow->destinations.values[cwindow->cwindow->destination]->get_text(),
// 	x, 
// 	y, 
// 	70, 
// 	200)
// {
// 	this->mwindow = mwindow;
// 	this->cwindow = cwindow;
// }
// 
// CWindowDestination::~CWindowDestination()
// {
// }
// 
// int CWindowDestination::handle_event()
// {
// 	return 1;
// }


CWindowTransport::CWindowTransport(MWindow *mwindow, 
	CWindowGUI *gui, 
	int x, 
	int y)
 : PlayTransport(mwindow, 
	gui, 
	x, 
	y)
{
	this->gui = gui;
}

EDL* CWindowTransport::get_edl()
{
	return mwindow->edl;
}

void CWindowTransport::goto_start()
{
	mwindow->goto_start();
}

void CWindowTransport::goto_end()
{
	mwindow->goto_end();
}



CWindowCanvas::CWindowCanvas(MWindow *mwindow, CWindowGUI *gui)
 : Canvas(gui,
		mwindow->theme->ccanvas_x,
		mwindow->theme->ccanvas_y,
		mwindow->theme->ccanvas_w,
		mwindow->theme->ccanvas_h,
		0,
		0,
		mwindow->edl->session->cwindow_scrollbars,
		1)
{
	this->mwindow = mwindow;
	this->gui = gui;
}

void CWindowCanvas::update_zoom(int x, int y, float zoom)
{
	use_scrollbars = mwindow->edl->session->cwindow_scrollbars;

	mwindow->edl->session->cwindow_xscroll = x;
	mwindow->edl->session->cwindow_yscroll = y;
	mwindow->edl->session->cwindow_zoom = zoom;
}

int CWindowCanvas::get_xscroll()
{
	return mwindow->edl->session->cwindow_xscroll;
}

int CWindowCanvas::get_yscroll()
{
	return mwindow->edl->session->cwindow_yscroll;
}


float CWindowCanvas::get_zoom()
{
	return mwindow->edl->session->cwindow_zoom;
}

void CWindowCanvas::draw_refresh()
{
	if(!canvas->video_is_on())
	{
		if(refresh_frame)
		{
			int in_x, in_y, in_w, in_h, out_x, out_y, out_w, out_h;
			get_transfers(mwindow->edl, 
				in_x, 
				in_y, 
				in_w, 
				in_h, 
				out_x, 
				out_y, 
				out_w, 
				out_h);


// printf("CWindowCanvas::draw_refresh %d %d %d %d -> %d %d %d %d\n", in_x, in_y, in_w, in_h, out_x, out_y, out_w, out_h);
// 			canvas->clear_box(0, 0, canvas->get_w(), canvas->get_h());


			canvas->draw_vframe(refresh_frame,
					out_x, 
					out_y, 
					out_w, 
					out_h,
					in_x, 
					in_y, 
					in_w, 
					in_h,
					0);
		}
		else
		{
			canvas->clear_box(0, 0, canvas->get_w(), canvas->get_h());
		}
	}
	
	draw_overlays();
	canvas->flash();
	canvas->flush();
}

#define CROPHANDLE_W 10
#define CROPHANDLE_H 10

void CWindowCanvas::draw_crophandle(int x, int y)
{
	canvas->draw_box(x, y, CROPHANDLE_W, CROPHANDLE_H);
}

void CWindowCanvas::draw_overlays()
{
	switch(mwindow->edl->session->cwindow_operation)
	{
		case CWINDOW_CAMERA:
			draw_bezier(1);
			break;
		
		case CWINDOW_PROJECTOR:
			draw_bezier(0);
			break;

		case CWINDOW_CROP:
			draw_crop();
			break;
	}
}

void CWindowCanvas::reset_keyframe(int do_camera)
{
	BezierAuto *keyframe = 0;
	Track *affected_track = 0;

	affected_track = gui->cwindow->calculate_affected_track();

	if(affected_track)
	{
//printf("CWindowCanvas::button_press_event 1\n");
		if(do_camera)
			keyframe = (BezierAuto*)gui->cwindow->calculate_affected_auto(affected_track->automation->camera_autos);
		else
			keyframe = (BezierAuto*)gui->cwindow->calculate_affected_auto(affected_track->automation->projector_autos);

//printf("CWindowCanvas::button_press_event 1 %p\n", gui->affected_auto);
		keyframe->center_x = 0;
		keyframe->center_y = 0;
		keyframe->center_z = 1;
//printf("CWindowCanvas::button_press_event 1\n");

		mwindow->sync_parameters(CHANGE_PARAMS);
		gui->cwindow->playback_engine->que->send_command(CURRENT_FRAME, 
			CHANGE_NONE,
			mwindow->edl,
			1);
	}
}

void CWindowCanvas::reset_camera()
{
	reset_keyframe(1);
}

void CWindowCanvas::reset_projector()
{
	reset_keyframe(0);
}

int CWindowCanvas::test_crop(int button_press, int &redraw)
{
	int result = 0;
	int handle_selected = -1;
	float x1 = mwindow->edl->session->crop_x1;
	float y1 = mwindow->edl->session->crop_y1;
	float x2 = mwindow->edl->session->crop_x2;
	float y2 = mwindow->edl->session->crop_y2;
	float cursor_x = get_cursor_x();
	float cursor_y = get_cursor_y();
	
	output_to_canvas(mwindow->edl, 0, x1, y1);
	output_to_canvas(mwindow->edl, 0, x2, y2);

	if(gui->current_operation == CWINDOW_CROP)
	{
		handle_selected = gui->crop_handle;
	}
	else
	if(cursor_x >= x1 && cursor_x < x1 + CROPHANDLE_W &&
		cursor_y >= y1 && cursor_y < y1 + CROPHANDLE_H)
	{
		handle_selected = 0;
		gui->crop_origin_x = x1;
		gui->crop_origin_y = y1;
	}
	else
	if(cursor_x >= x2 - CROPHANDLE_W && cursor_x < x2 &&
		cursor_y >= y1 && cursor_y < y1 + CROPHANDLE_H)
	{
		handle_selected = 1;
		gui->crop_origin_x = x2;
		gui->crop_origin_y = y1;
	}
	else
	if(cursor_x >= x1 && cursor_x < x1 + CROPHANDLE_W &&
		cursor_y >= y2 - CROPHANDLE_H && cursor_y < y2)
	{
		handle_selected = 2;
		gui->crop_origin_x = x1;
		gui->crop_origin_y = y2;
	}
	else
	if(cursor_x >= x2 - CROPHANDLE_W && cursor_x < x2 &&
		cursor_y >= y2 - CROPHANDLE_H && cursor_y < y2)
	{
		handle_selected = 3;
		gui->crop_origin_x = x2;
		gui->crop_origin_y = y2;
	}
	else
// Start new box
	{
		gui->crop_origin_x = cursor_x;
		gui->crop_origin_y = cursor_y;
		canvas_to_output(mwindow->edl, 
			0, 
			gui->crop_origin_x, 
			gui->crop_origin_y);
	}

// printf("test crop %d %d\n", 
// 	gui->current_operation,
// 	handle_selected);

// Start dragging.
	if(button_press)
	{
		gui->current_operation = CWINDOW_CROP;
		gui->crop_handle = handle_selected;
		result = 1;

		if(handle_selected < 0) 
		{
			x2 = x1 = cursor_x;
			y2 = y1 = cursor_y;
			canvas_to_output(mwindow->edl,
				      0,
				      x1,
				      y1);
			canvas_to_output(mwindow->edl,
				0, 
				x2, 
				y2);
			mwindow->edl->session->crop_x1 = (int)x1;
			mwindow->edl->session->crop_y1 = (int)y1;
			mwindow->edl->session->crop_x2 = (int)x2;
			mwindow->edl->session->crop_y2 = (int)y2;
			redraw = 1;
		}
	}
    else
// Update dragging
	if(gui->current_operation == CWINDOW_CROP)
	{
		float x_difference, y_difference;
		if(gui->crop_handle >= 0)
		{
			float zoom_x, zoom_y, conformed_w, conformed_h;
			get_zooms(mwindow->edl, 0, zoom_x, zoom_y, conformed_w, conformed_h);
			x_difference = cursor_x - gui->x_origin;
		}

		canvas_to_output(mwindow->edl,
				  0,
				  cursor_x,
				  cursor_y);
		x1 = gui->mwindow->edl->session->crop_x1;
		y1 = gui->mwindow->edl->session->crop_y1;
		x2 = gui->mwindow->edl->session->crop_x2;
		y2 = gui->mwindow->edl->session->crop_y2;
		switch(gui->crop_handle)
		{
			case -1:
				x1 = gui->crop_origin_x;
				y1 = gui->crop_origin_y;
				x2 = gui->crop_origin_x;
				y2 = gui->crop_origin_y;
				if(cursor_x < gui->crop_origin_x)
				{
					if(cursor_y < gui->crop_origin_y)
					{
						x1 = cursor_x;
						y1 = cursor_y;
					}
					else
					if(cursor_y >= gui->crop_origin_y)
					{
						x1 = cursor_x;
						y2 = cursor_y;
					}
				}
				else
				if(cursor_x  >= gui->crop_origin_x)
				{
					if(cursor_y < gui->crop_origin_y)
					{
						y1 = cursor_y;
						x2 = cursor_x;
					}
					else
					if(cursor_y >= gui->crop_origin_y)
					{
						x2 = cursor_x;
						y2 = cursor_y;
					}
				}

// printf("test crop %d %d %d %d\n", 
// 	mwindow->edl->session->crop_x1,
// 	mwindow->edl->session->crop_y1,
// 	mwindow->edl->session->crop_x2,
// 	mwindow->edl->session->crop_y2);
				break;
			case 0:
				x1 = cursor_x;
				y1 = cursor_y;
				break;
			case 1:
				x2 = cursor_x;
				y1 = cursor_y;
				break;
			case 2:
				x1 = cursor_x;
				y2 = cursor_y;
				break;
			case 3:
				x2 = cursor_x;
				y2 = cursor_y;
				break;
		}

		if(!EQUIV(mwindow->edl->session->crop_x1, x1) ||
			!EQUIV(mwindow->edl->session->crop_x2, x2) ||
			!EQUIV(mwindow->edl->session->crop_y1, y1) ||
			!EQUIV(mwindow->edl->session->crop_y2, y2))
		{
			mwindow->edl->session->crop_x1 = (int)x1;
			mwindow->edl->session->crop_y1 = (int)y1;
			mwindow->edl->session->crop_x2 = (int)x2;
			mwindow->edl->session->crop_y2 = (int)y2;
			result = 1;
			redraw = 1;
		}
	}
	else
// Update cursor font
	if(handle_selected >= 0)
	{
		switch(handle_selected)
		{
			case 0:
				set_cursor(UPLEFT_RESIZE);
				break;
			case 1:
				set_cursor(UPRIGHT_RESIZE);
				break;
			case 2:
				set_cursor(DOWNLEFT_RESIZE);
				break;
			case 3:
				set_cursor(DOWNRIGHT_RESIZE);
				break;
		}
		result = 1;
	}
	else
	{
		set_cursor(ARROW_CURSOR);
	}
#define CLAMP(x, y, z) ((x) = ((x) < (y) ? (y) : ((x) > (z) ? (z) : (x))))
	
	if(redraw)
	{
		CLAMP(mwindow->edl->session->crop_x1, 0, mwindow->edl->calculate_output_w(0));
		CLAMP(mwindow->edl->session->crop_x2, 0, mwindow->edl->calculate_output_w(0));
		CLAMP(mwindow->edl->session->crop_y1, 0, mwindow->edl->calculate_output_h(0));
		CLAMP(mwindow->edl->session->crop_y2, 0, mwindow->edl->calculate_output_h(0));
// printf("CWindowCanvas::test_crop %d %d %d %d\n", 
// 	mwindow->edl->session->crop_x2,
// 	mwindow->edl->session->crop_y2,
// 	mwindow->edl->calculate_output_w(0), 
// 	mwindow->edl->calculate_output_h(0));
	}
	return result;
}


void CWindowCanvas::draw_crop()
{
	canvas->set_inverse();
	canvas->set_color(WHITE);

	float x1 = mwindow->edl->session->crop_x1;
	float y1 = mwindow->edl->session->crop_y1;
	float x2 = mwindow->edl->session->crop_x2;
	float y2 = mwindow->edl->session->crop_y2;

//printf("CWindowCanvas::draw_overlays 1 %.0f %.0f\n", x1, y1);
	output_to_canvas(mwindow->edl, 0, x1, y1);
	output_to_canvas(mwindow->edl, 0, x2, y2);
//printf("CWindowCanvas::draw_overlays 2 %.0f %.0f\n", x1, y1);

	if(x2 - x1 && y2 - y1)
		canvas->draw_rectangle((int)x1, 
			(int)y1, 
			(int)(x2 - x1), 
			(int)(y2 - y1));

	draw_crophandle((int)x1, (int)y1);
	draw_crophandle((int)x2 - CROPHANDLE_W, (int)y1);
	draw_crophandle((int)x1, (int)y2 - CROPHANDLE_H);
	draw_crophandle((int)x2 - CROPHANDLE_W, (int)y2 - CROPHANDLE_H);
	canvas->set_opaque();
}



#define BEZIER_W 20
#define BEZIER_H 20

void CWindowCanvas::draw_bezier_center(BezierAuto *auto_)
{
	float center_x, center_y, center_z;
	BezierAuto *before = 0, *after = 0;

// Get center of current frame.  Draw everything relative to this.
	((BezierAutos*)(auto_->autos))->get_center(center_x, center_y, center_z, 
			mwindow->edl->local_session->selectionstart, 
			PLAY_FORWARD, 
			&before, 
			&after);
	
	float auto_x, auto_y;
	
	auto_x = auto_->center_x;
	auto_y = auto_->center_y;
	
	
	int screen_x = (int)(canvas->get_w() / 2 - (center_x - auto_x));
	int screen_y = (int)(canvas->get_h() / 2 - (center_y - auto_y));
	
	canvas->set_inverse();
	canvas->set_color(WHITE);
	
	canvas->draw_line(screen_x - BEZIER_W / 2, 
		screen_y - BEZIER_H / 2,
		screen_x + BEZIER_W / 2,
		screen_y + BEZIER_H / 2);
	canvas->draw_line(screen_x - BEZIER_W / 2, 
		screen_y + BEZIER_H / 2,
		screen_x + BEZIER_W / 2,
		screen_y - BEZIER_H / 2);
	
	canvas->set_opaque();
}



void CWindowCanvas::draw_bezier_joining(BezierAuto *first, 
	BezierAuto *last, 
	int dashed)
{
	if(first == last) return;
	
	float center_x, center_y, center_z;
	BezierAuto *before = 0, *after = 0;

// Get center of current frame.  Draw everything relative to this.
	((BezierAutos*)(first->autos))->get_center(center_x, center_y, center_z, 
			mwindow->edl->local_session->selectionstart, 
			PLAY_FORWARD, 
			&before, 
			&after);

#define SEGMENTS 5
	float old_x, old_y, old_z;
	canvas->set_inverse();
	canvas->set_color(WHITE);

	for(long frame = first->position; 
		frame < last->position; 
		frame += (last->position - first->position) / SEGMENTS)
	{
		float new_x, new_y, new_z;
		
		((BezierAutos*)(first->autos))->get_center(new_x, new_y, new_z, 
				frame, 
				PLAY_FORWARD, 
				&before, 
				&after);
		if(frame == first->position)
		{
			old_x = new_x;
			old_y = new_y;
			old_z = new_z;
		}
		
		int screen_x1, screen_y1, screen_x2, screen_y2;
		screen_x1 = (int)(canvas->get_w() / 2 - (center_x - old_x));
		screen_y1 = (int)(canvas->get_h() / 2 - (center_y - old_y));
		screen_x2 = (int)(canvas->get_w() / 2 - (center_x - new_x));
		screen_y2 = (int)(canvas->get_h() / 2 - (center_y - new_y));
		
		canvas->draw_line(screen_x1, screen_y1, screen_x2, screen_y2);
		
		
	}

	canvas->set_opaque();
}



void CWindowCanvas::draw_bezier(int do_camera)
{
	Track *track = gui->cwindow->calculate_affected_track();
	BezierAutos *autos;
	BezierAuto *first = 0, *mid = 0, *last = 0;

// No track at initialization
	if(!track) return;

	if(do_camera)
		autos = track->automation->camera_autos;
	else
		autos = track->automation->projector_autos;




// Rules for which autos to draw:
//
// No automation besides default:
//     mid = default
//
// Automation on or after current position only
//     mid = first, last = next
//
// Automation before current position only
//     mid = last, first = previous
// 
// Automation on current position only
//     mid = first
//
// Automation before and after current position
//     first = previous, mid = next, last = next->next

	for(first = (BezierAuto*)autos->last; first; first = (BezierAuto*)first->previous)
		if(first->position < mwindow->edl->local_session->selectionstart)
			break;

	if(first)
	{
		mid = (BezierAuto*)first->next;
		if(!mid)
		{
			mid = first;
			first = (BezierAuto*)first->previous;
		}
	}
	else
		mid = (BezierAuto*)autos->first;

	if(mid)
		last = (BezierAuto*)mid->next;
	else
		mid = (BezierAuto*)autos->default_auto;




//printf("draw_bezier 1 %p %p %p\n", first, mid, last);



// Draw centers of autos
	if(first) draw_bezier_center(first);
	if(mid) draw_bezier_center(mid);
	if(last) draw_bezier_center(last);




// Draw joining line after first auto
	if(first && mid)
		draw_bezier_joining(first, mid, 1);

	if(mid && last)
		draw_bezier_joining(mid, last, 0);



}



int CWindowCanvas::test_bezier(int button_press, 
	int &redraw, 
	int &rerender,
	int do_camera)
{
	int result = 0;

//printf("CWindowCanvas::test_bezier 1\n");
// Processing drag operation
	if(!button_press)
	{
		BezierAuto *keyframe = (BezierAuto*)gui->affected_auto;
		float last_center_x = keyframe->center_x;
		float last_center_y = keyframe->center_y;
		float last_center_z = keyframe->center_z;

//printf("CWindowCanvas::cursor_motion_event 1 %f %d %f\n", 
//	gui->center_y, get_cursor_y(), gui->y_origin);


		if(gui->shift_down() && !gui->translating_zoom)
		{
			gui->translating_zoom = 1;
			calculate_origin();
			gui->center_x = keyframe->center_x;
			gui->center_y = keyframe->center_y;
			gui->center_z = keyframe->center_z;
		}
		else
		if(!gui->shift_down() && gui->translating_zoom)
		{
			gui->translating_zoom = 0;
			calculate_origin();
			gui->center_x = keyframe->center_x;
			gui->center_y = keyframe->center_y;
			gui->center_z = keyframe->center_z;
		}

		if(gui->translating_zoom)
		{
			keyframe->center_z = gui->center_z + (get_cursor_y() - gui->y_origin) / 128;
		}
		else
		{
			keyframe->center_x = gui->center_x + get_cursor_x() - gui->x_origin;
			keyframe->center_y = gui->center_y + get_cursor_y() - gui->y_origin;
		}

//printf("CWindowCanvas::cursor_motion_event 2 %f %f %f\n", 
//	keyframe->center_x, keyframe->center_y, keyframe->center_z);
		if(!EQUIV(last_center_x,  keyframe->center_x) ||
			!EQUIV(last_center_y, keyframe->center_y) ||
			!EQUIV(last_center_z, keyframe->center_z))
		{
			rerender = 1;
		}

		result = 1;
	}
	else
// Begin drag operation
	{
// Get affected track off of the first recordable video track.
// Calculating based on the alpha channel would require rendering
// each layer and its effects and trapping the result in VirtualVNode before
// compositing onto the output.
		gui->affected_track = gui->cwindow->calculate_affected_track();
//printf("CWindowCanvas::test_bezier 2\n");
		if(gui->affected_track)
		{
			gui->current_operation = mwindow->edl->session->cwindow_operation;
		
//printf("CWindowCanvas::test_bezier 3 %d\n", gui->current_operation);
			if(gui->current_operation == CWINDOW_CAMERA)
				gui->affected_auto = 
					gui->cwindow->calculate_affected_auto(gui->affected_track->automation->camera_autos);
			else
			if(gui->current_operation == CWINDOW_PROJECTOR)
				gui->affected_auto = 
					gui->cwindow->calculate_affected_auto(gui->affected_track->automation->projector_autos);
//printf("CWindowCanvas::test_bezier 4\n");

//printf("CWindowCanvas::button_press_event 1 %p\n", gui->affected_auto);
			mwindow->gui->lock_window();
			mwindow->gui->canvas->draw_overlays();
			mwindow->gui->canvas->flash();
			mwindow->gui->unlock_window();
//printf("CWindowCanvas::test_bezier 5\n");

			BezierAuto *keyframe = (BezierAuto*)gui->affected_auto;
			gui->center_x = keyframe->center_x;
			gui->center_y = keyframe->center_y;
			gui->center_z = keyframe->center_z;
//printf("CWindowCanvas::test_bezier 6\n");
			result = 1;
//printf("CWindowCanvas::button_press_event 1\n");
		}
	}
	
	return result;
}

int CWindowCanvas::test_zoom(int &redraw)
{
	int result = 0;
	float zoom = get_zoom();
	
	float x = get_cursor_x();
	float y = get_cursor_y();
	canvas_to_output(mwindow->edl, 
			0, 
			x, 
			y);

//printf("CWindowCanvas::test_zoom 1 %f %f\n", x, y);

// Zoom out
	if(gui->ctrl_down() || gui->shift_down())
	{
		if(zoom > MIN_ZOOM)
		{
			zoom /= 2;
			x -= w_visible;
			y -= h_visible;
		}
	}
	else
// Zoom in
	{
		if(zoom < MAX_ZOOM)
		{
			zoom *= 2;
			x -= w_visible / 4;
			y -= h_visible / 4;
		}
	}


	int x_i = (int)x;
	int y_i = (int)y;
	check_boundaries(mwindow->edl, x_i, y_i, zoom);

//printf("CWindowCanvas::test_zoom 2 %d %d\n", x_i, y_i);
	update_zoom(x_i, 
			y_i, 
			zoom);
	reposition_window(mwindow->edl, 
			mwindow->theme->ccanvas_x,
			mwindow->theme->ccanvas_y,
			mwindow->theme->ccanvas_w,
			mwindow->theme->ccanvas_h);
	redraw = 1;
	result = 1;

	
	gui->zoom_panel->update(zoom);
	
	return result;
}


void CWindowCanvas::calculate_origin()
{
	gui->x_origin = get_cursor_x();
	gui->y_origin = get_cursor_y();
	canvas_to_output(mwindow->edl, 0, gui->x_origin, gui->y_origin);
}


int CWindowCanvas::cursor_leave_event()
{
	set_cursor(ARROW_CURSOR);
	return 1;
}

int CWindowCanvas::cursor_enter_event()
{
	int redraw = 0;
	switch(mwindow->edl->session->cwindow_operation)
	{
		case CWINDOW_CAMERA:
		case CWINDOW_PROJECTOR:
			set_cursor(CROSS_CURSOR);
			break;
		case CWINDOW_ZOOM:
			set_cursor(CROSS_CURSOR);
			break;
		case CWINDOW_CROP:
			test_crop(0, redraw);
			break;
		case CWINDOW_PROTECT:
			set_cursor(ARROW_CURSOR);
			break;
	}
	return 1;
}

int CWindowCanvas::cursor_motion_event()
{
	int redraw = 0, result = 0, rerender = 0;
//printf("CWindowCanvas::cursor_motion_event %d\n", gui->current_operation);


	switch(gui->current_operation)
	{
		case CWINDOW_SCROLL:
		{
			float zoom = get_zoom();
			int x = (int)(gui->x_origin - get_cursor_x() / zoom + gui->x_offset);
			int y = (int)(gui->y_origin - get_cursor_y() / zoom + gui->y_offset);

			check_boundaries(mwindow->edl, x, y, zoom);

			update_zoom(x, 
				y, 
				zoom);
			update_scrollbars();
			redraw = 1;
			result = 1;
			break;
		}

		case CWINDOW_CAMERA:
			result = test_bezier(0, redraw, rerender, 1);
			break;

		case CWINDOW_PROJECTOR:
			result = test_bezier(0, redraw, rerender, 0);
			break;

		case CWINDOW_CROP:
//printf("CWindowCanvas::cursor_motion_event 1 %d %d\n", x, y);
			result = test_crop(0, redraw);
			break;
	}



	if(!result)
	{
		switch(mwindow->edl->session->cwindow_operation)
		{
			case CWINDOW_CROP:
				result = test_crop(0, redraw);
				break;
		}
	}


	if(rerender)
	{
		mwindow->sync_parameters(CHANGE_PARAMS);
		gui->cwindow->playback_engine->que->send_command(CURRENT_FRAME, 
			CHANGE_NONE,
			mwindow->edl,
			1);
	}

	if(redraw) draw_refresh();

	return result;
}

int CWindowCanvas::button_press_event()
{
	int result = 0;
	int redraw = 0;
	int rerender = 0;

	if(Canvas::button_press_event()) return 1;

	gui->translating_zoom = gui->shift_down(); 

	calculate_origin();
//printf("CWindowCanvas::button_press_event 2 %f %f\n", gui->x_origin, gui->y_origin, gui->x_origin, gui->y_origin);

	float zoom_x, zoom_y, conformed_w, conformed_h;
	get_zooms(mwindow->edl, 0, zoom_x, zoom_y, conformed_w, conformed_h);
	gui->x_offset = get_x_offset(mwindow->edl, 0, zoom_x, conformed_w, conformed_h);
	gui->y_offset = get_y_offset(mwindow->edl, 0, zoom_y, conformed_w, conformed_h);

// Scroll view
	if(get_buttonpress() == 2)
	{
		gui->current_operation = CWINDOW_SCROLL;
		result = 1;
	}
	else
// Adjust parameter
	{
		switch(mwindow->edl->session->cwindow_operation)
		{
			case CWINDOW_CAMERA:
				result = test_bezier(1, redraw, rerender, 1);
				break;

			case CWINDOW_PROJECTOR:
				result = test_bezier(1, redraw, rerender, 0);
				break;

			case CWINDOW_ZOOM:
				result = test_zoom(redraw);
				break;

			case CWINDOW_CROP:
				result = test_crop(1, redraw);
				break;
		}
	}
	if(redraw) draw_refresh();
	return result;
}

int CWindowCanvas::button_release_event()
{
	int result = 0;

	switch(gui->current_operation)
	{
		case CWINDOW_SCROLL:
			result = 1;
			break;
	}

	gui->current_operation = CWINDOW_NONE;
//printf("CWindowCanvas::button_release_event %d\n", result);
	return result;
}

void CWindowCanvas::zoom_resize_window(float percentage)
{
	int canvas_w, canvas_h;
	calculate_sizes(mwindow->edl->get_aspect_ratio(), 
		mwindow->edl->calculate_output_w(0), 
		mwindow->edl->calculate_output_h(0), 
		percentage,
		canvas_w,
		canvas_h);
	int new_w, new_h;
	new_w = canvas_w + (gui->get_w() - mwindow->theme->ccanvas_w);
	new_h = canvas_h + (gui->get_h() - mwindow->theme->ccanvas_h);
	gui->resize_window(new_w, new_h);
	gui->resize_event(new_w, new_h);
}
