#ifndef CWINDOWGUI_H
#define CWINDOWGUI_H

#include "apanel.inc"
#include "auto.inc"
#include "bezierauto.inc"
#include "bezierautos.inc"
#include "canvas.h"
#include "cpanel.inc"
#include "cwindow.inc"
#include "cwindowtool.inc"
#include "editpanel.inc"
#include "guicast.h"
#include "mainclock.inc"
#include "meterpanel.h"
#include "mwindow.inc"
#include "playtransport.h"
#include "thread.h"
#include "track.inc"
#include "zoompanel.h"

class CWindowZoom;
class CWindowSlider;
class CWindowReset;
class CWindowDestination;
class CWindowMeters;
class CWindowTransport;
class CWindowCanvas;

class CWindowGUI : public BC_Window
{
public:
	CWindowGUI(MWindow *mwindow, CWindow *cwindow);
	~CWindowGUI();

    int create_objects();
	int resize_event(int w, int h);
	int close_event();
	int keypress_event();
	int translation_event();
	void set_operation(int value);

	MWindow *mwindow;
    CWindow *cwindow;
	EditPanel *edit_panel;
	APanel *automation_panel;
	CPanel *composite_panel;
	CWindowZoom *zoom_panel;
	CWindowSlider *slider;
	CWindowReset *reset;
	CWindowTransport *transport;
	CWindowCanvas *canvas;
//	MainClock *clock;

// REMOVE
	CWindowDestination *destination;

	CWindowMeters *meters;


// REMOVE
	ArrayList<BC_ListBoxItem*> destinations;
	CWindowTool *tool_panel;

// Cursor motion modification being done
	int current_operation;
// Track being modified in canvas
	Track *affected_track;
// Keyframe being modified
	Auto *affected_auto;
// Cursor location during the last button press relative to output
// and offset by scroll bars
	float x_offset, y_offset;
// Scrollbar offsets during last button press relative to output
	float x_origin, y_origin;
// Crop handle being dragged
	int crop_handle;
// Origin of crop handle during last button press
	float crop_origin_x, crop_origin_y;
// Origin for camera and projector operations during last button press
	float center_x, center_y, center_z;
// Must recalculate the origin when pressing shift.
// Switch toggle on and off to recalculate origin.
	int translating_zoom;
};

class CWindowMeters : public MeterPanel
{
public:
	CWindowMeters(MWindow *mwindow, CWindowGUI *gui, int x, int y, int h);
	~CWindowMeters();
	
	int change_status_event();
	
	MWindow *mwindow;
	CWindowGUI *gui;
};

class CWindowZoom : public ZoomPanel
{
public:
	CWindowZoom(MWindow *mwindow, CWindowGUI *gui, int x, int y);
	~CWindowZoom();
	int handle_event();
	MWindow *mwindow;
	CWindowGUI *gui;
};


class CWindowSlider : public BC_PercentageSlider
{
public:
	CWindowSlider(MWindow *mwindow, CWindow *cwindow, int x, int y, int pixels);
	~CWindowSlider();

	int handle_event();
	void set_position();

	MWindow *mwindow;
	CWindow *cwindow;
};

class CWindowReset : public BC_Button
{
public:
	CWindowReset(MWindow *mwindow, CWindowGUI *cwindow, int x, int y);
	~CWindowReset();
	int handle_event();
	CWindowGUI *cwindow;
	MWindow *mwindow;
};

// class CWindowDestination : public BC_PopupTextBox
// {
// public:
// 	CWindowDestination(MWindow *mwindow, CWindowGUI *cwindow, int x, int y);
// 	~CWindowDestination();
// 	int handle_event();
// 	CWindowGUI *cwindow;
// 	MWindow *mwindow;
// };

class CWindowTransport : public PlayTransport
{
public:
	CWindowTransport(MWindow *mwindow, 
		CWindowGUI *gui, 
		int x, 
		int y);
	EDL* get_edl();
	void goto_start();
	void goto_end();

	CWindowGUI *gui;
};


class CWindowCanvas : public Canvas
{
public:
	CWindowCanvas(MWindow *mwindow, CWindowGUI *gui);

	void zoom_resize_window(float percentage);
	void update_zoom(int x, int y, float zoom);
	int get_xscroll();
	int get_yscroll();
	float get_zoom();
	void draw_refresh();
	void draw_overlays();
// Cursor may have to be drawn
	int cursor_leave_event();
	int cursor_enter_event();
	int cursor_motion_event();
	int button_press_event();
	int button_release_event();
	int test_crop(int button_press, int &redraw);
	int test_bezier(int button_press, 
		int &redraw, 
		int &rerender,
		int do_camera);
	int test_zoom(int &redraw);
	void reset_camera();
	void reset_projector();
	void reset_keyframe(int do_camera);
	void draw_crophandle(int x, int y);
	void draw_bezier_center(BezierAuto *auto_);
	void draw_bezier_joining(BezierAuto *first, BezierAuto *last, int dashed);
	void draw_bezier(int do_camera);
	void draw_crop();
	void calculate_origin();

	MWindow *mwindow;
	CWindowGUI *gui;
};

#endif
