#include "automation.h"
#include "bezierauto.h"
#include "bezierautos.h"
#include "cpanel.h"
#include "cwindow.h"
#include "cwindowgui.h"
#include "cwindowtool.h"
#include "edl.h"
#include "edlsession.h"
#include "mainsession.h"
#include "mwindow.h"
#include "track.h"

CWindowTool::CWindowTool(MWindow *mwindow, CWindowGUI *gui)
 : Thread()
{
	this->mwindow = mwindow;
	this->gui = gui;
	tool_gui = 0;
	done = 0;
	set_synchronous(1);
	input_lock = new Mutex;
	input_lock->lock();
	output_lock = new Mutex;
}

CWindowTool::~CWindowTool()
{
	done = 1;
	stop_tool();
	input_lock->unlock();
	Thread::join();
	delete input_lock;
	delete output_lock;
}

void CWindowTool::start_tool(CWindowToolGUI *tool_gui)
{
	stop_tool();
	output_lock->lock();
	this->tool_gui = tool_gui;
	tool_gui->create_objects();
	input_lock->unlock();
}


void CWindowTool::stop_tool()
{
	if(tool_gui)
	{
		tool_gui->lock_window();
		tool_gui->set_done(0);
		tool_gui->unlock_window();
	}
}

void CWindowTool::run()
{
	while(!done)
	{
		input_lock->lock();
		if(!done)
		{
			tool_gui->run_window();
			delete tool_gui;
			tool_gui = 0;
		}
		output_lock->unlock();
	}
}

void CWindowTool::update_show_window()
{
	if(tool_gui)
	{
		tool_gui->lock_window();
		if(mwindow->edl->session->tool_window) 
			tool_gui->show_window();
		else
			tool_gui->hide_window();
		tool_gui->flush();
		tool_gui->unlock_window();
	}
}








CWindowToolGUI::CWindowToolGUI(MWindow *mwindow, 
	CWindowTool *thread, 
	char *title,
	int w, 
	int h)
 : BC_Window(title,
 	mwindow->session->ctool_x,
	mwindow->session->ctool_y,
	w,
	h,
	w,
	h,
	0,
	0,
	1)
{
	this->mwindow = mwindow;
	this->thread = thread;
	current_operation = 0;
}

CWindowToolGUI::~CWindowToolGUI()
{
}

int CWindowToolGUI::close_event()
{
	hide_window();
	flush();
	mwindow->edl->session->tool_window = 0;
	thread->gui->lock_window();
	thread->gui->composite_panel->set_operation(mwindow->edl->session->cwindow_operation);
	thread->gui->flush();
	thread->gui->unlock_window();
	return 1;
}

int CWindowToolGUI::keypress_event()
{
	if(get_keypress() == 'w' || get_keypress() == 'W')
		return close_event();
	return 0;
}

int CWindowToolGUI::translation_event()
{
	mwindow->session->ctool_x = get_x();
	mwindow->session->ctool_y = get_y();
	return 0;
}






CWindowCoord::CWindowCoord(CWindowToolGUI *gui, int x, int y, float value)
 : BC_TumbleTextBox(gui, 
		(float)value,
		(float)0,
		(float)65536,
		x, 
		y, 
		100)
{
	this->gui = gui;
}

CWindowCoord::CWindowCoord(CWindowToolGUI *gui, int x, int y, int value)
 : BC_TumbleTextBox(gui, 
		(long)value,
		(long)0,
		(long)65536,
		x, 
		y, 
		100)
{
	this->gui = gui;
}
int CWindowCoord::handle_event()
{
	gui->handle_event();
	return 1;
}


CWindowCropOK::CWindowCropOK(MWindow *mwindow, CWindowToolGUI *gui, int x, int y)
 : BC_GenericButton(x, y, "Do it")
{
	this->mwindow = mwindow;
	this->gui = gui;
}
int CWindowCropOK::handle_event()
{
	mwindow->crop_video();
	return 1;
}


int CWindowCropOK::keypress_event()
{
	if(get_keypress() == 0xd) 
	{
		handle_event();
		return 1;
	}
	return 0;
}







CWindowCropGUI::CWindowCropGUI(MWindow *mwindow, CWindowTool *thread)
 : CWindowToolGUI(mwindow, 
 	thread,
	PROGRAM_NAME ": Crop",
	330,
	100)
{
}


CWindowCropGUI::~CWindowCropGUI()
{
}

void CWindowCropGUI::create_objects()
{
	int x = 10, y = 10;
	BC_TumbleTextBox *textbox;
	BC_Title *title;

	add_subwindow(title = new BC_Title(x, y, "X1:"));
	x += title->get_w();
	x1 = new CWindowCoord(thread->tool_gui, x, y, mwindow->edl->session->crop_x1);
	x1->create_objects();
	x += x1->get_w() + 10;

	add_subwindow(title = new BC_Title(x, y, "Y1:"));
	x += title->get_w();
	y1 = new CWindowCoord(thread->tool_gui, x, y, mwindow->edl->session->crop_y1);
	y1->create_objects();
	y += y1->get_h() + 5;
	x = 10;

	add_subwindow(title = new BC_Title(x, y, "X2:"));
	x += title->get_w();
	x2 = new CWindowCoord(thread->tool_gui, x, y, mwindow->edl->session->crop_x2);
	x2->create_objects();
	x += x2->get_w() + 10;

	add_subwindow(title = new BC_Title(x, y, "Y2:"));
	x += title->get_w();
	y2 = new CWindowCoord(thread->tool_gui, x, y, mwindow->edl->session->crop_y2);
	y2->create_objects();
	y += y2->get_h() + 5;
	x = 10;
	add_subwindow(new CWindowCropOK(mwindow, thread->tool_gui, x, y));

	if(mwindow->edl->session->tool_window) show_window();
	flush();
}

void CWindowCropGUI::handle_event()
{
}








CWindowCameraGUI::CWindowCameraGUI(MWindow *mwindow, CWindowTool *thread)
 : CWindowToolGUI(mwindow, 
 	thread,
	PROGRAM_NAME ": Camera",
	200,
	120)
{
}
CWindowCameraGUI::~CWindowCameraGUI()
{
}

void CWindowCameraGUI::create_objects()
{
	int x = 10, y = 10;
	Track *track = mwindow->cwindow->calculate_affected_track();
	BezierAuto *keyframe = 0;
	BC_Title *title;

	if(track)
		keyframe = (BezierAuto*)mwindow->cwindow->calculate_affected_auto(track->automation->camera_autos);

	add_subwindow(title = new BC_Title(x, y, "X:"));
	x += title->get_w();
	this->x = new CWindowCoord(this, x, y, keyframe ? keyframe->center_x : (float)0);
	this->x->create_objects();
	y += 30;
	x = 10;
	add_subwindow(title = new BC_Title(x, y, "Y:"));
	x += title->get_w();
	this->y = new CWindowCoord(this, x, y, keyframe ? keyframe->center_y : (float)0);
	this->y->create_objects();
	y += 30;
	x = 10;
	add_subwindow(title = new BC_Title(x, y, "Z:"));
	x += title->get_w();
	this->z = new CWindowCoord(this, x, y, keyframe ? keyframe->center_z : (float)0);
	this->z->create_objects();

	if(mwindow->edl->session->tool_window) show_window();
	flush();
}

void CWindowCameraGUI::handle_event()
{
}










CWindowProjectorGUI::CWindowProjectorGUI(MWindow *mwindow, CWindowTool *thread)
 : CWindowToolGUI(mwindow, 
 	thread,
	PROGRAM_NAME ": Projector",
	200,
	120)
{
}
CWindowProjectorGUI::~CWindowProjectorGUI()
{
}
void CWindowProjectorGUI::create_objects()
{
	int x = 10, y = 10;
	Track *track = mwindow->cwindow->calculate_affected_track();
	BezierAuto *keyframe = 0;
	BC_Title *title;

	if(track)
		keyframe = (BezierAuto*)mwindow->cwindow->calculate_affected_auto(track->automation->projector_autos);

	add_subwindow(title = new BC_Title(x, y, "X:"));
	x += title->get_w();
	this->x = new CWindowCoord(this, x, y, keyframe ? keyframe->center_x : (float)0);
	this->x->create_objects();
	y += 30;
	x = 10;
	add_subwindow(title = new BC_Title(x, y, "Y:"));
	x += title->get_w();
	this->y = new CWindowCoord(this, x, y, keyframe ? keyframe->center_y : (float)0);
	this->y->create_objects();
	y += 30;
	x = 10;
	add_subwindow(title = new BC_Title(x, y, "Z:"));
	x += title->get_w();
	this->z = new CWindowCoord(this, x, y, keyframe ? keyframe->center_z : (float)0);
	this->z->create_objects();

	if(mwindow->edl->session->tool_window) show_window();
	flush();
}

void CWindowProjectorGUI::handle_event()
{
}



