#include "aedit.h"
#include "assets.h"
#include "automation.h"
#include "cache.h"
#include "edit.h"
#include "edits.h"
#include "edl.h"
#include "file.h"
#include "filexml.h"
#include "filesystem.h"
#include "localsession.h"
#include "mwindow.h"
#include "plugin.h"
#include "strategies.inc"
#include "track.h"
#include "transition.h"
#include "transportque.inc"

#include <string.h>

Edits::Edits(EDL *edl, Track *track)
 : List<Edit>()
{
	this->edl = edl;
	this->track = track;
}

Edits::~Edits()
{
}

Edits& Edits::operator=(Edits& edits)
{
	while(last) delete last;
	for(Edit *current = edits.first; current; current = NEXT)
	{
		Edit *new_edit = append(create_edit());
		new_edit->copy_from(current);
	}
	return *this;
}


void Edits::insert_asset(Asset *asset,
	long length,
	long position,
	int track_number)
{
	Edit *new_edit = insert_new_edit(position);

	new_edit->asset = asset;
	new_edit->startsource = 0;
	new_edit->startproject = position;
	new_edit->length = length;

	if(asset->audio_data)
		new_edit->channel = track_number % asset->channels;
	else
	if(asset->video_data)
		new_edit->channel = track_number % asset->layers;

printf("Edits::insert_asset %d %d\n", new_edit->channel, new_edit->length);
	for(Edit *current = new_edit->next; current; current = NEXT)
	{
		current->startproject += length;
	}
}

void Edits::insert_edits(Edits *source_edits, long position)
{
//printf("Edits::insert_edits 1\n");
	long clipboard_length = 
		track->to_units(source_edits->edl->local_session->clipboard_length, 1);


// Fill beginning with silence.  Can't call from insert_new_edit because it's recursive.
	if(position > total_length())
	{
		paste_silence(total_length(), position);
	}


	for(Edit *source_edit = source_edits->first;
		source_edit;
		source_edit = source_edit->next)
	{
//printf("Edits::insert_edits 1 %p\n", source_edit->asset);
// Update Assets
		Asset *dest_asset = edl->assets->update(source_edit->asset);
// Open destination area
//printf("Edits::insert_edits 1\n");
		Edit *dest_edit = insert_new_edit(position);
//printf("Edits::insert_edits 1\n");

		dest_edit->copy_from(source_edit);
//printf("Edits::insert_edits 2\n");
		dest_edit->asset = dest_asset;
//printf("Edits::insert_edits 1\n");
		dest_edit->startproject = position;
		for(Edit *future_edit = dest_edit->next;
			future_edit;
			future_edit = future_edit->next)
			future_edit->startproject += dest_edit->length;

//printf("Edits::insert_edits 2\n");
		position += dest_edit->length;
		clipboard_length -= dest_edit->length;
		
// Fill clipboard length with silence
		if(!source_edit->next && clipboard_length > 0)
		{
			paste_silence(dest_edit->startproject + dest_edit->length,
				dest_edit->startproject + dest_edit->length + clipboard_length);
		}
	}
//printf("Edits::insert_edits 3\n");
}


// Native units
Edit* Edits::insert_new_edit(long position)
{
// Get edit just before or on position
	Edit *current;

//printf("Edits::insert_new_edit 1\n");


	for(current = last; 
		current;
		current = PREVIOUS)
	{
//printf("Edits::insert_new_edit 2 %p %p\n", current, current->previous);
		if(current->startproject <= position) break;
	}

//printf("Edits::insert_new_edit 1\n");
	split_edit(current, position);
//printf("Edits::insert_new_edit 1\n");
	Edit *new_edit = create_edit();
//printf("Edits::insert_new_edit 1\n");
	insert_after(current, new_edit);
	new_edit->startproject = position;
//printf("Edits::insert_new_edit 2\n");
	return new_edit;
}

void Edits::split_edit(Edit *edit, long position)
{
	if(!edit) return;

	Edit *new_edit = create_edit();
	insert_after(edit, new_edit);
	new_edit->copy_from(edit);
	new_edit->length = new_edit->startproject + new_edit->length - position;
	edit->length = position - edit->startproject;
	new_edit->startproject = edit->startproject + edit->length;
	new_edit->startsource += edit->length;
	if(edit->transition && edit->transition->length > edit->length) 
		edit->transition->length = edit->length;
	if(new_edit->transition && new_edit->transition->length > new_edit->length)
		new_edit->transition->length = new_edit->length;
}

int Edits::save(FileXML *xml, char *output_path)
{
	copy(0, total_length(), xml, output_path);
	return 0;
}

void Edits::resample(double old_rate, double new_rate)
{
	double current_sample = 0;
	for(Edit *current = first; current; current = NEXT)
	{
		current->startproject = (long)(current_sample + 0.5);
		current->startsource = (long)((double)current->startsource /
			old_rate *
			new_rate +
			0.5);
		current_sample += (double)current->length * (double)new_rate / old_rate;
		current->length = (long)(current_sample - current->startproject + 0.5);
		current->resample(old_rate, new_rate);
	}
}














int Edits::optimize()
{
	int result = 1;
	Edit *current_edit;

//return 0;
	while(result)
	{
		result = 0;

// delete 0 length edits
		for(current_edit = first; 
			current_edit && !result; )
		{
			if(current_edit->length == 0)
			{
				Edit* next = current_edit->next;
				delete current_edit;
				result = 1;
				current_edit = next;
			}
			else
				current_edit = current_edit->next;
		}

// merge same files or transitions
		for(current_edit = first; 
			current_edit && current_edit->next && !result; )
		{
// assets identical
			Edit *next_edit = current_edit->next;
    		if(current_edit->asset == next_edit->asset && 
    		   	(!current_edit->asset ||
					(current_edit->startsource + current_edit->length == next_edit->startsource &&
	       			current_edit->channel == next_edit->channel)
				)
			)
        	{       
// source positions are consecutive
        		current_edit->length += next_edit->length;
        		remove(next_edit);
        		result = 1;
        	}

    		current_edit = (Plugin*)current_edit->next;
		}

// delete last edit of 0 length or silence
		if(last && 
			(last->silence() || 
			!last->length))
		{
			delete last;
			result = 1;
		}
	}

//track->dump();
	return 0;
}




















Edits::Edits(MWindow *mwindow, Track *track) : List<Edit>()
{
	this->track = track;
	this->mwindow = mwindow;
}

// ===================================== file operations

int Edits::load(FileXML *file, int track_offset)
{
	int result = 0;
	long startproject = 0;

	do{
		result = file->read_tag();

//printf("Edits::load 1 %s\n", file->tag.get_title());
		if(!result)
		{
			if(!strcmp(file->tag.get_title(), "EDIT"))
			{
				load_edit(file, startproject, track_offset);
			}
			else
			if(!strcmp(file->tag.get_title(), "/EDITS"))
			{
				result = 1;
			}
		}
	}while(!result);

	optimize();
}

int Edits::load_edit(FileXML *file, long &startproject, int track_offset)
{
	Edit* current;

//printf("Edits::load_edit 1 %d\n", total());
	current = append_new_edit();
//printf("Edits::load_edit 2 %d\n", total());

	current->load_properties(file, startproject);

	startproject += current->length;

	int result = 0;
//printf("Edits::load_edit 1\n");

	do{
//printf("Edits::load_edit 2\n");
		result = file->read_tag();
//printf("Edits::load_edit 3 %s\n", file->tag.get_title());

		if(!result)
		{
			if(file->tag.title_is("FILE"))
			{
				char filename[1024];
				sprintf(filename, SILENCE);
				file->tag.get_property("SRC", filename);
// Extend path
//printf("Edits::load_edit 4\n");
				if(strcmp(filename, SILENCE))
				{
					char directory[BCTEXTLEN], edl_directory[BCTEXTLEN];
					FileSystem fs;
					fs.set_current_dir("");
					fs.extract_dir(directory, filename);
					if(!strlen(directory))
					{
						fs.extract_dir(edl_directory, file->filename);
						fs.join_names(directory, edl_directory, filename);
						strcpy(filename, directory);
					}
					current->asset = edl->assets->get_asset(filename);
				}
				else
				{
					current->asset = edl->assets->get_asset(SILENCE);
				}
//printf("Edits::load_edit 5\n");
			}
			else
			if(file->tag.title_is(SILENCE))
			{
//printf("Edits::load_edit 6\n");
				current->asset = edl->assets->get_asset(SILENCE);
//printf("Edits::load_edit 7\n");
			}
			else
			if(file->tag.title_is("/EDIT"))
			{
				result = 1;
			}
		}
//printf("Edits::load_edit 8\n");
	}while(!result);

//printf("Edits::load_edit 5\n");
// in case of incomplete edit tag
	if(!current->asset) current->asset = edl->assets->get_asset(SILENCE);
	return 0;
}

// ============================================= accounting

long Edits::length()
{
	if(last) 
		return last->startproject + last->length;
	else 
		return 0;
}

long Edits::total_length() 
{
	long total = 0;
	Edit* current;
	for(current = first; current; current = NEXT)
	{
		total += current->length;
	}
	return total; 
};

Edit* Edits::editof(long position, int direction)
{
	Edit *current = 0;
	
	if(direction == PLAY_FORWARD)
	{
		for(current = last; current; current = PREVIOUS)
		{
			if(current->startproject <= position && current->startproject + current->length > position)
				return current;
		}
	}
	else
	if(direction == PLAY_REVERSE)
	{
		for(current = first; current; current = NEXT)
		{
			if(current->startproject < position && current->startproject + current->length >= position)
				return current;
		}
	}

	return 0;     // return 0 on failure
}

Edit* Edits::get_playable_edit(long position)
{
	Edit *current;

// Get the current edit
	for(current = first; current; current = NEXT)
	{
		if(current->startproject <= position && 
			current->startproject + current->length > position)
			break;
	}

// Get the edit's asset
	if(current)
	{
		if(!current->asset)
			current = 0;
	}

	return current;     // return 0 on failure
}

// ================================================ editing

Edit* Edits::insert(long start, long lengthsamples)
{
	Edit *old_edit, *new_edit;
	
	old_edit = editof(start, PLAY_FORWARD);

	if(old_edit)
	{       // ========================== split the edit
		new_edit = insert_edit_after(old_edit);

		new_edit->startsource = old_edit->startsource + (start - old_edit->startproject);
// will be shifted later
		new_edit->startproject = old_edit->startproject + (start - old_edit->startproject);
		new_edit->length = old_edit->length - (start - old_edit->startproject);
		new_edit->feather_left = 0;
		new_edit->feather_right = old_edit->feather_right;

		old_edit->length -= new_edit->length;
		old_edit->feather_right = 0;

		new_edit->asset = old_edit->asset;

		if(old_edit->transition)
		{
//			new_edit->transition = new Transition(old_edit->transition, new_edit);
		}
		else
			new_edit->transition = 0;

		clone_derived(new_edit, old_edit);

		new_edit = old_edit;
	}
	else
	{       // extend track with silence
		new_edit = append_new_edit();

		if(new_edit->previous)
			new_edit->startproject = new_edit->previous->startproject + new_edit->previous->length;

		new_edit->length = start - new_edit->startproject;

		new_edit->asset = mwindow->assets->update(SILENCE);
	}

	Edit* result = new_edit;

// shift everything after old_edit
	for(new_edit = new_edit->next; new_edit; new_edit = new_edit->next)
	{            // shift
		new_edit->startproject += lengthsamples;
	}

	return result;
}

// REMOVE
int Edits::paste_transition(long startproject, 
				long endproject, 
				Transition *transition)
{
	Edit *current_edit;
	clear(startproject, endproject);
	current_edit = insert(startproject, endproject - startproject);

	current_edit = insert_edit_after(current_edit);

	current_edit->asset = 0;
	current_edit->startproject = startproject;
	current_edit->length = endproject - startproject;
	current_edit->startsource = 0;
	current_edit->feather_left = 0;
	current_edit->feather_right = 0;
//	current_edit->transition = new Transition(transition, current_edit);
	if(transition->show)
	{
// Show the GUI of only the first transition pasted.
//		current_edit->transition->show_gui();
//		transition->show = 0;
	}

	optimize();
	return 0;
}

Edit* Edits::paste_edit_base(long start, 
				long end, 
				long startsource, 
				long length, 
				Asset *asset)
{
	int result;
	Edit *current_edit;

// clear selection
	clear(start, end);

// Insert silence before new edit.
	current_edit = insert(start, length);

// create a new edit for the data and point *current to that edit
	current_edit = insert_edit_after(current_edit);

// put the data in the new edit
	current_edit->startsource = startsource;
	current_edit->startproject = start;
	current_edit->length = length;
	current_edit->feather_left = 0;
	current_edit->feather_right = 0;
	current_edit->asset = asset;

	return current_edit;
}


int Edits::copy(long start, long end, FileXML *file, char *output_path)
{
	Edit *current_edit;

	file->tag.set_title("EDITS");
	file->append_tag();
	file->append_newline();

	for(current_edit = first; current_edit; current_edit = current_edit->next)
	{
		current_edit->copy(start, end, file, output_path);
	}

	file->tag.set_title("/EDITS");
	file->append_tag();
	file->append_newline();
}

// REPLACE with insertion
int Edits::paste(long start, long end, long total_length, FileXML *file)
{
	Edit *current_edit;
	current_edit = insert(start, total_length);

	int result = 0;
	long startproject = start;

	do{
		result = file->read_tag();

		if(!result)
		{
			if(!strcmp(file->tag.get_title(), "/EDITS"))
			{
				result = 1;
			}
			else
			if(!strcmp(file->tag.get_title(), "EDIT"))
			{
				current_edit = insert_edit_after(current_edit);

				current_edit->startsource = file->tag.get_property("STARTSOURCE", (long)0);
				current_edit->startproject = startproject;

				current_edit->load_properties_derived(file);

				startproject += current_edit->length;
				current_edit->paste(file);
			}
		}
	}while(!result);


// fill end manually since it can't be optimized
	current_edit = insert_edit_after(current_edit);

	current_edit->startproject = startproject;
	current_edit->length = total_length + start - startproject;

	current_edit->asset = mwindow->assets->update(SILENCE);

// optimize it out
	optimize();
	return 0;
}

int Edits::clear(long start, long end)
{
//printf("Edits::clear 1\n");
	Edit* edit1 = editof(start, PLAY_FORWARD);
	Edit* edit2 = editof(end, PLAY_FORWARD);
	Edit* current_edit;

//printf("Edits::clear 2\n");
	if(end == start) return 0;        // nothing selected
	if(!edit1 && !edit2) return 0;       // nothing selected
	if(!edit2)
	{                // edit2 beyond end of track
		edit2 = last;
		end = this->length();
	}

//printf("Edits::clear 3\n");
	if(edit1 != edit2)
	{
// in different edits
		edit1->length = start - edit1->startproject;
		edit2->length -= end - edit2->startproject;
		edit2->startsource += end - edit2->startproject;
		edit2->startproject += end - edit2->startproject;
// Transition is in the space being deleted
		if(edit2->transition) 
		{
			delete edit2->transition;
			edit2->transition = 0;
		}

// delete
		for(current_edit = edit1->next; current_edit && current_edit != edit2;)
		{            
			Edit* next = current_edit->next;
			remove(current_edit);
			current_edit = next;
		}
// shift
		for(current_edit = edit2; current_edit; current_edit = current_edit->next)
		{            
			current_edit->startproject -= end - start;
		}
	}
	else
	{
// in same edit. paste_edit depends on this
// create a new edit
		split_edit(edit1, start);
		current_edit = edit1->next;           

//printf("Edits::clear 4.1\n");
		current_edit->length -= end - start;
//printf("Edits::clear 4.2\n");
		current_edit->startsource += end - start;

//printf("Edits::clear 5 %p %p %p\n", current_edit->edits, current_edit->track, this->track);
// shift
		for(current_edit = current_edit->next; 
			current_edit; 
			current_edit = current_edit->next)
		{            
			current_edit->startproject -= end - start;
		}
	}

//printf("Edits::clear 6\n");
	optimize();
//printf("Edits::clear 7\n");
	return 0;
}

int Edits::clear_handle(long start, long end)
{
	Edit *current_edit;

//printf("Edits::clear_handle 1\n");
	for(current_edit = first; current_edit && current_edit->next; current_edit = current_edit->next)
	{
//printf("Edits::clear_handle 2\n");
		if(*current_edit->asset == *current_edit->next->asset)
		{           
// two consecutive edits in same source
//printf("Edits::clear_handle 3\n");
			if(current_edit->next->startproject == start)
			{
// handle selected
//printf("Edits::clear_handle 4\n");
				int length = -current_edit->length;
				current_edit->length = current_edit->next->startsource - current_edit->startsource;
				length += current_edit->length;

//printf("Edits::clear_handle 5\n");
				track->automation->paste_silence(current_edit->next->startproject, current_edit->next->startproject + length);
//printf("Edits::clear_handle 6\n");
				for(current_edit = current_edit->next; current_edit; current_edit = current_edit->next)
				{
					current_edit->startproject += length;
				}
//printf("Edits::clear_handle 7\n");
				optimize();
				break;
//printf("Edits::clear_handle 8\n");
			}
//printf("Edits::clear_handle 9\n");
		}
//printf("Edits::clear_handle 10\n");
	}
//printf("Edits::clear_handle 11\n");
	return 0;
}

int Edits::modify_handles(double oldposition, 
	double newposition, 
	int currentend, 
	int edit_mode)
{
	int result = 0;
	Edit *current_edit;

	if(currentend == 0)
	{            
// left handle
		for(current_edit = first; current_edit && !result;)
		{
			if(edl->equivalent(track->from_units(current_edit->startproject), oldposition))
			{
// edit matches selection
				oldposition = track->from_units(current_edit->startproject);
				result = 1;

				if(newposition >= oldposition)
				{
//printf("Edits::modify_handle 1 %s\n", track->title);
// shift start of edit in
					current_edit->shift_start_in(edit_mode, 
						track->to_units(newposition, 0), 
						track->to_units(oldposition, 0));
				}
				else
				{
//printf("Edits::modify_handle 2 %s\n", track->title);
// move start of edit out
					current_edit->shift_start_out(edit_mode, 
						track->to_units(newposition, 0), 
						track->to_units(oldposition, 0));
				}
			}

			if(!result) current_edit = current_edit->next;
		}
	}
	else
	{
// right handle selected
		for(current_edit = first; current_edit && !result;)
		{
//printf("Edits::modify_handle 1 %s\n", track->title);
			if(edl->equivalent(track->from_units(current_edit->startproject) + 
				track->from_units(current_edit->length), oldposition))
			{
//printf("Edits::modify_handle 2\n");
            	oldposition = track->from_units(current_edit->startproject) + 
					track->from_units(current_edit->length);
				result = 1;

//printf("Edits::modify_handle 3\n");
				if(newposition <= oldposition)
				{     // shift end of edit in
//printf("Edits::modify_handle 4\n");
					current_edit->shift_end_in(edit_mode, 
						track->to_units(newposition, 0), 
						track->to_units(oldposition, 0));
//printf("Edits::modify_handle 5\n");
				}
				else
				{     // move end of edit out
//printf("Edits::modify_handle 6\n");
					current_edit->shift_end_out(edit_mode, 
						track->to_units(newposition, 0), 
						track->to_units(oldposition, 0));
//printf("Edits::modify_handle 7\n");
				}
			}

			if(!result) current_edit = current_edit->next;
//printf("Edits::modify_handle 8\n");
		}
	}

	optimize();
	return 0;
}


// Used by other editing commands so no optimize
int Edits::paste_silence(long start, long end)
{
//printf("Edits::paste_silence 1\n");
	Edit *new_edit = insert_new_edit(start);
//printf("Edits::paste_silence 3\n");
	new_edit->length = end - start;
//printf("Edits::paste_silence 4\n");
	for(Edit *current = new_edit->next; current; current = NEXT)
	{
		current->startproject += end - start;
	}
	return 0;
}
				     
