#include "assets.h"
#include "atrack.h"
#include "autoconf.h"
#include "awindowgui.inc"
#include "clip.h"
#include "colormodels.h"
#include "defaults.h"
#include "edl.h"
#include "edlsession.h"
#include "filexml.h"
#include "guicast.h"
#include "labels.h"
#include "localsession.h"
#include "playbackconfig.h"
#include "sharedlocation.h"
#include "preferences.h"
#include "recordconfig.h"
#include "theme.h"
#include "tracks.h"
#include "transportque.inc"
#include "vtrack.h"

EDL::EDL(EDL *parent_edl)
{
	this->parent_edl = parent_edl;
	tracks = 0;
	labels = 0;
	local_session = 0;
	presentations = 0;

	new_folder(CLIP_FOLDER);
	new_folder(MEDIA_FOLDER);
	id = next_id();
}


EDL::~EDL()
{
//printf("EDL::~EDL 1\n");
	if(tracks)
	{
		delete tracks;
	}
	if(labels)
	{
		delete labels;
	}
	if(presentations)
	{
		delete presentations;
	}
	
	if(local_session)
	{
		delete local_session;
	}

	if(!parent_edl)
	{
		delete assets;
		delete session;
	}


	folders.remove_all_objects();
	clips.remove_all_objects();
//printf("EDL::~EDL 2\n");
}


int EDL::create_objects()
{
	tracks = new Tracks(this);
	if(!parent_edl)
	{
		assets = new Assets(this);
		session = new EDLSession(this);
	}
	else
	{
		assets = parent_edl->assets;
		session = parent_edl->session;
	}
	
	local_session = new LocalSession(this);
	labels = new Labels(this, "LABELS");
	presentations = new Presentations(this);
//	last_playback_position = 0;
	return 0;
}

EDL& EDL::operator=(EDL &edl)
{
//printf("EDL::operator= 1\n");
	copy_all(&edl);
//printf("EDL::operator= 2\n");
	return *this;
}

int EDL::load_defaults(Defaults *defaults)
{
	if(!parent_edl)
		session->load_defaults(defaults);

	local_session->load_defaults(defaults);
	return 0;
}

int EDL::save_defaults(Defaults *defaults)
{
	if(!parent_edl)
		session->save_defaults(defaults);
	
	local_session->save_defaults(defaults);
	return 0;
}

void EDL::boundaries()
{
	session->boundaries();
	local_session->boundaries();
}

int EDL::create_default_tracks()
{
//printf("EDL::create_default_tracks %d %d\n", audio_tracks, video_tracks);

	for(int i = 0; i < session->video_tracks; i++)
	{
		tracks->add_video_track();
	}
	for(int i = 0; i < session->audio_tracks; i++)
	{
		tracks->add_audio_track();
	}
	return 0;
}

int EDL::load_xml(ArrayList<PluginServer*> *plugindb,
	FileXML *file, 
	unsigned long load_flags)
{
	int result = 0;
// Track numbering offset for importing shared plugins
	int track_offset = 0;

	folders.remove_all_objects();

// Search for start of master EDL.

// The parent_edl test caused clip creation to fail since those XML files
// contained an EDL tag.

// The parent_edl test is required to make EDL loading work because
// when loading an EDL the EDL tag is already read by the parent.

	if(!parent_edl && (load_flags & LOAD_ALL) == LOAD_ALL)
	{
		do{
		  result = file->read_tag();
		}while(!result && 
			!file->tag.title_is("XML") && 
			!file->tag.title_is("EDL"));
	}
//printf("EDL::load_xml 1\n");

	if(!result)
	{
// Erase everything if undo
//printf("EDL::load_xml 1\n");
		while(tracks->last) delete tracks->last;
//printf("EDL::load_xml 2\n");
		if(load_flags & LOAD_TIMEBAR)
		{
			while(labels->last) delete labels->last;
		}
//printf("EDL::load_xml 3\n");

		do{
			result = file->read_tag();

//printf("EDL::load_xml 3 %s\n", file->tag.get_title());
			if(!result)
			{
				if(file->tag.title_is("/XML"))
				{
					result = 1;
				}
				else
				if(file->tag.title_is("/EDL"))
				{
					result = 1;
				}
				else
				if(file->tag.title_is("CLIPBOARD"))
				{
					local_session->clipboard_length = file->tag.get_property("LENGTH", 0);
				}
				else
				if(file->tag.title_is("VIDEO"))
				{
					if(load_flags & LOAD_VCONFIG)
						session->load_video_config(file, 0, load_flags);
				}
				else
				if(file->tag.title_is("AUDIO"))
				{
					if(load_flags & LOAD_ACONFIG)
						session->load_audio_config(file, 0, load_flags);
				}
				else
				if(file->tag.title_is("FOLDER"))
				{
					char folder[BCTEXTLEN];
					strcpy(folder, file->read_text());
					new_folder(folder);
				}
				else
				if(file->tag.title_is("ASSETS"))
				{
//printf("EDL::load_xml 3\n");
					if(load_flags & LOAD_ASSETS)
						assets->load(plugindb, file, load_flags);
//printf("EDL::load_xml 4\n");
				}
				else
				if(file->tag.title_is(labels->xml_tag))
				{
					if(load_flags & LOAD_TIMEBAR)
						labels->load(file, load_flags);
				}
				else
				if(file->tag.title_is(presentations->xml_tag))
				{
					if(load_flags & LOAD_TIMEBAR)
						presentations->load(file, load_flags);
				}
				else
				if(file->tag.title_is("LOCALSESSION"))
				{
					if(load_flags & LOAD_SESSION)
						local_session->load_xml(file);
				}
				else
				if(file->tag.title_is("SESSION"))
				{
					if((load_flags & LOAD_SESSION) &&
						!parent_edl)
						session->load_xml(file, 0, load_flags);
				}
				else
				if(file->tag.title_is("TRACK"))
				{
//printf("EDL::load_xml 5\n");
					tracks->load(file, track_offset, load_flags);
				}
				else
// Sub EDL.
// Causes clip creation to fail because that involves an opening EDL tag.
				if(file->tag.title_is("EDL") && !parent_edl)
				{
//printf("EDL::load_xml 5 %d\n", tracks->total());
					EDL *new_edl;
					clips.append(new_edl = new EDL(this));
					new_edl->create_objects();
					new_edl->load_xml(plugindb, file, LOAD_ALL);
				}
			}
		}while(!result);
//printf("EDL::load_xml 4\n");
	}
//printf("EDL::load_xml 6 %d\n", tracks->total());

	return 0;
}

// Output path is the path of the output file if name truncation is desired.
// It is a "" if complete names should be used.
// Called recursively by copy for clips
int EDL::save_xml(ArrayList<PluginServer*> *plugindb,
	FileXML *file, 
	char *output_path)
{
	copy(0, 
		tracks->total_length(), 
		1, 
		file, 
		plugindb, 
		output_path,
		0);
	return 0;
}

int EDL::copy_all(EDL *edl)
{
	copy_session(edl);
//printf("EDL::copy_all 1\n");
	copy_assets(edl);
	copy_clips(edl);
//printf("EDL::copy_all 1\n");
	*this->tracks = *edl->tracks;
	*this->labels = *edl->labels;
//printf("EDL::copy_all 2\n");
	return 0;
}

void EDL::copy_clips(EDL *edl)
{
	clips.remove_all_objects();
	for(int i = 0; i < edl->clips.total; i++)
	{
		add_clip(edl->clips.values[i]);
	}
}

void EDL::copy_assets(EDL *edl)
{
	if(!parent_edl)
		*this->assets = *edl->assets;
}

void EDL::copy_session(EDL *edl)
{
	folders.remove_all_objects();
	for(int i = 0; i < edl->folders.total; i++)
	{
		char *new_folder;
		folders.append(new_folder = new char[strlen(edl->folders.values[i]) + 1]);
		strcpy(new_folder, edl->folders.values[i]);
	}

	if(!parent_edl)
	{
		session->copy(edl->session);
	}
	
	local_session->copy_from(edl->local_session);
}

int EDL::copy_assets(double start, 
	double end, 
	FileXML *file, 
	int all, 
	ArrayList<PluginServer*> *plugindb,
	char *output_path)
{
	ArrayList<Asset*> asset_list;
	Track* current;

	file->tag.set_title("ASSETS");
	file->append_tag();
	file->append_newline();

	for(current = tracks->first; 
		current; 
		current = NEXT)
	{
		if(all || current->record)
		{
			current->copy_assets(start, 
				end, 
				&asset_list);
		}
	}

// Paths relativised here
	for(int i = 0; i < asset_list.total; i++)
	{
		asset_list.values[i]->write(plugindb,
			file, 
			0, 
			output_path);
	}

	file->tag.set_title("/ASSETS");
	file->append_tag();
	file->append_newline();
	file->append_newline();
	return 0;
}

int EDL::copy(double start, 
	double end, 
	int all, 
	FileXML *file, 
	ArrayList<PluginServer*> *plugindb, 
	char *output_path,
	int rewind_it)
{
//printf("EDL::copy 1\n");
// begin file
	file->tag.set_title("EDL");
	file->append_tag();
	file->append_newline();

// Set clipboard samples only if copying to clipboard
	if(!all)
	{
		file->tag.set_title("CLIPBOARD");
		file->tag.set_property("LENGTH", end - start);
		file->append_tag();
		file->append_newline();
		file->append_newline();
	}

// Sessions
	local_session->save_xml(file);


// Top level stuff
	if(!parent_edl)
	{
// Session
		session->save_xml(file);
		session->save_video_config(file);
		session->save_audio_config(file);

// Folders
		for(int i = 0; i < folders.total; i++)
		{
			file->tag.set_title("FOLDER");
			file->append_tag();
			file->append_text(folders.values[i]);
			file->tag.set_title("/FOLDER");
			file->append_tag();
			file->append_newline();
		}

// Media
		copy_assets(start, 
			end, 
			file, 
			all, 
			plugindb,
			output_path);

// Clips
		if(all)
		{
			for(int i = 0; i < clips.total; i++)
				clips.values[i]->save_xml(plugindb, file, output_path);
		}

		file->append_newline();
		file->append_newline();
	}


//printf("EDL::copy 1\n");

//printf("EDL::copy 1\n");
	labels->copy(start, end, file);
//printf("EDL::copy 1\n");
	presentations->copy(start, end, file);
//printf("EDL::copy 1\n");
	tracks->copy(start, end, all, file, output_path);
//printf("EDL::copy 2\n");

// terminate file
	file->tag.set_title("/EDL");
	file->append_tag();
	file->append_newline();


// For editing operations we want to rewind it for immediate pasting.
// For clips and saving to disk leave it alone.
	if(rewind_it)
	{
		file->terminate_string();
		file->rewind();
	}
	return 0;
}

void EDL::resample(double old_rate, double new_rate, int data_type)
{
	for(Track *current = tracks->first; current; current = NEXT)
	{
		if(current->data_type == data_type)
		{
			current->resample(old_rate, new_rate);
		}
	}
}


void EDL::synchronize_params(EDL *edl)
{
	session->synchronize_params(edl->session);
	for(Track *this_track = tracks->first, *that_track = edl->tracks->first; 
		this_track && that_track; 
		this_track = this_track->next,
		that_track = that_track->next)
	{
		this_track->synchronize_params(that_track);
	}
}


int EDL::equivalent(double position1, double position2)
{
	if((session->cursor_on_frames && 
		fabs(position2 - position1) < (double)1 / session->frame_rate) ||
		(fabs(position2 - position1) < (double)1 / session->sample_rate))
    	return 1;
    else
        return 0;
}


void EDL::set_inpoint(double position)
{
	if(equivalent(local_session->in_point, position) && local_session->in_point >= 0)
	{
		local_session->in_point = -1;
	}
	else
	{
		local_session->in_point = align_to_frame(position, 0);
		if(local_session->out_point <= local_session->in_point) local_session->out_point = -1;
	}
}

void EDL::set_outpoint(double position)
{
	if(equivalent(local_session->out_point, position) && local_session->out_point >= 0)
	{
		local_session->out_point = -1;
	}
	else
	{
		local_session->out_point = align_to_frame(position, 0);
		if(local_session->in_point >= local_session->out_point) local_session->in_point = -1;
	}
}


int EDL::clear(double start, double end, int clear_labels)
{
	if(start == end)
	{
		tracks->clear_handle(start, end);
	}
	else
	{
//printf("EDL::clear 1\n");
		tracks->clear(start, end);
//printf("EDL::clear 2\n");
		if(session->labels_follow_edits && clear_labels) 
			labels->clear(start, end, 1);
//printf("EDL::clear 3\n");
		presentations->clear(start, end);
//printf("EDL::clear 4\n");
	}

	local_session->selectionend = local_session->selectionstart;
	session->changes_made = 1;	
//dump();
	return 0;
}

void EDL::modify_handles(double oldposition, 
	double newposition, 
	int currentend, 
	int handle_mode)
{
	tracks->modify_handles(oldposition, 
		newposition, 
		currentend, 
		handle_mode);
	labels->modify_handles(oldposition, 
		newposition, 
		currentend,
		handle_mode);
	session->changes_made = 1;
}

void EDL::paste_silence(double start, double end, int clear_labels)
{
	if(session->labels_follow_edits && clear_labels) 
		labels->paste_silence(start, end);
	tracks->paste_silence(start, end);
}

int EDL::get_tracks_height(Theme *theme)
{
	int total_pixels = 0;
	for(Track *current = tracks->first;
		current;
		current = NEXT)
	{
		total_pixels += current->vertical_span(theme);
	}
	return total_pixels;
}

long EDL::get_tracks_width()
{
	long total_pixels = 0;
	for(Track *current = tracks->first;
		current;
		current = NEXT)
	{
		long pixels = current->horizontal_span();
		if(pixels > total_pixels) total_pixels = pixels;
	}
//printf("EDL::get_tracks_width %d\n", total_pixels);
	return total_pixels;
}

int EDL::calculate_output_w(int single_channel)
{
	if(single_channel) return session->output_w;

	int widest = 0;
	for(int i = 0; i < session->video_channels; i++)
	{
		if(session->vchannel_x[i] + session->output_w > widest) widest = session->vchannel_x[i] + session->output_w;
	}
	return widest;
}

int EDL::calculate_output_h(int single_channel)
{
	if(single_channel) return session->output_h;

	int tallest = 0;
	for(int i = 0; i < session->video_channels; i++)
	{
		if(session->vchannel_y[i] + session->output_h > tallest) tallest = session->vchannel_y[i] + session->output_h;
	}
	return tallest;
}

// Get the total output size scaled to aspect ratio
void EDL::calculate_conformed_dimensions(int single_channel, float &w, float &h)
{
	w = calculate_output_w(single_channel);
	h = calculate_output_h(single_channel);

	if((float)session->output_w / session->output_h > get_aspect_ratio())
	{
		h = (float)h * 
			(session->output_w / get_aspect_ratio() / session->output_h);
	}
	else
	{
		w = (float)w * 
			(h * get_aspect_ratio() / session->output_w);
	}
}

float EDL::get_aspect_ratio()
{
	return session->aspect_w / session->aspect_h;
}

int EDL::dump()
{
	if(parent_edl)
		printf("CLIP\n");
	else
		printf("EDL\n");
	printf("clip_title: %s parent_edl: %p\n", local_session->clip_title, parent_edl);
	printf("selectionstart %f selectionend %f\n", local_session->selectionstart, local_session->selectionend);

	if(!parent_edl)
	{
		printf("audio_channels: %d "
			"audio_tracks: %d \n"
			"sample_rate: %d\n",
			session->audio_channels,
			session->audio_tracks,
			session->sample_rate);
		printf("video_channels: %d "
			"video_tracks: %d "
			"frame_rate: %.2f "
			"frames_per_foot: %.2f\n"
    		"track_w: %d "
    		"track_h: %d "
    		"output_w: %d "
    		"output_h: %d "
    		"aspect_w: %f "
    		"aspect_h %f "
			"color_model %d\n",
				session->video_channels,
				session->video_tracks,
				session->frame_rate,
				session->frames_per_foot,
    			session->track_w,
    			session->track_h,
    			session->output_w,
    			session->output_h,
    			session->aspect_w,
    			session->aspect_h,
				session->color_model);

		printf(" EDLS\n");
		printf("  total: %d\n", clips.total);
	
		for(int i = 0; i < clips.total; i++)
		{
			printf("\n\n");
			clips.values[i]->dump();
			printf("\n\n");
		}

		printf(" ASSETS\n");
		assets->dump();
	}
	printf(" LABELS\n");
	labels->dump();
	printf(" TRACKS\n");
	tracks->dump();
//printf("EDL::dump 2\n");
	return 0;
}

void EDL::add_clip(EDL *edl)
{
// Copy argument.  New edls are deleted from MWindow::load_filenames.
	EDL *new_edl = new EDL(this);
	new_edl->create_objects();
	new_edl->copy_all(edl);
	clips.append(new_edl);
}

void EDL::insert_asset(Asset *asset, long sample)
{
// Insert asset into asset table
	Asset *new_asset = assets->update(asset);

// Paste video
	int vtrack = 0;
	Track *current;
	for(current = tracks->first;
		current && vtrack < new_asset->layers;
		current = NEXT)
	{
		if(!current->record || 
			current->data_type != TRACK_VIDEO)
			continue;

		current->insert_asset(new_asset, 
			(double)new_asset->video_length / 
				new_asset->frame_rate, 
			sample, 
			vtrack);
		vtrack++;
	}

	int atrack = 0;
	for(current = tracks->first;
		current && atrack < new_asset->channels;
		current = NEXT)
	{
		if(!current->record ||
			current->data_type != TRACK_AUDIO)
			continue;

		current->insert_asset(new_asset, 
			(double)new_asset->audio_length / 
				new_asset->sample_rate, 
			sample, 
			atrack);

		atrack++;
	}

//	optimize();
}

void EDL::set_index_file(Asset *asset)
{
	assets->update_index(asset);
}

void EDL::optimize()
{
	for(Track *current = tracks->first; current; current = NEXT)
		current->optimize();
}

int EDL::next_id()
{
	return EDLSession::current_id++;
}

void EDL::get_shared_plugins(Track *source, 
	ArrayList<SharedLocation*> *plugin_locations)
{
	for(Track *track = tracks->first; track; track = track->next)
	{
		if(track != source && 
			track->data_type == source->data_type)
		{
			for(int i = 0; i < track->plugin_set.total; i++)
			{
				Plugin *plugin = track->get_current_plugin(local_session->selectionstart, 
					i, 
					PLAY_FORWARD, 
					1);
				if(plugin)
				{
					plugin_locations->append(new SharedLocation(tracks->number_of(track), i));
				}
			}
		}
	}
}

void EDL::get_shared_tracks(Track *track, ArrayList<SharedLocation*> *module_locations)
{
	for(Track *current = tracks->first; current; current = NEXT)
	{
		if(current != track && 
			current->data_type == track->data_type)
		{
			module_locations->append(new SharedLocation(tracks->number_of(current), 0));
		}
	}
}

// Convert position to frames if cursor alignment is enabled
double EDL::align_to_frame(double position, int round)
{
//printf("EDL::align_to_frame 1 %d\n", position);
	if(session->cursor_on_frames)
	{
// Seconds -> Frames
		double temp = (double)position * session->frame_rate;
//printf("EDL::align_to_frame 2 %f\n", temp);

// Assert some things
		if(session->sample_rate == 0)
			printf("EDL::align_to_frame: sample_rate == 0\n");

		if(session->frame_rate == 0)
			printf("EDL::align_to_frame: frame_rate == 0\n");

// Round frames
// Always round down negative numbers
// but round up only if requested
		if(round) 
		{
			temp = Units::round(temp);
		}
		else
		{
			if(temp < 0)
				temp -= 0.5;
			else
				temp = Units::to_long(temp);
		}
//printf("EDL::align_to_frame 3 %f\n", temp);

// Frames -> Seconds
		temp /= session->frame_rate;

//printf("EDL::align_to_frame 5 %f\n", temp);

		return temp;
	}
//printf("EDL::align_to_frame 3 %d\n", position);


	return position;
}


void EDL::new_folder(char *folder)
{
	for(int i = 0; i < folders.total; i++)
	{
		if(!strcasecmp(folders.values[i], folder)) return;
	}

	char *new_folder;
	folders.append(new_folder = new char[strlen(folder) + 1]);
	strcpy(new_folder, folder);
}

void EDL::delete_folder(char *folder)
{
	int i;
	for(i = 0; i < folders.total; i++)
	{
		if(!strcasecmp(folders.values[i], folder))
		{
			break;
		}
	}

	if(i < folders.total) delete folders.values[i];

	for( ; i < folders.total - 1; i++)
	{
		folders.values[i] = folders.values[i + 1];
	}
}
