#ifndef EDL_H
#define EDL_H

#include "assets.inc"
#include "autoconf.inc"
#include "defaults.inc"
#include "edlsession.inc"
#include "filexml.inc"
#include "labels.inc"
#include "localsession.inc"
#include "maxchannels.h"
#include "playbackconfig.h"
#include "pluginserver.h"
#include "preferences.inc"
#include "presentations.h"
#include "sharedlocation.inc"
#include "theme.inc"
#include "tracks.inc"

class EDL
{
public:
	EDL(EDL *parent_edl = 0);
	~EDL();

	int create_objects();
	EDL& operator=(EDL &edl);

	int calculate_smp();
// Load configuration and track counts
	int load_defaults(Defaults *defaults);
	int save_defaults(Defaults *defaults);
// Clip default settings to boundaries.
	void boundaries();
// Create tracks using existing configuration
	int create_default_tracks();
	int load_xml(ArrayList<PluginServer*> *plugindb, 
		FileXML *file, 
		unsigned long load_flags);
	int save_xml(ArrayList<PluginServer*> *plugindb,
		FileXML *xml, 
		char *output_path);
    int load_audio_config(FileXML *file, int append_mode, unsigned long load_flags);
    int load_video_config(FileXML *file, int append_mode, unsigned long load_flags);



// Convert position to frames if cursor alignment is enabled
	double align_to_frame(double position, int round);



// Scale all sample values since everything is locked to audio
	void resample(double old_rate, double new_rate, int data_type);
	void copy_tracks(EDL *edl);
	void copy_session(EDL *edl);
	int copy_all(EDL *edl);
	void copy_assets(EDL *edl);
	void copy_clips(EDL *edl);
// Copy pan and fade settings from edl
	void synchronize_params(EDL *edl);
	int equivalent(double position1, double position2);
// Set points and labels
	void set_inpoint(double position);
	void set_outpoint(double position);
// Redraw resources during index builds
	void set_index_file(Asset *asset);
	void optimize();
// Debug
	int dump();
	static int next_id();
// Create a new folder if it doesn't exist already
	void new_folder(char *folder);
	void delete_folder(char *folder);
	void modify_handles(double oldposition, 
		double newposition, 
		int currentend, 
		int handle_mode);


// Editing functions
	int copy_assets(double start, 
		double end, 
		FileXML *file, 
		int all, 
		ArrayList<PluginServer*> *plugindb,
		char *output_path);
	int copy(double start, 
		double end, 
		int all,   // Ignore recordable status of tracks for saving
		FileXML *file, 
		ArrayList<PluginServer*> *plugindb,
		char *output_path,
		int rewind_it);     // Rewind EDL for easy pasting
	void paste_silence(double start, double end, int clear_labels = 1);
	int clear(double start, double end, int clear_labels = 1);
// Insert the asset at a point in the EDL
	void insert_asset(Asset *asset, long sample);
// Add a copy of EDL* to the clip array
	void add_clip(EDL *edl);

	void get_shared_plugins(Track *source, ArrayList<SharedLocation*> *plugin_locations);
	void get_shared_tracks(Track *track, ArrayList<SharedLocation*> *module_locations);


    int get_tracks_height(Theme *theme);
    long get_tracks_width();
// Return the dimension for a single pane if single_channel is set.
// Otherwise add all panes.
	int calculate_output_w(int single_channel);
	int calculate_output_h(int single_channel);
// Return dimensions for canvas if smaller dimensions has zoom of 1
	void calculate_conformed_dimensions(int single_channel, float &w, float &h);
// Get the total output size scaled to aspect ratio
	void output_dimensions_scaled(int &w, int &h);
	float get_aspect_ratio();

// Titles of all subfolders
	ArrayList<char*> folders;
// Clips
	ArrayList<EDL*> clips;

// Media files
// Shared between all EDLs
	Assets *assets;



	Tracks *tracks;
	Labels *labels;
	Presentations *presentations;
// Shared between all EDLs
	EDLSession *session;
// Specific to this EDL
	LocalSession *local_session;




// Use parent Assets if nonzero
	EDL *parent_edl;


// unique ID of this EDL for resource window
	int id;
};

#endif
