#include "assets.h"

#ifdef USE_AVIFILE
#include "avifile.h"
#include "except.h"
#endif

#include "clip.h"
#include "file.h"
#include "fileavi.h"
#include "fileavi.inc"
#include "vframe.h"


#include <string.h>

FileAVI::FileAVI(Asset *asset, File *file)
 : FileBase(asset, file)
{
	reset();
	asset->format = FILE_AVI;
	derivative = AVI_NONE;
}

FileAVI::~FileAVI()
{
	close_file();
}

int FileAVI::check_sig(Asset *asset)
{
// Pick whoever gets the most tracks
	int score_lavtools = 0;
	int score_arne2 = 0;
	int score_arne1 = 0;
	int score_avifile = 0;
	int final = 0;
	int result = AVI_NONE;





	check_sig_lavtools(asset, score_lavtools);
	check_sig_arne2(asset, score_arne2);
	check_sig_arne1(asset, score_arne1);
	check_sig_avifile(asset, score_avifile);

	if(score_lavtools > final) 
	{
		final = score_lavtools;
		result = AVI_LAVTOOLS;
	}
	if(score_arne2 > final)
	{
		final = score_arne2;
		result = AVI_ARNE2;
	}
	if(score_arne1 > final)
	{
		final = score_arne1;
		result = AVI_ARNE1;
	}
	if(score_avifile > final)
	{
		final = score_avifile;
		result = AVI_AVIFILE;
	}







	return result;
}

int FileAVI::check_sig_arne2(Asset *asset, int &score)
{
	return 0;
}

int FileAVI::check_sig_arne1(Asset *asset, int &score)
{
	return 0;
}

int FileAVI::check_sig_lavtools(Asset *asset, int &score)
{
	return 0;
}

int FileAVI::check_sig_avifile(Asset *asset, int &score)
{
#ifdef USE_AVIFILE
	IAviReadFile *in_fd = 0;

	try
	{
		in_fd = CreateIAviReadFile(asset->path);
	}
	catch(FatalError& error)
	{
		if(in_fd) delete in_fd;
		return 0;
	}

	int vtracks = in_fd->VideoStreamCount();
	int atracks = in_fd->AudioStreamCount();

	delete in_fd;
	
	score = vtracks + atracks;
	return 1;
#endif
	return 0;
}




int FileAVI::open_file(int rd, int wr)
{
	if(wr)
	{
		if(!strcmp(asset->compression, AVI_DV))
		{
			return open_arne2_out(asset);
		}
		else
		if(!strcmp(asset->compression, AVI_MJPG))
		{
			return open_lavtools_out(asset);
		}	
		else
		if(!strcmp(asset->compression, 	AVI_DIVX))
		{
			return open_avifile_out(asset);
		}
	}
	else
	if(rd)
	{
		derivative = check_sig(asset);


		switch(derivative)
		{
			case AVI_LAVTOOLS:
				return open_lavtools_in(asset);
				break;
			
			case AVI_ARNE2:
				return open_arne2_in(asset);
				break;
			
			case AVI_ARNE1:
				return open_arne1_in(asset);
				break;
			
			case AVI_AVIFILE:
				return open_avifile_in(asset);
				break;
		}


	}

	return 0;
}


	
int FileAVI::open_avifile_out(Asset *asset)
{
#ifdef USE_AVIFILE
	try
	{
		out_fd = CreateIAviWriteFile(asset->path);
	}
	catch(FatalError& error)
	{
		error.Print();
		close_file();
		return 1;
	}

	return 0;
#endif
	return 1;
}

int FileAVI::open_arne2_out(Asset *asset)
{
	return 0;
}

int FileAVI::open_arne1_out(Asset *asset)
{
	return 0;
}

int FileAVI::open_lavtools_out(Asset *asset)
{
	return 0;
}

	
	
	


int FileAVI::open_avifile_in(Asset *asset)
{
#ifdef USE_AVIFILE
	try
	{
		in_fd = CreateIAviReadFile(asset->path);
	}
	catch(FatalError& error)
	{
		error.Print();
		close_file();
		return 1;
	}

	asset->layers = in_fd->VideoStreamCount();
	if(asset->layers)
	{
		BITMAPINFOHEADER bitmapinfo;        // BITMAPINFOHEADER

		for(int i = 0; i < asset->layers && i < MAX_STREAMS; i++)
		{
			vstream[i] = in_fd->GetStream(i, IAviReadStream::Video);
			vstream[i]->StartStreaming();
			vstream[i]->GetDecoder()->SetDestFmt(24);
			vstream[i]->SeekToTime(0);
//printf("FileAVI::open_file %d %p\n", i, vstream[i]);
		}

		asset->video_data = 1;
		asset->frame_rate = (double)1 / vstream[0]->GetFrameTime();
		asset->video_length = vstream[0]->GetEndPos();
		vstream[0]->GetVideoFormatInfo(&bitmapinfo);
	    asset->width = bitmapinfo.biWidth;
	    asset->height = bitmapinfo.biHeight;

		asset->compression[0] = ((char*)&bitmapinfo.biCompression)[0];
		asset->compression[1] = ((char*)&bitmapinfo.biCompression)[1];
		asset->compression[2] = ((char*)&bitmapinfo.biCompression)[2];
		asset->compression[3] = ((char*)&bitmapinfo.biCompression)[3];
		source_cmodel = BC_RGB888;
	}

	asset->audio_data = in_fd->AudioStreamCount();

	if(asset->audio_data)
	{
		WAVEFORMATEX waveinfo;
		char *extinfo;

		for(int i = 0; i < 1 && i < MAX_STREAMS; i++)
		{
			astream[i] = in_fd->GetStream(i, IAviReadStream::Audio);
			astream[i]->StartStreaming();
		}

		astream[0]->GetAudioFormatInfo(&waveinfo, &extinfo);
		asset->channels = waveinfo.nChannels;
		asset->sample_rate = waveinfo.nSamplesPerSec;
		asset->bits = MAX(16, waveinfo.wBitsPerSample);
		asset->audio_length = astream[0]->GetEndPos();
	}
asset->dump();
	return 0;
#endif
	return 1;
}

int FileAVI::open_arne2_in(Asset *asset)
{
	return 0;
}

int FileAVI::open_arne1_in(Asset *asset)
{
	return 0;
}

int FileAVI::open_lavtools_in(Asset *asset)
{
	return 0;
}


















int FileAVI::close_file()
{
#ifdef USE_AVIFILE
	if(in_fd) delete in_fd;
	if(out_fd) delete out_fd;
#endif
	if(temp_audio) delete [] temp_audio;
	reset();
}

void FileAVI::reset()
{
#ifdef USE_AVIFILE
	in_fd = 0;
	out_fd = 0;
#endif
	temp_audio = 0;
	temp_allocated = 0;
}



void FileAVI::get_parameters(BC_WindowBase *parent_window, 
		Asset *asset, 
		BC_WindowBase* &format_window,
		int audio_options,
		int video_options,
		int lock_compressor)
{
}

int FileAVI::set_audio_position(long x)
{
#ifdef USE_AVIFILE
// quicktime sets positions for each track seperately so store position in audio_position
	if(x >= 0 && x < asset->audio_length)
		return astream[file->current_layer]->Seek(x);
	else
		return 1;
#endif
}

int FileAVI::set_video_position(long x)
{
#ifdef USE_AVIFILE
	if(x >= 0 && x < asset->video_length)
		return vstream[file->current_layer]->Seek(x);
	else
		return 1;
#endif
}

int FileAVI::read_samples(double *buffer, long len)
{
#ifdef USE_AVIFILE
	Unsigned samples_read, bytes_read;

printf("FileAVI::read_samples 1\n");
	if(temp_audio && temp_allocated < len * asset->bits / 8 * asset->channels)
	{
		delete [] temp_audio;
		temp_allocated = 0;
	}
	if(!temp_allocated)
	{
		temp_allocated = len * asset->bits / 8 * asset->channels;
		temp_audio = new unsigned char[temp_allocated];
	}

	astream[0]->ReadFrames((void*)temp_audio, 
		(unsigned int)temp_allocated,
	 	(unsigned int)len,
	 	samples_read, 
		bytes_read);
	
// Extract single channel
	switch(asset->bits)
	{
		case 16:
		{
			int16_t *temp_int16 = (int16_t*)temp_audio;
			for(int i = 0, j = file->current_channel; 
				i < len;
				i++, j += asset->channels)
			{
				buffer[i] = (double)temp_int16[j] / 32767;
			}
			break;
		}
	}

#endif


	return 0;
}

int FileAVI::read_frame(VFrame *frame)
{
	int result = 0;

#ifdef USE_AVIFILE
	vstream[file->current_layer]->ReadFrame();
	CImage *temp_image = vstream[file->current_layer]->GetFrame();
//printf("FileAVI::read_frame 1 %d %d\n", source_cmodel, frame->get_color_model());
	switch(source_cmodel)
	{
		case BC_RGB888:
			if(frame->get_color_model() == BC_RGB888)
				bcopy(temp_image->data(), 
					frame->get_data(), 
					VFrame::calculate_data_size(asset->width, 
						asset->height, 
						-1, 
						BC_RGB888));
			break;
	}
#endif
	return result;
}

long FileAVI::compressed_frame_size()
{
	int result = 0;
	return result;
}

int FileAVI::read_compressed_frame(VFrame *buffer)
{
	long result = 0;

	return result;
}

int FileAVI::write_compressed_frame(VFrame *buffer)
{
	int result = 0;

	return result;
}

int FileAVI::write_frames(VFrame ***frames, int len)
{
	return 0;
}


int FileAVI::write_samples(double **buffer, long len)
{
	return 0;
}
