#ifndef FILEAVI_H
#define FILEAVI_H

#include "assets.inc"
#include "file.inc"
#include "filebase.h"

class IAviWriteFile;
class IAviReadFile;
class IAviReadStream;

#define MAX_STREAMS 256

class FileAVI : public FileBase
{
public:
	FileAVI(Asset *asset, File *file);
	~FileAVI();

	static int check_sig(Asset *asset);
	static void get_parameters(BC_WindowBase *parent_window, 
		Asset *asset, 
		BC_WindowBase* &format_window,
		int audio_options,
		int video_options,
		int lock_compressor);

	int open_file(int rd, int wr);
	int close_file();
	void reset();
	int set_audio_position(long x);
	int set_video_position(long x);
	int write_samples(double **buffer, long len);
	int write_frames(VFrame ***frames, int len);
	int write_compressed_frame(VFrame *buffer);
	int read_compressed_frame(VFrame *buffer);
	long compressed_frame_size();
	int read_samples(double *buffer, long len);
	int read_frame(VFrame *frame);



private:
	static int check_sig_arne2(Asset *asset, int &score);
	static int check_sig_arne1(Asset *asset, int &score);
	static int check_sig_lavtools(Asset *asset, int &score);
	static int check_sig_avifile(Asset *asset, int &score);

	int open_avifile_in(Asset *asset);
	int open_arne2_in(Asset *asset);
	int open_arne1_in(Asset *asset);
	int open_lavtools_in(Asset *asset);


	int open_avifile_out(Asset *asset);
	int open_arne2_out(Asset *asset);
	int open_arne1_out(Asset *asset);
	int open_lavtools_out(Asset *asset);


// from fileavi.inc
	int derivative;
	
#ifdef USE_AVIFILE
	IAviWriteFile *out_fd;
	IAviReadFile *in_fd;

	IAviReadStream *astream[MAX_STREAMS];
	IAviReadStream *vstream[MAX_STREAMS];
#endif


	unsigned char *temp_audio;
	long temp_allocated;
	int source_cmodel;
};

#endif
