#include "assets.h"
#include "file.h"
#include "filelist.h"
#include "guicast.h"
#include "mwindow.inc"
#include "vframe.h"

#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>


FileList::FileList(Asset *asset, File *file)
 : FileBase(asset, file)
{
	reset_parameters();
	asset->video_data = 1;
}

FileList::~FileList()
{
	close_file();
}

int FileList::reset_parameters_derived()
{
	data = 0;
	stream = 0;
}

int FileList::open_file(int rd, int wr)
{
	char flags[10];
	this->rd = rd;
	this->wr = wr;
	get_mode(flags, rd, wr);

	if(!(stream = fopen(asset->path, flags))) 
	{
		perror("FileList::open_file");
		return 1;
	}

// skip header for write
	if(wr)
	{
	}
	else
	if(rd)
	{
		return read_header();
	}

	file->current_frame = 0;
	data = new VFrame;

	return 0;
}

long FileList::get_video_length()
{
	return path_list.total;
}

int FileList::set_video_position(long x)
{
	return 0;
}

long FileList::get_memory_usage()
{
	return 1;
}

int FileList::set_layer(int layer)
{
	return 0;
}


int FileList::close_file_derived()
{
	if(stream)
	{
		if(wr) write_header();
		fclose(stream);
		for(int i = 0; i < path_list.total; i++) delete [] path_list.values[i];
		path_list.remove_all();
	}
	if(data) delete data;
	reset_parameters();
	return 0;
}

int FileList::write_header()
{
	fprintf(stream, "%s\n", list_title());
	fprintf(stream, "# First line is always %s\n", list_title());
	fprintf(stream, "# Second line is frame rate\n");
	fprintf(stream, "%f\n", asset->frame_rate);
	fprintf(stream, "# Width\n");
	fprintf(stream, "%d\n", asset->width);
	fprintf(stream, "# Height\n");
	fprintf(stream, "%d\n", asset->height);
	fprintf(stream, "# List of image files follows\n");
	for(int i = 0; i < path_list.total; i++)
	{
		fprintf(stream, "%s\n", path_list.values[i]);
	}
	return 0;
}

int FileList::read_header()
{
	char string[1024], *new_entry;

// Don't want a user configured frame rate to get destroyed
// 	if(asset->frame_rate == 0)
// 		asset->frame_rate = 10;
	asset->width = 256;
	asset->height = 256;
	fseek(stream, 0, SEEK_SET);

// Get information about the frames
	do
	{
		fgets(string, 1024, stream);
	}while(!feof(stream) && (string[0] == '#' || string[0] == ' ' || isalpha(string[0])));

// Don't want a user configured frame rate to get destroyed
	if(asset->frame_rate == 0)
		asset->frame_rate = atof(string);

	do
	{
		fgets(string, 1024, stream);
	}while(!feof(stream) && (string[0] == '#' || string[0] == ' '));
	asset->width = atol(string);

	do
	{
		fgets(string, 1024, stream);
	}while(!feof(stream) && (string[0] == '#' || string[0] == ' '));
	asset->height = atol(string);

	asset->layers = 1;
	asset->audio_data = 0;
	asset->video_data = 1;

// Get all the paths
	while(!feof(stream))
	{
		fgets(string, 1024, stream);
		if(strlen(string) && string[0] != '#' && string[0] != ' ' && !feof(stream))
		{
			string[strlen(string) - 1] = 0;
			path_list.append(new_entry = new char[strlen(string) + 1]);
			strcpy(new_entry, string);
		}
	}
//for(int i = 0; i < path_list.total; i++) printf("%s\n", path_list.values[i]);
	return 0;
}

int FileList::read_frame(VFrame *frame)
{
	char *path = path_list.values[file->current_frame];
	FILE *in;
	if(!(in = fopen(path, "rb")))
	{
		perror("FileList::read_frame");
	}
	else
	{
		struct stat ostat;
		stat(path, &ostat);

		switch(frame->get_color_model())
		{
			case BC_COMPRESSED:
			{
				frame->allocate_compressed_data(ostat.st_size);
				frame->set_compressed_size(ostat.st_size);
				fread(frame->get_data(), ostat.st_size, 1, in);
				break;
			}
			default:
				data->allocate_compressed_data(ostat.st_size);
				data->set_compressed_size(ostat.st_size);
				fread(data->get_data(), ostat.st_size, 1, in);
				return read_frame(frame, data);
				break;
		}

		fclose(in);
	}

}

int FileList::write_frames(VFrame ***frames, int len)
{
	for(int i = 0; i < len; i++)
	{
		for(int j = 0; j < asset->layers; j++)
		{
			int result = 0;
			FILE *file;

			write_frame(frames[j][i], data);

			char *path = create_path();
			if(!(file = fopen(path, "wb"))) result = 1;
			else
			{
				fwrite(data->get_data(), data->get_compressed_size(), 1 , file);
				fclose(file);
			}
		}
	}
	return 0;
}









long FileList::compressed_frame_size()
{
	FILE *jpeg_in;
	long result;

	if(!(jpeg_in = fopen(path_list.values[file->current_frame], "rb")))
	{
		printf("FileList::compressed_frame_size %s", path_list.values[file->current_frame]);
		return 0;
	}

	fseek(jpeg_in, 0, SEEK_END);
	result = ftell(jpeg_in);
	fclose(jpeg_in);
	return result;
}

int FileList::read_compressed_frame(VFrame *buffer)
{
	FILE *jpeg_in;
	long result, length;

	if(!(jpeg_in = fopen(path_list.values[file->current_frame], "rb")))
	{
		printf("FileList::read_compressed_frame %s", path_list.values[file->current_frame]);
		return 1;
	}

	fseek(jpeg_in, 0, SEEK_END);
	length = ftell(jpeg_in);
	fseek(jpeg_in, 0, SEEK_SET);

	if(buffer->get_compressed_allocated() < length) 
	{
		fclose(jpeg_in);
		return 1;
	}
	result = !fread(buffer->get_data(), length, 1, jpeg_in);
	buffer->set_compressed_size(length);
	fclose(jpeg_in);
	return result;
}

int FileList::write_compressed_frame(VFrame *buffer)
{
	char *path;
	int result = 0;
	FILE *file;

	path = create_path();
	if(!(file = fopen(path, "wb")))
	{
		perror("FileList::write_compressed_frame\n");
		result = 1;
	}
	else
	{
		result = !fwrite(buffer->get_data(), buffer->get_compressed_size(), 1, file);
		fclose(file);
	}
	return result;
}



char* FileList::create_path()
{
	int k;
	char *path;
	char output[1024];
	if(file->current_frame >= path_list.total)
	{
		strcpy(output, asset->path);
		for(k = strlen(output) - 1; k > 0 && output[k] != '.'; k--)
			;
		if(k <= 0) k = strlen(output);
		sprintf(&output[k], "%06d%s", file->current_frame, extension());
		path = new char[strlen(output) + 1];
		strcpy(path, output);
		path_list.append(path);
		return path;
	}
	else
	{
// Overwrite an old path
		path = path_list.values[file->current_frame];
		return path;
	}
}

