#ifndef FILELIST_H
#define FILELIST_H

#include "file.inc"
#include "filebase.h"
#include "vframe.inc"

// Any file which is a list of frames

class FileList : public FileBase
{
public:
	FileList(Asset *asset, File *file);
	virtual ~FileList();

// basic commands for every file interpreter
	virtual char* list_title() { return ""; };
	virtual char* extension() { return "."; };
	int open_file(int rd, int wr);
	int close_file_derived();
	long get_video_length();
	long get_memory_usage();
	int set_layer(int layer);
	int set_video_position(long x);
	long compressed_frame_size(); // Return size of frame at current position
	int read_compressed_frame(VFrame *buffer);
	int write_compressed_frame(VFrame *buffer);

	int read_header();
	int read_frame(VFrame *frame);
	virtual int read_frame(VFrame *frame, VFrame *data) { return 0; };
	int write_frames(VFrame ***frames, int len);
	virtual int write_frame(VFrame *frame, VFrame *data) { return 0; };
	
	int read_frame(VFrame *frame, PluginBuffer *buffer, long byte_offset, 
			float in_x1, float in_y1, float in_x2, float in_y2,
			float out_x1, float out_y1, float out_x2, float out_y2, 
			int alpha, int use_alpha, int use_float, int interpolate);
	VFrame* read_frame(int use_alpha, int use_float);

	int write_frames(VFrame ***frames, 
			PluginBuffer *video_ram, 
			int len, 
			int use_alpha, 
			int use_float);

// Temp storage for compressed data
	VFrame *data;

private:
	char* create_path();
	int read_raw(VFrame *frame, 
		float in_x1, float in_y1, float in_x2, float in_y2,
		float out_x1, float out_y1, float out_x2, float out_y2, 
		int alpha, int use_alpha, int use_float, int interpolate);
	int reset_parameters_derived();
	int write_header();
	ArrayList<char*> path_list;     // List of JPEG files in the case of a list
	FILE *stream;
};


#endif
