#include "assets.h"
#include "bitspopup.h"
#include "byteorder.h"
#include "file.h"
#include "filemov.h"
#include "guicast.h"
#include "mwindow.inc"
#include "pluginbuffer.h"
#include "vframe.h"
#include "videodevice.inc"

#define DV_NAME "Direct copy DV"
#define PNG_NAME "PNG"
#define PNGA_NAME "PNG with Alpha"
#define RGB_NAME "Uncompressed RGB"
#define RGBA_NAME "Uncompressed RGBA"
#define YUV420_NAME "YUV 4:2:0 Planar"
#define YUV422_NAME "YUV 4:2:2 Packed"
#define YUV411_NAME "YUV 4:1:1 Planar"
#define YUV444_NAME "YUV 4:4:4 Planar"
#define YUVA4444_NAME "YUV 4:4:4:4 Planar alpha"
#define YUV444_10BIT_NAME "YUV 4:4:4 10 Bit Planar"
#define QTJPEG_NAME "JPEG Photo"
#define MJPA_NAME "Motion JPEG A"


FileMOV::FileMOV(Asset *asset, File *file)
 : FileBase(asset, file)
{
	reset_parameters();
	asset->format = FILE_MOV;
	asset->byte_order = 0;
	suffix_number = 0;
}

FileMOV::~FileMOV()
{
	close_file();
}

void FileMOV::get_parameters(BC_WindowBase *parent_window, 
	Asset *asset, 
	BC_WindowBase* &format_window,
	int audio_options,
	int video_options,
	int lock_compressor,
	int use_dv)
{
	if(audio_options)
	{
		MOVConfigAudio *window = new MOVConfigAudio(parent_window, asset);
		format_window = window;
		window->create_objects();
		window->run_window();
		delete window;
	}
	else
	if(video_options)
	{
		MOVConfigVideo *window = new MOVConfigVideo(parent_window, 
			asset, 
			lock_compressor,
			use_dv);
		format_window = window;
		window->create_objects();
		window->run_window();
		delete window;
	}
}

int FileMOV::reset_parameters_derived()
{
	fd = 0;
	prev_track = 0;
	frame = 0;
	temp_frame = 0;
	quicktime_atracks = 0;
	quicktime_vtracks = 0;
	depth = 24;
	threads = 0;
	frames_correction = 0;
	samples_correction = 0;
	temp_float = 0;
	temp_allocated = 0;
}


// Just create the Quicktime objects since this routine is also called
// for reopening.
int FileMOV::open_file(int rd, int wr)
{
	this->rd = rd;
	this->wr = wr;
//printf("FileMOV::open_file 1\n");
	if(suffix_number == 0) strcpy(prefix_path, asset->path);
//printf("FileMOV::open_file 1\n");

	if(!(fd = quicktime_open(asset->path, rd, wr)))
	{
		printf("FileMOV::open_file %s\n", asset->path);
		return 1;
	}
//printf("FileMOV::open_file 2\n");

	quicktime_set_cpus(fd, file->cpus);
//printf("FileMOV::open_file 2.1\n");
	if(rd) format_to_asset();
//printf("FileMOV::open_file 2.2\n");

	if(wr) asset_to_format();
//printf("FileMOV::open_file 3\n");

// set the compression parameters if there are any
	quicktime_set_jpeg(fd, asset->quality, 0);
//printf("FileMOV::open_file 4\n");
	return 0;
}

int FileMOV::close_file()
{
//printf("FileMOV::close_file_derived 1\n");
	if(fd)
	{
		quicktime_set_framerate(fd, asset->frame_rate);
		quicktime_close(fd);
	}

//printf("FileMOV::close_file_derived 1\n");
	if(frame)
		delete frame;
	if(temp_frame)
		delete temp_frame;

//printf("FileMOV::close_file_derived 1\n");
	if(threads)
	{
		for(int i = 0; i < file->cpus; i++)
		{
			threads[i]->stop_encoding();
			delete threads[i];
		}
		delete [] threads;
		threads = 0;
	}

//printf("FileMOV::close_file_derived 1\n");
	threadframes.remove_all_objects();
	
	
	if(temp_float) 
	{
		for(int i = 0; i < asset->channels; i++)
			delete [] temp_float[i];
		delete [] temp_float;
	}

//printf("FileMOV::close_file_derived 1\n");
	reset_parameters();
	FileBase::close_file();
//printf("FileMOV::close_file_derived 2\n");
	return 0;
}

void FileMOV::asset_to_format()
{
	if(!fd) return;
	char audio_codec[5];

//printf("FileMOV::asset_to_format 1\n");
// Fix up the Quicktime file.
	quicktime_set_copyright(fd, "Made with Cinelerra for Linux");
	quicktime_set_info(fd, "Quicktime for Linux");

	if(!asset->signed_ && asset->bits == BITSLINEAR8)
	{
		strcpy(audio_codec, QUICKTIME_RAW);
	}
	else
	if(asset->signed_ && asset->bits == BITSLINEAR8)
	{
		strcpy(audio_codec, QUICKTIME_TWOS);
	}
	else
	if(asset->bits == BITSULAW)
	{
		strcpy(audio_codec, QUICKTIME_ULAW);
	}
	else
	if(asset->bits == BITSIMA4)
	{
		strcpy(audio_codec, QUICKTIME_IMA4);
	}
	else
	if(asset->bits == BITSLINEAR16 || asset->bits == BITSLINEAR24)
	{
		strcpy(audio_codec, QUICKTIME_TWOS);
		asset->signed_ = 1;
	}

//printf("FileMOV::asset_to_format 2\n");
	if(asset->video_data)
	{
		char string[16];
// Set up the alpha channel compressors
		if(!strcmp(asset->compression, MOV_RGBA))
		{
			strcpy(string, QUICKTIME_RAW);
			depth = 32;
		}
		else
		if(!strcmp(asset->compression, MOV_PNGA))
		{
			strcpy(string, QUICKTIME_PNG);
			depth = 32;
		}
		else
		{
			strcpy(string, asset->compression);
			depth = 24;
		}

//printf("FileMOV::asset_to_format 3.1\n");
		quicktime_vtracks = quicktime_set_video(fd, 
					asset->layers, 
					asset->width, 
					asset->height,
					asset->frame_rate,
					string);
//printf("FileMOV::asset_to_format 3.2\n");
		for(int i = 0; i < asset->layers; i++)
			quicktime_set_depth(fd, depth, i);
//printf("FileMOV::asset_to_format 3.3\n");
	}
//printf("FileMOV::asset_to_format 3.4\n");

	if(asset->audio_data)
	{
		quicktime_atracks = quicktime_set_audio(fd, 
				asset->channels, 
				asset->sample_rate, 
				asset->bits, 
				audio_codec);
	}
// printf("FileMOV::asset_to_format 4 %d %d %d\n", asset->channels, 
// 				asset->sample_rate, 
// 				asset->bits);
}


void FileMOV::format_to_asset()
{
	if(!fd) return;
	asset->audio_data = quicktime_has_audio(fd);
	if(asset->audio_data)
	{
		asset->channels = 0;
		int qt_tracks = quicktime_audio_tracks(fd);
		for(int i = 0; i < qt_tracks; i++)
			asset->channels += quicktime_track_channels(fd, i);
	
		asset->sample_rate = quicktime_sample_rate(fd, 0);
		asset->bits = quicktime_audio_bits(fd, 0);
// Audio compressor not stored in asset
		char *compressor = quicktime_audio_compressor(fd, 0);
		asset->audio_length = quicktime_audio_length(fd, 0);

		if(quicktime_supported_audio(fd, 0))
		{
			asset->signed_ = 1;
		}

		if(match4(compressor, QUICKTIME_RAW)) 
		{
			asset->signed_ = 0;
		}
		else
		if(match4(compressor, QUICKTIME_TWOS)) 
		{
			asset->signed_ = 1;
		}
		else
// Fake the bits parameter for some codecs
		if(match4(compressor, QUICKTIME_ULAW)) asset->bits = BITSULAW;
		else
		if(match4(compressor, QUICKTIME_IMA4)) asset->bits = BITSIMA4;
	}

// determine if the video can be read before declaring video data
	if(quicktime_has_video(fd) && quicktime_supported_video(fd, 0))
			asset->video_data = 1;

	if(asset->video_data)
	{
		depth = quicktime_video_depth(fd, 0);
		asset->layers = quicktime_video_tracks(fd);
		asset->width = quicktime_video_width(fd, 0);
		asset->height = quicktime_video_height(fd, 0);
		asset->video_length = quicktime_video_length(fd, 0);
// Don't want a user configured frame rate to get destroyed
		if(!asset->frame_rate)
			asset->frame_rate = quicktime_frame_rate(fd, 0);

		strncpy(asset->compression, quicktime_video_compressor(fd, 0), 4);
	}
}

int FileMOV::get_best_colormodel(int driver, int colormodel)
{
	if(colormodel > -1)
	{
		return colormodel;
	}
	else
	{
		switch(driver)
		{
			case PLAYBACK_X11:
				if(match4(asset->compression, QUICKTIME_YUV420)) return BC_YUV420P;
				if(match4(asset->compression, QUICKTIME_YUV422)) return BC_YUV422P;
				if(match4(asset->compression, QUICKTIME_JPEG)) return BC_YUV420P;
				if(match4(asset->compression, QUICKTIME_MJPA)) return BC_YUV422P;
				if(match4(asset->compression, QUICKTIME_DV)) return BC_YUV422P;
				break;
			case PLAYBACK_LML:
			case PLAYBACK_BUZ:
				if(match4(asset->compression, QUICKTIME_MJPA)) 
					return BC_COMPRESSED;
				else
					return BC_YUV422P;
				break;
		}
	}
	return BC_RGB888;
}

int FileMOV::can_copy_from(Asset *asset)
{
	if(!fd) return 0;
	if(asset->format == FILE_JPEG_LIST && match4(this->asset->compression, QUICKTIME_JPEG))
		return 1;
	else
	if(asset->format == FILE_MOV && 
		(match4(asset->compression, this->asset->compression)))
		return 1;

	return 0;
}

int FileMOV::read_raw_frame_possible()
{
	if(depth == 24) return 1;
	else return 0;
}

// REMOVE
int FileMOV::get_render_strategy(ArrayList<int>* render_strategies)
{
// Fastest
	if(search_render_strategies(render_strategies, VRENDER_MJPG) &&
			match4(asset->compression, QUICKTIME_MJPA))
		return VRENDER_MJPG;

// Slowest
	if(search_render_strategies(render_strategies, VRENDER_RGB888))
		return VRENDER_RGB888;

	return VRENDER_VPIXEL;
}

long FileMOV::get_audio_length()
{
	if(!fd) return 0;
	long result = quicktime_audio_length(fd, 0) + samples_correction;

	return result;
}

long FileMOV::get_video_length()
{
	if(!fd) return 0;
	long result =  + frames_correction;
	return result;
}

int FileMOV::set_audio_position(long x)
{
	if(!fd) return 1;
// quicktime sets positions for each track seperately so store position in audio_position
	if(x >= 0 && x < asset->audio_length)
		return quicktime_set_audio_position(fd, x, 0);
	else
		return 1;
}

int FileMOV::set_video_position(long x)
{
	if(!fd) return 1;
	if(x >= 0 && x < asset->video_length)
		return quicktime_set_video_position(fd, x, file->current_layer);
	else
		return 1;
}

int FileMOV::set_channel(int channel)
{
	if(!fd) return 1;
	return 0;
}

int FileMOV::set_layer(int layer)
{
	if(!fd) return 1;
	return 0;
}

void FileMOV::new_audio_temp(long len)
{
	if(temp_allocated && temp_allocated < len)
	{
		for(int i = 0; i < asset->channels; i++)
			delete [] temp_float[i];
		delete [] temp_float;
		temp_allocated = 0;
	}

	if(!temp_allocated)
	{
		temp_allocated = len;
		temp_float = new float*[asset->channels];
		for(int i = 0; i < asset->channels; i++)
			temp_float[i] = new float[len];
	}
}



int FileMOV::write_samples(double **buffer, long len)
{
	int i, j;
	long bytes;
	int result = 0, track_channels = 0;
	int chunk_size;

	if(!fd) return 0;

	if(quicktime_supported_audio(fd, 0))
	{
// Use Quicktime's compressor. (Always used)
// Allocate temp buffer
		new_audio_temp(len);

// Copy to float buffer
		for(i = 0; i < asset->channels; i++)
		{
			for(j = 0; j < len; j++)
			{
				temp_float[i][j] = buffer[i][j];
			}
		}

// Because of the way Quicktime's compressors work we want to limit the chunk
// size to speed up decompression.
		float **channel_ptr;
		channel_ptr = new float*[asset->channels];

		for(j = 0; j < len && !result; )
		{
			chunk_size = asset->sample_rate;
			if(j + chunk_size > len) chunk_size = len - j;

			for(i = 0; i < asset->channels; i++)
			{
				channel_ptr[i] = &temp_float[i][j];
			}

			result = quicktime_encode_audio(fd, 0, channel_ptr, chunk_size);
			j += asset->sample_rate;
		}

		delete [] channel_ptr;
	}
	return result;
}

int FileMOV::write_frames(VFrame ***frames, int len)
{
	int i, j, k, result = 0;
	int default_compressor = 1;

	if(!fd) return 0;

	for(i = 0; i < asset->layers && !result; i++)
	{
		if(frames[i][0]->get_color_model() == BC_COMPRESSED)
		{
			default_compressor = 0;
			for(j = 0; j < len && !result; j++)
			{
//printf("FileMOV::write_frames %d\n", frames[i][j]->get_compressed_size());
				result = quicktime_write_frame(fd,
					frames[i][j]->get_data(),
					frames[i][j]->get_compressed_size(),
					i);
			}
		}
		else
		if(match4(asset->compression, QUICKTIME_YUV420) ||
			match4(asset->compression, QUICKTIME_YUV422) ||
			match4(asset->compression, QUICKTIME_RAW))
		{
// Direct copy planes where possible
			default_compressor = 0;
			for(j = 0; j < len && !result; j++)
			{
//printf("FileMOV::write_frames 1 %d\n", frames[i][j]->get_color_model());
				quicktime_set_cmodel(fd, frames[i][j]->get_color_model());
//printf("FileMOV::write_frames 1 %d\n", cmodel_is_planar(frames[i][j]->get_color_model()));
				if(cmodel_is_planar(frames[i][j]->get_color_model()))
				{
					unsigned char *planes[3];
					planes[0] = frames[i][j]->get_y();
					planes[1] = frames[i][j]->get_u();
					planes[2] = frames[i][j]->get_v();
					result = quicktime_encode_video(fd, planes, i);
				}
				else
				{
					result = quicktime_encode_video(fd, frames[i][j]->get_rows(), i);
//printf("FileMOV::write_frames 2 %d\n", result);
				}
//printf("FileMOV::write_frames 2\n");
			}
		}
		else
		if(file->cpus > 1 && 
			(match4(asset->compression, QUICKTIME_JPEG) || 
			match4(asset->compression, QUICKTIME_MJPA)))
		{
			default_compressor = 0;
// Compress symmetrically on an SMP system.
			ThreadStruct *threadframe;
			int fields = match4(asset->compression, QUICKTIME_MJPA) ? 2 : 1;

// Set up threads for symmetric compression.
			if(!threads)
			{
				threads = new FileMOVThread*[file->cpus];
				for(j = 0; j < file->cpus; j++)
				{
					threads[j] = new FileMOVThread(this, fields);
					threads[j]->start_encoding();
				}
			}

// Set up the frame structures for asynchronous compression.
// The mjpeg object must exist in each threadframe because it is where the output
// is stored.
			while(threadframes.total < len)
			{
				threadframes.append(threadframe = new ThreadStruct);
			}

// Load thread frame structures with new frames.
			for(j = 0; j < len; j++)
			{
				threadframes.values[j]->input = frames[i][j];
				threadframes.values[j]->completion_lock.lock();
			}
			total_threadframes = len;
			current_threadframe = 0;

// Start the threads compressing
			for(j = 0; j < file->cpus; j++)
			{
				threads[j]->encode_buffer();
			}

// Write the frames as they're finished
			for(j = 0; j < len; j++)
			{
				threadframes.values[j]->completion_lock.lock();
				threadframes.values[j]->completion_lock.unlock();
//printf("filemov 1\n");
				if(!result)
				{
					result = quicktime_write_frame(fd, 
						threadframes.values[j]->output,
						threadframes.values[j]->output_size,
						i);
				}
			}
		}

		if(default_compressor)
// Use the library's built in compressor.
			for(j = 0; j < len && !result; j++)
			{
				quicktime_set_cmodel(fd, frames[i][j]->get_color_model());
				if(cmodel_is_planar(frames[i][j]->get_color_model()))
				{
					unsigned char *planes[3];
					planes[0] = frames[i][j]->get_y();
					planes[1] = frames[i][j]->get_u();
					planes[2] = frames[i][j]->get_v();
					result = quicktime_encode_video(fd, planes, i);
				}
				else
				{
					result = quicktime_encode_video(fd, frames[i][j]->get_rows(), i);
				}
			}
	}
	return result;
}

int FileMOV::read_frame(VFrame *frame)
{
	if(!fd) return 1;
	int result = 0;

//printf("FileMOV::read_frame 1 %d %p %d\n", file->current_frame, frame, frame->get_color_model());
	switch(frame->get_color_model())
	{
		case BC_COMPRESSED:
			frame->allocate_compressed_data(quicktime_frame_size(fd, file->current_frame, file->current_layer));
			frame->set_compressed_size(quicktime_frame_size(fd, file->current_frame, file->current_layer));
//printf("FileMOV::read_frame %d\n", frame->get_compressed_size());
			result = quicktime_read_frame(fd, frame->get_data(), file->current_layer);
			break;

// Progressive
		case BC_YUV420P:
		case BC_YUV422P:
		{
			unsigned char *row_pointers[3];
			row_pointers[0] = frame->get_y();
			row_pointers[1] = frame->get_u();
			row_pointers[2] = frame->get_v();

//printf("FileMOV::read_frame 1\n");
			quicktime_decode_scaled(fd, 
				0,
				0,
				asset->width,
				asset->height,
				frame->get_w(), 
				frame->get_h(),
				frame->get_color_model(),
				row_pointers,
				file->current_layer);
//printf("FileMOV::read_frame 2\n");
//for(int i = 0; i < 5000; i++) row_pointers[0][i] = 255;
		}
			break;

// Packed
		default:
//printf("FileMOV::read_frame 2 %d %d\n", frame->get_color_model(), file->current_frame);
			result = quicktime_decode_scaled(fd, 
				0,
				0,
				asset->width,
				asset->height,
				frame->get_w(),
				frame->get_h(),
				frame->get_color_model(), 
				frame->get_rows(),
				file->current_layer);
//printf("FileMOV::read_frame 3\n");
			break;
	}
//for(int i = 0; i < frame->get_w() * 3 * 20; i++) 
//	((u_int16_t*)frame->get_rows()[0])[i] = 0xffff;
//printf("FileMOV::read_frame 4\n");
	return result;
}



long FileMOV::compressed_frame_size()
{
	if(!fd) return 0;
	return quicktime_frame_size(fd, file->current_frame, file->current_layer);
}

int FileMOV::read_compressed_frame(VFrame *buffer)
{
	long result;
	if(!fd) return 0;

	result = quicktime_read_frame(fd, buffer->get_data(), file->current_layer);
	buffer->set_compressed_size(result);
	result = !result;
	return result;
}

int FileMOV::write_compressed_frame(VFrame *buffer)
{
	int result = 0;
	if(!fd) return 0;

	result = quicktime_write_frame(fd, 
		buffer->get_data(), 
		buffer->get_compressed_size(), 
		file->current_layer);
	return result;
}


int FileMOV::read_raw_frame(VFrame *frame, PluginBuffer *buffer, long byte_offset)
{
	int result = 0;
	if(!fd) return 0;
	quicktime_set_video_position(fd, file->current_frame, file->current_layer);
	result = quicktime_decode_video(fd, frame->get_rows(), file->current_layer);
	return result;
}


int FileMOV::read_raw(VFrame *frame, 
		float in_x1, float in_y1, float in_x2, float in_y2,
		float out_x1, float out_y1, float out_x2, float out_y2, 
		int use_float, int interpolate)
{
	long i, color_channels, result = 0;
	if(!fd) return 0;
	quicktime_set_video_position(fd, file->current_frame, file->current_layer);
// Develop importing strategy
	switch(frame->get_color_model())
	{
		case BC_RGB888:
			result = quicktime_decode_video(fd, frame->get_rows(), file->current_layer);
			break;
		case BC_RGBA8888:
			break;
		case BC_RGB161616:
			break;
		case BC_RGBA16161616:
			break;
		case BC_YUV888:
			break;
		case BC_YUVA8888:
			break;
		case BC_YUV161616:
			break;
		case BC_YUVA16161616:
			break;
		case BC_YUV420P:
			break;
	}
	return result;
}

// Overlay samples
int FileMOV::read_samples(double *buffer, long len)
{
	int qt_track, qt_channel;

	if(!fd) return 0;

//printf("FileMOV::read_samples %ld %ld\n", file->current_sample, quicktime_audio_position(fd, 0));
	if(quicktime_supported_audio(fd, 0))
	{
		new_audio_temp(len);

		if(quicktime_decode_audio(fd, 0, temp_float[0], len, file->current_channel))
		{
			printf("FileMOV::read_samples: quicktime_decode_audio failed\n");
			return 1;
		}
		else
		{
			for(int i = 0; i < len; i++) buffer[i] = temp_float[0][i];
		}
	}

	return 0;
}


char* FileMOV::strtocompression(char *string)
{
	if(!strcasecmp(string, DV_NAME)) return QUICKTIME_DV;
	if(!strcasecmp(string, PNG_NAME)) return QUICKTIME_PNG;
	if(!strcasecmp(string, PNGA_NAME)) return MOV_PNGA;
	if(!strcasecmp(string, RGB_NAME)) return QUICKTIME_RAW;
	if(!strcasecmp(string, RGBA_NAME)) return MOV_RGBA;
	if(!strcasecmp(string, QTJPEG_NAME)) return QUICKTIME_JPEG;
	if(!strcasecmp(string, MJPA_NAME)) return QUICKTIME_MJPA;
	if(!strcasecmp(string, YUV420_NAME)) return QUICKTIME_YUV420;
	if(!strcasecmp(string, YUV411_NAME)) return QUICKTIME_YUV411;
	if(!strcasecmp(string, YUV422_NAME)) return QUICKTIME_YUV422;
	if(!strcasecmp(string, YUV444_NAME)) return QUICKTIME_YUV444;
	if(!strcasecmp(string, YUVA4444_NAME)) return QUICKTIME_YUVA4444;
	if(!strcasecmp(string, YUV444_10BIT_NAME)) return QUICKTIME_YUV444_10bit;
	return QUICKTIME_RAW;
}

char* FileMOV::compressiontostr(char *string)
{
	if(match4(string, QUICKTIME_PNG)) return PNG_NAME;
	if(match4(string, MOV_PNGA)) return PNGA_NAME;
	if(match4(string, QUICKTIME_RAW)) return RGB_NAME;
	if(match4(string, MOV_RGBA)) return RGBA_NAME;
	if(match4(string, QUICKTIME_JPEG)) return QTJPEG_NAME;
	if(match4(string, QUICKTIME_MJPA)) return MJPA_NAME;
	if(match4(string, QUICKTIME_DV)) return DV_NAME;
	if(match4(string, QUICKTIME_YUV420)) return YUV420_NAME;
	if(match4(string, QUICKTIME_YUV411)) return YUV411_NAME;
	if(match4(string, QUICKTIME_YUV422)) return YUV422_NAME;
	if(match4(string, QUICKTIME_YUV444)) return YUV444_NAME;
	if(match4(string, QUICKTIME_YUVA4444)) return YUVA4444_NAME;
	if(match4(string, QUICKTIME_YUV444_10bit)) return YUV444_10BIT_NAME;
	return RGB_NAME;
}





ThreadStruct::ThreadStruct()
{
	input = 0;
	output = 0;
	output_allocated = 0;
	output_size = 0;
}

ThreadStruct::~ThreadStruct()
{
	if(output) delete [] output;
}

void ThreadStruct::load_output(mjpeg_t *mjpeg)
{
	if(output_allocated < mjpeg_output_size(mjpeg))
	{
		delete [] output;
		output = 0;
	}
	if(!output)
	{
		output_allocated = mjpeg_output_size(mjpeg);
		output = new unsigned char[output_allocated];
	}
	
	output_size = mjpeg_output_size(mjpeg);
	memcpy(output, mjpeg_output_buffer(mjpeg), output_size);
}


FileMOVThread::FileMOVThread(FileMOV *filemov, int fields) : Thread()
{
	this->filemov = filemov;
	this->fields = fields;
	mjpeg = 0;
}
FileMOVThread::~FileMOVThread()
{
}

int FileMOVThread::start_encoding()
{
//printf("FileMOVThread::start_encoding 1 %d\n", fields);
	mjpeg = mjpeg_new(filemov->asset->width, 
		filemov->asset->height, 
		fields);
	mjpeg_set_quality(mjpeg, filemov->asset->quality);
	mjpeg_set_float(mjpeg, 0);
	done = 0;
	set_synchronous(1);
	input_lock.lock();
	start();
}

int FileMOVThread::stop_encoding()
{
	done = 1;
	input_lock.unlock();
	join();
	if(mjpeg) mjpeg_delete(mjpeg);
}

int FileMOVThread::encode_buffer()
{
	input_lock.unlock();
}

void FileMOVThread::run()
{
	while(!done)
	{
		input_lock.lock();

		if(!done)
		{
// Get a frame to compress.
			filemov->threadframe_lock.lock();
			if(filemov->current_threadframe < filemov->total_threadframes)
			{
// Frame is available to process.
				input_lock.unlock();
				threadframe = filemov->threadframes.values[filemov->current_threadframe];
				filemov->current_threadframe++;
				filemov->threadframe_lock.unlock();

//printf("FileMOVThread 1 %d\n", threadframe->input->get_color_model());
//printf("FileMOVThread 1 %02x%02x\n", mjpeg_output_buffer(mjpeg)[0], mjpeg_output_buffer(mjpeg)[1]);
				mjpeg_compress(mjpeg, 
					threadframe->input->get_rows(), 
					threadframe->input->get_y(), 
					threadframe->input->get_u(), 
					threadframe->input->get_v(),
					threadframe->input->get_color_model(),
					1);

//printf("FileMOVThread 1 %02x%02x\n", mjpeg_output_buffer(mjpeg)[0], mjpeg_output_buffer(mjpeg)[1]);
//printf("FileMOVThread 1\n");
				if(fields > 1)
				{
					long field2_offset;
					mjpeg_insert_quicktime_markers(&mjpeg->output_data,
						&mjpeg->output_size,
						&mjpeg->output_allocated,
						fields,
						&field2_offset);
				}
				threadframe->load_output(mjpeg);
//printf("FileMOVThread 2 %02x%02x\n", mjpeg_output_buffer(mjpeg)[0], mjpeg_output_buffer(mjpeg)[1]);

//printf("FileMOVThread 2\n");
				threadframe->completion_lock.unlock();
			}
			else
				filemov->threadframe_lock.unlock();
		}
	}
}






MOVConfigAudio::MOVConfigAudio(BC_WindowBase *parent_window, Asset *asset)
 : BC_Window(PROGRAM_NAME ": Audio Compression",
 	parent_window->get_abs_cursor_x(),
 	parent_window->get_abs_cursor_y(),
	210,
	200)
{
	this->parent_window = parent_window;
	this->asset = asset;
}

MOVConfigAudio::~MOVConfigAudio()
{
	delete bits_popup;
}
int MOVConfigAudio::create_objects()
{
	int x = 10, y = 10;
	add_tool(new BC_Title(x, y, "Compression:"));
	y += 25;
	bits_popup = new BitsPopup(this, x, y, &asset->bits, 1, 1, 0, 0, 0);
	bits_popup->create_objects();
	y += 40;
	x = 10;
	add_subwindow(new BC_CheckBox(x, y, &asset->dither, "Dither"));
	x += 100;
	add_subwindow(new BC_CheckBox(x, y, &asset->signed_, "Signed"));
	add_subwindow(new BC_OKButton(this));
	return 0;
}

int MOVConfigAudio::close_event()
{
	set_done(0);
	return 1;
}








MOVConfigVideoPopup::MOVConfigVideoPopup(MOVConfigVideo *popup, int x, int y)
 : BC_PopupTextBox(popup, 
		&popup->compression_items,
		FileMOV::compressiontostr(popup->asset->compression),
		x, 
		y, 
		300,
		200)
{
	this->popup = popup;
}

int MOVConfigVideoPopup::handle_event()
{
	strcpy(popup->asset->compression, FileMOV::strtocompression(get_text()));
	popup->create_quality();
	return 1;
}





MOVConfigVideo::MOVConfigVideo(BC_WindowBase *parent_window, 
	Asset *asset, 
	int lock_compressor,
	int use_dv)
 : BC_Window(PROGRAM_NAME ": Video Compression",
 	parent_window->get_abs_cursor_x(),
 	parent_window->get_abs_cursor_y(),
	400,
	160)
{
	this->parent_window = parent_window;
	this->asset = asset;
	quality = 0;
	this->lock_compressor = lock_compressor;
	this->use_dv = use_dv;
	compression_popup = 0;
}

MOVConfigVideo::~MOVConfigVideo()
{
	if(compression_popup) delete compression_popup;
	compression_items.remove_all_objects();
}

int MOVConfigVideo::create_objects()
{
	int x = 10, y = 10;
	if(use_dv) compression_items.append(new BC_ListBoxItem(DV_NAME));
	compression_items.append(new BC_ListBoxItem(QTJPEG_NAME));
	compression_items.append(new BC_ListBoxItem(MJPA_NAME));
	compression_items.append(new BC_ListBoxItem(PNG_NAME));
	compression_items.append(new BC_ListBoxItem(PNGA_NAME));
	compression_items.append(new BC_ListBoxItem(RGB_NAME));
	compression_items.append(new BC_ListBoxItem(RGBA_NAME));
	compression_items.append(new BC_ListBoxItem(YUV420_NAME));
	compression_items.append(new BC_ListBoxItem(YUV422_NAME));
	compression_items.append(new BC_ListBoxItem(YUV411_NAME));
	compression_items.append(new BC_ListBoxItem(YUV444_NAME));
	compression_items.append(new BC_ListBoxItem(YUVA4444_NAME));
	compression_items.append(new BC_ListBoxItem(YUV444_10BIT_NAME));

	add_subwindow(new BC_Title(x, y, "Compression:"));
	y += 25;
	if(!lock_compressor)
	{
		compression_popup = new MOVConfigVideoPopup(this, x, y);
		compression_popup->create_objects();
	}
	else
	{
		add_subwindow(new BC_Title(x, 
			y, 
			FileMOV::compressiontostr(asset->compression),
			MEDIUMFONT,
			RED,
			0));
	}
	y += 40;

	slider_x = x;
	slider_y = y;
	create_quality();

	add_subwindow(new BC_OKButton(this));
	return 0;
}

int MOVConfigVideo::close_event()
{
	set_done(0);
	return 1;
}

void MOVConfigVideo::create_quality()
{
	if(!strcmp(asset->compression, QUICKTIME_JPEG) ||
		!strcmp(asset->compression, QUICKTIME_MJPA))
	{
		if(!quality)
		{
			add_subwindow(quality_title = new BC_Title(slider_x, slider_y, "Quality:"));
			add_subwindow(quality = new BC_ISlider(slider_x + 80, 
				slider_y,
				0,
				200,
				200,
				0,
				100,
				asset->quality,
				0,
				0,
				&asset->quality));
		}
	}
	else
	if(quality)
	{
		delete quality_title;
		delete quality;
		quality = 0;
	}
}
