#ifndef FILEMOV_H
#define FILEMOV_H

#include "bitspopup.inc"
#include "file.inc"
#include "filebase.h"
#include "mutex.h"
#include "thread.h"

// ./Quicktime
#include "libmjpeg.h"
#include "quicktime.h"

class FileMOVThread;

class ThreadStruct
{
public:
	ThreadStruct();
	~ThreadStruct();

	void load_output(mjpeg_t *mjpeg);

	VFrame *input;
	unsigned char *output;  // Output buffer
	long output_size;        // Size of output buffer
	long output_allocated;  // Allocation of output buffer
	Mutex completion_lock;
};

class FileMOV : public FileBase
{
public:
	FileMOV(Asset *asset, File *file);
	~FileMOV();

	friend class FileMOVThread;

	static void get_parameters(BC_WindowBase *parent_window, 
		Asset *asset, 
		BC_WindowBase* &format_window,
		int audio_options,
		int video_options,
		int lock_compressor,
		int use_dv);

	int open_file(int rd, int wr);
	int close_file();
	void asset_to_format();
	void format_to_asset();
	long get_video_length();
	long get_audio_length();
	long get_position();
	int set_video_position(long x);
	int set_audio_position(long x);
	int set_channel(int channel);  // set audio channel for reading
	int set_layer(int layer);      // set layer for reading
	int write_samples(double **buffer, 
			long len);
	int write_frames(VFrame ***frames, int len);
	long compressed_frame_size();
	int read_compressed_frame(VFrame *buffer);
	int write_compressed_frame(VFrame *buffer);

	int read_frame(VFrame *frame);
	int read_samples(double *buffer, long len);
	int read_raw_frame_possible();
	int read_raw_frame(VFrame *buffer, PluginBuffer *plugin_buffer, long byte_offset);

// Direct copy routines
	int get_best_colormodel(int driver, int colormodel);
	int can_copy_from(Asset *asset); // This file can copy frames directly from the asset
	int get_render_strategy(ArrayList<int>* render_strategies);
	static char *strtocompression(char *string);
	static char *compressiontostr(char *string);

private:
	void new_audio_temp(long len);
// read raw audio data
	int read_raw(char *buffer, long samples, int track);  
// overlay raw frame from the current layer and position
	int read_raw(VFrame *frame, 
		float in_x1, float in_y1, float in_x2, float in_y2,
		float out_x1, float out_y1, float out_x2, float out_y2, 
		int use_float, int interpolate);
	int reset_parameters_derived();
	int quicktime_atracks;
	int quicktime_vtracks;
// current positions for when the file descriptor doesn't have the right position
	quicktime_t *fd;
// frame to return through read_frame
	VFrame *frame; 
// Temporary storage for color conversions
	VFrame *temp_frame;
	int depth;        // Depth in bits per pixel
	long frames_correction;  // Correction after 32bit overflow
	long samples_correction;  // Correction after 32bit overflow

// An array of frames for threads to look up and compress on their own.
	ArrayList<ThreadStruct*> threadframes;

	int total_threadframes;     // Number of thread frames in this buffer
	int current_threadframe;    // Next threadframe to compress
	Mutex threadframe_lock;     // Lock threadframe array.

	FileMOVThread **threads;   // One thread for every CPU
	char prefix_path[1024];    // Prefix for new file when 2G limit is exceeded
	int suffix_number;         // Number for new file

// Temp buffers for converting from double to float
	float **temp_float;
	long temp_allocated;
};


// Encoder thread to parallelize certain compression formats, mainly JPEG.
// Only works when no alpha.

class FileMOVThread : public Thread
{
public:
	FileMOVThread(FileMOV *filemov, int fields);
	~FileMOVThread();

	int start_encoding();
	int stop_encoding();
	int encode_buffer();
	void run();

	ThreadStruct *threadframe;    // The frame currently being processed.
	int done;
	FileMOV *filemov;
	Mutex input_lock;     // Wait for new array of threads or completion.
	Mutex quicktime_lock;  // Lock out reopen_file
	mjpeg_t *mjpeg;
	int fields;
};


class MOVConfigAudio : public BC_Window
{
public:
	MOVConfigAudio(BC_WindowBase *parent_window, Asset *asset);
	~MOVConfigAudio();

	int create_objects();
	int close_event();

	BC_WindowBase *parent_window;
	BitsPopup *bits_popup;
	Asset *asset;
};

class MOVConfigVideoPopup;

class MOVConfigVideo : public BC_Window
{
public:
	MOVConfigVideo(BC_WindowBase *parent_window, 
		Asset *asset, 
		int lock_compressor,
		int use_dv);
	~MOVConfigVideo();

	int create_objects();
	int close_event();

	void create_quality();

	ArrayList<BC_ListBoxItem*> compression_items;
	MOVConfigVideoPopup *compression_popup;
	BC_WindowBase *parent_window;
	BC_ISlider *quality;
	BC_Title *quality_title;
	Asset *asset;
	int slider_x, slider_y;
	int lock_compressor;
	int use_dv;
};

class MOVConfigVideoPopup : public BC_PopupTextBox
{
public:
	MOVConfigVideoPopup(MOVConfigVideo *popup, int x, int y);
	int handle_event();
	MOVConfigVideo *popup;
};


#endif
