#include "assets.h"
#include "bitspopup.h"
#include "byteorder.h"
#include "file.h"
#include "filempeg.h"
#include "guicast.h"
#include "mwindow.inc"
#include "pluginbuffer.h"
#include "vframe.h"
#include "videodevice.inc"


FileMPEG::FileMPEG(Asset *asset, File *file)
 : FileBase(asset, file)
{
	reset_parameters();
	asset->format = FILE_MPEG;
	asset->byte_order = 0;
}

FileMPEG::~FileMPEG()
{
	close_file();
}

void FileMPEG::get_parameters(BC_WindowBase *parent_window, 
	Asset *asset, 
	BC_WindowBase* &format_window,
	int audio_options,
	int video_options)
{
// 	if(audio_options)
// 	{
// 		MPEGConfigAudio *window = new MPEGConfigAudio(parent_window, asset);
// 		format_window = window;
// 		window->create_objects();
// 		window->run_window();
// 		delete window;
// 	}
// 	else
	if(video_options)
	{
		MPEGConfigVideo *window = new MPEGConfigVideo(parent_window, asset);
		format_window = window;
		window->create_objects();
		window->run_window();
		delete window;
	}
}

int FileMPEG::check_sig(Asset *asset)
{
	return mpeg3_check_sig(asset->path);
}

int FileMPEG::reset_parameters_derived()
{
	fd = 0;
	prev_track = 0;
	temp_frame = 0;
	last_sample = -1;
}


// Just create the Quicktime objects since this routine is also called
// for reopening.
int FileMPEG::open_file(int rd, int wr)
{
	this->rd = rd;
	this->wr = wr;

	if(rd)
	{
		if(!(fd = mpeg3_open(asset->path)))
		{
			printf("FileMPEG::open_file %s\n", asset->path);
			return 1;
		}
		
		mpeg3_set_cpus(fd, file->cpus);
		mpeg3_set_mmx(fd, 0);
		
		asset->audio_data = mpeg3_has_audio(fd);
		if(asset->audio_data)
		{
			asset->channels = 0;
			for(int i = 0; i < mpeg3_total_astreams(fd); i++)
			{
				asset->channels += mpeg3_audio_channels(fd, i);
			}
			asset->sample_rate = mpeg3_sample_rate(fd, 0);
			asset->audio_length = mpeg3_audio_samples(fd, 0); 
//printf("FileMPEG::open_file 1 %d\n", asset->audio_length);
		}
		
		asset->video_data = mpeg3_has_video(fd);
		if(asset->video_data)
		{
			asset->layers = mpeg3_total_vstreams(fd);
			asset->width = mpeg3_video_width(fd, 0);
			asset->height = mpeg3_video_height(fd, 0);
			asset->video_length = mpeg3_video_frames(fd, 0);
			if(!asset->frame_rate)
				asset->frame_rate = mpeg3_frame_rate(fd, 0);
//printf("FileMPEG::open_file 2 %d\n", asset->video_length);
		}
	}

	return 0;
}

int FileMPEG::close_file()
{
//printf("FileMPEG::close_file_derived 1\n");
	if(fd)
	{
		mpeg3_close(fd);
	}

//printf("FileMPEG::close_file_derived 1\n");
	if(temp_frame) delete temp_frame;

//printf("FileMPEG::close_file_derived 1\n");
	reset_parameters();
	FileBase::close_file();
//printf("FileMPEG::close_file_derived 2\n");
	return 0;
}


int FileMPEG::get_best_colormodel(int driver, int colormodel)
{
	if(colormodel > -1)
	{
		return colormodel;
	}
	else
	{
		switch(driver)
		{
			case PLAYBACK_X11:
				if(mpeg3_colormodel(fd, 0) == MPEG3_YUV420P) return BC_YUV420P;
				if(mpeg3_colormodel(fd, 0) == MPEG3_YUV422P) return BC_YUV422P;
				break;
			case PLAYBACK_LML:
			case PLAYBACK_BUZ:
				return BC_YUV422P;
				break;
		}
	}
	return colormodel;
}

int FileMPEG::can_copy_from(Asset *asset)
{
	if(!fd) return 0;
	return 0;
}

int FileMPEG::set_audio_position(long sample)
{
	if(!fd) return 1;
	
	int channel, stream;
	to_streamchannel(file->current_channel, stream, channel);

//printf("FileMPEG::set_audio_position %d %d %d\n", sample, mpeg3_get_sample(fd, stream), last_sample);
	if(sample != mpeg3_get_sample(fd, stream) &&
		sample != last_sample)
	{
		if(sample >= 0 && sample < asset->audio_length)
			return mpeg3_set_sample(fd, sample, stream);
		else
			return 1;
	}
	return 0;
}

int FileMPEG::set_video_position(long x)
{
	if(!fd) return 1;
	if(x >= 0 && x < asset->video_length)
		return mpeg3_set_frame(fd, x, file->current_layer);
	else
		return 1;
}

void FileMPEG::new_audio_temp(long len)
{
}



int FileMPEG::write_samples(double **buffer, long len)
{
	return 1;
}

int FileMPEG::write_frames(VFrame ***frames, int len)
{
	return 1;
}

int FileMPEG::read_frame(VFrame *frame)
{
	if(!fd) return 1;
	int result = 0;
	int src_cmodel;
	
	if(mpeg3_colormodel(fd, 0) == MPEG3_YUV420P)
		src_cmodel = BC_YUV420P;
	else
	if(mpeg3_colormodel(fd, 0) == MPEG3_YUV422P)
		src_cmodel = BC_YUV422P;

//printf("FileMPEG::read_frame 1 %p %d\n", frame, frame->get_color_model());
	switch(frame->get_color_model())
	{
		case MPEG3_RGB565:
		case MPEG3_BGR888:
		case MPEG3_BGRA8888:
		case MPEG3_RGB888:
		case MPEG3_RGBA8888:
		case MPEG3_RGBA16161616:
			mpeg3_read_frame(fd, 
					frame->get_rows(), /* Array of pointers to the start of each output row */
					0,                    /* Location in input frame to take picture */
					0, 
					asset->width, 
					asset->height, 
					asset->width,                   /* Dimensions of output_rows */
					asset->height, 
					frame->get_color_model(),             /* One of the color model #defines */
					file->current_layer);
			break;

// Use Temp
		default:
// Read these directly
			if(frame->get_color_model() == src_cmodel)
			{
				mpeg3_read_yuvframe(fd,
					(char*)frame->get_y(),
					(char*)frame->get_u(),
					(char*)frame->get_v(),
					0,
					0,
					asset->width,
					asset->height,
					file->current_layer);
			}
			else
// Process through temp frame
			{
				char *y, *u, *v;
				mpeg3_read_yuvframe_ptr(fd,
					&y,
					&u,
					&v,
					file->current_layer);
				cmodel_transfer(frame->get_rows(), 
					0,
					frame->get_y(),
					frame->get_u(),
					frame->get_v(),
					(unsigned char*)y,
					(unsigned char*)u,
					(unsigned char*)v,
					0,
					0,
					asset->width,
					asset->height,
					0,
					0,
					asset->width,
					asset->height,
					src_cmodel, 
					frame->get_color_model(),
					0, 
					asset->width,
					frame->get_w());
			}
//printf("FileMPEG::read_frame 2\n");
			break;
	}
//for(int i = 0; i < frame->get_w() * 3 * 20; i++) 
//	((u_int16_t*)frame->get_rows()[0])[i] = 0xffff;
//printf("FileMPEG::read_frame 3\n");
	return result;
}

void FileMPEG::to_streamchannel(int channel, int &stream_out, int &channel_out)
{
	for(stream_out = 0, channel_out = file->current_channel; 
		stream_out < mpeg3_total_astreams(fd) && 
			channel_out >= mpeg3_audio_channels(fd, stream_out);
		channel_out -= mpeg3_audio_channels(fd, stream_out), stream_out++)
	;
}

FileMPEGBuffer* FileMPEG::oldest_buffer()
{
	int oldest_age = 0, oldest_i = 0;
	for(int i = 0; i < HISTORY_SIZE; i++)
	{
		if(history[i].age > oldest_age)
		{
			oldest_age = history[i].age;
			oldest_i = i;
		}
	}

	return &history[oldest_i];
}

int FileMPEG::read_samples(double *buffer, long len)
{
	if(!fd) return 0;

	new_audio_temp(len);

// This is directed to a FileMPEGBuffer
	float *temp_float = 0;
	int found = 0;
	int stream, channel;
	to_streamchannel(file->current_channel, stream, channel);
	
	
	
//printf("FileMPEG::read_samples 1 %ld %ld %d %d\n", file->current_sample, last_sample, len, channel);



	for(int i = 0; i < HISTORY_SIZE; i++)
	{
		history[i].age++;
	}

// Search history
	for(int i = 0; i < HISTORY_SIZE && !found; i++)
	{
		if(history[i].allocated && 
			history[i].sample == file->current_sample &&
			history[i].channel == channel &&
			history[i].stream == stream &&
			history[i].len == len)
		{
			found = 1;
			temp_float = history[i].data;
			history[i].age = 0;
//printf("FileMPEG::read_samples: got history #: %d channel: %d stream: %d len: %d\n",
//	i, history[i].channel, history[i].stream, len);
		}
	}

// Load new samples for all channels in the current stream.
// Replace oldest FileMPEGBuffers
	if(!found)
	{
//printf("FileMPEG::read_samples: reading history stream: %d\n",
//	stream);
		for(int i = 0; i < mpeg3_audio_channels(fd, stream); i++)
		{
			FileMPEGBuffer *dest = oldest_buffer();

			dest->reallocate(len);
			dest->len = len;
			dest->sample = file->current_sample;
			dest->channel = i;
			dest->stream = stream;
			dest->age = 0;
			
			if(i == 0)
				mpeg3_read_audio(fd, 
					dest->data,      /* Pointer to pre-allocated buffer of floats */
					0,      /* Pointer to pre-allocated buffer of int16's */
					i,          /* Channel to decode */
					len,         /* Number of samples to decode */
					stream);          /* Stream containing the channel */
			else
				mpeg3_reread_audio(fd, 
					dest->data,      /* Pointer to pre-allocated buffer of floats */
					0,      /* Pointer to pre-allocated buffer of int16's */
					i,          /* Channel to decode */
					len,         /* Number of samples to decode */
					stream);          /* Stream containing the channel */
		
			if(i == channel)
				temp_float = dest->data;
		}
	}


	last_sample = file->current_sample;
	for(int i = 0; i < len; i++) buffer[i] = temp_float[i];

	return 0;
}

char* FileMPEG::strtocompression(char *string)
{
	return "";
}

char* FileMPEG::compressiontostr(char *string)
{
	return "";
}










MPEGConfigAudio::MPEGConfigAudio(BC_WindowBase *parent_window, Asset *asset)
 : BC_Window(PROGRAM_NAME ": Audio Compression",
 	parent_window->get_abs_cursor_x(),
 	parent_window->get_abs_cursor_y(),
	210,
	200)
{
	this->parent_window = parent_window;
	this->asset = asset;
}

MPEGConfigAudio::~MPEGConfigAudio()
{
	delete bits_popup;
}
int MPEGConfigAudio::create_objects()
{
	int x = 10, y = 10;
	add_tool(new BC_Title(x, y, "Layer:"));
	y += 30;
	add_tool(new BC_Title(x, y, "Kbits per second:"));
	add_subwindow(new BC_OKButton(this));
	return 0;
}

int MPEGConfigAudio::close_event()
{
	set_done(0);
	return 1;
}











MPEGConfigVideo::MPEGConfigVideo(BC_WindowBase *parent_window, 
	Asset *asset)
 : BC_Window(PROGRAM_NAME ": Video Compression",
 	parent_window->get_abs_cursor_x(),
 	parent_window->get_abs_cursor_y(),
	400,
	160)
{
	this->parent_window = parent_window;
	this->asset = asset;
}

MPEGConfigVideo::~MPEGConfigVideo()
{
}

int MPEGConfigVideo::create_objects()
{
	int x = 10, y = 10;
	add_subwindow(new BC_Title(x, y, "Derivative:"));
	y += 30;
	add_subwindow(new BC_Title(x, y, "Bitrate:"));
	y += 30;
	add_subwindow(new BC_Title(x, y, "Quantization:"));
	y += 30;
	add_subwindow(new BC_Title(x, y, "Frames between I frames:"));
	y += 30;
	add_subwindow(new BC_Title(x, y, "Frames between P frames:"));
	y += 30;
	add_subwindow(new BC_CheckBox(x, y, &asset->progressive, "Progressive frames"));
	y += 30;
	add_subwindow(new BC_CheckBox(x, y, &asset->denoise, "Denoise"));

	add_subwindow(new BC_OKButton(this));
	return 0;
}

int MPEGConfigVideo::close_event()
{
	set_done(0);
	return 1;
}


#if 0

MPEGDerivative::MPEGDerivative(int x, int y, MPEGConfigVideo *gui)
 : BC_PopupMenu()
{
int handle_event();
};

MPEGBitrate::MPEGBitrate(int x, int y, MPEGConfigVideo *gui)
 : BC_TextBox
{
public:
	;
	int handle_event();
};

MPEGQuant::MPEGQuant(int x, int y, MPEGConfigVideo *gui)
 : BC_TextBox
{
public:
	;
	int handle_event();
};

MPEGIFrameDistance::MPEGIFrameDistance(int x, int y, MPEGConfigVideo *gui)
 : BC_TextBox
{
public:
	;
	int handle_event();
};

MPEGPFrameDistance::MPEGPFrameDistance(int x, int y, MPEGConfigVideo *gui)
 : BC_TextBox
{
public:
	;
	int handle_event();
};

MPEGFixedBitrate::MPEGFixedBitrate(int x, int y, MPEGConfigVideo *gui)
 : BC_Radial
{
public:
	;
	int handle_event();
};

MPEGFixedQuant::MPEGFixedBitrate(int x, int y, MPEGConfigVideo *gui)
 : BC_Radial
{
public:
	;
	int handle_event();
};


#endif



FileMPEGBuffer::FileMPEGBuffer()
{
	len = 0;
	allocated = 0;
	sample = 0;
	channel = 0;
	stream = 0;
	data = 0;
	age = 0;
}

FileMPEGBuffer::~FileMPEGBuffer()
{
//printf("FileMPEGBuffer::~FileMPEGBuffer %p\n", data);
	if(data) delete [] data;
}

void FileMPEGBuffer::reallocate(long len)
{
	if(data && len > allocated)
	{
		delete [] data;
		allocated = 0;
		data = 0;
	}
	
	if(!data)
	{
		data = new float[len];
		allocated = len;
	}
	
	age = 0;
}


