#ifndef FILEMPEG_H
#define FILEMPEG_H

#include "bitspopup.inc"
#include "file.inc"
#include "filebase.h"
#include "mutex.h"
#include "resample.inc"
#include "thread.h"

#include "libmpeg3.h"



// Since MPEG audio can't be decoded randomly we need a store
// of previously read samples
#define HISTORY_SIZE (0x400000 / RESAMPLE_CHUNKSIZE)

class FileMPEGBuffer
{
public:
	FileMPEGBuffer();
	~FileMPEGBuffer();

	void reallocate(long len);

// Information about the data
	long len;
	long allocated;
// Where the data came from
	long sample;
	long channel;
	long stream;
	float *data;
	int age;
};


class FileMPEG : public FileBase
{
public:
	FileMPEG(Asset *asset, File *file);
	~FileMPEG();

	static void get_parameters(BC_WindowBase *parent_window, 
		Asset *asset, 
		BC_WindowBase* &format_window,
		int audio_options,
		int video_options);

	static int check_sig(Asset *asset);
	int open_file(int rd, int wr);
	int close_file();
	int set_video_position(long x);
	int set_audio_position(long x);
	int write_samples(double **buffer, 
			long len);
	int write_frames(VFrame ***frames, int len);

	int read_frame(VFrame *frame);
	int read_samples(double *buffer, long len);

// Direct copy routines
	int get_best_colormodel(int driver, int colormodel);
// This file can copy frames directly from the asset
	int can_copy_from(Asset *asset); 
	static char *strtocompression(char *string);
	static char *compressiontostr(char *string);

private:
	FileMPEGBuffer* oldest_buffer();
	void to_streamchannel(int channel, int &stream_out, int &channel_out);
	void new_audio_temp(long len);
	int reset_parameters_derived();
	mpeg3_t *fd;
// Temporary for color conversion
	VFrame *temp_frame;
	long last_sample;

	FileMPEGBuffer history[HISTORY_SIZE];
};


class MPEGConfigAudioPopup;


class MPEGConfigAudio : public BC_Window
{
public:
	MPEGConfigAudio(BC_WindowBase *parent_window, Asset *asset);
	~MPEGConfigAudio();

	int create_objects();
	int close_event();

	BC_WindowBase *parent_window;
	BitsPopup *bits_popup;
	Asset *asset;
};

class MPEGFixedBitrate;
class MPEGFixedQuant;

class MPEGConfigVideo : public BC_Window
{
public:
	MPEGConfigVideo(BC_WindowBase *parent_window, 
		Asset *asset);
	~MPEGConfigVideo();

	int create_objects();
	int close_event();

	BC_WindowBase *parent_window;
	Asset *asset;
	MPEGFixedBitrate *fixed_bitrate;
	MPEGFixedQuant *fixed_quant;
	
};

class MPEGDerivative : public BC_PopupMenu
{
public:
	MPEGDerivative(int x, int y, MPEGConfigVideo *gui);
	int handle_event();
};

class MPEGBitrate : public BC_TextBox
{
public:
	MPEGBitrate(int x, int y, MPEGConfigVideo *gui);
	int handle_event();
};

class MPEGQuant : public BC_TextBox
{
public:
	MPEGQuant(int x, int y, MPEGConfigVideo *gui);
	int handle_event();
};

class MPEGIFrameDistance : public BC_TextBox
{
public:
	MPEGIFrameDistance(int x, int y, MPEGConfigVideo *gui);
	int handle_event();
};

class MPEGPFrameDistance : public BC_TextBox
{
public:
	MPEGPFrameDistance(int x, int y, MPEGConfigVideo *gui);
	int handle_event();
};

class MPEGFixedBitrate : public BC_Radial
{
public:
	MPEGFixedBitrate(int x, int y, MPEGConfigVideo *gui);
	int handle_event();
};

class MPEGFixedQuant : public BC_Radial
{
public:
	MPEGFixedQuant(int x, int y, MPEGConfigVideo *gui);
	int handle_event();
};



#endif
