#include "assets.h"
#include "filetgalist.h"
#include "mwindow.inc"
#include "vframe.h"

#include <string.h>

/* Known image types. */
#define TGA_TYPE_MAPPED      1
#define TGA_TYPE_COLOR       2
#define TGA_TYPE_GRAY        3

/* Only known compression is RLE */
#define TGA_COMP_NONE        0 
#define TGA_COMP_RLE         1 


FileTGAList::FileTGAList(Asset *asset, File *file)
 : FileList(asset, file)
{
	asset->format = FILE_TGA_LIST;
	temp = 0;
}

FileTGAList::~FileTGAList()
{
	if(temp) delete temp;
}

int FileTGAList::check_sig(Asset *asset)
{
	FILE *stream;
	if(!(stream = fopen(asset->path, "rb")))
	{
// file not found
		return 0;
	}
	char test[16];
	fread(test, 16, 1, stream);
	fclose(stream);
	if(test[0] == 'T' && test[1] == 'G' && test[2] == 'A' && 
		test[3] == 'L' && test[4] == 'I' && test[5] == 'S' && 
		test[6] == 'T')
	{
		return 1;
	}
	return 0;
}

void FileTGAList::get_parameters(BC_WindowBase *parent_window, 
		Asset *asset, 
		BC_WindowBase* &format_window,
		int audio_options,
		int video_options)
{
	if(video_options)
	{
		TGAConfigVideo *window = new TGAConfigVideo(parent_window, asset);
		format_window = window;
		window->create_objects();
		window->run_window();
		delete window;
	}
}


#define TGA_RGB_RLE "rle "
#define TGA_RGBA_RLE "rlea"
#define TGA_RGB "raw "
#define TGA_RGBA "rawa"

#define TGA_RGB_RLE_NAME "RGB compressed"
#define TGA_RGBA_RLE_NAME "RGBA compressed"
#define TGA_RGB_NAME "RGB uncompressed"
#define TGA_RGBA_NAME "RGBA uncompressed"

char* FileTGAList::compression_to_str(char *compression)
{
	if(!strcasecmp(compression, TGA_RGB_RLE)) return TGA_RGB_RLE_NAME;
	if(!strcasecmp(compression, TGA_RGBA_RLE)) return TGA_RGBA_RLE_NAME;
	if(!strcasecmp(compression, TGA_RGB)) return TGA_RGB_NAME;
	if(!strcasecmp(compression, TGA_RGBA)) return TGA_RGBA_NAME;
	return TGA_RGB_NAME;
}

char* FileTGAList::str_to_compression(char *string)
{
	if(!strcasecmp(compression_to_str(TGA_RGB_RLE), string)) return TGA_RGB_RLE;
	if(!strcasecmp(compression_to_str(TGA_RGBA_RLE), string)) return TGA_RGBA_RLE;
	if(!strcasecmp(compression_to_str(TGA_RGB), string)) return TGA_RGB;
	if(!strcasecmp(compression_to_str(TGA_RGBA), string)) return TGA_RGBA;
	return TGA_RGB;
}

int FileTGAList::can_copy_from(Asset *asset)
{
	if(asset->format == FILE_TGA_LIST)
		return 1;
	
	return 0;
}

char* FileTGAList::list_title()
{
	return "TGALIST";
}

char* FileTGAList::extension()
{
	return ".tga";
}

int FileTGAList::get_best_colormodel(int driver, int colormodel)
{
	if(colormodel > -1)
	{
		switch(colormodel)
		{
			case BC_RGBA8888:
			case BC_YUVA8888:
			case BC_RGBA16161616:
			case BC_YUVA16161616:
				return BC_RGBA8888;
				break;
			default:
				return BC_RGB888;
				break;
		}
	}
	else
	{
		return BC_RGB888;
	}
}

int FileTGAList::read_frame(VFrame *frame, VFrame *data)
{
	read_tga(asset, frame, data, temp);
	return 0;
}

int FileTGAList::write_frame(VFrame *frame, VFrame *data)
{
	write_tga(asset, frame, data, temp);
	return 0;
}

void FileTGAList::read_tga(Asset *asset, VFrame *frame, VFrame *data, VFrame* &temp)
{
// Read header
#define FOOTERSIZE 26
#define HEADERSIZE 18
	unsigned char *footer, *header;
	int input_cmodel;
	long file_offset = 0;

	footer = data->get_data() + 
		data->get_compressed_size() - 
		FOOTERSIZE;
	header = data->get_data();
	file_offset += HEADERSIZE;

	int image_type;
	int image_compression;
	switch(header[2])
	{
		case 1:
			image_type = TGA_TYPE_MAPPED;
			image_compression = TGA_COMP_NONE;
			break;
		case 2:
			image_type = TGA_TYPE_COLOR;
			image_compression = TGA_COMP_NONE;
			break;
		case 3:
			image_type = TGA_TYPE_GRAY;
			image_compression = TGA_COMP_NONE;
			break;
		case 9:
			image_type = TGA_TYPE_MAPPED;
			image_compression = TGA_COMP_RLE;
			break;
		case 10:
			image_type = TGA_TYPE_COLOR;
			image_compression = TGA_COMP_RLE;
			break;
		case 11:
			image_type = TGA_TYPE_GRAY;
			image_compression = TGA_COMP_RLE;
			break;
		default:
			image_type = 0;
	}
	
	int idlength = header[0];
	int colormaptype = header[1];
	int colormapindex = header[3] + header[4] * 256;
	int colormaplength = header[5] + header[6] * 256;
	int colormapsize = header[7];
	int xorigin = header[8] + header[9] * 256;
	int yorigin = header[10] + header[11] * 256;
	int width = header[12] + header[13] * 256;
	int height = header[14] + header[15] * 256;
	int bpp = header[16];
	int bytes = (bpp + 7) / 8;
	int alphabits = header[17] & 0x0f;
	int fliphoriz = (header[17] & 0x10) ? 1 : 0;
	int flipvert = (header[17] & 0x20) ? 0 : 1;

	if(idlength) file_offset += idlength;

// Get colormap
	unsigned char *tga_cmap;
	unsigned char colormap[4 * 256];

	if(colormaptype == 1)
	{
		int cmap_bytes = (colormapsize + 7) / 8;
		tga_cmap = data->get_data() + file_offset;
		file_offset += colormaplength * cmap_bytes;
		
		switch(colormapsize)
		{
			case 32:
				bgr2rgb(colormap, tga_cmap, colormaplength, cmap_bytes, 1);
				break;
			case 24:
				bgr2rgb(colormap, tga_cmap, colormaplength, cmap_bytes, 0);
				break;
			case 16:
				upsample(colormap, tga_cmap, colormaplength, cmap_bytes);
				break;
		}
	}

	int source_cmodel = BC_RGB888;
	switch(bpp)
	{
		case 32:
			source_cmodel = BC_RGBA8888;
			break;
		case 24:
			source_cmodel = BC_RGB888;
			break;
	}

// Read image
	VFrame *output_frame;
	if(frame->get_color_model() == source_cmodel)
	{
		output_frame = frame;
	}
	else
	{
		if(temp && temp->get_color_model() != source_cmodel)
		{
			delete temp;
			temp = 0;
		}
		
		if(!temp)
		{
			temp = new VFrame(0, width, height, source_cmodel);
		}
		output_frame = temp;
	}

	if(flipvert)
	{
		for(int i = height - 1; i >= 0; i--)
		{
			read_line(output_frame->get_rows()[i], 
				data->get_data(), 
				file_offset,
				image_type,
				bpp,
				image_compression,
				bytes,
				width,
				fliphoriz,
				alphabits);
		}
	}
	else
	{
		for(int i = 0; i < height; i++)
		{
			read_line(output_frame->get_rows()[i], 
				data->get_data(), 
				file_offset,
				image_type,
				bpp,
				image_compression,
				bytes,
				width,
				fliphoriz,
				alphabits);
		}
	}

	if(output_frame != frame)
	{
		cmodel_transfer(frame->get_rows(), 
			output_frame->get_rows(),
			frame->get_y(),
			frame->get_u(),
			frame->get_v(),
			output_frame->get_y(),
			output_frame->get_u(),
			output_frame->get_v(),
			0, 
			0, 
			width, 
			height,
			0, 
			0, 
			frame->get_w(), 
			frame->get_h(),
			output_frame->get_color_model(), 
			frame->get_color_model(),
			0,
			width,
			frame->get_w());
	}
}

void FileTGAList::write_tga(Asset *asset, VFrame *frame, VFrame *data, VFrame* &temp)
{
	unsigned char header[18];
	unsigned char footer[26];
	long file_offset = 0;
	int out_bpp = 0;
	int rle = 0;
	int dest_cmodel = BC_RGB888;


	header[0] = 0;
	header[1] = 0;
	if(!strcasecmp(asset->compression, TGA_RGB_RLE))
	{
		header[2] = 10;
    	out_bpp = 3;
		rle = 1;
    	header[16] = 24; /* bpp */
    	header[17] = 0x20; /* alpha + orientation */
		dest_cmodel = BC_RGB888;
	}
	else
	if(!strcasecmp(asset->compression, TGA_RGBA_RLE))
	{
		header[2] = 10;
    	out_bpp = 4;
		rle = 1;
    	header[16] = 32; /* bpp */
    	header[17] = 0x28; /* alpha + orientation */
		dest_cmodel = BC_RGBA8888;
	}
	else
	if(!strcasecmp(asset->compression, TGA_RGB))
	{
		header[2] = 2;
    	out_bpp = 3;
		rle = 0;
    	header[16] = 24; /* bpp */
    	header[17] = 0x20; /* alpha + orientation */
		dest_cmodel = BC_RGB888;
	}
	else
	if(!strcasecmp(asset->compression, TGA_RGBA))
	{
		header[2] = 2;
    	out_bpp = 4;
		rle = 0;
    	header[16] = 32; /* bpp */
    	header[17] = 0x28; /* alpha + orientation */
		dest_cmodel = BC_RGBA8888;
	}
    header[3] = header[4] = header[5] = header[6] = header[7] = 0;

	VFrame *input_frame;
	if(frame->get_color_model() == dest_cmodel)
	{
		input_frame = frame;
	}
	else
	{
		if(temp && temp->get_color_model() != dest_cmodel)
		{
			delete temp;
			temp = 0;
		}
		
		if(!temp)
		{
			temp = new VFrame(0, frame->get_w(), frame->get_h(), dest_cmodel);
		}
		input_frame = temp;

		cmodel_transfer(input_frame->get_rows(), 
			frame->get_rows(),
			input_frame->get_y(),
			input_frame->get_u(),
			input_frame->get_v(),
			frame->get_y(),
			frame->get_u(),
			frame->get_v(),
			0, 
			0, 
			frame->get_w(), 
			frame->get_h(),
			0, 
			0, 
			frame->get_w(), 
			frame->get_h(),
			frame->get_color_model(), 
			input_frame->get_color_model(),
			0,
			frame->get_w(),
			frame->get_w());
	}

// xorigin
// yorigin
	header[8]  = header[9]  = 0;
	header[10] = header[11] = 0;

	header[12] = input_frame->get_w() % 256;
	header[13] = input_frame->get_w() / 256;

	header[14] = input_frame->get_h() % 256;
	header[15] = input_frame->get_h() / 256;
	
	write_data(header, data, file_offset, sizeof(header));

	unsigned char *output = new unsigned char[out_bpp * input_frame->get_w()];
	for(int i = 0; i < input_frame->get_h(); i++)
	{
		bgr2rgb(output, input_frame->get_rows()[i], input_frame->get_w(), out_bpp, (out_bpp == 4));
		
		if(rle)
		{
			rle_write(output, 
				input_frame->get_w(), 
				out_bpp,
				data,
				file_offset);
		}
		else
		{
			write_data(output, data, file_offset, input_frame->get_w() * out_bpp);
		}
	}
	delete [] output;
}

void FileTGAList::write_data(unsigned char *buffer, 
	VFrame *data, 
	long &file_offset,
	long len)
{
	if(data->get_compressed_allocated() <= data->get_compressed_size())
	{
		data->allocate_compressed_data(data->get_compressed_size() + 65536);
	}

	bcopy(buffer, data->get_data() + file_offset, len);
	file_offset += len;
	data->set_compressed_size(file_offset);
}

void FileTGAList::read_line(unsigned char *row,
	unsigned char *data,
	long &file_offset,
	int image_type,
	int bpp,
	int image_compression,
	int bytes,
	int width,
	int fliphoriz,
	int alphabits)
{
	if(image_compression == TGA_COMP_RLE)
	{
		rle_read(row,
			data,
			file_offset,
			bytes,
			width);
	}
	else
	{
		bcopy(data + file_offset, row, bytes * width);
		file_offset += bytes * width;
	}
	
	if(fliphoriz)
	{
		flip_line(row, bytes, width);
	}
	
	if(image_type == TGA_TYPE_COLOR)
	{
		if(bpp == 16)
		{
			upsample(row, row, width, bytes);
		}
		else
		{
			bgr2rgb(row, row, width, bytes, alphabits);
		}
	}
	else
	{
		;
	}
}

void FileTGAList::flip_line(unsigned char *row, int bytes, int width)
{
  	unsigned char temp;
	unsigned char *alt;
	int x, s;

	alt = row + (bytes * (width - 1));

	for (x = 0; x * 2 <= width; x++)
    {
    	for(s = 0; s < bytes; ++s)
		{
			temp = row[s];
			row[s] = alt[s];
			alt[s] = temp;
		}

    	row += bytes;
    	alt -= bytes;
    }
}

void FileTGAList::rle_read(unsigned char *row,
	unsigned char *data,
	long &file_offset,
	int bytes,
	int width)
{
	int repeat = 0;
	int direct = 0;
	unsigned char sample[4];
	int head;

	for(int x = 0; x < width; x++)
	{
		if(repeat == 0 && direct == 0)
		{
			head = data[file_offset++];
			if(head == EOF)
			{
				return;
			}
			else
			if(head >= 128)
			{
				repeat = head - 127;
				bcopy(data + file_offset, sample, bytes);
				file_offset += bytes;
			}
			else
			{
				direct = head + 1;
			}
		}
		
		if(repeat > 0)
		{
			for(int k = 0; k < bytes; k++)
			{
				row[k] = sample[k];
			}
			
			repeat--;
		}
		else
		{
			bcopy(data + file_offset, row, bytes);
			file_offset += bytes;
			
			direct--;
		}
		
		row += bytes;
	}
}

void FileTGAList::rle_write(unsigned char *buffer, 
	int width, 
	int bytes, 
	VFrame *frame, 
	long &file_offset)
{
	int repeat = 0;
  	int direct = 0;
  	unsigned char *from = buffer;
	unsigned char output;
  	int x;
	
	for(x = 1; x < width; ++x)
	{
/* next pixel is different */
		if(memcmp(buffer, buffer + bytes, bytes))
		{
			if(repeat)
			{
				output = 128 + repeat;
				write_data(&output, frame, file_offset, 1);
				write_data(from, frame, file_offset, bytes);
				from = buffer + bytes;
				repeat = 0;
				direct = 0;
			}
			else
			{
				direct++;
			}
		}
		else
/* next pixel is the same */
		{
			if(direct)
			{
				output = direct - 1;
				write_data(&output, frame, file_offset, 1);
				write_data(from, frame, file_offset, bytes * direct);
				from = buffer;
				direct = 0;
				repeat = 1;
			}
			else
			{
				repeat++;
			}
		}
		
		if(repeat == 128)
		{
			output = 255;
			write_data(&output, frame, file_offset, 1);
			write_data(from, frame, file_offset, bytes);
			from = buffer + bytes;
			direct = 0;
			repeat = 0;
		}
		else
		if(direct == 128)
		{
			output = 127;
			write_data(&output, frame, file_offset, 1);
			write_data(from, frame, file_offset, direct * bytes);
			from = buffer + bytes;
			direct = 0;
			repeat = 0;
		}
		
		buffer += bytes;
	}
	
	if(repeat > 0)
	{
		output = 128 + repeat;
		write_data(&output, frame, file_offset, 1);
		write_data(from, frame, file_offset, bytes);
	}
	else
	{
		output = direct;
		write_data(&output, frame, file_offset, 1);
		write_data(from, frame, file_offset, bytes * (direct + 1));
	}
}


void FileTGAList::bgr2rgb(unsigned char *dest,
	 unsigned char *src,
	 int width,
	 int bytes,
	 int alpha)
{
	int x;
	unsigned char r, g, b;

	if(alpha)
    {
    	for(x = 0; x < width; x++)
		{
			r = src[2];
			g = src[1];
			b = src[0];
			*(dest++) = r;
			*(dest++) = g;
			*(dest++) = b;
			*(dest++) = src[3];

			src += bytes;
		}
    }
	else
    {
    	for(x = 0; x < width; x++)
		{
			r = src[2];
			g = src[1];
			b = src[0];
			*(dest++) = r;
			*(dest++) = g;
			*(dest++) = b;

			src += bytes;
		}
    }
}

void FileTGAList::upsample(unsigned char *dest,
	  unsigned char *src,
	  int width,
	  int bytes)
{
	int x;

	dest += (width - 1) * 3;
	src += (width - 1) * bytes;
	for(x = width - 1; x >= 0; x--)
    {
    	dest[0] =  ((src[1] << 1) & 0xf8);
    	dest[0] += (dest[0] >> 5);

    	dest[1] =  ((src[0] & 0xe0) >> 2) + ((src[1] & 0x03) << 6);
    	dest[1] += (dest[1] >> 5);

    	dest[2] =  ((src[0] << 3) & 0xf8);
    	dest[2] += (dest[2] >> 5);

    	dest -= 3;
    	src -= bytes;
    }
}
















TGAConfigVideo::TGAConfigVideo(BC_WindowBase *gui, Asset *asset)
 : BC_Window(PROGRAM_NAME ": Video Compression",
 	gui->get_abs_cursor_x(),
 	gui->get_abs_cursor_y(),
	400,
	100)
{
	this->gui = gui;
	this->asset = asset;

	compression_items.append(new BC_ListBoxItem(FileTGAList::compression_to_str(TGA_RGB_RLE)));
	compression_items.append(new BC_ListBoxItem(FileTGAList::compression_to_str(TGA_RGBA_RLE)));
	compression_items.append(new BC_ListBoxItem(FileTGAList::compression_to_str(TGA_RGB)));
	compression_items.append(new BC_ListBoxItem(FileTGAList::compression_to_str(TGA_RGBA)));
}

TGAConfigVideo::~TGAConfigVideo()
{
	compression_items.remove_all_objects();
}

int TGAConfigVideo::create_objects()
{
	int x = 10, y = 10;

	TGACompression *textbox = new TGACompression(this, x, y, asset, &compression_items);
	textbox->create_objects();
	add_subwindow(new BC_OKButton(this));
	return 0;
}

int TGAConfigVideo::close_event()
{
	set_done(0);
	return 1;
}


TGACompression::TGACompression(TGAConfigVideo *gui,
	int x, 
	int y, 
	Asset *asset, 
	ArrayList<BC_ListBoxItem*> *compression_items)
 : BC_PopupTextBox(gui,
	compression_items,
 	FileTGAList::compression_to_str(gui->asset->compression),
	x, 
 	y, 
	100,
	1)
{
	this->asset = asset;
}
int TGACompression::handle_event()
{
	strcpy(asset->compression, FileTGAList::str_to_compression(get_text()));
	return 1;
}
