#include "filexml.h"
#include "keyframe.h"
#include "keyframes.h"

KeyFrames::KeyFrames()
 : List<KeyFrame>()
{
}

KeyFrames::~KeyFrames()
{
	while(last) delete last;
	delete default_keyframe;
}

void KeyFrames::create_objects()
{
	default_keyframe = new KeyFrame;
	default_keyframe->position = 0;
}

KeyFrame* KeyFrames::insert_keyframe(long position)
{
	for(KeyFrame *current = last; current; current = PREVIOUS)
	{
		if(current->position <= position)
		{
			if(current->position == position) return current;
			KeyFrame *new_keyframe = insert_after(current);
			new_keyframe->position = position;
			return new_keyframe;
		}
	}
	KeyFrame *new_keyframe = insert_after(0);
	new_keyframe->position = position;
	return new_keyframe;
}

void KeyFrames::copy(long start, long end, FileXML *file)
{
	default_keyframe->save(start, file);
	for(KeyFrame *current = first; current; current = NEXT)
	{
		if(current->position >= start && current->position < end)
		{
			current->save(start, file);
		}
	}
}

void KeyFrames::resample(double old_rate, double new_rate)
{
	for(KeyFrame *current = first; current; current = NEXT)
	{
		current->position = (long)((double)current->position * 
			new_rate / 
			old_rate + 
			0.5);
	}
}


	



void KeyFrames::copy_from(KeyFrames *keyframes)
{
	KeyFrame *this_keyframe, *that_keyframe, *next_keyframe;
//printf("Plugin::copy_keyframes 1\n");

	*default_keyframe = *keyframes->default_keyframe;

	for(this_keyframe = first, that_keyframe = keyframes->first; 
		that_keyframe;
		this_keyframe = this_keyframe->next, that_keyframe = that_keyframe->next)
	{
		if(!that_keyframe) break;
		if(!this_keyframe) append(this_keyframe = new KeyFrame);
		*this_keyframe = *that_keyframe;
//printf("Plugin::synchronize_params %s\n%s\n", this_keyframe->data, that_keyframe->data);
	}

//printf("Plugin::copy_keyframes 2\n");
// Delete excesss keyframes
	for(next_keyframe = this_keyframe ; next_keyframe; this_keyframe = next_keyframe)
	{
		next_keyframe = this_keyframe->next;
		delete this_keyframe;
	}
//printf("Plugin::copy_keyframes 3 %p %p\n", keyframes, keyframes->last);
}




void KeyFrames::optimize()
{
}
