#include "edl.h"
#include "edlsession.h"
#include "filexml.h"
#include "labels.h"
#include "mwindow.h"
#include "mwindowgui.h"
#include "patchbay.h"
#include "recordlabel.h"
#include "mainsession.h"
#include "stringfile.h"
#include "theme.h"
#include "timebar.h"
#include <string.h>



Labels::Labels(EDL *edl, char *xml_tag)
 : List<Label>()
{
	this->edl = edl;
	this->xml_tag = xml_tag;
}

Labels::~Labels()
{
	delete_all();
}

void Labels::dump()
{
	for(Label *current = first; current; current = NEXT)
	{
		printf("  label: %ld\n", current->position);
	}
}

int Labels::flip_vertical()
{
	Label *current;

	if(mwindow->gui)
	{	
		if(mwindow->session->tracks_vertical)
		{
			for(current = first; current; current = NEXT)
			{
				current->toggle->reposition_window(timebar->gui->get_x() + 2, current->toggle->get_y());
			}
		}
		else
		{
			for(current = first; current; current = NEXT)
			{
				current->toggle->reposition_window(current->toggle->get_x(), timebar->gui->get_y() + 2);
			}
		}
		samplemovement();
	}
    return 0;
}

void Labels::insert_labels(Labels *labels, double start)
{
	Label *new_label;
	Label *old_label;
	for(new_label = labels->first; new_label; new_label = new_label->next)
	{
		int exists = 0;
		for(old_label = first; old_label; old_label = old_label->next)
		{
			if(edl->equivalent(old_label->position, new_label->position))
			{
				exists = 1;
				break;
			}
			else
			if(old_label->position > new_label->position)
				break;
		}

		if(!exists)
		{
			if(old_label)
				insert_before(old_label, new Label(edl, this, new_label->position));
			else
				append(new Label(edl, this, new_label->position));
		}
	}
}

int Labels::toggle_label(double start, double end)
{
	Label *current;
printf("Labels::toggle_label 1 %f %f\n", start, end);

// handle selection start
// find label the selectionstart is after
	for(current = first; 
		current && current->position < start && !edl->equivalent(current->position, start); 
		current = NEXT)
	{
printf("Labels::toggle_label 2 %f %f %f\n", start, end, current->position);
		;
	}

	if(current)
	{
printf("Labels::toggle_label 3 %f %f %f\n", start, end, current->position);
		if(edl->equivalent(current->position, start))
		{        // remove it
printf("Labels::toggle_label 1\n");
			remove(current);
		}
		else
		{        // insert before it
			current = insert_before(current, new Label(edl, this, start));
		}
	}
	else
	{           // insert after last
		current = append(new Label(edl, this, start));
	}

// handle selection end
	if(start != end)
	{
printf("Labels::toggle_label 2\n");
// find label the selectionend is after
		for(current = first; 
			current && current->position < end && !edl->equivalent(current->position, end); 
			current = NEXT)
		{
			;
		}

		if(current)
		{
			if(edl->equivalent(current->position, end))
			{
				remove(current);
			}
			else
			{
				current = insert_before(current, new Label(edl, this, end));
			}
		}
		else
		{
			current = append(new Label(edl, this, end));
		}
	}
	return 0;
}

int Labels::draw()
{
	Label *current;
	
	if(mwindow->gui)
	{
		for(current = first; current; current = NEXT)
		{
			current->draw();
		}
	}
	return 0;
}

int Labels::samplemovement()
{
	return 0;
}

int Labels::resize_event(int x)
{
	Label* current;
	if(mwindow->gui && mwindow->session->tracks_vertical)
	{
		for(current = first; current; current = NEXT)
		{
			current->toggle->reposition_window(x, current->toggle->get_y());
		}
	}
	return 0;
}

int Labels::delete_all()
{
	while(last)
		remove(last);
	return 0;
}

int Labels::copy(double start, double end, FileXML *xml)
{
	char string[BCTEXTLEN];
	xml->tag.set_title(xml_tag);
	xml->append_tag();
	xml->append_newline();

	Label *current;
	sprintf(string, xml_tag);
	string[strlen(string) - 1] = 0;
	for(current = label_of(start); 
		current && current->position <= end; 
		current = NEXT)
	{
		xml->tag.set_title(string);
		xml->tag.set_property("SAMPLE", (long)current->position - start);
		xml->append_tag();
	}
	
	sprintf(string, "/%s", xml_tag);
	xml->tag.set_title(string);
	xml->append_tag();
	xml->append_newline();
	xml->append_newline();
	return 0;
}

int Labels::copy_length(long start, long end) // return number of Labels in selection
{
	int result = 0;
	Label *current;
	
	for(current = label_of(start); current && current->position <= end; current = NEXT)
	{
		result++;
	}
	return result;
}


// REMOVE
int Labels::paste(long start, long end, long total_length, FileXML *xml)
{
	return 0;
}

int Labels::paste_output(long startproject, long endproject, long startsource, long endsource, RecordLabels *labels)
{
	if(mwindow->session->labels_follow_edits) clear(startproject, endproject);
	if(mwindow->session->labels_follow_edits) insert(startproject, endsource - startsource);

	RecordLabel *reclabel;
	long position;
	Label *current;

	for(reclabel = labels->first; reclabel; reclabel = reclabel->next)
	{
		if(reclabel->position >= startsource)
		{
			position = reclabel->position - startsource;
			position += startproject;
			current = label_of(position);
			current = insert_before(current);
			current->create_objects(mwindow, this, position);
		}
	}
	optimize();
	return 0;
}

Labels& Labels::operator=(Labels &that)
{
	while(last) delete last;

	for(Label *current = that.first; current; current = NEXT)
	{
		append(new Label(edl, this, current->position));
	}

	return *this;
}


int Labels::save(FileXML *xml)
{
	xml->tag.set_title("LABELS");
	xml->append_tag();
	xml->append_newline();

	Label *current;

	for(current = first; current; current = NEXT)
	{
		xml->tag.set_title("LABEL");
		xml->tag.set_property("SAMPLE", (long)current->position);
		xml->append_tag();
	}
	
	xml->append_newline();
	xml->tag.set_title("/LABELS");
	xml->append_tag();
	xml->append_newline();
	xml->append_newline();
	return 0;
}

int Labels::load(FileXML *xml, unsigned long load_flags)
{
	int result = 0;
	char string1[BCTEXTLEN], string2[BCTEXTLEN];

	sprintf(string1, "/%s", xml_tag);
	strcpy(string2, xml_tag);
	string2[strlen(string2) - 1] = 0;

	do{
		result = xml->read_tag();
		if(!result)
		{
			if(xml->tag.title_is(string1))
			{
				result = 1;
			}
			else
			if(xml->tag.title_is(string2))
			{
				long position = xml->tag.get_property("SAMPLE", (long)-1);
				if(position > -1)
				{
					Label *current = label_of(position);
					current = insert_before(current, new Label(edl, this, position));
				}
			}
			else
			if(xml->tag.title_is("INPOINT"))
			{
				long position = xml->tag.get_property("SAMPLE", (long)-1);
				if(position > -1)
				{
					;
				}
			}
			else
			if(xml->tag.title_is("OUTPOINT"))
			{
				long position = xml->tag.get_property("SAMPLE", (long)-1);
				if(position > -1)
				{
					;
				}
			}
		}
	}while(!result);
	return 0;
}



int Labels::clear(double start, double end, int follow)
{
	Label *current;
	Label *next;

//printf("Labels::clear 1\n");
	current = label_of(start);
//printf("Labels::clear 2\n");
// remove selected labels
	while(current && current->position < end)
	{
		next = NEXT;
		delete current;              
		current = next;
	}
// Shift later labels
//printf("Labels::clear 3\n");
	if(follow)
	{
		while(current)
		{
			current->position -= end - start;   // shift labels forward
			current = NEXT;
		}
//printf("Labels::clear 4\n");
		optimize();
//printf("Labels::clear 5\n");
	}

	return 0;
}

int Labels::insert(double start, double length)
{      // shift every label including the first one back
	Label *current;

	for(current = label_of(start); current; current = NEXT)
	{
		current->position += length;
	}
	return 0;
}

int Labels::paste_silence(double start, double end)
{
	insert(start, end - start);
	optimize();
	return 0;
}

int Labels::modify_handles(double oldposition, 
	double newposition, 
	int currentend, 
	int handle_mode)
{
	if(edl->session->labels_follow_edits &&
		handle_mode == MOVE_ALL_EDITS)
	{
		if(currentend == 1)          // left handle
		{
			if(newposition < oldposition)
			{
				insert(oldposition, oldposition - newposition);    // shift all labels right
			}
			else
			{
				clear(oldposition, newposition);   // clear selection
			}
		}
		else
		{                            // right handle
			if(newposition < oldposition)
			{
				clear(newposition, oldposition);
			}
			else
			{
				insert(oldposition, newposition - oldposition);
			}
		}
	}
	return 0;
}

int Labels::optimize()
{
	int result = 1;
	Label *current;

	while(result)
	{
		result = 0;
		
		for(current = first; current && NEXT && !result;)
		{
			Label *next = NEXT;
			if(current->position == next->position)
			{
				delete current;
				result  = 1;
			}
			current = next;
		}
	}
	return 0;
}

Label* Labels::label_of(double position)
{
	Label *current;

	for(current = first; current; current = NEXT)
	{
		if(current->position >= position) return current;
	}
	return 0;
}










Label::Label(EDL *edl, Labels *labels, double position)
{
	this->edl = edl;
	this->labels = labels;
	this->position = position;
}


Label::Label() : ListItem<Label>()
{
	toggle = 0;
}

Label::~Label()
{
//	if(toggle) delete toggle;
}

int Label::create_objects(MWindow *mwindow, Labels *labels, long position)
{
	return 0;
}

int Label::samplemovement()
{
	return 0;
}

int Label::draw()
{
	return 0;
}

int Label::get_pixel(long position)
{
	return 0;
}

LabelToggle::LabelToggle(MWindow *mwindow, 
	Label *label, 
	int x, 
	int y, 
	long position)
 : BC_Label(x, y, 0)
{
	this->mwindow = mwindow;
	this->label = label;
}

LabelToggle::~LabelToggle() { }

int LabelToggle::handle_event()
{
	return 0;
}

