#include "edl.h"
#include "edlsession.h"
#include "levelwindow.h"
#include "levelwindowgui.h"
#include "mwindow.h"
#include "mwindowgui.h"
#include "preferences.h"
#include "theme.h"

LevelWindowGUI::LevelWindowGUI(MWindow *mwindow, LevelWindow *thread, int w, int h)
 : BC_Window(PROGRAM_NAME ": Levels", 
 	(int)INFINITY, 
	(int)INFINITY, 
	w, 
	h, 
	-1,
	-1,
	1,
	0, 
	1)
{
	METER_H = 205;
	this->thread = thread;
	this->mwindow = mwindow;
	total_meters = 0;
	db_scale = 0;
}

LevelWindowGUI::~LevelWindowGUI()
{
	for(int i = 0; i < total_meters; i++) delete meters[i];
//	delete_db_scale();
}

int LevelWindowGUI::create_objects()
{
	add_subwindow(reset_button = new LevelWindowReset(mwindow, this, 5, get_h() - 30));
}


int LevelWindowGUI::resize_event(int w, int h)
{
//printf("LevelWindowGUI::resize_event 1 %d %d\n", w, h);
	if(thread->horizontal)
	{
		if(w < METER_H - METER_H / 3)
		{
// change to vertical
			thread->horizontal = 0;
			resize_window(w, 260);
			int new_meters = total_meters;
			for(int i = 0; i < new_meters; i++) delete_meter();
			for(int i = 0; i < new_meters; i++) new_meter();
		}
		else
		if(w != 230) resize_window(230, h);
	}
	else
	{
		if(h < METER_H - METER_H / 3)
		{
// change to horizontal
			thread->horizontal = 1;
			resize_window(230, h);
			int new_meters = total_meters;
			for(int i = 0; i < new_meters; i++) delete_meter();
			for(int i = 0; i < new_meters; i++) new_meter();
		}
		else
		if(h != 260) resize_window(w, 260);
	}
	
	reset_button->reposition_window(5, this->get_h() - 30);
//printf("LevelWindowGUI::resize_event 2 %d %d\n", w, h);
}

int LevelWindowGUI::new_meter()
{
	char string[16];
	int format = thread->mwindow->edl->session->meter_format;
	int x, y, w, h, title_x, title_y;
	
	if(format == METER_DB)
	x = y = total_meters * 30 + 5 + 25;
	else
	x = y = total_meters * 30 + 5;

	if(thread->horizontal)
	{
		x = 20;
		if(format == METER_DB) y -= 5;
		title_x = 5;
		title_y = y;
		w = METER_H;
		h = 25;
	}
	else
	{
		y = 20;
		title_x = x;
		title_y = 2;
		w = 25;
		h = METER_H;
	}

	sprintf(string, "%d", total_meters + 1);
	add_subwindow(meters[total_meters] = new BC_Meter(x, y, w, h, thread->mwindow->edl->session->min_meter_db, thread->mwindow->edl->session->meter_format));
	add_subwindow(titles[total_meters] = new BC_Title(title_x, title_y, string));
	total_meters++;

	if(total_meters == 1 && format == METER_DB)
	{
//		draw_db_scale();
	}
	return 0;
}

int LevelWindowGUI::delete_meter()
{
	total_meters--;
	delete meters[total_meters];
	delete titles[total_meters];
	if(total_meters == 0)
	{
//		delete_db_scale();
	}
	return 0;
}

int LevelWindowGUI::close_event()
{
	thread->hide_window();
	return 1;
}

int LevelWindowGUI::keypress_event()
{
	if(get_keypress() == 'w')
	{
		thread->hide_window();
		return 1;
	}
	return 0;
}


int LevelWindowGUI::reset_over()
{
	for(int i = 0; i < total_meters; i++)
	{
		meters[i]->reset_over();
	}
	return 0;
}


LevelWindowReset::LevelWindowReset(MWindow *mwindow, LevelWindowGUI *gui, int x, int y)
 : BC_Button(x, y, mwindow->theme->over_button)
{
	this->gui = gui;
}

LevelWindowReset::~LevelWindowReset()
{
}

int LevelWindowReset::handle_event()
{
	gui->thread->mwindow->reset_meters();
	return 1;
}
