#include "loadbalance.h"




LoadPackage::LoadPackage()
{
	completion_lock.lock();
}
LoadPackage::~LoadPackage()
{
}









LoadClient::LoadClient(LoadServer *server)
 : Thread()
{
	Thread::set_synchronous(1);
	this->server = server;
	done = 0;
	input_lock.lock();
	completion_lock.lock();
}

LoadClient::~LoadClient()
{
	done = 1;
	input_lock.unlock();
	Thread::join();
}

void LoadClient::run()
{
	while(!done)
	{
		input_lock.lock();

		if(!done)
		{
// Read packet
			LoadPackage *package;
			
			
			server->client_lock.lock();
			if(server->current_package < server->total_packages)
			{
				package = server->packages[server->current_package++];
				server->client_lock.unlock();
				input_lock.unlock();

				process_package(package);

				package->completion_lock.unlock();
			}
			else
			{
				completion_lock.unlock();
				server->client_lock.unlock();
			}
		}
	}
}








LoadServer::LoadServer(int total_clients, int total_packages)
{
	this->total_clients = total_clients;
	this->total_packages = total_packages;
	current_package = 0;
	clients = 0;
	packages = 0;
}
LoadServer::~LoadServer()
{
	if(clients) 
	{
		for(int i = 0; i < total_clients; i++)
			delete clients[i];
		delete [] clients;
	}
	
	if(packages)
	{
		for(int i = 0; i < total_packages; i++)
			delete packages[i];
		delete [] packages;
	}
}

void LoadServer::create_clients()
{
	clients = new LoadClient*[total_clients];
	for(int i = 0; i < total_clients; i++)
	{
		clients[i] = new_client();
		clients[i]->start();
	}
}

void LoadServer::create_packages()
{
	packages = new LoadPackage*[total_packages];
	for(int i = 0; i < total_packages; i++)
		packages[i] = new_package();
}


void LoadServer::process_packages()
{
	if(!clients) create_clients();
	if(!packages) create_packages();
	
	
	
// Set up packages
	init_packages();

	current_package = 0;
// Start all clients
	for(int i = 0; i < total_clients; i++)
	{
		clients[i]->input_lock.unlock();
	}
	
// Wait for packages to get finished
	for(int i = 0; i < total_packages; i++)
	{
		packages[i]->completion_lock.lock();
	}

// Wait for clients to finish before allowing changes to packages
	for(int i = 0; i < total_clients; i++)
	{
		clients[i]->completion_lock.lock();
	}
}

