#ifndef LOADBALANCE_H
#define LOADBALANCE_H

#include "mutex.h"
#include "thread.h"




// Load balancing utils



class LoadServer;



class LoadPackage
{
public:
	LoadPackage();
	virtual ~LoadPackage();
	
	Mutex completion_lock;
};


class LoadClient : public Thread
{
public:
	LoadClient(LoadServer *server);
	virtual ~LoadClient();
	
	void run();
	virtual void process_package(LoadPackage *package) {};
	
	int done;
	Mutex input_lock, completion_lock;
	LoadServer *server;
};




class LoadServer
{
public:
	LoadServer(int total_clients, int total_packages);
	virtual ~LoadServer();

	friend class LoadClient;

	virtual void init_packages() {};
	virtual LoadClient* new_client() { return 0; };
	virtual LoadPackage* new_package() { return 0; };
	void process_packages();



	void create_clients();
	void create_packages();





	int current_package;
	LoadPackage **packages;
	int total_packages;
	LoadClient **clients;
	int total_clients;
	Mutex client_lock;
};



#endif


