#include "defaults.h"
#include "edl.h"
#include "filexml.h"
#include "localsession.h"





LocalSession::LocalSession(EDL *edl)
{
	this->edl = edl;

	selectionstart = selectionend = 0;
	in_point = out_point = -1;
	strcpy(folder, CLIP_FOLDER);
	sprintf(clip_title, "Program");
	strcpy(clip_notes, "Hello world");
	clipboard_length = 0;
}

LocalSession::~LocalSession()
{
}

void LocalSession::copy_from(LocalSession *that)
{
	selectionend = that->selectionend;
	selectionstart = that->selectionstart;
	in_point = that->in_point;
	out_point = that->out_point;
	strcpy(folder, that->folder);
	strcpy(clip_title, that->clip_title);
	strcpy(clip_notes, that->clip_notes);
}

void LocalSession::save_xml(FileXML *file)
{
	file->tag.set_title("LOCALSESSION");

	file->tag.set_property("IN_POINT", in_point);
	file->tag.set_property("OUT_POINT", out_point);
	file->tag.set_property("SELECTION_START", selectionstart);
	file->tag.set_property("SELECTION_END", selectionend);
	file->tag.set_property("CLIP_TITLE", clip_title);
	file->tag.set_property("CLIP_NOTES", clip_notes);
	file->tag.set_property("FOLDER", folder);
	file->append_tag();
	file->append_newline();
	file->append_newline();
}

void LocalSession::load_xml(FileXML *file)
{
	clipboard_length = 0;
	in_point = file->tag.get_property("IN_POINT", (long)-1);
	out_point = file->tag.get_property("OUT_POINT", (long)-1);
	selectionstart = file->tag.get_property("SELECTION_START", 0);
	selectionend = file->tag.get_property("SELECTION_END", 0);

// Overwritten by MWindow::load_filenames	
	file->tag.get_property("CLIP_TITLE", clip_title);
	file->tag.get_property("CLIP_NOTES", clip_notes);
	
	file->tag.get_property("FOLDER", folder);
}

void LocalSession::boundaries()
{
}

int LocalSession::load_defaults(Defaults *defaults)
{
	selectionstart = defaults->get("SELECTIONSTART", selectionstart);
	selectionend = defaults->get("SELECTIONEND", selectionend);
	return 0;
}

int LocalSession::save_defaults(Defaults *defaults)
{
	defaults->update("SELECTIONSTART", selectionstart);
	defaults->update("SELECTIONEND", selectionend);
	return 0;
}



