#include "assets.h"
#include "defaults.h"
#include "edl.h"
#include "filesystem.h"
#include "indexfile.h"
#include "loadfile.h"
#include "guicast.h"
#include "mainindexes.h"
#include "mainprogress.h"
#include "mwindow.h"
#include "mwindowgui.h"

#include <string.h>

MainIndexes::MainIndexes(MWindow *mwindow)
 : Thread()
{
	this->mwindow = mwindow;
	set_synchronous(1);
	interrupt_flag = 0;
	indexfile = new IndexFile(mwindow);
}

MainIndexes::~MainIndexes()
{
	delete indexfile;
}

void MainIndexes::add_next_asset(Asset *asset)
{
	next_lock.lock();

// Test current asset
//printf("MainIndexes::add_next_asset 1\n");
	IndexFile indexfile(mwindow);
//printf("MainIndexes::add_next_asset 1\n");
	if(!indexfile.open_index(asset))
	{
		asset->index_status = INDEX_READY;
//printf("MainIndexes::add_next_asset 1\n");
		indexfile.close_index();
//printf("MainIndexes::add_next_asset 1\n");
	}
	else
	{
//printf("MainIndexes::add_next_asset 1\n");
		Asset *new_asset = new Asset;
//printf("MainIndexes::add_next_asset 1\n");
		*new_asset = *asset;
//printf("MainIndexes::add_next_asset 1\n");
		next_assets.append(new_asset);
//printf("MainIndexes::add_next_asset 1\n");
	}
//printf("MainIndexes::add_next_asset 1\n");
	next_lock.unlock();
//printf("MainIndexes::add_next_asset 1\n");
}

void MainIndexes::delete_current_assets()
{
	current_assets.remove_all_objects();
}

void MainIndexes::start_loop()
{
	input_lock.lock();
	interrupt_flag = 0;
	start();
}

void MainIndexes::stop_loop()
{
	interrupt_flag = 1;
	input_lock.unlock();
	Thread::join();
}


void MainIndexes::start_build()
{
	interrupt_flag = 0;
	interrupt_lock.lock();
	input_lock.unlock();
}

void MainIndexes::interrupt_build()
{
//printf("MainIndexes::interrupt_build 1\n");
	interrupt_flag = 1;
	indexfile->interrupt_index();
//printf("MainIndexes::interrupt_build 2\n");
	interrupt_lock.lock();
//printf("MainIndexes::interrupt_build 3\n");
	interrupt_lock.unlock();
//printf("MainIndexes::interrupt_build 4\n");
}

void MainIndexes::load_next_assets()
{
	delete_current_assets();

// Transfer from new list
	next_lock.lock();
	for(int i = 0; i < next_assets.total; i++)
		current_assets.append(next_assets.values[i]);

// Clear pointers from new list only
	next_assets.remove_all();
	next_lock.unlock();
}


void MainIndexes::run()
{
	while(!interrupt_flag)
	{
// Wait for new assets to be released
		input_lock.lock();
		if(interrupt_flag) return;

//printf("MainIndexes::run 1\n");
		load_next_assets();

// test index of each asset
		MainProgressBar *progress = 0;
		for(int i = 0; i < current_assets.total && !interrupt_flag; i++)
		{
			Asset *current_asset = current_assets.values[i];
//printf("MainIndexes::run 2 %s %d %d\n", current_asset->path, current_asset->index_status, current_asset->audio_data);

			if(current_asset->index_status == INDEX_NOTTESTED && 
				current_asset->audio_data)
			{
//printf("MainIndexes::run 3\n");
				if(indexfile->open_index(current_asset))
				{
//printf("MainIndexes::run 4 %p %s %p %p\n", current_asset, current_asset->path, mwindow, mwindow->mainprogress);
// doesn't exist
// try to create index now
					if(!progress)
					{
						progress = mwindow->mainprogress->start_progress("Building Indexes...", 1);
					}

//printf("MainIndexes::run 5 %p %s\n", current_asset, current_asset->path);

					indexfile->create_index(current_asset, progress);
//printf("MainIndexes::run 6 %p %s\n", current_asset, current_asset->path);
					if(progress->is_cancelled()) interrupt_flag = 1;
//printf("MainIndexes::run 7 %p %s\n", current_asset, current_asset->path);
				}
				else
// Exists
				{
//printf("MainIndexes::run 8\n");
					if(current_asset->index_status == INDEX_NOTTESTED) 
						current_asset->index_status = INDEX_READY;
					indexfile->close_index();
				}
//printf("MainIndexes::run 8\n");
			}
//printf("MainIndexes::run 9\n");
		}
//printf("MainIndexes::run 10\n");

		if(progress)     // progress box is only created when an index is built
		{
			progress->stop_progress();
			delete progress;
			progress = 0;
		}
//printf("MainIndexes::run 11\n");

		interrupt_lock.unlock();
	}
}

