#include "bcdisplayinfo.h"
#include "clip.h"
#include "defaults.h"
#include "edl.inc"
#include "guicast.h"
#include "mainsession.h"


MainSession::MainSession()
{
	changes_made = 0;
	view_start = 0;
	track_start = 0;
	filename[0] = 0;
//	playback_cursor_visible = 0;
//	is_playing_back = 0;
	track_highlighted = 0;
	vcanvas_highlighted = 0;
	edit_highlighted = 0;
	current_operation = NO_OPERATION;
	drag_pluginserver = 0;
	drag_plugin = 0;
	drag_asset = 0;
	drag_edit = 0;
}

MainSession::~MainSession()
{
}

void MainSession::boundaries()
{
	mwindow_x = MAX(0, mwindow_x);
	mwindow_y = MAX(0, mwindow_y);
	cwindow_x = MAX(0, cwindow_x);
	cwindow_y = MAX(0, cwindow_y);
	vwindow_x = MAX(0, vwindow_x);
	vwindow_y = MAX(0, vwindow_y);
	awindow_x = MAX(0, awindow_x);
	awindow_y = MAX(0, awindow_y);
	rwindow_x = MAX(0, rwindow_x);
	rwindow_y = MAX(0, rwindow_y);
	rmonitor_x = MAX(0, rmonitor_x);
	rmonitor_y = MAX(0, rmonitor_y);
}

void MainSession::default_window_positions()
{
// Get defaults based on root window size
	BC_DisplayInfo display_info;
	int root_h = display_info.get_root_h();
	int root_w = display_info.get_root_w();

// Wider than 16:9, narrower than dual head
	if((float)root_w / root_h > 1.8) root_w /= 2;

// Make room for window manager garbage
	root_w -= 10;
	root_h -= 30;

	vwindow_x = 0;
	vwindow_y = 0;
	vwindow_h = root_h / 2;
	vwindow_w = root_w / 2 - 5;

	cwindow_x = vwindow_x + vwindow_w + 10;
	cwindow_y = 0;
	cwindow_w = root_w - cwindow_x;
	cwindow_h = vwindow_h;

	ctool_x = cwindow_x + cwindow_w / 2;
	ctool_y = cwindow_y + cwindow_h / 2;

	mwindow_x = 0;
	mwindow_y = root_h / 2 + 30;
	mwindow_w = (int)(root_w * 0.60 - 5);
	mwindow_h = root_h - mwindow_y;

	awindow_x = mwindow_x + mwindow_w + 10;
	awindow_y = mwindow_y;
	awindow_w = root_w - awindow_x;
	awindow_h = mwindow_h;

	rwindow_x = 0;
	rwindow_y = 0;
	rwindow_h = 500;
	rwindow_w = 650;

	rmonitor_x = rwindow_x + rwindow_w + 10;
	rmonitor_y = rwindow_y;
	rmonitor_w = root_w - rmonitor_x;
	rmonitor_h = rwindow_h;
}

int MainSession::load_defaults(Defaults *defaults)
{
	char string[BCTEXTLEN];

// REMOVE
	sample_rate = defaults->get("SAMPLERATE", 48000);
	audio_channels = defaults->get("ACHANNELS", 2);
// Default channel positions
	for(int i = 0; i < audio_channels; i++)
	{
		sprintf(string, "ACHANNEL_ANGLE_%d", i);
		int default_position = i * 30;

		if(i == 0) default_position = 270;
		else
		if(i == 1) default_position = 90;
		else
		if(default_position == 90) default_position = 300;
		
		achannel_positions[i] = defaults->get(string, default_position);
	}

// REMOVE
// Video session
	track_w = defaults->get("TRACKW", 720);
	track_h = defaults->get("TRACKH", 480);
	output_w = defaults->get("OUTPUTW", 720);
	output_h = defaults->get("OUTPUTH", 480);
	aspect_w = defaults->get("ASPECTW", (float)4);
	aspect_h = defaults->get("ASPECTH", (float)3);

	cwindow_zoom = defaults->get("CWINDOW_ZOOM", (float)1);
	vwindow_zoom = defaults->get("VWINDOW_ZOOM", (float)1);
	frame_rate = defaults->get("FRAMERATE", (float)10);
	labels_follow_edits = defaults->get("LABELSFOLLOWEDITS", 0);
	autos_follow_edits = defaults->get("AUTOSFOLLOWEDITS", 1);

	tracks_vertical = defaults->get("TRACKSVERTICAL", 0);
	zoom_sample = defaults->get("ZOOMSAMPLE", 1);
	zoom_y = defaults->get("ZOOMY", 64);
	zoom_track = defaults->get("ZOOMTRACK", 64);

// Editing mode
	editing_mode = defaults->get("EDITING_MODE", EDITING_IBEAM);

// Setup main windows
	default_window_positions();
	vwindow_x = defaults->get("VWINDOW_X", vwindow_x);
	vwindow_y = defaults->get("VWINDOW_Y", vwindow_y);
	vwindow_w = defaults->get("VWINDOW_W", vwindow_w);
	vwindow_h = defaults->get("VWINDOW_H", vwindow_h);
	cwindow_x = defaults->get("CWINDOW_X", cwindow_x);
	cwindow_y = defaults->get("CWINDOW_Y", cwindow_y);
	cwindow_w = defaults->get("CWINDOW_W", cwindow_w);
	cwindow_h = defaults->get("CWINDOW_H", cwindow_h);
	ctool_x = defaults->get("CTOOL_X", ctool_x);
	ctool_y = defaults->get("CTOOL_Y", ctool_y);
	mwindow_x = defaults->get("MWINDOW_X", mwindow_x);
	mwindow_y = defaults->get("MWINDOW_Y", mwindow_y);
	mwindow_w = defaults->get("MWINDOW_W", mwindow_w);
	mwindow_h = defaults->get("MWINDOW_H", mwindow_h);
	awindow_x = defaults->get("AWINDOW_X", awindow_x);
	awindow_y = defaults->get("AWINDOW_Y", awindow_y);
	awindow_w = defaults->get("AWINDOW_W", awindow_w);
	awindow_h = defaults->get("AWINDOW_H", awindow_h);

//printf("MainSession::load_defaults 1\n");

// Other windows
	afolders_w = defaults->get("ABINS_W", 100);
	rwindow_x = defaults->get("RWINDOW_X", rwindow_x);
	rwindow_y = defaults->get("RWINDOW_Y", rwindow_y);
	rwindow_w = defaults->get("RWINDOW_W", rwindow_w);
	rwindow_h = defaults->get("RWINDOW_H", rwindow_h);
	rmonitor_x = defaults->get("RMONITOR_X", rmonitor_x);
	rmonitor_y = defaults->get("RMONITOR_Y", rmonitor_y);
	rmonitor_w = defaults->get("RMONITOR_W", rmonitor_w);
	rmonitor_h = defaults->get("RMONITOR_H", rmonitor_h);

	show_vwindow = defaults->get("SHOW_VWINDOW", 1);
	show_awindow = defaults->get("SHOW_AWINDOW", 1);
	show_cwindow = defaults->get("SHOW_CWINDOW", 1);
	boundaries();
	return 0;
}

int MainSession::save_defaults(Defaults *defaults)
{
	defaults->update("EDITING_MODE", editing_mode);
	defaults->update("CWINDOW_ZOOM", cwindow_zoom);
	defaults->update("VWINDOW_ZOOM", vwindow_zoom);
	defaults->update("TRACKSVERTICAL", tracks_vertical);
	defaults->update("LABELSFOLLOWEDITS", labels_follow_edits);
	defaults->update("AUTOSFOLLOWEDITS", autos_follow_edits);
	defaults->update("ZOOMSAMPLE", zoom_sample);
	defaults->update("ZOOMY", zoom_y);
	defaults->update("ZOOMTRACK", zoom_track);

//printf("MainSession::save_defaults 1\n");
// Window positions
	defaults->update("MWINDOW_X", mwindow_x);
	defaults->update("MWINDOW_Y", mwindow_y);
	defaults->update("MWINDOW_W", mwindow_w);
	defaults->update("MWINDOW_H", mwindow_h);
	defaults->update("VWINDOW_X", vwindow_x);
	defaults->update("VWINDOW_Y", vwindow_y);
	defaults->update("VWINDOW_W", vwindow_w);
	defaults->update("VWINDOW_H", vwindow_h);
	defaults->update("CWINDOW_X", cwindow_x);
	defaults->update("CWINDOW_Y", cwindow_y);
	defaults->update("CWINDOW_W", cwindow_w);
	defaults->update("CWINDOW_H", cwindow_h);
	defaults->update("CTOOL_X", ctool_x);
	defaults->update("CTOOL_Y", ctool_y);
	defaults->update("AWINDOW_X", awindow_x);
	defaults->update("AWINDOW_Y", awindow_y);
	defaults->update("AWINDOW_W", awindow_w);
	defaults->update("AWINDOW_H", awindow_h);

 	defaults->update("ABINS_W", afolders_w);
	defaults->update("RMONITOR_X", rmonitor_x);
	defaults->update("RMONITOR_Y", rmonitor_y);
	defaults->update("RMONITOR_W", rmonitor_w);
	defaults->update("RMONITOR_H", rmonitor_h);
	defaults->update("RWINDOW_X", rwindow_x);
	defaults->update("RWINDOW_Y", rwindow_y);
	defaults->update("RWINDOW_W", rwindow_w);
	defaults->update("RWINDOW_H", rwindow_h);

	defaults->update("SHOW_VWINDOW", show_vwindow);
	defaults->update("SHOW_AWINDOW", show_awindow);
	defaults->update("SHOW_CWINDOW", show_cwindow);
	return 0;
}
