#ifndef MWINDOW_H
#define MWINDOW_H

#include "assets.inc"
#include "audiodevice.inc"
#include "awindow.inc"
#include "guicast.h"
#include "cache.inc"
#include "channel.inc"
#include "console.inc"
#include "cwindow.inc"
#include "defaults.inc"
#include "defaulttheme.inc"
#include "edl.inc"
#include "filexml.inc"
#include "levelwindow.inc"
#include "loadmode.inc"
#include "mainindexes.inc"
#include "mainprogress.inc"
#include "mainundo.inc"
#include "mwindow.inc"
#include "mwindowgui.inc"
#include "maxchannels.h"
#include "new.inc"
#include "patchbay.inc"
#include "tracking.inc"
#include "playbackengine.inc"
#include "plugin.inc"
#include "pluginserver.inc"
#include "preferences.inc"
#include "preferencesthread.inc"
#include "recordlabel.inc"
#include "render.inc"
#include "mainsession.inc"
#include "sharedlocation.inc"
#include "theme.inc"
#include "threadloader.inc"
#include "timebar.inc"
#include "timebomb.h"
#include "track.inc"
#include "tracks.inc"
#include "transition.inc"
#include "transportque.inc"
#include "videowindow.inc"
#include "vwindow.inc"


// All entry points for commands except for window locking should be here.
// This allows scriptability.

class MWindow
{
public:
	MWindow();
	~MWindow();

// ======================================== initialization commands
	void create_objects(int want_gui, int want_new);
	static void init_tuner(ArrayList<Channel*> &channeldb, char *path);

	int run_script(FileXML *script);
	int new_project();
	int delete_project(int flash = 1);

	int load_defaults();
	int save_defaults();
	int set_filename(char *filename);
// Total vertical pixels in timeline
	int get_tracks_height();
// Total horizontal pixels in timeline
	int get_tracks_width();
// Show windows
	void show_vwindow();
	void show_awindow();
	void show_cwindow();
	void tile_windows();
	void set_titles(int value);
	int asset_to_edl(EDL *new_edl, Asset *new_asset);
	int paste_edls(ArrayList<EDL*> *new_edls, int load_mode);
// Reset everything for a load
	void update_project();
	void fit_selection();
// move the window to include the cursor
	void find_cursor();
// Append a plugindb with pointers to the master plugindb
	void create_plugindb(int do_audio, 
		int do_video, 
		int is_realtime, 
		int is_transition,
		int is_theme,
		ArrayList<PluginServer*> &plugindb);
// Find the plugin whose title matches title and return it
	PluginServer* scan_plugindb(char *title);
	void dump_plugins();

// ========================================= synchronization

	int resize_lock;            // prevent tracks from redrawing on resize
	int lock_resize();
	int unlock_resize();


// ========================================= file operations

	
	int load_filenames(ArrayList<char*> *filenames, 
		int load_mode = LOAD_REPLACE);
	int load(FileXML *xml, int import_ = 0,       // load from a xml file
		int edits_only = 0,
		int patches_only = 0,
		int console_only = 0,
		int timebar_only = 0,
		int automation_only = 0);
	int load_video_config(FileXML *xml, int import_);
	int load_audio_config(FileXML *xml, int import_,
		int edits_only, 
		int patches_only,
		int console_only,
		int automation_only);

	int load_edits(FileXML *xml);
	int load_patches(FileXML *xml);
	int load_console(FileXML *xml);
	int load_timebar(FileXML *xml);
	int load_automation(FileXML *xml);
	
	int interrupt_indexes();  // Stop index building

// ========================================== drawing

	int draw();          // draw everything

	int redraw_time_dependancies();     // after reconfiguring the time format, sample rate, frame rate

	int flip_vertical(int new_orientation);
// =========================================== movement

	int reposition_timebar(int new_pixel, int new_height);
	int expand_sample();
	int zoom_in_sample();
	int zoom_sample(long zoom_sample);
	void zoom_amp(long zoom_amp);
	void zoom_track(long zoom_track);
	int fit_sample();
	int move_left(double distance = 0);
	int move_right(double distance = 0);
	void move_up(long distance = 0);
	void move_down(long distance = 0);
	int next_label();   // seek to labels
	int prev_label();
	void trackmovement(int track_start);
	int samplemovement(double distance);
	void select_all();
	int goto_start();
	int goto_end();
	int expand_y();
	int zoom_in_y();
	int expand_t();
	int zoom_in_t();
	int change_channels(int old_channels, int new_channels);
	void crop_video();
	void update_plugins();
	void show_plugin(Plugin *plugin);
	void hide_plugin(Plugin *plugin);


// ============================= editing commands ========================

	void delete_folder(char *folder);
	void new_folder(char *new_folder);
	void set_auto_keyframes(int value);
// Update the editing mode
	int set_editing_mode(int new_editing_mode);
	void set_inpoint(double position);
	void set_outpoint(double position);
	void delete_inpoint();
	void delete_outpoint();    
	void extract_selection();
	void lift_selection();
	void render_single();
	void render_list();
	void rebuild_index(Asset *asset);
	int cut(double start, double end);
	int copy(double start, double end);
	int paste(double start, double end, FileXML *file);
	void insert_effect(char *title, 
		SharedLocation *shared_location, 
		Track *track,
		int plugin_type);
	void insert(double position, FileXML *file);

	int paste_output(long startproject, 
				long endproject, 
				long startsource_sample, 
				long endsource_sample, 
				long startsource_frame,
				long endsource_frame,
				Asset *asset, 
				RecordLabels *new_labels);

	int paste_transition(long startproject, 
				long endproject, 
				Transition *transition);
	int paste_silence(double start, double end, int clear_labels = 1);
	int clear(double start, double end, int clear_labels = 1);
	int clear_labels(double start, double end);
	int feather_edits(long feather_samples, int audio, int video);
	long get_feather(int audio, int video);
	float get_aspect_ratio();
	static int create_aspect_ratio(float &w, float &h, int width, int height);
	int mute_selection(double start, double end);
	int trim_selection(double start, double end);
// Synchronize EDL settings with all playback engines depending on current 
// operation
	void sync_parameters(int change_type = CHANGE_PARAMS);
	void add_clip_to_edl(EDL *edl);

	int cut_automation();
	int copy_automation();
	int paste_automation();
	int clear_automation();

	int modify_handles(double oldposition, 
		double newposition, 
		int currentend, 
		int handle_mode);

	int add_audio_track();
	int add_video_track();
	void delete_track();
	void delete_tracks();
	int toggle_label();
	void move_tracks_up();
	void move_tracks_down();
	void concatenate_tracks();
	
	
	
	
	
	

// Send new EDL to caches
	void update_caches();
	int purge_asset(char *path);       // remove all traces of an asset from the project and cache
	int optimize_assets();            // delete unused assets from the cache and assets

// ================================= cursor selection ======================

// start a cursor selection
	int init_selection(long cursor_position, 
					int cursor_x, 
					int cursor_y, 
					int &current_end, 
					long &selection_midpoint1,
					long &selection_midpoint2,
					int &selection_type);
	int update_selection(long cursor_position,
					int cursor_x, 
					int cursor_y, 
					int &current_end, 
					long selection_midpoint1,
					long selection_midpoint2,
					int selection_type);
	int end_selection();
	void select_point(double position);
	int set_loop_boundaries();         // toggle loop playback and set boundaries for loop playback

// ================================ handle selection =======================

	int init_handle_selection(long cursor_position, int handle_pixel, int which_handle);     // handle selection
	int draw_floating_handle(int flash);          


	LevelWindow *level_window;
	Tracks *tracks;
	PatchBay *patches;
	MainUndo *undo;
	TimeBar *timebar;
	Defaults *defaults;
	Assets *assets;
// Caches for drawing timeline only
	Cache *audio_cache, *video_cache;
	ThreadLoader *threadloader;
	VideoWindow *video_window;
	Preferences *preferences;
	PreferencesThread *preferences_thread;
	MainSession *session;
	Theme *theme;
	DefaultTheme *default_theme;
	MainIndexes *mainindexes;
	MainProgress *mainprogress;

// Menu items
	ArrayList<ColormodelItem*> colormodels;

	int update_playback_cursor(long new_position, int view_follows_playback);
	int hide_playback_cursor(int flash = 1);
	int show_playback_cursor(long position  = -1, int flash = 1);
	int set_playback_range(long start_position = -1, int reverse = 0, float speed = 1);      // set the playback range for starting up
	int reset_meters();

// need playbutton to update when playback finishes
	int arm_playback(int follow_loop, 
					int use_buttons, 
					int infinite, 
					AudioDevice *audio);
	int pause_playback();                               // stop and arm playback
	int start_playback();                               // start instantly
	int wait_for_playback();							// when playing back from command line
	int stop_playback(int update_button = 0);           // stop
	int start_reconfigure(int unlock_window = 0);       // stop the playback for a reconfiguration
	int stop_reconfigure(int unlock_window = 0);        // restart playback after reconfiguration

	long get_playback_position();                   // get the exact position of the playback

// ====================================== plugins ==============================

// Contain file descriptors for all the dlopens
	ArrayList<PluginServer*> *plugindb;
// Currently visible plugins
	ArrayList<PluginServer*> *plugin_guis;


// TV stations to record from
	ArrayList<Channel*> channeldb_v4l;
	ArrayList<Channel*> channeldb_buz;
// Adjust sample position to line up with frames.
	int fix_timing(long &samples_out, long &frames_out, long samples_in);     

// ===================================== formatting parameters

// top pixel of track view
	int get_top();                         
// bottom pixel of track view
	int get_bottom();                      

// ================================== selection

	int set_selectionend(long new_position);
	int set_selectionstart(long new_position);
	int set_selection(long selectionstart, long selectionend);

// ==================================== cursor

	int get_affected_range(long *start, long *end, int reverse = 0);    // get range that rendering and playback affects

	Render *render;
	Render *renderlist;
// Master edl
	EDL *edl;
// Conosole
	Console *console;
// Main Window GUI
	MWindowGUI *gui;
// Compositor
	CWindow *cwindow;
// Viewer
	VWindow *vwindow;
// Asset manager
	AWindow *awindow;

private:
	void init_render();
	void init_defaults();
	void init_edl();
	void init_awindow();
	void init_plugins();
	void init_preferences();
	void init_theme();
	void init_compositor();
	void init_viewer();
	void init_cache();
	void init_menus();
	void init_indexes();
	void init_gui();
	void init_playbackcursor();
	void delete_plugins();
	void clean_indexes();
	void save_tuner(ArrayList<Channel*> &channeldb, char *path);
	TimeBomb timebomb;
};

#endif
