#include "clip.h"
#include "cplayback.h"
#include "cwindow.h"
#include "cwindowgui.h"
#include "edl.h"
#include "edlsession.h"
#include "labels.h"
#include "localsession.h"
#include "mainsession.h"
#include "mwindow.h"
#include "mwindowgui.h"
#include "patchbay.h"
#include "playbackengine.h"
#include "plugin.h"
#include "samplescroll.h"
#include "timebar.h"
#include "trackcanvas.h"
#include "tracks.h"
#include "transportque.h"
#include "zoombar.h"


void MWindow::update_plugins()
{
// Show plugins which are visible and hide plugins which aren't
// Update plugin pointers in plugin servers
}

void MWindow::show_plugin(Plugin *plugin)
{
	int done = 0;
	for(int i = 0; i < plugin_guis->total; i++)
	{
		if(plugin_guis->values[i]->plugin == plugin)
		{
			plugin_guis->values[i]->raise_window();
			done = 1;
			break;
		}
	}

	if(!done)
	{
		PluginServer *server = scan_plugindb(plugin->title);
		if(server && server->uses_gui)
		{
			PluginServer *gui = plugin_guis->append(new PluginServer(*server));
// Needs mwindow to do GUI
			gui->set_mwindow(this);
			gui->open_plugin(0, edl, plugin);
			gui->show_gui();
			plugin->show = 1;
		}
	}
}

void MWindow::hide_plugin(Plugin *plugin)
{
	plugin->show = 0;
	for(int i = 0; i < plugin_guis->total; i++)
	{
		if(plugin_guis->values[i]->plugin == plugin)
		{
//printf("MWindow::hide_plugin 1\n");
// Last command executed in client side close
			plugin_guis->remove_object(plugin_guis->values[i]);
//printf("MWindow::hide_plugin 2\n");
			break;
		}
	}
}

int MWindow::expand_sample()
{
	if(gui)
	{
		if(edl->session->zoom_sample < 0x100000)
		{
			edl->session->zoom_sample *= 2;
			gui->zoombar->sample_zoom->update(edl->session->zoom_sample);
			zoom_sample(edl->session->zoom_sample);
		}
	}
	return 0;
}

int MWindow::zoom_in_sample()
{
	if(gui)
	{
		if(edl->session->zoom_sample > 1)
		{
			edl->session->zoom_sample /= 2;
			gui->zoombar->sample_zoom->update(edl->session->zoom_sample);
			zoom_sample(edl->session->zoom_sample);
		}
	}
	return 0;
}

int MWindow::zoom_sample(long zoom_sample)
{
	CLIP(zoom_sample, 1, 0x100000);
	edl->session->zoom_sample = zoom_sample;
	find_cursor();
	gui->get_scrollbars();

	if(!gui->samplescroll) edl->session->view_start = 0;
	samplemovement(edl->session->view_start);
	gui->zoombar->sample_zoom->update(zoom_sample);
	return 0;
}

void MWindow::find_cursor()
{
	if((edl->local_session->selectionend > 
		(double)gui->canvas->get_w() * 
		edl->session->zoom_sample / 
		edl->session->sample_rate) ||
		(edl->local_session->selectionstart > 
		(double)gui->canvas->get_w() * 
		edl->session->zoom_sample / 
		edl->session->sample_rate))
	{
		edl->session->view_start = (edl->local_session->selectionend + 
			edl->local_session->selectionstart) / 
			2 - 
			(double)gui->canvas->get_w() * 
			edl->session->zoom_sample / 
			edl->session->sample_rate / 
			2;
	}
	else
		edl->session->view_start = 0.0;

//printf("MWindow::find_cursor %f\n", edl->session->view_start);
	if(edl->session->view_start < 0) edl->session->view_start = 0;
}


void MWindow::fit_selection()
{
	if(edl->local_session->selectionstart == edl->local_session->selectionend)
	{
		double total_samples = edl->tracks->total_length() * edl->session->sample_rate;
		for(edl->session->zoom_sample = 1; 
			gui->canvas->get_w() * edl->session->zoom_sample < total_samples; 
			edl->session->zoom_sample *= 2)
			;
	}
	else
	{
		double total_samples = (edl->local_session->selectionend - edl->local_session->selectionstart) * edl->session->sample_rate;
		for(edl->session->zoom_sample = 1; 
			gui->canvas->get_w() * edl->session->zoom_sample < total_samples; 
			edl->session->zoom_sample *= 2)
			;
	}

	edl->session->zoom_sample = MIN(0x100000, edl->session->zoom_sample);
	zoom_sample(edl->session->zoom_sample);
}

void MWindow::zoom_amp(long zoom_amp)
{
	edl->session->zoom_y = zoom_amp;
	gui->canvas->draw();
	gui->canvas->flash();
	gui->patchbay->update();
	gui->flush();
}

void MWindow::zoom_track(long zoom_track)
{
	edl->session->zoom_y = (long)((float)edl->session->zoom_y * zoom_track / edl->session->zoom_track);
	edl->session->zoom_track = zoom_track;
	trackmovement(edl->session->track_start);
//printf("MWindow::zoom_track %d %d\n", edl->session->zoom_y, edl->session->zoom_track);
}

void MWindow::trackmovement(int track_start)
{
	edl->session->track_start = track_start;
	if(edl->session->track_start < 0) edl->session->track_start = 0;
	edl->tracks->update_y_pixels(theme);
	gui->get_scrollbars();
	gui->canvas->draw();
	gui->patchbay->update();
	gui->canvas->flash();
	gui->flush();
}

void MWindow::move_up(long distance)
{
	if(!gui->trackscroll) return;
	if(distance == 0) distance = edl->session->zoom_track;
	edl->session->track_start -= distance;
	trackmovement(edl->session->track_start);
}

void MWindow::move_down(long distance)
{
	if(!gui->trackscroll) return;
	if(distance == 0) distance = edl->session->zoom_track;
	edl->session->track_start += distance;
	trackmovement(edl->session->track_start);
}

int MWindow::goto_end()
{
	double old_view_start = edl->session->view_start;

	if(edl->tracks->total_length() > (double)gui->canvas->get_w() * 
		edl->session->zoom_sample / 
		edl->session->sample_rate)
	{
		edl->session->view_start = 
			(edl->tracks->total_length() * 
				edl->session->sample_rate - 
				gui->canvas->get_w() * edl->session->zoom_sample / 
				2) /
				edl->session->sample_rate;
	}
	else
	{
		edl->session->view_start = 0.0;
	}

	if(gui->shift_down())
	{
		edl->local_session->selectionend = edl->tracks->total_length();
	}
	else
	{
		edl->local_session->selectionstart = 
			edl->local_session->selectionend = 
			edl->tracks->total_length();
	}

	if(edl->session->view_start != old_view_start) 
		samplemovement(edl->session->view_start);

	gui->cursor->update();
	gui->canvas->activate();
	cwindow->update_position();
	return 0;
}

int MWindow::goto_start()
{
	double old_view_start = edl->session->view_start;

	edl->session->view_start = 0;
	if(gui->shift_down())
	{
		edl->local_session->selectionstart = 0;
	}
	else
	{
		edl->local_session->selectionstart = 
			edl->local_session->selectionend = 
			0;
	}

	if(edl->session->view_start != old_view_start) 
		samplemovement(edl->session->view_start);

//printf("MWindow::goto_start 1\n");
	gui->cursor->update();
	gui->canvas->activate();
	cwindow->update_position();
	return 0;
}

int MWindow::samplemovement(double view_start)
{
	edl->session->view_start = view_start;
	if(edl->session->view_start < 0) edl->session->view_start = 0;
	gui->canvas->draw();
	gui->cursor->show();
	gui->canvas->flash();
	gui->timebar->update();
	gui->zoombar->update();

	if(gui->samplescroll) gui->samplescroll->set_position();
	return 0;
}

int MWindow::move_left(double distance)
{
//	if(gui->samplescroll)
//	{
		if(!distance) 
			distance = (double)edl->session->zoom_sample * 
				gui->canvas->get_w() / 
				5 /
				edl->session->sample_rate;
		edl->session->view_start -= distance;
		if(edl->session->view_start < 0) edl->session->view_start = 0;
		samplemovement(edl->session->view_start);
//	}
	return 0;
}

int MWindow::move_right(double distance)
{
//	if(gui->samplescroll)
//	{
		if(!distance) 
			distance = (double)edl->session->zoom_sample * 
				gui->canvas->get_w() / 
				5 /
				edl->session->sample_rate;
		edl->session->view_start += distance;
		samplemovement(edl->session->view_start);
//	}
	return 0;
}

void MWindow::select_all()
{
	edl->local_session->selectionstart = 0;
	edl->local_session->selectionend = edl->tracks->total_length();
	gui->update(0, 1, 1, 1, 0, 1);
	gui->canvas->activate();
	cwindow->update_position();
}

int MWindow::next_label()
{
	Label *current;
	Labels *labels = edl->labels;

// Test for label under cursor position
	for(current = labels->first; 
		current && !edl->equivalent(current->position, edl->local_session->selectionend); 
		current = NEXT)
		;

// Test for label before cursor position
	if(!current)
		for(current = labels->last;
			current && current->position > edl->local_session->selectionend;
			current = PREVIOUS)
			;

// Test for label after cursor position
	if(!current)
		current = labels->first;
	else
// Get next label
		current = NEXT;

	if(current)
	{

		edl->local_session->selectionend = current->position;
		if(!gui->shift_down()) edl->local_session->selectionstart = edl->local_session->selectionend;

		if(edl->local_session->selectionend >= edl->session->view_start + gui->canvas->time_visible() ||
			edl->local_session->selectionend < edl->session->view_start)
		{
			samplemovement(edl->local_session->selectionend - gui->canvas->time_visible() / 2);
		}
		else
		{
			gui->timebar->update();
			gui->cursor->hide();
			gui->cursor->draw();
			gui->zoombar->update();
			gui->canvas->flash();
			gui->flush();
		}
	}
	else
	{
		goto_end();
	}
	return 0;
}

int MWindow::prev_label()
{
	Label *current;
	Labels *labels = edl->labels;

// Test for label under cursor position
	for(current = labels->first; 
		current && !edl->equivalent(current->position, edl->local_session->selectionstart); 
		current = NEXT)
		;

// Test for label after cursor position
	if(!current)
		for(current = labels->first;
			current && current->position < edl->local_session->selectionstart;
			current = NEXT)
			;

// Test for label before cursor position
	if(!current) 
		current = labels->last;
	else
// Get previous label
		current = PREVIOUS;

	if(current)
	{
		edl->local_session->selectionstart = current->position;
		if(!gui->shift_down()) edl->local_session->selectionend = edl->local_session->selectionstart;

		if(edl->local_session->selectionstart >= edl->session->view_start + gui->canvas->time_visible() ||
			edl->local_session->selectionstart < edl->session->view_start)
		{
			samplemovement(edl->local_session->selectionstart - 
				gui->canvas->time_visible() / 
				2);
		}
		else
		{
			gui->timebar->update();
			gui->cursor->hide();
			gui->cursor->draw();
			gui->zoombar->update();
			gui->canvas->flash();
			gui->flush();
		}
	}
	else
	{
		goto_start();
	}
	return 0;
}

int MWindow::expand_y()
{
	int result = edl->session->zoom_y * 2;
	result = MIN(result, MAX_AMP_ZOOM);
	zoom_amp(result);
	gui->zoombar->update();
	return 0;
}

int MWindow::zoom_in_y()
{
	int result = edl->session->zoom_y / 2;
	result = MAX(result, MIN_AMP_ZOOM);
	zoom_amp(result);
	gui->zoombar->update();
	return 0;
}

int MWindow::expand_t()
{
	int result = edl->session->zoom_track * 2;
	result = MIN(result, MAX_TRACK_ZOOM);
	zoom_track(result);
	gui->zoombar->update();
	return 0;
}

int MWindow::zoom_in_t()
{
	int result = edl->session->zoom_track / 2;
	result = MAX(result, MIN_TRACK_ZOOM);
	zoom_track(result);
	gui->zoombar->update();
	return 0;
}

