#include "automation.h"
#include "edl.h"
#include "edlsession.h"
#include "mwindow.h"
#include "patchbay.h"
#include "playabletracks.h"
#include "preferences.h"
#include "renderengine.h"
#include "intauto.h"
#include "intautos.h"
#include "tracks.h"
#include "transportque.h"



PlayableTracks::PlayableTracks(RenderEngine *renderengine, 
	int *do_channel, 
	long current_position, 
	int data_type)
 : ArrayList<Track*>()
{
	this->renderengine = renderengine;
	this->data_type = data_type;
	this->do_channel = do_channel;

	for(Track *current_track = renderengine->edl->tracks->first; 
		current_track; 
		current_track = current_track->next)
	{
		if(is_playable(current_track, current_position))
		{
			append(current_track);
		}
	}
}


int PlayableTracks::is_playable(Track *current_track, long position)
{
	int result = 0;
	int direction = renderengine->command->get_direction();
	IntAuto *play_auto = (IntAuto*)current_track->automation->play_autos->get_prev_auto(position, direction);

	if(current_track->data_type != data_type) return 0;

// Track is off screen and not bounced to other modules
//printf("PlayableTracks::is_playable 1 %d %d\n", 
//	current_track->plugin_used(position, direction),
//	current_track->channel_is_playable(position, direction, do_channel));
	if(!current_track->plugin_used(position, direction) &&
		!current_track->channel_is_playable(position, direction, do_channel))
		return 0;

//printf("PlayableTracks::is_playable 2 %d\n", play_auto->value);
// Play patch is on
	if(play_auto->value)
	{
// Test if edit exists under the current position.
		if(renderengine->edl->session->test_playback_edits)
		{
			result = current_track->playable_edit(position);
		}
		else
		{
			result = 1;
		}
	}

	return result;
}


int PlayableTracks::is_listed(Track *track)
{
	for(int i = 0; i < total; i++)
	{
		if(values[i] == track) return 1;
	}
	return 0;
}
