#include "console.h"
#include "edl.h"
#include "edlsession.h"
#include "filexml.h"
#include "keyframe.h"
#include "keyframes.h"
#include "localsession.h"
#include "mwindow.h"
#include "messages.h"
#include "module.h"
#include "plugin.h"
#include "pluginbuffer.h"
#include "pluginpopup.h"
#include "pluginset.h"
#include "pluginserver.h"
#include "track.h"
#include "tracks.h"
#include "virtualnode.h"



Plugin::Plugin(EDL *edl, Track *track, PluginSet *plugin_set, char *title)
 : Edit(edl, track)
{
	this->track = track;
	this->plugin_set = plugin_set;
	strcpy(this->title, title);
	plugin_type = PLUGIN_NONE;
	in = 1;
	out = 1;
	show = 0;
	on = 1;
	keyframes = new KeyFrames;
	keyframes->create_objects();
}

Plugin::~Plugin()
{
//printf("Plugin::~Plugin 1 %p %p\n", keyframes, keyframes->last);
	while(keyframes->last) delete keyframes->last;
//printf("Plugin::~Plugin 2\n");
	delete keyframes;
//printf("Plugin::~Plugin 3\n");
}

Plugin& Plugin::operator=(Edit& edit)
{
printf("Plugin::operator= called\n");
	copy_from(&edit);
//printf("Plugin::operator= 2\n");
	return *this;
}

int Plugin::operator==(Plugin& that)
{
	char title1[BCTEXTLEN], title2[BCTEXTLEN];
	
	calculate_title(title1);
	that.calculate_title(title2);


printf("Plugin::operator== %s %s %d %d %s %s %d\n",
	title1, title2,
	plugin_type, that.plugin_type,
	keyframes->default_keyframe->data,
	that.keyframes->default_keyframe->data,
	*keyframes->default_keyframe == *that.keyframes->default_keyframe);
	return (!strcasecmp(title1, title2) &&
		plugin_type == that.plugin_type &&
		*keyframes->default_keyframe == *that.keyframes->default_keyframe);
}

int Plugin::silence()
{
	if(plugin_type != PLUGIN_NONE) 
		return 0;
	else
		return 1;
}

void Plugin::copy_from(Edit *edit)
{
	Plugin *plugin = (Plugin*)edit;
//printf("Plugin::copy_from 1\n");
	Edit::copy_from((Edit*)plugin);
	this->plugin_type = plugin->plugin_type;
	this->in = plugin->in;
	this->out = plugin->out;
	this->show = plugin->show;
	this->on = plugin->on;
	this->shared_location = plugin->shared_location;
	strcpy(this->title, plugin->title);
//printf("Plugin::copy_from 1\n");

	copy_keyframes(plugin);
//printf("Plugin::copy_from 3\n");
}

void Plugin::copy_keyframes(Plugin *plugin)
{
	keyframes->copy_from(plugin->keyframes);
}

void Plugin::synchronize_params(Edit *edit)
{
	Plugin *plugin = (Plugin*)edit;
	this->in = plugin->in;
	this->out = plugin->out;
	this->show = plugin->show;
	this->on = plugin->on;
	strcpy(this->title, plugin->title);
	copy_keyframes(plugin);
}

int Plugin::operator==(Edit& edit)
{
	Plugin *plugin = (Plugin*)&edit;
	if(this->plugin_type == plugin->plugin_type &&
		!strcasecmp(this->title, plugin->title) &&
		!strcasecmp(this->data, plugin->data) &&
		this->in == plugin->in &&
		this->out == plugin->out &&
		this->on == plugin->on)
		return 1;
	else
		return 0;
}

// FIXME
int Plugin::get_plugin_number()
{
	return 0;
}

KeyFrame* Plugin::get_prev_keyframe(long position)
{
	KeyFrame *current;

//printf("Plugin::get_prev_keyframe 1 %p %p\n", this, edl);
	if(position < 0) 
		position = track->to_units(edl->local_session->selectionstart, 0);
//printf("Plugin::get_prev_keyframe 1 %d\n", position);

	for(current = keyframes->last;
		current;
		current = PREVIOUS)
	{
		if(current->position <= position) break;
	}
	
	if(!current) current = keyframes->default_keyframe;
	
//printf("Plugin::get_prev_keyframe 2\n");
	return current;
}

KeyFrame* Plugin::get_next_keyframe(long position)
{
	KeyFrame *current;

	if(position < 0) 
		position = track->to_units(edl->local_session->selectionstart, 0);
	for(current = keyframes->first;
		current;
		current = NEXT)
	{
		if(current->position > position) break;
	}

	if(!current) current = keyframes->default_keyframe;
	return current;
}

KeyFrame* Plugin::get_keyframe()
{
// Search for keyframe on or before selection
	KeyFrame *result = 
		get_prev_keyframe(track->to_units(edl->local_session->selectionstart, 0));

//printf("Plugin::get_keyframe %p %p %p\n", result, edl, edl->session);
// Return nearest keyframe if not in automatic keyframe generation
	if(result->position == track->to_units(edl->local_session->selectionstart, 0) ||
		!edl->session->auto_keyframes)
	{
		return result;
	}
	else
	{
		return keyframes->insert_keyframe(track->to_units(edl->local_session->selectionstart, 0));
	}
	return 0;
}

void Plugin::copy(long start, long end, FileXML *file)
{
	long endproject = startproject + length;

	if((startproject >= start && startproject <= end) ||  // startproject in range
		 (endproject <= end && endproject >= start) ||	   // endproject in range
		 (startproject <= start && endproject >= end))    // range in project
	{
// edit is in range
		long startproject_in_selection = startproject; // start of edit in selection in project
		long startsource_in_selection = startsource; // start of source in selection in source
		long endsource_in_selection = startsource + length; // end of source in selection
		long length_in_selection = length;             // length of edit in selection

		if(startproject < start)
		{         // start is after start of edit in project
			long length_difference = start - startproject;

			startsource_in_selection += length_difference;
			startproject_in_selection += length_difference;
			length_in_selection -= length_difference;
		}

// end is before end of edit in project
		if(endproject > end)
		{         
			length_in_selection = end - startproject_in_selection;
		}

// Plugins don't store silence
		file->tag.set_title("PLUGIN");	
		file->tag.set_property("STARTPROJECT", startproject_in_selection);
		file->tag.set_property("LENGTH", length_in_selection);
		file->tag.set_property("TYPE", plugin_type);
		file->tag.set_property("TITLE", title);
		file->append_tag();
		file->append_newline();


		if(plugin_type == PLUGIN_SHAREDPLUGIN ||
			plugin_type == PLUGIN_SHAREDMODULE)
		{
			shared_location.save(file);
		}



		if(in)
		{
			file->tag.set_title("IN");
			file->append_tag();
		}
		if(out)
		{
			file->tag.set_title("OUT");
			file->append_tag();
		}
		if(show)
		{
			file->tag.set_title("SHOW");
			file->append_tag();
		}
		if(on)
		{
			file->tag.set_title("ON");
			file->append_tag();
		}
		file->append_newline();

// Keyframes
		keyframes->copy(start, end, file);

		file->tag.set_title("/PLUGIN");	
		file->append_tag();
		file->append_newline();
	}
}

void Plugin::load(FileXML *file)
{
	int result = 0;
	int first_keyframe = 1;
 	in = 0;
	out = 0;
	show = 0;
	on = 0;

//printf("Plugin::load 1\n");
	do{
		result = file->read_tag();

		if(!result)
		{
			if(file->tag.title_is("/PLUGIN"))
			{
				result = 1;
			}
			else
			if(file->tag.title_is("SHARED_LOCATION"))
			{
				shared_location.load(file);
			}
			else
			if(file->tag.title_is("IN"))
			{
				in = 1;
			}
			else
			if(file->tag.title_is("OUT"))
			{
				out = 1;
			}
			else
			if(file->tag.title_is("SHOW"))
			{
				show = 1;
			}
			else
			if(file->tag.title_is("ON"))
			{
				on = 1;
			}
			else
			if(file->tag.title_is("KEYFRAME"))
			{
				if(first_keyframe)
				{
					keyframes->default_keyframe->load(file);
				}
				else
				{
					KeyFrame *keyframe = keyframes->append(new KeyFrame);
					keyframe->load(file);
				}
			}
		}
	}while(!result);
//printf("Plugin::load 2 %s\n", title);
}

Track* Plugin::get_shared_track()
{
	return edl->tracks->get_item_number(shared_location.module);
}

Plugin* Plugin::get_shared_plugin()
{
	Track *track = get_shared_track();

	if(track && 
		shared_location.plugin >= 0)
	{
		return track->get_current_plugin(startproject, 
			shared_location.plugin, 
			PLAY_FORWARD,
			0);
	}
	else
		return 0;
}


void Plugin::calculate_title(char *string)
{
	if(plugin_type == PLUGIN_STANDALONE)
	{
		strcpy(string, title);
	}
	else
	if(plugin_type == PLUGIN_SHAREDPLUGIN)
	{
printf("Plugin::calculate_title %d %d\n", shared_location.module, shared_location.plugin);
		Track *track = get_shared_track();
printf("Plugin::calculate_title 1\n");
		Plugin *plugin = get_shared_plugin();
printf("Plugin::calculate_title 1\n");
		char track_title[BCTEXTLEN];
		char plugin_title[BCTEXTLEN];

printf("Plugin::calculate_title %p %p\n", track, plugin);
		if(track)
			strcpy(track_title, track->title);
		else
			sprintf(track_title, "None");
		
		if(plugin)
			strcpy(plugin_title, plugin->title);
		else
			sprintf(plugin_title, "None");
		sprintf(string, "%s: %s", track_title, plugin_title);
	}
	else
	if(plugin_type == PLUGIN_SHAREDMODULE)
	{
		Track *track = get_shared_track();
		char track_title[BCTEXTLEN];

		if(track)
			strcpy(string, track_title);
		else
			sprintf(string, "None");
	}
}


void Plugin::paste(FileXML *file)
{
	length = file->tag.get_property("LENGTH", (long)0);
}

void Plugin::resample(double old_rate, double new_rate)
{
// Resample keyframes in here
	keyframes->resample(old_rate, new_rate);
}



void Plugin::dump()
{
	printf("    PLUGIN: %s type %d title: %s\n", title, plugin_type, title);
	printf("    startproject %ld length %ld\n", startproject, length);
	printf("    KEYFRAMES\n");
	for(KeyFrame *current = keyframes->first;
		current;
		current = NEXT)
	{
		current->dump();
	}
}























Plugin::Plugin(MWindow *mwindow, Module *module, int plugin_number)
 : Edit((EDL*)0, (Edits*)0)
{
printf("Plugin::Plugin(MWindow *mwindow, Module *module, int plugin_number)\n");
	this->mwindow = mwindow;
	this->plugin_number = plugin_number;
	this->module = module;
//	plugin_server = 0;
	in = out = 0;
	on = 1;
	show = 1;
	plugin_type = 0;
//	sprintf(plugin_title, default_title());
	plugin_popup = 0;
	show_title = 0;
	show_toggle = 0;
	on_toggle = 0;
	on_title = 0;
//	total_input_buffers = 0;
//	new_total_input_buffers = 0;
}

char* Plugin::default_title()
{
	return "Plugin";
}

int Plugin::update_derived()
{
//	plugin_popup->update(in, out, plugin_title);
}

int Plugin::update_display()
{
	if(mwindow->gui)
	{
//		plugin_popup->update(in, out, plugin_title);
//		show_toggle->update(show);
//		on_toggle->update(on);
	}
}

int Plugin::swap_modules(int number1, int number2)
{
// 	if(shared_plugin_location.module == number1) shared_plugin_location.module == number2;
// 	else
// 	if(shared_plugin_location.module == number2) shared_plugin_location.module == number1;
// 
// 	if(shared_module_location.module == number1) shared_module_location.module == number2;
// 	else
// 	if(shared_module_location.module == number2) shared_module_location.module == number1;
}

int Plugin::set_show_derived(int value)
{
	if(show_toggle) show_toggle->update(value);
}

int Plugin::set_string()
{
// 	if(plugin_server)
// 	{
// 		if(use_gui())
// 		{
// 			char new_string[1024];
// 			sprintf(new_string, "%s: %s\n", get_module_title(), plugin_title);
// 			plugin_server->set_string(new_string);
// 		}
// 	}
}

char* Plugin::get_module_title()
{
	return module->title;
}

int Plugin::resize_plugin(int x, int y)
{
	if(plugin_popup) plugin_popup->reposition_window(x, y);
	if(show_toggle) show_toggle->reposition_window(x + 10, y + 23);
	if(show_title) show_title->reposition_window(x + 30, y + 25);
	if(on_toggle) on_toggle->reposition_window(x + 60, y + 23);
	if(on_title) on_title->reposition_window(x + 80, y + 25);
}




// ======================================= radial

PluginShowToggle::PluginShowToggle(Plugin *plugin, Console *console, int x, int y)
 : BC_Radial(x, y, plugin->show)
{
	this->console = console;
	this->plugin = plugin;
}

int PluginShowToggle::handle_event()
{
	if(shift_down())
	{
		int total_selected = console->toggles_selected(0, 1, 0);

		if(total_selected == 0)
		{
// nothing previously selected
			console->select_all_toggles(0, 1, 0);
		}
		else
		if(total_selected == 1)
		{
			if(plugin->show)
			{
// this patch was previously the only one on
				console->select_all_toggles(0, 1, 0);
			}
			else
			{
// another patch was previously the only one on
				console->deselect_all_toggles(0, 1, 0);
				plugin->show = 1;
			}
		}
		else
		if(total_selected > 1)
		{
// other patches were previously on
			console->deselect_all_toggles(0, 1, 0);
			plugin->show = 1;
		}
		
		update(plugin->show);
	}
	else
	{
		if(get_value() != plugin->show)
		{
			plugin->show = get_value();
//			if(plugin->plugin_server)
//			{
//				if(get_value()) plugin->show_gui();
//				else plugin->hide_gui();
//			}
		}
	}

	console->button_down = 1;
	console->new_status = get_value();
	return 1;
}

int PluginShowToggle::cursor_moved_over()
{
	if(console->button_down && console->new_status != get_value())
	{
		update(console->new_status);
		plugin->show = get_value();

// 		if(plugin->plugin_server && console->gui)
// 		{
// 			if(get_value()) plugin->show_gui();
// 			else plugin->hide_gui();
// 		}
	}
}

int PluginShowToggle::button_release()
{
	console->button_down = 0;
}

PluginOnToggle::PluginOnToggle(Plugin *plugin, Console *console, int x, int y)
 : BC_Radial(x, y, plugin->show)
{
	this->console = console;
	this->plugin = plugin;
}

int PluginOnToggle::handle_event()
{
	if(shift_down())
	{
		int total_selected = console->toggles_selected(1, 0, 0);

		if(total_selected == 0)
		{
// nothing previously selected
			console->select_all_toggles(1, 0, 0);
		}
		else
		if(total_selected == 1)
		{
			if(plugin->on)
			{
// this patch was previously the only one on
				console->select_all_toggles(1, 0, 0);
			}
			else
			{
// another patch was previously the only one on
				console->deselect_all_toggles(1, 0, 0);
				plugin->on = 1;
			}
		}
		else
		if(total_selected > 1)
		{
// other patches were previously on
			console->deselect_all_toggles(1, 0, 0);
			plugin->on = 1;
		}

		update(plugin->on);
	}
	else
	{
		if(/*plugin->plugin_server && */get_value() != plugin->on)
		{
			plugin->on = get_value();
		}
	}

	console->button_down = 1;
	console->reconfigure_trigger = 1;
	console->new_status = get_value();
	return 0;
}

int PluginOnToggle::cursor_moved_over()
{
	if(console->button_down && console->new_status != get_value())
	{
		update(console->new_status);
		plugin->on = get_value();
	}
}

int PluginOnToggle::button_release()
{
	console->button_down = 0;
}
