#include "edl.h"
#include "edlsession.h"
#include "pluginaclient.h"
#include "pluginserver.h"

#include <string.h>


PluginAClient::PluginAClient(PluginServer *server)
 : PluginClient(server)
{
}

PluginAClient::~PluginAClient()
{
}

int PluginAClient::is_audio()
{
	return 1;
}

int PluginAClient::create_buffer_ptrs()
{
	int i, j;
	
	for(i = 0; i < total_in_buffers; i++)
	{
		input_ptr_master.append(new float*[MAX_BUFFERS]);
//printf("double_buffers_in.values[i] %d\n", double_buffers_in.values[i]);
		for(j = 0; j < double_buffers_in.values[i]; j++)
		{
			input_ptr_master.values[i][j] = (float*)data_in_realtime.values[i][j]->get_data();
//printf("input_ptr_master.values[%d][%d] = %x\n", i, j, input_ptr_master.values[i][j]);
		}
	}
	input_ptr_render = new float*[total_in_buffers];

	for(i = 0; i < total_out_buffers; i++)
	{
		output_ptr_master.append(new float*[MAX_BUFFERS]);
		for(j = 0; j < double_buffers_out.values[i]; j++)
		{
			output_ptr_master.values[i][j] = (float*)data_out_realtime.values[i][j]->get_data();
		}
	}
	
	output_ptr_render = new float*[total_out_buffers];
	return 0;
}

int PluginAClient::delete_buffer_ptrs()
{
	int i;
// delete double buffer arrays
	for(int i = 0; i < total_in_buffers; i++)
	{
		delete input_ptr_master.values[i];
	}
	for(int i = 0; i < total_out_buffers; i++)
	{
		delete output_ptr_master.values[i];
	}
	input_ptr_master.remove_all();
	output_ptr_master.remove_all();
	delete input_ptr_render;
	delete output_ptr_render;
}

int PluginAClient::get_render_ptrs()
{
	int i, j, double_buffer, fragment_position;

	for(i = 0; i < total_in_buffers; i++)
	{
		double_buffer = double_buffer_in_render.values[i];
		fragment_position = offset_in_render.values[i];
		input_ptr_render[i] = &input_ptr_master.values[i][double_buffer][fragment_position];
//printf("PluginAClient::get_render_ptrs %x\n", input_ptr_master.values[i][double_buffer]);
	}

	for(i = 0; i < total_out_buffers; i++)
	{
		double_buffer = double_buffer_out_render.values[i];
		fragment_position = offset_out_render.values[i];
		output_ptr_render[i] = &output_ptr_master.values[i][double_buffer][fragment_position];
	}
//printf("PluginAClient::get_render_ptrs %x %x\n", input_ptr_render[0], output_ptr_render[0]);
	return 0;
}

int PluginAClient::init_realtime_parameters()
{
	project_sample_rate = server->edl->session->sample_rate;
	return 0;
}

void PluginAClient::plugin_process_realtime(double **input, 
		double **output, 
		long current_position, 
		long fragment_size,
		long total_len)
{
//printf("PluginAClient::plugin_process_realtime 1\n");
	this->source_position = current_position;
	this->total_len = total_len;

	if(is_multichannel())
		process_realtime(fragment_size, input, output);
	else
		process_realtime(fragment_size, input[0], output[0]);
}



int PluginAClient::plugin_process_loop(double **buffers, long &write_length)
{
	write_length = 0;

	if(is_multichannel())
		return process_loop(buffers, write_length);
	else
		return process_loop(buffers[0], write_length);
}

int PluginAClient::read_samples(double *buffer, int channel, long start_position, long total_samples)
{
//printf("PluginAClient::read_samples 1\n");
	return server->read_samples(buffer, channel, start_position, total_samples);
}


