#include "console.h"
#include "edl.h"
#include "localsession.h"
#include "mainsession.h"
#include "mwindow.h"
#include "mwindowgui.h"
#include "module.h"
#include "modules.h"
#include "plugin.h"
#include "plugindialog.h"
#include "pluginserver.h"
#include "theme.h"
#include "track.h"
#include "tracks.h"
#include "transition.h"


PluginDialogThread::PluginDialogThread(MWindow *mwindow)
 : Thread()
{
	this->mwindow = mwindow;
	window = 0;
	plugin = 0;
	edit = 0;
	Thread::set_synchronous(0);
}

PluginDialogThread::~PluginDialogThread()
{
	if(window)
	{
		window->set_done(1);
		completion.lock();
		completion.unlock();
	}
}

void PluginDialogThread::start_window(Track *track,
	Edit *edit, 
	Plugin *plugin, 
	char *title)
{
	if(Thread::running())
	{
		window->raise_window();
		window->flush();
	}
	else
	{
		this->edit = edit;
		this->track = track;
		this->data_type = track->data_type;
		this->plugin = plugin;

		if(plugin)
		{
			strcpy(plugin_title, plugin->title);
			this->shared_location = plugin->shared_location;
			this->plugin_type = plugin->plugin_type;
			this->in = plugin->in;
			this->out = plugin->out;
		}
		else
		{
			this->plugin_title[0] = 0;
			this->shared_location.plugin = -1;
			this->shared_location.module = -1;
			this->plugin_type = PLUGIN_NONE;
			this->in = 1;
			this->out = 1;
		}

		strcpy(this->window_title, title);
		completion.lock();
		Thread::start();
	}
}


int PluginDialogThread::set_dialog(Transition *transition, char *title)
{
	return 0;
}

void PluginDialogThread::run()
{
	int result = 0;

	window = new PluginDialog(mwindow, this, window_title);
	window->create_objects();
	result = window->run_window();
printf("PluginDialogThread::run 1\n");
	delete window;
printf("PluginDialogThread::run 1\n");
	window = 0;
printf("PluginDialogThread::run 1\n");
	completion.unlock();

// Done at closing
	if(!result)
	{
printf("PluginDialogThread::run 2\n");
		if(plugin_type)
		{
printf("PluginDialogThread::run 3\n");
			mwindow->gui->lock_window();
printf("PluginDialogThread::run 4\n");
			mwindow->insert_effect(plugin_title, 
							&shared_location,
							track,
							plugin_type);
printf("PluginDialogThread::run 5\n");
			mwindow->gui->update(1,
				1,
				0,
				1,
				1, 
				0);
			mwindow->gui->unlock_window();
printf("PluginDialogThread::run 6\n");
		}
	}
	plugin = 0;
	edit = 0;
}









PluginDialog::PluginDialog(MWindow *mwindow, 
	PluginDialogThread *thread, 
	char *window_title)
 : BC_Window(window_title, 
 	mwindow->gui->get_abs_cursor_x(), 
	mwindow->gui->get_abs_cursor_y(), 
	500, 
	415, 
	500, 
	415,
	0,
	0,
	1)
{
	this->mwindow = mwindow;  
	this->thread = thread;
	inoutthru = 0;
}

PluginDialog::~PluginDialog()
{
	int i;
	for(i = 0; i < standalone_data.total; i++) delete standalone_data.values[i];
	standalone_data.remove_all();
	
	for(i = 0; i < shared_data.total; i++) delete shared_data.values[i];
	shared_data.remove_all();
	
	for(i = 0; i < module_data.total; i++) delete module_data.values[i];
	module_data.remove_all();

	for(i = 0; i < plugin_locations.total; i++) delete plugin_locations.values[i];
	plugin_locations.remove_all();

	for(i = 0; i < module_locations.total; i++) delete module_locations.values[i];
	module_locations.remove_all();

	delete title;
	delete detach;
	delete standalone_list;
	delete shared_list;
	delete module_list;
	delete standalone_attach;
	delete shared_attach;
	delete module_attach;
	delete in;
	delete out;
}

int PluginDialog::create_objects()
{
	int use_default = 1;
	char string[1024];
	int module_number;
	int x = 10, y = 10;

//printf("PluginDialog::create_objects 1\n");
 	if(thread->plugin)
	{
// user wants in and out
		strcpy(string, thread->plugin->title);
		if(thread->in == 1 || thread->out == 1)
			use_default = 0;
	}
	else
	{
// no plugin
		sprintf(string, "None");
	}






// GET A LIST OF ALL THE PLUGINS AVAILABLE
	ArrayList<PluginServer*> plugindb;
	mwindow->create_plugindb(thread->data_type == TRACK_AUDIO, 
		thread->data_type == TRACK_VIDEO, 
		1, 
		0,
		0,
		plugindb);

	mwindow->edl->get_shared_plugins(thread->edit->track,
		&plugin_locations);

	mwindow->edl->get_shared_tracks(thread->edit->track,
		&module_locations);








// Construct listbox items
	for(int i = 0; i < plugindb.total; i++)
		standalone_data.append(new BC_ListBoxItem(plugindb.values[i]->title));
	for(int i = 0; i < plugin_locations.total; i++)
	{
		Track *track = mwindow->edl->tracks->number(plugin_locations.values[i]->module);
		char *track_title = track->title;
		int number = plugin_locations.values[i]->plugin;
		Plugin *plugin = track->get_current_plugin(mwindow->edl->local_session->selectionstart, 
			number, 
			PLAY_FORWARD,
			1);
		char *plugin_title = plugin->title;
		char string[BCTEXTLEN];

		sprintf(string, "%s: %s", track_title, plugin_title);
		shared_data.append(new BC_ListBoxItem(string));
	}
	for(int i = 0; i < module_locations.total; i++)
	{
		Track *track = mwindow->edl->tracks->number(module_locations.values[i]->module);
		module_data.append(new BC_ListBoxItem(track->title));
	}








	add_subwindow(new BC_Title(x, y, "Current:"));
	x += 100;
	add_subwindow(title = new PluginDialogTextBox(this, string, x, y));
	x += 210;
	add_subwindow(detach = new PluginDialogDetach(mwindow, this, x, y));

//printf("PluginDialog::create_objects 1\n");
	x = 10;
	y = 45;
	add_subwindow(new BC_Title(x, y, "Plugins:"));
	add_subwindow(standalone_list = new PluginDialogNew(this, &standalone_data, x, y + 20));
	add_subwindow(standalone_attach = new PluginDialogAttachNew(mwindow, this, x + 20, y + 255));

//printf("PluginDialog::create_objects 1\n");
	x = 170;
	add_subwindow(new BC_Title(x, y, "Shared plugins:"));
	add_subwindow(shared_list = new PluginDialogShared(this, &shared_data, x, y + 20));
	add_subwindow(shared_attach = new PluginDialogAttachShared(mwindow, this, x + 20, y + 255));

//printf("PluginDialog::create_objects 1\n");
	x = 320;
	add_subwindow(new BC_Title(x, y, "Shared modules:"));
	add_subwindow(module_list = new PluginDialogModules(this, &module_data, x, y + 20));
	add_subwindow(module_attach = new PluginDialogAttachModule(mwindow, this, x + 20, y + 255));
	
//printf("PluginDialog::create_objects 1\n");
	x = 160;
	y = 330;
	add_subwindow(in = new PluginDialogIn(this, use_default ? 1 : (thread->plugin ? thread->plugin->in : 0), x, y));
	x += 100;
	add_subwindow(out = new PluginDialogOut(this, use_default ? 1 : (thread->plugin ? thread->plugin->out : 0), x, y));
	add_subwindow(new BC_OKButton(this));
	add_subwindow(new BC_CancelButton(this));

//printf("PluginDialog::create_objects 1\n");
	selected_available = -1;
	selected_shared = -1;
	selected_modules = -1;
	
//printf("PluginDialog::create_objects 1\n");
	show_window();
	flush();
//printf("PluginDialog::create_objects 2\n");
	return 0;
}

int PluginDialog::attach_new(int number)
{
	if(number > -1 && number < standalone_data.total) 
	{
		title->update(standalone_data.values[number]->get_text());
		strcpy(thread->plugin_title, standalone_data.values[number]->get_text());
		thread->plugin_type = PLUGIN_STANDALONE;         // type is plugin
	}
	return 0;
}

int PluginDialog::attach_shared(int number)
{
	if(number > -1 && number < shared_data.total) 
	{
		title->update(shared_data.values[number]->get_text());
		thread->plugin_type = PLUGIN_SHAREDPLUGIN;         // type is shared plugin
		thread->shared_location = *(plugin_locations.values[number]); // copy location
	}
	return 0;
}

int PluginDialog::attach_module(int number)
{
	if(number > -1 && number < module_data.total) 
	{
		title->update(module_data.values[number]->get_text());
		thread->plugin_type = PLUGIN_SHAREDMODULE;         // type is module
		thread->shared_location = *(module_locations.values[number]); // copy location
	}
	return 0;
}

int PluginDialog::save_settings()
{
}








PluginDialogTextBox::PluginDialogTextBox(PluginDialog *dialog, char *text, int x, int y)
 : BC_TextBox(x, y, 200, 1, text) 
{ 
	this->dialog = dialog;
}
PluginDialogTextBox::~PluginDialogTextBox() 
{ }
int PluginDialogTextBox::handle_event() 
{ }

PluginDialogDetach::PluginDialogDetach(MWindow *mwindow, PluginDialog *dialog, int x, int y)
 : BC_GenericButton(x, y, "Detach") 
{ 
	this->dialog = dialog; 
}
PluginDialogDetach::~PluginDialogDetach() 
{ }
int PluginDialogDetach::handle_event() 
{
	dialog->title->update("None");
	dialog->thread->plugin_type = 0;         // type is none
	dialog->thread->plugin_title[0] = 0;
	return 1;
}














PluginDialogNew::PluginDialogNew(PluginDialog *dialog, 
	ArrayList<BC_ListBoxItem*> *standalone_data, 
	int x,
	int y)
 : BC_ListBox(x, 
 	y, 
	130, 
	220, 
	LISTBOX_TEXT,
	standalone_data) 
{ 
	this->dialog = dialog; 
}
PluginDialogNew::~PluginDialogNew() { }
int PluginDialogNew::handle_event() 
{ 
	dialog->attach_shared(get_selection_number(0, 0)); 
	deactivate();

	set_done(0); 
}
int PluginDialogNew::selection_changed()
{
	dialog->selected_available = get_selection_number(0, 0);
}

PluginDialogAttachNew::PluginDialogAttachNew(MWindow *mwindow, PluginDialog *dialog, int x, int y)
 : BC_GenericButton(x, y, "Attach") 
{ 
 	this->dialog = dialog; 
}
PluginDialogAttachNew::~PluginDialogAttachNew() 
{ }
int PluginDialogAttachNew::handle_event() 
{ 
	dialog->attach_new(dialog->selected_available); 
}










PluginDialogShared::PluginDialogShared(PluginDialog *dialog, 
	ArrayList<BC_ListBoxItem*> *shared_data, 
	int x,
	int y)
 : BC_ListBox(x, 
 	y, 
	130, 
	220, 
	LISTBOX_TEXT, 
	shared_data) 
{ 
	this->dialog = dialog; 
}
PluginDialogShared::~PluginDialogShared() { }
int PluginDialogShared::handle_event()
{ 
	dialog->attach_shared(get_selection_number(0, 0)); 
	deactivate();

	set_done(0); 
}
int PluginDialogShared::selection_changed()
{
	dialog->selected_shared = get_selection_number(0, 0);
}

PluginDialogAttachShared::PluginDialogAttachShared(MWindow *mwindow, 
	PluginDialog *dialog, 
	int x,
	int y)
 : BC_GenericButton(x, y, "Attach") 
{ 
	this->dialog = dialog; 
}
PluginDialogAttachShared::~PluginDialogAttachShared() { }
int PluginDialogAttachShared::handle_event() 
{ 
	dialog->attach_module(dialog->selected_shared); 
}












PluginDialogModules::PluginDialogModules(PluginDialog *dialog, 
	ArrayList<BC_ListBoxItem*> *module_data, 
	int x, 
	int y)
 : BC_ListBox(x, 
 	y, 
	150, 
	220, 
	LISTBOX_TEXT, 
	module_data) 
{ 
	this->dialog = dialog; 
}
PluginDialogModules::~PluginDialogModules() { }
int PluginDialogModules::handle_event()
{ 
	dialog->attach_module(get_selection_number(0, 0)); 
	deactivate();

	set_done(0); 
}
int PluginDialogModules::selection_changed()
{
	dialog->selected_modules = get_selection_number(0, 0);
}


PluginDialogAttachModule::PluginDialogAttachModule(MWindow *mwindow, 
	PluginDialog *dialog, 
	int x, 
	int y)
 : BC_GenericButton(x, y, "Attach") 
{ 
	this->dialog = dialog; 
}
PluginDialogAttachModule::~PluginDialogAttachModule() { }
int PluginDialogAttachModule::handle_event() 
{ 
	dialog->attach_module(dialog->selected_modules); 
}














PluginDialogIn::PluginDialogIn(PluginDialog *dialog, int setting, int x, int y)
 : BC_CheckBox(x, y, setting, "Send") 
{ 
	this->dialog = dialog; 
}
PluginDialogIn::~PluginDialogIn() { }
int PluginDialogIn::handle_event() 
{ 
	dialog->thread->in = get_value();
	return 1;
}
int PluginDialogIn::button_press()
{
	dialog->inoutthru = 1;
	dialog->new_value = get_value();
	return 1;
}
int PluginDialogIn::button_release()
{
	if(dialog->inoutthru) dialog->inoutthru = 0;
}
int PluginDialogIn::cursor_moved_over()
{
	if(dialog->inoutthru && get_value() != dialog->new_value)
	{
		update(dialog->new_value);
	}
}

PluginDialogOut::PluginDialogOut(PluginDialog *dialog, int setting, int x, int y)
 : BC_CheckBox(x, y, setting, "Receive") 
{ 
 	this->dialog = dialog; 
}
PluginDialogOut::~PluginDialogOut() { }
int PluginDialogOut::handle_event() 
{ 
	dialog->thread->out = get_value();
	return 1;
}
int PluginDialogOut::button_press()
{
	dialog->inoutthru = 1;
	dialog->new_value = get_value();
	return 1;
}
int PluginDialogOut::button_release()
{
	if(dialog->inoutthru) dialog->inoutthru = 0;
}
int PluginDialogOut::cursor_moved_over()
{
	if(dialog->inoutthru && get_value() != dialog->new_value)
	{
		update(dialog->new_value);
	}
}

PluginDialogThru::PluginDialogThru(PluginDialog *dialog, int setting)
 : BC_CheckBox(300, 350, setting, "Thru") 
{ 
	this->dialog = dialog; 
}
PluginDialogThru::~PluginDialogThru() { }
int PluginDialogThru::handle_event() { }
int PluginDialogThru::button_press()
{
	dialog->inoutthru = 1;
	dialog->new_value = get_value();
	return 1;
}
int PluginDialogThru::button_release()
{
	if(dialog->inoutthru) dialog->inoutthru = 0;
}
int PluginDialogThru::cursor_moved_over()
{
	if(dialog->inoutthru && get_value() != dialog->new_value)
	{
		update(dialog->new_value);
	}
}
