#include "edl.h"
#include "edlsession.h"
#include "filexml.h"
#include "keyframe.h"
#include "keyframes.h"
#include "plugin.h"
#include "pluginautos.h"
#include "pluginset.h"
#include "track.h"

#include <string.h>

PluginSet::PluginSet(EDL *edl, Track *track)
 : Edits(edl, track)
{
}

PluginSet::~PluginSet()
{
//printf("PluginSet::~PluginSet 1 %p %p\n", this, last);
	while(last) delete last;
//printf("PluginSet::~PluginSet 2\n");
}


PluginSet& PluginSet::operator=(PluginSet& plugins)
{
//printf("PluginSet::operator= 1\n");
	while(last) delete last;
//printf("PluginSet::operator= 2\n");
	for(Plugin *current = (Plugin*)plugins.first; current; current = (Plugin*)NEXT)
	{
//printf("PluginSet::operator= 3\n");
		Plugin *new_plugin;
//printf("PluginSet::operator= 4\n");
		append(new_plugin = (Plugin*)create_edit());
//printf("PluginSet::operator= 5\n");
		new_plugin->copy_from(current);
//printf("PluginSet::operator= 6\n");
	}
//printf("PluginSet::operator= 7\n");
	return *this;
}

void PluginSet::synchronize_params(PluginSet *plugin_set)
{
	for(Plugin *this_plugin = (Plugin*)first, *that_plugin = (Plugin*)plugin_set->first;
		this_plugin && that_plugin;
		this_plugin = (Plugin*)this_plugin->next, that_plugin = (Plugin*)that_plugin->next)
	{
		this_plugin->synchronize_params(that_plugin);
	}
}

Plugin* PluginSet::insert_plugin(char *title, 
	long unit_position, 
	long unit_length,
	int plugin_type,
	SharedLocation *shared_location,
	KeyFrame *default_keyframe)
{
	Plugin *plugin = (Plugin*)insert_new_edit(unit_position);
printf("PluginSet::insert_plugin 1\n");
	plugin->length = unit_length;
printf("PluginSet::insert_plugin 1\n");

	if(title) strcpy(plugin->title, title);
printf("PluginSet::insert_plugin 1\n");

	if(shared_location) plugin->shared_location = *shared_location;
printf("PluginSet::insert_plugin 1\n");

	plugin->plugin_type = plugin_type;
printf("PluginSet::insert_plugin 1\n");

	if(default_keyframe) 
		*plugin->keyframes->default_keyframe = *default_keyframe;
printf("PluginSet::insert_plugin 2\n");
	return plugin;
}

Edit* PluginSet::create_edit()
{
	Plugin* result = new Plugin(edl, track, this, "");
	return result;
}

Edit* PluginSet::insert_edit_after(Edit *previous_edit)
{
	Plugin *current = new Plugin(edl, track, this, "");
	List<Edit>::insert_after(previous_edit, current);
	return (Edit*)current;
}


int PluginSet::get_number()
{
	return track->plugin_set.number_of(this);
}

void PluginSet::copy(long start, long end, FileXML *file)
{
	file->tag.set_title("PLUGINSET");	
	file->append_tag();
	file->append_newline();

	for(Plugin *current = (Plugin*)first; current; current = (Plugin*)NEXT)
	{
		current->copy(start, end, file);
	}

	file->tag.set_title("/PLUGINSET");	
	file->append_tag();
	file->append_newline();
}

void PluginSet::save(FileXML *file)
{
	copy(0, total_length(), file);
}

void PluginSet::load(FileXML *file, unsigned long load_flags)
{
	int result = 0;
// Current plugin being amended
	Plugin *plugin = 0;
//printf("PluginSet::load 1\n");

	do{
		result = file->read_tag();

		if(!result)
		{
			if(file->tag.title_is("/PLUGINSET"))
			{
				result = 1;
			}
			else
			if(file->tag.title_is("PLUGIN"))
			{
				long startproject = file->tag.get_property("STARTPROJECT", (long)0);
				long length = file->tag.get_property("LENGTH", (long)0);
				int plugin_type = file->tag.get_property("TYPE", 1);
				char title[BCTEXTLEN];
				title[0] = 0;
				file->tag.get_property("TITLE", title);
				SharedLocation shared_location;
				shared_location.load(file);

//printf("PluginSet::load 2 %s\n", title);
				plugin = insert_plugin(title, 
					startproject, 
					length,
					plugin_type,
					&shared_location,
					0);
				plugin->load(file);
			}
		}
	}while(!result);
}



int PluginSet::optimize()
{
	int result = 1;
	Plugin *current_edit;

//return 0;
	while(result)
	{
		result = 0;

// delete 0 length edits
		for(current_edit = (Plugin*)first; 
			current_edit && !result; )
		{
			if(current_edit->length == 0)
			{
				Plugin* next = (Plugin*)current_edit->next;
				delete current_edit;
				result = 1;
				current_edit = next;
			}
			else
				current_edit = (Plugin*)current_edit->next;
		}


printf("PluginSet::optimize 1\n");
// merge pass
		for(current_edit = (Plugin*)first; 
			current_edit && current_edit->next && !result; )
		{
// plugins identical
			Plugin *next_edit = (Plugin*)current_edit->next;


printf("PluginSet::optimize 2 %d\n", *next_edit == *current_edit);
   			if(*next_edit == *current_edit)
        	{
printf("PluginSet::optimize 3\n");
        		current_edit->length += next_edit->length;
// Merge keyframes
				for(KeyFrame *source = next_edit->keyframes->first;
					source;
					source = (KeyFrame*)source->next)
				{
					KeyFrame *dest = new KeyFrame;
					*dest = *source;
					current_edit->keyframes->append(dest);
				}
        		remove(next_edit);
        		result = 1;
        	}

    		current_edit = (Plugin*)current_edit->next;
		}

// delete last edit of 0 length or silence
// 		if(last && 
// 			(last->silence() || 
// 			!last->length))
// 		{
// 			delete last;
// 			result = 1;
// 		}
	}

//track->dump();
	return 0;
}





void PluginSet::dump()
{
	printf("   PLUGIN_SET:\n");
	for(Plugin *current = (Plugin*)first; current; current =  (Plugin*)NEXT)
		current->dump();
}
