#include "edl.h"
#include "edlsession.h"
#include "pluginserver.h"
#include "pluginvclient.h"

#include <string.h>

PluginVClient::PluginVClient(PluginServer *server)
 : PluginClient(server)
{
	video_in = 0;
	video_out = 0;
}

PluginVClient::~PluginVClient()
{
}

int PluginVClient::is_video()
{
	return 1;
}

// Run before starting the realtime plugin
int PluginVClient::create_buffer_ptrs()
{
	long i, j, k, size;
	VPixel *data;

	if(total_in_buffers)
	{
		for(i = 0; i < total_in_buffers; i++)
		{
// get a single channel's double buffers
			input_ptr_master.append(new VFrame**[MAX_BUFFERS]);
			size = realtime_in_size.values[i];
			for(j = 0; j < double_buffers_in.values[i]; j++)
			{
// get a double buffer for the channel
				data = (VPixel*)data_in_realtime.values[i][j]->get_data();
				input_ptr_master.values[i][j] = new VFrame*[size];

// get the frames for the double buffer
				for(k = 0; k < size; k++)
				{
					input_ptr_master.values[i][j][k]
					 = new VFrame((unsigned char*)(&data[k * project_frame_w * project_frame_h]), project_frame_w, project_frame_h);
				}
			}
		}
		input_ptr_render = new VFrame**[total_in_buffers];
	}

	if(total_out_buffers)
	{
		for(i = 0; i < total_out_buffers; i++)
		{
			output_ptr_master.append(new VFrame**[MAX_BUFFERS]);
			size = realtime_out_size.values[i];
			for(j = 0; j < double_buffers_out.values[i]; j++)
			{
				data = (VPixel*)data_out_realtime.values[i][j]->get_data();
				output_ptr_master.values[i][j] = new VFrame*[size];
				for(k = 0; k < size; k++)
				{
					output_ptr_master.values[i][j][k]
					 = new VFrame((unsigned char*)(&data[k * project_frame_w * project_frame_h]), project_frame_w, project_frame_h);
				}
			}
		}
		output_ptr_render = new VFrame**[total_out_buffers];
	}
	return 0;
}

// Run before every realtime buffer is to be rendered.
int PluginVClient::get_render_ptrs()
{
	int i, j, double_buffer, fragment_position;

	for(i = 0; i < total_in_buffers; i++)
	{
		double_buffer = double_buffer_in_render.values[i];
		fragment_position = offset_in_render.values[i];
		input_ptr_render[i] = &input_ptr_master.values[i][double_buffer][fragment_position];
	}

	for(i = 0; i < total_out_buffers; i++)
	{
		double_buffer = double_buffer_out_render.values[i];
		fragment_position = offset_out_render.values[i];
		output_ptr_render[i] = &output_ptr_master.values[i][double_buffer][fragment_position];
	}
	return 0;
}

int PluginVClient::init_nonrealtime_parameters()
{
	long i, j, k, size;
	VPixel *data;

	project_frame_rate = server->edl->session->frame_rate;
	project_frame_w = server->edl->session->track_w;
	project_frame_h = server->edl->session->track_h;
	project_color_model = server->edl->session->color_model;

	if(total_in_buffers)
	{
// Get an array of channels
		video_in = new VFrame**[total_in_buffers];

		for(i = 0; i < total_in_buffers; i++)
		{
// Get an array of frames
			video_in[i] = new VFrame*[in_buffer_size];
			data = (VPixel*)data_in[i]->get_data();

// Usually this only allocates 1 frame per channel.
			for(j = 0; j < in_buffer_size; j++)
			{
// Get the frame.
				video_in[i][j] = new VFrame((unsigned char*)(data + j * project_frame_w * project_frame_h), project_frame_w, project_frame_h);
			}
// for(j = 0; j < project_frame_h; j++)
// {
// 	printf("%d %p\n", j, video_in[i][0]->get_rows()[j]);
// }
		}
	}
	
	if(total_out_buffers)
	{
// Get an array of channels
		video_out = new VFrame**[total_out_buffers];

		for(i = 0; i < total_out_buffers; i++)
		{
// Get an array of frames
			video_out[i] = new VFrame*[out_buffer_size];
			data = (VPixel*)data_out[i]->get_data();
			
			for(j = 0; j < out_buffer_size; j++)
			{
// Get the frame.
				video_out[i][j] = new VFrame((unsigned char*)(data + j * project_frame_w * project_frame_h), project_frame_w, project_frame_h);
			}
		}
	}
	return 0;
}

// Run after the non realtime plugin is run.
int PluginVClient::delete_nonrealtime_parameters()
{
	int i, j;

	for(i = 0; i < total_in_buffers; i++)
	{
		for(j = 0; j < in_buffer_size; j++)
		{
			delete video_in[i][j];
		}
	}

	for(i = 0; i < total_out_buffers; i++)
	{
		for(j = 0; j < out_buffer_size; j++)
		{
			delete video_out[i][j];
		}
	}
	video_in = 0;
	video_out = 0;

	return 0;
}

// Run after the realtime plugin is done.
int PluginVClient::delete_buffer_ptrs()
{
	int i, j, k, size;
	if(total_in_buffers)
	{
		for(i = 0; i < total_in_buffers; i++)
		{
			for(j = 0; j < double_buffers_in.values[i]; i++)
			{
// delete the frames from the double buffer
				size = realtime_in_size.values[i];
				for(k = 0; k < size; k++)
				{
					delete input_ptr_master.values[i][j][k];
				}
// delete the double buffer from the channel
				delete input_ptr_master.values[i][j];
			}
// delete array of double buffers
			delete input_ptr_master.values[i];
		}
// delete all the channels
		input_ptr_master.remove_all();
		delete input_ptr_render;
	}

	if(total_out_buffers)
	{
		for(i = 0; i < total_out_buffers; i++)
		{
			for(j = 0; j < double_buffers_out.values[i]; i++)
			{
// delete the frames from the double buffer
				size = realtime_out_size.values[i];
				for(k = 0; k < size; k++)
				{
					delete output_ptr_master.values[i][j][k];
				}
// delete the double buffer from the channel
				delete output_ptr_master.values[i][j];
			}
			delete output_ptr_master.values[i];
		}
// delete all the channels
		output_ptr_master.remove_all();
		delete output_ptr_render;
	}
	return 0;
}

int PluginVClient::init_realtime_parameters()
{
	project_frame_rate = server->edl->session->frame_rate;
	project_frame_w = server->edl->session->track_w;
	project_frame_h = server->edl->session->track_h;
	project_color_model = server->edl->session->color_model;
	aspect_w = server->edl->session->aspect_w;
	aspect_h = server->edl->session->aspect_h;
	return 0;
}


void PluginVClient::plugin_process_realtime(VFrame **input, 
		VFrame **output, 
		long current_position,
		long total_len)
{
	this->source_position = current_position;
	this->total_len = total_len;

//printf("PluginVClient::plugin_process_realtime 1\n");
	if(is_multichannel())
		process_realtime(input, output);
	else
		process_realtime(input[0], output[0]);
//printf("PluginVClient::plugin_process_realtime 2\n");
}

int PluginVClient::plugin_process_loop(VFrame **buffers, long &write_length)
{
	write_length = 0;

	if(is_multichannel())
		return process_loop(buffers, write_length);
	else
		return process_loop(buffers[0], write_length);
}


int PluginVClient::read_frame(VFrame *buffer, int channel, long start_position)
{
	return server->read_frame(buffer, channel, start_position);
}

