#include "arender.h"
#include "assets.h"
#include "auto.h"
#include "bcprogressbox.h"
#include "cache.h"
#include "compresspopup.h"
#include "cwindow.h"
#include "cwindowgui.h"
#include "defaults.h"
#include "edits.h"
#include "edl.h"
#include "edlsession.h"
#include "errorbox.h"
#include "file.h"
#include "filesystem.h"
#include "formatcheck.h"
#include "formatpopup.h"
#include "formattools.h"
#include "labels.h"
#include "loadmode.h"
#include "mainprogress.h"
#include "mainundo.h"
#include "mwindow.h"
#include "mwindowgui.h"
#include "module.h"
#include "neworappend.h"
#include "patchbay.h"
#include "playabletracks.h"
#include "playbackengine.h"
#include "preferences.h"
#include "mainsession.h"
#include "timebar.h"
#include "tracks.h"
#include "transportque.h"
#include "quicktime.h"
#include "vrender.h"
#include "render.h"
#include "renderengine.h"
#include "vedit.h"
#include "vframe.h"
#include "videoconfig.h"
#include "videodevice.h"

#include <ctype.h>
#include <string.h>


RenderItem::RenderItem(MWindow *mwindow)
 : BC_MenuItem("Render...")
{
	this->mwindow = mwindow;
}

int RenderItem::handle_event() 
{
	mwindow->render_single();
	return 1;
}

RenderListItem::RenderListItem(MWindow *mwindow)
 : BC_MenuItem("Render list...")
{
	this->mwindow = mwindow;
}

int RenderListItem::handle_event() 
{
	mwindow->render_list();
	return 1;
}










Render::Render(MWindow *mwindow)
 : Thread()
{
	this->mwindow = mwindow;
	in_progress = 0;
}

Render::~Render()
{
}


int Render::check_asset(EDL *edl, Asset &asset)
{
	if(asset.video_data && 
		edl->tracks->playable_video_tracks() &&
		(asset.format == FILE_MOV || 
			asset.format == FILE_JPEG_LIST || 
			asset.format == FILE_PNG || 
			asset.format == FILE_JPEG || 
			asset.format == FILE_TIFF))
	{
		asset.video_data = 1;
		asset.layers = 1;
		asset.width = edl->session->output_w;
		asset.height = edl->session->output_h;
	}
	else
	{
		asset.video_data = 0;
		asset.layers = 0;
	}

	if(asset.audio_data && 
		edl->tracks->playable_audio_tracks())
	{
		asset.audio_data = 1;
		asset.channels = edl->session->audio_channels;
		if(asset.format == FILE_MOV) asset.byte_order = 0;
	}
	else
	{
		asset.audio_data = 0;
		asset.channels = 0;
	}
//printf("Render::check_asset 3\n");
}

void Render::run()
{
	int result = 0;
	int format_error;
	ArrayList<Asset*> assets;
	Asset default_asset;
	char string[BCTEXTLEN];
// Total selection to render in seconds
	double total_start, total_end;
// Total length in seconds
	double total_length;
// Length of region between labels
	double fragment_start;
	double fragment_end;
	int last_audio_buffer;
	Label *current_label = 0;
	VideoDevice *video_device = 0;
	RenderEngine *render_engine = 0;
	MainProgressBar *progress = 0;
	FileSystem fs;
	int total_digits;       // Total number of digits including padding the user specified.
	int number_start;       // Character in the filename path at which the number begins
	int current_number;    // The number the being injected into the filename.
	Preferences *preferences;
// Pointer from file
// (VFrame*)(VFrame array [])(Channel [])
	VFrame ***video_output;
	PluginBuffer *video_shm;
// Pointer to output buffers
	VFrame *video_output_ptr[MAX_CHANNELS];
	double *audio_output_ptr[MAX_CHANNELS];


// Fix the EDL for rendering
	preferences = new Preferences(mwindow);
	*preferences = *mwindow->preferences;
	TransportCommand *command = new TransportCommand;
	command->command = NORMAL_FWD;
	*command->get_edl() = *mwindow->edl;
	command->change_type = CHANGE_ALL;
	command->set_playback_range();


// Configure preview monitor
	VideoOutConfig vconfig(PLAYBACK_LOCALHOST, 0);
	PlaybackConfig *playback_config = new PlaybackConfig(PLAYBACK_LOCALHOST, 0);
	for(int i = 0; i < MAX_CHANNELS; i++)
	{
		vconfig.do_channel[i] = (i < command->get_edl()->session->video_channels);
		playback_config->vconfig->do_channel[i] = (i < command->get_edl()->session->video_channels);
		playback_config->aconfig->do_channel[i] = (i < command->get_edl()->session->audio_channels);
	}

// Create caches
	audio_cache = new Cache(command->get_edl(), mwindow->plugindb);
	video_cache = new Cache(command->get_edl(), mwindow->plugindb);

	load_defaults(&default_asset);
	default_asset.frame_rate = command->get_edl()->session->frame_rate;
	default_asset.sample_rate = command->get_edl()->session->sample_rate;
	check_asset(command->get_edl(), default_asset);

//printf("Render 1 %d %d %d\n", playback_config->vconfig->total_playable_channels(), asset.audio_data, default_asset.video_data);
// Get format from user
	do
	{
		format_error = 0;
		result = 0;

		{
			RenderWindow window(mwindow, this, &default_asset);
			window.create_objects();
			result = window.run_window();
		}

		if(!result)
		{
// Check the asset format for errors.
			FormatCheck format_check(&default_asset);
			format_error = format_check.check_format();
		}
	}while(format_error && !result);

//printf("Render 2\n");
	if(!result)
	{
		check_asset(command->get_edl(), default_asset);
		save_defaults(&default_asset);
		mwindow->save_defaults();

// Get total range to render
		total_start = command->start_position;
		total_end = command->end_position;
		total_length = total_end - total_start;

// Nothing to render
		if(total_length == 0)
		{
			result = 1;
		}
	}

//printf("Render 3 %f %f\n", total_start, total_end);
	if(!result)
	{
		fs.complete_path(default_asset.path);

		current_label = command->get_edl()->labels->first;
		get_starting_number(default_asset.path, 
			current_number,
			number_start, 
			total_digits);
//printf("Render 3\n");
//command->get_edl()->dump();

// Render the range in fragments
		for(fragment_start = total_start; 
			fragment_start < total_end && !result;
			fragment_start = fragment_end)
		{
// Get the length of this segment.
			if(render_list)
			{
//printf("Render 4 %p\n", current_label);
				while(current_label && current_label->position <= fragment_start)
						current_label = current_label->next;

//printf("Render 4 %p\n", current_label);
				if(!current_label)
					fragment_end = total_end;
				else
					fragment_end = current_label->position;
			}
			else
			{
//printf("Render 5\n");
				fragment_end = total_end;
			}
//printf("Render 6 %ld %ld %ld\n", fragment_start, fragment_end, total_end);

// Create asset
			Asset *asset = new Asset(default_asset);
			if(render_list) 
				Render::create_filename(asset->path, 
					default_asset.path, 
					current_number,
					total_digits,
					number_start);
			else
				strcpy(asset->path, default_asset.path);


//printf("Render 7 %s %d\n", asset->path, current_number);
			current_number++;
			assets.append(asset);
			File *file = new File;


// test existance of file
			result = test_existence(mwindow, asset);

//printf("Render 8\n");
// open output file
// use rd and wr to allow append
			if(!result)
			{
				file->set_processors(command->get_edl()->session->smp + 1);

//printf("Render 9\n");
				result = file->open_file(mwindow->plugindb, 
					asset, 
					0, 
					1, 
					command->get_edl()->session->sample_rate, 
					command->get_edl()->session->frame_rate);

//printf("Render 10\n");
				if(result)
				{
// open failed
					sprintf(string, "Couldn't open %s", asset->path);
					ErrorBox error(PROGRAM_NAME ": Error",
						mwindow->gui->get_abs_cursor_x(),
						mwindow->gui->get_abs_cursor_y());
					error.create_objects(string);
					error.run_window();
					in_progress = 0;
					result = 1;
				}
			}

//printf("Render 11\n");
			if(!result)
			{
// figure out the length to read at a time
				long audio_read_length = command->get_edl()->session->sample_rate;
				long video_read_length = 1;
				char filename[BCTEXTLEN];

				command->get_edl()->session->audio_module_fragment = 
					command->get_edl()->session->audio_read_length = 
					audio_read_length;

//printf("Render 12\n");
// Thread out the progress box
				fs.extract_name(filename, asset->path);
				sprintf(string, "Rendering %s...", filename);

				progress = mwindow->mainprogress->start_progress(string, 
					(long)(fragment_end - fragment_start));

//printf("Render 13\n");
// start the render engine
				if(render_engine) delete render_engine;
//printf("Render 13\n");
				render_engine = new RenderEngine(0,
					playback_config,
					preferences,
					command,
					0,
					mwindow->plugindb,
					&mwindow->channeldb_buz);
//printf("Render 13\n");
				render_engine->set_acache(audio_cache);
//printf("Render 13\n");
				render_engine->set_vcache(video_cache);
//printf("Render 14\n");
				render_engine->arm_command(command);

//printf("Render 15\n");
// audio_position is the master determining what frame we're on
				long audio_position = Units::to_long(fragment_start * asset->sample_rate);
				long video_position = Units::to_long(fragment_start * asset->frame_rate);
				double **audio_output;

//printf("Render 16\n");
// Create output buffers
				if(asset->audio_data)
				{
					file->start_audio_thread(audio_read_length, 
						command->get_edl()->session->smp ? 2 : 1);
				}

//printf("Render 17\n");
				if(asset->video_data)
				{
					compressed_output = new VFrame;
//printf("Render 18\n");
					file->start_video_thread(1,
						command->get_edl()->session->color_model,
						command->get_edl()->session->smp ? 2 : 1,
						0);
//printf("Render 14\n");
					video_device = new VideoDevice;
 					video_device->open_output(&vconfig, 
 						command->get_edl()->session->frame_rate, 
						command->get_edl()->session->output_w, 
						command->get_edl()->session->output_h, 
 						mwindow->cwindow->gui->canvas,
						1);
//printf("Render 14\n");
					video_device->start_playback();
//printf("Render 14\n");
				}
//printf("Render 18\n");

				playable_tracks = new PlayableTracks(render_engine, 
					vconfig.do_channel, 
					Units::to_long(fragment_start * command->get_edl()->session->frame_rate), 
					TRACK_VIDEO);
				last_audio_buffer = 0;
				direct_frame_copying = 0;
//printf("Render 19 audio_position=%ld fragment_end=%ld frame_rate=%f\n", 
//	audio_position, Units::to_long(fragment_end * asset->sample_rate + 0.5), command->get_edl()->session->frame_rate);

// ================================== render it
				while(audio_position < 
					Units::to_long(fragment_end * asset->sample_rate) && 
					!result &&
					!last_audio_buffer)
				{
//printf("Render 20\n");
					if(audio_position + audio_read_length > 
						(long)(fragment_end * asset->sample_rate + 0.5))
					{
						last_audio_buffer = 1;
						audio_read_length = 
							Units::to_long(fragment_end * asset->sample_rate - audio_position);
					}

//printf("Render 21\n");
// Do audio data
					if(asset->audio_data)
					{
						audio_output = file->get_audio_buffer();
//printf("Render 22\n");
// Zero unused channels in output vector
						for(int i = 0; i < MAX_CHANNELS; i++)
							audio_output_ptr[i] = (i < asset->channels) ? audio_output[i] : 0;
//printf("Render 24\n");
						render_engine->arender->process_buffer(audio_output_ptr, 
							audio_read_length, 
							audio_position, 
							last_audio_buffer);
//printf("Render 25\n");
						result = file->write_audio_buffer(audio_read_length);
					}
//printf("Render 26\n");

// Do video data
					audio_position += audio_read_length;
					if(asset->video_data)
					{
// get the absolute video position from the audio position
						long video_end = Units::to_long((double)audio_position / 
							asset->sample_rate * 
							asset->frame_rate);
						int last_video_buffer = 0;

//printf("Render 20 video_position=%ld audio_position=%ld video_end=%ld\n", 
//	video_position, audio_position, video_end);
						while(video_position < video_end && !result)
						{
							if(video_position == video_end - 1) 
								last_video_buffer = last_audio_buffer;

// Try to copy the compressed frame directly from the input to output files
//printf("Render 21 video_position=%ld\n", video_position);
							if(direct_frame_copy(command->get_edl(), video_position, file))
							{
// Direct frame copy failed.
//printf("Render 22\n");
// Switch back to background compression
								if(direct_frame_copying)
								{
									file->start_video_thread(1, 
										command->get_edl()->session->color_model,
										command->get_edl()->session->smp ? 2 : 1,
										0);
									direct_frame_copying = 0;
								}
//printf("Render 23\n");

// Try to use the rendering engine to write the frame.
// Get a buffer for background writing.
 								video_output = file->get_video_buffer();
//printf("Render 24\n");
								for(int i = 0; i < MAX_CHANNELS; i++)
									video_output_ptr[i] = (i < asset->layers) ? video_output[i][0] : 0;
//printf("Render 25\n");
 								render_engine->vrender->process_buffer(video_output_ptr, 
									video_shm, 
									video_position, 
									last_video_buffer);
//printf("Render 26\n");
// Flash the frame to video out
 								if(video_device->output_visible())
								{
// Vector for video device
									VFrame *preview_output[MAX_CHANNELS];
// Data for video device
									PluginBuffer *preview_buffer;
	
//printf("Render 27.1\n");
									video_device->new_output_buffers(preview_output,
										command->get_edl()->session->color_model, 
										preview_buffer);
//printf("Render 27.2\n");
									
									for(int i = 0; i < MAX_CHANNELS; i++)
										if(preview_output[i])
											preview_output[i]->copy_from(video_output_ptr[i]);
//printf("Render 27.3\n");
									video_device->write_buffer(preview_output, command->get_edl());
//printf("Render 27.4\n");
								}
// Write to file
 								result = file->write_video_buffer(1);
//printf("Render 27.5\n");
							}

							video_position++;
							if(!result) result = progress->is_cancelled();
						}
					}

					if(result && !progress->is_cancelled())
					{
						ErrorBox error(PROGRAM_NAME ": Error",
							mwindow->gui->get_abs_cursor_x(),
							mwindow->gui->get_abs_cursor_y());
						error.create_objects("Error writing data.");
						error.run_window();
					}
					else
					{
						result = progress->update((long)((audio_position - 
							fragment_start * asset->sample_rate) / 
							asset->sample_rate));
					}
				}

//printf("Render 28\n");
				render_engine->stop_playback();
//printf("Render 28\n");
				delete render_engine;
				render_engine = 0;

//printf("Render 29\n");
				delete playable_tracks;

//printf("Render 30\n");
// Delete output buffers
				if(asset->audio_data)
				{
// stop file I/O
					file->stop_audio_thread();
				}

//printf("Render 31\n");
				if(asset->video_data)
				{
					delete compressed_output;
//printf("Render 31.1\n");
					file->stop_video_thread();
//printf("Render 31.2\n");
					video_device->close_all();
//printf("Render 31.4\n");
					delete video_device;
//printf("Render 31.5\n");
				}

				progress->stop_progress();     // mandatory
				delete progress;
			}

//printf("Render 33\n");
			file->close_file();
			delete file;

			if(!result && load_mode != LOAD_NOTHING)
			{             // paste it in
				mwindow->undo->update_undo_edits("Render", 0);

// 				result = mwindow->paste_output(start, 
// 												end, 
// 												startsource_sample, 
// 												endsource_sample, 
// 												startsource_frame,
// 												endsource_frame, 
// 												&asset, 
// 												0);
				mwindow->undo->update_undo_edits();
				if(!result) mwindow->edl->session->changes_made = 1;
			}

			if(render_list)
			{
				fragment_start = fragment_end;
			}
		} // file_number
	} // !result

	assets.remove_all_objects();
	delete preferences;
	delete command;
	delete playback_config;
	delete audio_cache;
	delete video_cache;
	in_progress = 0;
}

int Render::test_existence(MWindow *mwindow, Asset *asset)
{
	int result = 0;
	NewOrAppend window(mwindow);
// Append is not compatible with resample plugin.
	result = window.test_file(asset);
	switch(result)
	{
		case 0:
//			mwindow->purge_asset(asset->path);
//			remove(asset->path);              // overwrite
//			append_to_file = 0;
			break;

		case 1:
			result = 1;                      // cancel
			break;

		case 2:
//					append_to_file = 1;              // append
			result = 0;       // reset result;
			break;
	}
	return result;
}


void Render::create_filename(char *path, 
	char *default_path, 
	int current_number,
	int total_digits,
	int number_start)
{
	int i, j, k;
	int len = strlen(default_path);
	char printf_string[BCTEXTLEN];
	int found_number = 0;

	for(i = 0, j = 0; i < number_start; i++, j++)
	{
		printf_string[j] = default_path[i];
	}

// Found the number
	sprintf(&printf_string[j], "%%0%dd", total_digits);
	j = strlen(printf_string);
	i += total_digits;

// Copy remainder of string
	for( ; i < len; i++, j++)
	{
		printf_string[j] = default_path[i];
	}
	printf_string[j] = 0;
// Print the printf argument to the path
	sprintf(path, printf_string, current_number);
}

void Render::get_starting_number(char *path, 
	int &current_number,
	int &number_start, 
	int &total_digits)
{
	int i, j;
	int len = strlen(path);
	char number_text[BCTEXTLEN];

	total_digits = 0;
	number_start = 0;

// Search for last number
	for(i = len - 1; i >= 0; i--)
	{
		if(isdigit(path[i]))
			break;
	}

// Search for beginning of last number
	while(i >= 0 && isdigit(path[i]))
	{
		total_digits++;
		i--;
	}

	i++;
	if(i >= 0 && isdigit(path[i]))
	{
		number_start = i;
// Store the first number
		j = 0;
		while(isdigit(path[i]))
			number_text[j++] = path[i++];
		number_text[j] = 0;
		current_number = atol(number_text);
	}
	else
// No number found
	{
		current_number = 1;
		number_start = len;
		total_digits = 2;
	}
}


// Try to copy the compressed frame directly from the input to output files
// Return 1 on failure and 0 on success
int Render::direct_frame_copy(EDL *edl, long &video_position, File *file)
{
	Track *playable_track;
	Edit *playable_edit;
	long frame_size;
	int result = 0;

//printf("Render::direct_frame_copy 1\n");
	if(direct_copy_possible(edl, 
		video_position, 
		playable_track, 
		playable_edit, 
		file))
	{
// Switch to direct copying
		if(!direct_frame_copying)
		{
			file->stop_video_thread();
			direct_frame_copying = 1;
		}
//printf("Render::direct_frame_copy 2\n");

		result = ((VEdit*)playable_edit)->read_frame(compressed_output, 
			video_position,
			PLAY_FORWARD,
			video_cache);

//printf("Render::direct_frame_copy 3 %d\n", compressed_output->get_compressed_size());
		if(!result)
		{
			VFrame ***temp_output = new VFrame**[1];
			temp_output[0] = new VFrame*[1];
			temp_output[0][0] = compressed_output;
			result = file->write_frames(temp_output, 1);
			delete temp_output[0];
			delete temp_output;
		}
//printf("Render::direct_frame_copy 4\n");
		return result;
	}
	else
		return 1;
}

int Render::direct_copy_possible(EDL *edl,
				long current_position, 
				Track* playable_track,  // The one track which is playable
				Edit* &playable_edit, // The edit which is playing
				File *file)   // Output file
{
	int result = 1;
	int total_playable_tracks = 0;
	Track* current_track;
	Patch* current_patch;
	Auto* current_auto;
	int temp;

// Number of playable tracks must equal 1
	for(current_track = edl->tracks->first;
		current_track && result; 
		current_track = current_track->next)
	{
		if(current_track->data_type == TRACK_VIDEO)
		{
			if(playable_tracks->is_playable(current_track, current_position))
			{
				playable_track = current_track;
				total_playable_tracks++;
			}
		}
	}

//printf("Render::direct_copy_possible 1 %d\n", result);
	if(total_playable_tracks != 1) result = 0;
//printf("Render::direct_copy_possible 2 %d\n", result);

// Edit must have a source file
	if(result)
	{
//printf("Render::direct_copy_possible 3 %d\n", result);
		playable_edit = playable_track->edits->get_playable_edit(current_position);
//printf("Render::direct_copy_possible 4 %d %p\n", result, playable_edit);
		if(!playable_edit)
			result = 0;
	}

// Source file must be able to copy to destination file.
// Source file must be same size as project output.
	if(result)
	{
//printf("Render::direct_copy_possible 5 %d\n", result);
		if(!file->can_copy_from(playable_edit->asset, 
			edl->session->output_w, 
			edl->session->output_h))
			result = 0;
	}
//printf("Render::direct_copy_possible 6 %d\n", result);

// Test conditions mutual between vrender.C and this.
	if(result && !playable_track->direct_copy_possible(current_position, PLAY_FORWARD))
		result = 0;
//printf("Render::direct_copy_possible 7 %d\n", result);

	return result;
}

int Render::load_defaults(Asset *asset)
{
	mwindow->defaults->get("RENDER_PATH", asset->path);
//	to_tracks = mwindow->defaults->get("RENDER_TO_TRACKS", 1);
	render_list = mwindow->defaults->get("RENDER_LIST", 1);
	load_mode = mwindow->defaults->update("RENDER_LOADMODE", LOAD_NEW_TRACKS);
	asset->audio_data = mwindow->defaults->get("RENDER_AUDIO", 1);
	asset->video_data = mwindow->defaults->get("RENDER_VIDEO", 1);
//printf("Render::load_defaults 1 %d %d\n", asset->audio_data, asset->video_data);

	char string[BCTEXTLEN];
	sprintf(string, "WAV");
	mwindow->defaults->get("RENDER_FORMAT", string);
	sprintf(asset->compression, QUICKTIME_YUV2);
	mwindow->defaults->get("RENDER_COMPRESSION", asset->compression);
	asset->quality = mwindow->defaults->get("RENDER_QUALITY", 100);

	asset->format = File::strtoformat(mwindow->plugindb, string);

	asset->bits = mwindow->defaults->get("RENDER_BITS", 16);
	asset->dither = mwindow->defaults->get("RENDER_DITHER", 0);
	asset->signed_ = mwindow->defaults->get("RENDER_SIGNED", 1);
	asset->byte_order = mwindow->defaults->get("RENDER_BYTE_ORDER", 1);
	return 0;
}

int Render::save_defaults(Asset *asset)
{
//printf("Render::save_defaults 1\n");
	mwindow->defaults->update("RENDER_PATH", asset->path);
//	mwindow->defaults->update("RENDER_TO_TRACKS", to_tracks);
	mwindow->defaults->update("RENDER_LIST", render_list);
	mwindow->defaults->update("RENDER_LOADMODE", load_mode);
	mwindow->defaults->update("RENDER_AUDIO", asset->audio_data);
	mwindow->defaults->update("RENDER_VIDEO", asset->video_data);
//printf("Render::save_defaults 2\n");

	mwindow->defaults->update("RENDER_FORMAT", File::formattostr(mwindow->plugindb, asset->format));
//printf("Render::save_defaults 3\n");

	mwindow->defaults->update("RENDER_COMPRESSION", asset->compression);
	mwindow->defaults->update("RENDER_QUALITY", asset->quality);
	mwindow->defaults->update("RENDER_BITS", asset->bits);
	mwindow->defaults->update("RENDER_DITHER", asset->dither);
	mwindow->defaults->update("RENDER_SIGNED", asset->signed_);
	mwindow->defaults->update("RENDER_BYTE_ORDER", asset->byte_order);
//printf("Render::save_defaults 4\n");
	return 0;
}




#define WIDTH 410
#define HEIGHT 360


RenderWindow::RenderWindow(MWindow *mwindow, Render *render, Asset *asset)
 : BC_Window(PROGRAM_NAME ": Render", 
 	mwindow->gui->get_root_w() / 2 - WIDTH / 2,
	mwindow->gui->get_root_h() / 2 - HEIGHT / 2,
 	WIDTH, 
	HEIGHT,
	(int)INFINITY,
	(int)INFINITY,
	0,
	0,
	1)
{
	this->mwindow = mwindow;
	this->render = render;
	this->asset = asset;
}

RenderWindow::~RenderWindow()
{
	delete format_tools;
	delete loadmode;
}



int RenderWindow::create_objects()
{
	int x = 5, y = 5;
	add_subwindow(new BC_Title(x, y, 
		(char*)(render->render_list ? "Select the first file to render to:" : "Select a file to render to:")));
	y += 25;

	format_tools = new FormatTools(mwindow,
					this, 
					asset);
	format_tools->create_objects(x, 
		y, 
		1, 
		1, 
		1, 
		1, 
		0,
		1,
		0,
		1,
		0,
		&render->render_list);

	loadmode = new LoadMode(this, x, y, &render->load_mode, 1);
	loadmode->create_objects();

	add_subwindow(new BC_OKButton(this));
	add_subwindow(new BC_CancelButton(this));
	show_window();
	return 0;
}


