#ifndef RENDER_H
#define RENDER_H


#include "guicast.h"
#include "bitspopup.h"
#include "browsebutton.h"
#include "compresspopup.h"
#include "defaults.inc"
#include "edit.inc"
#include "errorbox.inc"
#include "file.inc"
#include "formatpopup.inc"
#include "formattools.inc"
#include "mwindow.inc"
#include "maxchannels.h"
#include "playabletracks.inc"
#include "bcprogressbox.inc"
#include "render.inc"
#include "track.inc"
#include "vframe.inc"

class RenderItem : public BC_MenuItem
{
public:
	RenderItem(MWindow *mwindow);
	int handle_event();
	MWindow *mwindow;
};

class RenderListItem : public BC_MenuItem
{
public:
	RenderListItem(MWindow *mwindow);
	int handle_event();
	MWindow *mwindow;
};

class Render : public Thread
{
public:
	Render(MWindow *mwindow);
	~Render();
	
	void run();
	int load_defaults(Asset *asset);
	int save_defaults(Asset *asset);
// force asset parameters regardless of window
// This should be integrated into the Asset Class.
	int check_asset(EDL *edl, Asset &asset); 
// Force filename to have a 0 padded number if rendering to a list.
	int check_numbering(Asset &asset);
	static void create_filename(char *path, 
		char *default_path, 
		int current_number,
		int total_digits,
		int number_start);
	static void get_starting_number(char *path, 
		int &current_number,
		int &number_start, 
		int &total_digits);
	static int test_existence(MWindow *mwindow, Asset *asset);
	int direct_frame_copy(EDL *edl, long &render_video_position, File *file);
	int direct_copy_possible(EDL *edl, 
			long current_position, 
			Track* playable_track,  // The one track which is playable
			Edit* &playable_edit, // The edit which is playing
			File *file);   // Output file

	int load_mode;
	int in_progress;
// Background compression must be disabled when direct frame copying and reenabled afterwards
	int direct_frame_copying;
	int render_list;          // Whether or not this render instance is for rendering a list of segments

	Cache *audio_cache, *video_cache;
	VFrame *compressed_output;
	VFrame *uncompressed_output;
	MWindow *mwindow;
	PlayableTracks *playable_tracks;
};

class RenderToTracks;

class RenderWindow : public BC_Window
{
public:
	RenderWindow(MWindow *mwindow, Render *render, Asset *asset);
	~RenderWindow();

	int create_objects();


	LoadMode *loadmode;
	FormatTools *format_tools;

	MWindow *mwindow;
	Render *render;
	Asset *asset;
};

#endif
