#include "amodule.h"
#include "arender.h"
#include "audiodevice.h"
#include "edl.h"
#include "edlsession.h"
#include "mwindow.h"
#include "playbackengine.h"
#include "preferences.h"
#include "renderengine.h"
#include "mainsession.h"
#include "tracks.h"
#include "transportque.h"
#include "videodevice.h"
#include "vrender.h"




RenderEngine::RenderEngine(PlaybackEngine *playback_engine,
	PlaybackConfig *config, 
 	Preferences *preferences, 
	TransportCommand *command,
	Canvas *output,
	ArrayList<PluginServer*> *plugindb,
	ArrayList<Channel*> *channeldb)
 : Thread()
{
	this->playback_engine = playback_engine;
	this->output = output;
	this->plugindb = plugindb;
	this->channeldb = channeldb;
	set_synchronous(1);
	audio = 0;
	video = 0;
	arender = 0;
	vrender = 0;
	do_audio = 0;
	do_video = 0;
	interrupted = 0;
	actual_frame_rate = 0;
	this->config = new PlaybackConfig(config->playback_strategy, config->engine_number);
 	this->preferences = new Preferences(0);
 	this->command = new TransportCommand;
 	*this->config = *config;
 	*this->preferences = *preferences;
 	*this->command = *command;
	edl = new EDL;
	edl->create_objects();
// EDL only changed in construction
	*this->edl = *command->get_edl();
	audio_cache = 0;
	video_cache = 0;
}

RenderEngine::~RenderEngine()
{
//printf("RenderEngine::~RenderEngine 1\n");
	close_output();
//printf("RenderEngine::~RenderEngine 2\n");
	delete command;
//printf("RenderEngine::~RenderEngine 3\n");
	delete config;
//printf("RenderEngine::~RenderEngine 4\n");
	delete preferences;
//printf("RenderEngine::~RenderEngine 5\n");
	if(arender) delete arender;
//printf("RenderEngine::~RenderEngine 6\n");
	if(vrender) delete vrender;
	delete edl;
//printf("RenderEngine::~RenderEngine 7\n");
}

int RenderEngine::arm_command(TransportCommand *command)
{
// Prevent this renderengine from accepting another command until finished.
// Since the renderengine is often deleted after the input_lock command it must
// be locked here as well as in the calling routine.
//printf("RenderEngine::arm_command 1\n");
	input_lock.lock();
//printf("RenderEngine::arm_command 2\n");
	*this->command = *command;
//printf("RenderEngine::arm_command 3 %p %p\n", this->edl, command->get_edl());
	done = 0;
	interrupted = 0;

//printf("RenderEngine::arm_command 4 %f %f\n", this->command->playbackstart, command->playbackstart);
	get_duty();

//edl->dump();
	if(do_audio)
	{
// Larger of audio_module_fragment and fragment length adjusted for speed
		adjusted_fragment_len = (long)((float)edl->session->audio_module_fragment / 
			command->get_speed() + 0.5);
		if(adjusted_fragment_len < edl->session->audio_module_fragment)
			adjusted_fragment_len = edl->session->audio_module_fragment;
	}

//printf("RenderEngine::arm_command 5 %d %d\n", do_audio, do_video);
	open_output();
//printf("RenderEngine::arm_command 5 %d %d\n", do_audio, do_video);
	create_render_threads();
//printf("RenderEngine::arm_command 5 %d %d\n", do_audio, do_video);
	arm_render_threads();
//printf("RenderEngine::arm_command 6\n");
	return 0;
}

void RenderEngine::get_duty()
{
	do_audio = 0;
	do_video = 0;
//printf("RenderEngine::get_duty 1 %d %d\n", edl->tracks->playable_audio_tracks(), config->vconfig->total_playable_channels());
	if(!command->single_frame() &&
		edl->tracks->playable_audio_tracks() &&
		config->aconfig->total_playable_channels())
	{
		do_audio = 1;
	}

//printf("RenderEngine::get_duty 1 %d %d\n", edl->tracks->playable_video_tracks(), config->vconfig->total_playable_channels());
	if(edl->tracks->playable_video_tracks() &&
		config->vconfig->total_playable_channels())
	{
		do_video = 1;
	}
}

void RenderEngine::create_render_threads()
{
	if(do_video && !vrender)
	{
		vrender = new VRender(this);
	}

	if(do_audio && !arender)
	{
		arender = new ARender(this);
	}
}


int RenderEngine::get_output_w()
{
	return edl->calculate_output_w(config->playback_strategy != PLAYBACK_LOCALHOST);
}

int RenderEngine::get_output_h()
{
	return edl->calculate_output_h(config->playback_strategy != PLAYBACK_LOCALHOST);
}

Channel* RenderEngine::get_current_channel()
{
	if(config->vconfig->buz_out_channel >= 0 && 
		config->vconfig->buz_out_channel < channeldb->total)
	{
		return channeldb->values[config->vconfig->buz_out_channel];
	}
	return 0;
}

Cache* RenderEngine::get_acache()
{
	if(playback_engine)
		return playback_engine->audio_cache;
	else
		return audio_cache;
}

Cache* RenderEngine::get_vcache()
{
	if(playback_engine)
		return playback_engine->video_cache;
	else
		return video_cache;
}

void RenderEngine::set_acache(Cache *cache)
{
	this->audio_cache = cache;
}

void RenderEngine::set_vcache(Cache *cache)
{
	this->video_cache = cache;
}


double RenderEngine::get_tracking_position()
{
	if(playback_engine) 
		return playback_engine->get_tracking_position();
	else
		return 0;
}

int RenderEngine::open_output()
{
//printf("RenderEngine::open_output 1 %d %d\n", do_audio, do_video);
	if(command->realtime)
	{
		if(do_audio)
		{
			audio = new AudioDevice;
//printf("RenderEngine::open_output 2 %d %d\n", do_audio, do_video);
			audio->open_output(config->aconfig, 
				edl->session->sample_rate, 
				adjusted_fragment_len,
				edl->session->real_time_playback);
			audio->set_software_positioning(edl->session->playback_software_position);
			audio->start_playback();
//printf("RenderEngine::open_output 3 %d %d\n", do_audio, do_video);
		}

		if(do_video)
		{
			video = new VideoDevice;
//printf("RenderEngine::open_output 1 %d\n", config->vconfig->driver);
			video->open_output(config->vconfig, 
				edl->session->frame_rate,
				get_output_w(),
				get_output_h(),
				output,
				command->single_frame());
			Channel *channel = get_current_channel();
			if(channel) video->set_channel(channel);
			video->set_quality(80);
			video->set_cpus(edl->session->smp + 1);
		}
	}
	return 0;
}

long RenderEngine::session_position()
{
	if(do_audio)
	{
		return audio->current_position();
	}

	if(do_video)
	{
		return (long)((double)vrender->session_frame / 
				edl->session->frame_rate * 
				edl->session->sample_rate /
				command->get_speed() + 0.5);
	}
}

long RenderEngine::sync_position()
{
	switch(edl->session->playback_strategy)
	{
		case PLAYBACK_LOCALHOST:
		case PLAYBACK_MULTIHEAD:
// Use audio device
// No danger of race conditions because the output devices are closed after all
// threads join.
//printf("RenderEngine::sync_position 1\n");
			if(do_audio)
			{
				return audio->current_position();
			}

//printf("RenderEngine::sync_position 2\n");
			if(do_video)
			{
				return timer.get_scaled_difference(edl->session->sample_rate);
			}
			break;

		case PLAYBACK_BLONDSYMPHONY:
			break;
	}
}

PluginServer* RenderEngine::scan_plugindb(char *title)
{
	for(int i = 0; i < plugindb->total; i++)
	{
		if(!strcasecmp(plugindb->values[i]->title, title)) return plugindb->values[i];
	}
	return 0;
}

int RenderEngine::start_command()
{
//printf("RenderEngine::start_command 1 %d\n", command->realtime);
	if(command->realtime)
	{
		interrupt_lock.lock();
		start_lock.lock();
		Thread::start();
		start_lock.lock();
		start_lock.unlock();
//printf("RenderEngine::start_command 2 %p %d\n", this, Thread::get_tid());
	}
	return 0;
}

void RenderEngine::arm_render_threads()
{
//printf("RenderEngine::arm_render_threads 1\n");
	if(do_audio)
	{
		arender->arm_command();
	}

	if(do_video)
	{
//printf("RenderEngine::arm_render_threads 1 %d %d\n", do_audio, do_video);
		vrender->arm_command();
//printf("RenderEngine::arm_render_threads 2 %d %d\n", do_audio, do_video);
	}
}


void RenderEngine::start_render_threads()
{
// Synchronization timer
	timer.update();

	if(do_audio)
	{
		arender->start_command();
	}

	if(do_video)
	{
		vrender->start_command();
	}
}

void RenderEngine::wait_render_threads()
{
	if(do_audio)
	{
		arender->Thread::join();
	}
//printf("RenderEngine::wait_render_threads 1\n");
	if(do_video)
	{
		vrender->Thread::join();
	}
//printf("RenderEngine::wait_render_threads 2\n");
}

int RenderEngine::wait_for_completion()
{
	input_lock.lock();
	input_lock.unlock();
	return 0;
}

void RenderEngine::interrupt_playback()
{
//printf("RenderEngine::interrupt_playback 0 %p\n", this);
	interrupt_lock.lock();
	interrupted = 1;
	if(audio)
	{
//printf("RenderEngine::interrupt_playback 1 %p\n", this);
		audio->interrupt_playback();
//printf("RenderEngine::interrupt_playback 2 %p\n", this);
	}
	if(video)
	{
//printf("RenderEngine::interrupt_playback 3 %p\n", this);
		video->interrupt_playback();
//printf("RenderEngine::interrupt_playback 4 %p\n", this);
	}
	interrupt_lock.unlock();
}

int RenderEngine::close_output()
{
//printf("RenderEngine::close_output 1\n");
	if(audio)
	{
		audio->close_all();
		delete audio;
		audio = 0;
	}
	if(video)
	{
		video->close_all();
//printf("RenderEngine::close_output 2\n");
		delete video;
//printf("RenderEngine::close_output 3\n");
		video = 0;
	}
//printf("RenderEngine::close_output 4\n");
	return 0;
}

void RenderEngine::get_output_levels(double *levels, long position)
{
	if(do_audio)
	{
		int history_entry = arender->get_history_number(arender->level_samples, position);
//printf("RenderEngine::get_output_levels %d\n", history_entry);
		for(int i = 0; i < MAXCHANNELS; i++)
		{
			if(arender->audio_out[i])
				levels[i] = arender->level_history[i][history_entry];
		}
	}
}

void RenderEngine::get_module_levels(ArrayList<double> *module_levels, long position)
{
	if(do_audio)
	{
		for(int i = 0; i < arender->total_modules; i++)
		{
//printf("RenderEngine::get_module_levels %p %p\n", ((AModule*)arender->modules[i]), ((AModule*)arender->modules[i])->level_samples);
			int history_entry = arender->get_history_number(((AModule*)arender->modules[i])->level_samples, position);

			module_levels->append(((AModule*)arender->modules[i])->level_history[history_entry]);
		}
	}
}





void RenderEngine::run()
{
//printf("RenderEngine::run 1 %p\n", this);
	start_render_threads();
	start_lock.unlock();
	interrupt_lock.unlock();

//printf("RenderEngine::run 2\n");
	wait_render_threads();

	interrupt_lock.lock();

	if(interrupted)
	{
		playback_engine->tracking_position = playback_engine->get_tracking_position();
	}

//printf("RenderEngine::run 3\n");
	close_output();
//printf("RenderEngine::run 4\n");

// Fix the tracking position
	if(playback_engine)
	{
		if(command->command == CURRENT_FRAME)
		{
//printf("RenderEngine::run 4.1 %d\n", playback_engine->tracking_position);
			playback_engine->tracking_position = command->playbackstart;
		}
		else
		{
// Make sure transport doesn't issue a pause command next
//printf("RenderEngine::run 4.1 %d\n", playback_engine->tracking_position);
			if(!interrupted)
			{
				if(do_audio)
					playback_engine->tracking_position = 
						(double)arender->current_position / 
							command->get_edl()->session->sample_rate;
				else
				if(do_video)
				{
					playback_engine->tracking_position = 
						(double)vrender->current_position / 
							command->get_edl()->session->frame_rate;
				}
			}

			if(!interrupted) playback_engine->command->command = STOP;
//printf("RenderEngine::run 4.3 %f\n", playback_engine->tracking_position);
			playback_engine->stop_tracking();

//printf("RenderEngine::run 4.4\n");
		}
		playback_engine->is_playing_back = 0;
	}

//printf("RenderEngine::run 5\n");
	input_lock.unlock();
//printf("RenderEngine::run 6\n");
	interrupt_lock.unlock();
//printf("RenderEngine::run 7 %p\n", this);
}

























int RenderEngine::reset_parameters()
{
	start_position = 0;
	follow_loop = 0;
	end_position = 0;
	infinite = 0;
	reverse = 0;
	start_position = 0;
	audio_channels = 0;
	do_audio = 0;
	do_video = 0;
	done = 0;
}

int RenderEngine::arm_playback_common(long start_sample, 
			long end_sample,
			long current_sample,
			int reverse, 
			float speed, 
			int follow_loop,
			int infinite)
{
	this->follow_loop = follow_loop;
	this->start_position = start_sample;
	this->end_position = end_sample;
	this->reverse = reverse;
	this->infinite = infinite;
	this->current_sample = current_sample;
	if(infinite) this->follow_loop = 0;
}

int RenderEngine::arm_playback_audio(long input_length, 
			long amodule_render_fragment, 
			long playback_buffer, 
			long output_length, 
			int audio_channels)
{
	this->audio_channels = audio_channels;

	do_audio = 1;

	arender = new ARender(mwindow, this);
	arender->arm_playback(current_sample, 
							input_length, 
							amodule_render_fragment, 
							playback_buffer, 
							output_length);
}

int RenderEngine::arm_playback_video(int every_frame, 
			long read_length, 
			long output_length,
			int track_w,
			int track_h,
			int output_w,
			int output_h)
{
	do_video = 1;
	this->every_frame = every_frame;

	vrender = new VRender(mwindow, this);
// 	vrender->arm_playback(current_sample, 
// 							read_length, 
// 							output_length, 
// 							output_length, 
// 							track_w,
// 							track_h,
// 							output_w,
// 							output_h);
}

int RenderEngine::start_video()
{
// start video for realtime
	if(video) video->start_playback();
	vrender->start_playback();
}

// REMOVE
int RenderEngine::stop_playback()
{
return 0;
printf("RenderEngine::stop_playback 1\n");
	if(do_audio || do_video)
	{
		follow_loop = 0;
		reverse = 0;
	}

printf("RenderEngine::stop_playback 1\n");
	if(do_audio)
	{
		arender->stop_rendering();
		delete arender;
		arender = 0;
	}

printf("RenderEngine::stop_playback 1\n");
	if(do_video)
	{
		vrender->stop_rendering();
		delete vrender;
	}

printf("RenderEngine::stop_playback 2\n");
	do_audio = 0;
	do_video = 0;
}


long RenderEngine::get_correction_factor(int reset)
{
	if(!every_frame)
	{
		long x;
//		x = playbackengine->correction_factor;
//		if(reset) playbackengine->correction_factor = 0;
		return x;
	}
	else
		return 0;
}

int RenderEngine::wait_for_startup()
{
	if(do_audio) arender->wait_for_startup();
	if(do_video) vrender->wait_for_startup();
}
