#include "aedit.h"
#include "assets.h"
#include "assets.inc"
#include "cache.h"
#include "clip.h"
#include "colormodels.h"
#include "datatype.h"
#include "edit.h"
#include "edits.h"
#include "edl.h"
#include "edlsession.h"
#include "file.h"
#include "filesystem.h"
#include "indexfile.h"
#include "mwindow.h"
#include "pluginbuffer.h"
#include "resourcepixmap.h"
#include "theme.h"
#include "track.h"
#include "trackcanvas.h"
#include "vedit.h"
#include "vframe.h"


ResourcePixmap::ResourcePixmap(MWindow *mwindow, 
	TrackCanvas *canvas, 
	Edit *edit, 
	int w, 
	int h)
 : BC_Pixmap(canvas, w, h)
{
	reset();

	this->mwindow = mwindow;
	this->canvas = canvas;
	startsource = edit->startsource;
	data_type = edit->track->data_type;
	source_framerate = edit->asset->frame_rate;
	project_framerate = edit->edl->session->frame_rate;
	source_samplerate = edit->asset->sample_rate;
	project_samplerate = edit->edl->session->sample_rate;
	edit_id = edit->id;
}

ResourcePixmap::~ResourcePixmap()
{
}


void ResourcePixmap::reset()
{
	edit_x = 0;
	pixmap_x = 0;
	pixmap_w = 0;
	pixmap_h = 0;
	zoom_sample = 0;
	zoom_track = 0;
	zoom_y = 0;
	visible = 1;
}
	
void ResourcePixmap::resize(int w, int h)
{
	int new_w = (w > get_w()) ? w : get_w();
	int new_h = (h > get_h()) ? h : get_h();

	BC_Pixmap::resize(new_w, new_h);
}

void ResourcePixmap::draw_data(Edit *edit,
	long edit_x,
	long edit_w, 
	long pixmap_x, 
	long pixmap_w,
	long pixmap_h,
	int force)
{
// Get new areas to fill in relative to pixmap
// Left side expansion
	int new_x1 = 0, new_w1 = 0;
// Right side expansion
	int new_x2 = 0, new_w2 = 0;
//printf("ResourcePixmap::draw_data 1\n");
	int y = 0;
	if(mwindow->edl->session->show_titles) y += mwindow->theme->title_bg_data->get_h();
	Track *track = edit->edits->track;

// Zoom changed or source position changes
	if(edit->startsource != this->startsource ||
		(data_type == TRACK_AUDIO && edit->asset->sample_rate != source_samplerate) ||
		(data_type == TRACK_VIDEO && !EQUIV(edit->asset->frame_rate, source_framerate)) ||
		mwindow->edl->session->sample_rate != project_samplerate ||
		mwindow->edl->session->frame_rate != project_framerate ||
		mwindow->edl->session->zoom_sample != zoom_sample || 
		mwindow->edl->session->zoom_track != zoom_track ||
		this->pixmap_h != pixmap_h ||
		(data_type == TRACK_AUDIO && 
		mwindow->edl->session->zoom_y != zoom_y) ||
		force)
	{
		new_x1 = 0;
		new_w1 = pixmap_w;
	}
	else
	{
// Coords relative to start of edit
		long new_pixmap_x1 = pixmap_x - edit_x;
		long new_pixmap_x2 = pixmap_w + new_pixmap_x1;
		long old_pixmap_x1 = this->pixmap_x - this->edit_x;
		long old_pixmap_x2 = this->pixmap_w + old_pixmap_x1;

// Left side exposed
		if(new_pixmap_x1 < old_pixmap_x1)
		{
			new_x1 = 0;
			new_w1 = old_pixmap_x1 - new_pixmap_x1;
// Moved completely off the pixmap
			if(new_w1 > pixmap_w)
			{
				new_w1 = pixmap_w;
			}
			else
			{
				copy_area(0, 
					y, 
					this->pixmap_w - new_w1, 
					mwindow->edl->session->zoom_track, 
					new_w1, 
					y);
			}
		}
		else
// Left side covered
		if(new_pixmap_x1 > old_pixmap_x1)
		{
// Moved completely off pixmap
			if(new_pixmap_x1 - old_pixmap_x1 > pixmap_w)
			{
				;
			}
			else
			{
				copy_area(new_pixmap_x1 - old_pixmap_x1, 
					y, 
					this->pixmap_w, 
					mwindow->edl->session->zoom_track, 
					0, 
					y);
			}
		}

		this->edit_x = edit_x;
		this->pixmap_x = pixmap_x;
		this->pixmap_w = pixmap_w;

// Right side exposed
		if(new_pixmap_x2 > old_pixmap_x2)
		{
			new_x2 = this->pixmap_w - (new_pixmap_x2 - old_pixmap_x2);
			if(new_x2 < 0) new_x2 = 0;
			new_w2 = this->pixmap_w - new_x2;
			if(new_w2 < 0) new_w2 = 0;
//printf("ResourcePixmap::draw_data %d %d\n", new_x2, new_w2);
		}
		else
// Right side covered
		if(new_pixmap_x2 < old_pixmap_x2)
		{
// Shift already done in left side exposed
		}
	}

// Update pixmap settings
	this->edit_id = edit->id;
	this->startsource = edit->startsource;
	this->source_framerate = edit->asset->frame_rate;
	this->source_samplerate = edit->asset->sample_rate;
	this->project_framerate = edit->edl->session->frame_rate;
	this->project_samplerate = edit->edl->session->sample_rate;
	this->edit_x = edit_x;
	this->pixmap_x = pixmap_x;
	this->pixmap_w = pixmap_w;
	this->pixmap_h = pixmap_h;
	this->zoom_sample = mwindow->edl->session->zoom_sample;
	this->zoom_track = mwindow->edl->session->zoom_track;
//printf("ResourcePixmap::draw_data 1 pixmap_w: %d pixmap_h: %d new_x1: %d new_w1: %d new_x2: %d new_w2: %d\n", 
// 	pixmap_w, pixmap_h, new_x1, new_w1, new_x2, new_w2);

// Draw in new areas
	mwindow->theme->draw_resource_bg(canvas,
		mwindow, 
		this, 
		new_x1, 
		y,
		new_x1 + new_w1,
		mwindow->edl->session->zoom_track + y);
	mwindow->theme->draw_resource_bg(canvas,
		mwindow, 
		this, 
		new_x2, 
		y,
		new_x2 + new_w2,
		mwindow->edl->session->zoom_track + y);

	if(track->draw)
	{
		switch(track->data_type)
		{
			case TRACK_AUDIO:
//printf("ResourcePixmap::draw_data 1\n");
				draw_audio_resource(edit, new_x1, new_w1);
//printf("ResourcePixmap::draw_data 2\n");
				draw_audio_resource(edit, new_x2, new_w2);
//printf("ResourcePixmap::draw_data 3\n");
				break;

			case TRACK_VIDEO:
//printf("ResourcePixmap::draw_data 4\n");
				draw_video_resource(edit, edit_x, edit_w, new_x1, new_w1);
//printf("ResourcePixmap::draw_data 5\n");
				draw_video_resource(edit, edit_x, edit_w, new_x2, new_w2);
//printf("ResourcePixmap::draw_data 6\n");
				break;
		}
	}

// Draw title
	if(mwindow->edl->session->show_titles)
		draw_title(edit, edit_x, edit_w, pixmap_x, pixmap_w);

//printf("new_x1 %d new_w1 %d new_x2 %d new_w2 %d\n", new_x1, new_w1, new_x2, new_w2);
}

void ResourcePixmap::draw_title(Edit *edit,
	long edit_x, 
	long edit_w, 
	long pixmap_x, 
	long pixmap_w)
{
// border coords relative to pixmap
	long border_x1 = edit_x - pixmap_x;
	long border_x2 = edit_x + edit_w - pixmap_x;

	if(border_x1 < 0) border_x1 = -10;
	if(border_x2 > pixmap_w) border_x2 = pixmap_w + 10;

	canvas->draw_3segment(border_x1, 
		0, 
		border_x2 - border_x1, 
		mwindow->theme->title_bg_data,
		this);

	long title_x1 = edit_x - pixmap_x;
	char title[BCTEXTLEN];
	FileSystem fs;
	fs.extract_name(title, edit->asset->path);

	if(title_x1 > -INFINITY)
	{
		canvas->set_color(WHITE);
		canvas->set_font(MEDIUMFONT_3D);
		canvas->draw_text(title_x1 + 10, 
			canvas->get_text_ascent(MEDIUMFONT_3D) + 2, 
			title,
			strlen(title),
			this);
	}
}


void ResourcePixmap::draw_audio_resource(Edit *edit, int x, int w)
{
	if(w <= 0) return;
// Develop strategy for drawing
//printf("ResourcePixmap::draw_audio_resource 1 %s %d\n", edit->asset->path, edit->asset->index_status);
	switch(edit->asset->index_status)
	{
		case INDEX_NOTTESTED:
			return;
			break;
// Disabled.  Always draw from index.
		case INDEX_TOOSMALL:
			draw_audio_source(edit, x, w);
			break;
		case INDEX_BUILDING:
		case INDEX_READY:
		{
//printf("ResourcePixmap::draw_audio_resource 1 %d %d\n", edit->asset->index_status, visible);
			IndexFile indexfile(mwindow);
			if(!indexfile.open_index(edit->asset))
			{
				if(edit->asset->index_zoom > mwindow->edl->session->zoom_sample)
					draw_audio_source(edit, x, w);
				else
					indexfile.draw_index(this, edit, x, w);
				indexfile.close_index();
			}
//printf("ResourcePixmap::draw_audio_resource 2 %d\n", edit->asset->index_status);
			break;
		}
	}
//printf("ResourcePixmap::draw_audio_resource 3\n");
}

void ResourcePixmap::draw_audio_source(Edit *edit, int x, int w)
{
	File *source = mwindow->audio_cache->check_out(edit->asset);
//printf("ResourcePixmap::draw_audio_source 1 %s\n", edit->asset->path);

	if(!source)
	{
		printf("ResourcePixmap::draw_audio_source: failed to check out %s for drawing.\n", edit->asset->path);
		return;
	}

	long source_sample = (pixmap_x - edit_x + x) * mwindow->edl->session->zoom_sample + edit->startsource;
	long source_len = w * mwindow->edl->session->zoom_sample;
	int center_pixel = mwindow->edl->session->zoom_track / 2;
	if(mwindow->edl->session->show_titles) center_pixel += mwindow->theme->title_bg_data->get_h();
// Single sample zoom
	if(mwindow->edl->session->zoom_sample == 1)
	{
		double oldsample, newsample;
		double *buffer = new double[source_len + 1];

//printf("ResourcePixmap::draw_audio_source 2 %p\n", buffer);
		source->set_audio_position(source_sample, mwindow->edl->session->sample_rate);
		source->set_channel(edit->channel);
		canvas->set_color(mwindow->theme->audio_color);
		if(!source->read_samples(buffer, source_len + 1, mwindow->edl->session->sample_rate))
		{
//printf("ResourcePixmap::draw_audio_source 3\n");
			oldsample = newsample = *buffer;
			for(int x1 = x, x2 = x + w, i = 0; x1 < x2; x1++, i++)
			{
				oldsample = newsample;
				newsample = buffer[i];
				canvas->draw_line(x1 - 1, 
					(int)(center_pixel - oldsample * mwindow->edl->session->zoom_y / 2),
					x1,
					(int)(center_pixel - newsample * mwindow->edl->session->zoom_y / 2),
					this);
			}
//printf("ResourcePixmap::draw_audio_source 4 %p\n", buffer);
		}

		delete [] buffer;
//printf("ResourcePixmap::draw_audio_source 5\n");
	}
	else
// Multiple sample zoom
	{
		long fragmentsize;
		long buffersize = fragmentsize = 65536;
		double *buffer = new double[buffersize + 1];
		double highsample, lowsample;
		long zoom_sampleframe = 0;
//printf("ResourcePixmap::draw_audio_source 6 %p\n", buffer);

		source->set_audio_position(source_sample, mwindow->edl->session->sample_rate);
		source->set_channel(edit->channel);
		canvas->set_color(mwindow->theme->audio_color);
		for(long sample = 0; sample < source_len; sample += buffersize)
		{
			fragmentsize = buffersize;
			if(source_len - sample < buffersize)
				fragmentsize = source_len - sample;

			if(sample == 0)
			{
				highsample = buffer[0];
				lowsample = buffer[0];
			}
//printf("ResourcePixmap::draw_audio_source 5 %p\n", buffer);

			if(!source->read_samples(buffer, fragmentsize, mwindow->edl->session->sample_rate))
			{
//printf("ResourcePixmap::draw_audio_source 6 %p\n", buffer);
				for(long bufferposition = 0; bufferposition < fragmentsize; bufferposition++)
				{

					if(zoom_sampleframe == mwindow->edl->session->zoom_sample)
					{
// draw column and reset
						canvas->draw_line(x, 
							(int)(center_pixel - highsample * mwindow->edl->session->zoom_y / 2),
							x,
							(int)(center_pixel - lowsample * mwindow->edl->session->zoom_y / 2),
							this);
						zoom_sampleframe = 0;
						x++;
					}
					
					if(zoom_sampleframe > 0)
					{
// update lowsample and highsample
						if(buffer[bufferposition] < lowsample) 
							lowsample = buffer[bufferposition];
						else 
						if(buffer[bufferposition] > highsample) 
							highsample = buffer[bufferposition];
					}
					else
					{
// initialize lowsample and highsample
						if(buffer[bufferposition] < lowsample)
						{        // all samples are below lowsample
							highsample = lowsample;
						}
						else if(buffer[bufferposition] > highsample) 
						{        // all samples are above highsample
							lowsample = highsample;
						}
						else
						{        // current sample is in between lowsample and highsample
							lowsample = highsample = buffer[bufferposition];
						}
					}
					zoom_sampleframe++;
				}
			}
		}
		delete [] buffer;
	}

	mwindow->audio_cache->check_in(edit->asset);
//printf("ResourcePixmap::draw_audio_source 2 %s\n", edit->asset->path);
}



void ResourcePixmap::draw_video_resource(Edit *edit, long edit_x, long edit_w, int x, int w)
{
//printf("ResourcePixmap::draw_video_resource 1 %d %d\n", x, w);
// pixels spanned by a picon
	long picon_w = (int)(edit->picon_w() + 0.5);
	long picon_h = edit->picon_h();
// pixels spanned by a frame for comparison
	long frame_w = (int)(edit->frame_w() + 0.5);
// Amount to increment x
	long pixel_increment = MAX(picon_w, frame_w);
	int y = 0;
	if(mwindow->edl->session->show_titles) y += mwindow->theme->title_bg_data->get_h();

	File *source = 0;

	for(int x1 = x, x2 = x + w; 
		x1 < x2; 
		x1 += pixel_increment)
	{
		if(pixmap_x + x1 + picon_w > edit_x + edit_w) break;
		if(!source)
		{
			if(!(source = mwindow->video_cache->check_out(edit->asset)))
			{
				printf("ResourcePixmap::draw_video_resource failed to check out %s for drawing.\n", edit->asset->path);
				return;
			}
		}

		long source_frame = (long)((float)(pixmap_x - edit_x + x1) / 
				edit->frame_w() + 0.5) + 
			edit->startsource;
		source->set_layer(edit->channel);
		source->set_video_position(source_frame, mwindow->edl->session->frame_rate);
//printf("ResourcePixmap::draw_video_resource 1 %d %d\n", x1 + edit_x - pixmap_x + picon_w, edit_x + edit_w);
		draw_vframe(source->read_frame(BC_RGB888), 
			x1, 
			y, 
			picon_w, 
			picon_h, 
			0, 
			0);
//printf("ResourcePixmap::draw_video_resource 2\n");
	}

	if(source)
	{
		mwindow->video_cache->check_in(edit->asset);
	}
//printf("ResourcePixmap::draw_video_resource 3\n");
}


void ResourcePixmap::dump()
{
	printf("ResourcePixmap %p\n", this);
	printf(" edit %x edit_x %d pixmap_x %d pixmap_w %d visible %d\n", edit_id, edit_x, pixmap_x, pixmap_w, visible);
}



