#include "confirmsave.h"
#include "defaults.h"
#include "edl.h"
#include "errorbox.h"
#include "file.h"
#include "filexml.h"
#include "fileformat.h"
#include "indexfile.h"
#include "mainmenu.h"
#include "mwindow.h"
#include "mwindowgui.h"
#include "savefile.h"
#include "mainsession.h"

#include <string.h>

Save::Save(MWindow *mwindow) : BC_MenuItem("Save", "s", 's')
{ 
	this->mwindow = mwindow; 
	quit_now = 0; 
}

int Save::create_objects(SaveAs *saveas)
{
	this->saveas = saveas;
	return 0;
}

int Save::handle_event()
{
	if(mwindow->session->filename[0] == 0) 
		saveas->start();
	else
	{
// save it
		FileXML file;
		mwindow->edl->save_xml(mwindow->plugindb, 
			&file, 
			mwindow->session->filename);

		if(file.write_to_file(mwindow->session->filename))
		{
			char string2[256];
			sprintf(string2, "Couldn't open %s", mwindow->session->filename);
			ErrorBox error(PROGRAM_NAME ": Error",
				mwindow->gui->get_abs_cursor_x(),
				mwindow->gui->get_abs_cursor_y());
			error.create_objects(string2);
			error.run_window();
			return 1;		
		}
		mwindow->session->changes_made = 0;
		if(saveas->quit_now) mwindow->gui->set_done(0);
	}
	return 1;
}

int Save::save_before_quit()
{
	saveas->quit_now = 1;
	handle_event();
	return 0;
}

SaveAs::SaveAs(MWindow *mwindow)
 : BC_MenuItem("Save as...", ""), Thread()
{ 
	this->mwindow = mwindow; 
	quit_now = 0;
}

int SaveAs::set_mainmenu(MainMenu *mmenu)
{
	this->mmenu = mmenu;
	return 0;
}

int SaveAs::handle_event() 
{ 
	quit_now = 0;
	start();
	return 1;
}

void SaveAs::run()
{
// ======================================= get path from user
	int result;
printf("SaveAs::run 1\n");
	char directory[1024], filename[1024];
	sprintf(directory, "~");
	mwindow->defaults->get("DIRECTORY", directory);

// Loop if file exists
	do{
		SaveFileWindow *window;

printf("SaveAs::run 1\n");
		window = new SaveFileWindow(mwindow, directory);
printf("SaveAs::run 2\n");
		window->create_objects();
printf("SaveAs::run 3\n");
		result = window->run_window();
printf("SaveAs::run 4\n");
		mwindow->defaults->update("DIRECTORY", window->get_path());
printf("SaveAs::run 5\n");
		strcpy(filename, window->get_path());
		delete window;

// Extend the filename with .xml
		if(strlen(filename) < 4 || 
			strcasecmp(&filename[strlen(filename) - 4], ".xml"))
		{
			strcat(filename, ".xml");
		}

// ======================================= try to save it
		if(filename[0] == 0) return;              // no filename given
		if(result == 1) return;          // user cancelled
		FILE *in;
		if(in = fopen(filename, "rb"))
		{
			fclose(in);
			ConfirmSaveWindow cwindow(mwindow, filename);
			cwindow.create_objects();
			int result2 = cwindow.run_window();
			if(result2) result = 1;
		}
	}while(result);        // file exists so repeat

printf("SaveAs::run 6 %s\n", filename);
// save it
	FileXML xml;
	mwindow->set_filename(filename);      // update the project name
	mwindow->edl->save_xml(mwindow->plugindb, &xml, filename);

	if(xml.write_to_file(filename))
	{
		char string2[256];
		mwindow->set_filename("");      // update the project name
		sprintf(string2, "Couldn't open %s.", filename);
		ErrorBox error(PROGRAM_NAME ": Error",
			mwindow->gui->get_abs_cursor_x(),
			mwindow->gui->get_abs_cursor_y());
		error.create_objects(string2);
		error.run_window();
		return;		
	}
	mwindow->session->changes_made = 0;
	mmenu->add_load(filename);
	if(quit_now) mwindow->gui->set_done(0);
	return;
}

SaveFileWindow::SaveFileWindow(MWindow *mwindow, char *init_directory)
 : BC_FileBox(mwindow->gui->get_abs_cursor_x(),
 	mwindow->gui->get_abs_cursor_y() - BC_WindowBase::get_resources()->filebox_h / 2,
 	init_directory, 
	PROGRAM_NAME ": Save", 
	"Enter a filename to save as")
{ 
	this->mwindow = mwindow; 
}

SaveFileWindow::~SaveFileWindow() {}

