#include "filexml.h"
#include "plugin.inc"
#include "sharedlocation.h"


// plugin locations





SharedLocation::SharedLocation()
{
	this->module = -1;
	this->plugin = -1;
}

SharedLocation::SharedLocation(int module, int plugin)
{
	this->module = module;
	this->plugin = plugin;
}

void SharedLocation::save(FileXML *file)
{
	file->tag.set_title("SHARED_LOCATION");
	if(module >= 0) file->tag.set_property("SHARED_MODULE", (long)module);
	if(plugin >= 0) file->tag.set_property("SHARED_PLUGIN", (long)plugin);
	file->append_tag();
	file->tag.set_title("/SHARED_LOCATION");
	file->append_tag();
	file->append_newline();
}

void SharedLocation::load(FileXML *file)
{
	module = -1;
	plugin = -1;

	module = file->tag.get_property("SHARED_MODULE", (long)module);
	plugin = file->tag.get_property("SHARED_PLUGIN", (long)plugin);
}

int SharedLocation::get_type()
{
	if(plugin < 0)
		return PLUGIN_SHAREDMODULE;
	else
		return PLUGIN_SHAREDPLUGIN;
}


int SharedLocation::operator==(const SharedLocation &that)
{
	if(
		module == that.module &&
		plugin == that.plugin
	) return 1;
	else
	return 0;
}

SharedLocation& SharedLocation::operator=(const SharedLocation &that)
{
	this->plugin = that.plugin;
	this->module = that.module;
	return *this;
}









SharedPluginLocation::SharedPluginLocation()
{
}

SharedPluginLocation::SharedPluginLocation(int module, int plugin)
{
	this->module = module;
	this->plugin = plugin;
}

int SharedPluginLocation::save(FileXML *xml)
{
	xml->tag.set_title("SHARED");
	xml->tag.set_property("MODULE", (long)module);
	xml->tag.set_property("PLUGIN", (long)plugin);
	xml->append_tag();
}

int SharedPluginLocation::load(FileXML *xml, int track_offset)
{
	module = xml->tag.get_property("MODULE", (long)0) + track_offset;
	plugin = xml->tag.get_property("PLUGIN", (long)0);
}

int SharedPluginLocation::operator==(const SharedPluginLocation &that)
{
	if(
		module == that.module &&
		plugin == that.plugin
	) return 1;
	else
	return 0;
}




SharedModuleLocation::SharedModuleLocation()
{
}

SharedModuleLocation::SharedModuleLocation(int module)
{
	this->module = module;
}

int SharedModuleLocation::save(FileXML *xml)
{
//printf("SharedModuleLocation::save module %d\n", module);
	xml->tag.set_title("MODULE");
	xml->tag.set_property("MODULE", (long)module);
	xml->append_tag();
}

int SharedModuleLocation::load(FileXML *xml, int track_offset)
{
	module = xml->tag.get_property("MODULE", (long)0) + track_offset;
}

int SharedModuleLocation::operator==(const SharedModuleLocation &that)
{
	if(
		module == that.module
	) return 1;
	else
	return 0;
}
