#include "awindowgui.h"
#include "defaulttheme.h"
#include "clip.h"
#include "colors.h"
#include "cwindowgui.h"
#include "edl.h"
#include "edlsession.h"
#include "loadmode.h"
#include "mainmenu.h"
#include "mbuttons.h"
#include "meterpanel.h"
#include "mwindow.h"
#include "mwindowgui.h"
#include "overlayframe.h"
#include "patchbay.h"
#include "playtransport.h"
#include "recordgui.h"
#include "recordmonitor.h"
#include "resourcepixmap.h"
#include "mainsession.h"
#include "statusbar.h"
#include "theme.h"
#include "timebar.h"
#include "trackcanvas.h"
#include "vframe.h"
#include "vwindowgui.h"
#include "zoombar.h"


Theme::Theme()
{
	aspect_ratios.append(new BC_ListBoxItem("3:2"));
	aspect_ratios.append(new BC_ListBoxItem("4:3"));
	aspect_ratios.append(new BC_ListBoxItem("16:9"));
	aspect_ratios.append(new BC_ListBoxItem("2.10:1"));
	aspect_ratios.append(new BC_ListBoxItem("2.20:1"));
	aspect_ratios.append(new BC_ListBoxItem("2.25:1"));
	aspect_ratios.append(new BC_ListBoxItem("2.35:1"));
	aspect_ratios.append(new BC_ListBoxItem("2.66:1"));
	frame_sizes.append(new BC_ListBoxItem("160x120"));
	frame_sizes.append(new BC_ListBoxItem("240x180"));
	frame_sizes.append(new BC_ListBoxItem("320x240"));
	frame_sizes.append(new BC_ListBoxItem("360x240"));
	frame_sizes.append(new BC_ListBoxItem("400x300"));
	frame_sizes.append(new BC_ListBoxItem("512x384"));
	frame_sizes.append(new BC_ListBoxItem("640x480"));
	frame_sizes.append(new BC_ListBoxItem("720x480"));
	frame_sizes.append(new BC_ListBoxItem("720x576"));
	frame_sizes.append(new BC_ListBoxItem("1920x1080"));
	sample_rates.append(new BC_ListBoxItem("8000"));
	sample_rates.append(new BC_ListBoxItem("16000"));
	sample_rates.append(new BC_ListBoxItem("22050"));
	sample_rates.append(new BC_ListBoxItem("32000"));
	sample_rates.append(new BC_ListBoxItem("44100"));
	sample_rates.append(new BC_ListBoxItem("48000"));
	sample_rates.append(new BC_ListBoxItem("96000"));
	sample_rates.append(new BC_ListBoxItem("192000"));
	frame_rates.append(new BC_ListBoxItem("1"));
	frame_rates.append(new BC_ListBoxItem("5"));
	frame_rates.append(new BC_ListBoxItem("10"));
	frame_rates.append(new BC_ListBoxItem("12"));
	frame_rates.append(new BC_ListBoxItem("15"));
	frame_rates.append(new BC_ListBoxItem("24"));
	frame_rates.append(new BC_ListBoxItem("24.976"));
	frame_rates.append(new BC_ListBoxItem("25"));
	frame_rates.append(new BC_ListBoxItem("29.97"));
	frame_rates.append(new BC_ListBoxItem("30"));
	frame_rates.append(new BC_ListBoxItem("50"));
	frame_rates.append(new BC_ListBoxItem("59.94"));
	frame_rates.append(new BC_ListBoxItem("60"));
	char string[BCTEXTLEN];
	for(int i = 1; i < 17; i++)
	{
		sprintf(string, "%d", (int)pow(2, i));
		zoom_values.append(new BC_ListBoxItem(string));
	}
	overlay = new OverlayFrame;


	build_icons();
	build_bg_data();
	build_patches();
	build_overlays();
}


// Need to delete everything here
Theme::~Theme()
{
	delete overlay;
}

void Theme::build_icons()
{
	#include "theme/heroine_icon_png.h"
	mwindow_icon = new VFrame(heroine_icon_png);
	vwindow_icon = new VFrame(heroine_icon_png);
	cwindow_icon = new VFrame(heroine_icon_png);
	awindow_icon = new VFrame(heroine_icon_png);
	record_icon = new VFrame(heroine_icon_png);

	#include "theme/clip_icon_png.h"
	clip_icon = new VFrame(clip_icon_png);
}

void Theme::build_bg_data()
{
// Audio settings
#include "theme/channel_bg_png.h"
#include "theme/channel_position_png.h"

	channel_bg_data = new VFrame(channel_bg_png);
	channel_position_data = new VFrame(channel_position_png);

// Track bitmaps
#include "theme/heroine_bg_png.h"
#include "theme/default_bg_png.h"
//	tracks_bg = new VFrame(heroine_bg_png);
	tracks_bg = new VFrame(default_bg_png);
//#include "theme/about_bg_png.h"
#include "theme2/about_png.h"
	about_bg = new VFrame(about_png);

#include "theme/resource256_png.h"
#include "theme/resource128_png.h"
#include "theme/resource64_png.h"
#include "theme/resource32_png.h"
	resource256_bg_data = new VFrame(resource256_png);
	resource128_bg_data = new VFrame(resource128_png);
	resource64_bg_data = new VFrame(resource64_png);
	resource32_bg_data = new VFrame(resource32_png);

#include "theme/plugin_bg_png.h"
	plugin_bg_data = new VFrame(plugin_bg_png);

#include "theme/title_bg_png.h"
	title_bg_data = new VFrame(title_bg_png);

#include "theme/timebar_bg_png.h"
	timebar_bg_data = new VFrame(timebar_bg_png);
	
#include "theme/vwindow_timebar_png.h"
	vtimebar_bg_data = new VFrame(vwindow_timebar_png);
}


void Theme::build_patches()
{
// Patchbay
#include "theme/patchbay_bg_png.h"
#include "theme/mutepatch_checkedhi_png.h"        
#include "theme/mutepatch_checked_png.h"          
#include "theme/mutepatch_hi_png.h"               
#include "theme/mutepatch_up_png.h"               
#include "theme/mutepatch_dn_png.h"               
#include "theme/drawpatch_checkedhi_png.h"        
#include "theme/drawpatch_checked_png.h"          
#include "theme/drawpatch_hi_png.h"               
#include "theme/drawpatch_up_png.h"               
#include "theme/drawpatch_dn_png.h"               
#include "theme/automationpatch_checkedhi_png.h"  
#include "theme/automationpatch_checked_png.h"    
#include "theme/automationpatch_hi_png.h"         
#include "theme/automationpatch_up_png.h"         
#include "theme/automationpatch_dn_png.h"         
#include "theme/gangpatch_checkedhi_png.h"  
#include "theme/gangpatch_checked_png.h"    
#include "theme/gangpatch_hi_png.h"         
#include "theme/gangpatch_up_png.h"         
#include "theme/gangpatch_dn_png.h"         
#include "theme/recordpatch_checkedhi_png.h"      
#include "theme/recordpatch_checked_png.h"        
#include "theme/recordpatch_hi_png.h"             
#include "theme/recordpatch_up_png.h"             
#include "theme/recordpatch_dn_png.h"             
#include "theme/playpatch_hi_png.h"               
#include "theme/playpatch_checkedhi_png.h"        
#include "theme/playpatch_checked_png.h"          
#include "theme/playpatch_up_png.h"               
#include "theme/playpatch_dn_png.h"               
#include "theme/expandpatch_checked_png.h"
#include "theme/expandpatch_checkedhi_png.h"
#include "theme/expandpatch_dn_png.h"
#include "theme/expandpatch_hi_png.h"
#include "theme/expandpatch_up_png.h"
	static VFrame *default_patchbay_bg = new VFrame(patchbay_bg_png);

	static VFrame *default_playpatch_data[] = 
	{
		new VFrame(playpatch_up_png), new VFrame(playpatch_hi_png), new VFrame(playpatch_checked_png), new VFrame(playpatch_dn_png), new VFrame(playpatch_checkedhi_png)
	};

	static VFrame *default_recordpatch_data[] = 
	{
		new VFrame(recordpatch_up_png), new VFrame(recordpatch_hi_png), new VFrame(recordpatch_checked_png), new VFrame(recordpatch_dn_png), new VFrame(recordpatch_checkedhi_png)
	};

	static VFrame *default_automationpatch_data[] = 
	{
		new VFrame(automationpatch_up_png), new VFrame(automationpatch_hi_png), new VFrame(automationpatch_checked_png), new VFrame(automationpatch_dn_png), new VFrame(automationpatch_checkedhi_png)
	};

	static VFrame *default_gangpatch_data[] = 
	{
		new VFrame(gangpatch_up_png), new VFrame(gangpatch_hi_png), new VFrame(gangpatch_checked_png), new VFrame(gangpatch_dn_png), new VFrame(gangpatch_checkedhi_png)
	};

	static VFrame *default_drawpatch_data[] = 
	{
		new VFrame(drawpatch_up_png), new VFrame(drawpatch_hi_png), new VFrame(drawpatch_checked_png), new VFrame(drawpatch_dn_png), new VFrame(drawpatch_checkedhi_png)
	};

	static VFrame *default_mutepatch_data[] = 
	{
		new VFrame(mutepatch_up_png), new VFrame(mutepatch_hi_png), new VFrame(mutepatch_checked_png), new VFrame(mutepatch_dn_png), new VFrame(mutepatch_checkedhi_png)
	};

	static VFrame *default_expandpatch_data[] = 
	{
		new VFrame(expandpatch_up_png), new VFrame(expandpatch_hi_png), new VFrame(expandpatch_checked_png), new VFrame(expandpatch_dn_png), new VFrame(expandpatch_checkedhi_png)
	};

// Patching
	playpatch_data = default_playpatch_data;
	recordpatch_data = default_recordpatch_data;
	automationpatch_data = default_automationpatch_data;
	gangpatch_data = default_gangpatch_data;
	drawpatch_data = default_drawpatch_data;
	mutepatch_data = default_mutepatch_data;
	expandpatch_data = default_expandpatch_data;
	patchbay_bg = default_patchbay_bg;
}

void Theme::build_overlays()
{
#include "theme/keyframe3_png.h"
	keyframe_data = new VFrame(keyframe3_png);
}


void Theme::build_transport(VFrame** &data,
	unsigned char *png_overlay,
	VFrame **bg_data,
	int third)
{
	VFrame default_data(png_overlay);
	data = new VFrame*[3];
	data[0] = new VFrame(0, default_data.get_w(), default_data.get_h(), BC_RGBA8888);
	data[1] = new VFrame(0, default_data.get_w(), default_data.get_h(), BC_RGBA8888);
	data[2] = new VFrame(0, default_data.get_w(), default_data.get_h(), BC_RGBA8888);
	for(int i = 0; i < 3; i++)
	{
		overlay->overlay(data[i], 
			bg_data[i], 
			default_data.get_w() * third, 
			0, 
			default_data.get_w() * third + default_data.get_w(), 
			bg_data[i]->get_h(), 
			0, 
			0, 
			data[i]->get_w(), 
			data[i]->get_h(), 
			1, 
			TRANSFER_NORMAL,
			NEAREST_NEIGHBOR);
		overlay->overlay(data[i], 
			&default_data, 
			0, 
			0, 
			default_data.get_w(), 
			default_data.get_h(), 
			0, 
			0, 
			data[i]->get_w(), 
			data[i]->get_h(), 
			1, 
			TRANSFER_NORMAL,
			NEAREST_NEIGHBOR);
	}
}

void Theme::build_button(VFrame** &data,
	unsigned char *png_overlay,
	VFrame *up_vframe,
	VFrame *hi_vframe,
	VFrame *dn_vframe)
{
	VFrame default_data(png_overlay);
	data = new VFrame*[3];
	data[0] = new VFrame(0, default_data.get_w(), default_data.get_h(), BC_RGBA8888);
	data[1] = new VFrame(0, default_data.get_w(), default_data.get_h(), BC_RGBA8888);
	data[2] = new VFrame(0, default_data.get_w(), default_data.get_h(), BC_RGBA8888);
	data[0]->copy_from(up_vframe);
	data[1]->copy_from(hi_vframe);
	data[2]->copy_from(dn_vframe);
	for(int i = 0; i < 3; i++)
		overlay->overlay(data[i], 
			&default_data, 
			0, 
			0, 
			default_data.get_w(), 
			default_data.get_h(), 
			0, 
			0, 
			data[i]->get_w(), 
			data[i]->get_h(), 
			1, 
			TRANSFER_NORMAL,
			NEAREST_NEIGHBOR);
}

void Theme::build_toggle(VFrame** &data,
	unsigned char *png_overlay,
	VFrame *up_vframe,
	VFrame *hi_vframe,
	VFrame *checked_vframe,
	VFrame *dn_vframe,
	VFrame *checkedhi_vframe)
{
	VFrame default_data(png_overlay);
	data = new VFrame*[5];
	data[0] = new VFrame(0, default_data.get_w(), default_data.get_h(), BC_RGBA8888);
	data[1] = new VFrame(0, default_data.get_w(), default_data.get_h(), BC_RGBA8888);
	data[2] = new VFrame(0, default_data.get_w(), default_data.get_h(), BC_RGBA8888);
	data[3] = new VFrame(0, default_data.get_w(), default_data.get_h(), BC_RGBA8888);
	data[4] = new VFrame(0, default_data.get_w(), default_data.get_h(), BC_RGBA8888);
	data[0]->copy_from(up_vframe);
	data[1]->copy_from(hi_vframe);
	data[2]->copy_from(checked_vframe);
	data[3]->copy_from(dn_vframe);
	data[4]->copy_from(checkedhi_vframe);
	for(int i = 0; i < 5; i++)
		overlay->overlay(data[i], 
			&default_data, 
			0, 
			0, 
			default_data.get_w(), 
			default_data.get_h(), 
			0, 
			0, 
			data[i]->get_w(), 
			data[i]->get_h(), 
			1, 
			TRANSFER_NORMAL,
			NEAREST_NEIGHBOR);
}

#define TIMEBAR_HEIGHT 10
#define PATCHBAY_W 145
#define STATUS_H 20
#define ZOOM_H 30

void Theme::get_mwindow_sizes(MWindowGUI *gui, int w, int h)
{
	mbuttons_x = 0;
	mbuttons_y = gui->mainmenu->get_h();
	mbuttons_w = w;
	mbuttons_h = arrow_data[0]->get_h() + 4;
	mclock_x = 0;
	mclock_y = mbuttons_y + mbuttons_h;
	mclock_w = PATCHBAY_W - 10;
	mclock_h = TIMEBAR_HEIGHT * 3;
	mtimebar_x = PATCHBAY_W;
	mtimebar_y = mclock_y;
	mtimebar_w = w - mtimebar_x;
	mtimebar_h = mclock_h;
	mstatus_x = 0;
	mstatus_y = h - STATUS_H;
	mstatus_w = w;
	mstatus_h = STATUS_H;
	mzoom_x = 0;
	mzoom_y = mstatus_y - ZOOM_H;
	mzoom_h = ZOOM_H;
	mzoom_w = w;
	patchbay_x = 0;
	patchbay_y = mtimebar_y + mtimebar_h;
	patchbay_w = PATCHBAY_W;
	patchbay_h = mzoom_y - patchbay_y;
	mcanvas_x = patchbay_x + patchbay_w;
	mcanvas_y = patchbay_y;
	mcanvas_w = w - patchbay_w;
	mcanvas_h = patchbay_h;
}

void Theme::draw_mbutton_bg(MButtons *gui)
{
}

void Theme::draw_mtimebar_bg(MWindow *mwindow, TimeBar *gui)
{
	gui->draw_3segment(0, 
		0, 
		gui->get_w(), 
		timebar_bg_data,
		0);
}

void Theme::draw_mstatus_bg(StatusBar *gui)
{
//	gui->draw_colored_box(0, 0, gui->get_w(), gui->get_h(), 0, 0);
}

void Theme::draw_zoombar_bg(ZoomBar *gui)
{
}

void Theme::draw_patchbay_bg(PatchBay *gui)
{
	int span;
	for(int i = 0; i < gui->get_h(); i += span)
	{
		span = patchbay_bg->get_h() - 1;
		if(i + span > gui->get_h())
			span = gui->get_h() - i;
		
		gui->draw_vframe(patchbay_bg, 0, i, patchbay_bg->get_w(), span);
	}
}

void Theme::draw_tracks_bg(TrackCanvas *gui, int x, int y, int w, int h)
{
	gui->clear_box(x, y, w, h, gui->background_pixmap);

	int bitmap_x1 = (gui->get_w() - tracks_bg->get_w()) / 2;
	int bitmap_y1 = (gui->get_h() - tracks_bg->get_h()) / 2;
	int bitmap_x2 = bitmap_x1 + tracks_bg->get_w();
	int bitmap_y2 = bitmap_y1 + tracks_bg->get_h();
	int src_x = 0, src_y = 0;

	if(MWindowGUI::visible(bitmap_x1, bitmap_x2, x, x + w) &&
		MWindowGUI::visible(bitmap_y1, bitmap_y2, y, y + h))
	{
		if(bitmap_x1 < x)
		{
			src_x += x - bitmap_x1;
			bitmap_x1 = x;
		}

		if(bitmap_y1 < y)
		{
			src_y += y - bitmap_y1;
			bitmap_y1 = y;
		}

		CLIP(bitmap_x2, x, x + w);
		CLIP(bitmap_y2, y, y + h);
// printf("DefaultTheme::draw_tracks_bg 1 %d %d %d %d\n",
// 			bitmap_x1, 
// 			bitmap_y1, 
// 			bitmap_x2 - bitmap_x1, 
// 			bitmap_y2 - bitmap_y1);

		gui->background_pixmap->draw_vframe(tracks_bg,
			bitmap_x1, 
			bitmap_y1, 
			bitmap_x2 - bitmap_x1, 
			bitmap_y2 - bitmap_y1,
			src_x,
			src_y);
//printf("DefaultTheme::draw_tracks_bg 2\n");
	}
}

void Theme::draw_awindow_bg(MWindow *mwindow, AWindowGUI *gui)
{
	gui->clear_box(0, 0, mwindow->session->awindow_w, mwindow->session->awindow_h);
	gui->flash();
}

void Theme::draw_vwindow_bg(MWindow *mwindow, VWindowGUI *gui)
{
	gui->clear_box(0, 
		0, 
		mwindow->session->vwindow_w, 
		mwindow->session->vwindow_h);
// Timebar
	gui->draw_3segment(vtimebar_x, 
		vtimebar_y, 
		vtimebar_w, 
		vtimebar_bg_data,
		0);
	gui->flash();
}


void Theme::draw_cwindow_bg(MWindow *mwindow, CWindowGUI *gui)
{
	gui->clear_box(0, 0, mwindow->session->cwindow_w, mwindow->session->cwindow_h);
	gui->flash();
}


void Theme::draw_rmonitor_bg(MWindow *mwindow, RecordMonitorGUI *gui)
{
	gui->clear_box(0, 0, mwindow->session->rmonitor_w, mwindow->session->rmonitor_h);
	gui->flash();
}


void Theme::draw_rwindow_bg(MWindow *mwindow, RecordGUI *gui)
{
	gui->clear_box(0, 0, mwindow->session->rwindow_w, mwindow->session->rwindow_h);
}


void Theme::draw_resource_bg(TrackCanvas *canvas,
	MWindow *mwindow, 
	ResourcePixmap *pixmap, 
	int x1, 
	int y1, 
	int x2,
	int y2)
{
	VFrame *image;
	switch(mwindow->edl->session->zoom_track)
	{
		case 256: image = resource256_bg_data;  break;
		case 128: image = resource128_bg_data;  break;
		case 64:  image = resource64_bg_data;   break;
		case 32:  image = resource32_bg_data;   break;
	}

	canvas->draw_3segment(x1, 
		y1, 
		x2 - x1, 
		image,
		pixmap);
}

void Theme::get_vwindow_sizes(MWindow *mwindow, VWindowGUI *gui)
{
	vmeter_y = 10;
	vmeter_h = mwindow->session->vwindow_h - cmeter_y;
	vcanvas_x = 0;
	vcanvas_y = 0;
	vcanvas_h = mwindow->session->vwindow_h - 110;

	if(mwindow->edl->session->vwindow_meter)
	{
		vmeter_x = mwindow->session->vwindow_w - MeterPanel::get_meters_width(mwindow->edl->session->audio_channels, 
			mwindow->edl->session->vwindow_meter);
		vcanvas_w = vmeter_x - vcanvas_x - 10;
	}
	else
	{
		vmeter_x = mwindow->session->vwindow_w;
		vcanvas_w = mwindow->session->vwindow_w;
	}

	vtimebar_x = 0;
	vtimebar_y = vcanvas_y + vcanvas_h;
	vtimebar_w = vcanvas_w - vcanvas_x;
	vslider_x = 10;
	vslider_y = vtimebar_y + 25;
	vslider_w = vtimebar_w - vslider_x;
	vtransport_x = 10;
	vtransport_y = vslider_y + vtimebar_bg_data->get_h() + 5;
	vedit_x = 10;
	vedit_y = vtransport_y + 30;
	vtime_x = 300;
	vtime_y = vtransport_y;
	vtime_w = 200;
	vzoom_x = vtime_x;
	vzoom_y = vtime_y;
	vsource_x = vzoom_x;
	vsource_y = vtransport_y + 30;
}

void Theme::get_cwindow_sizes(MWindow *mwindow, CWindowGUI *gui)
{
	cauto_x = 0;
	cauto_y = 0; 
	cauto_w = 0;
	cauto_h = 0;
	ccomposite_x = 0;
	ccomposite_y = 5;
	ccomposite_w = 30;
	ccomposite_h = mwindow->session->cwindow_h - ccomposite_y;
	ccanvas_x = ccomposite_x + ccomposite_w;
	ccanvas_y = 0;


	if(mwindow->edl->session->cwindow_meter)
	{
		cmeter_x = mwindow->session->cwindow_w - MeterPanel::get_meters_width(mwindow->edl->session->audio_channels, 
			mwindow->edl->session->cwindow_meter);
		ccanvas_w = cmeter_x - ccanvas_x - 10;
	}
	else
	{
		cmeter_x = mwindow->session->cwindow_w;
		ccanvas_w = cmeter_x - ccanvas_x;
	}

	cmeter_y = 10;
	cmeter_h = mwindow->session->cwindow_h - cmeter_y;
	cedit_x = 10;
	cedit_y = mwindow->session->cwindow_h - 30;
	ctransport_x = 10;
	ctransport_y = cedit_y - 30;
	cslider_x = 10;
	cslider_y = ctransport_y - 25;
	cslider_w = ccanvas_x + ccanvas_w - cslider_x;
	ccanvas_h = cslider_y - 5;
	ctime_x = ctransport_x + PlayTransport::get_transport_width(mwindow);
	ctime_y = ctransport_y;
//	czoom_x = ctime_x + 150;
	czoom_x = ctransport_x + PlayTransport::get_transport_width(mwindow) + 20;
	czoom_y = ctime_y;
	cdest_x = czoom_x;
	cdest_y = czoom_y + 30;
}

void Theme::get_awindow_sizes(MWindow *mwindow, AWindowGUI *gui)
{
	abuttons_x = 0; 
	abuttons_y = 0;
	afolders_x = 0;
//	afolders_y = deletedisk_data[0]->get_h();
	afolders_y = 0;
	afolders_w = mwindow->session->afolders_w;
	afolders_h = mwindow->session->awindow_h - afolders_y;
	adivider_x = afolders_x + afolders_w;
	adivider_y = 0;
	adivider_w = 5;
	adivider_h = afolders_h;
	alist_x = afolders_x + afolders_w + 5;
	alist_y = afolders_y;
	alist_w = mwindow->session->awindow_w - alist_x;
	alist_h = afolders_h;
}

void Theme::get_recordmonitor_sizes(MWindow *mwindow, 
	int do_audio, 
	int do_video)
{
	rmonitor_tx_x = 10;
	rmonitor_tx_y = 5;
	rmonitor_source_x = 150;
	rmonitor_source_y = 10;
	rmonitor_interlace_x = 450;
	rmonitor_interlace_y = 0;
	if(do_audio)
	{
		rmonitor_meter_x = mwindow->session->rmonitor_w - MeterPanel::get_meters_width(mwindow->edl->session->audio_channels, 1);
		rmonitor_meter_y = 40;
		rmonitor_meter_h = mwindow->session->rmonitor_h - 10 - rmonitor_meter_y;
	}
	else
	{
		rmonitor_meter_x = mwindow->session->rmonitor_w;
	}
	
	if(do_video)
	{
		rmonitor_canvas_x = 0;
		rmonitor_canvas_y = 35;
		rmonitor_canvas_w = rmonitor_meter_x - rmonitor_canvas_x - 10;
		rmonitor_canvas_h = mwindow->session->rmonitor_h - rmonitor_canvas_y;
		rmonitor_channel_x = 220;
		rmonitor_channel_y = 5;
	}
}

void Theme::get_recordgui_sizes(MWindow *mwindow, RecordGUI *gui, int w, int h)
{
	recordgui_status_x = 10;
	recordgui_status_y = 10;
	recordgui_batch_x = 310;
	recordgui_batch_y = 10;
	recordgui_batchcaption_x = recordgui_batch_x + 110;


	recordgui_transport_x = recordgui_batchcaption_x;
	recordgui_transport_y = recordgui_batch_y + 150;
	recordgui_buttons_x = recordgui_transport_x - 150;
	recordgui_buttons_y = recordgui_transport_y + 40;

	recordgui_batches_x = 10;
	recordgui_batches_y = 270;
	recordgui_batches_w = w - 20;
	recordgui_batches_h = h - recordgui_batches_y - 90;
	recordgui_loadmode_x = w / 2 - LoadMode::get_w() / 2;
	recordgui_loadmode_y = h - 85;
	recordgui_controls_x = 10;
	recordgui_controls_y = h - 30;
}

