#ifndef THEME_H
#define THEME_H

#include "awindowgui.inc"
#include "cwindowgui.inc"
#include "guicast.h"
#include "mbuttons.inc"
#include "mwindow.inc"
#include "mwindowgui.inc"
#include "overlayframe.inc"
#include "patchbay.inc"
#include "recordgui.inc"
#include "recordmonitor.inc"
#include "resourcepixmap.inc"
#include "timebar.inc"
#include "trackcanvas.inc"
#include "statusbar.inc"
#include "vframe.inc"
#include "vwindowgui.inc"
#include "zoombar.inc"

class Theme
{
public:
	Theme();
	virtual ~Theme();

	virtual void get_mwindow_sizes(MWindowGUI *gui, 
		int w, 
		int h);
	virtual void get_vwindow_sizes(MWindow *mwindow, 
		VWindowGUI *gui);
	virtual void get_cwindow_sizes(MWindow *mwindow, 
		CWindowGUI *gui);
	virtual void get_awindow_sizes(MWindow *mwindow, 
		AWindowGUI *gui);
	virtual void get_recordmonitor_sizes(MWindow *mwindow, 
		int do_audio, 
		int do_video);
	virtual void get_recordgui_sizes(MWindow *mwindow, 
		RecordGUI *gui,
		int w,
		int h);
	virtual void draw_mbutton_bg(MButtons *gui);
	virtual void draw_mtimebar_bg(MWindow *mwindow, 
		TimeBar *gui);
	virtual void draw_mstatus_bg(StatusBar *gui);
	virtual void draw_patchbay_bg(PatchBay *gui);
	virtual void draw_awindow_bg(MWindow *mwindow, AWindowGUI *gui);
	virtual void draw_cwindow_bg(MWindow *mwindow, CWindowGUI *gui);
	virtual void draw_vwindow_bg(MWindow *mwindow, VWindowGUI *gui);
	virtual void draw_rmonitor_bg(MWindow *mwindow, RecordMonitorGUI *gui);
	virtual void draw_rwindow_bg(MWindow *mwindow, RecordGUI *gui);
	virtual void draw_resource_bg(TrackCanvas *canvas,
		MWindow *mwindow, 
		ResourcePixmap *pixmap, 
		int x1, 
		int y1, 
		int x2,
		int y2);
	virtual void draw_tracks_bg(TrackCanvas *gui, 
		int x, 
		int y, 
		int w, 
		int h);
	virtual void draw_zoombar_bg(ZoomBar *gui);

	virtual void build_icons();
	virtual void build_bg_data();
	virtual void build_patches();
	virtual void build_overlays();

	VFrame *patchbay_bg;
	VFrame *tracks_bg;
	VFrame *about_bg;

// Icon
	VFrame *mwindow_icon;
	VFrame *vwindow_icon;
	VFrame *cwindow_icon;
	VFrame *awindow_icon;
	VFrame *record_icon;
	VFrame *clip_icon;

// Transport buttons
	VFrame **rewind_data;
	VFrame **fastrev_data;
	VFrame **reverse_data;
	VFrame **framerev_data;
	VFrame **stop_data;
	VFrame **framefwd_data;
	VFrame **forward_data;
	VFrame **fastfwd_data;
	VFrame **end_data;
	VFrame **rec_data;
	VFrame **recframe_data;
	VFrame **stoprec_data;
	VFrame **duplex_data;

	VFrame **asset_append_data;
	VFrame **asset_disk_data;
	VFrame **asset_index_data;
	VFrame **asset_info_data;
	VFrame **asset_project_data;
// Pause button up
	VFrame **pause_data;
// Pause button down
	VFrame **paused_data;

// Channel picker
	VFrame **select_data;
	VFrame **add_data;
	VFrame **edit_data;
	VFrame **moveup_data;
	VFrame **movedn_data;
	VFrame **delete_data;
	VFrame **picture_data;
	VFrame **channel_data;

	VFrame **uptriangle_data;
	VFrame **dntriangle_data;
	char *theme_title;

// MWindow Button bar
	VFrame **arrow_data;
	VFrame **ibeam_data;
	VFrame **autokeyframe_data;
	VFrame **over_button;
	VFrame **chain_data;
	VFrame **attach_data;
	VFrame **detach_data;
	VFrame **apply_data;
	VFrame **fit_data;
	VFrame **start_over_data;
	VFrame **calibrate_data;
	VFrame **reset_data;
	VFrame **prevlabel_data;
	VFrame **nextlabel_data;
	VFrame **magnify_button_data;
	VFrame **presentation_stop;
	VFrame **presentation_loop;
	VFrame **browse_data;
	VFrame **cancel_data;
	VFrame **delete_all_indexes_data;
	VFrame **options_data;
	VFrame **yes_data;
	VFrame **no_data;

// Status bar
	VFrame **statusbar_cancel_data;

// Time bar
	VFrame **label_toggle;
	VFrame **in_point;
	VFrame **out_point;
	VFrame *timebar_bg_data;
	VFrame *vtimebar_bg_data;

// Patching
	VFrame **playpatch_data;
	VFrame **recordpatch_data;
	VFrame **automationpatch_data;
	VFrame **gangpatch_data;
	VFrame **drawpatch_data;
	VFrame **mutepatch_data;
	VFrame **expandpatch_data;
// Sizes of patch tools
	int title_h;
	int play_h;
	int fade_h;
	int mode_h;
	int meter_h;
	int pan_h;



// Editing panel
	VFrame **cut_data;
	VFrame **copy_data;
	VFrame **append_data;
	VFrame **paste_data;
	VFrame **transition_data;
	VFrame **insert_data;
	VFrame **out_data;
	VFrame **in_data;
	VFrame **presentation_data;
	VFrame **undo_data;
	VFrame **redo_data;
	VFrame **labelbutton_data;
	VFrame **indelete_data;
	VFrame **outdelete_data;
	VFrame **lift_data;
	VFrame **overwrite_data;
	VFrame **extract_data;
	VFrame **splice_data;
	VFrame **toclip_data;

// Compositing panel
	VFrame **move_data;
	VFrame **erase_data;
	VFrame **antierase_data;
	VFrame **magnify_data;
	VFrame **camera_data;
	VFrame **protect_data;
	VFrame **proj_data;
	VFrame **crop_data;
	VFrame **cirselection_data;
	VFrame **rectselection_data;
	VFrame **tool_data;
	VFrame **show_meters;

	VFrame *channel_bg_data;
	VFrame *channel_position_data;
	VFrame *resource256_bg_data;
	VFrame *resource128_bg_data;
	VFrame *resource64_bg_data;
	VFrame *resource32_bg_data;
	VFrame *plugin_bg_data;
	VFrame *title_bg_data;

// Track canvas
	VFrame **transitionhandle_data;
	VFrame **edithandlein_data;
	VFrame **edithandleout_data;
	VFrame *keyframe_data;

// Main Window positions derived from window sizes
	int mbuttons_x, mbuttons_y, mbuttons_w, mbuttons_h;
	int mtimebar_x, mtimebar_y, mtimebar_w, mtimebar_h;
	int patchbay_x, patchbay_y, patchbay_w, patchbay_h;
	int mstatus_x, mstatus_y, mstatus_w, mstatus_h;
	int mzoom_x, mzoom_y, mzoom_w, mzoom_h;
// Dimensions of canvas with scrollbars
	int mcanvas_x, mcanvas_y, mcanvas_w, mcanvas_h;
	int mclock_x, mclock_y, mclock_w, mclock_h;
// Color of audio drawing
	int audio_color;

// Viewer Window positions
	int vcanvas_x, vcanvas_y, vcanvas_w, vcanvas_h;
	int vmeter_x, vmeter_y, vmeter_h;
	int vslider_x, vslider_y, vslider_w;
	int vtransport_x, vtransport_y;
	int vedit_x, vedit_y;
	int vtime_x, vtime_y, vtime_w;
	int vzoom_x, vzoom_y;
	int vsource_x, vsource_y;
	int vtimebar_x, vtimebar_y, vtimebar_w;

// Compositor Window positions
	int ccanvas_x, ccanvas_y, ccanvas_w, ccanvas_h;
	int cmeter_x, cmeter_y, cmeter_h;
	int cslider_x, cslider_y, cslider_w;
	int ctransport_x, ctransport_y;
	int cedit_x, cedit_y;
	int ctime_x, ctime_y;
	int czoom_x, czoom_y;
	int cdest_x, cdest_y;
	int ccomposite_x, ccomposite_y, ccomposite_w, ccomposite_h;
	int cauto_x, cauto_y, cauto_w, cauto_h;

// Resource window
	VFrame **deletedisk_data;
	VFrame **deletebin_data;
	VFrame **newbin_data;
	VFrame **renamebin_data;
	VFrame **deleteproject_data;
	VFrame **infoasset_data;
	VFrame **redrawindex_data;
	VFrame **pasteasset_data;
	VFrame **appendasset_data;
	VFrame **viewasset_data;

// Record window positions
	int recordgui_status_x, recordgui_status_y;
	int recordgui_batches_x, recordgui_batches_y;
	int recordgui_batches_w, recordgui_batches_h;
	int recordgui_batch_x, recordgui_batch_y, recordgui_batchcaption_x;
	int recordgui_transport_x, recordgui_transport_y;
	int recordgui_buttons_x, recordgui_buttons_y;
	int recordgui_controls_x, recordgui_controls_y;
	int recordgui_loadmode_x, recordgui_loadmode_y;

	int alist_x, alist_y, alist_w, alist_h;
	int afolders_x, afolders_y, afolders_w, afolders_h;
	int abinbuttons_x, abinbuttons_y;
	int abuttons_x, abuttons_y;
	int adivider_x, adivider_y, adivider_w, adivider_h;

// New project
	int new_audio_x, new_audio_y;
	int new_video_x, new_video_y;
	int new_ok_x, new_ok_y;

// Record window
	VFrame **wrench_data;

// Record monitor
	int rmonitor_tx_x, rmonitor_tx_y;
	int rmonitor_source_x, rmonitor_source_y;
	int rmonitor_canvas_x, rmonitor_canvas_y;
	int rmonitor_canvas_w, rmonitor_canvas_h;
	int rmonitor_meter_x, rmonitor_meter_y;
	int rmonitor_meter_h;
	int rmonitor_channel_x, rmonitor_channel_y;
	int rmonitor_interlace_x, rmonitor_interlace_y;

// Menu items
	ArrayList<BC_ListBoxItem*> aspect_ratios;
	ArrayList<BC_ListBoxItem*> frame_sizes;
	ArrayList<BC_ListBoxItem*> frame_rates;
	ArrayList<BC_ListBoxItem*> sample_rates;
	ArrayList<BC_ListBoxItem*> zoom_values;

// Tools for building widgets
	void build_transport(VFrame** &data,
		unsigned char *png_overlay,
		VFrame **bg_data,
		int third);
	void build_button(VFrame** &data,
		unsigned char *png_overlay,
		VFrame *up_vframe,
		VFrame *hi_vframe,
		VFrame *dn_vframe);
	void build_toggle(VFrame** &data,
		unsigned char *png_overlay,
		VFrame *up_vframe,
		VFrame *hi_vframe,
		VFrame *checked_vframe,
		VFrame *dn_vframe,
		VFrame *checkedhi_vframe);

	OverlayFrame *overlay;
};

#endif
