#include "clip.h"
#include "cplayback.h"
#include "cwindow.h"
#include "edl.h"
#include "edlsession.h"
#include "filexml.h"
#include "fonts.h"
#include "labels.h"
#include "localsession.h"
#include "mbuttons.h"
#include "mwindow.h"
#include "mwindowgui.h"
#include "patchbay.h"
#include "preferences.h"
#include "recordlabel.h"
#include "mainsession.h"
#include "theme.h"
#include "timebar.h"
#include "trackcanvas.h"
#include "tracks.h"
#include "transportque.h"
#include "units.h"
#include "vframe.h"
#include "vwindow.h"
#include "vwindowgui.h"
#include "zoombar.h"


LabelGUI::LabelGUI(MWindow *mwindow, 
	TimeBar *timebar, 
	long pixel, 
	int y, 
	double position,
	VFrame **data)
 : BC_Toggle(translate_pixel(mwindow, pixel), 
 		y, 
		data ? data : mwindow->theme->label_toggle,
		0)
{
	this->mwindow = mwindow;
	this->timebar = timebar;
	this->gui = 0;
	this->pixel = pixel;
	this->position = position;
}

LabelGUI::LabelGUI(MWindow *mwindow, 
	VWindowGUI *gui, 
	long pixel, 
	int y, 
	double position,
	VFrame **data)
 : BC_Toggle(translate_pixel(mwindow, pixel), 
 		y, 
		data ? data : mwindow->theme->label_toggle,
		0)
{
	this->mwindow = mwindow;
	this->timebar = 0;
	this->gui = gui;
	this->pixel = pixel;
	this->position = position;
}

LabelGUI::~LabelGUI()
{
}

int LabelGUI::get_y(MWindow *mwindow, TimeBar *timebar)
{
	if(timebar)
		return 0;
	else
		return mwindow->theme->vtimebar_y + 1;
}

int LabelGUI::translate_pixel(MWindow *mwindow, int pixel)
{
	int result = pixel - mwindow->theme->label_toggle[0]->get_w() / 2;
	return result;
}

void LabelGUI::reposition()
{
	reposition_window(translate_pixel(mwindow, pixel), BC_Toggle::get_y());
}

int LabelGUI::handle_event()
{
	update(1);
	double position;
	EDL *edl = mwindow->edl;

	mwindow->gui->unlock_window();
	mwindow->gui->mbuttons->transport->handle_transport(STOP, 1);
	mwindow->gui->lock_window();

	position = mwindow->edl->align_to_frame(this->position, 1);

	if(shift_down())
	{
		if(position > edl->local_session->selectionend / 2 + edl->local_session->selectionstart / 2)
		{
		
			edl->local_session->selectionend = position;
		}
		else
		{
			edl->local_session->selectionstart = position;
		}
	}
	else
	{
		edl->local_session->selectionstart = edl->local_session->selectionend = position;
	}

// Que the CWindow
	if(timebar)
	{
		mwindow->cwindow->update_position();
		mwindow->gui->cursor->hide();
		mwindow->gui->cursor->draw();
		mwindow->gui->canvas->activate();
		mwindow->gui->zoombar->update();
		timebar->update_highlights();
		mwindow->gui->canvas->flash();
	}
	return 1;
}








InPointGUI::InPointGUI(MWindow *mwindow, 
	TimeBar *timebar, 
	long pixel, 
	double position)
 : LabelGUI(mwindow, 
 	timebar, 
	pixel, 
	get_y(mwindow, timebar), 
	position, 
	mwindow->theme->in_point)
{
}
InPointGUI::InPointGUI(MWindow *mwindow,
	VWindowGUI *gui, 
	long pixel, 
	double position)
 : LabelGUI(mwindow, 
	 gui, 
	 pixel, 
	 mwindow->theme->vtimebar_y + 1, 
	 position, 
	 mwindow->theme->in_point)
{
}
InPointGUI::~InPointGUI()
{
}
int InPointGUI::get_y(MWindow *mwindow, TimeBar *timebar)
{
	int result;
	if(timebar)
		result = 0;
	else
		result = mwindow->theme->vtimebar_y + 1;
	return result;
}


OutPointGUI::OutPointGUI(MWindow *mwindow, 
	TimeBar *timebar, 
	long pixel, 
	double position)
 : LabelGUI(mwindow, 
 	timebar, 
	pixel, 
	get_y(mwindow, timebar), 
	position, 
	mwindow->theme->out_point)
{
}
OutPointGUI::OutPointGUI(MWindow *mwindow, 
	VWindowGUI *gui, 
	long pixel, 
	double position)
 : LabelGUI(mwindow, 
 	gui, 
	pixel, 
	mwindow->theme->vtimebar_y + 1, 
	position, 
	mwindow->theme->out_point)
{
}
OutPointGUI::~OutPointGUI()
{
}
int OutPointGUI::get_y(MWindow *mwindow, TimeBar *timebar)
{
	if(timebar)
		return 0;
	else
		return mwindow->theme->vtimebar_y + 1;
}


PresentationGUI::PresentationGUI(MWindow *mwindow, 
	TimeBar *timebar, 
	long pixel, 
	double position)
 : LabelGUI(mwindow, timebar, pixel, get_y(mwindow, timebar), position)
{
}
PresentationGUI::~PresentationGUI()
{
}







TimeBar::TimeBar(MWindow *mwindow, MWindowGUI *gui)
 : BC_SubWindow(mwindow->theme->mtimebar_x,
 		mwindow->theme->mtimebar_y,
		mwindow->theme->mtimebar_w,
		mwindow->theme->mtimebar_h)
{
	this->gui = gui;
	this->mwindow = mwindow;
}

TimeBar::~TimeBar()
{
	if(in_point) delete in_point;
	if(out_point) delete out_point;
	labels.remove_all_objects();
	presentations.remove_all_objects();
}

int TimeBar::create_objects()
{
	in_point = 0;
	out_point = 0;
	current_operation = TIMEBAR_NONE;
	draw_time();

	flash();
	return 0;
}

void TimeBar::update_labels()
{
	int output = 0;
	for(Label *current = mwindow->edl->labels->first;
		current;
		current = NEXT)
	{
		long pixel = (long)((current->position - mwindow->edl->session->view_start) * 
			mwindow->edl->session->sample_rate / 
			mwindow->edl->session->zoom_sample);

		if(pixel >= 0 && pixel < mwindow->theme->mtimebar_w)
		{
// Create new label
			if(output >= labels.total)
			{
				LabelGUI *new_label;
				add_subwindow(new_label = new LabelGUI(mwindow, this, pixel, 0, current->position));
				labels.append(new_label);
			}
			else
// Reposition old label
			{
				if(labels.values[output]->pixel != pixel)
				{
					labels.values[output]->pixel = pixel;
					labels.values[output]->reposition();
				}

				labels.values[output]->position = current->position;
			}

			if(mwindow->edl->local_session->selectionstart <= current->position &&
				mwindow->edl->local_session->selectionend >= current->position)
				labels.values[output]->update(1);
			else
			if(labels.values[output]->get_value())
				labels.values[output]->update(0);
			output++;
		}
	}

// Delete excess labels
	while(labels.total > output)
	{
		labels.remove_object();
	}
}

void TimeBar::update_highlights()
{
	for(int i = 0; i < labels.total; i++)
	{
		LabelGUI *label = labels.values[i];
		if(mwindow->edl->equivalent(label->position, mwindow->edl->local_session->selectionstart) ||
			mwindow->edl->equivalent(label->position, mwindow->edl->local_session->selectionend))
		{
			if(!label->get_value()) label->update(1);
		}
		else
			if(label->get_value()) label->update(0);
	}

	if(mwindow->edl->equivalent(mwindow->edl->local_session->in_point, mwindow->edl->local_session->selectionstart) ||
		mwindow->edl->equivalent(mwindow->edl->local_session->in_point, mwindow->edl->local_session->selectionend))
	{
		if(in_point) in_point->update(1);
	}
	else
		if(in_point) in_point->update(0);

	if(mwindow->edl->equivalent(mwindow->edl->local_session->out_point, mwindow->edl->local_session->selectionstart) ||
		mwindow->edl->equivalent(mwindow->edl->local_session->out_point, mwindow->edl->local_session->selectionend))
	{
		if(out_point) out_point->update(1);
	}
	else
		if(out_point) out_point->update(0);
}

void TimeBar::update_points()
{
	long pixel = (long)((mwindow->edl->local_session->in_point - 
		mwindow->edl->session->view_start) / 
		mwindow->edl->session->zoom_sample);

	if(in_point)
	{
		if(mwindow->edl->local_session->in_point >= 0 && pixel >= 0 && pixel < get_w())
		{
			if(mwindow->edl->local_session->in_point != in_point->position ||
				in_point->pixel != pixel)
			{
				in_point->pixel = pixel;
				in_point->position = mwindow->edl->local_session->in_point;
				in_point->reposition();
			}
		}
		else
		{
			delete in_point;
			in_point = 0;
		}
	}
	else
	if(mwindow->edl->local_session->in_point >= 0 && pixel >= 0 && pixel < get_w())
	{
printf("TimeBar::update_points 1 %p\n", mwindow->edl);
		add_subwindow(in_point = new InPointGUI(mwindow, 
			this, 
			pixel, 
			mwindow->edl->local_session->in_point));
printf("TimeBar::update_points 2\n");
	}

	pixel = (long)((mwindow->edl->local_session->out_point - 
		mwindow->edl->session->view_start) / 
		mwindow->edl->session->zoom_sample);

	if(out_point)
	{
		if(mwindow->edl->local_session->out_point >= 0 && pixel >= 0 && pixel < get_w())
		{
			if(mwindow->edl->local_session->out_point != out_point->position ||
				out_point->pixel != pixel) 
			{
				out_point->pixel = pixel;
				out_point->position = mwindow->edl->local_session->out_point;
				out_point->reposition();
			}
		}
		else
		{
			delete out_point;
			out_point = 0;
		}
	}
	else
	if(mwindow->edl->local_session->out_point >= 0 && pixel >= 0 && pixel < get_w())
	{
		add_subwindow(out_point = new OutPointGUI(mwindow, this, pixel, mwindow->edl->local_session->out_point));
	}
}

void TimeBar::update_presentations()
{
}


void TimeBar::update()
{
	draw_time();
	update_labels();
	update_points();
	update_presentations();
	flash();
}



int TimeBar::delete_project()
{
//	labels->delete_all();
	return 0;
}

int TimeBar::save(FileXML *xml)
{
//	labels->save(xml);
	return 0;
}



int TimeBar::clear_labels(long start, long end)
{
//	labels->clear(start, end, 0);
	return 0;
}

int TimeBar::toggle_label()
{
// 	long position1 = mwindow->local_session->selectionstart;
// 	long position2 = mwindow->local_session->selectionend;
// 
// 	if(mwindow->session->cursor_on_frames)
// 		position1 = mwindow->align_to_frames(position1);
// 	else
// 		position1 = position1;
// 
// 	if(mwindow->session->cursor_on_frames)
// 		position2 = mwindow->align_to_frames(position2);
// 	else
// 		position2 = position2;
// 
// 
// 	labels->toggle_label(position1, position2);
	return 0;
}

int TimeBar::toggle_label(double position)
{
	position = mwindow->edl->align_to_frame(position, 1);

//	labels->toggle_label(position, position);
	return 0;
}

void TimeBar::draw_time()
{
//printf("TimeBar::draw_time 1\n");
	mwindow->theme->draw_mtimebar_bg(mwindow, this);
//printf("TimeBar::draw_time 2\n");

// fit the time
	long windowspan, timescale1, timescale2, sample_rate, sample;
	float timescale3;
	int pixel;
	char string[256];
#define TIMESPACING 100

	sample_rate = mwindow->edl->session->sample_rate;
	windowspan = mwindow->edl->session->zoom_sample * get_w();
	timescale2 = TIMESPACING * mwindow->edl->session->zoom_sample;

	if(timescale2 <= sample_rate / 4) timescale1 = sample_rate / 4;
	else
	if(timescale2 <= sample_rate / 2) timescale1 = sample_rate / 2;
	else
	if(timescale2 <= sample_rate) timescale1 = sample_rate;
	else
	if(timescale2 <= sample_rate * 10) timescale1 = sample_rate * 10;
	else
	if(timescale2 <= sample_rate * 60) timescale1 = sample_rate * 60;
	else
	if(timescale2 <= sample_rate * 600) timescale1 = sample_rate * 600;
	else
	timescale1 = sample_rate * 3600;

	for(timescale3 = timescale1; timescale3 > timescale2; timescale3 /= 2)
		;

	timescale1 = (long)(timescale3 * 2);

	sample = (long)(mwindow->edl->session->view_start * 
		mwindow->edl->session->sample_rate + 0.5);

	sample /= timescale1;
	sample *= timescale1;
	pixel = (long)((sample - mwindow->edl->session->view_start * 
		mwindow->edl->session->sample_rate) / 
		mwindow->edl->session->zoom_sample);

	for(; pixel < get_w(); pixel += timescale1 / mwindow->edl->session->zoom_sample, sample += timescale1)
	{
		Units::totext(string, 
			sample, 
			sample_rate, 
			mwindow->edl->session->time_format, 
			mwindow->edl->session->frame_rate,
			mwindow->edl->session->frames_per_foot);
	 	set_color(BLACK);
		set_font(MEDIUMFONT);
		draw_text(pixel + 4, get_h(), string);
		draw_line(pixel, 3, pixel, get_h() - 4);
	}
}


int TimeBar::draw()
{
	return 0;
}

int TimeBar::samplemovement()
{
	if(gui)
	{
//		labels->samplemovement();
//		draw();
//		flash();
	}
	return 0;
}

int TimeBar::button_press_event()
{
//printf("TimeBar::button_press_event 1\n");
	if(is_event_win() && cursor_inside())
	{
//printf("TimeBar::button_press_event 2\n");
		gui->unlock_window();
		gui->mbuttons->transport->handle_transport(STOP, 1);
		gui->lock_window();
//printf("TimeBar::button_press_event 3\n");

// Select region between two labels
		if(get_double_click())
		{
			double position = (double)get_cursor_x() * 
				mwindow->edl->session->zoom_sample / 
				mwindow->edl->session->sample_rate + 
				mwindow->edl->session->view_start;
// Test labels
			 if(get_cursor_y() < TIMEBAR_PIXELS)
			 {
				 select_region(position);
			 }
			 else
// Test in/out points
			if(get_cursor_y() < 2 * TIMEBAR_PIXELS)
			{
			}
// Test presentation points
			else
			{
			}
			return 1;
		}
		else
// Reposition highlight cursor
		if(is_event_win() && cursor_inside())
		{
//printf("TimeBar::button_press_event 4\n");
			update_cursor();
//printf("TimeBar::button_press_event 5\n");
			mwindow->gui->canvas->activate();
//printf("TimeBar::button_press_event 6\n");
			return 1;
		}
	}
	return 0;
}

int TimeBar::cursor_motion_event()
{
	int result = 0;
	switch(current_operation)
	{
		case TIMEBAR_DRAG:
			update_cursor();
			if(get_cursor_x() > get_w() || get_cursor_x() < 0)
				mwindow->gui->canvas->start_dragscroll();
			else
				mwindow->gui->canvas->stop_dragscroll();
			result = 1;
			break;
	}
	return result;
}

int TimeBar::button_release_event()
{
//printf("TimeBar::button_release_event %d\n", current_operation);
	int result = 0;
	switch(current_operation)
	{
		case TIMEBAR_DRAG:
			mwindow->gui->canvas->stop_dragscroll();
			current_operation = TIMEBAR_NONE;
			result = 1;
			break;
	}
	return result;
}

// Update the selection cursor during a dragging operation
void TimeBar::update_cursor()
{
	double position = (double)get_cursor_x() * 
		mwindow->edl->session->zoom_sample / 
		mwindow->edl->session->sample_rate + 
		mwindow->edl->session->view_start;
	
	position = mwindow->edl->align_to_frame(position, 0);
	position = MAX(0, position);
	current_operation = TIMEBAR_DRAG;

	mwindow->select_point(position);
	update_highlights();
}


int TimeBar::select_region(double position)
{
	Label *start = 0, *end = 0, *current;
	for(current = mwindow->edl->labels->first; current; current = NEXT)
	{
		if(current->position > position)
		{
			end = current;
			break;
		}
	}

	for(current = mwindow->edl->labels->last ; current; current = PREVIOUS)
	{
		if(current->position <= position)
		{
			start = current;
			break;
		}
	}

// Select region
	if(end != start)
	{
		if(!start)
			mwindow->edl->local_session->selectionstart = 0;
		else
			mwindow->edl->local_session->selectionstart = start->position;

		if(!end)
			mwindow->edl->local_session->selectionend = mwindow->edl->tracks->total_length();
		else
			mwindow->edl->local_session->selectionend = end->position;
	}
	else
	if(end || start)
	{
		mwindow->edl->local_session->selectionstart = 
			mwindow->edl->local_session->selectionend = 
			start->position;
	}

// Que the CWindow
	mwindow->cwindow->update_position();
	mwindow->gui->cursor->hide();
	mwindow->gui->cursor->draw();
	mwindow->gui->canvas->flash();
	mwindow->gui->canvas->activate();
	mwindow->gui->zoombar->update();
	update_highlights();
	return 0;
}

int TimeBar::copy(long start, long end, FileXML *xml)
{
//	labels->copy(start, end, xml);
	return 0;
}

int TimeBar::paste(long start, long end, long sample_length, FileXML *xml)
{
//	labels->paste(start, end, sample_length, xml);
	return 0;
}

int TimeBar::paste_output(long startproject, long endproject, long startsource, long endsource, RecordLabels *new_labels)
{
//	labels->paste_output(startproject, endproject, startsource, endsource, new_labels);
	return 0;
}

int TimeBar::clear(long start, long end)
{ 
//labels->clear(start, end); 	
	return 0;
}

int TimeBar::paste_silence(long start, long end)
{ 
//labels->paste_silence(start, end); 	
	return 0;
}

int TimeBar::modify_handles(long oldposition, long newposition, int currentend)
{ 
//labels->modify_handles(oldposition, newposition, currentend); 	
	return 0;
}

int TimeBar::stop_playback()
{ 
	mwindow->stop_playback(1); 
	return 0;
}



int TimeBar::resize_event()
{
	reposition_window(mwindow->theme->mtimebar_x,
		mwindow->theme->mtimebar_y,
		mwindow->theme->mtimebar_w,
		mwindow->theme->mtimebar_h);
	return 1;
}


int TimeBar::delete_arrows()
{
	return 0;
}


