#ifndef TIMEBAR_H
#define TIMEBAR_H

#include "guicast.h"
#include "filexml.inc"
#include "labels.inc"
#include "mwindow.inc"
#include "recordlabel.inc"
#include "testobject.h"
#include "vwindowgui.inc"

class TimeBarLeftArrow;
class TimeBarRightArrow;

class LabelGUI;
class InPointGUI;
class OutPointGUI;
class PresentationGUI;

// Operations for cursor
#define TIMEBAR_NONE 0
#define TIMEBAR_DRAG 1

class LabelGUI : public BC_Toggle
{
public:
	LabelGUI(MWindow *mwindow, 
		TimeBar *timebar, 
		long pixel, 
		int y, 
		double position,
		VFrame **data = 0);
	LabelGUI(MWindow *mwindow, 
		VWindowGUI *gui, 
		long pixel, 
		int y, 
		double position,
		VFrame **data = 0);
	virtual ~LabelGUI();

	static int translate_pixel(MWindow *mwindow, int pixel);
	virtual int handle_event();
	static int get_y(MWindow *mwindow, TimeBar *timebar);
	void reposition();

	MWindow *mwindow;
	VWindowGUI *gui;
	TimeBar *timebar;
	long pixel;
	double position;
};

class TestPointGUI : public LabelGUI
{
public:
	TestPointGUI(MWindow *mwindow, 
		TimeBar *timebar, 
		long pixel, 
		double position);
};

class InPointGUI : public LabelGUI
{
public:
	InPointGUI(MWindow *mwindow, 
		TimeBar *timebar, 
		long pixel, 
		double position);
	InPointGUI(MWindow *mwindow, 
		VWindowGUI *gui, 
		long pixel, 
		double position);
	virtual ~InPointGUI();
	static int get_y(MWindow *mwindow, TimeBar *timebar);
};

class OutPointGUI : public LabelGUI
{
public:
	OutPointGUI(MWindow *mwindow, 
		TimeBar *timebar, 
		long pixel, 
		double position);
	OutPointGUI(MWindow *mwindow, 
		VWindowGUI *gui, 
		long pixel, 
		double position);
	virtual ~OutPointGUI();
	static int get_y(MWindow *mwindow, TimeBar *timebar);
};

class PresentationGUI : public LabelGUI
{
public:
	PresentationGUI(MWindow *mwindow, 
		TimeBar *timebar, 
		long pixel, 
		double position);
	~PresentationGUI();
};

class TimeBar : public BC_SubWindow
{
public:
	TimeBar(MWindow *mwindow, MWindowGUI *gui);
	~TimeBar();

	int create_objects();
	int update_defaults();
	int button_press_event();
	int button_release_event();
	int cursor_motion_event();
// Synchronize label, in/out, presentation display with master EDL
	void update();
	void draw_time();
	void update_labels();
	void update_points();
	void update_presentations();
// Make sure widgets are highlighted according to selection status
	void update_highlights();
// Update highlight cursor during a drag
	void update_cursor();

// ================================= file operations

	int save(FileXML *xml);
	int load(FileXML *xml, int undo_type);

	int delete_project();        // clear timebar of labels

	int draw();                  // draw everything over
	int samplemovement();
	int refresh_labels();
	int clear_labels(long start, long end);
	int toggle_label();
	int toggle_label(double position);

// ========================================= editing

	int copy(long start, long end, FileXML *xml);
	int paste(long start, long end, long sample_length, FileXML *xml);
	int paste_output(long startproject, long endproject, long startsource, long endsource, RecordLabels *new_labels);
	int clear(long start, long end);
	int paste_silence(long start, long end);
	int modify_handles(long oldposition, long newposition, int currentend);
	int stop_playback();
	int select_region(double position);

	MWindow *mwindow;
	MWindowGUI *gui;
	int resize_event();
	int flip_vertical(int w, int h);
	int delete_arrows();    // for flipping vertical

// Operation started by a buttonpress
	int current_operation;

// REMOVE
	TimeBarLeftArrow *left_arrow;
	TimeBarRightArrow *right_arrow;

private:
	int draw_bevel();
	ArrayList<LabelGUI*> labels;
	InPointGUI *in_point;
	OutPointGUI *out_point;
	ArrayList<PresentationGUI*> presentations;
};


#endif
