#include "assets.h"
#include "autoconf.h"
#include "automation.h"
#include "console.h"
#include "edit.h"
#include "edits.h"
#include "edl.h"
#include "edlsession.h"
#include "filexml.h"
#include "floatauto.h"
#include "floatautos.h"
#include "keyframe.h"
#include "mwindow.h"
#include "module.h"
#include "modules.h"
#include "patch.h"
#include "patchbay.h"
#include "plugin.h"
#include "pluginset.h"
#include "selection.h"
#include "selections.h"
#include "mainsession.h"
#include "theme.h"
#include "intautos.h"
#include "track.h"
#include "trackcanvas.h"
#include "tracks.h"
#include "transition.h"
#include "transportque.inc"
#include "vedit.h"
#include "vframe.h"
#include <string.h>


Track::Track(EDL *edl, Tracks *tracks) : ListItem<Track>()
{
	this->edl = edl;
	this->tracks = tracks;
	y_pixel = 0;
	module_view = 0;
	expand_view = 0;
	draw = 1;
	automate = 1;
	gang = 1;
	title[0] = 0;
	record = 1;
	id = EDL::next_id();
}

Track::~Track()
{
	delete automation;
	delete edits;
//for(int i = 0; i < plugin_set.total; i++)
//	printf("Track::~Track %p %p\n", plugin_set.values[i], plugin_set.values[i]->last);
	plugin_set.remove_all_objects();
}

int Track::create_objects()
{
	selections = new Selections(edl, this);
	return 0;
}

int Track::copy_settings(Track *track)
{
//printf("Track::copy_settings 1\n");
	this->expand_view = track->expand_view;
//printf("Track::copy_settings 1\n");
	this->draw = track->draw;
//printf("Track::copy_settings 1\n");
	this->automate = track->automate;
	this->gang = track->gang;
//printf("Track::copy_settings 1\n");
	this->record = track->record;
//printf("Track::copy_settings 1\n");
	strcpy(this->title, track->title);
//printf("Track::copy_settings 2\n");
	return 0;
}

int Track::load_defaults(Defaults *defaults)
{
	return 0;
}

Track& Track::operator=(Track& track)
{
//printf("Track::operator= 1\n");
	copy_settings(&track);
//printf("Track::operator= 1\n");
	*this->edits = *track.edits;
//printf("Track::operator= 1\n");
	for(int i = 0; i < this->plugin_set.total; i++)
		delete this->plugin_set.values[i];
//printf("Track::operator= 1\n");
	this->plugin_set.remove_all_objects();
//printf("Track::operator= 1\n");

	for(int i = 0; i < track.plugin_set.total; i++)
	{
		PluginSet *new_plugin_set = plugin_set.append(new PluginSet(edl, this));
		*new_plugin_set = *track.plugin_set.values[i];
	}
//printf("Track::operator= 1\n");
	*this->automation = *track.automation;
//printf("Track::operator= 1\n");
	*this->selections = *track.selections;
//printf("Track::operator= 2\n");
	return *this;
}

int Track::vertical_span(Theme *theme)
{
	int result = 0;
	if(expand_view)
		result = edl->session->zoom_track + plugin_set.total * theme->plugin_bg_data->get_h();
	else
		result = edl->session->zoom_track;

	if(edl->session->show_titles)
		result += theme->title_bg_data->get_h();

	return result;
}

double Track::get_length()
{
	double total_length = 0;
	double length = 0;

// Test edits
	if(edits->last)
	{
		length = from_units(edits->last->startproject + edits->last->length);
		if(length > total_length) total_length = length;
	}

// Test plugins
	for(int i = 0; i < plugin_set.total; i++)
	{
		if(plugin_set.values[i]->last)
		{
			length = from_units(plugin_set.values[i]->last->startproject + 
				plugin_set.values[i]->last->length);
			if(length > total_length) total_length = length;
		}
	}

	return total_length;
}


long Track::horizontal_span()
{
	return (long)(get_length() * 
		edl->session->sample_rate / 
		edl->session->zoom_sample + 
		0.5);
}


int Track::load(FileXML *file, int track_offset, unsigned long load_flags)
{
	int result = 0;
	int current_channel = 0;
	int current_plugin = 0;
//printf("Track::load 1 %d\n", edits->total());


	record = file->tag.get_property("RECORD", record);
	gang = file->tag.get_property("GANG", gang);
	draw = file->tag.get_property("DRAW", draw);
	expand_view = file->tag.get_property("EXPAND", expand_view);

//printf("Track::load %d\n", play);
	load_header(file, load_flags);

	do{
		result = file->read_tag();
//printf("Track::load 1 %s\n", file->tag.get_title());

		if(!result)
		{
			if(file->tag.title_is("/TRACK"))
			{
				result = 1;
			}
			else
			if(strstr(file->tag.get_title(), "AUTOS"))
			{
				if(load_flags & LOAD_AUTOMATION)
				{
					automation->load(file, current_plugin, current_channel);
				}
			}
			else
			if(file->tag.title_is("EDITS"))
			{
				if(load_flags & LOAD_EDITS)
					edits->load(file, track_offset);
			}
			else
			if(file->tag.title_is("PLUGINSET"))
			{
//printf("Track::load 2\n");
				if(load_flags & LOAD_EDITS)
				{
//printf("Track::load 3\n");
					PluginSet *plugin_set = new PluginSet(edl, this);
//printf("Track::load 4 %d\n", this->plugin_set.total);
					this->plugin_set.append(plugin_set);
//printf("Track::load 5 %p %p\n", plugin_set->first, plugin_set->last);
					plugin_set->load(file, load_flags);
//printf("Track::load 6\n");
				}
			}
			else
				load_derived(file, load_flags);
		}
	}while(!result);
//printf("Track::load 2 %p %f\n", this, ((FloatAuto*)automation->fade_autos->first)->value);

	return 0;
}

void Track::insert_asset(Asset *asset, 
		double length, 
		double position, 
		int track_number)
{
//printf("Track::insert_asset %f\n", length);
	edits->insert_asset(asset, 
		to_units(length, 1), 
		to_units(position, 0), 
		track_number);
}

// Insert data

// Default keyframes: We don't replace default keyframes in pasting but
// when inserting the first EDL of a load operation we need to replace
// the default keyframes.

// Plugins:  This is an arbitrary behavior
//
// 1) No plugin in source track: Paste silence into destination
// plugin sets.
// 2) Plugin in source track: plugin in source track is inserted into
// existing destination track plugin sets, new sets being added when
// necessary.

void Track::insert_track(Track *track, double position, int replace_default)
{
// Decide whether to copy settings based on load_mode
	if(replace_default) copy_settings(track);
	edits->insert_edits(track->edits, to_units(position, 0));

	insert_plugin_set(track, position);

	automation->insert_track(track->automation, 
		to_units(position, 0), 
		to_units(track->get_length(), 1),
		replace_default);

	optimize();
}

void Track::insert_plugin_set(Track *track, double position)
{
	for(int i = 0; i < track->plugin_set.total; i++)
	{
//printf("Track::insert_track 1\n");
		if(i >= plugin_set.total)
			plugin_set.append(new PluginSet(edl, this));
//printf("Track::insert_track 2\n");

		plugin_set.values[i]->insert_edits(track->plugin_set.values[i], 
			to_units(position, 0));
//printf("Track::insert_track 3\n");
	}
}


void Track::insert_effect(char *title, 
		SharedLocation *shared_location, 
		KeyFrame *default_keyframe,
		int plugin_type)
{
//printf("Track::insert_effect 1\n");
	PluginSet *plugin_set = new PluginSet(edl, this);
//printf("Track::insert_effect 1\n");
	this->plugin_set.append(plugin_set);
//printf("Track::insert_effect 1\n");
	Plugin *plugin = plugin_set->insert_plugin(title, 
		0, 
		to_units(get_length(), 1),
		plugin_type, 
		shared_location,
		default_keyframe);
//printf("Track::insert_effect 2\n");

	expand_view = 1;
}

void Track::paste_keyframe_silence(double start, double length)
{
}

void Track::paste_effect_silence(double start, double length)
{
}

void Track::detach_effect(Plugin *plugin)
{
	for(int i = 0; i < plugin_set.total; i++)
	{
		PluginSet *plugin_set = this->plugin_set.values[i];
		for(Plugin *dest = (Plugin*)plugin_set->first; 
			dest; 
			dest = (Plugin*)dest->next)
		{
			if(dest == plugin)
			{
				long start = plugin->startproject;
				long end = plugin->startproject + plugin->length;

				plugin_set->clear(start, end);
				plugin_set->paste_silence(start, end);

// Delete 0 length pluginsets				
				if(!plugin_set->length()) this->plugin_set.remove_object(plugin_set);
				return;
			}
		}
	}
}

void Track::resample(double old_rate, double new_rate)
{
	edits->resample(old_rate, new_rate);
	automation->resample(old_rate, new_rate);
	for(int i = 0; i < plugin_set.total; i++)
		plugin_set.values[i]->resample(old_rate, new_rate);
}



void Track::optimize()
{
	edits->optimize();
	for(int i = 0; i < plugin_set.total; i++)
		plugin_set.values[i]->optimize();
}

Plugin* Track::get_current_plugin(double position, 
	int plugin_set, 
	int direction, 
	int convert_units)
{
	Plugin *current;
	if(convert_units) position = to_units(position, 0);
	
	
	if(plugin_set >= this->plugin_set.total || plugin_set < 0) return 0;

//printf("Track::get_current_plugin 1 %d %d %d\n", position, plugin_set, direction);
	if(direction == PLAY_FORWARD)
	{
		for(current = (Plugin*)this->plugin_set.values[plugin_set]->last; 
			current; 
			current = (Plugin*)PREVIOUS)
		{
//printf("Track::get_current_plugin 2 %ld %ld %ld\n", current->startproject, position, current->length);
			if(current->startproject <= position && 
				current->startproject + current->length > position)
			{
				return current;
			}
		}
	}
	else
	if(direction == PLAY_REVERSE)
	{
		for(current = (Plugin*)this->plugin_set.values[plugin_set]->first; 
			current; 
			current = (Plugin*)NEXT)
		{
			if(current->startproject < position && 
				current->startproject + current->length >= position)
			{
				return current;
			}
		}
	}

	return 0;
}

Plugin* Track::get_current_transition(double position, int direction)
{
	Edit *current;
	Plugin *result = 0;
	position = to_units(position, 0);

	if(direction == PLAY_FORWARD)
	{
		for(current = edits->last; current; current = PREVIOUS)
		{
			if(current->startproject <= position && current->startproject + current->length > position)
			{
//printf("Track::get_current_transition %p\n", current->transition);
				if(current->transition && position < current->startproject + current->transition->length)
				{
					result = current->transition;
					break;
				}
			}
		}
	}
	else
	if(direction == PLAY_REVERSE)
	{
		for(current = edits->first; current; current = NEXT)
		{
			if(current->startproject < position && current->startproject + current->length >= position)
			{
				if(current->transition && position <= current->startproject + current->transition->length)
				{
					result = current->transition;
					break;
				}
			}
		}
	}

	return result;
}

void Track::synchronize_params(Track *track)
{
	for(Edit *this_edit = edits->first, *that_edit = track->edits->first;
		this_edit && that_edit;
		this_edit = this_edit->next, that_edit = that_edit->next)
	{
		this_edit->synchronize_params(that_edit);
	}

	for(int i = 0; i < plugin_set.total && i < track->plugin_set.total; i++)
		plugin_set.values[i]->synchronize_params(track->plugin_set.values[i]);

	automation->copy_from(track->automation);
}





int Track::dump()
{
	printf("   Data type %d\n", data_type);
	printf("   Title %s\n", title);
	printf("   Edits:\n");
	for(Edit* current = edits->first; current; current = NEXT)
	{
		current->dump();
	}
	automation->dump();
	printf("   Plugin Sets %d\n", plugin_set.total);

	for(int i = 0; i < plugin_set.total; i++)
		plugin_set.values[i]->dump();
//printf("Track::dump 2\n");
	return 0;
}





















Track::Track() : ListItem<Track>()
{
	y_pixel = 0;
	module_view = 0;
}

Track::Track(MWindow *mwindow, Tracks *tracks) : ListItem<Track>()
{
	this->mwindow = mwindow;
	this->tracks = tracks;
	transition = 0;
}

int Track::create_objects(int pixel, int flash)
{
	this->pixel = pixel;

	play_autos = new IntAutos(edl, this);
	play_autos->create_objects();

	mute_autos = new IntAutos(edl, this);
	mute_autos->create_objects();

	for(int i = 0; i < PLUGINS; i++)
	{
		plugin_autos[i] = new FloatAutos(edl, this, LTGREY, -1, 1, AUTOS_VIRTUAL_HEIGHT, 1);
		plugin_autos[i]->create_objects();
	}

	create_derived_objs(flash);
	return 0;
}

// ======================================== accounting

int Track::number_of() { return tracks->number_of(this); }

Patch* Track::get_patch_of()
{
	return mwindow->patches->number(tracks->number_of(this));
}

Module* Track::get_module_of()
{
//	return mwindow->console->modules->module_number(tracks->number_of(this));
}

int Track::track_visible(int x, int w, int y, int h)
{
	if(mwindow->session->tracks_vertical)
	{
		if(               // top edge of track in view
			(pixel >= x &&
			 pixel < x + w)
			 ||             // bottom edge of track in view
			(pixel + mwindow->session->zoom_track > x &&
			 pixel + mwindow->session->zoom_track <= x + w)
			 ||             // view in track
			(x >= pixel && x + w <= pixel + mwindow->session->zoom_track)
			)
		{
			return 1;
		}
	}
	else
	{
		if(               // top edge of track in view
			(pixel >= y &&
			 pixel < y + h)
			 ||             // bottom edge of track in view
			(pixel + mwindow->session->zoom_track > y &&
			 pixel + mwindow->session->zoom_track <= y + h)
			 ||             // view in track
			(y >= pixel && y + h <= pixel + mwindow->session->zoom_track)
			)
		{
			return 1;
		}
	}
	return 0;
}




	
	


// ========================================== drawing




int Track::draw_old(int x, int w, int y, int h, int flash)
{       // make sure this track is visible
	if(tracks->canvas && track_visible(x, w, y, h))
	{
		draw_derived(x, w, y, h, flash);

		for(Edit* current = edits->first; current; current = NEXT)
		{
			current->draw(flash, pixel + mwindow->session->zoom_track / 2, x, w, y, h, 0);
		}
	}
	return 0;
}

int Track::draw_clear(int x, int w, int y, int h, int flash)
{       // make sure this track is visible
	if(tracks->canvas && track_visible(x, w, y, h))
	{
		if(mwindow->session->tracks_vertical)
		tracks->canvas->clear_box(pixel, y, mwindow->session->zoom_track, h);
		else
		tracks->canvas->clear_box(x, pixel, w, mwindow->session->zoom_track);

		if(flash)
		{
// boundaries are the same for vertical
			tracks->canvas->flash(x, w, y, h);
		}
	}
	return 0;
}

int Track::draw_autos(AutoConf *auto_conf)
{
	int i;

	if(tracks->canvas && track_visible(0, tracks->canvas->get_w(), 0, tracks->canvas->get_h()))
	{
		double view_start, view_units, zoom_units;

		get_dimensions(view_start, view_units, zoom_units);

		if(auto_conf->play) 
			play_autos->draw(tracks->canvas, 
							pixel, 
							mwindow->session->zoom_track, 
							zoom_units, 
							view_start, 
							mwindow->session->tracks_vertical);

		for(i = 0; i < this->plugin_set.total; i++)
		{
// 			if(auto_conf->plugin[i])
// 			{
// 				plugin_autos[i]->draw(tracks->canvas,
// 									pixel,
// 									mwindow->session->zoom_track,
// 									zoom_units,
// 									view_start,
// 									mwindow->session->tracks_vertical);
// 			}
		}

		if(auto_conf->play) 
			play_autos->draw(tracks->canvas, 
							pixel, 
							mwindow->session->zoom_track, 
							zoom_units, 
							view_start, 
							mwindow->session->tracks_vertical);

		if(auto_conf->mute) 
			mute_autos->draw(tracks->canvas, 
							pixel, 
							mwindow->session->zoom_track, 
							zoom_units, 
							view_start, 
							mwindow->session->tracks_vertical);

		if(auto_conf->fade) 
			fade_autos->draw(tracks->canvas, 
							pixel, 
							mwindow->session->zoom_track, 
							zoom_units, 
							view_start, 
							mwindow->session->tracks_vertical);

		draw_autos_derived(view_start, zoom_units, auto_conf);
	}
	return 0;
}

int Track::draw_floating_autos(AutoConf *auto_conf, int flash)
{
	int i;
	
	if(tracks->canvas && track_visible(0, tracks->canvas->get_w(), 0, tracks->canvas->get_h()))
	{
		double view_start, view_units, zoom_units;

		get_dimensions(view_start, view_units, zoom_units);

		if(auto_conf->play) 
			play_autos->draw_floating_autos(tracks->canvas, 
								pixel, 
								mwindow->session->zoom_track, 
								zoom_units, 
								view_start, 
								mwindow->session->tracks_vertical, 
								flash);

		for(i = 0; i < this->plugin_set.total; i++)
		{
// 			if(auto_conf->plugin[i])
// 			{
// 				plugin_autos[i]->draw_floating_autos(tracks->canvas,
// 									pixel,
// 									mwindow->session->zoom_track,
// 									zoom_units,
// 									view_start,
// 									mwindow->session->tracks_vertical,
// 									flash);
// 			}
		}
		
// 		if(auto_conf->mute) 
// 			mute_autos->draw_floating_autos(tracks->canvas, 
// 								pixel, 
// 								mwindow->session->zoom_track, 
// 								zoom_units, 
// 								view_start, 
// 								mwindow->session->tracks_vertical, 
// 								flash);
// 		if(auto_conf->fade) 
// 			fade_autos->draw_floating_autos(tracks->canvas, 
// 								pixel, 
// 								mwindow->session->zoom_track, 
// 								zoom_units, 
// 								view_start, 
// 								mwindow->session->tracks_vertical, 
// 								flash);
	
		draw_floating_autos_derived(view_start, zoom_units, auto_conf, flash);
	}
	return 0;
}

// REMOVE
int Track::draw_handles()
{
// 	if(tracks->canvas && track_visible(0, tracks->canvas->get_w(), 0, tracks->canvas->get_h()))
// 	{
// 		double view_start, view_units, zoom_units;
// 		int view_pixels = tracks->view_pixels();
// 
// 		get_dimensions(view_start, view_units, zoom_units);
// 
// 		for(Edit* current = edits->first; current; current = NEXT)
// 		{
// 			current->draw_handles(tracks->canvas, view_start, view_units, zoom_units, view_pixels, pixel + mwindow->session->zoom_track / 2);
// 		}
// 	}
	return 0;
}

// REMOVE
int Track::draw_titles()
{
// 	if(tracks->canvas && track_visible(0, tracks->canvas->get_w(), 0, tracks->canvas->get_h()))
// 	{
// 		float view_start, view_units, zoom_units;
// 		int view_pixels = mwindow->tracks->view_pixels();
// 
// 		get_dimensions(view_start, view_units, zoom_units);
// 		for(Edit* current = edits->first; current; current = NEXT)
// 		{
// 			current->draw_titles(tracks->canvas, view_start, zoom_units, view_pixels, pixel + mwindow->session->zoom_track / 2);
// 		}
// 	}
	return 0;
}




// ================================================= editing

int Track::select_auto(AutoConf *auto_conf, int cursor_x, int cursor_y)
{
	int result = 0, i;

	Patch *patch = get_patch_of();

	if(patch->record)
	{
		double view_start, view_units, zoom_units;

		get_dimensions(view_start, view_units, zoom_units);

		if(auto_conf->play) 
			result = play_autos->select_auto(tracks->canvas, 
											pixel, 
											mwindow->session->zoom_track, 
											zoom_units, 
											view_start, 
											cursor_x, 
											cursor_y, 
											mwindow->session->tracks_vertical);

		for(i = 0; i < PLUGINS; i++)
		{
// 			if(auto_conf->plugin[i]) 
// 				result = plugin_autos[i]->select_auto(tracks->canvas, 
// 												pixel, 
// 												mwindow->session->zoom_track, 
// 												zoom_units, 
// 												view_start, 
// 												cursor_x, 
// 												cursor_y, 
// 												mwindow->session->tracks_vertical);
		}

// 		if(auto_conf->mute) 
// 			result = mute_autos->select_auto(tracks->canvas, 
// 											pixel, 
// 											mwindow->session->zoom_track, 
// 											zoom_units, 
// 											view_start, 
// 											cursor_x, 
// 											cursor_y, 
// 											mwindow->session->tracks_vertical);
// 
// 		if(!result && auto_conf->fade) 
// 			result = fade_autos->select_auto(tracks->canvas, 
// 											pixel, 
// 											mwindow->session->zoom_track, 
// 											zoom_units, 
// 											view_start, 
// 											cursor_x, 
// 											cursor_y, 
// 											mwindow->session->tracks_vertical);
// 
// 		if(!result) result = select_auto_derived(zoom_units, 
// 											view_start, 
// 											auto_conf, 
// 											cursor_x, 
// 											cursor_y);
	}

	return result;
}

int Track::move_auto(AutoConf *auto_conf, int cursor_x, int cursor_y, int shift_down)
{
	int result, i;
	result = 0;

//	float view_start = (float)mwindow->view_start;
//	float view_units = (float)tracks->view_samples();
//	float zoom_units = mwindow->session->zoom_sample;
//// need samples for proper cursor displaying
// the cursor doesn't display when moving autos, buddy

// 	float view_start, view_units, zoom_units;
// 	get_dimensions(view_start, view_units, zoom_units);
// 
// 	if(auto_conf->play) 
// 		result = play_autos->move_auto(tracks->canvas, 
// 										pixel, 
// 										mwindow->session->zoom_track, 
// 										zoom_units, 
// 										view_start, 
// 										cursor_x, 
// 										cursor_y, 
// 										shift_down, 
// 										mwindow->session->tracks_vertical);
// 
// 	for(i = 0; i < PLUGINS; i++)
// 	{
// 		if(auto_conf->plugin[i]) 
// 			result = plugin_autos[i]->move_auto(tracks->canvas, 
// 											pixel, 
// 											mwindow->session->zoom_track, 
// 											zoom_units, 
// 											view_start, 
// 											cursor_x, 
// 											cursor_y, 
// 											shift_down, 
// 											mwindow->session->tracks_vertical);
// 	}
// 
// 	if(auto_conf->mute) 
// 		result = mute_autos->move_auto(tracks->canvas, 
// 										pixel, 
// 										mwindow->session->zoom_track, 
// 										zoom_units, 
// 										view_start, 
// 										cursor_x, 
// 										cursor_y, 
// 										shift_down, 
// 										mwindow->session->tracks_vertical);
// 
// 	if(!result && auto_conf->fade) 
// 		result = fade_autos->move_auto(tracks->canvas, 
// 									pixel, 
// 									mwindow->session->zoom_track, 
// 									zoom_units, 
// 									view_start,
// 									cursor_x, 
// 									cursor_y, 
// 									shift_down, 
// 									mwindow->session->tracks_vertical);
// 
// 	if(!result) result = move_auto_derived(zoom_units, 
// 									view_start, 
// 									auto_conf, 
// 									cursor_x, 
// 									cursor_y, 
// 									shift_down);

	return result;
}

int Track::release_auto()
{
	int result, i;
	result = 0;

	result = play_autos->release_auto();
	for(i = 0; i < PLUGINS && !result; i++)
	{
		result = plugin_autos[i]->release_auto();
	}
	if(!result) result = mute_autos->release_auto();
	if(!result) result = fade_autos->release_auto();
	if(!result) result = release_auto_derived();		

	return result;
}

int Track::copy_automation(AutoConf *auto_conf, 
	double selectionstart, 
	double selectionend, 
	FileXML *file)
{
// used for copying automation alone and for copying everything with fake auto_conf
	int i;

	if(record)
	{
		selectionstart = to_units(selectionstart, 0);
		selectionend = to_units(selectionend, 0);

		if(auto_conf->play)
		{
			file->tag.set_title("PLAYAUTOS");
			file->append_tag();

			automation->play_autos->copy((long)selectionstart, 
						(long)selectionend, 
						file);

			file->tag.set_title("/PLAYAUTOS");
			file->append_tag();
			file->append_newline();
		}

		if(auto_conf->mute)
		{
			file->tag.set_title("MUTEAUTOS");
			file->append_tag();

			automation->mute_autos->copy((long)selectionstart, 
						(long)selectionend, 
						file);

			file->tag.set_title("/MUTEAUTOS");
			file->append_tag();
			file->append_newline();
		}

		if(auto_conf->fade)
		{
			file->tag.set_title("FADEAUTOS");
			file->append_tag();

			automation->fade_autos->copy((long)selectionstart, 
						(long)selectionend, 
						file);

			file->tag.set_title("/FADEAUTOS");
			file->append_tag();
			file->append_newline();
		}

		copy_automation_derived(auto_conf, selectionstart, selectionend, file);
	}
	return 0;
}

int Track::paste_automation(double selectionstart, 
	double selectionend, 
	double total_length, 
	FileXML *file, 
	int shift_autos)
{
// Only used for pasting automation alone.
// Called by MWindow::paste_automation directly.
	int result = 0;
	int current_pan = 0;
	int i;
	Patch *patch = get_patch_of();

	if(patch->record)
	{
		selectionstart = to_units(selectionstart, 0);
		selectionend = to_units(selectionend, 0);
		total_length = to_units(total_length, 0);

//printf("selectionstart %d selectionend %d total_length %d\n", selectionstart, selectionend, total_length);
		while(!result)
		{
			result = file->read_tag();
			
			if(!result)
			{
				if(file->tag.title_is("/TRACK"))
					result = 1;
				else
				if(file->tag.title_is("PLAYAUTOS"))
					play_autos->paste((long)selectionstart, 
						(long)selectionend, 
						(long)total_length, 
						file, 
						"/PLAYAUTOS", 
						1, 
						shift_autos);
				else
				if(file->tag.title_is("PLUGINAUTOS"))
				{
					i = file->tag.get_property("PLUGIN", i);
					plugin_autos[i]->paste((long)selectionstart, 
						(long)selectionend, 
						(long)total_length, 
						file, 
						"/PLUGINAUTOS", 
						1, 
						shift_autos);
				}
				else
				if(file->tag.title_is("MUTEAUTOS"))
					mute_autos->paste((long)selectionstart, 
						(long)selectionend, 
						(long)total_length, 
						file, 
						"/MUTEAUTOS", 
						1, 
						shift_autos);
				else
				if(file->tag.title_is("FADEAUTOS"))
					fade_autos->paste((long)selectionstart, 
						(long)selectionend, 
						(long)total_length, 
						file, 
						"/FADEAUTOS", 
						1, 
						shift_autos);
				else
					paste_automation_derived((long)selectionstart, 
						(long)selectionend, 
						(long)total_length, 
						file, 
						shift_autos, 
						current_pan);
			}
		}
		return 1;
	}
	else
	return 0;
}

int Track::clear_automation(AutoConf *auto_conf, 
	double selectionstart, 
	double selectionend, 
	int shift_autos)
{
// used by menu option
	int i;
	Patch *patch = get_patch_of();

	if(patch->record)
	{
// 		samples_to_units(selectionstart, selectionend);
// 
// 		if(auto_conf->play)
// 			play_autos->clear(selectionstart, selectionend, 1, shift_autos);
// 
// 		for(i = 0; i < PLUGINS; i++)
// 		{
// 			if(auto_conf->plugin[i])
// 				plugin_autos[i]->clear(selectionstart, selectionend, 1, shift_autos);
// 		}
// 
// 		if(auto_conf->mute)
// 			mute_autos->clear(selectionstart, selectionend, 1, shift_autos);
// 
// 		if(auto_conf->fade)
// 			fade_autos->clear(selectionstart, selectionend, 1, shift_autos);
// 
// 		clear_automation_derived(auto_conf, selectionstart, selectionend, shift_autos);
		return 1;
	}
	else
	return 0;
}


// REMOVE
int Track::paste_auto_silence(double start, double end)
{
// call automation->paste_silence instead
	int i;

	if(record)
	{
// unit conversion done in calling routine
	}
}

int Track::copy(double start, 
	double end, 
	FileXML *file, 
	char *output_path)
{
//printf("Track::copy 1\n");
// Use a copy of the selection in converted units
// So copy_automation doesn't reconvert.
	long start_unit = to_units(start, 0);
	long end_unit = to_units(end, 1);




	file->tag.set_title("TRACK");
//	file->tag.set_property("PLAY", play);
	file->tag.set_property("RECORD", record);
//	file->tag.set_property("MUTE", mute);
	file->tag.set_property("DRAW", draw);
	file->tag.set_property("EXPAND", expand_view);
	save_header(file);
	file->append_tag();
	file->append_newline();
	save_derived(file);

	file->tag.set_title("TITLE");
	file->append_tag();
	file->append_text(title);
	file->tag.set_title("/TITLE");
	file->append_tag();
	file->append_newline();

//printf("Track::copy 1\n");
	if(data_type == TRACK_AUDIO)
		file->tag.set_property("TYPE", "AUDIO");
	else
		file->tag.set_property("TYPE", "VIDEO");

//printf("Track::copy 1\n");
	file->append_tag();
	file->append_newline();

//printf("Track::copy 1\n");
	edits->copy(start_unit, end_unit, file, output_path);

//printf("Track::copy 1\n");
	AutoConf auto_conf;
	auto_conf.set_all();
	automation->copy(start_unit, end_unit, file);
	for(int i = 0; i < plugin_set.total; i++)
	{
		plugin_set.values[i]->copy(start_unit, end_unit, file);
	}

//printf("Track::copy 1\n");
	copy_derived(start_unit, end_unit, file);

//printf("Track::copy 1\n");
	file->tag.set_title("/TRACK");
	file->append_tag();
	file->append_newline();
	file->append_newline();
//printf("Track::copy 2\n");
	return 0;
}

int Track::copy_assets(double start, 
	double end, 
	ArrayList<Asset*> *asset_list)
{
	int i, result = 0;

	start = to_units(start, 0);
	end = to_units(end, 0);

	Edit *current = edits->editof((long)start, PLAY_FORWARD);

// Search all edits
	while(current && current->startproject < end)
	{
// Check for duplicate assets
		if(current->asset)
		{
			for(i = 0, result = 0; i < asset_list->total; i++)
			{
				if(asset_list->values[i] == current->asset) result = 1;
			}
// append pointer to new asset
			if(!result) asset_list->append(current->asset);
		}

		current = NEXT;
	}

	return 0;
}




// REMOVE
int Track::paste(long start, long end, long total_length, FileXML *file)
{
	return 0;
}

int Track::paste_transition(double startproject, 
				double endproject, 
				Transition *transition)
{
	startproject = to_units(startproject, 0);
	endproject = to_units(endproject, 0);
	edits->paste_transition((long)startproject, (long)endproject, transition);
	return 0;
}

int Track::clear(double start, double end, int convert_units)
{
// Edits::move_auto calls this routine after the units are converted to the track
// format.
	if(convert_units)
	{
printf("Track::clear 1 %f %f\n", start, end);
		start = to_units(start, 0);
		end = to_units(end, 0);
printf("Track::clear 2 %f %f\n", start, end);
	}

	if(edl->session->autos_follow_edits)
	{
		automation->clear((long)start, (long)end);
	}

	for(int i = 0; i < plugin_set.total; i++)
		plugin_set.values[i]->clear((long)start, (long)end);

	edits->clear((long)start, (long)end);
	return 0;
}

int Track::clear_handle(double start, double end)
{
	start = to_units(start, 0);
	end = to_units(end, 0);

	edits->clear_handle((long)start, (long)end);
}

int Track::popup_transition(int cursor_x, int cursor_y)
{
	int result = 0;

	if(cursor_y > pixel && cursor_y < pixel + mwindow->session->zoom_track)
	{
		double view_start, view_units, zoom_units;

		get_dimensions(view_start, view_units, zoom_units);
		for(Edit* current = edits->first; current; current = NEXT)
		{
			result = current->popup_transition(view_start, zoom_units, cursor_x, cursor_y);
		}
	}

	return result;
}

int Track::select_handle(int cursor_x, int cursor_y, long &selection)
{
	int result = 0;
	Edit *current_edit;
	Patch *patch = get_patch_of();
	
//	if(patch->record)
//	{
		double view_start, view_units, zoom_units;

		get_dimensions(view_start, view_units, zoom_units);
		for(current_edit = edits->first; current_edit && !result; current_edit = current_edit->next)
		{
			result = current_edit->select_handle(view_start, zoom_units, cursor_x, cursor_y, selection);
		}
//	}

	if(!patch->record && result) result = 3;

	return result;
}


int Track::modify_handles(double oldposition, 
	double newposition, 
	int currentend, 
	int handle_mode)
{
//printf("Track::modify_handles 1\n");
	edits->modify_handles(oldposition, 
		newposition, 
		currentend, 
		handle_mode);
//printf("Track::modify_handles 2\n");
	return 0;
}


int Track::paste_silence(double start, double end)
{
//printf("Track::paste_silence 1\n");
	start = to_units(start, 0);
	end = to_units(end, 1);

//printf("Track::paste_silence 1\n");
	edits->paste_silence((long)start, (long)end);
//printf("Track::paste_silence 1\n");
	if(edl->session->autos_follow_edits) automation->paste_silence((long)start, (long)end);
//printf("Track::paste_silence 1\n");

	edits->optimize();
	return 0;
}

int Track::select_edit(int cursor_x, 
	int cursor_y, 
	double &new_start, 
	double &new_end)
{
	int result = 0;
	long pixel1, pixel2;
	long left_unit, right_unit;
	double view_start, view_units, zoom_units;
	
	if(cursor_y > pixel && cursor_y < pixel + mwindow->session->zoom_track)
	{
		get_dimensions(view_start, view_units, zoom_units);
		for(Edit* edit = edits->first; edit && !result; edit = edit->next)
		{
			edit->get_handle_parameters(pixel1, 
				pixel2, 
				left_unit, 
				right_unit, 
				view_start, 
				zoom_units);

			if(cursor_x > pixel1 && cursor_x < pixel2)
			{
				result = 1;
				new_start = from_units(edit->startproject);
				new_end = from_units(edit->startproject + edit->length);
			}
		}
	}
	return result;
}

// REMOVE
int Track::feather_edits(long start, long end, long units)
{
	return 0;
}

// REMOVE
long Track::get_feather(long selectionstart, long selectionend)
{
	return 0;
}

int Track::scale_time(float rate_scale, int scale_edits, int scale_autos, long start, long end)
{
// 	Edit *current_edit;
// 	long current_position = 0;
// 	int i;
// 
// 	samples_to_units(start, end);
// 	for(current_edit = edits->first; current_edit && scale_edits; current_edit = current_edit->next)
// 	{
// 		current_edit->length = (long)(current_edit->length * rate_scale + 0.5);
// 		current_edit->startsource = (long)(current_edit->startsource * rate_scale + 0.5);
// 		current_edit->startproject = current_position;
// 		current_position += current_edit->length;
// 	}
// 
// 	play_autos->scale_time(rate_scale, scale_edits, scale_autos, start, end);
// 	for(i = 0; i < PLUGINS; i++)
// 	{
// 		plugin_autos[i]->scale_time(rate_scale, scale_edits, scale_autos, start, end);
// 	}
// 	mute_autos->scale_time(rate_scale, scale_edits, scale_autos, start, end);
// 	fade_autos->scale_time(rate_scale, scale_edits, scale_autos, start, end);
// 
// 	scale_time_derived(rate_scale, scale_edits, scale_autos, start, end);
	return 0;
}

int Track::swap_module_pointers(int number1, int number2)
{
//printf("Module::swap_plugins %d %d\n", number1, number2);
	for(int i = 0; i < plugin_set.total; i++)
	{
		for(Plugin *plugin = (Plugin*)plugin_set.values[i]->first; 
			plugin; 
			plugin = (Plugin*)plugin->next)
		{
			if(plugin->plugin_type == PLUGIN_SHAREDPLUGIN ||
				plugin->plugin_type == PLUGIN_SHAREDMODULE)
			{
				if(plugin->shared_location.module == number1)
					plugin->shared_location.module = number2;
				else
				if(plugin->shared_location.module == number2)
					plugin->shared_location.module = number1;
			}
		}
	}
	return 0;
}

int Track::swap_transitions(int number1, int number2)
{
	Edit *current_edit;
	Plugin *transition;

	for(current_edit = edits->first; current_edit; current_edit = current_edit->next)
	{
		transition = current_edit->transition;
		if(transition)
		{
			if(transition->plugin_type == 2)
			{
				if(transition->shared_location.module == number1)
					transition->shared_location.module = number2;
				else
				if(transition->shared_location.module == number2)
					transition->shared_location.module = number1;
			}

			if(transition->plugin_type == 3)
			{
				if(transition->shared_location.module == number1)
					transition->shared_location.module = number2;
				else
				if(transition->shared_location.module == number2)
					transition->shared_location.module = number1;
			}
		}
	}
	return 0;
}

int Track::delete_module_pointers(int deleted_track)
{
	for(int i = 0; i < plugin_set.total; i++)
	{
		for(Plugin *plugin = (Plugin*)plugin_set.values[i]->first; 
			plugin; 
			plugin = (Plugin*)plugin->next)
		{
			if(plugin->plugin_type == PLUGIN_SHAREDPLUGIN ||
				plugin->plugin_type == PLUGIN_SHAREDMODULE)
			{
				if(plugin->shared_location.module == deleted_track)
				{
					plugin->on = 0;
				}
				else
				{
					plugin->shared_location.module--;
				}
			}
		}
	}

	return 0;
}

// REMOVE
int Track::test_transition(long current_position)
{
	return 0;
}

int Track::playable_edit(long position)
{
	int result = 0;
	for(Edit *current = edits->first; current && !result; current = NEXT)
	{
		if(current->startproject <= position && current->startproject + current->length > position)
		{
			if(current->transition || current->asset) result = 1;
		}
	}
	return result;
}


int Track::edit_is_interesting(Edit *current, int test_transitions)
{
	return ((test_transitions && current->transition) ||
		(!test_transitions && current->asset));
}

long Track::edit_change_duration(long input_position, long input_length, int reverse, int test_transitions)
{
	Edit *current;
	long edit_length = input_length;

	if(reverse)
	{
// ================================= Reverse playback
// Get first edit on or after position
		for(current = edits->first; 
			current && current->startproject + current->length <= input_position;
			current = NEXT)
			;

		if(current)
		{
			if(current->startproject > input_position)
			{
// Before first edit
				;
			}
			else
			if(edit_is_interesting(current, test_transitions))
			{
// Over an edit of interest.
				if(input_position - current->startproject < input_length)
					edit_length = input_position - current->startproject + 1;
			}
			else
			{
// Over an edit that isn't of interest.
// Search for next edit of interest.
				for(current = PREVIOUS ; 
					current && 
					current->startproject + current->length > input_position - input_length &&
					!edit_is_interesting(current, test_transitions);
					current = PREVIOUS)
					;

					if(current && 
						edit_is_interesting(current, test_transitions) &&
						current->startproject + current->length > input_position - input_length)
                    	edit_length = input_position - current->startproject - current->length + 1;
			}
		}
		else
		{
// Not over an edit.  Try the last edit.
			current = edits->last;
			if(current && 
				((test_transitions && current->transition) ||
				(!test_transitions && current->asset)))
				edit_length = input_position - edits->last->startproject - edits->last->length + 1;
		}
	}
	else
	{
// =================================== forward playback
// Get first edit on or before position
		for(current = edits->last; 
			current && current->startproject > input_position;
			current = PREVIOUS)
			;

		if(current)
		{
			if(current->startproject + current->length <= input_position)
			{
// Beyond last edit.
				;
			}
			else
			if(edit_is_interesting(current, test_transitions))
			{
// Over an edit of interest.
// Next edit is going to require a change.
				if(current->length + current->startproject - input_position < input_length)
					edit_length = current->startproject + current->length - input_position;
			}
			else
			{
// Over an edit that isn't of interest.
// Search for next edit of interest.
				for(current = NEXT ; 
					current && 
					current->startproject < input_position + input_length &&
					!edit_is_interesting(current, test_transitions);
					current = NEXT)
					;

					if(current && 
						edit_is_interesting(current, test_transitions) &&
						current->startproject < input_position + input_length) 
						edit_length = current->startproject - input_position;
			}
		}
		else
		{
// Not over an edit.  Try the first edit.
			current = edits->first;
			if(current && 
				((test_transitions && current->transition) ||
				(!test_transitions && current->asset)))
				edit_length = edits->first->startproject - input_position;
		}
	}

	if(edit_length < input_length)
		return edit_length;
	else
		return input_length;
}

int Track::purge_asset(Asset *asset)
{
	Edit* current_edit;
	int result = 0;

	for(current_edit = edits->first; current_edit; current_edit = current_edit->next)
	{
		if(current_edit->asset == asset)
		{
			current_edit->asset = mwindow->assets->get_asset(SILENCE);
			result++;
		}
	}
	return result;
}

int Track::asset_used(Asset *asset)
{
	Edit* current_edit;
	int result = 0;

	for(current_edit = edits->first; current_edit; current_edit = current_edit->next)
	{
		if(current_edit->asset == asset)
		{
			result++;
		}
	}
	return result;
}

int Track::channel_is_playable(long position, int direction, int *do_channel)
{
	return 1;
}


int Track::plugin_used(long position, long direction)
{
//printf("Track::plugin_used 1 %d\n", this->plugin_set.total);
	for(int i = 0; i < this->plugin_set.total; i++)
	{
		Plugin *current_plugin = get_current_plugin(position, i, direction, 0);
		
//printf("Track::plugin_used 2 %p\n", current_plugin);
		if(current_plugin && 
			(current_plugin->on && 
			current_plugin->plugin_type != PLUGIN_NONE)) return 1;
	}
//printf("Track::plugin_used 3 %p\n", current_plugin);
	return 0;
}

// Audio is always rendered through VConsole
int Track::direct_copy_possible(long start, int direction)
{
	return 1;
}

long Track::to_units(double position, int round)
{
	return (long)position;
}

double Track::from_units(long position)
{
	return (double)position;
}
