#ifndef TRACK_H
#define TRACK_H

#include "arraylist.h"
#include "assets.inc"
#include "autoconf.inc"
#include "automation.inc"
#include "datatype.h"
#include "defaults.inc"
#include "edit.inc"
#include "edits.inc"
#include "edl.inc"
#include "filexml.inc"
#include "floatautos.inc"
#include "guicast.h"
#include "keyframe.inc"
#include "linklist.h"
#include "mwindow.inc"
#include "module.inc"
#include "patch.inc"
#include "plugin.inc"
#include "pluginset.inc"
#include "selections.inc"
#include "sharedlocation.inc"
#include "theme.inc"
#include "intautos.inc"
#include "trackcanvas.inc"
#include "tracks.inc"
#include "transition.inc"

// UNITS ARE SAMPLES FOR ALL

class Track : public ListItem<Track>
{
public:
	Track(EDL *edl, Tracks *tracks);
	Track();
	virtual ~Track();

	int create_objects();
	virtual int load_defaults(Defaults *defaults);
	int load(FileXML *file, int track_offset, unsigned long load_flags);
	virtual int save_header(FileXML *file) { return 0; };
	virtual int save_derived(FileXML *file) { return 0; };
	virtual int load_header(FileXML *file, unsigned long load_flags) { return 0; };
	virtual int load_derived(FileXML *file, unsigned long load_flags) { return 0; };
	Track& operator=(Track& track);
	virtual PluginSet* new_plugins() { return 0; };
// Synchronize playback numbers
	virtual void synchronize_params(Track *track);

// Get number of pixels to display
	virtual int vertical_span(Theme *theme);
	long horizontal_span();
	void resample(double old_rate, double new_rate);

// Get length of track in seconds
	double get_length();

// Editing
	void insert_asset(Asset *asset, 
		double length, 
		double position, 
		int track_number);
	void insert_effect(char *title, 
		SharedLocation *shared_location, 
		KeyFrame *keyframe,
		int plugin_type);
	void insert_plugin_set(Track *track, double position);
	void detach_effect(Plugin *plugin);
// Insert a track from another EDL
	void insert_track(Track *track, double position, int replace_default);
// Optimize editing
	void optimize();
	int is_muted(long position, int direction);  // Test muting status


// Used for determining a selection for editing so leave as int.
// converts the selection to SAMPLES OR FRAMES and stores in value
	virtual long to_units(double position, int round);
	virtual double from_units(long position);


//	virtual double samples_to_units(long samples);
//	long samples_to_units(long &start, long &end);    
//	long samples_to_units(long &start, long &end, long &total_length);



// Convert the range to SAMPLES from whatever units the track is in and stores in the arguments
//	virtual double units_to_samples(long units);
//	double units_to_samples(long &start, long &end);




// Positions are identical for handle modifications
    virtual int identical(long sample1, long sample2) { return 0; };
// Get the plugin belonging to the set.
	Plugin* get_current_plugin(double position, 
		int plugin_set, 
		int direction, 
		int convert_units);
	Plugin* get_current_transition(double position, int direction);
	virtual int channel_is_playable(long position, int direction, int *do_channel);
// Test direct copy conditions common to all the rendering routines
	virtual int direct_copy_possible(long start, int direction);
	int plugin_used(long position, long direction);
	virtual int copy_settings(Track *track);
	void paste_keyframe_silence(double start, double length);
	void paste_effect_silence(double start, double length);
	int swap_module_pointers(int number1, int number2);
	int delete_module_pointers(int deleted_track);

	EDL *edl;
	Tracks *tracks;

	Edits *edits;
// Plugin set uses key frames for automation
	ArrayList<PluginSet*> plugin_set;
	Automation *automation;
// Only used by video track
	Selections *selections;

// Vertical offset from top of timeline
	int y_pixel;
	int expand_view;
	int draw;
// REMOVE
	int automate;
	int gang;
	char title[BCTEXTLEN];
	int record;
// TRACK_AUDIO or TRACK_VIDEO
	int data_type;     
// Identification of the track
	int id;















// If module is visible
	int module_view;

	Track(MWindow *mwindow, Tracks *tracks);
	int create_objects(int pixel, int flash);
	virtual int create_derived_objs(int flash) { return 0; };

	virtual int set_index_files(int flash, Asset *asset) { return 0; };

	int load_automation(FileXML *file);
	int load_edits(FileXML *file);

	virtual int change_channels(int oldchannels, int newchannels) { return 0; };
	virtual int dump();

// ==================================== drawing
	int draw_old(int x, int w, int y, int h, int flash = 1);
	virtual int draw_derived(int x, int w, int y, int h, int flash = 1) { return 0; };
	int draw_clear(int x, int w, int y, int h, int flash = 1);
	int draw_handles();   // position is y position of canvas
	int draw_titles();

	int draw_autos(AutoConf *auto_conf);
	int draw_floating_autos(AutoConf *auto_conf, int flash);
	virtual int draw_autos_derived(float view_start, float zoom_units, AutoConf *auto_conf) { return 0; };
	virtual int draw_floating_autos_derived(float view_start, float zoom_units, AutoConf *auto_conf, int flash) { return 0; };

	int track_visible(int x, int w, int y, int h);

// ===================================== editing
	int copy(double start, 
		double end, 
		FileXML *file, 
		char *output_path = "");
	int copy_assets(double start, 
		double end, 
		ArrayList<Asset*> *asset_list);
	virtual int copy_derived(long start, long end, FileXML *file) { return 0; };
	int paste(long start, long end, long total_length, FileXML *file);
	virtual int paste_derived(long start, long end, long total_length, FileXML *file, int &current_channel) { return 0; };
	int paste_transition(double startproject, 
		double endproject, 
		Transition *transition);
	int clear(double start, 
		double end, 
		int convert_units = 1);
	virtual int clear_derived(double start, 
		double end) { return 0; };

	int copy_automation(AutoConf *auto_conf, 
		double selectionstart, 
		double selectionend, 
		FileXML *file);
	virtual int copy_automation_derived(AutoConf *auto_conf, 
		double selectionstart, 
		double selectionend, 
		FileXML *file) { return 0; };
	int paste_automation(double selectionstart, 
		double selectionend, 
		double total_length, 
		FileXML *file, 
		int shift_autos = 1);
	virtual int paste_automation_derived(double selectionstart, 
		double selectionend, 
		double total_length, 
		FileXML *file, 
		int shift_autos, 
		int &current_pan) { return 0; };
	int clear_automation(AutoConf *auto_conf, 
		double selectionstart, 
		double selectionend, 
		int shift_autos = 1);
	virtual int clear_automation_derived(AutoConf *auto_conf, 
		double selectionstart, 
		double selectionend, 
		int shift_autos = 1) { return 0; };
	int paste_auto_silence(double start, double end);
	virtual int paste_auto_silence_derived(long start, long end) { return 0; };
	int scale_time(float rate_scale, int scale_edits, int scale_autos, long start, long end);
	virtual int scale_time_derived(float rate_scale, int scale_edits, int scale_autos, long start, long end) { return 0; };
	int purge_asset(Asset *asset);
	int asset_used(Asset *asset);
	int clear_handle(double start, double end);
	int paste_silence(double start, double end);
	virtual int select_translation(int cursor_x, int cursor_y) { return 0; };  // select video coordinates for frame
	virtual int update_translation(int cursor_x, int cursor_y, int shift_down) { return 0; };  // move video coordinates
	int select_auto(AutoConf *auto_conf, int cursor_x, int cursor_y);
	virtual int select_auto_derived(float zoom_units, float view_start, AutoConf *auto_conf, int cursor_x, int cursor_y) { return 0; };
	int move_auto(AutoConf *auto_conf, int cursor_x, int cursor_y, int shift_down);
	virtual int move_auto_derived(float zoom_units, float view_start, AutoConf *auto_conf, int cursor_x, int cursor_y, int shift_down) { return 0; };
	int release_auto();
	virtual int release_auto_derived() { return 0; };
// Return whether automation would prevent direct frame copies.  Not fully implemented.
	int automation_is_used(long start, long end);
	virtual int automation_is_used_derived(long start, long end) { return 0; }

	int popup_transition(int cursor_x, int cursor_y);

// Return 1 if the left handle was selected 2 if the right handle was selected 3 if the track isn't recordable
	int select_handle(int cursor_x, int cursor_y, long &selection);
	int modify_handles(double oldposition, 
		double newposition, 
		int currentend, 
		int handle_mode);
	int select_edit(int cursor_x, 
		int cursor_y, 
		double &new_start, 
		double &new_end);
	virtual int end_translation() { return 0; };
	virtual int reset_translation(long start, long end) { return 0; };
	int feather_edits(long start, long end, long units);
	long get_feather(long selectionstart, long selectionend);
	int swap_transitions(int number1, int number2);
	int shift_module_pointers(int deleted_track);

// ===================================== automation

// ===================================== accounting
	int number_of();           // number of this track
	Patch* get_patch_of();
	Module* get_module_of();

// get_dimensions is used for getting drawing regions so use floats for partial frames
// get the display dimensions in SAMPLES OR FRAMES
	virtual int get_dimensions(double &view_start, 
		double &view_units, 
		double &zoom_units) { return 0; };   
// Compare last transition to current position
 	int test_transition(long current_position); 
// Longest time from current_position in which nothing changes
	long edit_change_duration(long input_position, long input_length, int reverse, int test_transitions);
// Utility for edit_change_duration.
	int edit_is_interesting(Edit *current, int test_transitions);
// If the edit under position is playable
	int playable_edit(long position);

// ===================================== for handles, titles, etc

	Patch *patch;
	Module *module;
	long old_view_start;
	int pixel;   // pixel position from top of track view
	IntAutos *play_autos;
	IntAutos *mute_autos;
	FloatAutos *fade_autos;
	FloatAutos *plugin_autos[PLUGINS];
	MWindow *mwindow;
// Transition currently being rendered
	Transition *transition;
};

#endif
