#include "assets.h"
#include "autoconf.h"
#include "automation.h"
#include "bezierauto.h"
#include "bezierautos.h"
#include "clip.h"
#include "colors.h"
#include "cplayback.h"
#include "cursors.h"
#include "cwindow.h"
#include "cwindowgui.h"
#include "edithandles.h"
#include "editpopup.h"
#include "edits.h"
#include "edl.h"
#include "edlsession.h"
#include "floatauto.h"
#include "floatautos.h"
#include "intauto.h"
#include "keyframe.h"
#include "keyframes.h"
#include "keys.h"
#include "localsession.h"
#include "mainclock.h"
#include "mainundo.h"
#include "mbuttons.h"
#include "mwindow.h"
#include "mwindowgui.h"
#include "patchbay.h"
#include "tracking.h"
#include "panautos.h"
#include "playbackengine.h"
#include "playtransport.h"
#include "plugin.h"
#include "pluginpopup.h"
#include "pluginset.h"
#include "pluginserver.h"
#include "preferences.h"
#include "resourcepixmap.h"
#include "mainsession.h"
#include "transitionhandles.h"
#include "transitionpopup.h"
#include "transportque.h"
#include "zoombar.h"
#include "theme.h"
#include "timebar.h"
#include "intautos.h"
#include "trackcanvas.h"
#include "tracks.h"
#include "transition.h"
#include "vframe.h"

#include <string.h>

TrackCanvas::TrackCanvas(MWindow *mwindow, MWindowGUI *gui)
 : BC_SubWindow(mwindow->theme->mcanvas_x,
 	mwindow->theme->mcanvas_y,
	gui->view_w,
	gui->view_h)
{
	this->mwindow = mwindow;
	this->gui = gui;
	current_end = 0;
	selection_midpoint1 = selection_midpoint2 = 0;
	selection_type = 0;
	region_selected = 0;
	handle_selected = 0;
	auto_selected = 0;
	translate_selected = 0;
	which_handle = 0;
	handle_pixel = 0;
	drag_scroll = 0;
	active = 0;
}

TrackCanvas::~TrackCanvas()
{
	for(int i = 0; i < resource_pixmaps.total; i++)
		delete resource_pixmaps.values[i];
//	delete transition_handles;
	delete edit_handles;
	delete keyframe_pixmap;
	delete background_pixmap;
}

int TrackCanvas::create_objects()
{
	background_pixmap = new BC_Pixmap(this, get_w(), get_h());
//	transition_handles = new TransitionHandles(mwindow, this);
	edit_handles = new EditHandles(mwindow, this);
	keyframe_pixmap = new BC_Pixmap(this, mwindow->theme->keyframe_data, PIXMAP_ALPHA);
	draw();
	update_cursor();
	flash();
	return 0;
}

void TrackCanvas::resize_event()
{
	draw();
}

int TrackCanvas::keypress_event()
{
	int result = 0;
//	if(active)
//	{
		switch(get_keypress())
		{
			case LEFT:
				if(!ctrl_down()) 
				{ 
					mwindow->move_left(); 
					result = 1; 
				}
				break;
			case RIGHT:
				if(!ctrl_down()) 
				{ 
					mwindow->move_right(); 
					result = 1; 
				}
				break;

			case UP:
				if(!ctrl_down())
				{
					mwindow->expand_sample();
					result = 1;
				}
				else
				{
					mwindow->expand_y();
					result = 1;
				}
				break;

			case DOWN:
				if(!ctrl_down())
				{
					mwindow->zoom_in_sample();
					result = 1;
				}
				else
				{
					mwindow->zoom_in_y();
					result = 1;
				}
				break;

			case PGUP:
				if(!ctrl_down())
				{
					mwindow->move_up();
					result = 1;
				}
				else
				{
					mwindow->expand_t();
					result = 1;
				}
				break;

			case PGDN:
				if(!ctrl_down())
				{
					mwindow->move_down();
					result = 1;
				}
				else
				{
					mwindow->zoom_in_t();
					result = 1;
				}
				break;
		}
//	}
	return result;
}

int TrackCanvas::drag_motion()
{
	int cursor_x = get_relative_cursor_x();
	int cursor_y = get_relative_cursor_y();
	Track *over_track = 0;
	Edit *over_edit = 0;
	int redraw = 0;

	if(cursor_x >= 0 && 
		cursor_y >= 0 && 
		cursor_x < get_w() && 
		cursor_y < get_h())
	{
// Test every edit
		for(Track *track = mwindow->edl->tracks->first; track; track = track->next)
		{
			long track_x, track_y, track_w, track_h;
			track_dimensions(track, track_x, track_y, track_w, track_h);

			if(cursor_x >= track_x && 
				cursor_y >= track_y && 
				cursor_x < track_x + track_w && 
				cursor_y < track_y + track_h)
			{
				over_track = track;
				for(Edit *edit = track->edits->first; edit; edit = edit->next)
				{
					long edit_x, edit_y, edit_w, edit_h;
					edit_dimensions(edit, edit_x, edit_y, edit_w, edit_h);

					if(cursor_x >= edit_x && 
						cursor_y >= edit_y && 
						cursor_x < edit_x + edit_w && 
						cursor_y < edit_y + edit_h)
					{
						over_edit = edit;
						break;
					}
				}
				break;
			}
		}
	}

	if(mwindow->session->track_highlighted != over_track) 
	{
		mwindow->session->track_highlighted = over_track;
		redraw = 1;
	}

	if(mwindow->session->edit_highlighted != over_edit)
	{
		mwindow->session->edit_highlighted = over_edit;
		redraw = 1;
	}

	if(redraw)
	{
		lock_window();
		draw_overlays();
		flash();
		unlock_window();
	}

	return 0;
}

int TrackCanvas::drag_start_event()
{
	int result = 0;

	if(is_event_win())
	{
		if(test_plugins(get_drag_x(), 
			get_drag_y(), 
			1,
			0))
		{
			result = 1;
		}
	}
	
	
	return result;
}

int TrackCanvas::drag_stop()
{
	int result = 0, redraw = 0;

	switch(mwindow->session->current_operation)
	{
		case DRAG_VTRANSITION:
		case DRAG_ATRANSITION:
			if(mwindow->session->edit_highlighted)
			{
				if((mwindow->session->current_operation == DRAG_ATRANSITION &&
					mwindow->session->track_highlighted->data_type == TRACK_AUDIO) ||
					(mwindow->session->current_operation == DRAG_VTRANSITION &&
					mwindow->session->track_highlighted->data_type == TRACK_VIDEO))
				{
//printf("TrackCanvas::drag_stop 1\n");
					mwindow->session->edit_highlighted->insert_transition(mwindow->session->drag_pluginserver->title);
//printf("TrackCanvas::drag_stop 1\n");
					result = 1;
//printf("TrackCanvas::drag_stop 1\n");
					mwindow->cwindow->playback_engine->que->send_command(CURRENT_FRAME, 
						CHANGE_EDL,
						mwindow->edl,
						1);
//printf("TrackCanvas::drag_stop 2\n");
				}
			}
			redraw = 1;
			break;

		case DRAG_AEFFECT:
		case DRAG_VEFFECT:
			if(mwindow->session->track_highlighted)
			{
//printf("TrackCanvas::drag_stop 1\n");
				if((mwindow->session->current_operation == DRAG_AEFFECT &&
					mwindow->session->track_highlighted->data_type == TRACK_AUDIO) ||
					(mwindow->session->current_operation == DRAG_VEFFECT &&
					mwindow->session->track_highlighted->data_type == TRACK_VIDEO))
				{
					mwindow->insert_effect(mwindow->session->drag_pluginserver->title, 
						0,
						mwindow->session->track_highlighted,
						PLUGIN_STANDALONE);
					result = 1;
					redraw = 1;
				}
//printf("TrackCanvas::drag_stop 2\n");
			}
			break;

		case DRAG_ASSET:
			if(mwindow->session->edit_highlighted)
			{
			}
			else
			if(mwindow->session->track_highlighted)
			{
			}
			break;
	}

	if(result)
	{
		mwindow->session->track_highlighted = 0;
		mwindow->session->edit_highlighted = 0;
		mwindow->session->current_operation = NO_OPERATION;
	}
//printf("TrackCanvas::drag_stop 5\n");

	if(redraw)
	{
//printf("TrackCanvas::drag_stop 6\n");
//		gui->cursor->hide();
		mwindow->edl->tracks->update_y_pixels(mwindow->theme);
		gui->get_scrollbars();
		draw();
		gui->patchbay->update();
		gui->cursor->update();
		flash();
		flush();
//printf("TrackCanvas::drag_stop 11\n");
	}
//printf("TrackCanvas::drag_stop 12\n");

	return result;
}


void TrackCanvas::draw()
{
// Swap pixmap layers
	if(get_w() != background_pixmap->get_w() ||
		get_h() != background_pixmap->get_h())
	{
//printf("TrackCanvas::draw 1\n");
		delete background_pixmap;
		background_pixmap = new BC_Pixmap(this, get_w(), get_h());
	}

//printf("TrackCanvas::draw 2\n");
	if(gui->cursor) gui->cursor->hide();
//printf("TrackCanvas::draw 3\n");
	mwindow->theme->draw_tracks_bg(this, 0, 0, get_w(), get_h());
//printf("TrackCanvas::draw 4\n");
	draw_resources();
//printf("TrackCanvas::draw 5\n");
	draw_overlays();
//printf("TrackCanvas::draw 6\n");
}

void TrackCanvas::update_cursor()
{
	switch(mwindow->edl->session->editing_mode)
	{
		case EDITING_ARROW: set_cursor(ARROW_CURSOR); break;
		case EDITING_IBEAM: set_cursor(IBEAM_CURSOR); break;
	}
}


void TrackCanvas::draw_indexes(Asset *asset)
{
// Don't redraw raw samples
	if(asset->index_zoom > mwindow->edl->session->zoom_sample)
		return;

	draw_resources(1, asset);

	draw_overlays();
	draw_automation();
	flash();
	flush();
//printf("TrackCanvas::draw_indexes 4\n");
}

void TrackCanvas::draw_resources(int indexes_only, Asset *index_asset)
{
// Age resource pixmaps for deletion
	if(!indexes_only)
		for(int i = 0; i < resource_pixmaps.total; i++)
			resource_pixmaps.values[i]->visible--;
//printf("TrackCanvas::draw_resources 1\n");

// Search every edit
	for(Track *current = mwindow->edl->tracks->first;
		current;
		current = NEXT)
	{
//printf("TrackCanvas::draw_resources 2\n");
		for(Edit *edit = current->edits->first; edit; edit = edit->next)
		{
//printf("TrackCanvas::draw_resources 3\n");
			if(!edit->asset) continue;
			if(indexes_only)
			{
				if(edit->track->data_type != TRACK_AUDIO) continue;
				if(!edit->asset->test_path(index_asset->path)) continue;
			}
//printf("TrackCanvas::draw_resources 3.1 %d %s %s\n", 
//	indexes_only, 
//	edit->asset->path, 
//	index_asset ? index_asset->path : "");

			long edit_x, edit_y, edit_w, edit_h;
			edit_dimensions(edit, edit_x, edit_y, edit_w, edit_h);
//printf("TrackCanvas::draw_resources 3.2\n");

// Edit is visible
			if(MWindowGUI::visible(edit_x, edit_x + edit_w, 0, get_w()) &&
				MWindowGUI::visible(edit_y, edit_y + edit_h, 0, get_h()))
			{
				long pixmap_x, pixmap_w, pixmap_h;

//printf("TrackCanvas::draw_resources 3.3\n");
// Search for existing pixmap containing edit
				for(int i = 0; i < resource_pixmaps.total; i++)
				{
					ResourcePixmap* pixmap = resource_pixmaps.values[i];
//printf("TrackCanvas::draw_resources 3.4 %p %p\n", resource_pixmaps.values[i]->edit, edit);
// Same pointer can be different edit if editing took place
					if(pixmap->edit_id == edit->id)
					{
//printf("TrackCanvas::draw_resources 4 %p\n", edit);
						pixmap->visible = 1;
						break;
					}
//printf("TrackCanvas::draw_resources 5\n");
				}

// Get new size, offset of pixmap needed
				get_pixmap_size(edit, 
					edit_x, 
					edit_w, 
					pixmap_x, 
					pixmap_w, 
					pixmap_h);
//printf("TrackCanvas::draw_resources 6\n");
//printf("TrackCanvas::draw_resources 7 %d %d\n", pixmap_w, pixmap_h);

// Draw new data
				if(pixmap_w && pixmap_h)
				{
// Resize pixmap if it exists or create one
					ResourcePixmap* pixmap = create_pixmap(edit, 
						edit_x, 
						pixmap_x, 
						pixmap_w, 
						pixmap_h);
					pixmap->draw_data(edit,
						edit_x, 
						edit_w, 
						pixmap_x, 
						pixmap_w, 
						pixmap_h, 
						indexes_only ? 1 : 0);
//printf("TrackCanvas::draw_resources 8\n");
// Copy pixmap to background canvas
					background_pixmap->draw_pixmap(pixmap, 
						pixmap->pixmap_x, 
						current->y_pixel,
						pixmap->pixmap_w,
						edit_h);
				}
//printf("TrackCanvas::draw_resources 9\n");
			}
		}
	}
//printf("TrackCanvas::draw_resources 10\n");

// Delete unused pixmaps
	if(!indexes_only)
		for(int i = resource_pixmaps.total - 1; i >= 0; i--)
			if(resource_pixmaps.values[i]->visible < -5)
			{
				delete resource_pixmaps.values[i];
				resource_pixmaps.remove(resource_pixmaps.values[i]);
			}
}

ResourcePixmap* TrackCanvas::create_pixmap(Edit *edit, long edit_x, long pixmap_x, long pixmap_w, long pixmap_h)
{
	ResourcePixmap *result = 0;

	for(int i = 0; i < resource_pixmaps.total; i++)
	{
//printf("TrackCanvas::create_pixmap 1 %d %d\n", edit->id, resource_pixmaps.values[i]->edit->id);
		if(resource_pixmaps.values[i]->edit_id == edit->id) 
		{
			result = resource_pixmaps.values[i];
			break;
		}
	}

	if(!result)
	{
//printf("TrackCanvas::create_pixmap 2\n");
		result = new ResourcePixmap(mwindow, 
			this, 
			edit, 
			pixmap_w, 
			pixmap_h);
		resource_pixmaps.append(result);
	}

	result->resize(pixmap_w, pixmap_h);
	return result;
}

void TrackCanvas::get_pixmap_size(Edit *edit, 
	long edit_x, 
	long edit_w, 
	long &pixmap_x, 
	long &pixmap_w,
	long &pixmap_h)
{

// Fix media_x
	switch(edit->edits->track->data_type)
	{
		case TRACK_AUDIO:
			pixmap_x = edit_x;
			pixmap_w = edit_w;
			if(pixmap_x < 0)
			{
				pixmap_w -= 0 - edit_x;
				pixmap_x = 0;
			}

			if(pixmap_x + pixmap_w > get_w())
			{
				pixmap_w = get_w() - pixmap_x;
			}
			break;

		case TRACK_VIDEO:
		{
			long picon_w = (long)(edit->picon_w() + 0.5);
			long frame_w = (long)(edit->frame_w() + 0.5);
			long pixel_increment = MAX(picon_w, frame_w);
			long pixmap_x1 = edit_x;
			long pixmap_x2 = edit_x + edit_w;

			if(pixmap_x1 < 0)
			{
				pixmap_x1 = (long)((double)-edit_x / pixel_increment) * 
					pixel_increment + 
					edit_x;
			}

			if(pixmap_x2 > get_w())
			{
				pixmap_x2 = (long)((double)(get_w() - edit_x) / pixel_increment + 1) * 
					pixel_increment + 
					edit_x;
			}
//printf("get_pixmap_size %d %d %d\n", pixel_increment, pixmap_x1, pixmap_x2);
			pixmap_x = pixmap_x1;
			pixmap_w = pixmap_x2 - pixmap_x1;
			break;
		}
	}
	pixmap_h = mwindow->edl->session->zoom_track;
	if(mwindow->edl->session->show_titles) pixmap_h += mwindow->theme->title_bg_data->get_h();
//printf("get_pixmap_size %d %d %d %d\n", edit_x, edit_w, pixmap_x, pixmap_w);
}

void TrackCanvas::edit_dimensions(Edit *edit, 
	long &x, 
	long &y, 
	long &w, 
	long &h)
{
//printf("TrackCanvas::edit_dimensions 1 %p\n", edit->track);
	w = (long)(Units::round(edit->track->from_units(edit->length) * 
		mwindow->edl->session->sample_rate / 
		mwindow->edl->session->zoom_sample));
//printf("TrackCanvas::edit_dimensions 1\n");

	h = resource_h();

//printf("TrackCanvas::edit_dimensions 1\n");
	x = (long)(Units::round((edit->track->from_units(edit->startproject) - 
		mwindow->edl->session->view_start) *
		mwindow->edl->session->sample_rate / 
		mwindow->edl->session->zoom_sample));

//printf("TrackCanvas::edit_dimensions 1\n");
	y = edit->edits->track->y_pixel;
//printf("TrackCanvas::edit_dimensions 1\n");

	if(mwindow->edl->session->show_titles) 
		h += mwindow->theme->title_bg_data->get_h();
//printf("TrackCanvas::edit_dimensions 2\n");
}

void TrackCanvas::track_dimensions(Track *track, long &x, long &y, long &w, long &h)
{
	x = 0;
	w = get_w();
	y = track->y_pixel;
	h = track->vertical_span(mwindow->theme);
}

void TrackCanvas::plugin_dimensions(Plugin *plugin, long &x, long &y, long &w, long &h)
{
	x = (long)(Units::round(plugin->track->from_units(plugin->startproject) - 
		mwindow->edl->session->view_start *
		mwindow->edl->session->sample_rate / 
		mwindow->edl->session->zoom_sample));
	w = (long)(Units::round(plugin->track->from_units(plugin->length) *
		mwindow->edl->session->sample_rate / 
		mwindow->edl->session->zoom_sample));
	y = plugin->track->y_pixel + 
			mwindow->edl->session->zoom_track +
			plugin->plugin_set->get_number() * 
			mwindow->theme->plugin_bg_data->get_h();
	if(mwindow->edl->session->show_titles)
		y += mwindow->theme->title_bg_data->get_h();
	h = mwindow->theme->plugin_bg_data->get_h();
}

int TrackCanvas::resource_h()
{
	return mwindow->edl->session->zoom_track;
}

void TrackCanvas::draw_highlight_rectangle(int x, int y, int w, int h)
{
	x = MAX(x, -10);
	y = MAX(y, -10);
	w = MIN(w, get_w() + 20);
	h = MIN(h, get_h() + 20);
	set_color(WHITE);
	set_inverse();
	draw_rectangle(x, y, w, h);
	draw_rectangle(x + 1, y + 1, w - 2, h - 2);
	set_opaque();
//printf("TrackCanvas::draw_highlight_rectangle %d %d %d %d\n", x, y, w, h);
}

void TrackCanvas::draw_playback_cursor()
{
// Called before playback_cursor exists
// 	if(mwindow->playback_cursor && mwindow->playback_cursor->visible)
// 	{
// 		mwindow->playback_cursor->visible = 0;
// 		mwindow->playback_cursor->draw();
// 	}
}

void TrackCanvas::get_handle_coords(Edit *edit, long &x, long &y, long &w, long &h, int side)
{
	int handle_w = mwindow->theme->edithandlein_data[0]->get_w();
	int handle_h = mwindow->theme->edithandlein_data[0]->get_h();

	edit_dimensions(edit, x, y, w, h);

	if(mwindow->edl->session->show_titles)
	{
		y += mwindow->theme->title_bg_data->get_h();
	}
	else
	{
		y = 0;
	}

	if(side == EDIT_OUT)
	{
		x += w - handle_w;
	}

	h = handle_h;
	w = handle_w;
}

void TrackCanvas::get_transition_coords(long &x, long &y, long &w, long &h)
{
	int transition_w = mwindow->theme->transitionhandle_data[0]->get_w();
	int transition_h = mwindow->theme->transitionhandle_data[0]->get_h();

	if(mwindow->edl->session->show_titles)
		y += mwindow->theme->title_bg_data->get_h();

	y += (h - y) / 2 - transition_h / 2;
	x -= transition_w / 2;

	h = transition_h;
	w = transition_w;
}

void TrackCanvas::draw_highlighting()
{
	int draw_box = 0;
	int done = 0;

	if(mwindow->session->edit_highlighted)
	{
		long x, y, w, h;
		edit_dimensions(mwindow->session->edit_highlighted, x, y, w, h);

		if(MWindowGUI::visible(x, x + w, 0, get_w()) &&
			MWindowGUI::visible(y, y + h, 0, get_h()))
		{
			switch(mwindow->session->current_operation)
			{
				case DRAG_ATRANSITION:
					if(mwindow->session->track_highlighted->data_type == TRACK_AUDIO)
					{
						draw_box = 1;
						get_transition_coords(x, y, w, h);
						done = 1;
					}
					break;

				case DRAG_VTRANSITION:
					if(mwindow->session->track_highlighted->data_type == TRACK_VIDEO)
					{
						draw_box = 1;
						get_transition_coords(x, y, w, h);
						done = 1;
					}
					break;
				
				case DRAG_ASSET:
					break;
			}

			if(draw_box)
			{
				draw_highlight_rectangle(x, y, w, h);
			}
		}
	}

	if(mwindow->session->track_highlighted && !done)
	{
		long x, y, w, h;
		track_dimensions(mwindow->session->track_highlighted, x, y, w, h);

		if(MWindowGUI::visible(x, x + w, 0, get_w()) &&
			MWindowGUI::visible(y, y + h, 0, get_h()))
		{
			switch(mwindow->session->current_operation)
			{
				case DRAG_ASSET:
					break;

				case DRAG_AEFFECT:
					if(mwindow->session->track_highlighted->data_type == TRACK_AUDIO)
					{
						done = 1;
						x = 0;
						draw_highlight_rectangle(x, y, w, h);
					}
					break;

				case DRAG_VEFFECT:
					if(mwindow->session->track_highlighted->data_type == TRACK_VIDEO)
					{
						done = 1;
						x = 0;
						draw_highlight_rectangle(x, y, w, h);
					}
					break;
			}
		}
	}
}

void TrackCanvas::draw_plugins()
{
	char string[BCTEXTLEN];

	for(Track *track = mwindow->edl->tracks->first;
		track;
		track = track->next)
	{
//printf("TrackCanvas::draw_plugins 1\n");
		if(track->expand_view)
		{
//printf("TrackCanvas::draw_plugins 2\n");
			for(int i = 0; i < track->plugin_set.total; i++)
			{
				PluginSet *pluginset = track->plugin_set.values[i];
//printf("TrackCanvas::draw_plugins 3\n");
				
				for(Plugin *plugin = (Plugin*)pluginset->first; plugin; plugin = (Plugin*)plugin->next)
				{
					long x, y, w, h;
					plugin_dimensions(plugin, x, y, w, h);
					
					if(MWindowGUI::visible(x, x + w, 0, get_w()) &&
						MWindowGUI::visible(y, y + h, 0, get_h()) &&
						plugin->plugin_type != PLUGIN_NONE)
					{
//printf("TrackCanvas::draw_plugins 4\n");
						draw_3segment(x, 
							y, 
							w, 
							mwindow->theme->plugin_bg_data,
							0);
//printf("TrackCanvas::draw_plugins 4\n");
						set_color(WHITE);
//printf("TrackCanvas::draw_plugins 4\n");
						set_font(MEDIUMFONT_3D);
//printf("TrackCanvas::draw_plugins 4 %p\n", plugin);
						plugin->calculate_title(string);
//printf("TrackCanvas::draw_plugins 4\n");
						draw_text(x + 10, 
							y + get_text_ascent(MEDIUMFONT_3D) + 2, 
							string,
							strlen(string),
							0);
//printf("TrackCanvas::draw_plugins 5\n");
					}
				}
			}
		}
	}
}


void TrackCanvas::draw_inout_points()
{
}


void TrackCanvas::draw_drag_handle()
{
	if(mwindow->session->current_operation == DRAG_EDITHANDLE2)
	{
//printf("TrackCanvas::draw_drag_handle 1 %ld %ld\n", mwindow->session->drag_sample, mwindow->edl->session->view_start);
		long pixel1 = (long)((mwindow->session->drag_position - mwindow->edl->session->view_start) *
			mwindow->edl->session->sample_rate / 
			mwindow->edl->session->zoom_sample + 0.5);
//printf("TrackCanvas::draw_drag_handle 2 %d\n", pixel1);
		set_color(GREEN);
		set_inverse();
//printf("TrackCanvas::draw_drag_handle 3\n");
		draw_line(pixel1, 0, pixel1, get_h());
		set_opaque();
//printf("TrackCanvas::draw_drag_handle 4\n");
	}
}


void TrackCanvas::draw_transitions()
{
	long x, y, w, h;

	for(Track *track = mwindow->edl->tracks->first;
		track;
		track = track->next)
	{
		for(Edit *edit = track->edits->first;
			edit;
			edit = edit->next)
		{
			if(edit->transition)
			{
				edit_dimensions(edit, x, y, w, h);
				get_transition_coords(x, y, w, h);

				if(MWindowGUI::visible(x, x + w, 0, get_w()) &&
					MWindowGUI::visible(y, y + h, 0, get_h()))
				{
					PluginServer *server = mwindow->scan_plugindb(edit->transition->title);
					draw_vframe(server->picon, 
						x, 
						y, 
						w, 
						h, 
						0, 
						0, 
						server->picon->get_w(), 
						server->picon->get_h());
				}
			}
		}
	}
}

void TrackCanvas::draw_loop_points()
{
//printf("TrackCanvas::draw_loop_points 1\n");
	if(mwindow->edl->session->loop_playback)
	{
//printf("TrackCanvas::draw_loop_points 2\n");
		long x = (long)(Units::round(((double)mwindow->edl->session->loop_start - 
			mwindow->edl->session->view_start) /
			mwindow->edl->session->zoom_sample));
//printf("TrackCanvas::draw_loop_points 3\n");

		if(MWindowGUI::visible(x, x + 1, 0, get_w()))
		{
			set_color(GREEN);
			draw_line(x, 0, x, get_h());
		}
//printf("TrackCanvas::draw_loop_points 4\n");

		x = (long)(Units::round(((double)mwindow->edl->session->loop_end - 
			mwindow->edl->session->view_start) /
			mwindow->edl->session->zoom_sample));
//printf("TrackCanvas::draw_loop_points 5\n");

		if(MWindowGUI::visible(x, x + 1, 0, get_w()))
		{
			set_color(GREEN);
			draw_line(x, 0, x, get_h());
		}
//printf("TrackCanvas::draw_loop_points 6\n");
	}
//printf("TrackCanvas::draw_loop_points 7\n");
}

int TrackCanvas::do_keyframes(int cursor_x, 
	int cursor_y, 
	int test_only, 
	int buttonpress)
{
	int current_tool = 0;
	int result = 0;
	EDLSession *session = mwindow->edl->session;

	for(Track *track = mwindow->edl->tracks->first;
		track && !result;
		track = track->next)
	{
		Automation *automation = track->automation;

//printf("TrackCanvas::draw_keyframes 1\n");
		if(session->auto_conf->fade)
		{
			if(!test_only) set_color(WHITE);
			result = do_float_autos(track, 
				automation->fade_autos,
				cursor_x, 
				cursor_y, 
				test_only, 
				buttonpress);
		}
//printf("TrackCanvas::draw_keyframes 1 %d\n", result);
		if(!result && session->auto_conf->play)
		{
			if(!test_only) set_color(GREEN);
			result = do_toggle_autos(track, 
				automation->play_autos,
				cursor_x, 
				cursor_y, 
				test_only, 
				buttonpress);
		}
//printf("TrackCanvas::draw_keyframes 1 %d\n", result);
		if(!result && session->auto_conf->mute)
		{
			if(!test_only) set_color(WHITE);
			result = do_toggle_autos(track, 
				automation->mute_autos,
				cursor_x, 
				cursor_y, 
				test_only, 
				buttonpress);
		}
//printf("TrackCanvas::draw_keyframes 1 %d\n", result);
		if(!result && session->auto_conf->camera && automation->camera_autos)
		{
			result = do_autos(track, 
				automation->camera_autos,
				cursor_x, 
				cursor_y, 
				test_only, 
				buttonpress);
		}
		if(!result && session->auto_conf->mode && automation->mode_autos)
		{
			result = do_autos(track, 
				automation->mode_autos,
				cursor_x, 
				cursor_y, 
				test_only, 
				buttonpress);
		}
//printf("TrackCanvas::draw_keyframes 1 %d\n", result);
		if(!result && session->auto_conf->projector && automation->projector_autos)
		{
			result = do_autos(track, 
				automation->projector_autos,
				cursor_x, 
				cursor_y, 
				test_only, 
				buttonpress);
		}
//printf("TrackCanvas::draw_keyframes 1 %d\n", result);
		if(!result && session->auto_conf->pan && automation->pan_autos)
		{
			result = do_autos(track, 
				automation->pan_autos,
				cursor_x, 
				cursor_y, 
				test_only, 
				buttonpress);
		}
//printf("TrackCanvas::draw_keyframes 1 %d\n", result);
		if(!result && session->auto_conf->plugins)
		{
			result = do_plugin_autos(track,
				cursor_x, 
				cursor_y, 
				test_only, 
				buttonpress);
		}
//printf("TrackCanvas::draw_keyframes 1 %d\n", result);
	}
	
//printf("TrackCanvas::draw_keyframes 2 %d\n", result);
	if(result)
	{
		if(get_cursor() != UPRIGHT_ARROW_CURSOR)
		{
			set_cursor(UPRIGHT_ARROW_CURSOR);
		}
	}
	else
	if(get_cursor() == UPRIGHT_ARROW_CURSOR)
	{
		update_cursor();
	}

//printf("TrackCanvas::do_keyframes 3 %d\n", result);
	return result;
}

void TrackCanvas::draw_auto(Auto *current, 
	int x, 
	int y, 
	int center_pixel, 
	int zoom_track, 
	int show_caption)
{
	int x1, y1, x2, y2;
	char string[BCTEXTLEN];

	x1 = x - HANDLE_W / 2;
	x2 = x + HANDLE_W / 2;
	y1 = y - HANDLE_W / 2;
	y2 = y + HANDLE_W / 2;

	if(y1 < -zoom_track / 2) y1 = -zoom_track / 2;
	if(y2 > zoom_track / 2) y2 = zoom_track / 2;

	draw_box(x1, center_pixel + y1, x2 - x1, y2 - y1);

	if(show_caption)
	{
		current->get_caption(string);
		if(y > 0)
		{
			draw_text(x + 5, center_pixel + y - 5, string);
		}
		else
		{
			draw_text(x + 5, center_pixel + y + 20, string);
		}
	}
}

int TrackCanvas::test_auto(Auto *current, 
	int x, 
	int y, 
	int center_pixel, 
	int zoom_track, 
	int cursor_x, 
	int cursor_y, 
	int buttonpress)
{
	int x1, y1, x2, y2;
	char string[BCTEXTLEN];
	int result = 0;

	x1 = x - HANDLE_W / 2;
	x2 = x + HANDLE_W / 2;
	y1 = center_pixel + y - HANDLE_W / 2;
	y2 = center_pixel + y + HANDLE_W / 2;

	if(y1 < -zoom_track / 2) y1 = -zoom_track / 2;
	if(y2 > zoom_track / 2) y2 = zoom_track / 2;

	if(cursor_x >= x1 && cursor_x < x2 && cursor_y >= y1 && cursor_y < y2)
	{
		return 1;
	}

	return result;
}

void TrackCanvas::draw_floatline(int center_pixel, 
	int x1,
	int y1,
	int x2,
	int y2)
{
	draw_line(x1, center_pixel + y1, x2, center_pixel + y2);
}

int TrackCanvas::test_floatline(int center_pixel, 
	int x1,
	int y1,
	int x2,
	int y2, 
	int cursor_x, 
	int cursor_y, 
	int buttonpress)
{
	int result = 0;
	if(cursor_x >= x1 && cursor_x < x2)
	{
		float slope = (float)(y2 - y1) / (x2 - x1);
		int miny = center_pixel + y1 + (int)((cursor_x - x1) * slope) - HANDLE_W / 2;
		int maxy = miny + HANDLE_W;
	
		if(cursor_y >= miny && cursor_y < maxy)
		{
			result = 1;
		}
	}

	return result;
}

void TrackCanvas::draw_toggleline(int center_pixel, 
	int x1,
	int y1,
	int x2,
	int y2)
{
	draw_line(x1, center_pixel + y1, x2, center_pixel + y1);
	if(y2 != y1)
	{
		draw_line(x2, center_pixel + y1, x2, center_pixel + y2);
	}
}

int TrackCanvas::test_toggleline(int center_pixel, 
	int x1,
	int y1,
	int x2,
	int y2, 
	int cursor_x, 
	int cursor_y, 
	int buttonpress)
{
	int result = 0;
	if(cursor_x >= x1 && cursor_x < x2)
	{
		int miny = center_pixel + y1 - HANDLE_W / 2;
		int maxy = center_pixel + y1 + HANDLE_W / 2;
		if(cursor_y >= miny && cursor_y < maxy) result = 1;
	}
	return result;
}

void TrackCanvas::calculate_viewport(Track *track, 
	double &view_start,
	long &unit_start,
	double &view_end,
	long &unit_end,
	double &yscale,
	int &center_pixel,
	double &zoom_sample,
	double &zoom_units)
{
	view_start = mwindow->edl->session->view_start;
	unit_start = track->to_units(view_start, 0);
	view_end = view_start + 
		get_w() * 
		mwindow->edl->session->zoom_sample / 
		mwindow->edl->session->sample_rate;
	unit_end = (long)(track->to_units(view_end, 1));
	yscale = mwindow->edl->session->zoom_track;
	center_pixel = (int)(track->y_pixel + yscale / 2) + 
		(mwindow->edl->session->show_titles ? 
			mwindow->theme->title_bg_data->get_h() : 
			0);
	zoom_sample = mwindow->edl->session->zoom_sample;
	zoom_units = track->to_units(zoom_sample / mwindow->edl->session->sample_rate, 1);
}

int TrackCanvas::do_float_autos(Track *track, 
		Autos *autos, 
		int cursor_x, 
		int cursor_y, 
		int test_only, 
		int buttonpress)
{
	int result = 0;

	double view_start;
	long unit_start;
	double view_end;
	long unit_end;
	double yscale;
	int center_pixel;
	double zoom_sample;
	double zoom_units;
	double ax, ay, ax2, ay2;
	double slope;
	
	calculate_viewport(track, 
		view_start,
		unit_start,
		view_end,
		unit_end,
		yscale,
		center_pixel,
		zoom_sample,
		zoom_units);


// Get first auto before start
	Auto *current;
	for(current = autos->last; current && current->position >= unit_start; current = PREVIOUS)
		;

	if(current)
	{
		ax = (double)(current->position - unit_start) / zoom_units;
		ay = ((FloatAuto*)current)->value_to_percentage() * -yscale;
		current = NEXT;
	}
	else
	{
		current = autos->first ? autos->first : autos->default_auto;
		if(current)
		{		
			ax = 0;
			ay = ((FloatAuto*)current)->value_to_percentage() * -yscale;
		}
		else
		{
			ax = 0;
			ay = 0;
		}
	}

	do
	{
		if(current)
		{
			ax2 = (double)(current->position - unit_start) / zoom_units;
			ay2 = ((FloatAuto*)current)->value_to_percentage() * -yscale;
		}
		else
		{
			ax2 = get_w();
			ay2 = ay;
		}

		slope = (ay2 - ay) / (ax2 - ax);

		if(ax2 > get_w())
		{
			ax2 = get_w();
			ay2 = ay + slope * (get_w() - ax);
		}
		
		if(ax < 0)
		{
			ay = ay + slope * (0 - ax);
			ax = 0;
		}

		if(current && !result)
		{
			if(test_only)
				result = test_auto(current, 
					(int)ax2, 
					(int)ay2, 
					(int)center_pixel, 
					(int)yscale, 
					cursor_x, 
					cursor_y, 
					buttonpress);
			else
				draw_auto(current, 
					(int)ax2, 
					(int)ay2, 
					(int)center_pixel, 
					(int)yscale, 
					0);

			current = NEXT;
		}

		if(test_only)
		{
			if(!result)
				result = test_floatline(center_pixel, 
					(int)ax, 
					(int)ay, 
					(int)ax2, 
					(int)ay2,
					cursor_x, 
					cursor_y, 
					buttonpress);
		}
		else
			draw_floatline(center_pixel, 
				(int)ax, 
				(int)ay, 
				(int)ax2, 
				(int)ay2);
		ax = ax2;
		ay = ay2;
	}while(current && current->position <= unit_end && !result);

	if(ax < get_w() && !result)
	{
		ax2 = get_w();
		ay2 = ay;
		if(test_only)
			result = test_floatline(center_pixel, 
				(int)ax, 
				(int)ay, 
				(int)ax2, 
				(int)ay2,
				cursor_x, 
				cursor_y, 
				buttonpress);
		else
			draw_floatline(center_pixel, 
				(int)ax, 
				(int)ay, 
				(int)ax2, 
				(int)ay2);
	}
	return result;
}


int TrackCanvas::do_toggle_autos(Track *track, 
		Autos *autos, 
		int cursor_x, 
		int cursor_y, 
		int test_only, 
		int buttonpress)
{
	int result = 0;
	double view_start;
	long unit_start;
	double view_end;
	long unit_end;
	double yscale;
	int center_pixel;
	double zoom_sample;
	double zoom_units;
	double ax, ay, ax2, ay2;

	calculate_viewport(track, 
		view_start,
		unit_start,
		view_end,
		unit_end,
		yscale,
		center_pixel,
		zoom_sample,
		zoom_units);


	double high = -yscale * 0.8 / 2;
	double low = yscale * 0.8 / 2;

// Get first auto before start
	Auto *current;
	for(current = autos->last; current && current->position >= unit_start; current = PREVIOUS)
		;

	if(current)
	{
		ax = 0;
		ay = ((IntAuto*)current)->value > 0 ? high : low;
		current = NEXT;
	}
	else
	{
		current = autos->first ? autos->first : autos->default_auto;
		if(current)
		{
			ax = 0;
			ay = ((IntAuto*)current)->value > 0 ? high : low;
		}
		else
		{
			ax = 0;
			ay = yscale;
		}
	}

	do
	{
		if(current)
		{
			ax2 = (double)(current->position - unit_start) / zoom_units;
			ay2 = ((IntAuto*)current)->value > 0 ? high : low;
		}
		else
		{
			ax2 = get_w();
			ay2 = ay;
		}

		if(ax2 > get_w()) ax2 = get_w();

	    if(current && !result) 
		{
			if(test_only)
				result = test_auto(current, 
					(int)ax2, 
					(int)ay2, 
					(int)center_pixel, 
					(int)yscale, 
					cursor_x, 
					cursor_y, 
					buttonpress);
			else
				draw_auto(current, 
					(int)ax2, 
					(int)ay2, 
					(int)center_pixel, 
					(int)yscale, 
					0);
			current = NEXT;
		}

		if(test_only)
		{
			if(!result)
				result = test_toggleline(center_pixel, 
					(int)ax, 
					(int)ay, 
					(int)ax2, 
					(int)ay2,
					cursor_x, 
					cursor_y, 
					buttonpress);
		}
		else
			draw_toggleline(center_pixel, 
				(int)ax, 
				(int)ay, 
				(int)ax2, 
				(int)ay2);

		ax = ax2;
		ay = ay2;
	}while(current && current->position <= unit_end && !result);

	if(ax < get_w() && !result)
	{
		ax2 = get_w();
		ay2 = ay;
		if(test_only)
			result = test_toggleline(center_pixel, 
				(int)ax, 
				(int)ay, 
				(int)ax2, 
				(int)ay2,
				cursor_x, 
				cursor_y, 
				buttonpress);
		else
			draw_toggleline(center_pixel, 
				(int)ax, 
				(int)ay, 
				(int)ax2, 
				(int)ay2);
	}
	return result;
}

int TrackCanvas::do_autos(Track *track, 
		Autos *autos, 
		int cursor_x, 
		int cursor_y, 
		int test_only, 
		int buttonpress)
{
	int result = 0;

	double view_start;
	long unit_start;
	double view_end;
	long unit_end;
	double yscale;
	int center_pixel;
	double zoom_sample;
	double zoom_units;

	calculate_viewport(track, 
		view_start,
		unit_start,
		view_end,
		unit_end,
		yscale,
		center_pixel,
		zoom_sample,
		zoom_units);

	Auto *current;

	for(current = autos->first; current && !result; current = NEXT)
	{
		if(current->position >= unit_start && current->position < unit_end)
		{
			long x, y;
			x = (long)((current->position - unit_start) / zoom_units);
			y = center_pixel - keyframe_pixmap->get_h() / 2;

			if(test_only)
			{
				if(cursor_x >= x && cursor_y >= y &&
					cursor_x < x + keyframe_pixmap->get_w() &&
					cursor_y < y + keyframe_pixmap->get_h()) result = 1;
			}
			else
				draw_pixmap(keyframe_pixmap, x, y);
		}
	}
	return result;
}

int TrackCanvas::do_plugin_autos(Track *track, 
		int cursor_x, 
		int cursor_y, 
		int test_only, 
		int buttonpress)
{
	int result = 0;

	double view_start;
	long unit_start;
	double view_end;
	long unit_end;
	double yscale;
	int center_pixel;
	double zoom_sample;
	double zoom_units;

	calculate_viewport(track, 
		view_start,
		unit_start,
		view_end,
		unit_end,
		yscale,
		center_pixel,
		zoom_sample,
		zoom_units);

//printf("TrackCanvas::draw_plugin_autos 1 %d\n", track->plugin_set.total);
	for(int i = 0; i < track->plugin_set.total && !result; i++)
	{
		PluginSet *plugin_set = track->plugin_set.values[i];
		int center_pixel = (int)(track->y_pixel + 
			mwindow->edl->session->zoom_track +
			(i + 0.5) * mwindow->theme->plugin_bg_data->get_h() + 
			(mwindow->edl->session->show_titles ? mwindow->theme->title_bg_data->get_h() : 0));
		
//printf("TrackCanvas::draw_plugin_autos 2\n");
		for(Plugin *plugin = (Plugin*)plugin_set->first; 
			plugin && !result; 
			plugin = (Plugin*)plugin->next)
		{
			for(KeyFrame *keyframe = plugin->keyframes->first; 
				keyframe && !result; 
				keyframe = keyframe->next)
			{
//printf("TrackCanvas::draw_plugin_autos 3 %d\n", keyframe->position);
				if(keyframe->position >= unit_start && keyframe->position < unit_end)
				{
					long x = (long)((keyframe->position - unit_start) / zoom_units);
					int y = center_pixel - keyframe_pixmap->get_h() / 2;

//printf("TrackCanvas::draw_plugin_autos 4 %d %d\n", x, center_pixel);
					if(test_only)
					{
						if(cursor_x >= x && cursor_y >= y &&
							cursor_x < x + keyframe_pixmap->get_w() &&
							cursor_y < y + keyframe_pixmap->get_h()) result = 1;
					}
					else
						draw_pixmap(keyframe_pixmap, 
							x, 
							y);
				}
			}
		}
	}
//printf("TrackCanvas::draw_plugin_autos 5\n");
	return result;
}

void TrackCanvas::draw_overlays()
{
// Move background pixmap to foreground pixmap
	draw_pixmap(background_pixmap, 
		0, 
		0,
		get_w(),
		get_h(),
		0,
		0);

// Loop points
	draw_loop_points();

// Highlighted areas
	draw_highlighting();

// In/Out points
	draw_inout_points();

// Transitions
	if(mwindow->edl->session->auto_conf->transitions) draw_transitions();
//	transition_handles->update();

// Plugins
	draw_plugins();

// Automation
	do_keyframes(0, 0, 0, 0);

// Selection cursor
	if(gui->cursor) gui->cursor->restore();

// Handle dragging
	draw_drag_handle();

// Playback cursor
	draw_playback_cursor();
}

int TrackCanvas::activate()
{
	if(!active)
	{
		get_top_level()->deactivate();
		active = 1;
		set_active_subwindow(this);
		gui->cursor->activate();
	}
	return 0;
}

int TrackCanvas::deactivate()
{
	if(active)
	{
		active = 0;
		gui->cursor->deactivate();
	}
	return 0;
}


void TrackCanvas::update_drag_edithandle()
{
	double new_position;

	new_position = 
		(double)get_cursor_x() *
		mwindow->edl->session->zoom_sample /
		mwindow->edl->session->sample_rate + 
		mwindow->edl->session->view_start;
	new_position = 
		mwindow->edl->align_to_frame(new_position, 0);


	if(new_position != mwindow->session->drag_position)
	{
		mwindow->session->drag_position = new_position;
		mwindow->edl->local_session->selectionend = new_position;
		mwindow->edl->local_session->selectionstart = new_position;
		gui->mainclock->update(mwindow->edl->local_session->selectionstart);
// Que the CWindow
		mwindow->cwindow->update_position();
	}
}

int TrackCanvas::update_drag_edit()
{
	int result = 0;
	
	
	
	return result;
}

int TrackCanvas::cursor_motion_event()
{
	int result, cursor_x, cursor_y;
	int update_clock = 0, update_scroll = 0, update_overlay = 0, rerender = 0;
	double position = 0;
//printf("TrackCanvas::cursor_motion 1\n");
	result = 0;

	switch(mwindow->session->current_operation)
	{
		case DRAG_EDITHANDLE1:
// Outside threshold.  Upgrade status
//printf("TrackCanvas::cursor_motion_event 1\n");
			if(labs(get_cursor_x() - mwindow->session->drag_origin_x) > HANDLE_W)
			{
//printf("TrackCanvas::cursor_motion_event 2\n");
				mwindow->session->current_operation = DRAG_EDITHANDLE2;
				update_overlay = 1;
			}
			break;

		case DRAG_EDITHANDLE2:
			update_drag_edithandle();
			update_overlay = 1;
			break;

		case DRAG_EDIT1:
			if(labs(get_cursor_x() - mwindow->session->drag_origin_x) > HANDLE_W ||
				labs(get_cursor_y() - mwindow->session->drag_origin_y) > HANDLE_H)
			{
				mwindow->session->current_operation = DRAG_EDIT2;
				update_overlay = 1;
			}
			break;
		
		case DRAG_EDIT2:
			update_overlay =  update_drag_edit();
			break;

		case SELECT_REGION:
		{
			cursor_x = get_cursor_x();
			cursor_y = get_cursor_y();
			position = (double)cursor_x * 
				mwindow->edl->session->zoom_sample /
				mwindow->edl->session->sample_rate + 
				mwindow->edl->session->view_start;

			position = mwindow->edl->align_to_frame(position, 0);
			position = MAX(position, 0);

			if(position < selection_midpoint1)
			{
				mwindow->edl->local_session->selectionend = selection_midpoint1;
				mwindow->edl->local_session->selectionstart = position;
// Que the CWindow
				mwindow->cwindow->update_position();
			}
			else
			{
				mwindow->edl->local_session->selectionstart = selection_midpoint1;
				mwindow->edl->local_session->selectionend = position;
// Don't que the CWindow
			}

			gui->cursor->hide();
			gui->cursor->draw();
			flash();
			result = 1;
			update_clock = 1;
			update_scroll = 1;
			break;
		}

		default:
			if(is_event_win() && cursor_inside())
			{
// Update clocks
				cursor_x = get_cursor_x();
				position = cursor_x * 
					mwindow->edl->session->zoom_sample / 
					mwindow->edl->session->sample_rate + 
					mwindow->edl->session->view_start;
				position = mwindow->edl->align_to_frame(position, 0);
				update_clock = 1;

// Update cursor
				if(do_keyframes(get_cursor_x(), get_cursor_y(), 1, 0))
				{
					break;
				}
				else
// Edit boundaries
				if(test_handles(get_cursor_x(), get_cursor_y(), 0))
				{
					break;
				}
				else
				if(test_edits(get_cursor_x(), get_cursor_y(), 0, update_overlay, rerender))
				{
					break;
				}
			}
			break;
	}

	if(update_clock)
	{
		gui->mainclock->update(position);
		gui->zoombar->update();
	}

	if(update_scroll)
	{
		if(!drag_scroll && 
			(cursor_x > get_w() || cursor_x < 0 || cursor_y > get_h() || cursor_y < 0))
			start_dragscroll();
		else
		if(drag_scroll)
			stop_dragscroll();
	}

	if(update_overlay)
	{
		draw_overlays();
		flash();
	}
	return result;
}

void TrackCanvas::start_dragscroll()
{
	if(!drag_scroll)
	{
		drag_scroll = 1;
		set_repeat(BC_WindowBase::get_resources()->scroll_repeat);
	}
}

void TrackCanvas::stop_dragscroll()
{
	if(drag_scroll)
	{
		drag_scroll = 0;
		unset_repeat(BC_WindowBase::get_resources()->scroll_repeat);
	}
}

int TrackCanvas::repeat_event(long duration)
{
	if(!drag_scroll) return 0;
	if(duration != BC_WindowBase::get_resources()->scroll_repeat) return 0;

	switch(mwindow->session->current_operation)
	{
		case SELECT_REGION:
			if(get_cursor_x() > get_w())
			{
				long distance = (get_cursor_x() - get_w()) * mwindow->edl->session->zoom_sample;
				mwindow->samplemovement(mwindow->edl->session->view_start + distance);
			}
			else
			if(get_cursor_x() < 0)
			{
				long distance = -get_cursor_x() * mwindow->edl->session->zoom_sample;
				mwindow->samplemovement(mwindow->edl->session->view_start - distance);
			}
			else
			if(get_cursor_y() > get_h())
			{
				long distance = get_cursor_y() - get_h();
				mwindow->trackmovement(mwindow->edl->session->track_start + distance);
			}
			else
			if(get_cursor_y() < 0)
			{
				long distance = -get_cursor_y();
				mwindow->trackmovement(mwindow->edl->session->track_start - distance);
			}
			break;
	}
	return 0;
}

int TrackCanvas::button_release_event()
{
	int redraw = 0, update_overlay = 0, result = 0;
//printf("TrackCanvas::button_release_event 1 %d\n", mwindow->session->current_operation);
	switch(mwindow->session->current_operation)
	{
		case DRAG_EDITHANDLE2:
//printf("TrackCanvas::button_release_event 2\n");
			mwindow->session->current_operation = NO_OPERATION;
			drag_scroll = 0;
			result = 1;
			
			end_handle_selection();
			break;

		default:
			if(mwindow->session->current_operation)
			{
//printf("TrackCanvas::button_release_event 3\n");
				mwindow->session->current_operation = NO_OPERATION;
				drag_scroll = 0;
// Traps button release events
//				result = 1;
			}
			break;
	}
//printf("TrackCanvas::button_release_event 4 %d\n", result);

	if(update_overlay)
	{
		draw_overlays();
		flash();
	}
	if(redraw)
	{
		draw();
		flash();
	}
	return result;
}

int TrackCanvas::test_handles(int cursor_x, int cursor_y, int button_press)
{
	Edit *edit_result;
	int handle_result;
	int result = 0;
	edit_result = 0;
	handle_result = 0;

	for(Track *track = mwindow->edl->tracks->first;
		track;
		track = track->next)
	{
		for(Edit *edit = track->edits->first;
			edit && !result;
			edit = edit->next)
		{
			long edit_x, edit_y, edit_w, edit_h;
			edit_dimensions(edit, edit_x, edit_y, edit_w, edit_h);

			if(cursor_x >= edit_x && cursor_x < edit_x + edit_w &&
				cursor_y >= edit_y && cursor_y < edit_y + edit_h)
			{
				if(cursor_x < edit_x + HANDLE_W)
				{
					edit_result = edit;
					handle_result = 0;
					result = 1;
				}
				else
				if(cursor_x >= edit_x + edit_w - HANDLE_W)
				{
					edit_result = edit;
					handle_result = 1;
					result = 1;
				}
				else
				{
					result = 0;
				}
			}
		}
	}
	
	if(result)
	{
		double position;
		if(handle_result == 0)
		{
			position = edit_result->track->from_units(edit_result->startproject);

			if(get_cursor() != LEFT_CURSOR)
			{
				set_cursor(LEFT_CURSOR);
			}
		}
		else
		if(handle_result == 1)
		{
			position = edit_result->track->from_units(edit_result->startproject + edit_result->length);

			if(get_cursor() != RIGHT_CURSOR)
			{
				set_cursor(RIGHT_CURSOR);
			}
		}

// Reposition cursor
		if(button_press)
		{
			mwindow->session->drag_edit = edit_result;
			mwindow->session->drag_handle = handle_result;
			mwindow->session->drag_position = position;
			mwindow->session->current_operation = DRAG_EDITHANDLE1;
			mwindow->session->drag_origin_x = get_cursor_x();
			mwindow->session->drag_origin_y = get_cursor_y();

			int rerender = start_selection(position);
			if(rerender)
				mwindow->cwindow->update_position();
			gui->timebar->update_highlights();
			gui->zoombar->update();
			gui->cursor->hide();
			gui->cursor->draw();
			draw_overlays();
			flash();
			flush();
		}
	}
	else
	if(get_cursor() == LEFT_CURSOR || get_cursor() == RIGHT_CURSOR)
	{
		update_cursor();
	}

	return result;
}

int TrackCanvas::test_edits(int cursor_x, 
	int cursor_y, 
	int button_press,
	int &redraw,
	int &rerender)
{
	int result = 0;
	int over_edit_handle = 0;
	
	for(Track *track = mwindow->edl->tracks->first;
		track && !result;
		track = track->next)
	{
		for(Edit *edit = track->edits->first;
			edit;
			edit = edit->next)
		{
			long edit_x, edit_y, edit_w, edit_h;
//printf("TrackCanvas::test_edits 1\n");
			edit_dimensions(edit, edit_x, edit_y, edit_w, edit_h);
// Cursor inside an edit
			if(cursor_x >= edit_x && cursor_x < edit_x + edit_w &&
				cursor_y >= edit_y && cursor_y < edit_y + edit_h)
			{
// Select duration of edit
//printf("TrackCanvas::test_edits 2\n");
				if(button_press)
				{
					if(get_double_click())
					{
//printf("TrackCanvas::test_edits 3\n");
						mwindow->edl->local_session->selectionstart = 
							edit->track->from_units(edit->startproject);
						mwindow->edl->local_session->selectionend = 
							edit->track->from_units(edit->startproject) + 
							edit->track->from_units(edit->length);
						if(mwindow->edl->session->cursor_on_frames) 
						{
							mwindow->edl->local_session->selectionstart = 
								mwindow->edl->align_to_frame(mwindow->edl->local_session->selectionstart, 0);
							mwindow->edl->local_session->selectionend = 
								mwindow->edl->align_to_frame(mwindow->edl->local_session->selectionend, 1);
						}
						redraw = 1;
						rerender = 1;
						result = 1;
					}
					else
					if(get_buttonpress() != 3)
					{
						if(mwindow->edl->session->editing_mode == EDITING_ARROW)
						{
// Need to create drag window
							mwindow->session->current_operation = DRAG_EDIT1;
							mwindow->session->drag_edit = edit;
							mwindow->session->drag_origin_x = cursor_x;
							mwindow->session->drag_origin_y = cursor_y;
							result= 1;
						}
					}
					else
					if(get_buttonpress() == 3)
					{
						gui->edit_menu->update(edit);
						gui->edit_menu->activate_menu();
						result = 1;
					}
				}
			}
		}
	}
	return result;
}


int TrackCanvas::test_resources(int cursor_x, int cursor_y)
{
	return 0;
}

int TrackCanvas::test_plugins(int cursor_x, 
	int cursor_y, 
	int drag_start,
	int button_press)
{
	Plugin *plugin = 0;
	int done = 0;
	long x, y, w, h;

	for(Track *track = mwindow->edl->tracks->first;
		track && !done;
		track = track->next)
	{
		for(int i = 0; i < track->plugin_set.total && !done; i++)
		{
			PluginSet *plugin_set = track->plugin_set.values[i];
			for(plugin = (Plugin*)plugin_set->first;
				plugin && !done;
				plugin = (Plugin*)plugin->next)
			{
				plugin_dimensions(plugin, x, y, w, h);
				if(MWindowGUI::visible(x, x + w, 0, get_w()) &&
					MWindowGUI::visible(y, y + h, 0, get_h()))
				{
					if(cursor_x >= x && cursor_x < x + w &&
						cursor_y >= y && cursor_y < y + h)
					{
						done = 1;
						break;
					}
				}
			}
		}
	}

	if(plugin)
	{
// Start plugin popup
		if(button_press)
		{
			if(get_buttonpress() == 3)
			{
				gui->plugin_menu->update(plugin);
				gui->plugin_menu->activate_menu();
				return 1;
			}
			else
			switch(mwindow->edl->session->editing_mode)
			{
				case EDITING_ARROW:
					break;
				case EDITING_IBEAM:
					break;
			}
		}
		else
		if(drag_start)
		{
			if(mwindow->edl->session->editing_mode == EDITING_ARROW)
			{
				if(plugin->track->data_type == TRACK_AUDIO)
					mwindow->session->current_operation = DRAG_AEFFECT_COPY;
				else
				if(plugin->track->data_type == TRACK_VIDEO)
					mwindow->session->current_operation = DRAG_VEFFECT_COPY;

				mwindow->session->drag_plugin = plugin;
			}
		}
	}
	return 0;
}

int TrackCanvas::test_transitions(int cursor_x, int cursor_y)
{
	Transition *transition = 0;
	int done = 0;
	long x, y, w, h;
	
	for(Track *track = mwindow->edl->tracks->first;
		track && !done;
		track = track->next)
	{
		for(Edit *edit = track->edits->first;
			edit;
			edit = edit->next)
		{
			if(edit->transition)
			{
				edit_dimensions(edit, x, y, w, h);
				get_transition_coords(x, y, w, h);

				if(MWindowGUI::visible(x, x + w, 0, get_w()) &&
					MWindowGUI::visible(y, y + h, 0, get_h()))
				{
					if(cursor_x >= x && cursor_x < x + w &&
						cursor_y >= y && cursor_y < y + h)
					{
						transition = edit->transition;
						done = 1;
						break;
					}
				}
			}
		}
	}
	
	if(transition)
	{
		if(get_buttonpress() == 3)
		{
			gui->transition_menu->update(transition);
			gui->transition_menu->activate_menu();
			return 1;
		}
	}
	return 0;
}

int TrackCanvas::button_press_event()
{
	int result = 0;
	int cursor_x, cursor_y;

	cursor_x = get_cursor_x();
	cursor_y = get_cursor_y();

	if(is_event_win() && cursor_inside())
	{
		if(!active)
		{
			activate();
		}

		if(get_buttonpress() == 1)
		{
			gui->unlock_window();
			gui->mbuttons->transport->handle_transport(STOP, 1);
			gui->lock_window();
		}

		int redraw = 0, rerender = 0;
		switch(mwindow->edl->session->editing_mode)
		{
// Test handles and resource boundaries and highlight a track
			case EDITING_ARROW:
			{
				Edit *edit;
				int handle;
				if(test_transitions(cursor_x, cursor_y))
				{
					break;
				}
				else
// Test edit boundaries
				if(test_handles(cursor_x, cursor_y, 1))
				{
					break;
				}
				else
				if(test_edits(cursor_x, cursor_y, 1, redraw, rerender))
				{
					break;
				}
				else
				if(test_plugins(cursor_x, cursor_y, 0, 1))
				{
					break;
				}
				else
				if(test_resources(cursor_x, cursor_y))
				{
					break;
				}
				else
				{
					break;
				}
				break;
			}

// Test handles only and select a region
			case EDITING_IBEAM:
			{
				double position = (double)cursor_x * 
					mwindow->edl->session->zoom_sample /
					mwindow->edl->session->sample_rate + 
					mwindow->edl->session->view_start;
//printf("TrackCanvas::button_press_event %d\n", position);

				if(test_transitions(cursor_x, cursor_y))
				{
					break;
				}
				else
// Test edit boundaries
				if(test_handles(cursor_x, cursor_y, 1))
				{
					break;
				}
				else
				if(test_edits(cursor_x, cursor_y, 1, redraw, rerender))
				{
					break;
				}
				else
				if(test_plugins(cursor_x, cursor_y, 0, 1))
				{
					break;
				}
				else
// Highlight selection
				{
					rerender = start_selection(position);
					mwindow->session->current_operation = SELECT_REGION;
					redraw = 1;
				}

				break;
			}
		}


		if(rerender)
			mwindow->cwindow->update_position();

		if(redraw)
		{
			gui->timebar->update_highlights();
			gui->cursor->hide();
			gui->cursor->show();
			gui->zoombar->update();
			flash();
			result = 1;
		}



	}
// 	else
// 	if(active)
// 		deactivate();
	return result;
}

int TrackCanvas::start_selection(double position)
{
	int rerender = 0;
	position = mwindow->edl->align_to_frame(position, 0);

// Extend a border
	if(shift_down())
	{
		double midpoint = (mwindow->edl->local_session->selectionstart + 
			mwindow->edl->local_session->selectionend) / 2;

		if(position < midpoint)
		{
			mwindow->edl->local_session->selectionstart = position;
			selection_midpoint1 = mwindow->edl->local_session->selectionend;
// Que the CWindow
			rerender = 1;
		}
		else
		{
			mwindow->edl->local_session->selectionend = position;
			selection_midpoint1 = mwindow->edl->local_session->selectionstart;
// Don't que the CWindow for the end
		}
	}
	else
// Start a new selection
	{
//printf("TrackCanvas::start_selection %f\n", position);
		mwindow->edl->local_session->selectionstart = 
			mwindow->edl->local_session->selectionend = 
			position;
		selection_midpoint1 = position;
// Que the CWindow
		rerender = 1;
	}
	
	return rerender;
}

void TrackCanvas::end_handle_selection()
{
//printf("TrackCanvas::end_handle_selection 1\n");
	mwindow->undo->update_undo_before("drag handle", LOAD_EDITS | LOAD_TIMEBAR);
//printf("TrackCanvas::end_handle_selection 1\n");
	mwindow->modify_handles(mwindow->session->drag_start, 
		mwindow->session->drag_position, 
		mwindow->session->drag_handle, 
		mwindow->edl->session->edit_handle_mode[get_buttonpress() - 1]);
//printf("TrackCanvas::end_handle_selection 1\n");
	if(mwindow->session->drag_handle == 1) 
		mwindow->edl->local_session->selectionstart = 
			mwindow->edl->local_session->selectionend = 
			mwindow->session->drag_position;
	else
		mwindow->edl->local_session->selectionstart = 
			mwindow->edl->local_session->selectionend = 
			mwindow->session->drag_start;

//printf("TrackCanvas::end_handle_selection 1\n");
	if(mwindow->edl->local_session->selectionstart < 0) 
		mwindow->edl->local_session->selectionstart = 
			mwindow->edl->local_session->selectionend = 0;
//printf("TrackCanvas::end_handle_selection 1\n");

	mwindow->undo->update_undo_after();
	mwindow->cwindow->playback_engine->que->send_command(CURRENT_FRAME, 
							CHANGE_EDL,
							mwindow->edl,
							1);
//printf("TrackCanvas::end_handle_selection 1\n");
	gui->update(1, 1, 1, 1, 1, 1);
//printf("TrackCanvas::end_handle_selection 2\n");
}


double TrackCanvas::time_visible()
{
	return (double)get_w() * 
		mwindow->edl->session->zoom_sample / 
		mwindow->edl->session->sample_rate;
}









































void TrackCanvas::draw_automation()
{
}


int TrackCanvas::set_index_file(int flash, Asset *asset)
{
	for(Track *current = mwindow->edl->tracks->first;
		current;
		current = NEXT)
	{
		for(Edit *edit = current->edits->first; edit; edit = edit->next)
		{
			
		}
	}
	return 0;
}
















int TrackCanvas::button_release()
{
	region_selected = 0;
	if(auto_selected) mwindow->tracks->release_auto();
	auto_selected = 0;
// only end handle_selecion if floating
	if(handle_selected == 2) end_handle_selection();
	handle_selected = 0;
	handle_pixel = mwindow->session->tracks_vertical ? get_w() / 2 : get_h() / 2;
	if(translate_selected) end_translation();
	translate_selected = 0;
	return 1;
}


int TrackCanvas::auto_reposition(int &cursor_x, int &cursor_y, long cursor_position)
{
	if(!auto_selected)
	{
// don't want vertical movement for automation
		if(cursor_y < 0 && mwindow->tracks->view_start > 0)
		{
			int old_view_start = mwindow->tracks->view_start;

			if(mwindow->session->tracks_vertical)
			{ mwindow->tracks->move_down(-cursor_y); }
			else
			{ mwindow->tracks->move_up(-cursor_y); }

			cursor_y += old_view_start - mwindow->tracks->view_start;
		}
		else if(cursor_y > (mwindow->session->tracks_vertical ? get_w() : get_h()))
		{
			int old_view_start = mwindow->tracks->view_start;

			if(mwindow->session->tracks_vertical)
			{ mwindow->tracks->move_up(cursor_y - get_w()); }
			else
			{ mwindow->tracks->move_down(cursor_y - get_h()); }

			cursor_y += old_view_start - mwindow->tracks->view_start;
		}
	}

	if(auto_selected < 2)
	{
		if(cursor_position < mwindow->session->view_start && mwindow->session->view_start > 0)
		{
			if(cursor_position < 0) cursor_position = 0;
			long distance = mwindow->session->view_start - cursor_position;
			cursor_x += distance / mwindow->session->zoom_sample;
			mwindow->move_left(distance);
		}
		else 
		if(cursor_position > mwindow->session->view_start + mwindow->tracks->view_samples())
		{
			long distance = cursor_position -  mwindow->session->view_start - mwindow->tracks->view_samples();
			cursor_x -= distance / mwindow->session->zoom_sample;
			mwindow->move_right(distance);
		}
	}
	mwindow->gui->zoombar->update_playback(cursor_position);
	return 0;
}


int TrackCanvas::draw_floating_handle(int flash)
{
	static int pixel, hx, hy;

	set_inverse();
	set_color(WHITE);

	pixel = (handle_position - mwindow->session->view_start) / mwindow->session->zoom_sample;

	if(mwindow->session->tracks_vertical)
	{
		hx = get_w() / 2;
		hy = pixel;
		draw_line(0, pixel, get_w(), pixel);
	}
	else
	{
		hx = pixel;
		hy = get_h() / 2;
		draw_line(pixel, 0, pixel, get_h());
	}

// 	if(which_handle == 1) 
// 		draw_start_edit(hx, hy, mwindow->session->tracks_vertical);
// 	else 
// 		draw_end_edit(hx, hy, mwindow->session->tracks_vertical);

	set_opaque();
	if(flash) this->flash();
	return 0;
}

int TrackCanvas::draw_loop_point(long position, int flash)
{
	static int pixel;
	set_inverse();
	set_color(GREEN);
	
	pixel = (position - mwindow->session->view_start) / mwindow->session->zoom_sample;

	if(mwindow->session->tracks_vertical)
		draw_line(0, pixel, get_w(), pixel);
	else
		draw_line(pixel, 0, pixel, get_h());
	
	set_opaque();
	if(flash) this->flash();
	return 0;
}

int TrackCanvas::draw_playback_cursor(int pixel, int flash)
{
// 	set_inverse();
// 	set_color(PINK);
// 	
// printf("TrackCanvas::draw_playback_cursor\n");
// 	if(mwindow->session->tracks_vertical)
// 	{
// 		draw_line(0, pixel, get_w(), pixel);
// 		set_opaque();
// 		if(flash) this->flash(0, pixel, get_w(), 1);
// 	}
// 	else
// 	{
// 		draw_line(pixel, 0, pixel, get_h());
// 		set_opaque();
// 		if(flash) this->flash(pixel, 0, 1, get_h());
// 	}
	return 0;
}


int TrackCanvas::update_handle_selection(long cursor_position)
{
	handle_position = cursor_position;
	draw_floating_handle(1);         // show
	return 0;
}

int TrackCanvas::end_translation()
{
	mwindow->tracks->end_translation();
	return 0;
}

