#include "assets.h"
#include "atrack.h"
#include "automation.h"
#include "console.h"
#include "aedits.h"
#include "edit.h"
#include "edits.h"
#include "edl.h"
#include "edlsession.h"
#include "filexml.h"
#include "intauto.h"
#include "intautos.h"
#include "localsession.h"
#include "mainundo.h"
#include "mwindow.h"
#include "mwindowgui.h"
#include "module.h"
#include "modules.h"
#include "mainsession.h"
#include "timebar.h"
#include "trackcanvas.h"
#include "tracks.h"
#include "trackscroll.h"
#include "transition.h"
#include "vtrack.h"
#include <string.h>

int Tracks::import_audio_track(long length, int channel, Asset *asset)
{
	if(mwindow->gui)
	{
		hide_overlays(0);
	}

	add_audio_track();
	((ATrack*)last)->paste_output(0, length, 0, length, channel, asset);

	if(mwindow->gui)
	{
		if(mwindow->session->tracks_vertical)
		last->draw_old(0, vertical_pixels(), 0, view_pixels(), 0);
		else
		last->draw_old(0, view_pixels(), 0, vertical_pixels(), 0);

		mwindow->gui->trackscroll->update();
		show_overlays(1);
	}
}

int Tracks::import_video_track(long length, int layer, Asset *asset)
{
	if(mwindow->gui)
	{
		hide_overlays(0);
	}

	add_video_track();
	((VTrack*)last)->paste_output(0, length, 0, length, layer, asset);

	if(mwindow->gui)
	{
		if(mwindow->session->tracks_vertical)
		last->draw_old(0, vertical_pixels(), 0, view_pixels(), 0);
		else
		last->draw_old(0, view_pixels(), 0, vertical_pixels(), 0);

		mwindow->gui->trackscroll->update();
		show_overlays(1);
	}
}

int Tracks::delete_tracks()
{
	int result = 1, total_deleted = 0;
	Track *track, *next_track, *shifted_track;
	int deleted_number;
	
	while(result)
	{
// keep deleting until all the recordable tracks are gone
		result = 0;

printf("Tracks::delete_tracks 1\n");
// Stop when first recordable track is reached
		for(track = first; 
			track && !result;  
			track = next_track)
		{
printf("Tracks::delete_tracks 2\n");
			next_track = track->next;
			
			if(track->record)
			{
				deleted_number = number_of(track);
// Delete the track.
				delete track;

// Shift all the plugin pointers.
				for(shifted_track = next_track;
					shifted_track;
					shifted_track = shifted_track->next)
				{
					shifted_track->delete_module_pointers(deleted_number);
				}
				result = 1;
				total_deleted++;
			}
		}
printf("Tracks::delete_tracks 3\n");
	}

	if(total_deleted)
	{
		edl->session->changes_made = 1;
	}
printf("Tracks::delete_tracks 4\n");
	return 0;
}

int Tracks::move_tracks_up()
{
	Track *track, *next_track;
	int result = 0;

	for(track = first;
		track; 
		track = next_track)
	{
		next_track = track->next;

		if(track->record)
		{
			if(track->previous)
			{
				track->swap_module_pointers(number_of(track->previous), number_of(track));

				swap(track->previous, track);
				result = 1;
			}
		}
	}

	if(result)
	{
		edl->session->changes_made = 1;
	}
	return 0;
}

int Tracks::move_tracks_down()
{
	Track *track, *previous_track;
	int result = 0;
	
	for(track = last;
		track; 
		track = previous_track)
	{
		previous_track = track->previous;

		if(track->record)
		{
			if(track->next)
			{
				swap_transitions(track, track->next);
				track->swap_module_pointers(number_of(track), number_of(track->next));

				swap(track, track->next);
				result = 1;
			}
		}
	}
	
	if(result)
	{
		edl->session->changes_made = 1;
	}
	return 0;
}

int Tracks::concatenate_tracks()
{
	Track *output_track, *first_output_track, *input_track;
	int i, data_type = TRACK_AUDIO;
	long output_start;
	FileXML *clipboard;
	int result = 0;

// Relocate tracks
	for(i = 0; i < 2; i++)
	{
// Get first output track
		for(output_track = first; 
			output_track; 
			output_track = output_track->next)
			if(output_track->data_type == data_type && 
				output_track->record) break;

		first_output_track = output_track;

// Get first input track
		for(input_track = first;
			input_track;
			input_track = input_track->next)
		{
			long unit_start = input_track->to_units(edl->local_session->selectionstart, 0);

			if(input_track->data_type == data_type &&
				((IntAuto*)input_track->automation->play_autos->get_prev_auto(unit_start, PLAY_FORWARD))->value && 
				!input_track->record) break;
		}


		if(output_track && input_track)
		{
// Transfer input track to end of output track one at a time
			while(input_track)
			{
				output_start = (long)(output_track->get_length() + 0.5);
				output_track->insert_track(input_track, output_start, 0);

// Get next source and destination
				for(input_track = input_track->next; 
					input_track; 
					input_track = input_track->next)
				{
					long unit_start = input_track->to_units(edl->local_session->selectionstart, 0);

					if(input_track->data_type == data_type && 
						!input_track->record && 
						((IntAuto*)input_track->automation->play_autos->get_prev_auto(unit_start, PLAY_FORWARD))->value) break;
				}

				for(output_track = output_track->next; 
					output_track; 
					output_track = output_track->next)
				{
					if(output_track->data_type == data_type && 
					output_track->record) break;
				}

				if(!output_track)
				{
					output_track = first_output_track;
				}
			}
			result = 1;
		}

		if(data_type == TRACK_AUDIO) data_type = TRACK_VIDEO;
	}

	if(result)
	{
		edl->session->changes_made = 1;
	}

	return 0;
}

int Tracks::delete_audio_track()
{
	Track *current;

	for(current = last; current && current->data_type != TRACK_AUDIO; current = PREVIOUS)
	{
		;
	}

	if(current) delete_track(current);
}

int Tracks::delete_video_track()
{
	Track *current;

	for(current = last; current && current->data_type != TRACK_VIDEO; current = PREVIOUS)
	{
		;
	}

	if(current) delete_track(current);
}

int Tracks::delete_all_tracks()
{
	while(last) delete last;
	return 0;
}

// REMOVE
int Tracks::delete_all(int flash)
{
	while(last) delete_track();
	
	if(flash) draw(flash);
	return 0;
}

int Tracks::swap_transitions(Track *track1, Track *track2)
{
	int number1 = number_of(track1) - 1;
	int number2 = number_of(track2) - 1;
	Track* current = first;

//printf("Tracks::swap_transitions %d %d\n", number1, number2);
	for( ; current; current = current->next)
	{
		current->swap_transitions(number1, number2);
	}
	return 0;
}


// =========================================== EDL editing


int Tracks::copy(double start, 
	double end, 
	int all, 
	FileXML *file, 
	char *output_path)
{
// nothing selected
	if(start == end) return 1;

	Track* current;

	for(current = first; 
		current; 
		current = NEXT)
	{
		if(current->record || all)
		{
			current->copy(start, end, file,output_path);
		}
	}

	return 0;
}





// REMOVE
int Tracks::paste(long start, long end)
{
	FileXML xml;

// load from the clipboard
	{
		char *string;
		string = new char[mwindow->gui->get_clipboard()->clipboard_len(0) + 1];
		mwindow->gui->get_clipboard()->from_clipboard(string, mwindow->gui->get_clipboard()->clipboard_len(0) + 1, 0);

		xml.read_from_string(string);
		delete [] string;
	}

	int result = 0;

// scan file for XML block	
	do{
		result = xml.read_tag();
	}while(!result && 
		strcmp(xml.tag.get_title(), "XML") && 
		strcmp(xml.tag.get_title(), "EDL"));

// scan file for clipboard	
	if(!result)
	do{
		result = xml.read_tag();
	}while(!result && strcmp(xml.tag.get_title(), "CLIPBOARD"));

	long total_length = 0;
	if(!result)
	{
		total_length = xml.tag.get_property("SAMPLES", (long)0);
	}

	Track *current_video_track = first;
	Track *current_audio_track = first;
	char track_type[1024];
	sprintf(track_type, "AUDIO");

	while(!result)
	{
		result = xml.read_tag();

		if(!result)
		{
			if(!strcmp(xml.tag.get_title(), "/CLIPBOARD"))
			{
				result = 1;
			}
			else
			if(!strcmp(xml.tag.get_title(), "ASSETS"))
			{
				paste_assets(&xml);
			}
			else
			if(!strcmp(xml.tag.get_title(), "LABELS"))
			{
				mwindow->timebar->paste(start, end, total_length, &xml);
			}
			else
			if(!strcmp(xml.tag.get_title(), "TRACK"))
			{
				xml.tag.get_property("TYPE", track_type);
				if(!strcmp(track_type, "AUDIO"))
				{
// get next recordable audio track
					while(current_audio_track &&
						(!current_audio_track->record || 
						current_audio_track->data_type != TRACK_AUDIO))
					{
						current_audio_track = current_audio_track->next;
					}
					
					if(current_audio_track)
					{
						current_audio_track->paste(start, end, total_length, &xml);
						current_audio_track = current_audio_track->next;
					}
				}
				else
				if(!strcmp(track_type, "VIDEO") && current_video_track)
				{
// get a recordable video track
					while(current_video_track &&
						(!current_video_track->record || 
						current_video_track->data_type != TRACK_VIDEO))
					{
						current_video_track = current_video_track->next;
					}
					
					if(current_video_track)
					{
						current_video_track->paste(start, end, total_length, &xml);
						current_video_track = current_video_track->next;
					}
				}
			}      // end TRACK tag
		}
	}

// fill remaining tracks with silence
	while(current_audio_track || current_video_track)
	{
		if(current_audio_track)
		{
			if(current_audio_track->record && 
				current_audio_track->data_type == TRACK_AUDIO)
			{
				current_audio_track->paste_silence(start, start + total_length);
			}

			current_audio_track = current_audio_track->next;
		}

		if(current_video_track)
		{
			if(current_video_track->record && 
				current_video_track->data_type == TRACK_VIDEO)
			{
				current_video_track->paste_silence(start, start + total_length);
			}

			current_video_track = current_video_track->next;
		}
	}

	mwindow->draw();
	mwindow->set_selection(start, start + total_length);
	mwindow->session->changes_made = 1;
	return result;
}

int Tracks::paste_assets(FileXML *xml)
{
	int result = 0;

	while(!result)
	{
		result = xml->read_tag();
		if(!result)
		{
			if(xml->tag.title_is("/ASSETS"))
			{
				result = 1;
			}
			else
			if(xml->tag.title_is("ASSET"))
			{
				char *path = xml->tag.get_property("SRC");
				Asset new_asset(path ? path : SILENCE);
				new_asset.read(mwindow->plugindb, xml);
				mwindow->assets->update(&new_asset);
			}
		}
	}
	return 0;
}

// REMOVE
int Tracks::paste_output(long startproject, 
				long endproject, 
				long startsource_sample, 
				long endsource_sample, 
				long startsource_frame, 
				long endsource_frame, 
				Asset *asset)
{
// 	Track *current_track;
// 	int result, channel, layer;
// 
// 	result = 0;
// 	channel = 0;
// 	layer = 0;
// 
// 	for(current_track = first; 
// 		current_track;
// 		current_track = current_track->next)
// 	{
// 		if(current_track->record)
// 		{
// 			if(current_track->data_type == TRACK_AUDIO)
// 			{
// 				if(channel < asset->channels && asset->audio_data)
// 				{
// 					((ATrack*)current_track)->paste_output(startproject, 
// 												endproject, 
// 												startsource_sample, 
// 												endsource_sample, 
// 												channel, 
// 												asset);
// 					channel++;
// 				}
// // inserting silence into extra tracks sucked
// 				//else { current_track->paste_silence(startproject, endsource - startsource); }
// 			}
// 			else
// 			if(current_track->data_type == TRACK_VIDEO)
// 			{
// 				if(layer < asset->layers && asset->video_data)
// 				{
// 					long start = startproject, end = endproject;
// 					current_track->samples_to_units(start);
// 					current_track->samples_to_units(end);
// 
// 					((VTrack*)current_track)->paste_output(start, 
// 												end, 
// 												startsource_frame, 
// 												endsource_frame, 
// 												layer, 
// 												asset);
// 					layer++;
// 				}
// // inserting silence into extra tracks sucked
// 				//else { current_track->paste_silence(startproject, endsource - startsource); }
// 			}
// 		}
// 	}
	return 0;
}

int Tracks::paste_transition(long startproject, 
				long endproject, 
				Transition *transition)
{
	Track *current_track;

	for(current_track = first; 
		current_track;
		current_track = current_track->next)
	{
		if(current_track->record)
		{
			if((current_track->data_type == TRACK_AUDIO && transition->audio) ||
				(current_track->data_type == TRACK_VIDEO && transition->video))
			{
					current_track->paste_transition(startproject, 
												endproject, 
												transition);
			}
		}
	}
	
	return 0;
}

int Tracks::clear(double start, double end)
{
	Track *current_track;

	for(current_track = first; current_track; current_track = current_track->next)
	{
		if(current_track->record) 
		{ 
			current_track->clear(start, end, 1); 
		}
	}
	return 0;
}

int Tracks::copy_automation(long selectionstart, long selectionend, FileXML *xml)
{
// called by MWindow::copy_automation for copying automation alone
	Track* current_track;

	for(current_track = first; current_track; current_track = current_track->next)
	{
		if(current_track->record)
		{
			xml->tag.set_title("TRACK");
			xml->append_tag();
			xml->append_newline();

			current_track->copy_automation(&auto_conf, 
				selectionstart, 
				selectionend, 
				xml);

			xml->tag.set_title("/TRACK");
			xml->append_tag();
			xml->append_newline();
			xml->append_newline();
		}
	}
	return 0;
}

int Tracks::clear_automation(double selectionstart, double selectionend)
{
	Track* current_track;
	
	for(current_track = first; current_track; current_track = current_track->next)
	{
		if(current_track->record) 
		{ 
// 			current_track->clear_automation(&auto_conf, 
// 				selectionstart, 
// 				selectionend, 
// 				0); 
		}
	}
	return 0;
}

int Tracks::clear_handle(double start, double end)
{
	Track* current_track;

	for(current_track = first; current_track; current_track = current_track->next)
	{
		if(current_track->record)
		{
			current_track->clear_handle(start, end); 
		}
	}
	return 0;
}

int Tracks::paste_silence(double start, double end)
{
	Track* current_track;

	for(current_track = first; 
		current_track; 
		current_track = current_track->next)
	{
		if(current_track->record) 
		{ 
			current_track->paste_silence(start, end); 
		}
	}
	return 0;
}

int Tracks::select_translation(int cursor_x, int cursor_y)
{
// cursor_x is relative to samples
	int result = 0;
	Track* current_track;
	
	for(current_track = first; 
		current_track && !result; 
		current_track = current_track->next)
	{
		if(current_track->record) 
			result = current_track->select_translation(cursor_x, cursor_y);
	}
	if(result)
	{
		mwindow->undo->update_undo_edits("Translation", 0);
	}
	return result;
}

int Tracks::update_translation(int cursor_x, int cursor_y, int shift_down)
{
	int result = 0;
	for(Track* current = first; current && !result; current = NEXT)
	{
		result = current->update_translation(cursor_x, cursor_y, shift_down);
	}
}

int Tracks::end_translation()
{
	Track *current;
	int result = 0;

	for(current = first; current; current = NEXT)
	{
		result = current->end_translation();
	}
	if(result)
	{
		mwindow->session->changes_made = 1;
		mwindow->undo->update_undo_edits();
	}
}

int Tracks::select_handle(int cursor_x, int cursor_y, long &handle_oldposition, long &handle_position, int &handle_pixel)
{
	int center_pixel;
	int result = 0;
	long selection;
	
	if(handles)
	{
		for(Track* current = first; current && !result; current = NEXT) 
		{
			center_pixel = current->pixel + mwindow->session->zoom_track / 2;

			if(cursor_y > center_pixel - 6 && cursor_y < center_pixel + 6)
				result = current->select_handle(cursor_x, cursor_y, selection);
		}
	}

// Result is 3 if the track was recordable or 1,2 if it wasn't recordable
	if(result) 
	{
// Modify selected region
		result = mwindow->init_handle_selection(selection, cursor_x, result);

		if(result && result != 3)
		{
// not a region selection and a recordable track
			handle_oldposition = selection;
			handle_position = selection;
			handle_pixel = cursor_x;
		}
	}
	
	return result;
}

int Tracks::select_auto(int cursor_x, int cursor_y)
{
	int result = 0;
	for(Track* current = first; current && !result; current = NEXT) { result = current->select_auto(&auto_conf, cursor_x, cursor_y); }
	return result;
}

int Tracks::move_auto(int cursor_x, int cursor_y, int shift_down)
{
	int result = 0;

	for(Track* current = first; current && !result; current = NEXT) 
	{
		result = current->move_auto(&auto_conf, cursor_x, cursor_y, shift_down); 
	}
	return 0;
}

int Tracks::release_auto()
{
	mwindow->session->changes_made = 1;
// save the before undo
	mwindow->undo->update_undo_automation("Automation", 0);

	hide_overlays(0);

	int result = 0;
	for(Track* current = first; current && !result; current = NEXT) 
	{ 
		result = current->release_auto(); 
	}

// save the after undo
	mwindow->undo->update_undo_automation();
	draw();
	show_overlays(1);
	return 0;
}

int Tracks::modify_handles(double &oldposition, 
	double &newposition, 
	int currentend, 
	int handle_mode)
{
	Track *current;

	for(current = first; current; current = NEXT)
	{
		if(current->record)
		{
			current->modify_handles(oldposition, 
				newposition, 
				currentend, 
				handle_mode);
		}
	}
	return 0;
}

int Tracks::select_edit(long cursor_position, 
	int cursor_x, 
	int cursor_y, 
	long &new_start, 
	long &new_end)
{
 	int result = 0;
// 	for(Track *track = first; track && !result; track = track->next)
// 	{
// 		result = track->select_edit(cursor_position, 
// 			cursor_x, 
// 			cursor_y, 
// 			new_start, 
// 			new_end);
// 	}
 	return result;
}

// REMOVE
int Tracks::feather_edits(long start, long end, long samples, int audio, int video)
{
// 	Track *current_track;
// 
// 	for(current_track = first; 
// 		current_track;
// 		current_track = current_track->next)
// 	{
// 		if(current_track->record && 
// 			((audio && current_track->data_type == TRACK_AUDIO) ||
// 			(video && current_track->data_type == TRACK_VIDEO)))
// 		{ 
// 			current_track->feather_edits(start, end, samples); 
// 		}
// 	}
	return 0;
}

// REMOVE
long Tracks::get_feather(long selectionstart, long selectionend, int audio, int video)
{
// 	Track *current_track;
// 
// 	for(current_track = first; 
// 		current_track; 
// 		current_track = current_track->next)
// 	{
// 		if(current_track->record && 
// 			((audio && current_track->data_type == TRACK_AUDIO) ||
// 			(video && current_track->data_type == TRACK_VIDEO)))
// 		{ 
// 			return current_track->get_feather(edl->local_session->selectionstart, edl->local_session->selectionend);
// 		}
// 	}
	return 0;
}

int Tracks::reset_translation(long start, long end)
{
	Track *current_track;
	int result = 0;

	for(current_track = first; 
		current_track; 
		current_track = current_track->next)
	{
		if(current_track->record) { result += current_track->reset_translation(start, end); }
	}

	if(result)
	{
		mwindow->draw();
		mwindow->session->changes_made = 1;
	}
}



int Tracks::purge_asset(Asset *asset)
{
	Track *current_track;
	int result = 0;
	
	for(current_track = first; current_track; current_track = current_track->next)
	{
		result += current_track->purge_asset(asset); 
	}
	return result;
}

int Tracks::asset_used(Asset *asset)
{
	Track *current_track;
	int result = 0;
	
	for(current_track = first; current_track; current_track = current_track->next)
	{
		result += current_track->asset_used(asset); 
	}
	return result;
}

int Tracks::scale_video(int *dimension, int *offsets, int scale_data)
{
	Track *current_track;
	int result = 0;
// Tracks are scaled using a single z curve and independant x and y pans, 
// so take the lowest of the horizontal and vertical scales as the zoom factor.
	float camera_scale = 1, projector_scale = 1;
	if(scale_data)
	{
		float hscale;
		float vscale;
		hscale = dimension[0] / mwindow->session->track_w;
		vscale = dimension[1] / mwindow->session->track_h;
		camera_scale = hscale < vscale ? hscale : vscale;
		hscale = dimension[2] / mwindow->session->output_w;
		vscale = dimension[3] / mwindow->session->output_h;
		projector_scale = hscale < vscale ? hscale : vscale;
	}

	for(current_track = first; current_track; current_track = current_track->next)
	{
		if(current_track->data_type == TRACK_VIDEO)
			result += ((VTrack*)current_track)->scale_video(camera_scale, projector_scale, offsets);
	}
	return result;	
}

int Tracks::scale_time(float rate_scale, int ignore_record, int scale_edits, int scale_autos, long start, long end)
{
	Track *current_track;

	for(current_track = first; 
		current_track; 
		current_track = current_track->next)
	{
		if((current_track->record || ignore_record) && 
			current_track->data_type == TRACK_VIDEO)
		{
			current_track->scale_time(rate_scale, scale_edits, scale_autos, start, end);
		}
	}
	return 0;
}

