#include "attachmentpoint.h"
#include "auto.h"
#include "automation.h"
#include "autos.h"
#include "console.h"
#include "edit.h"
#include "floatauto.h"
#include "floatautos.h"
#include "intauto.h"
#include "intautos.h"
#include "mwindow.h"
#include "module.h"
#include "modules.h"
#include "panauto.h"
#include "panautos.h"
#include "patchbay.h"
#include "plugin.h"
#include "pluginbuffer.h"
#include "pluginserver.h"
#include "renderengine.h"
#include "tracks.h"
#include "transition.h"
#include "transportque.h"
#include "virtualconsole.h"
#include "virtualnode.h"

VirtualNode::VirtualNode(RenderEngine *renderengine, 
		VirtualConsole *vconsole, 
		Module *real_module, 
		Plugin *real_plugin,
		Track *track, 
		VirtualNode *parent_module, 
		int input_is_master,
		int output_is_master,
		int in,
		int out)
{
	this->renderengine = renderengine;
	this->vconsole = vconsole;
	this->real_module = real_module;
	this->real_plugin = real_plugin;
	this->track = track;
	this->parent_module = parent_module;
	this->ring_buffers = vconsole->total_ring_buffers();
	this->input_is_master = input_is_master;
	this->output_is_master = output_is_master;
	this->in = in;
	this->out = out;
	virtual_transition = 0;
	shared_input = 1;
	shared_output = 1;
	render_count = 0;
	plugin_type = 0;
	waiting_real_plugin = 0;
	plugin_buffer_number = 0;
	plugin_autos = 0;
	plugin_auto_before = plugin_auto_after = 0;
}

VirtualNode::~VirtualNode()
{
	if(virtual_transition) delete virtual_transition;
	vplugins.remove_all_objects();
}

void VirtualNode::dump()
{
	printf("  VirtualNode %s\n", track->title);
	if(real_module)
	{
		printf("   Plugins\n");
		for(int i = 0; i < vplugins.total; i++)
		{
			vplugins.values[i]->dump();
		}
	}
	else
	if(real_plugin)
	{
		printf("    Plugin %s\n", real_plugin->title);
	}
}

int VirtualNode::expand(int persistant_plugins, long current_position)
{
//printf("VirtualNode::expand 1 %p %p\n", real_module, real_plugin);
	expand_buffers();

// module needs to know where the input data for the next process is
	data_in_input = 1;
	if(real_module)
	{
		expand_as_module(persistant_plugins, current_position);
	}
	else
	if(real_plugin)
	{
// attach to a real plugin for a plugin
// plugin always takes data from input to output
		expand_as_plugin(persistant_plugins);
	}
//printf("VirtualNode::expand 2\n");

	return 0;
}

int VirtualNode::expand_buffers()
{
// temp needed for the output
	if(!out)
	{
		new_output_buffer();
		shared_output = 0;
		output_is_master = 0;
	}

// temp needed for the output
	if(!in)
	{
		new_input_buffer();
		shared_input = 0;
		input_is_master = 0;
	}

	return 0;
}

int VirtualNode::expand_as_module(int duplicate, long current_position)
{
	Transition *transition = 0;

// Expand the transition if this is a track has a transition
// FIXME
// 	if(track && (transition = track->get_current_transition(current_position)))
// 	{
// 		if(transition->on)
// 		{
// 			int plugin_type = transition->plugin_type;
// 			if(plugin_type == 3)
// 			{
// // transition is a module
// 				attach_virtual_module(0, duplicate, current_position, transition);
// 			}
// 			else
// 			{
// // transition is a plugin
// 				attach_virtual_plugin(0, duplicate, current_position, transition);
// 			}
// 		}
// 	}

// create the plugins for this module
	for(int i = 0; i < track->plugin_set.total; i++)
	{
		Plugin *plugin = track->get_current_plugin(current_position, 
			i, 
			renderengine->command->get_direction(),
			0);

printf("VirtualNode::expand_as_module 2 %p\n", plugin);
		if(plugin)
		{
			if(plugin->on)
			{
				int plugin_type = plugin->plugin_type;
				if(plugin_type == PLUGIN_SHAREDMODULE)
				{
// plugin is a module
//printf("VirtualNode::expand_as_module 3\n");
					attach_virtual_module(i, 
						duplicate, 
						current_position, 
						0);
//printf("VirtualNode::expand_as_module 4\n");
				}
				else
				if(plugin_type == PLUGIN_SHAREDPLUGIN ||
					plugin_type == PLUGIN_STANDALONE)
				{
// plugin is a plugin
//printf("VirtualNode::expand_as_module 5\n");
					attach_virtual_plugin(plugin, 
						i, 
						duplicate, 
						current_position, 
						0);
//printf("VirtualNode::expand_as_module 6\n");
				}
			}
		}
//printf("VirtualNode::expand_as_module 7\n");
	}
	return 0;
}

int VirtualNode::expand_as_plugin(int duplicate)
{
	plugin_type = real_plugin->plugin_type;
	if(plugin_type == PLUGIN_SHAREDPLUGIN)
	{
// Attached to a shared plugin.
// Get the real plugin it's attached to.

// Redirect the real_plugin to the shared plugin.
		int real_module_number = real_plugin->shared_location.module;
		int real_plugin_number = real_plugin->shared_location.plugin;
		Module *real_module = 0;


		real_module = vconsole->module_number(real_module_number);


//printf("VirtualNode::expand_as_plugin 1 %p %d %d\n", real_module, real_module_number, real_plugin_number);
		
		attachment = real_module->attachments[real_plugin_number];
		real_plugin = attachment->plugin;
//printf("VirtualNode::expand_as_plugin 1 %p\n", real_plugin);

// Real plugin not on then null it.
		if(!real_plugin->on) real_plugin = 0;
//printf("VirtualNode::expand_as_plugin 1\n");
	}
//printf("VirtualNode::expand_as_plugin 2\n");






	if(plugin_type == PLUGIN_STANDALONE)
	{
// Get plugin server
		Module *module = vconsole->module_of(track);
		attachment = module->attachment_of(real_plugin);
//printf("VirtualNode::expand_as_plugin %p %p %p\n", module, real_plugin, attachment);
	}






// Add to real plugin's list of virtual plugins for configuration updates
// and plugin_server initializations.
// Input and output are taken care of when the parent module creates this plugin.
// Get number for passing to render.
// real_plugin may become NULL after shared plugin test.
	if(real_plugin && attachment)
	{

		if(attachment) 
			plugin_buffer_number = attachment->attach_virtual_plugin(this);
	}




//printf("VirtualNode::expand_as_plugin 3\n");
	return 0;
}

int VirtualNode::expand_as_transition(int duplicate)
{
	plugin_type = real_transition->plugin_type;

	if(plugin_type == 2)
	{
// Aattached to a shared plugin.
// Get the real plugin it's attached to.
// Redirect the real_plugin to the shared plugin.
		int real_module_number = real_transition->shared_location.module;
		int real_plugin_number = real_transition->shared_location.plugin;
		Module *real_module = /* mwindow->console->modules->module_number(real_module_number) */ 0;
		real_plugin = real_module->plugins[real_plugin_number];

// Real plugin not on then null it.
		if(!real_plugin->on) real_plugin = 0;

//		if(real_plugin)
//			plugin_buffer_number = real_plugin->attach_virtual_plugin(this);
	}
	else
	{
		if(!real_transition->on) real_transition = 0;

// Add to the real plugin's list of virtual plugins for configuration updates
// and plugin_server initializations.
// Input and output are taken care of when the parent module creates this plugin.
// Get number for passing to render.
//		if(real_transition)
//			plugin_buffer_number = real_transition->attach_virtual_plugin(this);
	}
}

int VirtualNode::attach_virtual_module(int plugin_number, int duplicate, long current_position, Transition *real_transition)
{
	if(!real_transition)
	{
		int real_module_number = real_module->plugins[plugin_number]->shared_location.module;
		Module *real_module = /* mwindow->console->modules->module_number(real_module_number) */ 0;
		Plugin *real_plugin = this->real_module->plugins[plugin_number];
		Track *track = /* mwindow->tracks->number(real_module_number) */ 0;

		vplugins.values[plugin_number]
			 = create_module(real_plugin, real_module, parent_module, track);
		vplugins.values[plugin_number]->expand(duplicate, current_position);

// working data is now in output
		if(real_plugin->out) data_in_input = 0;
	}
	else
	{
		int real_module_number = real_transition->shared_location.module;
		Module *real_module = /* mwindow->console->modules->module_number(real_module_number) */ 0;
		Track *track = /* mwindow->tracks->number(real_module_number) */ 0;

		virtual_transition
			= create_module(0, real_module, parent_module, track);
		virtual_transition->expand(duplicate, current_position);

// working data is now in output
		if(real_transition->out) data_in_input = 0;
	}
	return 0;
}

int VirtualNode::attach_virtual_plugin(Plugin *plugin, 
	int plugin_number, 
	int duplicate, 
	long current_position, 
	Transition *real_transition)
{
//printf("VirtualNode::attach_virtual_plugin 1\n");
	if(plugin->on)
	{
// working data is now in output
		if(plugin->out) data_in_input = 0;

		VirtualNode *virtual_plugin = create_plugin(plugin);
		vplugins.append(virtual_plugin);
		virtual_plugin->expand(duplicate, current_position);
//printf("VirtualNode::attach_virtual_plugin 5\n");
	}
	return 0;
}

int VirtualNode::sort(ArrayList<VirtualNode*>*render_list)
{
	int result = 0, total_result = 0;

//printf("VirtualNode::sort %p %p\n", real_module, real_plugin);
	if(real_module)
	{
		sort_as_module(render_list, result, total_result);
	}
	else
	if(real_plugin || real_transition)
	{
		sort_as_plugin(render_list, result, total_result);
	}

	if(!result && total_result) result = total_result;
// if a plugin that wasn't patched out couldn't be rendered, try again

	return result;
}

int VirtualNode::sort_as_module(ArrayList<VirtualNode*>*render_list, int &result, int &total_result)
{
// stop when rendering can't continue without another higher level module
//printf("VirtualNode::sort_as_module 1\n");
	if(virtual_transition)
	{
		result = virtual_transition->sort(render_list);
		
		if(result && !virtual_transition->out)
		{
			total_result = 1;
			result = 0;
// couldn't render the last plugin but it wasn't patched out so continue to next plugin
		}
	}
//printf("VirtualNode::sort_as_module 1\n");

	for(int i = 0; i < vplugins.total && !result; i++)
	{
// stop when rendering can't continue without another higher level module
		result = vplugins.values[i]->sort(render_list);

		if(result && !vplugins.values[i]->out)
		{
			total_result = 1;
			result = 0;
// couldn't render the last plugin but it wasn't patched out so continue to next plugin
		}
	}
//printf("VirtualNode::sort_as_module 1\n");

// all plugins rendered or not patched out
// render this module
	if(render_count == 0 && !result)
	{
		render_list->append(this);
		render_count++;
		result = 0;
	}
//printf("VirtualNode::sort_as_module 2\n");
	return 0;
}

int VirtualNode::sort_as_plugin(ArrayList<VirtualNode*>*render_list, int &result, int &total_result)
{
// Plugin server does not exist at this point.
// need to know if plugin requires all inputs to be armed before rendering
//printf("VirtualNode::sort_as_plugin 1\n");
	int multichannel = 0, singlechannel = 0;

//printf("VirtualNode::sort_as_plugin 2 %p\n", attachment);
	if(plugin_type == PLUGIN_STANDALONE || plugin_type == PLUGIN_SHAREDPLUGIN)
	{
		multichannel = attachment->multichannel_shared(1);
		singlechannel = attachment->singlechannel();
	}

//printf("VirtualNode::sort_as_plugin 3\n");
	if(plugin_type == PLUGIN_STANDALONE && !multichannel)
	{
// unshared single channel plugin
// render now
//printf("VirtualNode::sort_as_plugin 4\n");
		if(!render_count)
		{
			render_list->append(this);
			render_count++;
			result = 0;
		}
//printf("VirtualNode::sort_as_plugin 5\n");
	}
	else
	if(plugin_type == PLUGIN_SHAREDPLUGIN || multichannel)
	{

// Shared plugin
//printf("VirtualNode::sort_as_plugin 6\n");
		if(!render_count)
		{
			if(singlechannel)
			{
// shared single channel plugin
// render now
//printf("VirtualNode::sort_as_plugin 7\n");
				render_list->append(this);
//printf("VirtualNode::sort_as_plugin 7\n");
				render_count++;
				result = 0;
			}
			else
			{
// shared multichannel plugin
// all buffers must be armed before rendering at the same time
				if(!waiting_real_plugin)
				{
//printf("VirtualNode::sort_as_plugin 8 %p\n", attachment);
					waiting_real_plugin = 1;
					if(real_plugin)
						result = attachment->sort(this);
//printf("VirtualNode::sort_as_plugin 9\n");

					render_list->append(this);
//printf("VirtualNode::sort_as_plugin 10\n");
					render_count++;
				}
				else
				{
// Assume it was rendered later in the first pass
					result = 0;
				}
			}
		}
	}
	return 0;
}

int VirtualNode::get_plugin_input(int &ring_buffer_in, long &fragment_position_in,
							int &ring_buffer_out, long &fragment_position_out,
							int ring_buffer, long fragment_position)
{
	if(input_is_master)
	{
		ring_buffer_in = ring_buffer;
		fragment_position_in = fragment_position;
	}
	else
	{
		ring_buffer_in = 0;
		fragment_position_in = 0;
	}

	if(output_is_master)
	{
		ring_buffer_out = ring_buffer;
		fragment_position_out = fragment_position;
	}
	else
	{
		ring_buffer_out = 0;
		fragment_position_out = 0;
	}
}

int VirtualNode::render_as_plugin(long source_len,
		long source_position,
		int ring_buffer,
		long fragment_position,
		long fragment_len)
{
// need numbers for actual buffers
	int direction = renderengine->command->get_direction();
	int ring_buffer_in, ring_buffer_out;
	long fragment_position_in, fragment_position_out;
	int multichannel = 0;
printf("VirtualNode::render_as_plugin 1 %p\n", attachment);

// Redirect the attachmentpoint to the shared plugin or standalone plugin.
// 	if(real_transition)
// 	{
// 		if(plugin_type == 2)
// 			attachmentpoint = real_plugin;
// 		else
// 			attachmentpoint = real_transition;
// 
// // Override the source positions if a transition.
// 		source_len = real_transition->plugin->length;
// 
// 		if(direction == PLAY_REVERSE)
// 			source_position = real_transition->plugin->startproject + real_transition->plugin->length - source_position;
// 		else
// 			source_position -= real_transition->plugin->startproject;
// 	}
// 	else
// 	{
// 		attachmentpoint = real_plugin;
// 	}

// Abort if no plugin
	if(!attachment ||
		!real_plugin ||
		!real_plugin->on) return 0;

// 	attachment->render(source_position, 
// 		fragment_len, 
// 		fragment_position);

// 	if(plugin_type == 1 || plugin_type == 2) 
// 	{
// 		if(!attachmentpoint->plugin_server)
// 			return 0;
// 		else
// 			multichannel = attachmentpoint->multichannel_shared(0);
// 	}
// 
// 	if(plugin_type == 1 && !multichannel)
// 	{
// // standalone plugin
// // plugins copy from their input to their output
// 		plugin_server->arm_buffer(plugin_buffer_number,
// 							fragment_position_in,
// 							fragment_position_out,
// 							ring_buffer_in,
// 							ring_buffer_out);
// 		plugin_server->set_automation(plugin_autos, &plugin_auto_before, &plugin_auto_after, direction);
// 		plugin_server->process_realtime(source_len, source_position, fragment_len);
// 	}
// 	else
// 	if(plugin_type == 2 || multichannel)
// 	{
// // render a shared plugin
// // shared single channel plugin can be processed now
// 		if(!plugin_server->multichannel)
// 		{
// 			plugin_server->arm_buffer(plugin_buffer_number,
// 							fragment_position_in,
// 							fragment_position_out,
// 							ring_buffer_in,
// 							ring_buffer_out);
// 			plugin_server->set_automation(plugin_autos, &plugin_auto_before, &plugin_auto_after, direction);
// 			plugin_server->process_realtime(source_len, 
// 											source_position, 
// 											fragment_len);
// 		}
// 		else
// 		{
// // try multi channel plugin now
// // flag real plugin by setting render count here
// 			render_count = 0;
// 			attachmentpoint->render(ring_buffer_in, ring_buffer_out, 
// 									fragment_position_in, fragment_position_out,
// 									fragment_len, plugin_buffer_number, 
// 									source_position, source_len,
// 									plugin_autos, 
// 									&plugin_auto_before, 
// 									&plugin_auto_after);
// 		}
// 	}
printf("VirtualNode::render_as_plugin 2\n");
}



void VirtualNode::get_mute_fragment(long input_position,
				int &mute_constant, 
				long &fragment_len, 
				Autos *autos)
{
	long mute_len;
	int direction = renderengine->command->get_direction();

	IntAuto *prev_keyframe = (IntAuto*)autos->get_prev_auto(input_position, direction);
	IntAuto *next_keyframe = (IntAuto*)autos->get_next_auto(input_position, direction);

	if(direction == PLAY_FORWARD)
	{
// Two distinct keyframes within range
		if(next_keyframe->position > prev_keyframe->position)
		{
			mute_constant = prev_keyframe->value;

			if(next_keyframe->position < input_position + fragment_len)
				fragment_len = next_keyframe->position - input_position;
		}
		else
// One keyframe within range
		{
			mute_constant = prev_keyframe->value;
		}
	}
	else
	{
// Two distinct keyframes within range
		if(next_keyframe->position < prev_keyframe->position)
		{
			mute_constant = next_keyframe->value;

			if(next_keyframe->position > input_position - fragment_len)
				fragment_len = input_position - next_keyframe->position;
		}
		else
// One keyframe within range
		{
			mute_constant = next_keyframe->value;
		}
	}
}



void VirtualNode::get_fade_automation(double &slope,
	double &intercept,
	long input_position,
	long &slope_len,
	Autos *autos)
{
	int direction = renderengine->command->get_direction();

	FloatAuto *prev_keyframe = (FloatAuto*)autos->get_prev_auto(input_position, direction);
	FloatAuto *next_keyframe = (FloatAuto*)autos->get_next_auto(input_position, direction);
	
	if(direction == PLAY_FORWARD)
	{
// Two distinct automation points within range
		if(next_keyframe->position > prev_keyframe->position)
		{
			slope = ((double)next_keyframe->value - prev_keyframe->value) / slope_len;
			intercept = ((double)input_position - prev_keyframe->position) * slope + prev_keyframe->value;

			if(next_keyframe->position < input_position + slope_len)
				slope_len = next_keyframe->position - input_position;
		}
		else
// One automation point within range
		{
			slope = 0;
			intercept = prev_keyframe->value;
		}
	}
	else
	{
// Two distinct automation points within range
		if(next_keyframe->position < prev_keyframe->position)
		{
			slope = ((double)next_keyframe->value - prev_keyframe->value) / slope_len;
			intercept = ((double)prev_keyframe->position - input_position) * slope + prev_keyframe->value;

			if(prev_keyframe->position > input_position - slope_len)
				slope_len = input_position - prev_keyframe->position;
		}
		else
// One automation point within range
		{
			slope = 0;
			intercept = next_keyframe->value;
		}
	}
}

void VirtualNode::get_pan_automation(double &slope,
	double &intercept,
	long input_position,
	long &slope_len,
	Autos *autos,
	int channel)
{
	int direction = renderengine->command->get_direction();
	intercept = 0;
	slope = 0;

	PanAuto *prev_keyframe = (PanAuto*)autos->get_prev_auto(input_position, direction);
	PanAuto *next_keyframe = (PanAuto*)autos->get_next_auto(input_position, direction);
	
	if(direction == PLAY_FORWARD)
	{
// Two distinct automation points within range
		if(next_keyframe->position > prev_keyframe->position)
		{
			slope = ((double)next_keyframe->values[channel] - prev_keyframe->values[channel]) / slope_len;
			intercept = ((double)input_position - prev_keyframe->position) * slope + prev_keyframe->values[channel];

			if(next_keyframe->position < input_position + slope_len)
				slope_len = next_keyframe->position - input_position;
		}
		else
// One automation point within range
		{
			slope = 0;
			intercept = prev_keyframe->values[channel];
//printf("VirtualNode::get_pan_automation %d %f\n", channel, intercept);
		}
	}
	else
	{
// Two distinct automation points within range
		if(next_keyframe->position < prev_keyframe->position)
		{
			slope = ((double)next_keyframe->values[channel] - prev_keyframe->values[channel]) / slope_len;
			intercept = ((double)prev_keyframe->position - input_position) * slope + prev_keyframe->values[channel];

			if(prev_keyframe->position > input_position - slope_len)
				slope_len = input_position - prev_keyframe->position;
		}
		else
// One automation point within range
		{
			slope = 0;
			intercept = next_keyframe->values[channel];
		}
	}
}


int VirtualNode::init_automation(int &automate, 
				double &constant, 
				long input_position,
				long buffer_len,
				Autos *autos,
				Auto **before, 
				Auto **after)
{
	

	return autos->init_automation(buffer_position,
				input_start, 
				input_end, 
				automate, 
				constant, 
				input_position,
				buffer_len,
				before, 
				after,
				reverse);
}

int VirtualNode::init_slope(Autos *autos, Auto **before, Auto **after)
{
	return autos->init_slope(&current_auto,
				slope_start, 
				slope_value,
				slope_position, 
				input_start, 
				input_end, 
				before, 
				after,
				reverse);
}

int VirtualNode::get_slope(Autos *autos, long buffer_len, long buffer_position)
{
	return autos->get_slope(&current_auto, 
				slope_start, 
				slope_end, 
				slope_value, 
				slope, 
				buffer_len, 
				buffer_position,
				reverse);
}

int VirtualNode::advance_slope(Autos *autos)
{
	return autos->advance_slope(&current_auto, 
				slope_start, 
				slope_value,
				slope_position, 
				reverse);
}
