#include "console.h"
#include "datatype.h"
#include "edl.h"
#include "edlsession.h"
#include "mwindow.h"
#include "modules.h"
#include "playabletracks.h"
#include "pluginbuffer.h"
#include "preferences.h"
#include "renderengine.h"
#include "tracks.h"
#include "transportque.h"
#include "vframe.h"
#include "videodevice.h"
#include "virtualvconsole.h"
#include "virtualvnode.h"
#include "vmodule.h"
#include "vrender.h"
#include "vtrack.h"

VirtualVConsole::VirtualVConsole(RenderEngine *renderengine, VRender *vrender)
 : VirtualConsole(renderengine, vrender, TRACK_VIDEO)
{
//printf("VirtualVConsole::VirtualVConsole\n");
	this->vrender = vrender;
}

VirtualVConsole::~VirtualVConsole()
{
}

int VirtualVConsole::total_ring_buffers()
{
//printf("VirtualVConsole::total_ring_buffers\n");
	return 1;
}

void VirtualVConsole::get_playable_tracks()
{
	if(!playable_tracks)
		playable_tracks = new PlayableTracks(renderengine, 
			renderengine->config->vconfig->do_channel, 
			commonrender->current_position, 
			TRACK_VIDEO);
}

void VirtualVConsole::new_input_buffer(int ring_buffer)
{
	buffer_in = new VFrame*[total_tracks];
	for(int i = 0; i < total_tracks; i++)
	{
		buffer_in[i] = new VFrame(0,
			renderengine->edl->session->track_w,
			renderengine->edl->session->track_h,
			renderengine->edl->session->color_model,
			-1);
	}
}

void VirtualVConsole::delete_input_buffer(int ring_buffer)
{
	for(int i = 0; i < total_tracks; i++)
	{
		delete buffer_in[i];
	}
	delete [] buffer_in;
}

VirtualNode* VirtualVConsole::new_toplevel_node(Track *track, 
	Module *module,
	int track_number)
{
	return new VirtualVNode(renderengine, 
		this, 
		module,
		0,
		track,
		0,
		buffer_in[track_number],
		buffer_in[track_number],
		1,
		1,
		1,
		1);
}

// start of buffer in project if forward / end of buffer if reverse
int VirtualVConsole::process_buffer(long input_position)
{
	int i, j, k;

//printf("VirtualVConsole::process_buffer 1 %p\n", vrender->video_out[0]);
// clear output buffers
	for(i = 0; i < MAX_CHANNELS; i++)
	{
		if(vrender->video_out[i])
			vrender->video_out[i]->clear_frame();
	}

//printf("VirtualVConsole::process_buffer 2 %ld\n", input_position);
// Arm input buffers
	for(i = 0; i < total_tracks; i++)
		((VModule*)virtual_modules[i]->real_module)->render(buffer_in[i],
			input_position,
			renderengine->command->get_direction());

//printf("VirtualVConsole::process_buffer 3\n");
// render nodes in sorted list
	for(i = 0; i < render_list.total; i++)
	{
		((VirtualVNode*)render_list.values[i])->render(vrender->video_out, 
				input_position);
	}

//printf("VirtualVConsole::process_buffer 4\n");
	return 0;
}

















VirtualVConsole::VirtualVConsole(MWindow *mwindow, VRender *vrender)
 : VirtualConsole(mwindow, vrender)
{
	this->vrender = vrender;
}

int VirtualVConsole::init_rendering(int duplicate)
{
// 	playable_tracks = new PlayableTracks(mwindow, 
// 							vrender->current_position, 
// 							renderengine->reverse, 
// 							TRACK_VIDEO);
// 
// 	total_tracks = playable_tracks->total;
// 
// 	allocate_input_buffers();
// 	build_virtual_console(duplicate, vrender->current_position);
// 	sort_virtual_console();
// 	mwindow->console->modules->vrender_init(duplicate);
// 	mwindow->tracks->vrender_init(duplicate, vrender->current_position);
}

// int VirtualVConsole::build_virtual_console(int duplicate, long current_position)
// {
// // allocate the virtual modules and fork plugins
// 	total_virtual_modules = total_tracks;
// 	virtual_modules = new VirtualNode*[total_virtual_modules];
// 	PluginBuffer *buffer[MAX_BUFFERS];
// 	int i, j;
// 
// // build virtual console table
// 	for(i = 0, j; i < playable_tracks->total; i++)
// 	{
// // create a double buffer of just one track
// 		for(j = 0; j < total_buffers; j++)
// 			buffer[j] = buffer_in[j][i];
// 
// 		virtual_modules[i] = new VirtualVNode(mwindow, 
// 			playable_tracks->values[i]->get_module_of(), 
// 			0, 
// 			0, 
// 			playable_tracks->values[i]->get_patch_of(), 
// 			playable_tracks->values[i], 
// 			0,
// 			buffer,
// 			buffer,
// 			vrender->input_length,
// 			vrender->vmodule_render_fragment, 
// 			total_buffers,
// 			1,
// 			1,
// 			1,
// 			1,
// 			vrender->track_w, 
// 			vrender->track_h);
// 
// 		virtual_modules[i]->expand(duplicate, current_position);
// 	}
// }

int VirtualVConsole::stop_rendering(int duplicate)
{
// stop plugins
//	mwindow->console->modules->vrender_stop(duplicate);
//	mwindow->tracks->vrender_stop(duplicate);

//	delete_virtual_console();
//	delete_input_buffers();
}


int VirtualVConsole::send_last_output_buffer()
{
}
