#include "automation.h"
#include "edl.h"
#include "edlsession.h"
#include "floatautos.h"
#include "intauto.h"
#include "intautos.h"
#include "mwindow.h"
#include "module.h"
#include "overlayframe.h"
#include "plugin.h"
#include "pluginbuffer.h"
#include "preferences.h"
#include "renderengine.h"
#include "transition.h"
#include "transportque.h"
#include "vattachmentpoint.h"
#include "vframe.h"
#include "virtualvnode.h"
#include "vmodule.h"
#include "vtrack.h"

#include <string.h>


VirtualVNode::VirtualVNode(RenderEngine *renderengine, 
		VirtualConsole *vconsole, 
		Module *real_module, 
		Plugin *real_plugin,
		Track *track, 
		VirtualNode *parent_module, 
		VFrame *buffer_in,
		VFrame *buffer_out,
		int input_is_master,
		int output_is_master,
		int in,
		int out)
 : VirtualNode(renderengine, 
		vconsole, 
		real_module, 
		real_plugin,
		track, 
		parent_module, 
		input_is_master,
		output_is_master,
		in,
		out)
{
	this->buffer_in = buffer_in;
	this->buffer_out = buffer_out;
	overlayer = new OverlayFrame;
}

VirtualVNode::~VirtualVNode()
{
	if(!shared_output) delete [] buffer_out;
	if(!shared_input) delete [] buffer_in;
	delete overlayer;
}

VirtualNode* VirtualVNode::create_module(Plugin *real_plugin, 
							Module *real_module, 
							VirtualNode *parent_module, 
							Track *track)
{
	return new VirtualVNode(renderengine, 
		vconsole, 
		real_module,
		0,
		track,
		this,
		data_in_input ? buffer_in : buffer_out,
		buffer_out,
		data_in_input ? input_is_master : output_is_master,
		output_is_master,
		real_plugin->in,
		real_plugin->out);
}


VirtualNode* VirtualVNode::create_plugin(Plugin *real_plugin)
{
	return new VirtualVNode(renderengine, 
		vconsole, 
		0,
		real_plugin,
		track,
		this,
		data_in_input ? buffer_in : buffer_out,
		buffer_out,
		data_in_input ? input_is_master : output_is_master,
		output_is_master,
		real_plugin->in,
		real_plugin->out);
}


void VirtualVNode::new_output_buffer()
{
	buffer_out = new VFrame(0,
			renderengine->edl->session->track_w,
			renderengine->edl->session->track_h,
			renderengine->edl->session->color_model,
			-1);
}

void VirtualVNode::new_input_buffer()
{
	buffer_in = new VFrame(0,
			renderengine->edl->session->track_w,
			renderengine->edl->session->track_h,
			renderengine->edl->session->color_model,
			-1);
}

VFrame* VirtualVNode::get_module_input()
{
	VFrame* result;
	if(data_in_input)
	{
		result = buffer_in;
	}
	else
	{
		result = buffer_out;
	}
	return result;
}

VFrame* VirtualVNode::get_module_output()
{
	VFrame* result;
	result = buffer_out;
	return result;
}

int VirtualVNode::render(VFrame **video_out, long input_position)
{
	if(real_module)
	{
		render_as_module(video_out, input_position);
	}
	else
	if(real_plugin || real_transition)
	{
		render_as_plugin(input_position);
	}
	return 0;
}

void VirtualVNode::render_as_plugin(long input_position)
{
	if(!attachment ||
		!real_plugin ||
		!real_plugin->on) return;

	((VAttachmentPoint*)attachment)->render(buffer_in,
		buffer_out,
		input_position);
}


int VirtualVNode::render_as_module(VFrame **video_out, long input_position)
{
	this->reverse = reverse;
	VFrame *buffer_in = get_module_input();
	VFrame *buffer_out = get_module_output();

//printf("VirtualVNode::render_as_module 1 %d\n", buffer_out->get_color_model());
	render_fade(buffer_in, 
				buffer_out,
				input_position,
				track->automation->fade_autos);

//printf("VirtualVNode::render_as_module 2 %d\n", track->mute);

// video is definitely in output buffer now
// overlay on the final output
// Get mute status
	int mute_constant;
	long mute_fragment = 1;
	long mute_position = 0;

//printf("VirtualVNode::render_as_module 3\n");
// Is frame muted?
	get_mute_fragment(input_position,
			mute_constant, 
			mute_fragment, 
			(Autos*)((VTrack*)track)->automation->mute_autos);

//printf("VirtualVNode::render_as_module 4 %d\n", mute_constant);
	if(!mute_constant)
	{
// Fragment is playable
		render_projector(buffer_out,
			video_out,
			input_position);
	}

//printf("VirtualVNode::render_as_module 5\n");
	return 0;
}

int VirtualVNode::render_fade(VFrame *input,         // start of input fragment
			VFrame *output,        // start of output fragment
			long input_position,  // start of input fragment in project if forward / end of input fragment if reverse
			Autos *autos)
{
	double slope, intercept;
	long slope_len = 1;

	get_fade_automation(slope,
		intercept,
		input_position,
	   	slope_len,
		autos);

// copy only if buffers are different
	if(output->get_rows()[0] != input->get_rows()[0])
	{
		output->replace_from(input, intercept / 100);
	}
	else
	{
// apply fade directly
		input->apply_fade(intercept / 100);
	}
	return 0;
}

// Start of input fragment in project if forward.  End of input fragment if reverse.
int VirtualVNode::render_projector(VFrame *input,
			VFrame **output,
			long input_position)
{
	float in_x1, in_y1, in_x2, in_y2;
	float out_x1, out_y1, out_x2, out_y2;
	float float_input_position = input_position;

	for(int i = 0; i < MAX_CHANNELS; i++)
	{
		if(output[i])
		{
			((VTrack*)track)->calculate_output_transfer(i,
				(long)float_input_position,
				renderengine->command->get_direction(),
				in_x1, 
				in_y1, 
				in_x2, 
				in_y2,
				out_x1, 
				out_y1, 
				out_x2, 
				out_y2);

			in_x2 += in_x1;
			in_y2 += in_y1;
			out_x2 += out_x1;
			out_y2 += out_y1;

//for(int j = 0; j < input->get_w() * 3 * 5; j++)
//	input->get_rows()[0][j] = 255;
// 
// printf("VirtualVNode::render_projector %f %f %f %f -> %f %f %f %f\n",
// 	in_x1, 
// 	in_y1, 
// 	in_x2, 
// 	in_y2, 
// 	out_x1, 	
// 	out_y1, 
// 	out_x2, 
// 	out_y2);
			if(out_x2 > out_x1 && 
				out_y2 > out_y1 && 
				in_x2 > in_x1 && 
				in_y2 > in_y1)
			{
				IntAuto *mode_keyframe = 
					(IntAuto*)track->automation->mode_autos->get_prev_auto(input_position, renderengine->command->get_direction());

// overlay it depending on the function for this module
				transfer_from(output[i], 
					input, 
					in_x1, 
					in_y1, 
					in_x2, 
					in_y2,
					out_x1, 
					out_y1, 
					out_x2, 
					out_y2, 
					1,
					mode_keyframe->value);
			}
// for(int j = 0; j < output[i]->get_w() * 3 * 5; j++)
//  	output[i]->get_rows()[0][j] = 255;
		}
	}
	return 0;
}

int VirtualVNode::transfer_from(VFrame *frame_out, 
	VFrame *frame_in, 
	float in_x1, 
	float in_y1, 
	float in_x2, 
	float in_y2, 
	float out_x1, 
	float out_y1, 
	float out_x2, 
	float out_y2, 
	float alpha, 
	int mode)
{
//printf("VirtualVNode::transfer_from %d\n", track->mode);
	overlayer->overlay(frame_out, 
		frame_in,
		in_x1, 
		in_y1, 
		in_x2, 
		in_y2,
		out_x1, 
		out_y1, 
		out_x2, 
		out_y2, 
		alpha,
		mode, 
		renderengine->edl->session->interpolation_type);
	return 0;
}
