#include "assets.h"
#include "cache.h"
#include "clip.h"
#include "commonrender.h"
#include "console.h"
#include "edits.h"
#include "edl.h"
#include "edlsession.h"
#include "file.h"
#include "filexml.h"
#include "floatautos.h"
#include "mwindow.h"
#include "modules.h"
#include "overlayframe.h"
#include "patch.h"
#include "pluginbuffer.h"
#include "renderengine.h"
#include "sharedlocation.h"
#include "transition.h"
#include "transportque.h"
#include "units.h"
#include "vattachmentpoint.h"
#include "vedit.h"
#include "vframe.h"
#include "vmodule.h"
#include "vplugin.h"
#include "vtrack.h"
#include <string.h>

VModule::VModule(RenderEngine *renderengine, 
	CommonRender *commonrender, 
	Track *track)
 : Module(renderengine, commonrender, track)
{
	data_type = TRACK_VIDEO;
	input_temp = transition_temp = 0;
	overlayer = 0;
}

VModule::~VModule()
{
	if(input_temp) 
	{
		delete input_temp;
	}
	if(transition_temp) 
	{
		delete transition_temp;
	}
	if(overlayer) delete overlayer;
}


AttachmentPoint* VModule::new_attachment(Plugin *plugin)
{
	return new VAttachmentPoint(renderengine, plugin);
}

Cache* VModule::get_cache()
{
	if(renderengine) 
		return renderengine->get_vcache();
	else
		return cache;
}

void VModule::import_frame(VFrame *output,
	VEdit *current_edit,
	long input_position,
	int direction)
{
	long corrected_position = input_position;
	if(direction == PLAY_REVERSE) corrected_position--;
// Translation of edit
	float in_x1;
	float in_y1;
	float in_w1;
	float in_h1;
	float out_x1;
	float out_y1;
	float out_w1;
	float out_h1;

// Load frame into output
	if(current_edit &&
		current_edit->asset)
	{
//printf("VModule::import_frame 1\n");
		File *source = get_cache()->check_out(current_edit->asset);
		if(source)
		{
//printf("VModule::import_frame 1\n");
			source->set_video_position(corrected_position - 
				current_edit->startproject + 
				current_edit->startsource,
				get_edl()->session->frame_rate);
			source->set_layer(current_edit->channel);
//printf("VModule::import_frame 2\n");

			((VTrack*)track)->calculate_input_transfer(current_edit->asset, 
				input_position, 
				direction, 
				in_x1, 
				in_y1, 
				in_w1, 
				in_h1,
				out_x1, 
				out_y1, 
				out_w1, 
				out_h1);
// printf("VModule::import_frame 1 %0.2f %0.2f %0.2f %0.2f -> %0.2f %0.2f %0.2f %0.2f\n",
// 	in_x1,
// 	in_y1,
// 	in_w1,
// 	in_h1,
// 	out_x1,
// 	out_y1,
// 	out_w1,
// 	out_h1);

// file -> temp -> output
			if( !EQUIV(in_x1, 0) || 
				!EQUIV(in_y1, 0) || 
				!EQUIV(in_w1, get_edl()->session->track_w) || 
				!EQUIV(in_h1, get_edl()->session->track_h) || 
				!EQUIV(out_x1, 0) ||
				!EQUIV(out_y1, 0) ||
				!EQUIV(out_w1, get_edl()->session->track_w) ||
				!EQUIV(out_h1, get_edl()->session->track_h))
			{
// Fix buffers
//printf("VModule::import_frame 3\n");
				if(input_temp && 
					(input_temp->get_w() != current_edit->asset->width ||
					input_temp->get_h() != current_edit->asset->height))
				{
					delete input_temp;
					input_temp = 0;
				}

//printf("VModule::import_frame 4\n");
				if(!input_temp)
				{
					input_temp = new VFrame(0,
						current_edit->asset->width,
						current_edit->asset->height,
						get_edl()->session->color_model,
						-1);
				}
//printf("VModule::import_frame 5\n");

// file -> temp
				source->read_frame(input_temp);
				if(!overlayer)
				{
					overlayer = new OverlayFrame;
				}
// printf("VModule::import_frame 1 %.2f %.2f %.2f %.2f %.2f %.2f %.2f %.2f\n", 
// 	in_x1, 
// 	in_y1, 
// 	in_w1, 
// 	in_h1, 
// 	out_x1, 
// 	out_y1, 
// 	out_w1, 
// 	out_h1);

// temp -> output
// for(int j = 0; j < output->get_w() * 3 * 5; j++)
// 	output->get_rows()[0][j] = 255;

				output->clear_frame();
				overlayer->overlay(output,
					input_temp, 
					in_x1,
					in_y1,
					in_x1 + in_w1,
					in_y1 + in_h1,
					out_x1,
					out_y1,
					out_x1 + out_w1,
					out_y1 + out_h1,
					1,
					TRANSFER_REPLACE,
					get_edl()->session->interpolation_type);
			}
			else
// file -> output
			{
				source->read_frame(output);
			}
//printf("VModule::import_frame 6\n");

			get_cache()->check_in(current_edit->asset);
		}
		else
		{
			output->clear_frame();
		}
	}
	else
	{
		output->clear_frame();
	}
}



void VModule::render(VFrame *output,
	long input_position,
	int direction)
{
//printf("VModule::render 1\n");
	update_transition(input_position, direction);

	VEdit* current_edit = (VEdit*)track->edits->editof(input_position, 
		direction);
	VEdit* previous_edit = 0;
//printf("VModule::render 1\n");

	if(!current_edit) return;

//printf("VModule::render 1\n");
// Load output buffer
	import_frame(output, 
		current_edit, 
		input_position,
		direction);

//printf("VModule::render 1\n");
// Process transition
	if(transition)
	{
// Load transition buffer
//printf("VModule::render 2\n");
		previous_edit = (VEdit*)current_edit->previous;

		if(!transition_temp)
		{
			transition_temp = new VFrame(0,
				get_edl()->session->track_w,
				get_edl()->session->track_h,
				get_edl()->session->color_model,
				-1);
		}

//printf("VModule::render 3\n");
		import_frame(transition_temp, 
			previous_edit, 
			input_position,
			direction);

// Execute plugin with transition_temp and output here
//printf("VModule::render 4 %d - %d %d\n", input_position, current_edit->startproject, transition->length);
		transition_server->process_realtime(&transition_temp, 
			&output,
			(direction == PLAY_FORWARD) ? 
				(input_position - current_edit->startproject) :
				(input_position - current_edit->startproject - 1),
			transition->length);
	}
//printf("VModule::render 5\n");
}











































void VModule::create_objects()
{
	Module::create_objects();
}

int VModule::create_plugins(int &x, int &y)
{
	for(int i = 0; i < PLUGINS; i++)
	{
		plugins[i] = new VPlugin(mwindow, this, i+1);
		((VPlugin*)plugins[i])->create_objects(x, y);
		y += 44;

		if(!console->vertical && y + 44 > VMODULEHEIGHT) 
		{
			y = 3;
			x += VMODULEWIDTH - 3;
		}
	}
	if(!console->vertical && y > 3) { x += VMODULEWIDTH; y = 3; }
	return 0;
}

int VModule::flip_plugins(int &x, int &y)
{
	if(mwindow->gui)
	{
		for(int i = 0; i < PLUGINS; i++)
		{
			((VPlugin*)plugins[i])->resize_plugin(x, y);
			y += 44;
			if(!console->vertical && y + 44 > VMODULEHEIGHT)
			{
				y = 3;
				x += VMODULEWIDTH - 3;
			}
		}
		if(!console->vertical && y > 3) { x += VMODULEWIDTH; y = 3; }
	}
	return 0;
}

int VModule::set_pixel(int pixel)
{
	if(mwindow->gui)
	{
		this->pixel = pixel;

		if(console->vertical)
		gui->reposition_window(pixel, 0, gui->get_w(), gui->get_h());
		else
		gui->reposition_window(0, pixel, gui->get_w(), gui->get_h());
	}
	return 0;
}








// ================================ file operations

int VModule::save(FileXML *xml)
{
	xml->tag.set_title("MODULE");
	xml->append_tag();
	xml->append_newline();


// title must come before plugins to allow plugins to boot
	xml->tag.set_title("TITLE");
	xml->append_tag();
	xml->append_text(title);
	xml->tag.set_title("/TITLE");
	xml->append_tag();
	xml->append_newline();

	int i;
	
	if(mute)
	{
		xml->tag.set_title("MUTE");
		xml->append_tag();
		xml->append_newline();
	}
	
//	for(i = 0; i < PLUGINS; i++) plugins[i]->save(xml, "PLUGIN");
	
	xml->tag.set_title("MODE");
	xml->tag.set_property("VALUE", mode);
	xml->append_tag();
	xml->append_newline();

	xml->tag.set_title("FADE");
	xml->tag.set_property("VALUE", fade);
	xml->append_tag();
	xml->append_newline();

	xml->tag.set_title("/MODULE");
	xml->append_tag();
	xml->append_newline();
	return 0;
}

int VModule::load(FileXML *xml, int track_offset)
{
	int result = 0;
	int current_pan = 0;
	int current_plugin = 0;
	int i;
	DB db;

	mode = TRANSFER_NORMAL;
	do{
		result = xml->read_tag();
		
		if(!result)
		{
			if(!strcmp(xml->tag.get_title(), "/MODULE"))
			{
				result = 1;
			}
			else
			if(!strcmp(xml->tag.get_title(), "PLUGIN"))
			{
// title must come before plugins to allow plugins to boot
				if(current_plugin < PLUGINS)
				{
//					plugins[current_plugin]->load(xml, track_offset, "/PLUGIN");
					current_plugin++;
				}
			}
			else
			if(!strcmp(xml->tag.get_title(), "MUTE"))
			{
				mute = 1;
				if(gui) gui->mute_toggle->update(mute);
			}
			else
			if(!strcmp(xml->tag.get_title(), "TITLE"))
			{
// title must come before plugins to allow plugins to boot properly
				strcpy(title, xml->read_text());
				if(gui) gui->title->update(title);
			}
			else
			if(!strcmp(xml->tag.get_title(), "FADE"))
			{
				fade = xml->tag.get_property("VALUE", (float)100);
				if(gui) gui->fade_slider->update((long)fade);
			}
			else
			if(!strcmp(xml->tag.get_title(), "MODE"))
			{
				mode = xml->tag.get_property("VALUE", TRANSFER_NORMAL);
				if(gui) gui->mode_popup->set_text(gui->mode_popup->mode_to_text(mode));
			}
		}
	}while(!result);
	return 0;
}

int VModule::set_title(char *text)
{
	strcpy(title, text);
	if(gui) gui->title->update(text);
	
	for(int i = 0; i < PLUGINS; i++)
	{
		((VPlugin*)plugins[i])->set_string();
	}
	return 0;
}

int VModule::flip_vertical(int pixel) { if(gui) gui->flip_vertical(pixel); 	return 0;
}

int VModule::change_x(int new_pixel) { if(gui) gui->reposition_window(new_pixel, gui->get_x()); 	return 0;
}

int VModule::change_y(int new_pixel) { if(gui) gui->reposition_window(gui->get_x(), new_pixel); 	return 0;
}

int VModule::set_mute(int value)
{
	mute = value;
	if(gui) gui->mute_toggle->update(value);
	return 0;
}




VModuleGUI::VModuleGUI(MWindow *mwindow, VModule *module, int x, int y, int w, int h)
 : BC_SubWindow(x, y, w, h, MEGREY)
{
	this->mwindow = mwindow;
	this->console = mwindow->console;
	this->module = module;
}

VModuleGUI::~VModuleGUI()
{
	delete title;
	delete fade_slider;
}

int VModuleGUI::create_objects()
{
//	add_border();
	
	if(console->vertical)
	{
		int x, y;
// ================================== title

		x = 3; y = 3;
		add_subwindow(this->title = new VModuleTitle(module, module->get_patch_of(), x, y));

		y += 22;
		module->create_plugins(x, y);

// =================================== mode

		add_subwindow(mode_popup = new VModuleMode(mwindow->console, module, x, y));
		add_subwindow(mode_title = new BC_Title(x, y + 25, "Mode", SMALLFONT));

// ================================ fader

		y += 25;
		add_subwindow(fade_title = new BC_Title(70, y - 3, "Fade", SMALLFONT));
		y += 10;
		add_subwindow(fade_slider = new VModuleFade(module, 70, y, 30, 184));

// =================================== mute

		y += 65;
		add_subwindow(mute_toggle = new VModuleMute(mwindow->console, module, 40, y));
		add_subwindow(mute_title = new BC_Title(40, y + 20, "Mute", SMALLFONT));
	}
	else
	{
		int x, y;
// ================================== title

		x = 3; y = 3;
		add_subwindow(this->title = new VModuleTitle(module, module->get_patch_of(), x, y));

		y = 3;
		x += VMODULEWIDTH - 3;
		module->create_plugins(x, y);

// ================================ fader

		x += 70;
		add_subwindow(fade_title = new BC_Title(x, y + 30, "Fade", SMALLFONT));
		//y += 10;
		add_subwindow(fade_slider = new VModuleFade(module, x, y, 184, 30));

// =================================== mute

		y += 30;
		add_subwindow(mute_toggle = new VModuleMute(mwindow->console, module, x + 32, y));
		add_subwindow(mute_title = new BC_Title(x + 52, y+2, "Mute", SMALLFONT));

// =================================== mode

		y += 25;
		add_subwindow(mode_title = new BC_Title(x, y + 5, "Mode", SMALLFONT));
		x += 32; 
		add_subwindow(mode_popup = new VModuleMode(mwindow->console, module, x, y));
	}
	return 0;
}

int VModuleGUI::flip_vertical(int pixel)
{
	module->pixel = pixel;

	if(console->vertical)
	reposition_window(pixel, 0, VMODULEWIDTH, console->gui->get_h() - 17);
	else
	reposition_window(0, pixel, console->gui->get_w() - 17, VMODULEHEIGHT);
	
	if(console->vertical)
	{
		int x, y;
// ================================== title
		x = 3; y = 3;
		title->reposition_window(x, y);

// =================================== plugins
		y += 22;
		module->flip_plugins(x, y);

// =================================== mode

		mode_popup->reposition_window(x, y);
		mode_title->reposition_window(x, y + 25);

// ================================ fader
		y += 25;
		fade_title->reposition_window(70, y - 3);
		y += 10;
		fade_slider->reposition_window(70, y, 30, 184);

// =================================== mute
		y += 65;
		mute_toggle->reposition_window(40, y);
		mute_title->reposition_window(40, y+20);
	}
	else
	{
		int x, y;
// ================================== title
		x = 3; y = 3;
		title->reposition_window(x, y);

		y = 3;
		x += VMODULEWIDTH - 3;
		module->flip_plugins(x, y);

// ================================ fader
		x += 70;
		fade_slider->reposition_window(x, y, 184, 30);

// =================================== mute
		y += 30;
		mute_toggle->reposition_window(x + 32, y);
		mute_title->reposition_window(x + 52, y+2);

// =================================== mode
		y += 25;
		mode_title->reposition_window(x, y + 5);
		x += 32;
		mode_popup->reposition_window(x, y);
	}
	return 0;
}


VModuleTitle::VModuleTitle(VModule *module, Patch *patch, int x, int y)
 : BC_TextBox(x, y, 100, 1, patch->title, 0)
{
	this->module = module;
	this->patch = patch;
}

int VModuleTitle::handle_event()
{
	patch->set_title(get_text());
	strcpy(module->title, get_text());
	
	for(int i = 0; i < PLUGINS; i++)
	{
		((VPlugin*)module->plugins[i])->set_string();
	}
	return 0;
}

VModuleMute::VModuleMute(Console *console, VModule *module, int x, int y)
 : BC_CheckBox(x, y, 0)
{
	this->console = console;
	this->module = module;
}

int VModuleMute::handle_event()
{
	// value is changed before this
	console->button_down = 1;
	console->new_status = get_value();
	module->mute = get_value();
	return 0;
}

int VModuleMute::cursor_moved_over()
{
	if(console->button_down && console->new_status != get_value())
	{
		update(console->new_status);
		module->mute = get_value();
	}
	return 0;
}

int VModuleMute::button_release()
{
	console->button_down = 0;
	return 0;
}

VModuleFade::VModuleFade(VModule *module, int x, int y, int w, int h)
 : BC_ISlider(x, 
 	y, 
	1, 
	200, 
	200, 
	0, 
	100, 
	100)
{
	this->module = module;
}

VModuleFade::~VModuleFade()
{
}

int VModuleFade::handle_event()
{
		// value is changed before this
	module->fade = (float)get_value();
//printf("VModuleFade::handle_event %f\n", module->fade);
	return 0;
}


VModuleMode::VModuleMode(Console *console, VModule *module, int x, int y)
 : BC_PopupMenu(x, y, 80, mode_to_text(module->mode), 1)
{
	this->module = module;
	this->console = console;
}

VModuleMode::~VModuleMode()
{
}

int VModuleMode::handle_event()
{
	return 0;
}

int VModuleMode::add_items()         // add initial items
{
	return 0;
}


char* VModuleMode::mode_to_text(int mode)
{
	switch(mode)
	{
		case TRANSFER_NORMAL:
			return "Normal";
			break;
		
		case TRANSFER_REPLACE:
			return "Replace";
			break;
		
		case TRANSFER_ADDITION:
			return "Addition";
			break;
		
		case TRANSFER_SUBTRACT:
			return "Subtract";
			break;
		
		case TRANSFER_MULTIPLY:
			return "Multiply";
			break;
		
		case TRANSFER_DIVIDE:
			return "Divide";
			break;
		
		default:
			return "Normal";
			break;
	}
	return 0;
}

VModuleModeItem::VModuleModeItem(VModuleMode *popup, char *text, int mode)
 : BC_MenuItem(text)
{
	this->popup = popup;
	this->mode = mode;
}

VModuleModeItem::~VModuleModeItem()
{
}

int VModuleModeItem::handle_event()
{
	popup->set_text(get_text());
	popup->module->mode = mode;
	return 0;
}

