#ifndef MONTEVIDEO_H
#define MONTEVIDEO_H

#include "arraylist.h"
#include "assets.inc"
#include "audiodevice.inc"
#include "channel.inc"
#include "defaults.inc"
#include "montevideo.inc"
#include "mvaudioin.inc"
#include "mvaudioout.inc"
#include "mvconsole.inc"
#include "mvdatabase.inc"
#include "mvinterface.inc"
#include "mvvideoin.inc"
#include "mvvideoout.inc"
#include "mvwindow.inc"
#include "playbackconfig.inc"
#include "recordconfig.inc"
#include "vframe.inc"
#include "videodevice.inc"

class MonteVideo
{
public:
	MonteVideo();
	~MonteVideo();

	void reset();
	void load_defaults();
	void save_defaults();
	int calculate_cpus();
// Open input devices
	void open_input();
// Open output devices
	void open_output();
	void open_loopback_data();
	void write_audio(float **buffer, long samples, long time);
	void write_video(VFrame *frame, long time);
	int read_audio(float **buffer, long samples, long time);
	int read_video(VFrame *frame, long &time);
	void start_recording();
	void start_playback();
	void wait_recording();
	void wait_playback();
	void stop_all();
// Calculate size of audio fragments to read
	int calculate_input_size();
	void usage();
	static char* inttofourcc(int compressor);
	static char* inttocompressor(int compressor);
	void parse_args(int argc, char *argv[]);
	void init_interface();
	void create_objects(int argc, char *argv[]);
	void main_loop();
	void close_devices();
	void write_composite();
// The loopback machine
	void do_loopback();
// The inactive machine
	void do_inactive();
// Get number of samples since start of current machine for video synchronization
	long input_sync_position();
	long output_sync_position();
	void next_channel();
	void prev_channel();
	ArrayList<Channel*>* get_current_channeldb();
	Channel* get_current_channel();
	Channel* get_current_outchannel();

	Defaults *defaults;
	AudioInConfig *aconfig_in;
	VideoInConfig *vconfig_in;
	AudioOutConfig *aconfig_out;
	VideoOutConfig *vconfig_out;
	int canvas_w;
	int canvas_h;
	int window_x;
	int window_y;
	VideoDevice *video_in, *video_out;
	AudioDevice *audio_in, *audio_out;
	Asset *asset;
	int current_channel;
	int current_outchannel;
	int cpus;
	int done;
	int field_order;
	int deinterlace;
// Number of minutes to store in loop
	long loop_duration;
// Recording threads
	MVAudioIn *record_audio;
	MVVideoIn *record_video;
// Playback threads
	MVAudioOut *play_audio;
	MVVideoOut *play_video;
// Window for X11 playback
	MVWindow *window;
// Console for hardware playback
	MVConsole *console;
// Handle user input
	MVInterface *interface;
	int state;
	MVDatabase *database;
	ArrayList<Channel*> channeldb_v4l;
	ArrayList<Channel*> channeldb_buz;
// Overlay composited on screen
	VFrame *overlay;
// Time to display overlay
	long overlay_start, overlay_end, overlay_eternal;
};

#endif
