#include "assets.h"
#include "audiodevice.h"
#include "montevideo.h"
#include "mvaudioin.h"

MVAudioIn::MVAudioIn(MonteVideo *montevideo)
 : Thread()
{
	set_synchronous(1);
	this->montevideo = montevideo;
	done = 0;
	total_samples = 0;
	sync_timer = new Timer;
	for(int i = 0; i < montevideo->asset->channels; i++)
	{
		buffer[i] = new float[AUDIOBUFFER];
	}
}

MVAudioIn::~MVAudioIn()
{
	for(int i = 0; i < montevideo->asset->channels; i++)
	{
		delete [] buffer[i];
	}
}


long MVAudioIn::get_sync_position()
{
	timer_lock.lock();
	long result = total_samples + 
		sync_timer->get_scaled_difference(montevideo->asset->rate);
	timer_lock.unlock();
	return result;
}

void MVAudioIn::run()
{
	long fragment_size = montevideo->calculate_input_size();
	long buffer_offset = 0;
	float *buffer_ptr[MAXCHANNELS];
	int over[MAXCHANNELS];
	float max[MAXCHANNELS];


	timer_lock.lock();
	sync_timer->update();
	timer_lock.unlock();

	while(!done)
	{
		for(int i = 0; i < montevideo->asset->channels; i++)
		{
			buffer_ptr[i] = buffer[i] + buffer_offset;
		}

		montevideo->audio_in->read_buffer(buffer_ptr, 
			fragment_size, 
			montevideo->asset->channels, 
			over, 
			max, 
			0);

		buffer_offset += fragment_size;
		if(buffer_offset >= AUDIOBUFFER)
		{
			montevideo->write_audio(buffer, AUDIOBUFFER, total_samples);
			timer_lock.lock();
			total_samples += AUDIOBUFFER;
			buffer_offset = 0;
			sync_timer->update();
			timer_lock.unlock();
		}
	}
	
}
