#include "assets.h"
#include "clip.h"
#include "filehtal.h"
#include "montevideo.h"
#include "mutex.h"
#include "mvdatabase.h"
#include "sema.h"
#include "vframe.h"



MVDatabase::MVDatabase(MonteVideo *montevideo)
{
	this->montevideo = montevideo;
	loopback = new MVLoopBack(montevideo);
}

void MVDatabase::create_path(char *output, char *input)
{
	sprintf(output, "%s.db", input);
}

void MVDatabase::load()
{
	char path[BCTEXTLEN];
	create_path(path, montevideo->asset->path);
	
	FileHTAL file;
	if(!file.read_from_file(path, 0))
	{
		
	}
}

void MVDatabase::save()
{
}






ChunkTable::ChunkTable()
{
	current_chunk = 0;
}

ChunkTable::~ChunkTable()
{
}

Chunk* ChunkTable::get_next_chunk()
{
	if(current_chunk < total)
	{
		return values[current_chunk];
	}
	else
	{
		return new Chunk;
	}
	return 0;
}

void ChunkTable::set_next_chunk(Chunk *chunk)
{
	if(current_chunk >= total)
		append(chunk);
	values[current_chunk] = chunk;
	current_chunk++;
}




MVFileSet::MVFileSet(MonteVideo *montevideo)
{
	this->montevideo = montevideo;
	read_fd = 0;
	write_fd = 0;
	current_infile = -1;
	current_outfile = -1;
	db_lock = new Mutex;
	audio_temp_out = new int16_t[AUDIOBUFFER * montevideo->asset->channels];
	audio_temp_in = new int16_t[AUDIOBUFFER * montevideo->asset->channels];
}

MVFileSet::~MVFileSet()
{
	delete db_lock;
	delete [] audio_temp_out;
	delete [] audio_temp_in;
}


void MVFileSet::create_path(char *path, int number)
{
	sprintf(path, "%s.dat%003d", montevideo->asset->path, number);
}


void MVFileSet::open_file(int number)
{
	open_writes(number);
	open_reads(number);
}

void MVFileSet::open_writes(int number)
{
	char path[BCTEXTLEN];
	if(number != current_outfile)
	{
		if(current_outfile >= 0)
		{
			fclose(write_fd);
		}

		create_path(path, number);
		write_fd = fopen(path, "r");
		if(write_fd)
		{
			fclose(write_fd);
			write_fd = fopen(path, "rb+");
		}
		else
		{
			write_fd = fopen(path, "wb+");
		}
		current_outfile = number;
	}
}

void MVFileSet::open_reads(int number)
{
	char path[BCTEXTLEN];
	if(number != current_infile)
	{
		if(current_infile >= 0)
		{
			fclose(read_fd);
		}

		create_path(path, number);
		read_fd = fopen(path, "rb");
		current_infile = number;
	}
}


void MVFileSet::rewind_loop()
{
	open_writes(0);
	video_chunks.current_chunk = 0;
	audio_chunks.current_chunk = 0;
}

void MVFileSet::test_size()
{
}

int MVFileSet::write_data(unsigned char *buffer, long bytes, Chunk *chunk)
{
	test_size();
	chunk->file_number = current_outfile;
	chunk->offset = ftell(write_fd);
	chunk->bytes = bytes;
	int result = !fwrite(buffer, bytes, 1, write_fd);
	return result;
}

void MVFileSet::write_video(VFrame *frame, long time)
{
	db_lock->lock();
	Chunk *chunk = video_chunks.get_next_chunk();
	chunk->time = time;
	write_data(frame->get_data(), frame->get_compressed_size(), chunk);
	video_chunks.set_next_chunk(chunk);
	db_lock->unlock();
}

void MVFileSet::write_audio(float **buffer, long samples, long time)
{
	long offset = 0;
	long bytes = samples * montevideo->asset->channels * sizeof(int16_t);
	for(int i = 0; i < samples; i++)
	{
		for(int j = 0; j < montevideo->asset->channels; j++)
		{
			int value = (int)(buffer[j][i] * 32767 + 0.5);
			RECLIP(value, -32767, 32767);
			audio_temp_out[offset] = value;
			offset++;
		}
	}

	db_lock->lock();
	Chunk *chunk = audio_chunks.get_next_chunk();
	chunk->time = time;
	write_data((unsigned char*)audio_temp_out, bytes, chunk);
	audio_chunks.set_next_chunk(chunk);
	db_lock->unlock();
}

// Get the first chunk which ends after time
long MVFileSet::get_chunk(ChunkTable *chunks, long time)
{
	long chunk_number = -1;

	for(int i = chunks->current_chunk, count = 0; 
		count < chunks->total; 
		i++, count++)
	{
		if(i >= chunks->total) i = 0;
		if(chunks->values[i]->time >= time)
		{
			chunk_number = i;
			break;
		}
	}
	return chunk_number;
}

int MVFileSet::read_data(unsigned char *buffer, long bytes, Chunk *chunk)
{
	int result = 0;
//printf("MVFileSet::read_data %d\n", chunk->offset);
	if(fseek(read_fd, chunk->offset, SEEK_SET)) result = 1;
	if(!fread(buffer, bytes, 1, read_fd)) result = 1;
	return result;
}

int MVFileSet::read_audio_chunk(unsigned char *data, 
	long bytes, 
	ChunkTable *chunks, 
	long &time,
	long duration)
{
	long chunk_number = -1;
//printf("MVFileSet::read_chunk %d\n", time);

// Get closest chunk
	do
	{
		db_lock->lock();
		chunk_number = get_chunk(chunks, time);
// Spin until the chunk is available
		if(chunk_number < 0)
		{
			db_lock->unlock();
			Timer timer;
			timer.delay((long)((float)duration / montevideo->asset->rate * 1000 / 2));
		}
	}while(chunk_number < 0 && !montevideo->done);

	int result = 1;
	if(chunk_number >= 0)
	{
		time = chunks->values[chunk_number]->time;
//printf("MVFileSet::read_chunk %d\n", time);
		result = read_data(data, bytes, chunks->values[chunk_number]);
	}

	db_lock->unlock();
	return result;
}

int MVFileSet::read_audio(float **buffer, long samples, long time)
{
	int result = 0;
	long actual_time = time;
	long out_sample = 0;
	long in_sample;
	long offset;

	while(out_sample < samples && 
		!result &&
		!montevideo->done)
	{
//printf("MVFileSet::read_audio 1 %d %d\n", out_sample, samples);
// Get closest chunk
		result = read_audio_chunk((unsigned char*)audio_temp_in, 
			AUDIOBUFFER * montevideo->asset->channels * sizeof(int16_t), 
			&audio_chunks, 
			actual_time,
			samples);
//fwrite(audio_temp_in, AUDIOBUFFER * montevideo->asset->channels * sizeof(int16_t), 1, stdout);
//printf("MVFileSet::read_audio 1 %d %d %d\n", actual_time, time, out_sample);
		if(!result)
		{
			if(actual_time < time + out_sample)
				in_sample = time + out_sample - actual_time;
			else
				in_sample = 0;

		offset = in_sample * montevideo->asset->channels;
//printf("MVFileSet::read_audio 2 %p\n", audio_temp_in);
			for( ; in_sample < AUDIOBUFFER; in_sample++, out_sample++)
			{
				for(int j = 0; j < montevideo->asset->channels; j++)
				{
					buffer[j][out_sample] = (float)audio_temp_in[offset] / 0x7fff;
					offset++;
				}
			}
		}
	}

	return result;
}

int MVFileSet::read_video(VFrame *frame, long &time)
{
	int result = 0;
	long chunk_number = -1;

//printf("MVFileSet::read_video 1\n");
	do
	{
		db_lock->lock();
		chunk_number = get_chunk(&video_chunks, time);
// Spin until the chunk is available
		if(chunk_number < 0)
		{
			db_lock->unlock();
			Timer timer;
			timer.delay((long)((float)1000 / montevideo->asset->frame_rate / 2));
		}
	}while(chunk_number < 0 && !montevideo->done);
//printf("MVFileSet::read_video 1 %d %d\n", chunk_number, video_chunks.values[chunk_number]->bytes);

	if(chunk_number >= 0)
	{
		frame->allocate_compressed_data(video_chunks.values[chunk_number]->bytes);
//printf("MVFileSet::read_video 1\n");
		frame->set_compressed_size(video_chunks.values[chunk_number]->bytes);
		result = read_data(frame->get_data(), 
			frame->get_compressed_size(), 
			video_chunks.values[chunk_number]);
//printf("MVFileSet::read_video 1 %d\n", frame->get_compressed_size());
		time = video_chunks.values[chunk_number]->time;
	}
	db_lock->unlock();
//printf("MVFileSet::read_video 2 %d\n", result);

	return result;
}



// Recording fileset
MVRecording::MVRecording(MonteVideo *montevideo)
 : MVFileSet(montevideo)
{
}

// Loopback fileset
MVLoopBack::MVLoopBack(MonteVideo *montevideo)
 : MVFileSet(montevideo)
{
}





Chunk::Chunk()
{
	file_number = 0;
	offset = 0;
	bytes = 0;
	time = 0;
}

Chunk::Chunk(long time)
{
	this->time = time;
}


