#ifndef MVDATABASE_H
#define MVDATABASE_H

#include "arraylist.h"
#include "montevideo.inc"
#include "mutex.inc"
#include "sema.inc"

#include <stdint.h>
#include <stdio.h>

class Chunk
{
public:
	Chunk();
	Chunk(long time);

// Number of file
	int file_number;
// Bytes from start of file
	long offset;
// Size in bytes
	long bytes;
// Presentation time in samples
	long time;
};

class ChunkTable : public ArrayList<Chunk*>
{
public:
	ChunkTable();
	~ChunkTable();

	Chunk* get_next_chunk();
	void set_next_chunk(Chunk *chunk);
// Next chunk to write
	long current_chunk;
};


class MVFileSet
{
public:
	MVFileSet(MonteVideo *montevideo);
	~MVFileSet();

	void open_file(int number);
	void open_writes(int number);
	void open_reads(int number);
	void create_path(char *path, int number);
	void write_audio(float **buffer, long samples, long time);
	void write_video(VFrame *frame, long time);
	int write_data(unsigned char *buffer, long bytes, Chunk *chunk);
	long get_chunk(ChunkTable *chunks, long time);
	int read_audio_chunk(unsigned char *data, 
		long bytes, 
		ChunkTable *chunks, 
		long &time,
		long duration);
	int read_audio(float **buffer, long samples, long time);
	int read_video(VFrame *frame, long &time);
	int read_data(unsigned char *buffer, long bytes, Chunk *chunk);
// Rewind to the start of the file set
	void rewind_loop();
// Open new files after 32bit overflow
	void test_size();

	FILE *read_fd;
	FILE *write_fd;
// Number of current file open
	int current_infile;
	int current_outfile;
	ArrayList<char*> files;
	MonteVideo *montevideo;
	ChunkTable video_chunks;
	ChunkTable audio_chunks;
// Lock database
	Mutex *db_lock;
	int16_t *audio_temp_out;
	int16_t *audio_temp_in;
};


// Recording fileset
class MVRecording : public MVFileSet
{
public:
	MVRecording(MonteVideo *montevideo);
};

// Loopback fileset
class MVLoopBack : public MVFileSet
{
public:
	MVLoopBack(MonteVideo *montevideo);
};


class MVDatabase
{
public:
	MVDatabase(MonteVideo *montevideo);

	void load();
	void save();
	void create_path(char *output, char *input);

// One fileset for the loopback
	MVLoopBack *loopback;
// One fileset for each recording made
	ArrayList<MVRecording*> recording;

	MonteVideo *montevideo;
};

#endif
