#include "assets.h"
#include "libmjpeg.h"
#include "montevideo.h"
#include "mvvideoin.h"
#include "quicktime.h"
#include "timer.h"
#include "vframe.h"
#include "videodevice.h"

MVVideoIn::MVVideoIn(MonteVideo *montevideo)
 : Thread()
{
	set_synchronous(1);
	this->montevideo = montevideo;
	done = 0;
}


void MVVideoIn::run()
{
// Compressed data
	VFrame *frame = new VFrame;
// Temporary storage for uncompressed data
	VFrame *temp_frame = 0;
	Timer delayer;

	create_mjpeg();

	total_frames = 0;
//printf("MVVideoIn 1\n");
	if(!montevideo->video_in->is_compressed())
	{
		int colormodel = montevideo->video_in->get_best_colormodel(montevideo->asset);
		temp_frame = new VFrame(0, 
			0,
			0,
			0, 
			montevideo->asset->width, 
			montevideo->asset->height, 
			colormodel, 
			-1);
	}
//printf("MVVideoIn 1\n");

	while(!done)
	{
		long next_sample = (long)((float)total_frames / 
			montevideo->asset->frame_rate * 
			montevideo->asset->rate);
		long current_sample = montevideo->input_sync_position();

//printf("MVVideoIn 1 %d %d\n", current_sample, next_sample);
		if(current_sample < next_sample)
		{
			long delay = (long)((float)(next_sample - current_sample) / 
				montevideo->asset->rate * 
				1000);
//printf("MVVideoIn::run 2 %d\n", delay);
// Sanity check and delay.
			if(delay < 10000 && delay > 0) delayer.delay(delay);
		}

//printf("MVVideoIn 2 %d\n", montevideo->field_order);
// Read frame
// Get presentation time of frame
		current_sample = montevideo->input_sync_position();
		montevideo->video_in->set_field_order(montevideo->field_order);
		if(temp_frame)
		{
			montevideo->video_in->read_buffer(temp_frame);
			compress_frame(temp_frame, frame);
		}
		else
		{
			montevideo->video_in->read_buffer(frame);
		}

//printf("MVVideoIn 1 %d\n", frame->get_compressed_size());
// Write to disk
		montevideo->write_video(frame, current_sample);
//printf("MVVideoIn 2\n");
		total_frames++;
	}
	mjpeg_delete(mjpeg);
}

void MVVideoIn::delete_mjpeg()
{
	mjpeg_delete(mjpeg);
}

void MVVideoIn::create_mjpeg()
{
	int fields = (!strcmp(montevideo->asset->compression, QUICKTIME_MJPA)) ? 2 : 1;

	mjpeg = mjpeg_new(montevideo->asset->width, 
		montevideo->asset->height, 
		fields);
	mjpeg_set_quality(mjpeg, montevideo->asset->quality);
	mjpeg_set_float(mjpeg, 0);
}

void MVVideoIn::compress_frame(VFrame *frame, VFrame *output)
{
	mjpeg_set_deinterlace(mjpeg, montevideo->deinterlace);
	mjpeg_compress(mjpeg, 
		frame->get_rows(), 
		frame->get_y(), 
		frame->get_u(), 
		frame->get_v(),
		frame->get_color_model(),
		montevideo->cpus);
	output->allocate_compressed_data(mjpeg_output_size(mjpeg));
	output->set_compressed_size(mjpeg_output_size(mjpeg));
	bcopy(mjpeg_output_buffer(mjpeg), output->get_data(), mjpeg_output_size(mjpeg));
}







