#include "assets.h"
#include "colormodels.h"
#include "libmjpeg.h"
#include "montevideo.h"
#include "mvvideoout.h"
#include "playbackconfig.h"
#include "pluginbuffer.inc"
#include "quicktime.h"
#include "vframe.h"
#include "videodevice.h"

#include <stdio.h>

MVVideoOut::MVVideoOut(MonteVideo *montevideo)
 : Thread()
{
	set_synchronous(1);
	this->montevideo = montevideo;
	done = 0;
}

MVVideoOut::~MVVideoOut()
{
}

int MVVideoOut::get_best_colormodel()
{
	switch(montevideo->vconfig_out->driver)
	{
		case PLAYBACK_X11:
			if(!strcmp(montevideo->asset->compression, QUICKTIME_JPEG)) return BC_YUV420P;
			if(!strcmp(montevideo->asset->compression, QUICKTIME_MJPA)) return BC_YUV422P;
			break;
		case PLAYBACK_BUZ:
			if(!strcmp(montevideo->asset->compression, QUICKTIME_MJPA)) return BC_COMPRESSED;
			break;
	}
	return BC_YUV888;
}

void MVVideoOut::run()
{
	VFrame *output_frame[MAX_CHANNELS], *temp_frame = 0;
	PluginBuffer *pluginbuffer;
//printf("MVVideoOut::run 1\n");
	int fields = (!strcmp(montevideo->asset->compression, QUICKTIME_MJPA)) ? 2 : 1;
//fields = 1;
	mjpeg_t *mjpeg = mjpeg_new(montevideo->asset->width, 
		montevideo->asset->height, 
		fields);
	long current_sample;
//printf("MVVideoOut::run 1\n");

// Negotiate colormodel
	int colormodel = get_best_colormodel();
//printf("MVVideoOut::run 1\n");

	while(!done)
	{
//printf("MVVideoOut::run 2\n");
		montevideo->video_out->new_output_buffers(output_frame,
		 	colormodel, 
			pluginbuffer);
//printf("MVVideoOut::run 2\n");
// Make storage for compressed data
		if(output_frame[0]->get_color_model() != BC_COMPRESSED)
		{
			if(!temp_frame)
			{
				temp_frame = new VFrame;
			}
		}
//printf("MVVideoOut::run 2\n");

		VFrame *compressed_frame = temp_frame ? temp_frame : output_frame[0];
		long frame_sample = current_sample;
		int result = 0;
//printf("MVVideoOut::run 2\n");

		current_sample = montevideo->output_sync_position();
		result = montevideo->read_video(compressed_frame, frame_sample);
//printf("MVVideoOut::run 2\n");

		if(temp_frame && !result)
		{
			mjpeg_decompress(mjpeg, 
				compressed_frame->get_data(), 
				compressed_frame->get_compressed_size(),
				mjpeg_get_field2(compressed_frame->get_data(), 
					compressed_frame->get_compressed_size()),  
				output_frame[0]->get_rows(), 
				output_frame[0]->get_y(), 
				output_frame[0]->get_u(), 
				output_frame[0]->get_v(),
				output_frame[0]->get_color_model(),
				montevideo->cpus);
		}
//printf("MVVideoOut::run 4\n");

		current_sample = montevideo->output_sync_position();
//printf("MVVideoOut::run 5\n");
		if(frame_sample > current_sample)
		{
			Timer timer;
			timer.delay((long)((float)(frame_sample - current_sample) / 
				montevideo->asset->rate *
				1000));
		}
//printf("MVVideoOut::run 6\n");
		montevideo->video_out->write_buffer(output_frame);
//printf("MVVideoOut::run 7\n");
	}
	mjpeg_delete(mjpeg);
}
