#include "bcdisplayinfo.h"
#include "blur.h"
#include "blurwindow.h"


BlurThread::BlurThread(BlurMain *client)
 : Thread()
{
	this->client = client;
	set_synchronous(0);
	gui_started.lock();
	completion.lock();
}

BlurThread::~BlurThread()
{
	delete window;
}
	
void BlurThread::run()
{
	BC_DisplayInfo info;
	window = new BlurWindow(client, 
		info.get_abs_cursor_x() - 75, 
		info.get_abs_cursor_y() - 65);
	window->create_objects();
	gui_started.unlock();
	int result = window->run_window();
	completion.unlock();
// Last command executed in thread
//printf("BlurThread::run 1\n");
	if(result) client->client_side_close();
//printf("BlurThread::run 2\n");
}






BlurWindow::BlurWindow(BlurMain *client, int x, int y)
 : BC_Window(client->gui_string, 
 	x,
	y,
	150, 
	130, 
	150, 
	130, 
	0, 
	1)
{ 
	this->client = client; 
}

BlurWindow::~BlurWindow()
{
}

int BlurWindow::create_objects()
{
	int x = 10, y = 10;
	add_tool(new BC_Title(x, y, "Blur"));
	y += 20;
	add_tool(horizontal = new BlurHorizontal(client, this, x, y));
	y += 30;
	add_tool(vertical = new BlurVertical(client, this, x, y));
	y += 25;
	add_tool(radius = new BlurRadius(client, x, y));
	x += 50;
	add_tool(new BC_Title(x, y, "Radius"));
	show_window();
	flush();
	return 0;
}

int BlurWindow::close_event()
{
// Set result to 1 to indicate a client side close
	set_done(1);
	return 1;
}

BlurRadius::BlurRadius(BlurMain *client, int x, int y)
 : BC_IPot(x, 
 	y, 
	client->radius, 
	0, 
	MAXRADIUS)
{
	this->client = client;
}
BlurRadius::~BlurRadius()
{
}
int BlurRadius::handle_event()
{
	client->radius = get_value();
	client->send_configure_change();
}

BlurVertical::BlurVertical(BlurMain *client, BlurWindow *window, int x, int y)
 : BC_CheckBox(x, 
 	y, 
	client->vertical, 
	"Vertical")
{
	this->client = client;
	this->window = window;
}
BlurVertical::~BlurVertical()
{
}
int BlurVertical::handle_event()
{
	client->vertical = get_value();
	client->send_configure_change();
}

BlurHorizontal::BlurHorizontal(BlurMain *client, BlurWindow *window, int x, int y)
 : BC_CheckBox(x, 
 	y, 
	client->horizontal, 
	"Horizontal")
{
	this->client = client;
	this->window = window;
}
BlurHorizontal::~BlurHorizontal()
{
}
int BlurHorizontal::handle_event()
{
	client->horizontal = get_value();
	client->send_configure_change();
}
