#include "plugincolors.h"

#include <stdio.h>

HSV::HSV()
{
}


HSV::~HSV()
{
}



int HSV::rgb_to_hsv(float r, float g, float b, float &h, float &s, float &v)
{
    register int i;
	register float min, max, delta;
	register float f, p, q, t;
	min = ((r < g) ? r : g) < b ? ((r < g) ? r : g) : b;
	max = ((r > g) ? r : g) > b ? ((r > g) ? r : g) : b;
	v = max;                               // v

	delta = max - min;

//printf("HSV::rgb_to_hsv %f %f %f\n", max, min, delta);
	if(max != 0 && delta != 0)
        s = delta / max;               // s
	else 
	{
        // r = g = b = 0                // s = 0, v is undefined
        s = 0;
        h = -1;
        return 0;
	}

	if(r == max)
        h = (g - b) / delta;         // between yellow & magenta
	else 
	if(g == max)
        h = 2 + (b - r) / delta;     // between cyan & yellow
	else
        h = 4 + (r - g) / delta;     // between magenta & cyan

	h *= 60;                               // degrees
	if(h < 0)
        h += 360;
}

int HSV::hsv_to_rgb(float &r, float &g, float &b, float h, float s, float v)
{
    register int i;
	register float min, max, delta;
	register float f, p, q, t;
    if(s == 0) 
	{
        // achromatic (grey)
        r = g = b = v;
        return 0;
    }

    h /= 60;                        // sector 0 to 5
    i = (int)h;
    f = h - i;                      // factorial part of h
    p = v * (1 - s);
    q = v * (1 - s * f);
    t = v * (1 - s * (1 - f));

    switch(i) 
	{
        case 0:
            r = v;
            g = t;
            b = p;
            break;
        case 1:
            r = q;
            g = v;
            b = p;
            break;
        case 2:
            r = p;
            g = v;
            b = t;
            break;
        case 3:
            r = p;
            g = q;
            b = v;
            break;
        case 4:
            r = t;
            g = p;
            b = v;
            break;
        default:                // case 5:
            r = v;
            g = p;
            b = q;
            break;
    }
}


YUV::YUV()
{
	for(int i = 0; i < 0x100; i++)
	{
// compression
		rtoy_tab_8[i] = (long)( 0.2990 * 0x100 * i);
		rtou_tab_8[i] = (long)(-0.1687 * 0x100 * i);
		rtov_tab_8[i] = (long)( 0.5000 * 0x100 * i);

		gtoy_tab_8[i] = (long)( 0.5870 * 0x100 * i);
		gtou_tab_8[i] = (long)(-0.3320 * 0x100 * i);
		gtov_tab_8[i] = (long)(-0.4187 * 0x100 * i);

		btoy_tab_8[i] = (long)( 0.1140 * 0x100 * i);
		btou_tab_8[i] = (long)( 0.5000 * 0x100 * i) + 0x8000;
		btov_tab_8[i] = (long)(-0.0813 * 0x100 * i) + 0x8000;
	}

	vtor_8 = &(vtor_tab_8[(0x100) / 2]);
	vtog_8 = &(vtog_tab_8[(0x100) / 2]);
	utog_8 = &(utog_tab_8[(0x100) / 2]);
	utob_8 = &(utob_tab_8[(0x100) / 2]);

	for(int i = (-0x100) / 2; i < (0x100) / 2; i++)
	{
// decompression
		vtor_8[i] = (long)( 1.4020 * 0x100 * i);
		vtog_8[i] = (long)(-0.7141 * 0x100 * i);

		utog_8[i] = (long)(-0.3441 * 0x100 * i);
		utob_8[i] = (long)( 1.7720 * 0x100 * i);
	}

	for(int i = 0; i < 0x10000; i++)
	{
// compression
		rtoy_tab_16[i] = (long)( 0.2990 * 0x100 * i);
		rtou_tab_16[i] = (long)(-0.1687 * 0x100 * i);
		rtov_tab_16[i] = (long)( 0.5000 * 0x100 * i);

		gtoy_tab_16[i] = (long)( 0.5870 * 0x100 * i);
		gtou_tab_16[i] = (long)(-0.3320 * 0x100 * i);
		gtov_tab_16[i] = (long)(-0.4187 * 0x100 * i);

		btoy_tab_16[i] = (long)( 0.1140 * 0x100 * i);
		btou_tab_16[i] = (long)( 0.5000 * 0x100 * i) + 0x800000;
		btov_tab_16[i] = (long)(-0.0813 * 0x100 * i) + 0x800000;
	}

	vtor_16 = &(vtor_tab_16[(0x10000) / 2]);
	vtog_16 = &(vtog_tab_16[(0x10000) / 2]);
	utog_16 = &(utog_tab_16[(0x10000) / 2]);
	utob_16 = &(utob_tab_16[(0x10000) / 2]);

	for(int i = (-0x10000) / 2; i < (0x10000) / 2; i++)
	{
// decompression
		vtor_16[i] = (long)( 1.4020 * 0x100 * i);
		vtog_16[i] = (long)(-0.7141 * 0x100 * i);

		utog_16[i] = (long)(-0.3441 * 0x100 * i);
		utob_16[i] = (long)( 1.7720 * 0x100 * i);
	}
}

YUV::~YUV()
{
}
