#include "bcdisplayinfo.h"
#include "crossfade.h"
#include "crossfadewindow.h"


CrossfadeThread::CrossfadeThread(CrossfadeMain *client)
 : Thread()
{
	this->client = client;
	set_synchronous(1); // make thread wait for join
	gui_started.lock();
}

CrossfadeThread::~CrossfadeThread()
{
}
	
void CrossfadeThread::run()
{
	BC_DisplayInfo info;
	window = new CrossfadeWindow(client, 
		info.get_abs_cursor_x(), 
		info.get_abs_cursor_y());
	window->create_objects();
	gui_started.unlock();
	window->run_window();
	delete window;
}






CrossfadeWindow::CrossfadeWindow(CrossfadeMain *client, int x, int y)
 : BC_Window(client->gui_string, 
 	x,
	y,
	150, 
	130, 
	150, 
	130, 
	0, 
	1)
{ this->client = client; }

CrossfadeWindow::~CrossfadeWindow()
{
	delete vertical;
	delete horizontal;
	delete radius;
}

int CrossfadeWindow::create_objects()
{
	int x = 10, y = 10;
	add_tool(new BC_Title(x, y, "Crossfade"));
	y += 20;
	add_tool(horizontal = new CrossfadeHorizontal(client, this, x, y));
	y += 30;
	add_tool(vertical = new CrossfadeVertical(client, this, x, y));
	y += 25;
	add_tool(radius = new CrossfadeRadius(client, x, y));
	x += 50;
	add_tool(new BC_Title(x, y, "Radius"));
	y += 20;
	add_tool(new BC_Title(x, y, "(Automated)"));
	show_window();
	flush();
	return 0;
}

int CrossfadeWindow::close_event()
{
	hide_window();
	client->send_hide_gui();
}

CrossfadeRadius::CrossfadeRadius(CrossfadeMain *client, int x, int y)
 : BC_IPot(x, 
 	y, 
	client->radius, 
	0, 
	MAXRADIUS)
{
	this->client = client;
}
CrossfadeRadius::~CrossfadeRadius()
{
}
int CrossfadeRadius::handle_event()
{
	client->radius = get_value();
	client->send_configure_change();
}

CrossfadeVertical::CrossfadeVertical(CrossfadeMain *client, CrossfadeWindow *window, int x, int y)
 : BC_CheckBox(x, 
 	y, 
	client->vertical, 
	"Vertical")
{
	this->client = client;
	this->window = window;
}
CrossfadeVertical::~CrossfadeVertical()
{
}
int CrossfadeVertical::handle_event()
{
	client->vertical = get_value();
	client->send_configure_change();
}

CrossfadeHorizontal::CrossfadeHorizontal(CrossfadeMain *client, CrossfadeWindow *window, int x, int y)
 : BC_CheckBox(x, 
 	y, 
	client->horizontal, 
	"Horizontal")
{
	this->client = client;
	this->window = window;
}
CrossfadeHorizontal::~CrossfadeHorizontal()
{
}
int CrossfadeHorizontal::handle_event()
{
	client->horizontal = get_value();
	client->send_configure_change();
}
