#include "bcdisplayinfo.h"
#include "dissolve.h"
#include "dissolvewindow.h"


DissolveThread::DissolveThread(DissolveMain *client)
 : Thread()
{
	this->client = client;
	set_synchronous(1); // make thread wait for join
	gui_started.lock();
}

DissolveThread::~DissolveThread()
{
}
	
void DissolveThread::run()
{
	BC_DisplayInfo info;
	window = new DissolveWindow(client, 
		info.get_abs_cursor_x(), 
		info.get_abs_cursor_y());
	window->create_objects();
	gui_started.unlock();
	window->run_window();
	delete window;
}






DissolveWindow::DissolveWindow(DissolveMain *client, int x, int y)
 : BC_Window(client->gui_string, 
 	x,
	y,
	150, 
	130, 
	150, 
	130, 
	0, 
	1)
{ this->client = client; }

DissolveWindow::~DissolveWindow()
{
	delete vertical;
	delete horizontal;
	delete radius;
}

int DissolveWindow::create_objects()
{
	int x = 10, y = 10;
	add_tool(new BC_Title(x, y, "Dissolve"));
	y += 20;
	add_tool(horizontal = new DissolveHorizontal(client, this, x, y));
	y += 30;
	add_tool(vertical = new DissolveVertical(client, this, x, y));
	y += 25;
	add_tool(radius = new DissolveRadius(client, x, y));
	x += 50;
	add_tool(new BC_Title(x, y, "Radius"));
	y += 20;
	add_tool(new BC_Title(x, y, "(Automated)"));
	show_window();
	flush();
	return 0;
}

int DissolveWindow::close_event()
{
	hide_window();
	client->send_hide_gui();
}

DissolveRadius::DissolveRadius(DissolveMain *client, int x, int y)
 : BC_IPot(x, 
 	y, 
	client->radius, 
	0, 
	MAXRADIUS)
{
	this->client = client;
}
DissolveRadius::~DissolveRadius()
{
}
int DissolveRadius::handle_event()
{
	client->radius = get_value();
	client->send_configure_change();
}

DissolveVertical::DissolveVertical(DissolveMain *client, DissolveWindow *window, int x, int y)
 : BC_CheckBox(x, 
 	y, 
	client->vertical, 
	"Vertical")
{
	this->client = client;
	this->window = window;
}
DissolveVertical::~DissolveVertical()
{
}
int DissolveVertical::handle_event()
{
	client->vertical = get_value();
	client->send_configure_change();
}

DissolveHorizontal::DissolveHorizontal(DissolveMain *client, DissolveWindow *window, int x, int y)
 : BC_CheckBox(x, 
 	y, 
	client->horizontal, 
	"Horizontal")
{
	this->client = client;
	this->window = window;
}
DissolveHorizontal::~DissolveHorizontal()
{
}
int DissolveHorizontal::handle_event()
{
	client->horizontal = get_value();
	client->send_configure_change();
}
