#include "microtheme.h"


PluginClient* new_plugin(PluginServer *server)
{
	return new MicroThemeMain(server);
}


MicroThemeMain::MicroThemeMain(PluginServer *server)
 : PluginTClient(server)
{
}

MicroThemeMain::~MicroThemeMain()
{
}


char* MicroThemeMain::plugin_title() 
{ 
	return "Microscopic"; 
}

Theme* MicroThemeMain::new_theme()
{
	return theme = new MicroTheme;
}





MicroTheme::MicroTheme()
 : Theme()
{
	init_transports();
	init_buttonbars();
	init_backgrounds();
	init_buttons();
}
MicroTheme::~MicroTheme()
{
}


void MicroTheme::init_transports()
{
	#include "transportup_png.h"
	#include "transporthi_png.h"
	#include "transportdn_png.h"
	VFrame *transport_bg[] = 
	{
		new VFrame(transportup_png), new VFrame(transporthi_png), new VFrame(transportdn_png)
	};

	#include "rewind_png.h"
	Theme::build_transport(rewind_data,
		rewind_png,
		transport_bg, 
		0);
	#include "fastrev_png.h"
	Theme::build_transport(fastrev_data,
		fastrev_png,
		transport_bg, 
		1);
	#include "framerev_png.h"
	Theme::build_transport(framerev_data,
		framerev_png,
		transport_bg, 
		1);
	#include "reverse_png.h"
	Theme::build_transport(reverse_data,
		reverse_png,
		transport_bg, 
		1);
	#include "stop_png.h"
	Theme::build_transport(stop_data,
		stop_png,
		transport_bg, 
		1);
	Theme::build_transport(stoprec_data,
		stop_png,
		transport_bg, 
		2);
	#include "framefwd_png.h"
	Theme::build_transport(framefwd_data,
		framefwd_png,
		transport_bg, 
		1);
	#include "play_png.h"
	Theme::build_transport(forward_data,
		play_png,
		transport_bg, 
		1);
	#include "framefwd_png.h"
	Theme::build_transport(framefwd_data,
		framefwd_png,
		transport_bg, 
		1);
	#include "fastfwd_png.h"
	Theme::build_transport(fastfwd_data,
		fastfwd_png,
		transport_bg, 
		1);
	#include "end_png.h"
	Theme::build_transport(end_data,
		end_png,
		transport_bg, 
		2);
	#include "record_png.h"
	Theme::build_transport(rec_data,
		record_png,
		transport_bg, 
		1);
	#include "duplex_png.h"
	Theme::build_transport(duplex_data,
		duplex_png,
		transport_bg, 
		1);
	#include "singleframe_png.h"
	Theme::build_transport(recframe_data,
		singleframe_png,
		transport_bg, 
		1);

}




void MicroTheme::init_buttonbars()
{
#include "editpanel_up_png.h"
#include "editpanel_hi_png.h"
#include "editpanel_dn_png.h"
#include "editpanel_checked_png.h"
#include "editpanel_checkedhi_png.h"
// Editing panel
	VFrame editpanel_up(editpanel_up_png);
	VFrame editpanel_hi(editpanel_hi_png);
	VFrame editpanel_dn(editpanel_dn_png);
	VFrame editpanel_checked(editpanel_checked_png);
	VFrame editpanel_checkedhi(editpanel_checkedhi_png);

#include "arrow_png.h"
	Theme::build_toggle(arrow_data, arrow_png, &editpanel_up, &editpanel_hi, &editpanel_checked, &editpanel_dn, &editpanel_checkedhi);
#include "autokeyframe_png.h"
	Theme::build_toggle(autokeyframe_data, autokeyframe_png, &editpanel_up, &editpanel_hi, &editpanel_checked, &editpanel_dn, &editpanel_checkedhi);
#include "theme2/camera_png.h"
	Theme::build_toggle(camera_data, camera_png, &editpanel_up, &editpanel_hi, &editpanel_checked, &editpanel_dn, &editpanel_checkedhi);
#include "copy_png.h"
	Theme::build_button(copy_data, copy_png, &editpanel_up, &editpanel_hi, &editpanel_dn);
#include "theme2/crop_png.h"
	Theme::build_toggle(crop_data, crop_png, &editpanel_up, &editpanel_hi, &editpanel_checked, &editpanel_dn, &editpanel_checkedhi);
#include "cut_png.h"
	Theme::build_button(cut_data, cut_png, &editpanel_up, &editpanel_hi, &editpanel_dn);
#include "fit_png.h"
	Theme::build_button(fit_data, fit_png, &editpanel_up, &editpanel_hi, &editpanel_dn);
#include "ibeam_png.h"
	Theme::build_toggle(ibeam_data, ibeam_png, &editpanel_up, &editpanel_hi, &editpanel_checked, &editpanel_dn, &editpanel_checkedhi);
#include "inpoint_png.h"
	Theme::build_button(out_data, inpoint_png, &editpanel_up, &editpanel_hi, &editpanel_dn);
#include "label_png.h"
	Theme::build_button(labelbutton_data, label_png, &editpanel_up, &editpanel_hi, &editpanel_dn);
#include "nextlabel_png.h"
	Theme::build_button(nextlabel_data, nextlabel_png, &editpanel_up, &editpanel_hi, &editpanel_dn);
#include "outpoint_png.h"
	Theme::build_button(in_data, outpoint_png, &editpanel_up, &editpanel_hi, &editpanel_dn);
#include "overwrite_png.h"
	Theme::build_button(overwrite_data, overwrite_png, &editpanel_up, &editpanel_hi, &editpanel_dn);
#include "paste_png.h"
	Theme::build_button(paste_data, paste_png, &editpanel_up, &editpanel_hi, &editpanel_dn);
#include "prevlabel_png.h"
	Theme::build_button(prevlabel_data, prevlabel_png, &editpanel_up, &editpanel_hi, &editpanel_dn);
#include "splice_png.h"
	Theme::build_button(splice_data, splice_png, &editpanel_up, &editpanel_hi, &editpanel_dn);
#include "undo_png.h"
	Theme::build_button(undo_data, undo_png, &editpanel_up, &editpanel_hi, &editpanel_dn);
#include "redo_png.h"
	Theme::build_button(redo_data, redo_png, &editpanel_up, &editpanel_hi, &editpanel_dn);
#include "toclip_png.h"
	Theme::build_button(toclip_data, toclip_png, &editpanel_up, &editpanel_hi, &editpanel_dn);
#include "over_png.h"
	Theme::build_button(over_button, over_png, &editpanel_up, &editpanel_hi, &editpanel_dn);
#include "theme2/projector_png.h"
	Theme::build_toggle(proj_data, projector_png, &editpanel_up, &editpanel_hi, &editpanel_checked, &editpanel_dn, &editpanel_checkedhi);
#include "theme2/magnify_png.h"
	Theme::build_toggle(magnify_data, magnify_png, &editpanel_up, &editpanel_hi, &editpanel_checked, &editpanel_dn, &editpanel_checkedhi);
#include "theme2/toolwindow_png.h"
	Theme::build_toggle(tool_data, toolwindow_png, &editpanel_up, &editpanel_hi, &editpanel_checked, &editpanel_dn, &editpanel_checkedhi);
#include "theme2/magnify_png.h"
	Theme::build_button(magnify_button_data, magnify_png, &editpanel_up, &editpanel_hi, &editpanel_dn);



}

void MicroTheme::init_backgrounds()
{
#include "title_bg_png.h"
	plugin_bg_data = new VFrame(title_bg_png);
	title_bg_data = new VFrame(title_bg_png);

}

void MicroTheme::init_buttons()
{
#include "statusbar_cancel_dn_png.h"
#include "statusbar_cancel_up_png.h"
#include "statusbar_cancel_hi_png.h"
	static VFrame *default_cancel_data[] = 
	{
		new VFrame(statusbar_cancel_up_png), new VFrame(statusbar_cancel_hi_png), new VFrame(statusbar_cancel_dn_png)
	};

	statusbar_cancel_data = default_cancel_data;


#include "apply_dn_png.h"
#include "apply_up_png.h"
#include "apply_hi_png.h"
	static VFrame *default_apply_data[] = 
	{
		new VFrame(apply_up_png), new VFrame(apply_hi_png), new VFrame(apply_dn_png)
	};

	apply_data = default_apply_data;

}
