#include "confirmsave.h"
#include "defaults.h"
#include "errorbox.h"
#include "filexml.h"
#include "picon_png.h"
#include "reverb.h"
#include "reverbwindow.h"

#include "vframe.h"

#include <math.h>
#include <string.h>
#include <time.h>


PluginClient* new_plugin(PluginServer *server)
{
	return new Reverb(server);
}



Reverb::Reverb(PluginServer *server)
 : PluginAClient(server)
{
	srand(time(0));
	redo_buffers = 1;       // set to redo buffers before the first render
	dsp_in_length = 0;
	thread = 0;
	ref_channels = 0;
	ref_offsets = 0;
	ref_levels = 0;
	ref_lowpass = 0;
	dsp_in = 0;
	load_defaults();
}

Reverb::~Reverb()
{
//printf("Reverb::~Reverb 1\n");
	if(thread)
	{
// Set result to 0 to indicate a server side close
		thread->window->set_done(0);
		thread->completion.lock();
		delete thread;
	}
//printf("Reverb::~Reverb 1\n");

	save_defaults();
//printf("Reverb::~Reverb 1\n");
	delete defaults;
//printf("Reverb::~Reverb 1\n");
	if(ref_channels) delete ref_channels;
//printf("Reverb::~Reverb 1\n");
	if(ref_offsets) delete ref_offsets;
//printf("Reverb::~Reverb 1\n");
	if(ref_levels) delete ref_levels;
//printf("Reverb::~Reverb 1\n");
	if(ref_lowpass) delete ref_lowpass;
//printf("Reverb::~Reverb 1\n");
	if(dsp_in) delete dsp_in;
//printf("Reverb::~Reverb 2\n");
}

char* Reverb::plugin_title() { return "Heroine College Concert Hall"; }
int Reverb::is_realtime() { return 1; }
int Reverb::is_multichannel() { return 1; }

VFrame* Reverb::new_picon()
{
	return new VFrame(picon_png);
}

int Reverb::start_realtime()
{
	int i;

//printf("Reverb::start_realtime %d\n", total_in_buffers);
	dsp_in = new double*[total_in_buffers];
	ref_channels = new long*[total_in_buffers];
	ref_offsets = new long*[total_in_buffers];
	ref_levels = new double*[total_in_buffers];
	ref_lowpass = new long*[total_in_buffers];
	lowpass_in1 = new double*[total_in_buffers];
	lowpass_in2 = new double*[total_in_buffers];

	for(i = 0; i < total_in_buffers; i++)
	{
		dsp_in[i] = new double[1];
		ref_channels[i] = new long[1];
		ref_offsets[i] = new long[1];
		ref_levels[i] = new double[1];
		ref_lowpass[i] = new long[1];
		lowpass_in1[i] = new double[1];
		lowpass_in2[i] = new double[1];
	}

	engine = new ReverbEngine*[(smp + 1)];
	for(i = 0; i < (smp + 1); i++)
	{
		engine[i] = new ReverbEngine(this);
//printf("Reverb::start_realtime %d\n", Thread::calculate_realtime());
		engine[i]->set_realtime(realtime_priority);
		engine[i]->start();
	}
	redo_buffers = 1;
	return 0;
}

int Reverb::stop_realtime()
{
	for(int i = 0; i < total_in_buffers; i++)
	{
		delete dsp_in[i];
		delete ref_channels[i];
		delete ref_offsets[i];
		delete ref_levels[i];
		delete ref_lowpass[i];
		delete lowpass_in1[i];
		delete lowpass_in2[i];
	}
	delete dsp_in;
	delete ref_channels;
	delete ref_offsets;
	delete ref_levels;
	delete ref_lowpass;
	delete lowpass_in1;
	delete lowpass_in2;
	
	for(int i = 0; i < (smp + 1); i++)
	{
		delete engine[i];
	}
	delete engine;
	return 0;
}

int Reverb::process_realtime(long size, double **input_ptr, double **output_ptr)
{
	long new_dsp_length, i, j;
	main_in = input_ptr;
	main_out = output_ptr;

	load_configuration();

	new_dsp_length = size + 
		(config.delay_init + config.ref_length) * project_sample_rate / 1000;

	if(redo_buffers || new_dsp_length != dsp_in_length)
	{
		for(i = 0; i < total_in_buffers; i++)
		{
			double *old_dsp = dsp_in[i];
			double *new_dsp = new double[new_dsp_length];
			for(j = 0; j < dsp_in_length && j < new_dsp_length; j++) 
				new_dsp[j] = old_dsp[j];
			for( ; j < new_dsp_length; j++) new_dsp[j] = 0;
			delete [] old_dsp;
			dsp_in[i] = new_dsp;
		}

		dsp_in_length = new_dsp_length;
		redo_buffers = 1;
	}

	if(redo_buffers)
	{
		for(i = 0; i < total_in_buffers; i++)
		{
			delete [] ref_channels[i];
			delete [] ref_offsets[i];
			delete [] ref_lowpass[i];
			delete [] ref_levels[i];
			delete [] lowpass_in1[i];
			delete [] lowpass_in2[i];
			
			ref_channels[i] = new long[config.ref_total + 1];
			ref_offsets[i] = new long[config.ref_total + 1];
			ref_lowpass[i] = new long[config.ref_total + 1];
			ref_levels[i] = new double[config.ref_total + 1];
			lowpass_in1[i] = new double[config.ref_total + 1];
			lowpass_in2[i] = new double[config.ref_total + 1];

// set channels			
			ref_channels[i][0] = i;         // primary noise
			ref_channels[i][1] = i;         // first reflection
// set offsets
			ref_offsets[i][0] = 0;
			ref_offsets[i][1] = config.delay_init * project_sample_rate / 1000;
// set levels
			ref_levels[i][0] = db.fromdb(config.level_init);
			ref_levels[i][1] = db.fromdb(config.ref_level1);
// set lowpass
			ref_lowpass[i][0] = -1;     // ignore first noise
			ref_lowpass[i][1] = config.lowpass1;
			lowpass_in1[i][0] = 0;
			lowpass_in2[i][0] = 0;
			lowpass_in1[i][1] = 0;
			lowpass_in2[i][1] = 0;

			long ref_division = config.ref_length * project_sample_rate / 1000 / (config.ref_total + 1);
			for(j = 2; j < config.ref_total + 1; j++)
			{
// set random channels for remaining reflections
				ref_channels[i][j] = rand() % total_in_buffers;

// set random offsets after first reflection
				ref_offsets[i][j] = ref_offsets[i][1];
				ref_offsets[i][j] += ref_division * j - (rand() % ref_division) / 2;

// set changing levels
				ref_levels[i][j] = db.fromdb(config.ref_level1 + (config.ref_level2 - config.ref_level1) / (config.ref_total - 1) * (j - 2));
				//ref_levels[i][j] /= 100;

// set changing lowpass as linear
				ref_lowpass[i][j] = (long)(config.lowpass1 + (double)(config.lowpass2 - config.lowpass1) / (config.ref_total - 1) * (j - 2));
				lowpass_in1[i][j] = 0;
				lowpass_in2[i][j] = 0;
			}
		}
		
		redo_buffers = 0;
	}

	for(i = 0; i < total_in_buffers; )
	{
		for(j = 0; j < (smp + 1) && (i + j) < total_in_buffers; j++)
		{
			engine[j]->process_overlays(i + j, size);
		}
		
		for(j = 0; j < (smp + 1) && i < total_in_buffers; j++, i++)
		{
			engine[j]->wait_process_overlays();
		}
	}
//printf("Reverb::process_realtime 1 %d %d\n", total_in_buffers, size);

	for(i = 0; i < total_in_buffers; i++)
	{
		double *current_out = main_out[i];
		double *current_in = dsp_in[i];

		for(j = 0; j < size; j++) current_out[j] = current_in[j];

		long k;
		for(k = 0; j < dsp_in_length; j++, k++) current_in[k] = current_in[j];
		
		for(; k < dsp_in_length; k++) current_in[k] = 0;
	}
//printf("Reverb::process_realtime 2 %d %d\n", total_in_buffers, size);
	return 0;
}



int Reverb::show_gui()
{
	load_configuration();
	thread = new ReverbThread(this);
	thread->start();
	return 0;
}

int Reverb::set_string()
{
	if(thread) thread->window->set_title(gui_string);
	return 0;
}

void Reverb::raise_window()
{
	if(thread)
	{
		thread->window->raise_window();
		thread->window->flush();
	}
}

int Reverb::load_defaults()
{
	char directory[1024];

// set the default directory
	sprintf(directory, "%sreverb.rc", get_defaultdir());
	
// load the defaults

	defaults = new Defaults(directory);

	defaults->load();

	config.level_init = defaults->get("LEVEL_INIT", 0);
	config.delay_init = defaults->get("DELAY_INIT", 100);
	config.ref_level1 = defaults->get("REF_LEVEL1", -6);
	config.ref_level2 = defaults->get("REF_LEVEL2", INFINITYGAIN);
	config.ref_total = defaults->get("REF_TOTAL", 12);
	config.ref_length = defaults->get("REF_LENGTH", 1000);
	config.lowpass1 = defaults->get("LOWPASS1", 20000);
	config.lowpass2 = defaults->get("LOWPASS2", 20000);
	
	sprintf(config_directory, "~");
	defaults->get("CONFIG_DIRECTORY", config_directory);
	
//printf("Reverb::load_defaults config.ref_level2 %f\n", config.ref_level2);
	return 0;
}

int Reverb::save_defaults()
{
	defaults->update("LEVEL_INIT", config.level_init);
	defaults->update("DELAY_INIT", config.delay_init);
	defaults->update("REF_LEVEL1", config.ref_level1);
	defaults->update("REF_LEVEL2", config.ref_level2);
	defaults->update("REF_TOTAL", config.ref_total);
	defaults->update("REF_LENGTH", config.ref_length);
	defaults->update("LOWPASS1", config.lowpass1);
	defaults->update("LOWPASS2", config.lowpass2);
	defaults->update("CONFIG_DIRECTORY", config_directory);
	defaults->save();
	return 0;
}

void Reverb::load_configuration()
{
	KeyFrame *prev_keyframe, *next_keyframe;
//printf("BlurMain::load_configuration 1\n");

	prev_keyframe = get_prev_keyframe(-1);
	next_keyframe = get_next_keyframe(-1);
//printf("BlurMain::load_configuration %s\n", prev_keyframe->data);
	read_data(prev_keyframe);
}

void Reverb::save_data(KeyFrame *keyframe)
{
//printf("Reverb::save_data 1\n");
	FileXML output;
//printf("Reverb::save_data 1\n");

// cause xml file to store data directly in text
	output.set_shared_string(keyframe->data, MESSAGESIZE);
//printf("Reverb::save_data 1\n");

	output.tag.set_title("REVERB");
	output.tag.set_property("LEVELINIT", config.level_init);
	output.tag.set_property("DELAY_INIT", config.delay_init);
	output.tag.set_property("REF_LEVEL1", config.ref_level1);
	output.tag.set_property("REF_LEVEL2", config.ref_level2);
	output.tag.set_property("REF_TOTAL", config.ref_total);
//printf("Reverb::save_data 1\n");
	output.tag.set_property("REF_LENGTH", config.ref_length);
	output.tag.set_property("LOWPASS1", config.lowpass1);
	output.tag.set_property("LOWPASS2", config.lowpass2);
//printf("Reverb::save_data config.ref_level2 %f\n", config.ref_level2);
	output.append_tag();
	output.append_newline();
//printf("Reverb::save_data 1\n");
	
	
	
	output.terminate_string();
//printf("Reverb::save_data 2\n");
}

void Reverb::read_data(KeyFrame *keyframe)
{
	FileXML input;
// cause xml file to read directly from text
	input.set_shared_string(keyframe->data, strlen(keyframe->data));
	int result = 0;
	ReverbConfig new_config;

	result = input.read_tag();

	if(!result)
	{
		if(input.tag.title_is("REVERB"))
		{
			new_config.level_init = input.tag.get_property("LEVELINIT", new_config.level_init);
			new_config.delay_init = input.tag.get_property("DELAY_INIT", new_config.delay_init);
			new_config.ref_level1 = input.tag.get_property("REF_LEVEL1", new_config.ref_level1);
			new_config.ref_level2 = input.tag.get_property("REF_LEVEL2", new_config.ref_level2);
			new_config.ref_total = input.tag.get_property("REF_TOTAL", new_config.ref_total);
			new_config.ref_length = input.tag.get_property("REF_LENGTH", new_config.ref_length);
			new_config.lowpass1 = input.tag.get_property("LOWPASS1", new_config.lowpass1);
			new_config.lowpass2 = input.tag.get_property("LOWPASS2", new_config.lowpass2);
		}
		
		if(!(new_config == config))
		{
//printf("Reverb::read_data %f\n", new_config.ref_level1);
			config = new_config;
			redo_buffers = 1;
			update_gui();
		}
	}
}

int Reverb::update_gui()
{
	if(thread)
	{
		thread->window->level_init->update(config.level_init);
		thread->window->delay_init->update(config.delay_init);
		thread->window->ref_level1->update(config.ref_level1);
		//thread->window->ref_level2->update(config.ref_level2);
		thread->window->ref_total->update(config.ref_total);
		thread->window->ref_length->update(config.ref_length);
		thread->window->lowpass1->update(config.lowpass1);
		thread->window->lowpass2->update(config.lowpass2);
	}
}




int Reverb::load_from_file(char *path)
{
	FILE *in;
	int result = 0;
	int length;
	char string[1024];
	
	if(in = fopen(path, "rb"))
	{
		fseek(in, 0, SEEK_END);
		length = ftell(in);
		fseek(in, 0, SEEK_SET);
		fread(string, length, 1, in);
		fclose(in);
//		read_data(string);
	}
	else
	{
		perror("fopen:");
// failed
		ErrorBox errorbox("");
		char string[1024];
		sprintf(string, "Couldn't open %s.", path);
		errorbox.create_objects(string);
		errorbox.run_window();
		result = 1;
	}
	
	return result;
}

int Reverb::save_to_file(char *path)
{
	FILE *out;
	int result = 0;
	char string[1024];
	
	{
// 		ConfirmSave confirm;
// 		result = confirm.test_file("", path);
	}
	
	if(!result)
	{
		if(out = fopen(path, "wb"))
		{
//			save_data(string);
			fwrite(string, strlen(string), 1, out);
			fclose(out);
		}
		else
		{
			result = 1;
// failed
			ErrorBox errorbox("");
			char string[1024];
			sprintf(string, "Couldn't save %s.", path);
			errorbox.create_objects(string);
			errorbox.run_window();
			result = 1;
		}
	}
	
	return result;
}

ReverbEngine::ReverbEngine(Reverb *plugin)
 : Thread()
{
	this->plugin = plugin;
	completed = 0;
	input_lock.lock();
	output_lock.lock();
}

ReverbEngine::~ReverbEngine()
{
	completed = 1;
	input_lock.unlock();
	join();
}

int ReverbEngine::process_overlays(int output_buffer, long size)
{
	this->output_buffer = output_buffer;
	this->size = size;
	input_lock.unlock();
}

int ReverbEngine::wait_process_overlays()
{
	output_lock.lock();
}
	
int ReverbEngine::process_overlay(double *in, double *out, double &out1, double &out2, double level, long lowpass, long samplerate, long size)
{
	if(lowpass == -1 || lowpass >= plugin->project_sample_rate / 2 - 2500)
	{
// no lowpass filter
		for(register int i = 0; i < size; i++) out[i] += in[i] * level;
	}
	else
	{
		register double coef = 0.25 * 2.0 * M_PI * (double)lowpass / (double)plugin->project_sample_rate;
		register double a = coef * 0.25;
		register double b = coef * 0.50;

		for(register int i = 0; i < size; i++)
		{
			out2 += a * (3 * out1 + in[i] - out2);
			out2 += b * (out1 + in[i] - out2);
			out2 += a * (out1 + 3 * in[i] - out2);
			out2 += coef * (in[i] - out2);
			out1 = in[i];
			out[i] += out2 * level;
		}
	}
}

void ReverbEngine::run()
{
	int j, i;
	while(1)
	{
		input_lock.lock();
		if(completed) return;

// Process reverb
		for(i = 0; i < plugin->total_in_buffers; i++)
		{
			for(j = 0; j < plugin->config.ref_total + 1; j++)
			{
				if(plugin->ref_channels[i][j] == output_buffer)
					process_overlay(plugin->main_in[i], 
								&(plugin->dsp_in[plugin->ref_channels[i][j]][plugin->ref_offsets[i][j]]), 
								plugin->lowpass_in1[i][j], 
								plugin->lowpass_in2[i][j], 
								plugin->ref_levels[i][j], 
								plugin->ref_lowpass[i][j], 
								plugin->project_sample_rate, 
								size);
			}
		}

		output_lock.unlock();
	}
}







ReverbConfig::ReverbConfig()
{
}

int ReverbConfig::operator==(ReverbConfig& that)
{
	return(level_init == that.level_init &&
		delay_init == that.delay_init &&
		ref_level1 == that.ref_level1 &&
		ref_level2 == that.ref_level2 &&
		ref_total == that.ref_total &&
		ref_length == that.ref_length &&
		lowpass1 == that.lowpass1 &&
		lowpass2 == that.lowpass2);
}

ReverbConfig& ReverbConfig::operator=(ReverbConfig& that)
{
	level_init = that.level_init;
	delay_init = that.delay_init;
	ref_level1 = that.ref_level1;
	ref_level2 = that.ref_level2;
	ref_total = that.ref_total;
	ref_length = that.ref_length;
	lowpass1 = that.lowpass1;
	lowpass2 = that.lowpass2;
	return *this;
}
