#ifndef TITLE_H
#define TITLE_H

class TitleMain;

#include "defaults.h"
#include "mutex.h"
#include "pluginvclient.h"
#include "titlewindow.h"

#include <sys/types.h>

// Style bitwise ORed
#define FONT_ITALIC    0x1
#define FONT_BOLD      0x2

// Motion strategy
#define TOTAL_PATHS 3
#define NO_MOTION     0x0
#define BOTTOM_TO_TOP 0x1
#define RIGHT_TO_LEFT 0x2

// Horizontal justification
#define JUSTIFY_LEFT   0x0
#define JUSTIFY_CENTER 0x1
#define JUSTIFY_RIGHT  0x2


class TitleConfig
{
public:
	TitleConfig();

// Font information
	char font[BCTEXTLEN];
	long style;
	int size;
	int color;
// Motion of title across frame
	int motion_strategy;
	int justification;
// Number of seconds the fade in and fade out of the title take
	double fade_in, fade_out;
// Position in frame
	int x, y;

// Text to display
	char text[BCTEXTLEN];
};

typedef struct 
{
	char path[BCTEXTLEN];
	char title[BCTEXTLEN];
} font_table;

class TitleMain : public PluginVClient
{
public:
	TitleMain(PluginServer *server);
	~TitleMain();

// required for all realtime plugins
	int process_realtime(VFrame *input_ptr, VFrame *output_ptr);
	int is_realtime();
	char* plugin_title();
	int start_realtime();
	int stop_realtime();
	int show_gui();
	void raise_window();
	int set_string();
	void load_configuration();
	void save_data(KeyFrame *keyframe);
	void read_data(KeyFrame *keyframe);
	int load_defaults();
	int save_defaults();
	VFrame* new_picon();

	static char* motion_to_text(int motion);
	static int text_to_motion(char *text);
// a thread for the GUI
	TitleThread *thread;
// Current configuration
	TitleConfig config;
// Size of window
	int window_w, window_h;
	ArrayList<font_table*> fonts;
	Defaults *defaults;
};


#endif
