#include "bcdisplayinfo.h"
#include "titlewindow.h"

#include <string.h>

TitleThread::TitleThread(TitleMain *client)
 : Thread()
{
	this->client = client;
	set_synchronous(0);
	gui_started.lock();
	completion.lock();
}

TitleThread::~TitleThread()
{
// Window always deleted here
	delete window;
}
	
void TitleThread::run()
{
	BC_DisplayInfo info;
	window = new TitleWindow(client, 
		info.get_abs_cursor_x() - client->window_w / 2, 
		info.get_abs_cursor_y() - client->window_h / 2);
	window->create_objects();
	gui_started.unlock();
	int result = window->run_window();
	completion.unlock();
// Last command executed in thread
	if(result) client->client_side_close();
}









TitleWindow::TitleWindow(TitleMain *client, int x, int y)
 : BC_Window(client->gui_string, 
	x,
	y,
	client->window_w, 
	client->window_h, 
	client->window_w, 
	client->window_h, 
	0, 
	0,
	1)
{ 
	this->client = client; 
}

TitleWindow::~TitleWindow()
{
	sizes.remove_all_objects();
	delete color_thread;
	delete title_x;
	delete title_y;
}

int TitleWindow::create_objects()
{
	int x = 10, y = 10;

	sizes.append(new BC_ListBoxItem("8"));
	sizes.append(new BC_ListBoxItem("9"));
	sizes.append(new BC_ListBoxItem("10"));
	sizes.append(new BC_ListBoxItem("11"));
	sizes.append(new BC_ListBoxItem("12"));
	sizes.append(new BC_ListBoxItem("13"));
	sizes.append(new BC_ListBoxItem("14"));
	sizes.append(new BC_ListBoxItem("16"));
	sizes.append(new BC_ListBoxItem("18"));
	sizes.append(new BC_ListBoxItem("20"));
	sizes.append(new BC_ListBoxItem("22"));
	sizes.append(new BC_ListBoxItem("24"));
	sizes.append(new BC_ListBoxItem("26"));
	sizes.append(new BC_ListBoxItem("28"));
	sizes.append(new BC_ListBoxItem("32"));
	sizes.append(new BC_ListBoxItem("36"));
	sizes.append(new BC_ListBoxItem("40"));
	sizes.append(new BC_ListBoxItem("48"));
	sizes.append(new BC_ListBoxItem("56"));
	sizes.append(new BC_ListBoxItem("64"));
	sizes.append(new BC_ListBoxItem("72"));

	paths.append(new BC_ListBoxItem(TitleMain::motion_to_text(NO_MOTION)));
	paths.append(new BC_ListBoxItem(TitleMain::motion_to_text(BOTTOM_TO_TOP)));
	paths.append(new BC_ListBoxItem(TitleMain::motion_to_text(RIGHT_TO_LEFT)));

	for(int i = 0; i < client->fonts.total; i++)
		fonts.append(new BC_ListBoxItem(client->fonts.values[i]->title));

	add_tool(new BC_Title(x, y, "Font:"));
	font = new TitleFont(client, this, x, y + 20);
	font->create_objects();
	x += 230;

	add_tool(new BC_Title(x, y, "Size:"));
	char string[BCTEXTLEN];
	sprintf(string, "%d", client->config.size);
	size = new TitleSize(client, this, x, y + 20, string);
	size->create_objects();
	x += 150;

	add_tool(new BC_Title(x, y, "Style:"));
	add_tool(italic = new TitleItalic(client, this, x, y + 20));
	add_tool(bold = new TitleBold(client, this, x, y + 50));
	x += 100;

	add_tool(new BC_Title(x, y, "Justify:"));
	add_tool(left = new TitleLeft(client, this, x, y + 20));
	add_tool(center = new TitleCenter(client, this, x, y + 50));
	add_tool(right = new TitleRight(client, this, x, y + 80));
	


	y += 50;
	x = 10;

	add_tool(new BC_Title(x, y, "X:"));
	title_x = new TitleX(client, this, x, y + 20);
	title_x->create_objects();
	x += 100;

	add_tool(new BC_Title(x, y, "Y:"));
	title_y = new TitleY(client, this, x, y + 20);
	title_y->create_objects();
	x += 100;

	add_tool(new BC_Title(x, y, "Motion type:"));
	motion = new TitleMotion(client, this, x, y + 20);
	motion->create_objects();
	x += 150;
	
	x = 10;
	y += 50;

	add_tool(new BC_Title(x, y, "Fade in (sec):"));
	add_tool(fade_in = new TitleFade(client, this, &client->config.fade_in, x, y + 20));
	x += 100;
	
	add_tool(new BC_Title(x, y, "Fade out (sec):"));
	add_tool(fade_out = new TitleFade(client, this, &client->config.fade_out, x, y + 20));
	x += 100;

	add_tool(color_button = new TitleColorButton(client, this, x, y + 20));
	x += 80;
	color_x = x;
	color_y = y + 20;
	color_thread = new TitleColorThread(client, this);

	x = 10;
	y += 50;

	add_tool(new BC_Title(x, y, "Text:"));
	add_tool(text = new TitleText(client, 
		this, 
		x, 
		y + 20, 
		get_w() - x - 10, 
		get_h() - y - 10));

	update_color();

	show_window();
	flush();
	return 0;
}

int TitleWindow::close_event()
{
// Set result to 1 to indicate a client side close
	set_done(1);
	return 1;
}

void TitleWindow::update_color()
{
printf("TitleWindow::update_color %x\n", client->config.color);
	set_color(client->config.color);
	draw_box(color_x, color_y, 100, 30);
	flash(color_x, color_y, 100, 30);
}

void TitleWindow::update_justification()
{
	left->update(client->config.justification == JUSTIFY_LEFT);
	center->update(client->config.justification == JUSTIFY_CENTER);
	right->update(client->config.justification == JUSTIFY_RIGHT);
}

void TitleWindow::update()
{
	title_x->update((long)client->config.x);
	title_y->update((long)client->config.y);
	italic->update(client->config.style & FONT_ITALIC);
	bold->update(client->config.style & FONT_BOLD);
	size->update(client->config.size);
	motion->update(TitleMain::motion_to_text(client->config.motion_strategy));
	fade_in->update((float)client->config.fade_in);
	fade_out->update((float)client->config.fade_out);
	font->update(client->config.font);
	text->update(client->config.text);
	update_justification();
	update_color();
}

int TitleWindow::resize_event(int w, int h)
{
	return 1;
}


TitleBold::TitleBold(TitleMain *client, TitleWindow *window, int x, int y)
 : BC_CheckBox(x, y, client->config.style & FONT_BOLD, "Bold")
{
	this->client = client;
	this->window = window;
}

int TitleBold::handle_event()
{
	client->config.style = (client->config.style & ~FONT_BOLD) | (get_value() ? FONT_BOLD : 0);
	client->send_configure_change();
	return 1;
}

TitleItalic::TitleItalic(TitleMain *client, TitleWindow *window, int x, int y)
 : BC_CheckBox(x, y, client->config.style & FONT_ITALIC, "Italic")
{
	this->client = client;
	this->window = window;
}
int TitleItalic::handle_event()
{
	client->config.style = (client->config.style & ~FONT_ITALIC) | (get_value() ? FONT_ITALIC : 0);
	client->send_configure_change();
	return 1;
}

TitleSize::TitleSize(TitleMain *client, TitleWindow *window, int x, int y, char *text)
 : BC_PopupTextBox(window, 
		&window->sizes,
		text,
		x, 
		y, 
		100,
		300)
{
	this->client = client;
	this->window = window;
}
TitleSize::~TitleSize()
{
}
int TitleSize::handle_event()
{
	client->config.size = atol(get_text());
	client->send_configure_change();
	return 1;
}
void TitleSize::update(int size)
{
	char string[BCTEXTLEN];
	sprintf(string, "%d", size);
	BC_PopupTextBox::update(string);
}

TitleColorButton::TitleColorButton(TitleMain *client, TitleWindow *window, int x, int y)
 : BC_GenericButton(x, y, "Color...")
{
	this->client = client;
	this->window = window;
}
int TitleColorButton::handle_event()
{
	window->color_thread->start_window(client->config.color);
	return 1;
}

TitleMotion::TitleMotion(TitleMain *client, TitleWindow *window, int x, int y)
 : BC_PopupTextBox(window, 
		&window->paths,
		client->motion_to_text(client->config.motion_strategy),
		x, 
		y, 
		150,
		100)
{
	this->client = client;
	this->window = window;
}
int TitleMotion::handle_event()
{
	client->send_configure_change();
	return 1;
}

TitleFade::TitleFade(TitleMain *client, TitleWindow *window, double *value, int x, int y)
 : BC_TextBox(x, y, 90, 1, (float)*value)
{
	this->client = client;
	this->window = window;
	this->value = value;
}

int TitleFade::handle_event()
{
	*value = atof(get_text());
	client->send_configure_change();
	return 1;
}

TitleFont::TitleFont(TitleMain *client, TitleWindow *window, int x, int y)
 : BC_PopupTextBox(window, 
		&window->fonts,
		client->config.font,
		x, 
		y, 
		200,
		300)
{
	this->client = client;
	this->window = window;
}
int TitleFont::handle_event()
{
	strcpy(client->config.font, get_text());
	client->send_configure_change();
	return 1;
}

TitleText::TitleText(TitleMain *client, TitleWindow *window, int x, int y, int w, int h)
 : BC_TextBox(x, 
 	y, 
	window->get_w() - 20, 
	(window->get_h() - y - 10) / window->get_text_height(MEDIUMFONT) - 2, 
	client->config.text)
{
	this->client = client;
	this->window = window;
//printf("TitleText::TitleText %s\n", client->config.text);
}

int TitleText::handle_event()
{
	strcpy(client->config.text, get_text());
	client->send_configure_change();
	return 1;
}


TitleX::TitleX(TitleMain *client, TitleWindow *window, int x, int y)
 : BC_TumbleTextBox(window,
 	(long)client->config.x,
	(long)0,
	(long)2000,
	x, 
	y, 
	70)
{
	this->client = client;
	this->window = window;
}
int TitleX::handle_event()
{
	client->config.x = atol(get_text());
	client->send_configure_change();
	return 1;
}

TitleY::TitleY(TitleMain *client, TitleWindow *window, int x, int y)
 : BC_TumbleTextBox(window,
 	(long)client->config.y, 
	(long)0,
	(long)2000,
	x, 
	y, 
	70)
{
	this->client = client;
	this->window = window;
}
int TitleY::handle_event()
{
	client->config.y = atol(get_text());
	client->send_configure_change();
	return 1;
}

TitleLeft::TitleLeft(TitleMain *client, TitleWindow *window, int x, int y)
 : BC_Radial(x, y, client->config.justification == JUSTIFY_LEFT, "Left")
{
	this->client = client;
	this->window = window;
}
int TitleLeft::handle_event()
{
	client->config.justification = JUSTIFY_LEFT;
	window->update_justification();
	client->send_configure_change();
	return 1;
}

TitleCenter::TitleCenter(TitleMain *client, TitleWindow *window, int x, int y)
 : BC_Radial(x, y, client->config.justification == JUSTIFY_CENTER, "Center")
{
	this->client = client;
	this->window = window;
}
int TitleCenter::handle_event()
{
	client->config.justification = JUSTIFY_CENTER;
	window->update_justification();
	client->send_configure_change();
	return 1;
}

TitleRight::TitleRight(TitleMain *client, TitleWindow *window, int x, int y)
 : BC_Radial(x, y, client->config.justification == JUSTIFY_RIGHT, "Right")
{
	this->client = client;
	this->window = window;
}
int TitleRight::handle_event()
{
	client->config.justification = JUSTIFY_RIGHT;
	window->update_justification();
	client->send_configure_change();
	return 1;
}

TitleColorThread::TitleColorThread(TitleMain *client, TitleWindow *window)
 : ColorThread()
{
	this->client = client;
	this->window = window;
}

int TitleColorThread::handle_event(int output)
{
	client->config.color = output;
	window->update_color();
	window->flush();
	client->send_configure_change();
	return 1;
}
