#ifndef TITLEWINDOW_H
#define TITLEWINDOW_H

#include "guicast.h"

class TitleThread;
class TitleWindow;
class TitleInterlace;

#include "../colors/colorpicker.h"
#include "filexml.h"
#include "mutex.h"
#include "title.h"

class TitleThread : public Thread
{
public:
	TitleThread(TitleMain *client);
	~TitleThread();

	void run();

// prevent loading data until the GUI is started
 	Mutex gui_started, completion;
	TitleMain *client;
	TitleWindow *window;
};




class TitleItalic;
class TitleBold;
class TitleSize;
class TitleColorButton;
class TitleMotion;
class TitleFade;
class TitleFont;
class TitleText;
class TitleX;
class TitleY;
class TitleLeft;
class TitleCenter;
class TitleRight;
class TitleColorThread;

class TitleWindow : public BC_Window
{
public:
	TitleWindow(TitleMain *client, int x, int y);
	~TitleWindow();
	
	int create_objects();
	int close_event();
	int resize_event(int w, int h);
	void update_color();
	void update_justification();
	void update();

	TitleMain *client;
	TitleX *title_x;
	TitleY *title_y;
	TitleItalic *italic;
	TitleBold *bold;
	TitleSize *size;
	TitleColorButton *color_button;
	TitleColorThread *color_thread;
	TitleMotion *motion;
	TitleFade *fade_in;
	TitleFade *fade_out;
	TitleFont *font;
	TitleText *text;
	TitleLeft *left;
	TitleCenter *center;
	TitleRight *right;
// Color preview
	int color_x, color_y;
	ArrayList<BC_ListBoxItem*> sizes;
	ArrayList<BC_ListBoxItem*> paths;
	ArrayList<BC_ListBoxItem*> fonts;
};

class TitleItalic : public BC_CheckBox
{
public:
	TitleItalic(TitleMain *client, TitleWindow *window, int x, int y);
	int handle_event();
	TitleMain *client;
	TitleWindow *window;
};
class TitleBold : public BC_CheckBox
{
public:
	TitleBold(TitleMain *client, TitleWindow *window, int x, int y);
	int handle_event();
	TitleMain *client;
	TitleWindow *window;
};
class TitleSize : public BC_PopupTextBox
{
public:
	TitleSize(TitleMain *client, TitleWindow *window, int x, int y, char *text);
	~TitleSize();
	int handle_event();
	void update(int size);
	TitleMain *client;
	TitleWindow *window;
};
class TitleColorButton : public BC_GenericButton
{
public:
	TitleColorButton(TitleMain *client, TitleWindow *window, int x, int y);
	int handle_event();
	TitleMain *client;
	TitleWindow *window;
};
class TitleMotion : public BC_PopupTextBox
{
public:
	TitleMotion(TitleMain *client, TitleWindow *window, int x, int y);
	int handle_event();
	TitleMain *client;
	TitleWindow *window;
};
class TitleFade : public BC_TextBox
{
public:
	TitleFade(TitleMain *client, TitleWindow *window, double *value, int x, int y);
	int handle_event();
	TitleMain *client;
	TitleWindow *window;
	double *value;
};
class TitleFont : public BC_PopupTextBox
{
public:
	TitleFont(TitleMain *client, TitleWindow *window, int x, int y);
	int handle_event();
	TitleMain *client;
	TitleWindow *window;
};
class TitleText : public BC_TextBox
{
public:
	TitleText(TitleMain *client, TitleWindow *window, int x, int y, int w, int h);
	int handle_event();
	TitleMain *client;
	TitleWindow *window;
};
class TitleX : public BC_TumbleTextBox
{
public:
	TitleX(TitleMain *client, TitleWindow *window, int x, int y);
	int handle_event();
	TitleMain *client;
	TitleWindow *window;
};
class TitleY : public BC_TumbleTextBox
{
public:
	TitleY(TitleMain *client, TitleWindow *window, int x, int y);
	int handle_event();
	TitleMain *client;
	TitleWindow *window;
};
class TitleLeft : public BC_Radial
{
public:
	TitleLeft(TitleMain *client, TitleWindow *window, int x, int y);
	int handle_event();
	TitleMain *client;
	TitleWindow *window;
};
class TitleCenter : public BC_Radial
{
public:
	TitleCenter(TitleMain *client, TitleWindow *window, int x, int y);
	int handle_event();
	TitleMain *client;
	TitleWindow *window;
};
class TitleRight : public BC_Radial
{
public:
	TitleRight(TitleMain *client, TitleWindow *window, int x, int y);
	int handle_event();
	TitleMain *client;
	TitleWindow *window;
};

class TitleColorThread : public ColorThread
{
public:
	TitleColorThread(TitleMain *client, TitleWindow *window);
	int handle_event(int output);
	TitleMain *client;
	TitleWindow *window;
};

#endif
