#include "mwindow.inc"
#include "position.h"

#include <GL/glut.h>
#include <math.h>
#include <stdio.h>

Position::Position()
{
	pos[0] = 0;
	pos[1] = 0;
	pos[2] = 0;
	dir[0] = 0;
	dir[1] = 0;
	dir[2] = 1;
	rot[0] = 0;
	rot[1] = 0;
	rot[2] = 0;
	phi_sign = 1;
}

Position::~Position()
{
}

// MATH PROBLEMS

// Normalize to magnitude of 1
void Position::normalize(double *coords)
{
	cart_to_sphere(coords);
	coords[0] = 1;
	sphere_to_cart(coords);
}

void Position::cart_to_sphere(double *coords)
{
	double x = coords[0], y = coords[1], z = coords[2];
	double rho, phi, theta, S;
	rho = sqrt(x * x + y * y + z * z);
	S = sqrt(x * x + y * y);
	phi = rho ? acos(z / rho) : 0;
	theta = S ? asin(y / S) : 0;

	if(x < 0) theta = M_PI - theta;

	coords[0] = rho;
	coords[1] = phi;
	coords[2] = theta;
}

void Position::sphere_to_cart(double *coords)
{
	double rho = coords[0], phi = coords[1], theta = coords[2];
	double x, y, z;

	x = rho * sin(phi) * cos(theta);
	y = rho * sin(phi) * sin(theta);
	z = rho * cos(phi);
	coords[0] = x;
	coords[1] = y;
	coords[2] = z;
}

double Position::dot_product(double *a, double *b)
{
	return a[0] * b[0] + a[1] * b[1] + a[2] * b[2];
}

void Position::cross_product(double *c, double *a, double *b)
{
	c[0] = a[1] * b[2] - a[2] * b[1];
	c[1] = a[2] * b[0] - a[0] * b[2];
	c[2] = a[0] * b[1] - a[1] * b[0];
}

void Position::update_phi_sign(double &phi_sign, double phi)
{
	if(phi > M_PI || phi < 0)
	{
		phi_sign *= -1;
	}
}


Position& Position::operator=(Position& that)
{
	pos[0] = that.pos[0];
	pos[1] = that.pos[1];
	pos[2] = that.pos[2];
	dir[0] = that.dir[0];
	dir[1] = that.dir[1];
	dir[2] = that.dir[2];
	phi_sign = that.phi_sign;
}

Position& Position::operator+=(Position& that)
{
	pos[0] += that.pos[0];
	pos[1] += that.pos[1];
	pos[2] += that.pos[2];
	dir[0] = that.dir[0];
	dir[1] = that.dir[1];
	dir[2] = that.dir[2];
	phi_sign = that.phi_sign;
}

// Translate and rotate the working matrix
void Position::translate_rotate()
{
	double axis[3], angle;
	static double up[3] = { 0, 0, 1 };
    glTranslatef(pos[0], pos[1], pos[2]);
	cross_product(axis, dir, up);
	angle = acos(dot_product(dir, up)) / 2 / M_PI * 360;
	glRotatef(angle, axis[0], axis[1], axis[2]);
}

void Position::trackball_action(int which_button, int last_x, int last_y, int x, int y)
{
	cart_to_sphere(dir);
	if(which_button == GLUT_LEFT_BUTTON)
	{
		dir[2] -= ANG_SCALE * (last_x - x);
		dir[1] = dir[1] + phi_sign * ANG_SCALE * (last_y - y);
		
		update_phi_sign(phi_sign, dir[1]);
	}
	else
	{
		pos[1] -= SCALE * (last_y - y);
	}
//printf("%f %f %f\n", dir[0], dir[1], dir[2]);

	sphere_to_cart(dir);
}

void Position::translation_action(int which_button, int last_x, int last_y, int x, int y)
{
	pos[0] -= SCALE * (last_x - x);
	if(which_button == GLUT_LEFT_BUTTON)
		pos[2] += SCALE * (last_y - y);
	else
		pos[1] += SCALE * (last_y - y);
}

void Position::dump()
{
	printf("Position::dump %f %f %f %f %f %f\n", pos[0], pos[1], pos[2], dir[0], dir[1], dir[2]);
}
