#include "shuttlebooster.h"

#include "mwindow.h"
#include "texture.h"

#include <stdio.h>

#define RADIUS (3.71856 / 2 * SCALE)
#define CONE_H (6.096 * SCALE)
#define TUBE_H (39.624 * SCALE)

#define FLAME_W 8.0 * SCALE
#define FLAME_H 32.0 * SCALE

ShuttleBooster::ShuttleBooster(MWindow *mwindow)
 : Part(mwindow)
{
	sprintf(title, "Shuttle booster");
	max_thrust = 14679131;  // N
	fuel_qty = 498951.607;    // kg
	dry_weight = 87089.73504;    // kg
	nonstop = 1;

	GLUquadricObj *q = gluNewQuadric();
	CylinderObj = glGenLists(1);
	glNewList(CylinderObj, GL_COMPILE);
/* cylinder */
	gluQuadricNormals(q, GL_SMOOTH);
	gluQuadricTexture(q, GL_TRUE);
	gluCylinder(q, RADIUS, RADIUS, TUBE_H, 24, 1);

/* end cap */
// 	glTranslatef(0.0, 0.0, 2.0);
 	gluDisk(q, 0.0, RADIUS, 24, 1);
// 
// /* other end cap */
// 	glTranslatef(0.0, 0.0, -2.0);
// 	gluQuadricOrientation(q, GLU_INSIDE);
// 	gluDisk(q, 0.0, RADIUS, 24, 1);

	glEndList();
	gluDeleteQuadric(q);
}

Part* ShuttleBooster::make_copy()
{
	ShuttleBooster  *result = new ShuttleBooster(mwindow);
	*result = *this;
	return result;
}

void ShuttleBooster::draw()
{
	mwindow->set_color(highlighted ? 0xFFFF00 : 0xFFFFFF);
	
    glPushMatrix();
	glutSolidCone(RADIUS, CONE_H, 24, 1);
    glTranslatef(0, 0, -TUBE_H);
	
	glCallList(CylinderObj);

// Draw flame
	if(throttle > 0)
	{
	    glPushMatrix();
    	glEnable(GL_TEXTURE_2D);
		glTranslatef(0.0, 0.0, -FLAME_H);
		if(flame_position)
			glRotatef(180, 0.0, 0.0, 1.0 );
		glBindTexture(GL_TEXTURE_2D, mwindow->flame1->texName);

		glBegin(GL_QUADS);
		glTexCoord2f(0.0, 0.0);   glVertex3f(0, -FLAME_W / 2.0, 0);
		glTexCoord2f(1.0, 0.0);   glVertex3f(0,  FLAME_W / 2.0, 0);
		glTexCoord2f(1.0, 1.0);   glVertex3f(0,  FLAME_W / 2.0, FLAME_H);
		glTexCoord2f(0.0, 1.0);   glVertex3f(0, -FLAME_W / 2.0, FLAME_H);
		glEnd();
		glRotatef(90, 0.0, 0.0, 1.0 );
		glBegin(GL_QUADS);
		glTexCoord2f(0.0, 0.0);   glVertex3f(0, -FLAME_W / 2.0, 0);
		glTexCoord2f(1.0, 0.0);   glVertex3f(0,  FLAME_W / 2.0, 0);
		glTexCoord2f(1.0, 1.0);   glVertex3f(0,  FLAME_W / 2.0, FLAME_H);
		glTexCoord2f(0.0, 1.0);   glVertex3f(0, -FLAME_W / 2.0, FLAME_H);
		glEnd();

    	glDisable(GL_TEXTURE_2D);
		glPopMatrix();
//		flame_position = !flame_position;
	}

    glPopMatrix();
}
