#include "buildwindow.h"
#include "launchwindow.h"
#include "mwindow.h"
#include "part.h"
#include "partwindow.h"
#include "position.h"
#include "rocket.h"

#include <GL/glut.h>
#include <math.h>
#include <stdio.h>

BuildWindow::BuildWindow(MWindow *mwindow)
 : SubWindow(mwindow)
{
}

BuildWindow::~BuildWindow()
{
	
}

void BuildWindow::draw_overlays()
{
	int y = HELP_MARGIN;
	char string[1024];

	draw_mode("Construction");
}

void BuildWindow::draw_help()
{
	int y = mwindow->height - HELP_MARGIN - mwindow->text_height() * 2;
	mwindow->set_font(GLUT_BITMAP_HELVETICA_12);

	mwindow->draw_text(LEFT_HELP_MARGIN, y, "p -> add new part");
	y -= mwindow->text_height();
	mwindow->draw_text(LEFT_HELP_MARGIN, y, "q -> quit");
	y -= mwindow->text_height();
	mwindow->draw_text(LEFT_HELP_MARGIN, y, "? -> toggle help");
	y -= mwindow->text_height();
	
	if(mwindow->new_part)
	{
		mwindow->draw_text(LEFT_HELP_MARGIN, y, "! -> make part perpendicular to Earth");
		y -= mwindow->text_height();
		mwindow->draw_text(LEFT_HELP_MARGIN, y, "enter -> deselect part");
		y -= mwindow->text_height();
	}
	else
	{
		mwindow->draw_text(LEFT_HELP_MARGIN, y, "! -> make rocket perpendicular to Earth");
		y -= mwindow->text_height();
		mwindow->draw_text(LEFT_HELP_MARGIN, y, "space -> enter launch mode");
		y -= mwindow->text_height();
	}
}

void BuildWindow::cursor_motion(int x, int y)
{
	Part *new_part = mwindow->new_part;
// Drag part
	if(new_part)
	{
		if(mwindow->modifiers & GLUT_ACTIVE_SHIFT)
		{
			mwindow->new_part->position->trackball_action(mwindow->which_button,
				mwindow->last_x,
				mwindow->last_y,
				x, 
				y);
		}
		else
		{
			mwindow->new_part->position->translation_action(mwindow->which_button,
				mwindow->last_x,
				mwindow->last_y,
				x, 
				y);
		}
		glutPostRedisplay();
	}
	else
// Rotate part
	{
		if(mwindow->modifiers & GLUT_ACTIVE_SHIFT)
		{
			mwindow->rocket->position->trackball_action(mwindow->which_button,
				mwindow->last_x,
				mwindow->last_y,
				x, 
				y);
		}
		else
		{
			mwindow->rocket->position->translation_action(mwindow->which_button,
				mwindow->last_x,
				mwindow->last_y,
				x, 
				y);
		}
		glutPostRedisplay();
	}
}


void BuildWindow::deselect_part()
{
	if(mwindow->new_part)
	{
		Rocket *rocket = mwindow->rocket;
		Part *new_part = mwindow->new_part;
		mwindow->new_part = 0;
		if(mwindow->rocket->parts.total == 1)
		{
			*rocket->position += *new_part->position;
			new_part->position->pos[0] = 0;
			new_part->position->pos[1] = 0;
			new_part->position->pos[2] = 0;
			new_part->position->dir[0] = 0;
			new_part->position->dir[1] = 0;
			new_part->position->dir[2] = 1;
		}
		new_part->highlighted = 0;
		glutPostRedisplay();
	}
}

void BuildWindow::keypress(unsigned char k)
{
	int redraw = 0;
	switch(k)
	{
		case 'q':
		case 0x1b:
    		exit(0);
    		break;
		
		case 'p':
// Select part
			deselect_part();
			mwindow->mode = MODE_NEWPART;
			redraw = 1;
			mwindow->current_window = mwindow->pwindow;
			break;

// Switch to launch mode
		case ' ':
			deselect_part();
			mwindow->mode = MODE_LAUNCH;
			redraw = 1;
			*mwindow->rocket2 = *mwindow->rocket;
			mwindow->current_window = mwindow->lwindow;
			mwindow->lwindow->reset_physics();
			glutIdleFunc(MWindow::draw_event);
			break;

// Make perpendicular
		case '!':
		{
			Part *new_part = mwindow->new_part;
			Rocket *rocket = mwindow->rocket;
			if(new_part)
			{
				new_part->position->dir[0] = rocket->position->dir[0];
				new_part->position->dir[1] = rocket->position->dir[1];
				new_part->position->dir[2] = rocket->position->dir[2];
			}
			else
			{
				rocket->position->dir[0] = 0;
				rocket->position->dir[1] = 0;
				rocket->position->dir[2] = 1;
			}
			redraw = 1;
			break;
		}

		case 0xd:
// Place part
			deselect_part();
			break;

		case 'h':
		case '?':
			mwindow->show_help = !mwindow->show_help;
			redraw = 1;
			break;
	}
	if(redraw) glutPostRedisplay();
}

void BuildWindow::keypress2(int k)
{
//printf("BuildWindow::keypress2 %x\n", k);
	int redraw = 0;
// Move camera
// Change direction

#define SPEED 5

	if(mwindow->modifiers & GLUT_ACTIVE_SHIFT)
	{
		switch(k)
		{
			case GLUT_KEY_UP:
				mwindow->position->pos[1] += 1 * SPEED * SCALE;
				redraw = 1;
				break;
			case GLUT_KEY_DOWN:
				mwindow->position->pos[1] -= 1 * SPEED * SCALE;
				redraw = 1;
				break;
			case GLUT_KEY_LEFT:
				mwindow->position->pos[0] -= 1 * SPEED * SCALE;
				redraw = 1;
				break;
			case GLUT_KEY_RIGHT:
				mwindow->position->pos[0] += 1 * SPEED * SCALE;
				redraw = 1;
				break;
		}
	}
	else
	{
		Position::cart_to_sphere(mwindow->position->dir);

		switch(k)
		{
			case GLUT_KEY_DOWN:
				mwindow->position->dir[1] = mwindow->position->dir[1] + mwindow->position->phi_sign * SPEED * ANG_SCALE;
				Position::update_phi_sign(mwindow->position->phi_sign, mwindow->position->dir[1]);
				redraw = 1;
				break;
			case GLUT_KEY_UP:
				mwindow->position->dir[1] = mwindow->position->dir[1] - mwindow->position->phi_sign * SPEED * ANG_SCALE;
				Position::update_phi_sign(mwindow->position->phi_sign, mwindow->position->dir[1]);
				redraw = 1;
				break;
			case GLUT_KEY_LEFT:
				mwindow->position->dir[2] -= SPEED * ANG_SCALE;
				redraw = 1;
				break;
			case GLUT_KEY_RIGHT:
				mwindow->position->dir[2] += SPEED * ANG_SCALE;
				redraw = 1;
				break;
		}

//printf("rho %f phi %f theta %f\n", mwindow->dir[0], mwindow->sdir[1], mwindow->dir[2]);
//printf("%f %f %f\n", TODEG(dir[0]), TODEG(dir[1]), TODEG(dir[2]));
		Position::sphere_to_cart(mwindow->position->dir);
	}

// Forward and backward along direction vector
// Normalize direction vector
	double direction[3];
	direction[0] = mwindow->position->dir[0];
	direction[1] = mwindow->position->dir[1];
	direction[2] = mwindow->position->dir[2];

	Position::normalize(direction);
	switch(k)
	{
		case GLUT_KEY_PAGE_UP:
			mwindow->position->pos[0] += direction[0] * SPEED * SCALE;
			mwindow->position->pos[1] += direction[1] * SPEED * SCALE;
			mwindow->position->pos[2] += direction[2] * SPEED * SCALE;
			redraw = 1;
			break;
		case GLUT_KEY_PAGE_DOWN:
			mwindow->position->pos[0] -= direction[0] * SPEED * SCALE;
			mwindow->position->pos[1] -= direction[1] * SPEED * SCALE;
			mwindow->position->pos[2] -= direction[2] * SPEED * SCALE;
			redraw = 1;
			break;
	}

	if(redraw) glutPostRedisplay();
}

void BuildWindow::draw()
{
	mwindow->set_observation();
	mwindow->draw_planets();
	mwindow->draw_rocket(mwindow->rocket);
}
