#include "buildwindow.h"
#include "launchwindow.h"
#include "mwindow.h"
#include "part.h"
#include "planet.h"
#include "position.h"
#include "rocket.h"
#include "timer.h"

#include <GL/glut.h>
#include <math.h>
#include <stdio.h>



LaunchWindow::LaunchWindow(MWindow *mwindow)
 : SubWindow(mwindow)
{
	timer = new Timer;
}

LaunchWindow::~LaunchWindow()
{
	delete timer;
}

void LaunchWindow::draw_overlays()
{
	int y = HELP_MARGIN;
	char string[1024];

	draw_mode("Launch");
}

void LaunchWindow::draw_help()
{
	int y = mwindow->height - HELP_MARGIN - mwindow->text_height() * 2;
	mwindow->set_font(GLUT_BITMAP_HELVETICA_12);

	mwindow->draw_text(LEFT_HELP_MARGIN, y, "q -> return to construction mode");
	y -= mwindow->text_height();
	mwindow->draw_text(LEFT_HELP_MARGIN, y, "space -> fire engine set");
	y -= mwindow->text_height();
	mwindow->draw_text(LEFT_HELP_MARGIN, y, "e -> eject part set");
	y -= mwindow->text_height();
	mwindow->draw_text(LEFT_HELP_MARGIN, y, "? -> toggle help");
	y -= mwindow->text_height();
}

void LaunchWindow::cursor_motion(int x, int y)
{
}

void LaunchWindow::fire_engine_set()
{
// Get engine set
	ArrayList<int> *engine_set = &mwindow->rocket2->engine_sets[mwindow->rocket2->current_engineset];
	if(engine_set->total)
	{
		for(int i = 0; i < engine_set->total; i++)
		{
			Part *part = mwindow->rocket2->parts.values[engine_set->values[i]];
			part->throttle = 1;
		}
	}
	else
// Use first engine
	{
		for(int i = 0; i < mwindow->rocket2->parts.total; i++)
		{
			Part *part = mwindow->rocket2->parts.values[i];
printf("LaunchWindow::fire_engine_set 1 %f %f\n", part->max_thrust, part->fuel_qty);
			if(part->max_thrust > 0 && part->fuel_qty > 0)
			{
				part->throttle = 1;
				break;
			}
		}
	}
}

void LaunchWindow::reset_physics()
{
	timer->update();
}

void LaunchWindow::update_physics()
{
	long milliseconds = timer->get_difference();
	timer->update();
	Rocket *rocket = mwindow->rocket2;

// Apply force vector from every planet to rocket over time difference
	for(int i = 0; i < mwindow->planets.total; i++)
	{
		Planet *planet = mwindow->planets.values[i];
		Position gravity;
// Direction of gravity force to planet
		gravity.dir[0] = planet->position->pos[0] - rocket->position->pos[0];
		gravity.dir[1] = planet->position->pos[1] - rocket->position->pos[1];
		gravity.dir[2] = planet->position->pos[2] - rocket->position->pos[2];

// Magnitude of gravity force to planet
		Position::cart_to_sphere(gravity.dir);
	}
}

void LaunchWindow::keypress(unsigned char k)
{
	int redraw = 0;
	switch(k)
	{
		case 'q':
		case 0x1b:
			mwindow->mode = MODE_BUILD;
			mwindow->current_window = mwindow->bwindow;
			glutIdleFunc(0);
			glutPostRedisplay();
			break;
		
		case ' ':
			fire_engine_set();
			reset_physics();
			break;

		case 'h':
		case '?':
			mwindow->show_help = !mwindow->show_help;
			break;
	}
}

void LaunchWindow::keypress2(int k)
{
}

void LaunchWindow::draw()
{
	update_physics();
	mwindow->set_observation();
	mwindow->draw_planets();
	mwindow->draw_rocket(mwindow->rocket2);
}
