#include "buildwindow.h"
#include "launchwindow.h"
#include "mwindow.h"
#include "mwindow.inc"
#include "part.h"
#include "partwindow.h"
#include "planet.h"
#include "position.h"
#include "rocket.h"
#include "subwindow.h"
#include "texture.h"

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <GL/glut.h>

#define PROGRAM_NAME "Rocket Construction Set"

MWindow *mwindow = 0;

MWindow::MWindow()
{
	position = new Position;
	width = 640;
	height = 480;

	show_help = 1;
	selected_part = 0;
	mode = MODE_BUILD;
	button_state = GLUT_UP;
	new_part = 0;
}

MWindow::~MWindow()
{
	delete rocket;
	delete rocket2;
	delete position;
}

void MWindow::set_color(int color)
{
    static GLfloat light_ambient[] = { 0.0, 0.0, 0.0, 1.0 };
    static GLfloat light_diffuse[] = { 1.0, 1.0, 1.0, 1.0 };
    static GLfloat light_specular[] = { 1.0, 1.0, 1.0, 1.0 };
	float intensity = 1;

	light_ambient[0] = (double)((color & 0xff0000) >> 16) / 255 * intensity;
	light_ambient[1] = (double)((color & 0xff00) >> 8) / 255 * intensity;
	light_ambient[2] = (double)(color & 0xff) / 255 * intensity;
    glLightfv(GL_LIGHT0, GL_AMBIENT, light_ambient);
    glLightfv(GL_LIGHT0, GL_DIFFUSE, light_diffuse);
    glLightfv(GL_LIGHT0, GL_SPECULAR, light_specular);
}

void MWindow::begin_text()
{
    glDisable(GL_LIGHTING);

 	glMatrixMode(GL_PROJECTION);
    glPushMatrix();
 	glLoadIdentity();
 	glOrtho(-0.5, width - 0.5, -0.5, height - 0.5, -1.0, 1.0);

 	glMatrixMode(GL_MODELVIEW);
    glPushMatrix();
 	glLoadIdentity();
}

void MWindow::end_text()
{
 	glMatrixMode(GL_PROJECTION);
	glPopMatrix();

 	glMatrixMode(GL_MODELVIEW);
	glPopMatrix();

    glEnable(GL_LIGHTING);
}

void MWindow::draw_overlays()
{
 	begin_text();
 	current_window->draw_overlays();
 	end_text();
}

void MWindow::draw_help()
{
#define MARGIN 10
 	begin_text();

  	glColor4f(0.5, 0.5, 0.5, 0.5);
	glRecti(LEFT_HELP_MARGIN - MARGIN, HELP_MARGIN - MARGIN, width - HELP_MARGIN +MARGIN, height - HELP_MARGIN + MARGIN);

	glColor3f(1.0, 0.0, 0.0);
	set_font(GLUT_BITMAP_TIMES_ROMAN_24);
	draw_text(LEFT_HELP_MARGIN, height - HELP_MARGIN - text_height(), "Help");

	current_window->draw_help();
 	end_text();
}

void MWindow::set_observation()
{
	double result[3], direction[3];

	result[0] = position->pos[0];
	result[1] = position->pos[1];
	result[2] = position->pos[2];
// Direction always faces rocket
	direction[0] = position->pos[0] + position->dir[0];
	direction[1] = position->pos[1] + position->dir[1];
	direction[2] = position->pos[2] + position->dir[2];
	gluLookAt(result[0], 
		result[1], 
		result[2],
	    direction[0], 
		direction[1], 
		direction[2],
	    0.0, 
		0.0, 
		1.0);
}

void MWindow::part_observation(Part *part)
{
	double part_dir[3];
	part_dir[0] = part->position->pos[0] - part_obs[0];
	part_dir[1] = part->position->pos[1] - part_obs[1];
	part_dir[2] = part->position->pos[2] - part_obs[2];

	gluLookAt(part_obs[0], 
		part_obs[1], 
		part_obs[2],
	    part_dir[0], 
		part_dir[1], 
		part_dir[2],
	    0.0, 
		0.0, 
		1.0);
}

int MWindow::text_width(char *string)
{
	int len, i, result = 0;

	len = strlen(string);
	for(i = 0; i < len; i++)
    	result += glutBitmapWidth(font, string[i]);
	return result;
}

int MWindow::text_height()
{
	if(font == GLUT_BITMAP_TIMES_ROMAN_24)
	{
		return 20;
	}
	else
	if(font == GLUT_BITMAP_HELVETICA_12)
	{
		return 20;
	}
	else
	if(font == GLUT_BITMAP_HELVETICA_10)
	{
		return 15;
	}
	return 20;
}

void MWindow::set_font(void *font)
{
	this->font = font;
}

void MWindow::draw_center_text(int x, int y, char *string)
{
	draw_text(x - text_width(string) / 2, y, string);
}

void MWindow::draw_text(int x, int y, char *string)
{
	int len, i;

	glRasterPos2i(x, y);
	len = strlen(string);
	for(i = 0; i < len; i++)
    	glutBitmapCharacter(font, string[i]);
}

void MWindow::draw_part(Part *part)
{
    glPushMatrix();
	part->position->translate_rotate();
	part->draw();
    glPopMatrix();
}

void MWindow::draw_planets()
{
	for(int i = 0; i < planets.total; i++)
	{
		Planet *planet = planets.values[i];

    	glPushMatrix();
		set_color(planet->color);
    	glTranslatef(planet->position->pos[0], planet->position->pos[1], planet->position->pos[2]);
    	glutSolidSphere(planet->radius, 50, 50);
    	glPopMatrix();
	}
}

void MWindow::draw_rocket(Rocket *rocket)
{
    glPushMatrix();
	rocket->position->translate_rotate();
	rocket->draw();
    glPopMatrix();
}

void MWindow::draw()
{
    glClearColor(0.0, 0.0, 0.0, 0.0);
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

 	glPushMatrix();
	current_window->draw();
    glPopMatrix();

	draw_overlays();
	if(show_help) draw_help();
	glutSwapBuffers();
}

void MWindow::keypress(unsigned char k)
{
	current_window->keypress(k);
}

void MWindow::keypress2(int k)
{
	current_window->keypress2(k);
}

void MWindow::reshape(int w, int h)
{
	width = w;
	height = h;
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(45.0, width / (float)height, 0.2, 40.0);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glViewport(0, 0, width, height);
}

void MWindow::button_release(int x, int y)
{
}

void MWindow::button_press(int x, int y)
{
	last_x = x;
	last_y = y;
}

void MWindow::cursor_motion(int x, int y)
{
	current_window->cursor_motion(x, y);
	last_x = x;
	last_y = y;
}

void MWindow::draw_event()
{
	mwindow->draw();
}

void MWindow::keypress_event(unsigned char k, int x, int y)
{
	mwindow->modifiers = glutGetModifiers();
	mwindow->keypress(k);
}

void MWindow::keypress_event2(int k, int x, int y)
{
	mwindow->modifiers = glutGetModifiers();
	mwindow->keypress2(k);
}

void MWindow::reshape_event(int w, int h) 
{
	mwindow->reshape(w, h);
}

void MWindow::button_event(int button, int state, int x, int y)
{
	mwindow->modifiers = glutGetModifiers();
	mwindow->which_button = button;
	mwindow->button_state = state;
	if(state == GLUT_UP) 
	{
		mwindow->button_release(x, y);
	}
	else
	{
		mwindow->button_press(x, y);
	}
}

void MWindow::motion_event(int x, int y)
{
	mwindow->cursor_motion(x, y);
}

#include "shuttlebooster.h"
#include "shuttletank.h"

void MWindow::load_parts()
{
	Part *part;

	default_parts.append(part = new ShuttleBooster(this));
	default_parts.append(part = new ShuttleTank(this));
}

void MWindow::load_rocket()
{
	rocket = new Rocket(this);
	rocket2 = new Rocket(this);
}

void MWindow::load_planets()
{
	Planet *planet;

// Earth
	planets.append(planet = new Planet);
	planet->color = 0x0000ff;
	planet->radius = 1000 / 2 * SCALE;  // m
	planet->mass = 5.972E24; // kg
	planet->position->pos[0] = 0;
	planet->position->pos[1] = 0;
	planet->position->pos[2] = 0;

	position->pos[0] = 0;
	position->pos[1] = -1000 * SCALE;
	position->pos[2] = planet->radius + 50 * SCALE;
	position->dir[0] = 0;
	position->dir[1] = 1;
	position->dir[2] = 0;
	part_obs[0] = 0;
	part_obs[1] = -200 * SCALE;
	part_obs[2] = 0;
	rocket->position->pos[0] = 0;
	rocket->position->pos[1] = 0;
	rocket->position->pos[2] = planet->radius + 6 * SCALE;


// Mars
// 	planets.append(planet = new Planet);
// 	planet->color = 0xff0000;
// 	planet->radius = 100;
// 	planet->gravity = 3.22;
// 	planet->position->pos[0] = 0;
// 	planet->position->pos[1] = 50;
// 	planet->position->pos[2] = 0;
}

void MWindow::load_textures()
{
#include "flame_png.h"
	flame1 = new Texture(flame_png);
	
}


void MWindow::create_objects(int ac, char **av)
{
	pwindow = new PartWindow(this);
	bwindow = new BuildWindow(this);
	lwindow = new LaunchWindow(this);
	current_window = bwindow;

	glutInit(&ac, av);
    glutInitDisplayMode(GLUT_RGB | GLUT_DEPTH | GLUT_DOUBLE);
	glutInitWindowPosition(0, 0);
	glutInitWindowSize(mwindow->width, mwindow->height);
	glutCreateWindow(PROGRAM_NAME);

	load_rocket();
	load_parts();
	load_planets();
	load_textures();

    glEnable(GL_LIGHTING);
    glEnable(GL_LIGHT0);
    glEnable(GL_DEPTH_TEST);
    glEnable(GL_BLEND);
	glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

    static GLfloat light_position[] = { 0.0, 0.0, 1000.0, 0.0 };
    glLightfv(GL_LIGHT0, GL_POSITION, light_position);

	glutReshapeFunc(reshape_event);
	glutDisplayFunc(draw_event);
	glutKeyboardFunc(keypress_event);
	glutSpecialFunc(keypress_event2);
    glutMotionFunc(motion_event);
	glutMouseFunc(button_event);
	glutIdleFunc(0);
}

void MWindow::main_loop()
{
	glutMainLoop();
}



