#include "buildwindow.h"
#include "mwindow.h"
#include "part.h"
#include "partwindow.h"
#include "position.h"
#include "rocket.h"

#include <GL/glut.h>
#include <stdio.h>

PartWindow::PartWindow(MWindow *mwindow)
 : SubWindow(mwindow)
{
}

PartWindow::~PartWindow()
{
	
}

void PartWindow::cursor_motion(int x, int y)
{
	if(mwindow->button_state == GLUT_DOWN)
	{
// Rotate part coords
		if(mwindow->last_x != x || mwindow->last_y != y)
		{
			Part *part = mwindow->default_parts.values[mwindow->selected_part];
			part->position->trackball_action(mwindow->which_button,
				mwindow->last_x,
				mwindow->last_y,
				x, 
				y);
			glutPostRedisplay();
		}
	}
}

void PartWindow::keypress(unsigned char k)
{
	switch(k)
	{
		case 'p':
		case 'q':
		case 0x1b:
// Abandon part
			mwindow->mode = MODE_BUILD;
			glutPostRedisplay();
			mwindow->current_window = mwindow->bwindow;
    		break;

		case ' ':
		case 0xd:
// Place part
			mwindow->mode = MODE_BUILD;
			mwindow->rocket->parts.append(mwindow->new_part = mwindow->default_parts.values[mwindow->selected_part]->make_copy());
			mwindow->new_part->highlighted = 1;
			glutPostRedisplay();
			mwindow->current_window = mwindow->bwindow;
			break;

		case 'h':
		case '?':
			mwindow->show_help = !mwindow->show_help;
			glutPostRedisplay();
			break;
	}
}

void PartWindow::keypress2(int k)
{
	int redraw = 0, rotation = 0;
	switch(k)
	{
		case GLUT_KEY_RIGHT:
			mwindow->selected_part++;
			if(mwindow->selected_part >= mwindow->default_parts.total) mwindow->selected_part = 0;
			redraw = 1;
			break;

		case GLUT_KEY_LEFT:
			mwindow->selected_part--;
			if(mwindow->selected_part < 0) mwindow->selected_part = mwindow->default_parts.total - 1;
			redraw = 1;
			break;
	}

	if(redraw) glutPostRedisplay();
}

void PartWindow::draw_help()
{
	int y = mwindow->height - HELP_MARGIN - mwindow->text_height() * 2;
	mwindow->set_font(GLUT_BITMAP_HELVETICA_12);

	mwindow->draw_text(LEFT_HELP_MARGIN, y, "p -> quit part selection");
	y -= mwindow->text_height();
	mwindow->draw_text(LEFT_HELP_MARGIN, y, "enter -> add part to rocket");
	y -= mwindow->text_height();
	mwindow->draw_text(LEFT_HELP_MARGIN, y, "? -> toggle help");
	y -= mwindow->text_height();
}


void PartWindow::draw_overlays()
{
	Part *part = mwindow->default_parts.values[mwindow->selected_part];
	int y;
	char string[1024];
	
	draw_mode("Part Selection");
	mwindow->begin_text();

	glColor3f(0.0, 1.0, 0.0);
	mwindow->set_font(GLUT_BITMAP_HELVETICA_12);

	y = mwindow->height - INFO_MARGIN - mwindow->text_height();
	mwindow->draw_text(INFO_MARGIN, y, part->title);
	y -= mwindow->text_height();
	sprintf(string, "Thrust: %.0f lbs\n", part->max_thrust);
	mwindow->draw_text(INFO_MARGIN, y, string);
	y -= mwindow->text_height();
	sprintf(string, "Fuel qty: %.0f lbs\n", part->fuel_qty);
	mwindow->draw_text(INFO_MARGIN, y, string);
	y -= mwindow->text_height();
	sprintf(string, "Dry weight: %.0f lbs\n", part->dry_weight);
	mwindow->draw_text(INFO_MARGIN, y, string);
	y -= mwindow->text_height();
	
	mwindow->end_text();
}

void PartWindow::draw()
{
	Part *part = mwindow->default_parts.values[mwindow->selected_part];
	mwindow->part_observation(part);
	mwindow->draw_part(part);
}


void PartWindow::button_press(int x, int y)
{
}
