#ifndef EDIT_H
#define EDIT_H

#include "assets.inc"
#include "edl.inc"
#include "guicast.h"
#include "edits.inc"
#include "filexml.inc"
#include "mwindow.inc"
#include "plugin.inc"
#include "track.inc"
#include "transition.inc"

// UNITS ARE SAMPLES FOR AUDIO / FRAMES FOR VIDEO
// zoom_units was mwindow->zoom_sample for AEdit

// Generic edit of something

class Edit : public ListItem<Edit>
{
public:
	Edit(EDL *edl, Edits *edits);
	Edit(EDL *edl, Track *track);
	Edit() {};
	virtual ~Edit();

	void reset();
	virtual void copy_from(Edit *edit);
	virtual int identical(Edit &edit);
	virtual Edit& operator=(Edit& edit);
	virtual int operator==(Edit& edit);
	virtual void synchronize_params(Edit *edit);

	double units_to_samples(long units);
	double samples_to_units(long samples);
// Get size of frame to draw on timeline
	double picon_w();
	int picon_h();
	double frame_w();
	double frames_per_picon();
	int copy(long start, long end, FileXML *xml, char *output_path);
	void insert_transition(char  *title);
	void detach_transition();
// Determine if silence depending on existance of asset or plugin title
	virtual int silence();

// Media edit information
// Units are native units for the track.
// Start of edit in source file.
	long startsource;  
// Start of edit in project file.
	long startproject;    
// # of units in edit.
	long length;  
// Channel or layer of source
	int channel;
// ID for resource pixmaps
	int id;

// translation coordinates
	int center_x, center_y;
	float center_z;      

// Transition if one is present at the beginning of this edit
// This stores the length of the transition
	Transition *transition;
	EDL *edl;
	Track *track;
// Asset is 0 if silence
	Asset *asset;














	Edit(MWindow *mwindow, Edits *edits);

// ============================= initialization

	int load_properties(FileXML *xml, long &startproject);
	virtual int load_properties_derived(FileXML *xml) {};

// ============================= drawing

	virtual int draw(int flash, int center_pixel, int x, int w, int y, int h, int set_index_file) {};
	virtual int set_index_file(int flash, int center_pixel, int x, int y, int w, int h) {};
	int draw_transition(int flash, int center_pixel, int x, int w, int y, int h, int set_index_file);

	int draw_handles(BC_SubWindow *canvas, float view_start, float view_units, float zoom_units, int view_pixels, int center_pixel);
	int draw_titles(BC_SubWindow *canvas, float view_start, float zoom_units, int view_pixels, int center_pixel);

// ============================= editing

	virtual int copy_properties_derived(FileXML *xml, long length_in_selection) {};
	int paste(FileXML *xml);

	int popup_transition(float view_start, float zoom_units, int cursor_x, int cursor_y);

// Return 1 if the left handle was selected 2 if the right handle was selected
	int select_handle(float view_start, float zoom_units, int cursor_x, int cursor_y, long &selection);
	int shift_start_in(int edit_mode, long newposition, long oldposition);
	int shift_start_out(int edit_mode, long newposition, long oldposition);
	int shift_end_in(int edit_mode, long newposition, long oldposition);
	int shift_end_out(int edit_mode, long newposition, long oldposition);
	virtual int get_handle_parameters(long &left, long &right, long &left_sample, long &right_sample, float view_start, float zoom_units) {};
	virtual int long get_source_end() {};
	int dump();
	virtual int dump_derived() {};

// REMOVE
	long feather_left, feather_right;

// REMOVE
	Edits *edits;
	MWindow *mwindow;
};





#endif
