#include "awindow.h"
#include "awindowgui.h"
#include "clipedit.h"
#include "cplayback.h"
#include "cwindow.h"
#include "editpanel.h"
#include "edl.h"
#include "edlsession.h"
#include "filexml.h"
#include "keys.h"
#include "localsession.h"
#include "mainclock.h"
#include "mainundo.h"
#include "mbuttons.h"
#include "meterpanel.h"
#include "mwindow.h"
#include "mwindowgui.h"
#include "playbackengine.h"
#include "theme.h"
#include "timebar.h"
#include "trackcanvas.h"
#include "transportque.h"
#include "zoombar.h"


EditPanel::EditPanel(MWindow *mwindow, 
	BC_WindowBase *subwindow,
	int x, 
	int y, 
	int editing_mode, 
	int use_editing_mode,
	int use_keyframe, 
	int use_splice,   // Extra buttons
	int use_overwrite,
	int use_lift,
	int use_extract,
	int use_copy, 
	int use_paste, 
	int use_undo,
	int use_fit,
	int use_labels,
	int use_toclip,
	int use_meters)
{
	this->editing_mode = editing_mode;
	this->use_editing_mode = use_editing_mode;
	this->use_keyframe = use_keyframe;
	this->use_splice = use_splice;
	this->use_overwrite = use_overwrite;
	this->use_lift = 0;
	this->use_extract = 0;
	this->use_copy = use_copy;
	this->use_paste = use_paste;
	this->use_undo = use_undo;
	this->mwindow = mwindow;
	this->subwindow = subwindow;
	this->use_fit = use_fit;
	this->use_labels = use_labels;
	this->use_toclip = use_toclip;
	this->use_meters = use_meters;
	
	this->x = x;
	this->y = y;
	this->meter_panel = 0;
	arrow = 0;
	ibeam = 0;
}

EditPanel::~EditPanel()
{
}

void EditPanel::set_meters(MeterPanel *meter_panel)
{
	this->meter_panel = meter_panel;
}


void EditPanel::update_toggles(int new_editing_mode)
{
	if(arrow) arrow->update(new_editing_mode == EDITING_ARROW);
	if(ibeam) ibeam->update(new_editing_mode == EDITING_IBEAM);
	subwindow->flush();
}

void EditPanel::delete_buttons()
{
	if(use_editing_mode)
	{
		if(arrow) delete arrow;
		if(ibeam) delete ibeam;
	}
	
	if(use_keyframe)
		delete keyframe;


// 	if(editing_mode == EDITING_ARROW)
// 	{
		if(inpoint) delete inpoint;
		if(outpoint) delete outpoint;
//		delete delinpoint;
//		delete deloutpoint;
		if(use_copy) delete copy;
		if(use_splice) delete splice;
		if(use_overwrite) delete overwrite;
		if(use_lift) delete lift;
		if(use_extract) delete extract;
// 	}
// 	else
// 	{
		if(cut) delete cut;
		if(copy) delete copy;
		if(use_paste) delete paste;
//	}

	if(use_labels)
	{	
		delete labelbutton;
		delete prevlabel;
		delete nextlabel;
	}

	if(use_fit) delete fit;
	if(use_undo)
	{
		delete undo;
		delete redo;
	}
}

void EditPanel::create_buttons()
{
	x1 = x, y1 = y;

	if(use_editing_mode)
	{
		subwindow->add_subwindow(arrow = new ArrowButton(mwindow, this, x1, y1));
		x1 += arrow->get_w();
		subwindow->add_subwindow(ibeam = new IBeamButton(mwindow, this, x1, y1));
		x1 += ibeam->get_w();
	}

	if(use_keyframe)
	{
		subwindow->add_subwindow(keyframe = new KeyFrameButton(mwindow, x1, y1));
		x1 += keyframe->get_w();
	}

// Mandatory
	subwindow->add_subwindow(inpoint = new EditInPoint(mwindow, this, x1, y1));
	x1 += inpoint->get_w();
	subwindow->add_subwindow(outpoint = new EditOutPoint(mwindow, this, x1, y1));
	x1 += outpoint->get_w();
	if(use_splice)
	{
		subwindow->add_subwindow(splice = new EditSplice(mwindow, this, x1, y1));
		x1 += splice->get_w();
	}
	if(use_overwrite)
	{
		subwindow->add_subwindow(overwrite = new EditOverwrite(mwindow, this, x1, y1));
		x1 += overwrite->get_w();
	}
	if(use_lift)
	{
		subwindow->add_subwindow(lift = new EditLift(mwindow, this, x1, y1));
		x1 += lift->get_w();
	}
	if(use_extract)
	{
		subwindow->add_subwindow(extract = new EditExtract(mwindow, this, x1, y1));
		x1 += extract->get_w();
	}
	if(use_toclip)
	{
		subwindow->add_subwindow(clip = new EditToClip(mwindow, this, x1, y1));
		x1 += clip->get_w();
	}
	subwindow->add_subwindow(cut = new EditCut(mwindow, this, x1, y1));
	x1 += cut->get_w();
	if(use_copy)
	{
		subwindow->add_subwindow(copy = new EditCopy(mwindow, this, x1, y1));
		x1 += copy->get_w();
	}
	if(use_paste)
	{
		subwindow->add_subwindow(paste = new EditPaste(mwindow, this, x1, y1));
		x1 += paste->get_w();
	}
	
	if(use_meters)
	{
		if(!meter_panel)
		{
			printf("EditPanel::create_objects: meter_panel == 0\n");
		}
		subwindow->add_subwindow(meters = new MeterShow(mwindow, meter_panel, x1, y1));
		x1 += meters->get_w();
	}

	if(use_labels)
	{
		subwindow->add_subwindow(labelbutton = new EditLabelbutton(mwindow, this, x1, y1));
		x1 += labelbutton->get_w();
		subwindow->add_subwindow(prevlabel = new EditPrevLabel(mwindow, this, x1, y1));
		x1 += prevlabel->get_w();
		subwindow->add_subwindow(nextlabel = new EditNextLabel(mwindow, this, x1, y1));
		x1 += nextlabel->get_w();
	}

	if(use_fit)
	{
		subwindow->add_subwindow(fit = new EditFit(mwindow, this, x1, y1));
		x1 += fit->get_w();
	}

	if(use_undo)
	{
		subwindow->add_subwindow(undo = new EditUndo(mwindow, this, x1, y1));
		x1 += undo->get_w();
		subwindow->add_subwindow(redo = new EditRedo(mwindow, this, x1, y1));
		x1 += redo->get_w();
	}
}

void EditPanel::reposition_buttons(int x, int y)
{
	this->x = x; 
	this->y = y;
	x1 = x, y1 = y;

	if(use_editing_mode)
	{
		arrow->reposition_window(x1, y1);
		x1 += arrow->get_w();
		ibeam->reposition_window(x1, y1);
		x1 += ibeam->get_w();
	}

	if(use_keyframe)
	{
		keyframe->reposition_window(x1, y1);
		x1 += keyframe->get_w();
	}

	inpoint->reposition_window(x1, y1);
	x1 += inpoint->get_w();
	outpoint->reposition_window(x1, y1);
	x1 += outpoint->get_w();
	if(use_splice)
	{
		splice->reposition_window(x1, y1);
		x1 += splice->get_w();
	}
	if(use_overwrite)
	{
		overwrite->reposition_window(x1, y1);
		x1 += overwrite->get_w();
	}
	if(use_lift)
	{
		lift->reposition_window(x1, y1);
		x1 += lift->get_w();
	}
	if(use_extract)
	{
		extract->reposition_window(x1, y1);
		x1 += extract->get_w();
	}
	if(use_toclip)
	{
		clip->reposition_window(x1, y1);
		x1 += clip->get_w();
	}
	cut->reposition_window(x1, y1);
	x1 += cut->get_w();
	if(use_copy)
	{
		copy->reposition_window(x1, y1);
		x1 += copy->get_w();
	}
	if(use_paste)
	{
		paste->reposition_window(x1, y1);
		x1 += paste->get_w();
	}

	if(use_meters)
	{
		meters->reposition_window(x1, y1);
		x1 += meters->get_w();
	}

	if(use_labels)
	{
		labelbutton->reposition_window(x1, y1);
		x1 += labelbutton->get_w();
		prevlabel->reposition_window(x1, y1);
		x1 += prevlabel->get_w();
		nextlabel->reposition_window(x1, y1);
		x1 += nextlabel->get_w();
	}

	if(use_fit)
	{
		fit->reposition_window(x1, y1);
		x1 += fit->get_w();
	}

	if(use_undo)
	{
		undo->reposition_window(x1, y1);
		x1 += undo->get_w();
		redo->reposition_window(x1, y1);
		x1 += redo->get_w();
	}
}



int EditPanel::create_objects()
{
	create_buttons();
	return 0;
}

int EditPanel::get_w()
{
	return x1 - x;
}


void EditPanel::copy_selection()
{
	mwindow->copy(mwindow->edl->local_session->selectionstart, 
		mwindow->edl->local_session->selectionend);
}

void EditPanel::splice_selection()
{
}

void EditPanel::overwrite_selection()
{
}

void EditPanel::set_inpoint()
{
	mwindow->set_inpoint(mwindow->edl->local_session->selectionstart);
}

void EditPanel::set_outpoint()
{
	mwindow->set_outpoint(mwindow->edl->local_session->selectionend);
}

void EditPanel::clear_inpoint()
{
	mwindow->delete_inpoint();
}

void EditPanel::clear_outpoint()
{
	mwindow->delete_outpoint();
}

void EditPanel::to_clip()
{
	FileXML file;
	long start, end;
	
	
	
	
	
	
	mwindow->edl->copy(mwindow->edl->local_session->selectionstart, 
		mwindow->edl->local_session->selectionend, 
		1,
		&file,
		mwindow->plugindb,
		"",
		1);





//printf("VWindowEditing::to_clip 1 %d\n", mwindow->edl->assets->total());
	EDL *new_edl = new EDL(mwindow->edl);
	new_edl->create_objects();
	new_edl->load_xml(mwindow->plugindb, &file, LOAD_ALL);

//printf("VWindowEditing::to_clip 2 %d\n", mwindow->edl->assets->total());
	mwindow->awindow->clip_edit->create_clip(new_edl);
//printf("VWindowEditing::to_clip 3 %d\n", mwindow->edl->assets->total());
}


EditInPoint::EditInPoint(MWindow *mwindow, EditPanel *panel, int x, int y)
 : BC_Button(x, y, mwindow->theme->in_data)
{
	this->mwindow = mwindow;
	this->panel = panel;
	set_tooltip("In point ( ] )");
}
EditInPoint::~EditInPoint()
{
}
int EditInPoint::handle_event()
{
	panel->set_inpoint();
	return 1;
}
int EditInPoint::keypress_event()
{
	if(get_keypress() == ']') 
	{
		panel->set_inpoint();
		return 1;
	}
	return 0;
}

EditOutPoint::EditOutPoint(MWindow *mwindow, EditPanel *panel, int x, int y)
 : BC_Button(x, y, mwindow->theme->out_data)
{
	this->mwindow = mwindow;
	this->panel = panel;
	set_tooltip("Out point ( [ )");
}
EditOutPoint::~EditOutPoint()
{
}
int EditOutPoint::handle_event()
{
	panel->set_outpoint();
	return 1;
}
int EditOutPoint::keypress_event()
{
	if(get_keypress() == '[') 
	{
		panel->set_outpoint();
		return 1;
	}
	return 0;
}

EditDelInPoint::EditDelInPoint(MWindow *mwindow, EditPanel *panel, int x, int y)
 : BC_Button(x, y, mwindow->theme->indelete_data)
{
	this->mwindow = mwindow;
	this->panel = panel;
	set_tooltip("Delete in point");
}
EditDelInPoint::~EditDelInPoint()
{
}
int EditDelInPoint::handle_event()
{
	panel->clear_inpoint();
	return 1;
}
int EditDelInPoint::keypress_event()
{
	if(get_keypress() == '{')
	{
		panel->clear_inpoint();
		return 1;
	}
	return 0;
}

EditDelOutPoint::EditDelOutPoint(MWindow *mwindow, EditPanel *panel, int x, int y)
 : BC_Button(x, y, mwindow->theme->outdelete_data)
{
	this->mwindow = mwindow;
	this->panel = panel;
	set_tooltip("Delete out point");
}
EditDelOutPoint::~EditDelOutPoint()
{
}
int EditDelOutPoint::handle_event()
{
	panel->clear_outpoint();
	return 1;
}
int EditDelOutPoint::keypress_event()
{
	if(get_keypress() == '}') 
	{
		panel->clear_outpoint();
		return 1;
	}
	return 0;
}

EditNextLabel::EditNextLabel(MWindow *mwindow, EditPanel *panel, int x, int y)
 : BC_Button(x, y, mwindow->theme->nextlabel_data)
{
	this->mwindow = mwindow;
	this->panel = panel;
	set_tooltip("Next label");
}
EditNextLabel::~EditNextLabel()
{
}
int EditNextLabel::keypress_event()
{
	if(get_keypress() == RIGHT && ctrl_down())
		return handle_event();
	return 0;
}
int EditNextLabel::handle_event()
{
	mwindow->gui->mbuttons->transport->handle_transport(STOP);
	mwindow->next_label();
	return 1;
}

EditPrevLabel::EditPrevLabel(MWindow *mwindow, EditPanel *panel, int x, int y)
 : BC_Button(x, y, mwindow->theme->prevlabel_data)
{
	this->mwindow = mwindow;
	this->panel = panel;
	set_tooltip("Previous label");
}
EditPrevLabel::~EditPrevLabel()
{
}
int EditPrevLabel::keypress_event()
{
	if(get_keypress() == LEFT && ctrl_down())
		return handle_event();
	return 0;
}
int EditPrevLabel::handle_event()
{
	mwindow->gui->mbuttons->transport->handle_transport(STOP);
	mwindow->prev_label();
	return 1;
}

EditLift::EditLift(MWindow *mwindow, EditPanel *panel, int x, int y)
 : BC_Button(x, y, mwindow->theme->lift_data)
{
	this->mwindow = mwindow;
	this->panel = panel;
	set_tooltip("Lift");
}
EditLift::~EditLift()
{
}
int EditLift::handle_event()
{
	mwindow->lift_selection();
	return 1;
}

EditOverwrite::EditOverwrite(MWindow *mwindow, EditPanel *panel, int x, int y)
 : BC_Button(x, y, mwindow->theme->overwrite_data)
{
	this->mwindow = mwindow;
	this->panel = panel;
	set_tooltip("Overwrite");
}
EditOverwrite::~EditOverwrite()
{
}
int EditOverwrite::handle_event()
{
	panel->overwrite_selection();
	return 1;
}

EditExtract::EditExtract(MWindow *mwindow, EditPanel *panel, int x, int y)
 : BC_Button(x, y, mwindow->theme->extract_data)
{
	this->mwindow = mwindow;
	this->panel = panel;
	set_tooltip("Extract");
}
EditExtract::~EditExtract()
{
}
int EditExtract::handle_event()
{
	mwindow->extract_selection();
	return 1;
}

EditToClip::EditToClip(MWindow *mwindow, EditPanel *panel, int x, int y)
 : BC_Button(x, y, mwindow->theme->toclip_data)
{
	this->mwindow = mwindow;
	this->panel = panel;
	set_tooltip("To clip");
}
EditToClip::~EditToClip()
{
}
int EditToClip::handle_event()
{
	panel->to_clip();
	return 1;
}

EditSplice::EditSplice(MWindow *mwindow, EditPanel *panel, int x, int y)
 : BC_Button(x, y, mwindow->theme->splice_data)
{
	this->mwindow = mwindow;
	this->panel = panel;
	set_tooltip("Splice");
}
EditSplice::~EditSplice()
{
}
int EditSplice::handle_event()
{
	panel->splice_selection();
	return 1;
}

EditCut::EditCut(MWindow *mwindow, EditPanel *panel, int x, int y)
 : BC_Button(x, y, mwindow->theme->cut_data)
{
	this->mwindow = mwindow;
	this->panel = panel;
	set_tooltip("Cut");
}
EditCut::~EditCut()
{
}
int EditCut::keypress_event()
{
	if(get_keypress() == 'x')
		return handle_event();
	return 0;
}

int EditCut::handle_event()
{
	mwindow->undo->update_undo_before("cut", LOAD_EDITS | LOAD_TIMEBAR);
	mwindow->cut(mwindow->edl->local_session->selectionstart, 
		mwindow->edl->local_session->selectionend);
	mwindow->undo->update_undo_after();

	mwindow->gui->update(1, 1, 1, 1, 1, 1);
	mwindow->cwindow->playback_engine->que->send_command(CURRENT_FRAME, 
							CHANGE_EDL,
							mwindow->edl,
							1);
	return 1;
}

EditCopy::EditCopy(MWindow *mwindow, EditPanel *panel, int x, int y)
 : BC_Button(x, y, mwindow->theme->copy_data)
{
	this->mwindow = mwindow;
	this->panel = panel;
	set_tooltip("Copy");
}
EditCopy::~EditCopy()
{
}

int EditCopy::keypress_event()
{
	if(get_keypress() == 'c')
		return handle_event();
	return 0;
}
int EditCopy::handle_event()
{
	panel->copy_selection();
	return 1;
}

EditAppend::EditAppend(MWindow *mwindow, EditPanel *panel, int x, int y)
 : BC_Button(x, y, mwindow->theme->append_data)
{
	this->mwindow = mwindow;
	this->panel = panel;
	set_tooltip("Append to end of track");
}
EditAppend::~EditAppend()
{
}


int EditAppend::handle_event()
{
	return 1;
}


EditInsert::EditInsert(MWindow *mwindow, EditPanel *panel, int x, int y)
 : BC_Button(x, y, mwindow->theme->insert_data)
{
	this->mwindow = mwindow;
	this->panel = panel;
	set_tooltip("Insert before beginning of track");
}
EditInsert::~EditInsert()
{
}


int EditInsert::handle_event()
{
	
	return 1;
}


EditPaste::EditPaste(MWindow *mwindow, EditPanel *panel, int x, int y)
 : BC_Button(x, y, mwindow->theme->paste_data)
{
	this->mwindow = mwindow;
	this->panel = panel;
	set_tooltip("Paste");
}
EditPaste::~EditPaste()
{
}

int EditPaste::keypress_event()
{
	if(get_keypress() == 'v')
		return handle_event();
	return 0;
}
int EditPaste::handle_event()
{
	long len = mwindow->gui->get_clipboard()->clipboard_len(SECONDARY_SELECTION);

	char *string = new char[len + 1];
	mwindow->gui->get_clipboard()->from_clipboard(string, len, SECONDARY_SELECTION);
	FileXML file;
	file.read_from_string(string);


printf("EditPaste::handle_event 1\n%s\n", string);
	mwindow->undo->update_undo_before("paste", LOAD_EDITS);
	mwindow->paste(mwindow->edl->local_session->selectionstart, 
		mwindow->edl->local_session->selectionend, &file);
	mwindow->undo->update_undo_after();
	mwindow->gui->update(1, 1, 1, 1, 0, 1);
	mwindow->awindow->gui->update_assets();
	
	delete [] string;
	return 1;
}



EditTransition::EditTransition(MWindow *mwindow, EditPanel *panel, int x, int y)
 : BC_Button(x, y, mwindow->theme->transition_data)
{
	this->mwindow = mwindow;
	this->panel = panel;
	set_tooltip("Set transition");
}
EditTransition::~EditTransition()
{
}
int EditTransition::handle_event()
{
	return 1;
}

EditPresentation::EditPresentation(MWindow *mwindow, EditPanel *panel, int x, int y)
 : BC_Button(x, y, mwindow->theme->presentation_data)
{
	this->mwindow = mwindow;
	this->panel = panel;
	set_tooltip("Set presentation up to current position");
}
EditPresentation::~EditPresentation()
{
}
int EditPresentation::handle_event()
{
	return 1;
}

EditUndo::EditUndo(MWindow *mwindow, EditPanel *panel, int x, int y)
 : BC_Button(x, y, mwindow->theme->undo_data)
{
	this->mwindow = mwindow;
	this->panel = panel;
	set_tooltip("Undo");
}
EditUndo::~EditUndo()
{
}
int EditUndo::keypress_event()
{
	if(get_keypress() == 'z')
		return handle_event();
	return 0;
}
int EditUndo::handle_event()
{
	mwindow->undo->undo(); 

	mwindow->gui->update(1, 1, 1, 1, 1, 1);
	mwindow->cwindow->playback_engine->que->send_command(CURRENT_FRAME, 
							CHANGE_ALL,
							mwindow->edl,
							1);
	return 1;
}

EditRedo::EditRedo(MWindow *mwindow, EditPanel *panel, int x, int y)
 : BC_Button(x, y, mwindow->theme->redo_data)
{
	this->mwindow = mwindow;
	this->panel = panel;
	set_tooltip("Redo");
}
EditRedo::~EditRedo()
{
}
int EditRedo::keypress_event()
{
	if(get_keypress() == 'Z')
		return handle_event();
	return 0;
}
int EditRedo::handle_event()
{
	mwindow->undo->redo(); 
	mwindow->gui->update(1, 1, 1, 1, 1, 1);
	mwindow->cwindow->playback_engine->que->send_command(CURRENT_FRAME, 
							CHANGE_ALL,
							mwindow->edl,
							1);
	return 1;
}

EditLabelbutton::EditLabelbutton(MWindow *mwindow, EditPanel *panel, int x, int y)
 : BC_Button(x, y, mwindow->theme->labelbutton_data)
{
	this->mwindow = mwindow;
	this->panel = panel;
	set_tooltip("Toggle label at current position");
}

EditLabelbutton::~EditLabelbutton()
{
}
int EditLabelbutton::keypress_event()
{
	if(get_keypress() == 'l')
		return handle_event();
	return 0;
}
int EditLabelbutton::handle_event()
{
	mwindow->undo->update_undo_before("label", LOAD_TIMEBAR);
	mwindow->toggle_label();
	mwindow->undo->update_undo_after();
	mwindow->gui->canvas->activate();
	return 1;
}


EditFit::EditFit(MWindow *mwindow, EditPanel *panel, int x, int y)
 : BC_Button(x, y, mwindow->theme->fit_data)
{
	this->mwindow = mwindow;
	this->panel = panel;
	set_tooltip("Fit selection to display");
}
EditFit::~EditFit()
{
}
int EditFit::keypress_event()
{
	if(get_keypress() == 'f' || get_keypress() == 'F') 
	{
		handle_event();
		return 1;
	}
	return 0;
}
int EditFit::handle_event()
{
	mwindow->fit_selection();
	return 1;
}


ArrowButton::ArrowButton(MWindow *mwindow, EditPanel *panel, int x, int y)
 : BC_Toggle(x, 
 	y, 
	mwindow->theme->arrow_data,
	mwindow->edl->session->editing_mode == EDITING_ARROW,
	"",
	0,
	0,
	0)
{
	this->mwindow = mwindow;
	this->panel = panel;
	set_tooltip("Drag and drop editing mode");
}

int ArrowButton::handle_event()
{
	update(1);
	panel->ibeam->update(0);
	mwindow->set_editing_mode(EDITING_ARROW);
// Nothing after this
	return 1;
}


IBeamButton::IBeamButton(MWindow *mwindow, EditPanel *panel, int x, int y)
 : BC_Toggle(x, 
 	y, 
	mwindow->theme->ibeam_data,
	mwindow->edl->session->editing_mode == EDITING_IBEAM,
	"",
	0,
	0,
	0)
{
	this->mwindow = mwindow;
	this->panel = panel;
	set_tooltip("Cut and paste editing mode");
}

int IBeamButton::handle_event()
{
	update(1);
	panel->arrow->update(0);
	mwindow->set_editing_mode(EDITING_IBEAM);
// Nothing after this
	return 1;
}

KeyFrameButton::KeyFrameButton(MWindow *mwindow, int x, int y)
 : BC_Toggle(x, 
 	y, 
	mwindow->theme->autokeyframe_data,
	mwindow->edl->session->auto_keyframes,
	"",
	0,
	0,
	0)
{
	this->mwindow = mwindow;
	set_tooltip("Generate keyframes while tweeking");
}

int KeyFrameButton::handle_event()
{
	mwindow->set_auto_keyframes(get_value());
	return 1;
}


