#include "edl.h"
#include "filexml.h"
#include "mainmenu.h"
#include "mainundo.h"
#include "mwindow.h"
#include "mwindowgui.h"
#include <string.h>

MainUndo::MainUndo(MWindow *mwindow)
{ 
	this->mwindow = mwindow; 
}

MainUndo::~MainUndo()
{
}

// REMOVE
int MainUndo::update_undo_all(char *description, int after)
{
	update_undo(description, "ALL", after);
}

// REMOVE
int MainUndo::update_undo_audio(char *description, int after)
{
	update_undo(description, "AUDIO", after);
}

// REMOVE
int MainUndo::update_undo_edits(char *description, int after)
{
	update_undo(description, "EDITS", after);
}

// REMOVE
int MainUndo::update_undo_patches(char *description, int after)
{
	update_undo(description, "PATCHES", after);
}

// REMOVE
int MainUndo::update_undo_console(char *description, int after)
{
	update_undo(description, "CONSOLE", after);
}

// REMOVE
int MainUndo::update_undo_timebar(char *description, int after)
{
	update_undo(description, "TIMEBAR", after);
}

// REMOVE
int MainUndo::update_undo_automation(char *description, int after)
{
	update_undo(description, "AUTOMATION", after);
}

void MainUndo::update_undo_before(char *description, unsigned long load_flags)
{
	FileXML file;
//printf("MainUndo::update_undo_before 1 %p\n", mwindow);
	mwindow->edl->save_xml(mwindow->plugindb, &file, "");

//printf("MainUndo::update_undo_before 1\n");
	current_entry = undo_stack.push();
	current_entry->load_flags = load_flags;
	current_entry->set_data_before(file.string);
	current_entry->set_description(description);
//printf("MainUndo::update_undo_before 1\n");

// the after update is always without a description
	mwindow->gui->lock_window();
	mwindow->gui->mainmenu->undo->update_caption(description);
	mwindow->gui->mainmenu->redo->update_caption("");
	mwindow->gui->unlock_window();
//printf("MainUndo::update_undo_before 2\n");
}

void MainUndo::update_undo_after()
{
	FileXML file;
	mwindow->edl->save_xml(mwindow->plugindb, &file, "");
	current_entry->set_data_after(file.string);
}






int MainUndo::update_undo(char *description, char *type, int after)
{
	FileXML file;
	mwindow->edl->save_xml(mwindow->plugindb, &file, "");

	if(!after)
	{
// this is the before buffer
		current_entry = undo_stack.push();
		current_entry->set_type(type);
		current_entry->set_data_before(file.string);
	}
	else
	{
		current_entry->set_data_after(file.string);
	}

// the after update is always without a description
	if(!after)
		current_entry->set_description(description);

	if(mwindow->gui)
	{
// the after update is always without a description
		if(!after)
			mwindow->gui->mainmenu->undo->update_caption(description);

		mwindow->gui->mainmenu->redo->update_caption("");
	}
	return 0;
}






int MainUndo::undo()
{
	if(undo_stack.current)
	{
		current_entry = undo_stack.current;
		if(current_entry->description && mwindow->gui) 
			mwindow->gui->mainmenu->redo->update_caption(current_entry->description);
		
		FileXML file;
		
		file.read_from_string(current_entry->data_before);
		load_from_undo(&file, current_entry->load_flags);
		
		undo_stack.pull();    // move current back
		if(mwindow->gui)
		{
			current_entry = undo_stack.current;
			if(current_entry)
				mwindow->gui->mainmenu->undo->update_caption(current_entry->description);
			else
				mwindow->gui->mainmenu->undo->update_caption("");
		}
	}
	return 0;
}

int MainUndo::redo()
{
	current_entry = undo_stack.pull_next();
	
	if(current_entry)
	{
		FileXML file;
		file.read_from_string(current_entry->data_after);
		load_from_undo(&file, current_entry->load_flags);

		if(mwindow->gui)
		{
			mwindow->gui->mainmenu->undo->update_caption(current_entry->description);
			
			if(current_entry->next)
				mwindow->gui->mainmenu->redo->update_caption(current_entry->next->description);
			else
				mwindow->gui->mainmenu->redo->update_caption("");
		}
	}
	return 0;
}


// Here the master EDL loads 
int MainUndo::load_from_undo(FileXML *file, unsigned long load_flags)
{
	mwindow->edl->load_xml(mwindow->plugindb, file, load_flags);

	return 0;
}




