#include "assetmanager.h"
#include "assets.h"
#include "awindow.h"
#include "awindowgui.h"
#include "defaulttheme.h"
#include "editpanel.h"
#include "mbuttons.h"
#include "cache.h"
#include "channel.h"
#include "colormodels.h"
#include "console.h"
#include "cplayback.h"
#include "cwindow.h"
#include "cwindowgui.h"
#include "defaults.h"
#include "edl.h"
#include "edlsession.h"
#include "errorbox.h"
#include "file.h"
#include "fileformat.h"
#include "filesystem.h"
#include "filexml.h"
#include "indexfile.h"
#include "levelwindow.h"
#include "levelwindowgui.h"
#include "loadfile.inc"
#include "localsession.h"
#include "mainmenu.h"
#include "samplescroll.h"
#include "mainindexes.h"
#include "mainprogress.h"
#include "mainundo.h"
#include "mwindow.h"
#include "mwindowgui.h"
#include "module.h"
#include "modules.h"
#include "new.h"
#include "patchbay.h"
#include "tracking.h"
#include "playbackengine.h"
#include "pluginserver.h"
#include "preferences.h"
#include "record.h"
#include "recordlabel.h"
#include "render.h"
#include "mainsession.h"
#include "statusbar.h"
#include "trackcanvas.h"
#include "theme.h"
#include "threadloader.h"
#include "timebar.h"
#include "track.h"
#include "tracks.h"
#include "trackscroll.h"
#include "transportque.h"
#include "videodevice.inc"
#include "videowindow.h"
#include "vwindow.h"
#include "vwindowgui.h"
#include "zoombar.h"

#include <string.h>



extern "C"
{




// Hack for libdv
// Hack to remove glib dependancy

void
g_log (const char    *log_domain,
       int  log_level,
       const char    *format,
       ...)
{
}

void
g_logv (const char    *log_domain,
       int  log_level,
       const char    *format,
       ...)
{
}



// Hack for XFree86 4.1.0

int atexit(void (*function)(void))
{
	return 0;
}



}





MWindow::MWindow()
{
}

MWindow::~MWindow()
{
//printf("MWindow::~MWindow 1\n");
	clean_indexes();
//printf("MWindow::~MWindow 1\n");
	delete mainprogress;
//printf("MWindow::~MWindow 1\n");
	delete tracks;
//	delete console;
//	delete level_window;
//	delete video_window;
//	delete assets;
//printf("MWindow::~MWindow 1\n");
	delete audio_cache;             // delete the cache after the assets
//printf("MWindow::~MWindow 1\n");
	delete video_cache;             // delete the cache after the assets
//printf("MWindow::~MWindow 2\n");
//	delete threadloader;
	if(gui) delete gui;
//printf("MWindow::~MWindow 1\n");
	delete undo;
//printf("MWindow::~MWindow 1\n");
	delete preferences;
//printf("MWindow::~MWindow 1\n");
//	delete_plugins();
	delete defaults;
//printf("MWindow::~MWindow 1\n");
	delete render;
//printf("MWindow::~MWindow 1\n");
	delete renderlist;
//printf("MWindow::~MWindow 1\n");
	delete awindow;
//printf("MWindow::~MWindow 1\n");
	delete vwindow;
//printf("MWindow::~MWindow 1\n");
	delete cwindow;
//printf("MWindow::~MWindow 1\n");
	plugin_guis->remove_all_objects();
//printf("MWindow::~MWindow 2\n");
	delete plugin_guis;
//printf("MWindow::~MWindow 2\n");
}

void MWindow::init_defaults()
{
// set the .bcast directory
	char directory[1024];
	FileSystem fs;

	sprintf(directory, "%s", BCASTDIR);
	fs.complete_path(directory);
	if(fs.is_dir(directory)) 
	{
		fs.create_dir(directory); 
	}

// load the defaults
	strcat(directory, "Cinelerra_rc");

	defaults = new Defaults(directory);
	defaults->load();
}

void MWindow::init_plugins()
{
	FileSystem fs;
	int result, pass; 
	char path[1024], *directory;
	PluginServer *newplugin;
	plugindb = new ArrayList<PluginServer*>;

	fs.set_filter("*.plugin");

	for(pass = 0; pass < 2; pass++)
	{
		result = 1;          // default is abort

		switch(pass)
		{
// on first pass use global dir	
			case 0: 
				directory = preferences->global_plugin_dir;
				break;
// on second pass use local dir
			case 1: 
				directory = preferences->local_plugin_dir;
				break;
		}

		if(!strlen(directory)) 
			result = 1;
		else
			result = fs.update(directory);

//printf("MWindow::init_plugins 1 %s\n", directory);
		if(!result)
		{
//printf("MWindow::init_plugins 1 %d\n", fs.dir_list.total);
			for(int i = 0; i < fs.dir_list.total; i++)
			{
				char path[BCTEXTLEN];
				strcpy(path, fs.dir_list.values[i]->path);
//printf("MWindow::init_plugins 2 %s\n", path);

// File is a directory
				if(!fs.is_dir(path))
				{
					continue;
				}
				else
				{
// Try to query the plugin
					fs.complete_path(path);
					PluginServer *new_plugin = new PluginServer(path);
//printf("MWindow::init_plugins 3 %s\n", path);
					if(!new_plugin->open_plugin(1, 0, 0))
					{
//printf("MWindow::init_plugins 4 %s\n", path);
						plugindb->append(new_plugin);
//printf("MWindow::init_plugins 5 %s\n", path);
						new_plugin->close_plugin();
//printf("MWindow::init_plugins 6 %s\n", path);
					}
					else
// Plugin failed to open
					{
						delete new_plugin;
					}
				}
			}
		}
		else
// notify user of failed directory search
		{
			switch(pass)
			{
// on first pass use global dir	
				case 0: 
					if(strlen(preferences->global_plugin_dir)) 
						printf("MWindow::init_plugins: Couldn't open global plugin directory.\n");  
					break;
// on second pass use local dir
				case 1: 
					if(strlen(preferences->local_plugin_dir)) 
						printf("MWindow::init_plugins: Couldn't open local plugin directory.\n");    
					break;
			}
		}
	}
}

void MWindow::delete_plugins()
{
	for(int i = 0; i < plugindb->total; i++)
	{
		delete plugindb->values[i];
	}
	delete plugindb;
}

void MWindow::create_plugindb(int do_audio, 
		int do_video, 
		int is_realtime, 
		int is_transition,
		int is_theme,
		ArrayList<PluginServer*> &plugindb)
{
// Get plugins
//printf("MWindow::create_plugindb 1 %d\n", this->plugindb->total);
	for(int i = 0; i < this->plugindb->total; i++)
	{
		PluginServer *current = this->plugindb->values[i];
//printf("MWindow::create_plugindb 2 %d %d %d %d %d\n", this->plugindb->total,
//	i, 
// 	current->audio,
//	current->video,
//	current->realtime,
//	current->transition);

		if(current->audio == do_audio &&
			current->video == do_video &&
			(current->realtime == is_realtime || is_realtime < 0) &&
			current->transition == is_transition &&
			current->theme == is_theme)
			plugindb.append(current);
	}
//printf("MWindow::create_plugindb 3 %d\n");

// Alphabetize list by title
	int done = 0;
	while(!done)
	{
		done = 1;
		
		for(int i = 0; i < plugindb.total - 1; i++)
		{
			PluginServer *value1 = plugindb.values[i];
			PluginServer *value2 = plugindb.values[i + 1];
			if(strcmp(value1->title, value2->title) > 0)
			{
				done = 0;
				plugindb.values[i] = value2;
				plugindb.values[i + 1] = value1;
			}
		}
	}
//printf("MWindow::create_plugindb 4\n");
}

PluginServer* MWindow::scan_plugindb(char *title)
{
	for(int i = 0; i < plugindb->total; i++)
	{
		if(!strcasecmp(plugindb->values[i]->title, title)) return plugindb->values[i];
	}
	return 0;
}

void MWindow::init_preferences()
{
//printf("MWindow::init_preferences 1\n");
	preferences = new Preferences(this);
	preferences->load_defaults(defaults);
	session = new MainSession;
	session->load_defaults(defaults);
//printf("MWindow::init_preferences 2\n");
}

void MWindow::clean_indexes()
{
	FileSystem fs;
	int total_excess;
	long oldest;
	int oldest_item;
	int result;
	char string[1024];

// Delete extra indexes
	fs.set_filter("*.idx");
	fs.complete_path(preferences->index_directory);
	fs.update(preferences->index_directory);
//printf("MWindow::clean_indexes 1 %d\n", fs.dir_list.total);

// Eliminate directories
	result = 1;
	while(result)
	{
		result = 0;
		for(int i = 0; i < fs.dir_list.total && !result; i++)
		{
			fs.join_names(string, preferences->index_directory, fs.dir_list.values[i]->name);
			if(!fs.is_dir(string))
			{
				delete fs.dir_list.values[i];
				fs.dir_list.remove_number(i);
				result = 1;
			}
		}
	}
	total_excess = fs.dir_list.total - preferences->index_count;

//printf("MWindow::clean_indexes 2 %d\n", fs.dir_list.total);
	while(total_excess > 0)
	{
// Get oldest
		for(int i = 0; i < fs.dir_list.total; i++)
		{
			fs.join_names(string, preferences->index_directory, fs.dir_list.values[i]->name);

			if(i == 0 || fs.get_date(string) <= oldest)
			{
				oldest = fs.get_date(string);
				oldest_item = i;
			}
		}

		fs.join_names(string, preferences->index_directory, fs.dir_list.values[oldest_item]->name);
		if(remove(string))
			perror("delete_indexes");
		delete fs.dir_list.values[oldest_item];
		fs.dir_list.remove_number(oldest_item);
		total_excess--;
	}
}

void MWindow::init_awindow()
{
	awindow = new AWindow(this);
	awindow->create_objects();
}

void MWindow::init_theme()
{
	default_theme = new DefaultTheme;
//printf("MWindow::init_theme 1\n");	
	theme = 0;

	for(int i = 0; i < plugindb->total; i++)
	{
		if(plugindb->values[i]->theme &&
			!strcasecmp(preferences->theme, plugindb->values[i]->title))
		{
			PluginServer plugin = *plugindb->values[i];
			plugin.open_plugin(0, 0, 0);
			theme = plugin.new_theme();
			plugin.close_plugin();
		}
	}

	if(!theme)
		theme = default_theme;
}

void MWindow::init_edl()
{
	edl = new EDL;
//printf("MWindow::init_edl 1\n");
	edl->create_objects();
//printf("MWindow::init_edl 1\n");
    edl->load_defaults(defaults);
//printf("MWindow::init_edl 1\n");
	edl->create_default_tracks();
	edl->tracks->update_y_pixels(theme);
//printf("MWindow::init_edl 2\n");
}

void MWindow::init_compositor()
{
	cwindow = new CWindow(this);
    cwindow->create_objects();
}

void MWindow::init_viewer()
{
	vwindow = new VWindow(this);
	vwindow->load_defaults();
	vwindow->create_objects();
}

void MWindow::init_cache()
{
	audio_cache = new Cache(edl, plugindb);
	video_cache = new Cache(edl, plugindb);
}

void MWindow::init_tuner(ArrayList<Channel*> &channeldb, char *path)
{
	FileSystem fs;
	char directory[1024];
	FileXML file;
	Channel *channel;
	int done;

	sprintf(directory, BCASTDIR);
	fs.complete_path(directory);
	fs.join_names(directory, directory, path);
	done = file.read_from_file(directory, 1);

// Load channels
	while(!done)
	{
		channel = new Channel;
		if(!(done = channel->load(&file)))
			channeldb.append(channel);
		else
		{
			delete channel;
		}
	}
}

void MWindow::save_tuner(ArrayList<Channel*> &channeldb, char *path)
{
	FileSystem fs;
	char directory[1024];
	FileXML file;

	sprintf(directory, BCASTDIR);
	fs.complete_path(directory);
	strcat(directory, path);

	if(channeldb.total)
	{
		for(int i = 0; i < channeldb.total; i++)
		{
// Save channel here
			channeldb.values[i]->save(&file);
		}
		file.terminate_string();
		file.write_to_file(directory);
	}
}

void MWindow::init_menus()
{
	char string[BCTEXTLEN];
	cmodel_to_text(string, BC_RGB888);
	colormodels.append(new ColormodelItem(string, BC_RGB888));
	cmodel_to_text(string, BC_RGBA8888);
	colormodels.append(new ColormodelItem(string, BC_RGBA8888));
	cmodel_to_text(string, BC_RGB161616);
	colormodels.append(new ColormodelItem(string, BC_RGB161616));
	cmodel_to_text(string, BC_RGBA16161616);
	colormodels.append(new ColormodelItem(string, BC_RGBA16161616));
	cmodel_to_text(string, BC_YUV888);
	colormodels.append(new ColormodelItem(string, BC_YUV888));
	cmodel_to_text(string, BC_YUVA8888);
	colormodels.append(new ColormodelItem(string, BC_YUVA8888));
	cmodel_to_text(string, BC_YUV161616);
	colormodels.append(new ColormodelItem(string, BC_YUV161616));
	cmodel_to_text(string, BC_YUVA16161616);
	colormodels.append(new ColormodelItem(string, BC_YUVA16161616));
}

void MWindow::init_indexes()
{
	mainindexes = new MainIndexes(this);
	mainindexes->start_loop();
}

void MWindow::init_gui()
{
	gui = new MWindowGUI(this);
	gui->create_objects();
	gui->load_defaults(defaults);
}

void MWindow::init_render()
{
	render = new Render(this);
	renderlist = new Render(this);
}

void MWindow::create_objects(int want_gui, int want_new)
{
	char string[1024];
	FileSystem fs;
// Work around X bug
	BC_WindowBase::get_resources()->use_xvideo = 0;

	init_menus();
//printf("MWindow::create_objects 1\n");
	init_defaults();
//printf("MWindow::create_objects 1\n");
	init_preferences();
//printf("MWindow::create_objects 1\n");
	init_plugins();
//printf("MWindow::create_objects 1\n");
	init_theme();
// Default project created here
//printf("MWindow::create_objects 1\n");
	init_edl();
//printf("MWindow::create_objects 1\n");
	init_awindow();
//printf("MWindow::create_objects 1\n");
	init_compositor();
//printf("MWindow::create_objects 1\n");
	init_viewer();
//printf("MWindow::create_objects 1\n");
	init_tuner(channeldb_v4l, "channels_v4l");
//printf("MWindow::create_objects 1\n");
	init_tuner(channeldb_buz, "channels_buz");
//printf("MWindow::create_objects 1\n");
	init_cache();
//printf("MWindow::create_objects 1\n");
	init_indexes();
//printf("MWindow::create_objects 1\n");
	init_gui();
//printf("MWindow::create_objects 1\n");
	init_render();
	mainprogress = new MainProgress(this, gui);
	undo = new MainUndo(this);

	plugin_guis = new ArrayList<PluginServer*>;

//printf("MWindow::create_objects 1\n");
	if(session->show_vwindow) vwindow->gui->show_window();
	if(session->show_cwindow) cwindow->gui->show_window();
	if(session->show_awindow) awindow->gui->show_window();
//printf("MWindow::create_objects 1\n");

	vwindow->start();
	awindow->start();
	cwindow->start();
//printf("MWindow::create_objects 1\n");

	gui->mainmenu->load_defaults(defaults);
	gui->mainmenu->update_toggles();
	gui->patchbay->update();
	gui->canvas->draw();
	gui->cursor->draw();
	gui->raise_window();
	gui->show_window();
	gui->flush();
//printf("MWindow::create_objects 2\n");













return;

// REMOVE
	resize_lock = 0;
	assets = new Assets(edl);


// ================================== output channels before menu

// REMOVE
	tracks = new Tracks(this);
// REMOVE
tracks->canvas = gui->canvas;

// REMOVE
// subwindows are owned not inherited so call create_objects()
	int top = get_top();
	int bottom = get_bottom();

	if(gui)
		tracks->create_objects(defaults, gui->get_w(), gui->get_h(), top, bottom);
	else
		tracks->create_objects(defaults, 0, 0, 0, 0);


// ============================ console
	console = new Console(this);
	console->create_objects(defaults->get("CONSOLEW", 400), defaults->get("CONSOLEH", 181), defaults->get("HIDECONSOLE", 0), defaults->get("CONSOLEVERTICAL", 0));

// ======================= objects which depend on other objects
	timebar->draw();
	gui->zoombar->draw();
	gui->mainmenu->load_defaults(defaults);

// =============================== thread out the level window

	level_window = new LevelWindow(this);
	level_window->load_defaults(defaults);
	level_window->create_objects();
	level_window->start();

// ============================== thread out the video window
	
	video_window = new VideoWindow(this);
	video_window->load_defaults(defaults);
	video_window->create_objects();
	video_window->start();

	threadloader = new ThreadLoader(this);

// ================================== thread out console
	console->start();
}

void MWindow::show_vwindow()
{
	session->show_vwindow = 1;
	vwindow->gui->lock_window();
	vwindow->gui->show_window();
	vwindow->gui->raise_window();
	vwindow->gui->flush();
	vwindow->gui->unlock_window();
	gui->mainmenu->show_vwindow->set_checked(1);
}

void MWindow::show_awindow()
{
	session->show_awindow = 1;
	awindow->gui->lock_window();
	awindow->gui->show_window();
	awindow->gui->raise_window();
	awindow->gui->flush();
	awindow->gui->unlock_window();
	gui->mainmenu->show_awindow->set_checked(1);
}

void MWindow::show_cwindow()
{
	session->show_cwindow = 1;
	cwindow->gui->lock_window();
	cwindow->gui->show_window();
	cwindow->gui->raise_window();
	cwindow->gui->flush();
	cwindow->gui->unlock_window();
	gui->mainmenu->show_cwindow->set_checked(1);
}

void MWindow::tile_windows()
{
	session->default_window_positions();
	gui->default_positions();
}

void MWindow::set_titles(int value)
{
	edl->session->show_titles = value;
	trackmovement(edl->session->track_start);
}

void MWindow::set_auto_keyframes(int value)
{
	gui->lock_window();
	edl->session->auto_keyframes = value;
	gui->mbuttons->edit_panel->keyframe->update(value);
	gui->flush();
	gui->unlock_window();
	cwindow->gui->lock_window();
	cwindow->gui->edit_panel->keyframe->update(value);
	cwindow->gui->flush();
	cwindow->gui->unlock_window();
}

int MWindow::set_editing_mode(int new_editing_mode)
{
	gui->lock_window();
	edl->session->editing_mode = new_editing_mode;
//	gui->mbuttons->edit_panel->delete_buttons();
	gui->mbuttons->edit_panel->editing_mode = edl->session->editing_mode;
	gui->mbuttons->edit_panel->update_toggles(new_editing_mode);
//	gui->mbuttons->edit_panel->create_buttons();
	gui->canvas->update_cursor();
//	gui->flush();
	gui->unlock_window();
	cwindow->gui->lock_window();
//	cwindow->gui->edit_panel->delete_buttons();
	cwindow->gui->edit_panel->update_toggles(new_editing_mode);
	cwindow->gui->edit_panel->editing_mode = edl->session->editing_mode;
//	cwindow->gui->edit_panel->create_buttons();
//	cwindow->gui->flush();
	cwindow->gui->unlock_window();
	return 0;
}

void MWindow::update_caches()
{
	audio_cache->set_edl(edl);
	video_cache->set_edl(edl);
}


int MWindow::load_filenames(ArrayList<char*> *filenames, int load_mode)
{
	ArrayList<EDL*> new_edls;
	ArrayList<Asset*> new_assets;
//printf("load_filenames 1\n");

// Delete current project
	if(load_mode == LOAD_REPLACE || load_mode == LOAD_REPLACE_CONCATENATE)
	{
		edl->save_defaults(defaults);
		delete edl;
		edl = new EDL;
		edl->create_objects();
		edl->load_defaults(defaults);
	}
//printf("MWindow::load_filenames 1 %d\n", edl->session->audio_module_fragment);







// Define new_edls and new_assets to load
	for(int i = 0; i < filenames->total; i++)
	{
// Get type of file
		File *new_file = new File;
		Asset *new_asset = new Asset(filenames->values[i]);
		EDL *new_edl = new EDL;
		char string[BCTEXTLEN];

		new_edl->create_objects();
		new_edl->copy_session(edl);

		sprintf(string, "Loading %s", new_asset->path);
		gui->show_message(string, BLACK);
		int result = new_file->open_file(plugindb, new_asset, 1, 0, 0, 0);
//printf("load_filenames 1\n");

		switch(result)
		{
// Convert media file to EDL
			case FILE_OK:
//printf("load_filenames 1.1\n");
				if(load_mode != LOAD_RESOURCESONLY)
				{
					asset_to_edl(new_edl, new_asset);
//printf("load_filenames 1.2\n");
					new_edls.append(new_edl);
				}
				else
//printf("load_filenames 1.3\n");
				{
					new_assets.append(new_asset);
				}
				result = 0;
				break;

// File not found
			case FILE_NOT_FOUND:
				sprintf(string, "Failed to open %s", new_asset->path);
				gui->show_message(string, RED);
				result = 1;
				break;

// Unknown format
			case FILE_UNRECOGNIZED_CODEC:
			{
// Test index file
				IndexFile indexfile(this);
				result = indexfile.open_index(this, new_asset);
				if(!result)
				{
					indexfile.close_index();
				}

// Test existing EDLs
				if(result)
				{
					for(int j = 0; j < new_edls.total + 1; j++)
					{
						Asset *old_asset;
						if(j == new_edls.total)
						{
							if(old_asset = edl->assets->get_asset(new_asset->path))
							{
								*new_asset = *old_asset;
								result = 0;
							}
						}
						else
						{
							if(old_asset = new_edls.values[j]->assets->get_asset(new_asset->path))
							{
								*new_asset = *old_asset;
								result = 0;
							}
						}
					}
				}

// Prompt user
				if(result)
				{
					char string[BCTEXTLEN];
					FileSystem fs;
					fs.extract_name(string, new_asset->path);

					strcat(string, "'s format couldn't be determined.");
					new_asset->audio_data = 1;
					new_asset->format = FILE_PCM;
					new_asset->channels = defaults->get("AUDIO_CHANNELS", 2);
					new_asset->sample_rate = defaults->get("SAMPLE_RATE", 44100);
					new_asset->bits = defaults->get("AUDIO_BITS", 16);
					new_asset->byte_order = defaults->get("BYTE_ORDER", 1);
					new_asset->signed_ = defaults->get("SIGNED_", 1);
					new_asset->header = defaults->get("HEADER", 0);

					FileFormat fwindow(this);
					fwindow.create_objects(new_asset, string);
					result = fwindow.run_window();

					defaults->update("AUDIO_CHANNELS", new_asset->channels);
					defaults->update("SAMPLE_RATE", new_asset->sample_rate);
					defaults->update("AUDIO_BITS", new_asset->bits);
					defaults->update("BYTE_ORDER", new_asset->byte_order);
					defaults->update("SIGNED_", new_asset->signed_);
					defaults->update("HEADER", new_asset->header);
				}

// Append to list
				if(!result)
				{
// Recalculate length
					delete new_file;
					new_file = new File;
					result = new_file->open_file(plugindb, new_asset, 1, 0, 0, 0);

					if(load_mode != LOAD_RESOURCESONLY)
					{
						asset_to_edl(new_edl, new_asset);
//printf("MWindow::load_files: new_edl\n");
//new_edl->dump();
						new_edls.append(new_edl);
					}
					else
					{
						new_assets.append(new_asset);
					}
				}
				else
				{
					result = 1;
				}
				break;
			}

			case FILE_IS_XML:
			{
				FileXML xml_file;
				xml_file.read_from_file(filenames->values[i]);
				strcpy(session->filename, filenames->values[i]);
				gui->lock_window();
				gui->update_title(session->filename);
				gui->unlock_window();
// Load EDL for pasting
//printf("load_filenames 1\n");
				new_edl->load_xml(plugindb, &xml_file, LOAD_ALL);
				strcpy(new_edl->local_session->clip_title, filenames->values[i]);
//printf("load_filenames 1\n");
//new_edl->dump();
//printf("load_filenames 2\n");
				new_edls.append(new_edl);
//printf("load_filenames 1\n");
				result = 0;
				break;
			}
		}
//printf("load_filenames 1\n");

		if(result)
		{
			delete new_edl;
			delete new_asset;
		}
//printf("load_filenames 2\n");

		delete new_file;
//printf("load_filenames 4\n");
	}
//printf("MWindow::load_filenames 5 %d\n", new_edls.total);


	

	gui->statusbar->default_message();




	if(new_edls.total)
	{
		paste_edls(&new_edls, load_mode);
	}

// Should only happen if LOAD_RESOURCESONLY
	if(new_assets.total)
	{
		for(int i = 0; i < new_assets.total; i++)
			edl->assets->update(new_assets.values[i]);
		update_project();
	}
//printf("MWindow::load_filenames 6\n");

	new_edls.remove_all_objects();
	new_assets.remove_all_objects();
//printf("MWindow::load_filenames 7 %d\n", edl->session->audio_module_fragment);

	return 0;
}

int MWindow::asset_to_edl(EDL *new_edl, Asset *new_asset)
{
//printf("MWindow::asset_to_edl 1\n");
//	new_edl->load_defaults(defaults);

// Keep frame rate and sample rate unchanged
	if(new_asset->video_data)
	{
		new_edl->session->video_tracks = new_asset->layers;
//		new_edl->session->frame_rate = new_asset->frame_rate;
		new_edl->session->track_w = new_asset->width;
		new_edl->session->track_h = new_asset->height;
		new_edl->session->output_w = new_asset->width;
		new_edl->session->output_h = new_asset->height;
	}
	else
		new_edl->session->video_tracks = 0;

	if(new_asset->audio_data)
	{
		new_edl->session->audio_tracks = new_asset->channels;
//		new_edl->session->sample_rate = new_asset->sample_rate;
	}
	else
		new_edl->session->audio_tracks = 0;
//printf("MWindow::asset_to_edl 2\n");

	new_edl->create_default_tracks();
//printf("MWindow::asset_to_edl 3\n");
	new_edl->insert_asset(new_asset, 0);
	
	
	
	
	char string[BCTEXTLEN];
	FileSystem fs;
	fs.extract_name(string, new_asset->path);
	
	strcpy(new_edl->local_session->clip_title, string);
//printf("MWindow::asset_to_edl 4 %s\n", string);

//	new_edl->dump();
	return 0;
}

int MWindow::paste_edls(ArrayList<EDL*> *new_edls, int load_mode)
{
	ArrayList<Track*> destination_tracks;
	int need_new_tracks = 0;

//printf("MWindow::paste_edls 1 %p\n", vwindow->get_edl());

	if(load_mode == LOAD_REPLACE ||
		load_mode == LOAD_REPLACE_CONCATENATE)
	{
		edl->copy_session(new_edls->values[0]);
		gui->mainmenu->update_toggles();
	}

// Assume any paste operation from the same EDL won't contain any clips.
// If it did it would duplicate every clip here.
	for(int i = 0; i < new_edls->total; i++)
	{
		EDL *new_edl = new_edls->values[i];

		for(int j = 0; j < new_edl->clips.total; j++)
		{
			edl->add_clip(new_edl->clips.values[j]);
		}
	}

//printf("MWindow::paste_edls 2 %p\n", vwindow->get_edl());

// Create new tracks in master EDL
	if(load_mode == LOAD_REPLACE || 
		load_mode == LOAD_REPLACE_CONCATENATE ||
		load_mode == LOAD_NEW_TRACKS)
	{
		need_new_tracks = 1;
		for(int i = 0; i < new_edls->total; i++)
		{
			EDL *new_edl = new_edls->values[i];
			for(Track *current = new_edl->tracks->first;
				current;
				current = NEXT)
			{
				if(current->data_type == TRACK_VIDEO)
				{
					edl->tracks->add_video_track();
					destination_tracks.append(edl->tracks->last);
				}
				else
				if(current->data_type == TRACK_AUDIO)
				{
					edl->tracks->add_audio_track();
					destination_tracks.append(edl->tracks->last);
				}
				edl->session->highlighted_track = edl->tracks->total() - 1;
			}

// Base track count on first EDL only for concatenation
			if(load_mode == LOAD_REPLACE_CONCATENATE) break;
		}
	}
	else
// Recycle existing tracks of master EDL
	if(load_mode == LOAD_CONCATENATE || load_mode == LOAD_PASTE)
	{
		for(Track *current = edl->tracks->first; current; current = NEXT)
		{
			if(current->record)
			{
				destination_tracks.append(current);
			}
		}
	}

//printf("MWindow::paste_edls 1 %p\n", vwindow->get_edl());
	int destination_track = 0;
// Iterate through the edls
	for(int i = 0; i < new_edls->total; i++)
	{
		EDL *new_edl = new_edls->values[i];
		long current_sample = 0;

//printf("MWindow::paste_edls 1\n");
// Prepare index files
		for(Asset *new_asset = new_edl->assets->first;
			new_asset;
			new_asset = new_asset->next)
		{
			mainindexes->add_next_asset(new_asset);
		}
//printf("MWindow::paste_edls 1 %d %d\n", load_mode, new_edl->assets->first->format);

// Get starting point
		if(load_mode == LOAD_REPLACE ||
			load_mode == LOAD_NEW_TRACKS)
		{
		   	current_sample = 0;
		}
		else
		if(load_mode == LOAD_CONCATENATE ||
			load_mode == LOAD_REPLACE_CONCATENATE)
		{
			destination_track = 0;
			current_sample = (long)(edl->tracks->total_samples() + 0.5);
		}
		else
		if(load_mode == LOAD_PASTE)
		{
			destination_track = 0;
			current_sample = edl->local_session->selectionstart;
		}
		else
		if(load_mode == LOAD_RESOURCESONLY)
		{
			edl->add_clip(new_edl);
		}

		if(load_mode != LOAD_RESOURCESONLY)
		{
// Insert labels
			edl->labels->insert_labels(new_edl->labels, current_sample);

			for(Track *new_track = new_edl->tracks->first; 
				new_track; 
				new_track = new_track->next)
			{
// Get destination track of same type as new_track
				for(int k = 0; 
					k < destination_tracks.total &&
					destination_tracks.values[destination_track]->data_type != new_track->data_type;
					k++, destination_track++)
				{
					if(destination_track >= destination_tracks.total - 1)
						destination_track = 0;
				}

// Insert data into destination track
				if(destination_track < destination_tracks.total &&
					destination_tracks.values[destination_track]->data_type == new_track->data_type)
				{
					Track *track = destination_tracks.values[destination_track];
// Replace default keyframes if first EDL and new tracks were created
					int replace_default = (i == 0) && need_new_tracks;

// Insert new track at current position
					track->insert_track(new_track, 
						current_sample, 
						replace_default);
				}



// Get next destination track
				destination_track++;
				if(destination_track >= destination_tracks.total)
					destination_track = 0;
			}
		}
//printf("MWindow::paste_edls 2 %d\n", new_edl->assets->first->format);
	}

//printf("MWindow::paste_edls 2 %p\n", vwindow->get_edl());

//edl->dump();
	update_project();

//printf("MWindow::paste_edls 3 %p\n", vwindow->get_edl());

// Start examining next batch of index files
	mainindexes->start_build();
//printf("MWindow::paste_edls 5\n");
	return 0;
}

// Reset everything for a load
void MWindow::update_project()
{
//printf("MWindow::update_project 1\n");
	edl->tracks->update_y_pixels(theme);
//printf("MWindow::update_project 2\n");
// Draw timeline
	update_caches();
//printf("MWindow::update_project 3\n");
	gui->get_scrollbars();
//printf("MWindow::update_project 4\n");
// Draw after getting scrollbars to avoid double drawing
	gui->canvas->draw();
//printf("MWindow::update_project 5\n");
	gui->patchbay->update();
//printf("MWindow::update_project 6\n");
	gui->timebar->update();
//printf("MWindow::update_project 7\n");
	gui->canvas->activate();







//printf("MWindow::update_project 8\n");
//	vwindow->gui->update_sources("None");
//	vwindow->change_source((EDL*)0);
	vwindow->change_source(edl->session->vwindow_folder, 
		edl->session->vwindow_source);
//printf("MWindow::update_project 9\n");







	cwindow->gui->slider->set_position();
//printf("MWindow::update_project 10\n");
	cwindow->playback_engine->que->send_command(CURRENT_FRAME, 
		CHANGE_ALL,
		edl,
		1);
//printf("MWindow::update_project 11\n");
	gui->cursor->show();
	gui->canvas->flash();

	awindow->gui->lock_window();
	awindow->gui->update_assets();
	awindow->gui->flush();
	awindow->gui->unlock_window();
//printf("MWindow::update_project 12\n");
	gui->flush();
//printf("MWindow::update_project 13\n");
}

void MWindow::rebuild_index(Asset *asset)
{
	char source_filename[BCTEXTLEN], index_filename[BCTEXTLEN];
	IndexFile::get_index_filename(source_filename, 
		preferences->index_directory,
		index_filename, 
		asset->path);
	remove(index_filename);
	asset->index_status = INDEX_NOTTESTED;
	mainindexes->add_next_asset(asset);
	mainindexes->start_build();
}

int MWindow::create_aspect_ratio(float &w, float &h, int width, int height)
{
	int denominator;
	float fraction = (float)width / height;

	for(denominator = 1; 
		denominator < 100 && 
			fabs(fraction * denominator - (int)(fraction * denominator)) > .001; 
		denominator++)
		;

	w = denominator * width / height;
	h = denominator;
}

void MWindow::render_single()
{
	if(!render->running())
		render->start();
}

void MWindow::render_list()
{
	if(!renderlist->running())
		renderlist->start();
}

void MWindow::dump_plugins()
{
	for(int i = 0; i < plugindb->total; i++)
	{
		printf("%25s audio: %d video: %d realtime: %d transition: %d theme: %d\n",
			plugindb->values[i]->title,
			plugindb->values[i]->audio,
			plugindb->values[i]->video,
			plugindb->values[i]->realtime,
			plugindb->values[i]->transition,
			plugindb->values[i]->theme);
	}
}

























int MWindow::save_defaults()
{
//printf("MWindow::save_defaults 1\n");
	gui->save_defaults(defaults);
	edl->save_defaults(defaults);
//printf("MWindow::save_defaults 1\n");
	session->save_defaults(defaults);
	preferences->save_defaults(defaults);
//printf("MWindow::save_defaults 1\n");

	save_tuner(channeldb_v4l, "channels_v4l");
	save_tuner(channeldb_buz, "channels_buz");
//printf("MWindow::save_defaults 1\n");
	defaults->save();
//printf("MWindow::save_defaults 2\n");
	return 0;
}

int MWindow::run_script(FileXML *script)
{
	int result = 0, result2 = 0;
	while(!result && !result2)
	{
		result = script->read_tag();
		if(!result)
		{
			if(script->tag.title_is("new_project"))
			{
// Run new in immediate mode.
				gui->mainmenu->new_project->run_script(script);
			}
			else
			if(script->tag.title_is("record"))
			{
// Run record as a thread.  It is a terminal command.
				;
// Will read the complete scipt file without letting record read it if not
// terminated.
				result2 = 1;
			}
			else
			{
				printf("MWindow::run_script: Unrecognized command: %s\n",script->tag.get_title() );
			}
		}
	}
	return result2;
}

// ================================= synchronization

int MWindow::lock_resize() { resize_lock = 1; return 0; }

int MWindow::unlock_resize() { resize_lock = 0; return 0; }


// =============================== file operations

// REMOVE
int MWindow::load(FileXML *xml, int import_,
		int edits_only,
		int patches_only,
		int console_only,
		int timebar_only,
		int automation_only)
{

	return 0;
}

// REMOVE
int MWindow::load_video_config(FileXML *xml, int import_)
{
	if(!import_) 
	{
// 		session->frame_rate = xml->tag.get_property("FRAMERATE", session->frame_rate);
// 		preferences->frames_per_foot = xml->tag.get_property("FRAMES_PER_FOOT", preferences->frames_per_foot);
// 		session->track_w = xml->tag.get_property("TRACKW", session->track_w);
// 		session->track_h = xml->tag.get_property("TRACKH", session->track_h);
// 		session->output_w = xml->tag.get_property("OUTPUTW", session->output_w);
// 		session->output_h = xml->tag.get_property("OUTPUTH", session->output_h);
// 		session->aspect_w = xml->tag.get_property("ASPECTW", session->aspect_w);
// 		session->aspect_h = xml->tag.get_property("ASPECTH", session->aspect_h);
// 		video_window->resize_window();
	}
	return 0;
}

// REMOVE
int MWindow::load_audio_config(FileXML *xml, int import_,
		int edits_only, 
		int patches_only,
		int console_only,
		int automation_only)
{
	if(!import_)
	{
		if(edits_only + patches_only + console_only + automation_only == 0)
		{
// load channels setting
			long new_channels;

			new_channels = xml->tag.get_property("OUTCHANNELS", (long)session->audio_channels);
			if(new_channels != session->audio_channels) change_channels(session->audio_channels, new_channels);	
			
			char string1[32];
			
			for(int i = 0; i < session->audio_channels; i++)
			{
				sprintf(string1, "CHPOSITION%d", i);
				session->achannel_positions[i] = xml->tag.get_property(string1, session->achannel_positions[i]);
			}
		}
	
		session->sample_rate = xml->tag.get_property("SAMPLERATE", (long)session->sample_rate);
	}
	return 0; 
}

// REMOVE
int MWindow::load_edits(FileXML *xml)
{
	load(xml, 0, 1);
	return 0; 
}

// REMOVE
int MWindow::load_patches(FileXML *xml)
{
	load(xml, 0, 0, 1);
	return 0; 
}

// REMOVE
int MWindow::load_console(FileXML *xml)	
{
	load(xml, 0, 0, 0, 1);
	return 0; 
}

// REMOVE
int MWindow::load_timebar(FileXML *xml)	
{
	load(xml, 0, 0, 0, 0, 1);
	return 0; 
}

// REMOVE
int MWindow::load_automation(FileXML *xml)
{
	load(xml, 0, 0, 0, 0, 0, 1);
	return 0; 
}

int MWindow::interrupt_indexes()
{
//	assets->interrupt_indexes();
//	gui->mainmenu->asset_manager->interrupt_indexes();
	mainindexes->interrupt_build();
	return 0; 
}





int MWindow::set_filename(char *filename)
{
	strcpy(session->filename, filename);
	if(gui)
	{
		if(filename[0] == 0)
		{
			gui->set_title(PROGRAM_NAME);
		}
		else
		{
			FileSystem dir;
			char string[1024], string2[1024];
			dir.extract_name(string, filename);
			sprintf(string2, PROGRAM_NAME ": %s\n", string);
			gui->set_title(string2);
		}
	}
	return 0; 
}

// ========================================== drawing

int MWindow::draw()
{
	if(gui)
	{
		gui->zoombar->draw();
		tracks->draw();
		gui->samplescroll->set_position();
		timebar->draw();

		tracks->show_overlays(1);
	}
	return 0; 
}


// ======================================= cursors



int MWindow::draw_floating_handle(int flash) 
{ 
	tracks->draw_floating_handle(flash); 
	return 0; 
}







// ============================================ selecting

int MWindow::set_selectionend(long new_position)
{
	return 0; 
}

int MWindow::set_selectionstart(long new_position)
{
	return 0; 
}

int MWindow::set_selection(long selectionstart, long selectionend)
{
	return 0; 
}

int MWindow::get_affected_range(long *start, long *end, int reverse)
{
	return 0; 
}


int MWindow::set_loop_boundaries()
{
	double start, end;
	
	if(edl->local_session->selectionstart != edl->local_session->selectionend) 
	{
		start = edl->local_session->selectionstart;
		end = edl->local_session->selectionend;
	}
	else
	if(edl->tracks->total_samples())
	{
		start = 0;
		end = edl->tracks->total_samples();
	}
	else
	{
		start = end = 0;
	}

	if(!edl->session->loop_playback && start != end)
	{
		edl->session->loop_start = (long)(start + 0.5);
		edl->session->loop_end = (long)(end + 0.5);
		edl->session->loop_playback = 1;
	}
	else
	if(edl->session->loop_playback)
	{
		edl->session->loop_playback = 0;
	}

	gui->canvas->draw_overlays();
	gui->canvas->flash();
	sync_parameters(CHANGE_PARAMS);
	return 0; 
}


// ======================================= playback

int MWindow::update_playback_cursor(long new_position, int view_follows_playback)
{
	if(!gui) return 1;
	
	long cursor_x1, cursor_x2;
	long view_end;
	
	gui->lock_window();
	
	
	view_end = session->view_start + tracks->view_samples();
//	cursor_x1 = (session->last_playback_position - session->view_start) / session->zoom_sample;
	cursor_x2 = (new_position - session->view_start) / session->zoom_sample;
	if(cursor_x1 != cursor_x2)
	{
// recalculate in case of window movement
		view_end = session->view_start + tracks->view_samples();
//		cursor_x1 = (session->last_playback_position - session->view_start) / session->zoom_sample;
		cursor_x2 = (new_position - session->view_start) / session->zoom_sample;

		if(new_position >= session->view_start && 
			new_position <= view_end)
			tracks->draw_playback_cursor(cursor_x2);

// 		if(session->playback_cursor_visible && 
// 			session->last_playback_position >= session->view_start && 
// 			session->last_playback_position <= view_end) 
// 			tracks->draw_playback_cursor(cursor_x1, 1);
// 
//		session->playback_cursor_visible = 1;

//		session->last_playback_position = new_position;
		
// 		if(view_follows_playback && !gui->samplescroll->in_use())
// 		{
// 			if(playback_engine->reverse)
// 			{
// 				if(cursor_x2 < 0 &&
// 					cursor_x2 > -tracks->view_pixels() / 4 &&
// 					((!session->loop_playback && playback_engine->playback_start < session->view_start) ||
// 					(session->loop_playback && session->loop_start > session->view_start)))
// 				{
// 					move_left(session->zoom_sample * tracks->view_pixels());
// 				}
// 			}
// 			else
// 			{
// 				if(cursor_x2 > tracks->view_pixels() &&
// 					cursor_x2 < tracks->view_pixels() + tracks->view_pixels() / 4 &&
// 					((!session->loop_playback && playback_engine->playback_end > view_end) ||
// 					(session->loop_playback && session->loop_end > view_end)))
// 				{
// 					move_right(session->zoom_sample * tracks->view_pixels());
// 				}
// 			}
// 		}
	}
	
	gui->zoombar->update_playback(new_position);
	gui->unlock_window();
	return 0; 
}

int MWindow::show_playback_cursor(long position, int flash)
{
// 	if(playback_engine->is_playing_back)
// 	{
// 		session->playback_cursor_visible = 1;
// 		if(position == -1) position = session->last_playback_position;
// 
// 		long cursor_x1 = (position - session->view_start) / session->zoom_sample;
// 		if(position >= session->view_start && 
// 			position <= session->view_start + tracks->view_samples())
// 		{
// 			tracks->draw_playback_cursor(cursor_x1, flash);
// 		}
// 		session->last_playback_position = position;
// 	}
	return 0;
}

int MWindow::hide_playback_cursor(int flash)
{
// 	if(session->playback_cursor_visible)
// 	{
// 		session->playback_cursor_visible = 0;
// 		long cursor_x1 = (session->last_playback_position - session->view_start) / session->zoom_sample;
// 		if(session->last_playback_position >= session->view_start && 
// 			session->last_playback_position <= session->view_start + tracks->view_samples())
// 		{
// 			tracks->draw_playback_cursor(cursor_x1, flash);
// 		}
// 	}
	return 0; 
}

int MWindow::set_playback_range(long start_position, int reverse, float speed)
{
	long start, end;

//	playback_engine->reset_parameters();
	get_affected_range(&start, &end, reverse);

//printf("MWindow::set_playback_range %ld %ld %f\n", start, end, speed);
// use default
	if(start_position == -1)  
	{
//		if(reverse) 
//			playback_engine->set_range(start, end, end, reverse, speed);      // start at the end
//		else 
//			playback_engine->set_range(start, end, start, reverse, speed);     // start at the start
	}
	else
// use current position
	{
//		if(reverse)
//		{
//			playback_engine->set_range(start, start_position, start_position, reverse, speed);
//		}
//		else
//		{
//			playback_engine->set_range(start_position, end, start_position, reverse, speed);
//		}
	}
	return 0; 
}

int MWindow::arm_playback(int follow_loop, 
					int use_buttons, 
					int infinite, 
					AudioDevice *audio)
{
// 	if(!session->is_playing_back)
// 	{
// 		session->is_playing_back = 1;
// //printf("MWindow::arm_playback 1\n");
// 		playback_engine->arm_playback(follow_loop, 
// 					use_buttons, 
// 					infinite, 
// 					audio);
// //printf("MWindow::arm_playback 2\n");
// 	}
	return 0; 
}



int MWindow::start_playback() 
{ 
//	playback_engine->start_playback(); 
	return 0; 
}

int MWindow::stop_playback(int update_button) 
{
// 	session->is_playing_back = 0;
// // need cursor to finish so unlock
// 	if(gui) gui->unlock_window();
// 	playback_engine->stop_playback(update_button); 
// 	if(gui) gui->lock_window();
	return 0; 
}

int MWindow::wait_for_playback()
{
//	if(session->is_playing_back) playback_engine->wait_for_completion();
	return 0; 
}




long MWindow::get_playback_position() 
{
// Want the position rendered
//	return playback_engine->get_position(0);
}

int MWindow::start_reconfigure(int unlock_window)
{
// 	if(session->is_playing_back)
// 	{
// // call stop_playback here because calling from console screws up window locking
// 		if(unlock_window && gui) gui->unlock_window();
// 		playback_engine->start_reconfigure(); 
// 	}
	return 0; 
}

int MWindow::stop_reconfigure(int unlock_window)
{
// 	if(session->is_playing_back)
// 	{
// 		playback_engine->stop_reconfigure();		
// 		if(unlock_window && gui) gui->lock_window();
// 	}
	return 0; 
}


int MWindow::reset_meters()
{
	cwindow->gui->lock_window();
	cwindow->gui->meters->reset_meters();
	cwindow->gui->unlock_window();

	vwindow->gui->lock_window();
	vwindow->gui->meters->reset_meters();
	vwindow->gui->unlock_window();

	gui->lock_window();
	gui->patchbay->reset_meters();
	gui->unlock_window();
	return 0; 
}


// ======================================= window sizes


int MWindow::get_top() 
{ 
	if(gui)
	{
		if(session->tracks_vertical)
			return gui->get_w() - 18 - 17;     // timebar + scrollbar
		else
			return timebar->gui->get_y() + timebar->gui->get_h();
	}
	else
	return 0;
}

int MWindow::get_bottom() 
{  
	if(gui)
	{
		if(session->tracks_vertical)
			return BUTTONBARWIDTH;         // mbuttons
		else
			return gui->get_h() - 24 - 17;      // zoombar + scrollbar
	}
	else
	return 0;
}
