#include "assets.h"
#include "awindow.h"
#include "awindowgui.h"
#include "console.h"
#include "cplayback.h"
#include "cwindow.h"
#include "edl.h"
#include "edlsession.h"
#include "filexml.h"
#include "keyframe.h"
#include "labels.h"
#include "levelwindow.h"
#include "localsession.h"
#include "mainclock.h"
#include "mainmenu.h"
#include "mainundo.h"
#include "mwindow.h"
#include "mwindowgui.h"
#include "patchbay.h"
#include "playbackengine.h"
#include "mainsession.h"
#include "samplescroll.h"
#include "trackcanvas.h"
#include "zoombar.h"
#include "timebar.h"
#include "track.h"
#include "tracks.h"
#include "trackscroll.h"
#include "transportque.h"
#include "units.h"
#include "vwindow.h"



#include <string.h>


void MWindow::set_inpoint(double position)
{
//printf("MWindow::set_inpoint 1\n");
	edl->set_inpoint(position);
//printf("MWindow::set_inpoint 2\n");
	gui->timebar->update();
//printf("MWindow::set_inpoint 3\n");
	gui->flush();
//printf("MWindow::set_inpoint 4\n");
}

void MWindow::set_outpoint(double position)
{
	edl->set_outpoint(position);
	gui->timebar->update();
	gui->flush();
}

void MWindow::delete_inpoint()
{
	edl->local_session->in_point = -1;
}

void MWindow::delete_outpoint()
{
	edl->local_session->out_point = -1;
}

void MWindow::new_folder(char *new_folder)
{
	undo->update_undo_before("new folder", LOAD_ALL);
	edl->new_folder(new_folder);
	undo->update_undo_after();
	awindow->gui->lock_window();
	awindow->gui->update_assets();
	awindow->gui->unlock_window();
}

void MWindow::delete_folder(char *folder)
{
	undo->update_undo_before("new folder", LOAD_ALL);
	undo->update_undo_after();
}

int MWindow::init_selection(long cursor_position, 
				int cursor_x, 
				int cursor_y, 
				int &current_end, 
				long &selection_midpoint1,
				long &selection_midpoint2,
				int &selection_type)
{
	return 0;
}

int MWindow::update_selection(long cursor_position,
				int cursor_x, 
				int cursor_y, 
				int &current_end, 
				long selection_midpoint1,
				long selection_midpoint2,
				int selection_type)
{
}


int MWindow::init_handle_selection(long cursor_position, int handle_pixel, int which_handle)
{
}

void MWindow::select_point(long sample)
{
	edl->local_session->selectionstart = edl->local_session->selectionend = sample;
// Que the CWindow
	cwindow->update_position();
	gui->cursor->hide();
	gui->cursor->draw();
	gui->mainclock->update(edl->local_session->selectionstart);
	gui->zoombar->update();
	gui->canvas->flash();
	gui->flush();
}

int MWindow::add_audio_track()
{
	edl->tracks->add_audio_track();
	edl->tracks->update_y_pixels(theme);
	return 0;
}

int MWindow::add_video_track()
{
	edl->tracks->add_video_track();
	edl->tracks->update_y_pixels(theme);
	return 0;
}

int MWindow::delete_project(int flash)
{
	if(gui) tracks->hide_overlays(0);
	tracks->delete_all(flash);
	patches->delete_all();
	timebar->delete_project();
	console->delete_project();
	assets->delete_all();
	if(gui)
	{
		gui->trackscroll->update();
		gui->samplescroll->set_position();
		tracks->show_overlays(0); // don't flash since this is an intermediate step
	}
}

void MWindow::lift_selection()
{
	long start = -1;
	long end = -1;

printf("MWindow::lift_selection 1\n");
	if(edl->session->editing_mode == EDITING_ARROW)
	{
		start = edl->local_session->in_point;
		end = edl->local_session->out_point;
		if(start < 0) start = 0;
		if(end < 0) end = (long)(edl->tracks->total_samples() + 0.5);
	}
	else
	if(edl->local_session->selectionstart < edl->local_session->selectionend)
	{
		start = edl->local_session->selectionstart;
		end = edl->local_session->selectionend;
	}

	if(edl->session->cursor_on_frames)
	{
		start = edl->align_to_frame(start, 0);
		end = edl->align_to_frame(end, 1);
	}

printf("MWindow::lift_selection 2\n");
	if(start != end)
	{
		copy(start, end);
printf("MWindow::lift_selection 3\n");
		clear(start, end);
printf("MWindow::lift_selection 4\n");
		paste_silence(start, end);
printf("MWindow::lift_selection 5\n");
	}
}

void MWindow::extract_selection()
{
	long start = -1;
	long end = -1;
	if(edl->session->editing_mode == EDITING_ARROW)
	{
		start = edl->local_session->in_point;
		end = edl->local_session->out_point;
		if(start < 0) start = 0;
		if(end < 0) end = (long)(edl->tracks->total_samples() + 0.5);
	}
	else
	if(edl->local_session->selectionstart < edl->local_session->selectionend)
	{
		start = edl->local_session->selectionstart;
		end = edl->local_session->selectionend;
	}

	start = edl->align_to_frame(start, 0);
	end = edl->align_to_frame(end, 1);

	if(start != end)
	{
		copy(start, end);
		clear(start, end);
	}
}

int MWindow::cut(double start, double end)
{
//printf("MWindow::cut 1\n");
	copy(start, end);
//printf("MWindow::cut 1\n");
	clear(start, end);
//printf("MWindow::cut 2\n");
	return 0;
}

int MWindow::copy(double start, double end)
{
	if(start == end) return 1;

//printf("MWindow::copy 1\n");
	FileXML file;
//printf("MWindow::copy 1\n");
	edl->copy(start, 
		end, 
		0,
		&file, 
		this->plugindb,
		"",
		1);
//printf("MWindow::copy 1\n");

// File is now terminated and rewound

//printf("MWindow::copy 1\n");
	gui->get_clipboard()->to_clipboard(file.string, strlen(file.string), SECONDARY_SELECTION);
//printf("MWindow::copy\n%s\n", file.string);
//printf("MWindow::copy 2\n");
	return 0;
}

int MWindow::paste(double start, double end, FileXML *file)
{
//printf("MWindow::paste 1 %p\n", vwindow->get_edl());
	if(start != end)
	{
		clear(start, end);
	}
//printf("MWindow::paste 1 %p\n", vwindow->get_edl());

	insert(start, file);
//printf("MWindow::paste 2 %p\n", vwindow->get_edl());

	return 0;
}

// Insert data from clipboard
void MWindow::insert(double position, FileXML *file)
{
	EDL edl(this->edl);
	ArrayList<EDL*> new_edls;
	unsigned long load_flags = LOAD_ALL;

	new_edls.append(&edl);
	edl.create_objects();



//printf("MWindow::insert 1\n");
	
	if(!edl.session->autos_follow_edits) load_flags &= ~LOAD_AUTOMATION;
//printf("MWindow::insert 2 %p\n", vwindow->get_edl());
	if(!edl.session->labels_follow_edits) load_flags &= ~LOAD_TIMEBAR;
//printf("MWindow::insert 3 %p\n", vwindow->get_edl());
	edl.load_xml(plugindb, file, load_flags);





//edl.dump();
//printf("MWindow::insert 4 %p\n", vwindow->get_edl());
	paste_edls(&new_edls, LOAD_PASTE);
//printf("MWindow::insert 5 %p\n", vwindow->get_edl());
	new_edls.remove_all();
//printf("MWindow::insert 6 %p\n", vwindow->get_edl());
}

int MWindow::clear(double start, double end, int clear_labels)
{
	edl->clear(start, end, clear_labels);
	return 0;
}

int MWindow::clear_labels(long start, long end)
{
	edl->labels->clear(start, end, 0);
	return 0;
}


void MWindow::insert_effect(char *title, 
	SharedLocation *shared_location, 
	Track *track,
	int plugin_type)
{
	KeyFrame *default_keyframe = 0;
	PluginServer *server = 0;

//printf("MWindow::insert_effect 1\n");
	if(plugin_type == PLUGIN_STANDALONE)
	{
// Get default keyframe
		default_keyframe = new KeyFrame;
		server = new PluginServer(*scan_plugindb(title));

		server->open_plugin(0, edl, 0);
		server->save_data(default_keyframe);
	}
printf("MWindow::insert_effect 1 %p\n", track);

// Insert plugin object
	track->insert_effect(title, 
		shared_location, 
		default_keyframe, 
		plugin_type);
	
printf("MWindow::insert_effect 1\n");
	if(plugin_type == PLUGIN_STANDALONE)
	{
		server->close_plugin();
		delete server;
		delete default_keyframe;
	}
printf("MWindow::insert_effect 1\n");

edl->dump();
printf("MWindow::insert_effect 2\n");
	cwindow->playback_engine->que->send_command(CURRENT_FRAME, 
		CHANGE_EDL,
		edl,
		1);
printf("MWindow::insert_effect 3\n");
}

void MWindow::sync_parameters(int change_type)
{
// TODO: block keyframes until synchronization is done

// Sync engines which are playing back
	if(cwindow->playback_engine->is_playing_back)
	{
		if(change_type == CHANGE_PARAMS)
		{
			cwindow->playback_engine->sync_parameters(edl);
		}
		else
// Stop and restart
		{
		}
	}
	else
	{
//printf("MWindow::sync_parameters 1\n");
		cwindow->playback_engine->que->send_command(CURRENT_FRAME, 
							change_type,
							edl,
							1);
//printf("MWindow::sync_parameters 2\n");
	}
}


int MWindow::paste_output(long startproject, 
				long endproject, 
				long startsource_sample, 
				long endsource_sample, 
				long startsource_frame,
				long endsource_frame,
				Asset *asset, 
				RecordLabels *new_labels)
{
// Sanity check
// 	if(startsource_frame < 0) startsource_frame = 0;
// 
// 	purge_asset(asset->path);
// 	Asset *new_asset = assets->update(asset);
// 	tracks->paste_output(startproject, 
// 						endproject, 
// 						startsource_sample, 
// 						endsource_sample, 
// 						startsource_frame, 
// 						endsource_frame, 
// 						new_asset);
// 
// 	if(new_labels)
// 	{
// 		timebar->paste_output(startproject, 
// 						endproject, 
// 						startsource_sample, 
// 						endsource_sample, 
// 						new_labels);
// 	}
// 
// 	if(gui)
// 	{
// 		draw();
// 	}
// 
// 	if(asset->audio_data)
// 	{
// 		set_local_session->selectionend(startproject + endsource_sample - startsource_sample);
// 		if(endsource_sample > startsource_sample) assets->build_indexes();    // create if necessary and draw indexes
// 	}
// 	else 
// 	if(asset->video_data) set_local_session->selectionend(startproject + 
// 		Units::tosamples(endsource_frame, session->sample_rate, session->frame_rate) - 
// 		Units::tosamples(startsource_frame, session->sample_rate, session->frame_rate));
	return 0;
}

int MWindow::paste_transition(long startproject, 
				long endproject, 
				Transition *transition)
{
	return 0;
}



int MWindow::feather_edits(long feather_samples, int audio, int video)
{
	return 0;
}

long MWindow::get_feather(int audio, int video) 
{ 
	return 0;
}

float MWindow::get_aspect_ratio()
{
	return 0;
}

int MWindow::mute_selection(long start, long end)
{
	if(start != end)
	{
		clear(start, end, 0);
		edl->local_session->selectionend = end;
		edl->local_session->selectionstart = start;
		paste_silence(start, end, 0);
		edl->session->changes_made = 1;	
	}
	return 0;
}

int MWindow::trim_selection(long start, long end)
{
	if(start != end)
	{
// clear the data
		clear(0, start);
		clear(end - start, (long)(edl->tracks->total_samples() + 0.5));
		edl->session->changes_made = 1;	
	}
	return 0;
}

int MWindow::paste_silence(long start, long end, int clear_labels)
{
	if(start != end)
	{
		edl->paste_silence(start, end, clear_labels);
	}
	return 0;
}

int MWindow::cut_automation()
{
	copy_automation();
	clear_automation();
}

int MWindow::copy_automation()
{
	return 0;
}

int MWindow::paste_automation()
{
	return 0;
}

int MWindow::clear_automation()
{
	return 0;
}

void MWindow::delete_tracks()
{
	undo->update_undo_before("delete tracks", LOAD_ALL);
//printf("MWindow::delete_tracks 1\n");
	edl->tracks->delete_tracks();
//printf("MWindow::delete_tracks 2\n");
	undo->update_undo_after();

//printf("MWindow::delete_tracks 3\n");
	gui->update(1, 1, 0, 0, 1, 0);
//printf("MWindow::delete_tracks 4\n");
	cwindow->playback_engine->que->send_command(CURRENT_FRAME, 
	    		   CHANGE_EDL,
	    		   edl,
	    		   1);
//printf("MWindow::delete_tracks 5\n");
}

void MWindow::delete_track()
{
	undo->update_undo_before("delete track", LOAD_ALL);
	edl->tracks->delete_track();
	undo->update_undo_after();

	gui->update(1, 1, 0, 0, 1, 0);
	cwindow->playback_engine->que->send_command(CURRENT_FRAME, 
	    		   CHANGE_EDL,
	    		   edl,
	    		   1);
}

void MWindow::move_tracks_up()
{
	undo->update_undo_before("move tracks up", LOAD_ALL);
	edl->tracks->move_tracks_up();
	undo->update_undo_after();

	gui->update(1, 1, 0, 0, 1, 0);
	cwindow->playback_engine->que->send_command(CURRENT_FRAME, 
		CHANGE_EDL,
		edl,
		1);
}

void MWindow::move_tracks_down()
{
	undo->update_undo_before("move tracks down", LOAD_ALL);
	edl->tracks->move_tracks_down();
	undo->update_undo_after();

	gui->update(1, 1, 0, 0, 1, 0);
	cwindow->playback_engine->que->send_command(CURRENT_FRAME, 
		CHANGE_EDL,
		edl,
		1);
}


void MWindow::concatenate_tracks()
{
	undo->update_undo_before("move tracks down", LOAD_EDITS);
	edl->tracks->concatenate_tracks();
	undo->update_undo_after();

	gui->update(1, 1, 0, 0, 1, 0);
	cwindow->playback_engine->que->send_command(CURRENT_FRAME, 
		CHANGE_EDL,
		edl,
		1);
}



int MWindow::toggle_label()
{
	long position1, position2;

	edl->session->changes_made = 1;
	undo->update_undo_before("Label", LOAD_TIMEBAR);

	if(cwindow->playback_engine->is_playing_back) 
	{
//		position1 = position2 = cwindow->playback_engine->get_tracking_position();
	}
	else
	{
		position1 = edl->local_session->selectionstart;
		position2 = edl->local_session->selectionend;
	}

	if(edl->session->cursor_on_frames)
	{
		position1 = (long)((double)position1 / edl->session->sample_rate * edl->session->frame_rate + 0.5);
		position1 = (long)((double)position1 / edl->session->frame_rate * edl->session->sample_rate + 0.5);
		position2 = (long)((double)position2 / edl->session->sample_rate * edl->session->frame_rate + 0.5);
		position2 = (long)((double)position2 / edl->session->frame_rate * edl->session->sample_rate + 0.5);
	}
//printf("MWindow::toggle_label %d %d\n", edl->local_session->selectionstart, position1);

	edl->labels->toggle_label(position1, position2);
	gui->timebar->update();
	gui->flush();

	undo->update_undo_after();
	return 0;
}

int MWindow::modify_handles(long oldposition, 
	long newposition, 
	int currentend, 
	int handle_mode)
{
	edl->modify_handles(oldposition, 
		newposition, 
		currentend, 
		handle_mode);
	return 0;
}

int MWindow::change_channels(int old_channels, int new_channels)
{
	if(gui) gui->mainmenu->change_channels(old_channels, new_channels);
	tracks->change_channels(old_channels, new_channels);
	if(gui) console->gui->lock_window();
	console->change_channels(old_channels, new_channels);
	session->audio_channels = new_channels;
	if(gui) console->gui->unlock_window();
	
	if(gui) level_window->change_channels(new_channels);
}

void MWindow::crop_video()
{
	
}



int MWindow::fix_timing(long &samples_out, long &frames_out, long samples_in)
{
	frames_out = (long)((float)samples_in / session->sample_rate * session->frame_rate);
	samples_out = (long)((float)frames_out / session->frame_rate * session->sample_rate);
}

int MWindow::purge_asset(char *path)
{
	Asset *asset = assets->get_asset(path);

	if(asset)
	{
//		cache->delete_entry(asset);
		int result = tracks->purge_asset(asset);
		delete asset;



		if(result) draw();
	}
}

int MWindow::optimize_assets()
{
	Asset *asset = assets->first;
	Asset *next;
	
	while(asset)
	{
		next = asset->next;
		
		if(!tracks->asset_used(asset))
		{
//			cache->delete_entry(asset);
			tracks->purge_asset(asset);
			delete asset;
		}
		asset = next;
	}
}
