#include "cache.h"
#include "defaults.h"
#include "edl.h"
#include "edlsession.h"
#include "mbuttons.h"
#include "mwindow.h"
#include "mwindowgui.h"
#include "modules.h"
#include "patchbay.h"
#include "tracking.h"
#include "playbackengine.h"
#include "playtransport.h"
#include "preferences.h"
#include "renderengine.h"
#include "mainsession.h"
#include "trackcanvas.h"
#include "transportque.h"
#include "videodevice.h"
#include "vrender.h"


PlaybackEngine::PlaybackEngine(MWindow *mwindow, Canvas *output)
 : Thread()
{
	this->mwindow = mwindow;
	this->output = output;
	is_playing_back = 0;
	tracking_position = 0;
	tracking_active = 0;
	audio_cache = 0;
	video_cache = 0;
	last_command = STOP;
	Thread::set_synchronous(1);
}

PlaybackEngine::~PlaybackEngine()
{
//printf("PlaybackEngine::~PlaybackEngine 1\n");
	done = 1;
	que->send_command(STOP,
		CHANGE_NONE, 
		0,
		0);
	interrupt_playback();
//printf("PlaybackEngine::~PlaybackEngine 1\n");
	Thread::join();
//printf("PlaybackEngine::~PlaybackEngine 1\n");
	delete preferences;
//printf("PlaybackEngine::~PlaybackEngine 1\n");
	delete command;
//printf("PlaybackEngine::~PlaybackEngine 1\n");
	delete que;
//printf("PlaybackEngine::~PlaybackEngine 1\n");
	delete_render_engines();
//printf("PlaybackEngine::~PlaybackEngine 1\n");
	if(audio_cache) delete audio_cache;
//printf("PlaybackEngine::~PlaybackEngine 1\n");
	if(video_cache) delete video_cache;
//printf("PlaybackEngine::~PlaybackEngine 2\n");
}

int PlaybackEngine::create_objects()
{
	int result = 0;
	preferences = new Preferences(mwindow);
	command = new TransportCommand;
	que = new TransportQue;
// Set the first change to maximum
	que->command.change_type = CHANGE_ALL;

	*preferences = *mwindow->preferences;

	done = 0;
	start_lock.lock();
//printf("PlaybackEngine::create_objects 1\n");
	Thread::start();
//printf("PlaybackEngine::create_objects 1\n");
	start_lock.lock();
//printf("PlaybackEngine::create_objects 1\n");
	start_lock.unlock();
//printf("PlaybackEngine::create_objects 1\n");
	return result;
}


int PlaybackEngine::create_render_engines()
{
//printf("PlaybackEngine::create_render_engines 1 %d\n", command->edl->session->playback_strategy);
// Fix playback configurations
	int playback_strategy = command->get_edl()->session->playback_strategy;
	int current_vchannel = 0;
	int current_achannel = 0;

	delete_render_engines();
	for(int i = 0; 
		i < command->get_edl()->session->playback_config[playback_strategy].total; 
		i++)
	{
		VideoOutConfig *vconfig = command->get_edl()->session->playback_config[playback_strategy].values[i]->vconfig;
		AudioOutConfig *aconfig = command->get_edl()->session->playback_config[playback_strategy].values[i]->aconfig;
		int device_channels;

		if(command->get_edl()->session->playback_strategy == PLAYBACK_LOCALHOST)
		{
			vconfig->driver = PLAYBACK_X11;
			device_channels = command->get_edl()->session->video_channels;
		}
		else
			device_channels =  1;

		for(int i = 0; i < MAX_CHANNELS; i++)
		{
			vconfig->do_channel[i] = 
				(i == current_vchannel && 
					current_vchannel < command->get_edl()->session->video_channels &&
					device_channels);
			if(vconfig->do_channel[i])
			{
				current_vchannel++;
				device_channels--;
			}
		}

		device_channels = aconfig->total_output_channels();
		for(int i = 0; i < MAX_CHANNELS; i++)
		{
			aconfig->do_channel[i] = 
				(i == current_achannel && 
					current_achannel < command->get_edl()->session->audio_channels &&
					device_channels);
			if(aconfig->do_channel[i])
			{
				current_achannel++;
				device_channels--;
			}
		}

// One vchannel per device
		current_vchannel++;
	}

//printf("PlaybackEngine::create_render_engines %d\n", command->get_edl()->session->playback_config[playback_strategy].total);
	for(int i = 0; 
		i < command->get_edl()->session->playback_config[playback_strategy].total; 
		i++)
	{
		RenderEngine *engine = new RenderEngine(this,
			command->get_edl()->session->playback_config[playback_strategy].values[i], 
			preferences, 
			command, 
			output,
			mwindow->plugindb,
			&mwindow->channeldb_buz);  
		render_engines.append(engine);
	}
	return 0;
}

void PlaybackEngine::delete_render_engines()
{
	render_engines.remove_all_objects();
}

void PlaybackEngine::arm_render_engines()
{
	for(int i = 0; i < render_engines.total; i++)
	{
//printf("PlaybackEngine::arm_render_engines %ld\n", command->playbackstart);
		render_engines.values[i]->arm_command(command);
	}
}

void PlaybackEngine::start_render_engines()
{
//printf("PlaybackEngine::start_render_engines 1 %d\n", render_engines.total);
	for(int i = 0; i < render_engines.total; i++)
	{
		render_engines.values[i]->start_command();
	}
}

void PlaybackEngine::wait_render_engines()
{
//printf("PlaybackEngine::wait_render_engines 1 %d %d\n", command->command, command->realtime);
	if(command->realtime)
	{
		for(int i = 0; i < render_engines.total; i++)
		{
//printf("PlaybackEngine::wait_render_engines 2 %p %d\n", render_engines.values[i], render_engines.values[i]->get_tid());
			render_engines.values[i]->join();
		}
//printf("PlaybackEngine::wait_render_engines 3 %d\n", render_engines.total);
	}
}

void PlaybackEngine::create_cache()
{
	if(audio_cache) delete audio_cache;
	audio_cache = 0;
	if(video_cache) delete video_cache;
	video_cache = 0;
	if(!audio_cache) 
		audio_cache = new Cache(command->get_edl(), mwindow->plugindb);
	else
		audio_cache->set_edl(command->get_edl());

	if(!video_cache) 
		video_cache = new Cache(command->get_edl(), mwindow->plugindb);
	else
		video_cache->set_edl(command->get_edl());
}


void PlaybackEngine::perform_change()
{
//printf("PlaybackEngine::perform_change 1 %x\n", command->change_type);
	switch(command->change_type)
	{
		case CHANGE_ALL:
			create_cache();
		case CHANGE_EDL:
			audio_cache->set_edl(command->get_edl());
			video_cache->set_edl(command->get_edl());
//printf("PlaybackEngine::perform_change 1\n");
			create_render_engines();
		case CHANGE_PARAMS:
//printf("PlaybackEngine::perform_change 1\n");
			if(command->change_type != CHANGE_EDL &&
				command->change_type != CHANGE_ALL)
				for(int i = 0; i < render_engines.total; i++)
					render_engines.values[i]->edl->synchronize_params(command->get_edl());
		case CHANGE_NONE:
//printf("PlaybackEngine::perform_change 2\n");
			break;
	}
}

void PlaybackEngine::sync_parameters(EDL *edl)
{
// TODO: lock out render engine EDL deletions
	for(int i = 0; i < render_engines.total; i++)
		render_engines.values[i]->edl->synchronize_params(edl);
}


void PlaybackEngine::interrupt_playback(int wait_tracking)
{
//printf("PlaybackEngine::interrupt_playback 1\n");
	for(int i = 0; i < render_engines.total; i++)
		render_engines.values[i]->interrupt_playback();
// Stop pausing
	pause_lock.unlock();
// Wait for tracking to finish if it is running
//printf("PlaybackEngine::interrupt_playback 1\n");
	if(wait_tracking)
	{
		tracking_done.lock();
		tracking_done.unlock();
	}
//printf("PlaybackEngine::interrupt_playback 2\n");
//printf("PlaybackEngine::interrupt_playback 2\n");
}


// Return 1 if levels exist
int PlaybackEngine::get_output_levels(double *levels, long position)
{
	int result = 0;
	for(int j = 0; j < render_engines.total; j++)
	{
		if(render_engines.values[j]->do_audio)
		{
			result = 1;
			render_engines.values[j]->get_output_levels(levels, position);
		}
	}
	return result;
}


int PlaybackEngine::get_module_levels(ArrayList<double> *module_levels, long position)
{
	int result = 0;
	for(int j = 0; j < render_engines.total; j++)
	{
		if(render_engines.values[j]->do_audio)
		{
			result = 1;
			render_engines.values[j]->get_module_levels(module_levels, position);
			break;
		}
	}
	return result;
}


void PlaybackEngine::init_cursor()
{
}

void PlaybackEngine::stop_cursor()
{
}


void PlaybackEngine::init_tracking()
{
	if(!command->single_frame()) 
		tracking_active = 1;
	else
		tracking_active = 0;
	tracking_position = command->playbackstart;
	tracking_done.lock();
	init_cursor();
//printf("PlaybackEngine::init_tracking %ld\n", tracking_position);
}

void PlaybackEngine::stop_tracking()
{
// Set the tracking position to right now before stopping it.
//	tracking_position = get_tracking_position();
//printf("PlaybackEngine::stop_tracking %ld\n", tracking_position);
	tracking_active = 0;
	stop_cursor();
printf("PlaybackEngine::stop_tracking 1\n");
	tracking_done.unlock();
}

void PlaybackEngine::update_tracking(long position)
{
	tracking_lock.lock();
	tracking_position = position;
	tracking_timer.update();
	tracking_lock.unlock();
}

long PlaybackEngine::get_tracking_position()
{
	long result = 0;

	tracking_lock.lock();
// Adjust for elapsed time since last update_tracking
	if(tracking_active)
	{
//printf("PlaybackEngine::get_tracking_position %d %d %d\n", command->get_direction(), tracking_position, tracking_timer.get_scaled_difference(command->get_edl()->session->sample_rate));
		if(command->get_direction() == PLAY_FORWARD)
			result = (long)(tracking_position + 
				command->get_speed() * 
				tracking_timer.get_scaled_difference(command->get_edl()->session->sample_rate));
		else
			result = (long)(tracking_position - 
				command->get_speed() * 
				tracking_timer.get_scaled_difference(command->get_edl()->session->sample_rate));
	}
	else
		result = tracking_position;
	tracking_lock.unlock();
//printf("PlaybackEngine::get_tracking_position %ld %d\n", result, tracking_active);

// Adjust for loop

	return result;
}

void PlaybackEngine::update_transport(int command, int paused)
{
//	mwindow->gui->lock_window();
//	mwindow->gui->mbuttons->transport->update_gui_state(command, paused);
//	mwindow->gui->unlock_window();
}

void PlaybackEngine::run()
{
	start_lock.unlock();
	do
	{
// Wait for current command to finish
//printf("PlaybackEngine::run 1\n");
		que->output_lock.lock();
//printf("PlaybackEngine::run 2\n");

		wait_render_engines();

//printf("PlaybackEngine::run 3 %d\n", que->command.command);

// Read the new command
		que->input_lock.lock();
		if(done) return;

//printf("PlaybackEngine::run 3\n");
		*command = que->command;
//printf("PlaybackEngine::run 3 %ld %ld\n", command->playbackstart, que->command.playbackstart);
		que->command.reset();
//printf("PlaybackEngine::run 3 %p\n", this);
		que->input_lock.unlock();

//printf("PlaybackEngine::run 4 %d %x %x\n", 
//	this->command->command, 
//	this->command->change_type, 
//	que->command.change_type);


		switch(command->command)
		{
// Parameter change only
			case COMMAND_NONE:
				command->command = last_command;
				perform_change();
				break;

			case PAUSE:
				init_cursor();
				pause_lock.lock();
				pause_lock.lock();
				pause_lock.unlock();
				stop_cursor();
				break;

			case STOP:
// No changing
				break;

			case CURRENT_FRAME:
//printf("PlaybackEngine::run 5\n");
				last_command = command->command;
//printf("PlaybackEngine::run 5\n");
				perform_change();
//printf("PlaybackEngine::run 7\n");
				arm_render_engines();
// Dispatch the command
				start_render_engines();
//printf("PlaybackEngine::run 8\n");
				break;

			default:
//printf("PlaybackEngine::run 5\n");
				last_command = command->command;
				is_playing_back = 1;
//printf("PlaybackEngine::run 6 %d %d\n", command->playbackstart, get_tracking_position());
 				if(command->command == SINGLE_FRAME_FWD ||
					command->command == SINGLE_FRAME_REWIND)
				{
 					command->playbackstart = get_tracking_position();
				}

				perform_change();
//printf("PlaybackEngine::run 9 %d\n", command->playbackstart);
				arm_render_engines();

// Start tracking after arming so the tracking position doesn't change.
// The tracking for a single frame command occurs during PAUSE
//printf("PlaybackEngine::run 9 %d\n", command->playbackstart);
				init_tracking();

// Dispatch the command
//printf("PlaybackEngine::run 10\n");
				start_render_engines();
//printf("PlaybackEngine::run 11\n");
				break;
		}

//printf("PlaybackEngine::run 12\n");
	}while(!done);
}






















int PlaybackEngine::reset_parameters()
{
// called before every playback
	is_playing_back = 0;
	follow_loop = 0;
	speed = 1;
	reverse = 0;
	cursor = 0;
	last_position = 0;
	playback_start = playback_end = 0;
	infinite = 0;
	use_buttons = 0;
	audio = 0;
	video = 0;
	shared_audio = 0;
	return 0;
}

int PlaybackEngine::init_parameters()
{
//	is_playing_back = 1;
	update_button = 1;
	correction_factor = 0;

// correct playback buffer sizes
	input_length = 
		playback_buffer = 
		output_length = 
		audio_module_fragment = 
		command->get_edl()->session->audio_module_fragment;

// get maximum actual buffer size written to device plus padding
	if(speed != 1) output_length = (long)(output_length / speed) + 16;   
	if(output_length < playback_buffer) output_length = playback_buffer;

// samples to read at a time is a multiple of the playback buffer greater than read_length
	while(input_length < command->get_edl()->session->audio_read_length)
		input_length += playback_buffer;
	return 0;
}

int PlaybackEngine::init_audio_device()
{
// open audio device
// 	if(!shared_audio)
// 	{
// 		audio = new AudioDevice;
// 		audio->open_output(mwindow->preferences->aconfig, 
// 			mwindow->session->sample_rate, 
// 			output_length);
// 		if(mwindow->preferences->playback_software_timer) audio->set_software_positioning();
// 	}
	return 0;
}

int PlaybackEngine::init_video_device()
{
// open audio device
// 	if(mwindow->preferences->actual_frame_rate < 0) 
// 		mwindow->preferences->actual_frame_rate = mwindow->session->frame_rate;
// 	video = new VideoDevice(mwindow);
// 	return video->open_output(mwindow->preferences->vconfig, 
// 		mwindow->session->frame_rate, 
// 		mwindow->session->output_w, 
// 		mwindow->session->output_h, 
// 		OUTPUT_RGB,
// 		0);
	return 0;
}


int PlaybackEngine::set_range(long start,
					long end,
					long current,
					int reverse,
					float speed)
{
	this->playback_start = start;
	this->playback_end = end;
	this->last_position = current;
	this->reverse = reverse;
	this->speed = speed;
	if(speed == FRAME_SPEED)
	{
// Fix positions for frame advance
		long frame = (long)(((float)(last_position + 1) / mwindow->session->sample_rate) * mwindow->session->frame_rate);
		last_position = (long)(((float)frame / mwindow->session->frame_rate) * mwindow->session->sample_rate);

		if(reverse)
		{
			playback_end = last_position;
			playback_start = (long)(((float)(frame - 1) / mwindow->session->frame_rate) * mwindow->session->sample_rate);
		}
		else
		{
			playback_start = last_position;
			playback_end = (long)(((float)(frame + 1) / mwindow->session->frame_rate) * mwindow->session->sample_rate);
		}
	}
	return 0;
}


int PlaybackEngine::start_reconfigure()
{
	reconfigure_status = is_playing_back;
//	if(is_playing_back) stop_playback(0);
	return 0;
}

int PlaybackEngine::stop_reconfigure()
{
	return 0;
}

int PlaybackEngine::reset_buttons()
{
	if(mwindow->gui && update_button) 
	{
//		mwindow->gui->mbuttons->transport->reset_transport();
	}
	return 0;
}


long PlaybackEngine::absolute_position(int sync_time)
{
	return 0;
}

long PlaybackEngine::get_position(int sync_time)
{
	long result;

	switch(is_playing_back)
	{
		case 2:
// playing back
			result = absolute_position(sync_time);
// adjust for speed
			result = (long)(result * speed);

// adjust direction and initial position
			if(reverse)
			{
				result = playback_end - result;
				result += loop_adjustment;

// adjust for looping
// 				while(mwindow->session->loop_playback && follow_loop && 
// 					result < mwindow->session->loop_start)
// 				{
// 					result += mwindow->session->loop_end - mwindow->session->loop_start;
// 					loop_adjustment += mwindow->session->loop_end - mwindow->session->loop_start;
// 				}
			}
			else
			{
				result += playback_start;
				result -= loop_adjustment;
				
// 				while(mwindow->session->loop_playback && follow_loop && 
// 					result > mwindow->session->loop_end)
// 				{
// 					result -= mwindow->session->loop_end - mwindow->session->loop_start;
// 					loop_adjustment += mwindow->session->loop_end - mwindow->session->loop_start;
// 				}
			}
			break;

		case 1:
// paused
			result = last_position;
			break;

		default:
// no value
			result = -1;
			break;
	}

	return result;
}


int PlaybackEngine::move_right(long distance) 
{ 
	mwindow->move_right(distance); 
	return 0;
}


int PlaybackEngine::cleanup()
{
	return 0;
}

int PlaybackEngine::wait_for_startup()
{
	return 0;
}

int PlaybackEngine::wait_for_completion()
{
// must use mutex not join since playback engine also terminates itself
	complete.lock();
	complete.unlock();
	return 0;
}
