#include "assets.h"
#include "autoconf.h"
#include "bezierautos.h"
#include "cache.h"
#include "clip.h"
#include "datatype.h"
#include "edit.h"
#include "edits.h"
#include "edl.h"
#include "edlsession.h"
#include "filexml.h"
#include "floatautos.h"
#include "mwindow.h"
#include "patch.h"
#include "pluginbuffer.h"
#include "selections.h"
#include "mainsession.h"
#include "theme.h"
#include "trackcanvas.h"
#include "tracks.h"
#include "transportque.inc"
#include "units.h"
#include "vautomation.h"
#include "vedit.h"
#include "vedits.h"
#include "vframe.h"
#include "vmodule.h"
#include "vpluginset.h"
#include "vtrack.h"

VTrack::VTrack(EDL *edl, Tracks *tracks)
 : Track(edl, tracks)
{
	data_type = TRACK_VIDEO;
	selections = 0;
	draw = 0;
}

VTrack::~VTrack()
{
	if(selections) delete selections;
}

int VTrack::create_objects()
{
	Track::create_objects();
	automation = new VAutomation(edl, this);
	automation->create_objects();
	edits = new VEdits(edl, this);
	selections = new Selections(edl, this);
	return 0;
}

// Used by PlaybackEngine
void VTrack::synchronize_params(Track *track)
{
	Track::synchronize_params(track);

	VTrack *vtrack = (VTrack*)track;
}

// Used by EDL::operator=
int VTrack::copy_settings(Track *track)
{
	Track::copy_settings(track);

	VTrack *vtrack = (VTrack*)track;
	return 0;
}

int VTrack::vertical_span(Theme *theme)
{
	int track_h = Track::vertical_span(theme);
	int patch_h = 0;
	if(expand_view)
	{
		patch_h += theme->title_h + theme->play_h + theme->fade_h + theme->mode_h;
	}
	return MAX(track_h, patch_h);
}


PluginSet* VTrack::new_plugins()
{
	return new VPluginSet(edl, this);
}

int VTrack::load_defaults(Defaults *defaults)
{
	Track::load_defaults(defaults);
	return 0;
}

void VTrack::set_default_title()
{
	Track *current = ListItem<Track>::owner->first;
	int i;
	for(i = 0; current; current = NEXT)
	{
		if(current->data_type == TRACK_VIDEO) i++;
	}
	sprintf(title, "Video %d", i);
}

long VTrack::to_units(double position, int round)
{
	if(round)
		return Units::round(position * edl->session->frame_rate);
	else
		return (long)(position * edl->session->frame_rate);
}

double VTrack::from_units(long position)
{
	return (double)position / edl->session->frame_rate;
}




int VTrack::identical(long sample1, long sample2)
{
// Units of frames
	if(labs(sample1 - sample2) <= 1) return 1; else return 0;
}

int VTrack::save_header(FileXML *file)
{
	file->tag.set_property("TYPE", "VIDEO");
	return 0;
}

int VTrack::save_derived(FileXML *file)
{
	return 0;
}

int VTrack::load_header(FileXML *file, unsigned long load_flags)
{
	return 0;
}

int VTrack::load_derived(FileXML *file, unsigned long load_flags)
{
	return 0;
}


int VTrack::direct_copy_possible(long start, int direction)
{
	int i;

//printf("VTrack::direct_copy_possible 1\n");
// No automation must be present in the track
	if(!automation->direct_copy_possible(start, direction))
		return 0;
//printf("VTrack::direct_copy_possible 2\n");

// No plugin must be present
	if(plugin_used(start, direction)) 
		return 0;

//printf("VTrack::direct_copy_possible 3\n");
// No transition
	if(get_current_transition(start, direction))
		return 0;
//printf("VTrack::direct_copy_possible 4\n");

	return 1;
}

















VTrack::VTrack(MWindow *mwindow, Tracks *tracks) : Track(mwindow, tracks)
{
	data_type = TRACK_VIDEO;
	translation_selected = 0;
	translation_zoom = 0;
}

int VTrack::create_derived_objs(int flash)
{
	int i;
	edits = new VEdits(edl, this);
	return 0;
}


long VTrack::length()
{
	return Units::tosamples(edits->length(), mwindow->session->sample_rate, mwindow->session->frame_rate);
}

int VTrack::get_dimensions(float &view_start, float &view_units, float &zoom_units)
{
	view_start = Units::toframes(mwindow->session->view_start, mwindow->session->sample_rate, mwindow->session->frame_rate);
	view_units = Units::toframes(tracks->view_samples(), mwindow->session->sample_rate, mwindow->session->frame_rate);
	zoom_units = Units::toframes(mwindow->session->zoom_sample, mwindow->session->sample_rate, mwindow->session->frame_rate);
}

int VTrack::copy_derived(long start, long end, FileXML *xml)
{
// automation is copied in the Track::copy
}

int VTrack::copy_automation_derived(AutoConf *auto_conf, long start, long end, FileXML *file)
{
// used for copying only automation
// 	if(auto_conf->camera)
// 	{
// 		file->tag.set_title("CAMERAAUTOS");
// 		file->append_tag();
// 
// 		automation->camera_autos->copy(start, end, file, 1);
// 
// 		file->tag.set_title("/CAMERAAUTOS");
// 		file->append_tag();
// 		file->append_newline();
// 	}
// 
// 	if(auto_conf->projector)
// 	{
// 		file->tag.set_title("PROJECTORAUTOS");
// 		file->append_tag();
// 
// 		automation->projector_autos->copy(start, end, file, 1);
// 
// 		file->tag.set_title("/PROJECTORAUTOS");
// 		file->append_tag();
// 		file->append_newline();
// 	}
	return 0;
}

int VTrack::paste_derived(long start, long end, long total_length, FileXML *xml, int &current_channel)
{
	if(xml->tag.title_is("CAMERAAUTOS"))
	{
		camera_autos->paste(start, end, total_length, xml, "/CAMERAAUTOS", 1);
	}
	else
	if(xml->tag.title_is("PROJECTORAUTOS"))
	{
		projector_autos->paste(start, end, total_length, xml, "/PROJECTORAUTOS", 1);
	}
}

int VTrack::paste_output(long startproject, long endproject, long startsource, long endsource, int layer, Asset *asset)
{
	int result = 0;

//printf("VTrack::paste_output startproject %ld endproject %ld\n", startproject, endproject);
	//Track::samples_to_units(startproject, endproject);

	result = ((VEdits*)edits)->paste_edit(startproject, 
						endproject, 
						startsource, 
						endsource - startsource, 
						layer, 
						0,
						0,
						1,
						asset);

	//if(!result && mwindow->autos_follow_edits)
	//{
	//	paste_auto_silence(startproject, endproject);
	//}
	return result;
}

int VTrack::clear_derived(long start, long end)
{
	return 0;
}

int VTrack::paste_automation_derived(long start, long end, long total_length, FileXML *xml, int shift_autos, int &current_pan)
{
// only used for pasting automation
	if(xml->tag.title_is("CAMERAAUTOS"))
	{
		camera_autos->paste(start, end, total_length, xml, "/CAMERAAUTOS", 1, shift_autos);
	}
	else
	if(xml->tag.title_is("PROJECTORAUTOS"))
		projector_autos->paste(start, end, total_length, xml, "/PROJECTORAUTOS", 1, shift_autos);
}

int VTrack::clear_automation_derived(AutoConf *auto_conf, long start, long end, int shift_autos)
{
//	if(auto_conf->camera)
//		camera_autos->clear(start, end, mwindow->session->autos_follow_edits, shift_autos);

//	if(auto_conf->projector)
//		projector_autos->clear(start, end, mwindow->session->autos_follow_edits, shift_autos);
}

int VTrack::paste_auto_silence_derived(long start, long end)
{
	camera_autos->paste_silence(start, end);
	projector_autos->paste_silence(start, end);
}

int VTrack::draw_autos_derived(float view_start, float zoom_units, AutoConf *auto_conf)
{
// 		if(auto_conf->camera) 
// 			camera_autos->draw(tracks->canvas, 
// 							pixel,
// 							mwindow->session->zoom_track, 
// 							zoom_units, 
// 							view_start, 
// 							mwindow->session->tracks_vertical);
// 
// 		if(auto_conf->projector)
// 			projector_autos->draw(tracks->canvas, 
// 							pixel,
// 							mwindow->session->zoom_track, 
// 							zoom_units, 
// 							view_start, 
// 							mwindow->session->tracks_vertical);
}

int VTrack::select_translation(int cursor_x, int cursor_y)
{
// cursor position is relative to time
	int result = 0;
	float view_start, view_units, zoom_units;
	get_dimensions(view_start, view_units, zoom_units);

	if(cursor_y > pixel && cursor_y < pixel + mwindow->session->zoom_track)
	{
		for(Edit* current = edits->first; current && !result; current = NEXT)
		{
			result = ((VEdit*)current)->select_translation(cursor_x, cursor_y, view_start, zoom_units);
		}
	}
	return result;
}

int VTrack::update_translation(int cursor_x, int cursor_y, int shift_down)
{
	int result = 0;
	float view_start, view_units, zoom_units;
	get_dimensions(view_start, view_units, zoom_units);

	for(Edit* current = edits->first; current/* && !result*/; current = NEXT)
	{
		result = ((VEdit*)current)->update_translation(cursor_x, cursor_y, shift_down, view_start, zoom_units);
	}
	return result;
}

int VTrack::end_translation()
{
	int result = 0;
	for(Edit* current = edits->first; current && !result; current = NEXT)
	{
		result = ((VEdit*)current)->end_translation();
	}
	return result;
}

int VTrack::reset_translation(long start, long end)
{
	int result = 0;
	Track::samples_to_units(start, end);
	for(Edit* current = edits->first; current; current = NEXT)
	{
		result += ((VEdit*)current)->reset_translation(start, end);
	}
	return result;
}


int VTrack::select_auto_derived(float zoom_units, float view_start, AutoConf *auto_conf, int cursor_x, int cursor_y)
{
	int result = 0;

// 	if(auto_conf->camera) 
// 		result = camera_autos->select_auto(tracks->canvas, 
// 										pixel, 
// 										mwindow->session->zoom_track, 
// 										zoom_units, 
// 										view_start, 
// 										cursor_x, 
// 										cursor_y, 
// 										tracks->canvas->shift_down(),
// 										tracks->canvas->ctrl_down(),
// 										tracks->canvas->get_buttonpress(),
// 										mwindow->session->tracks_vertical);
// 
// 	if(auto_conf->projector && !result) 
// 		result = projector_autos->select_auto(tracks->canvas, 
// 										pixel, 
// 										mwindow->session->zoom_track, 
// 										zoom_units, 
// 										view_start, 
// 										cursor_x, 
// 										cursor_y, 
// 										tracks->canvas->shift_down(),
// 										tracks->canvas->ctrl_down(),
// 										tracks->canvas->get_buttonpress(),
// 										mwindow->session->tracks_vertical);

	return result;
}


int VTrack::move_auto_derived(float zoom_units, float view_start, AutoConf *auto_conf, int cursor_x, int cursor_y, int shift_down)
{
	int result;
	result = 0;
// 
// 	if(auto_conf->camera)
// 		result = camera_autos->move_auto(tracks->canvas, 
// 									pixel, 
// 									mwindow->session->zoom_track, 
// 									zoom_units, 
// 									view_start, 
// 									cursor_x, 
// 									cursor_y, 
// 									shift_down, 
// 									mwindow->session->tracks_vertical);
// 
// 	if(auto_conf->projector && !result)
// 		result = projector_autos->move_auto(tracks->canvas, 
// 									pixel, 
// 									mwindow->session->zoom_track, 
// 									zoom_units, 
// 									view_start, 
// 									cursor_x, 
// 									cursor_y, 
// 									shift_down, 
// 									mwindow->session->tracks_vertical);
// 
// 	if(result)
// 	{
// 		mwindow->tracks->hide_overlays(0);
// 		draw_clear(0, tracks->canvas->get_w(), 0, tracks->canvas->get_h(), 0);
// 		draw_old(0, tracks->canvas->get_w(), 0, tracks->canvas->get_h(), 0);
// 		mwindow->tracks->show_overlays(0);
// 	}

	return result;
}

int VTrack::draw_floating_autos_derived(float view_start, float zoom_units, AutoConf *auto_conf, int flash)
{
// 	if(auto_conf->camera) 
// 		camera_autos->draw_floating_autos(tracks->canvas, 
// 								pixel, 
// 								mwindow->session->zoom_track, 
// 								zoom_units, 
// 								view_start, 
// 								mwindow->session->tracks_vertical, flash);
// 
// 	if(auto_conf->projector) 
// 		projector_autos->draw_floating_autos(tracks->canvas, 
// 								pixel, 
// 								mwindow->session->zoom_track, 
// 								zoom_units, 
// 								view_start, 
// 								mwindow->session->tracks_vertical, flash);
}

int VTrack::release_auto_derived()
{
	int result;
	result = 0;

	result = camera_autos->release_auto();
	if(!result) result = projector_autos->release_auto();
		
	return result;
}

int VTrack::scale_video(float camera_scale, float projector_scale, int *offsets)
{
// Fix the camera.
	for(VEdit *current = (VEdit*)edits->first; current; current = (VEdit*)NEXT)
	{
		current->center_z *= camera_scale;
		current->center_x = -offsets[0];
		current->center_y = -offsets[1];
	}

// Fix the projector.
	projector_autos->scale_video(projector_scale, &offsets[2]);
}

int VTrack::channel_is_playable(long position, int direction, int *do_channel)
{
	int result = 0;
	for(int i = 0; i < edl->session->video_channels && !result; i++)
	{
		if(do_channel[i])
		{
			float in_x, in_y, in_w, in_h;
			float out_x, out_y, out_w, out_h;

			calculate_output_transfer(i, position, direction, 
				in_x, in_y, in_w, in_h,
				out_x, out_y, out_w, out_h);

// printf("VTrack::channel_is_playable %0.0f %0.0f %0.0f %0.0f %0.0f %0.0f %0.0f %0.0f\n", 
// 	in_x, in_y, in_w, in_h, out_x, out_y, out_w, out_h);
			if(out_w > 0 && out_h > 0) 
				result = 1;
		}
	}
	return result;
}

void VTrack::calculate_input_transfer(Asset *asset, 
	long position, 
	int direction, 
	float &in_x, 
	float &in_y, 
	float &in_w, 
	float &in_h,
	float &out_x, 
	float &out_y, 
	float &out_w, 
	float &out_h)
{
	float auto_x, auto_y, auto_z;
	BezierAuto *before = 0, *after = 0;
	float camera_z = 1;
	float camera_x = asset->width / 2;
	float camera_y = asset->height / 2;
	float z[6], x[6], y[6];        // camera and output coords

// get camera center in asset
	if(automate)
	{
		automation->camera_autos->get_center(auto_x, 
			auto_y, 
			auto_z, 
			(float)position, 
			direction, 
			&before, 
			&after);
	}
	else
	{
		auto_x = 0;
		auto_y = 0;
		auto_z = 1;
	}

	camera_z *= auto_z;
	camera_x += auto_x;
	camera_y += auto_y;

// get camera coords on asset
	x[0] = camera_x - (float)edl->session->track_w / 2 / camera_z;
	y[0] = camera_y - (float)edl->session->track_h / 2 / camera_z;
	x[1] = x[0] + (float)edl->session->track_w / camera_z;
	y[1] = y[0] + (float)edl->session->track_h / camera_z;

// get asset coords on camera
	x[2] = 0;
	y[2] = 0;
	x[3] = edl->session->track_w;
	y[3] = edl->session->track_h;

// crop asset coords on camera
	if(x[0] < 0)
	{
		x[2] -= x[0] * camera_z;
		x[0] = 0;
	}
	if(y[0] < 0)
	{
		y[2] -= y[0] * camera_z;
		y[0] = 0;
	}
	if(x[1] > asset->width)
	{
		x[3] -= (x[1] - asset->width) * camera_z;
		x[1] = asset->width;
	}
	if(y[1] > asset->height)
	{
		y[3] -= (y[1] - asset->height) * camera_z;
		y[1] = asset->height;
	}

// get output bounding box
	out_x = x[2];
	out_y = y[2];
	out_w = x[3] - x[2];
	out_h = y[3] - y[2];

	in_x = x[0];
	in_y = y[0];
	in_w = x[1] - x[0];
	in_h = y[1] - y[0];
}

void VTrack::calculate_output_transfer(int channel, 
	long position, 
	int direction, 
	float &in_x, 
	float &in_y, 
	float &in_w, 
	float &in_h,
	float &out_x, 
	float &out_y, 
	float &out_w, 
	float &out_h)
{
	BezierAuto *before = 0, *after = 0;
	float center_x, center_y, center_z;
	float x[4], y[4];
	float channel_x1 = edl->session->vchannel_x[channel];
	float channel_y1 = edl->session->vchannel_y[channel];
	float channel_x2 = channel_x1 + edl->session->output_w;
	float channel_y2 = channel_y1 + edl->session->output_h;

	x[0] = y[0] = 0;
	x[1] = edl->session->track_w;
	y[1] = edl->session->track_h;

	if(automate)
	{
		automation->projector_autos->get_center(center_x, 
			center_y, 
			center_z, 
			(float)position, 
			direction, 
			&before, 
			&after);
	}
	else
	{
		center_x = 0;
		center_y = 0;
		center_z = 1;
	}

	center_x += edl->session->output_w / 2;
	center_y += edl->session->output_h / 2;

	x[2] = center_x - (edl->session->track_w / 2) * center_z;
	y[2] = center_y - (edl->session->track_h / 2) * center_z;
	x[3] = x[2] + edl->session->track_w * center_z;
	y[3] = y[2] + edl->session->track_h * center_z;

// Clip to boundaries of channel
	if(x[2] < channel_x1)
	{
		x[0] -= (x[2] - channel_x1) / center_z;
		x[2] = channel_x1;
	}
	if(y[2] < 0)
	{
		y[0] -= (y[2] - channel_y1) / center_z;
		y[2] = 0;
	}
	if(x[3] > channel_x2)
	{
		x[1] -= (x[3] - channel_x2) / center_z;
		x[3] = channel_x2;
	}
	if(y[3] > channel_y2)
	{
		y[1] -= (y[3] - channel_y2) / center_z;
		y[3] = channel_y2;
	}

	in_x = x[0];
	in_y = y[0];
	in_w = x[1] - x[0];
	in_h = y[1] - y[0];
	out_x = x[2];
	out_y = y[2];
	out_w = x[3] - x[2];
	out_h = y[3] - y[2];
}



int VTrack::get_projection(int channel, 
	float &in_x1, 
	float &in_y1, 
	float &in_x2, 
	float &in_y2, 
	float &out_x1, 
	float &out_y1, 
	float &out_x2, 
	float &out_y2, 
	int frame_w, 
	int frame_h, 
	long real_position, 
	int direction)
{
	float center_x, center_y, center_z;
	float x[4], y[4];
	BezierAuto *before = 0, *after = 0;

	automation->projector_autos->get_center(center_x, 
		center_y, 
		center_z, 
		(float)real_position, 
		direction, 
		&before, 
		&after);

	x[0] = y[0] = 0;
	x[1] = frame_w;
	y[1] = frame_h;

	center_x += edl->session->output_w / 2;
	center_y += edl->session->output_h / 2;

	x[2] = center_x - (frame_w / 2) * center_z;
	y[2] = center_y - (frame_h / 2) * center_z;
	x[3] = x[2] + frame_w * center_z;
	y[3] = y[2] + frame_h * center_z;

	if(x[2] < 0)
	{
		x[0] -= x[2] / center_z;
		x[2] = 0;
	}
	if(y[2] < 0)
	{
		y[0] -= y[2] / center_z;
		y[2] = 0;
	}
	if(x[3] > edl->session->output_w)
	{
		x[1] -= (x[3] - edl->session->output_w) / center_z;
		x[3] = edl->session->output_w;
	}
	if(y[3] > edl->session->output_h)
	{
		y[1] -= (y[3] - edl->session->output_h) / center_z;
		y[3] = edl->session->output_h;
	}

	in_x1 = x[0];
	in_y1 = y[0];
	in_x2 = x[1];
	in_y2 = y[1];
	out_x1 = x[2];
	out_y1 = y[2];
	out_x2 = x[3];
	out_y2 = y[3];
	return 0;
}

int VTrack::automation_is_used_derived(long start, long end)
{
	if(camera_autos->total() ||
		projector_autos->total())
		return 1;
	else
		return 0;
}

int VTrack::scale_time_derived(float rate_scale, int scale_edits, int scale_autos, long start, long end)
{
	camera_autos->scale_time(rate_scale, scale_edits, scale_autos, start, end);
	projector_autos->scale_time(rate_scale, scale_edits, scale_autos, start, end);
}


int VTrack::get_virtual_center(VEdit *edit, int cursor_x, int cursor_y, float frame_w, float frame_h, float picture_zoom)
{
	if(translation_zoom)
	{
		virtual_center_y = (int)(cursor_y - edit->center_z * frame_w / mwindow->get_aspect_ratio());
	}
	else
	{
		virtual_center_x = (int)(cursor_x - edit->center_x * edit->center_z);
		virtual_center_y = (int)(cursor_y - edit->center_y * edit->center_z);
	}
}
